#!/bin/csh -f 

#
# BuildTopLevel.csh - select top level unit for given VHDL source and link with kernel
# for a VHDL file which has been compiled
# 

#
# Initialization
#
setenv CLSI /usr/local/clsi
setenv VTIP $CLSI/vtip
setenv RVCGR $CLSI/rvcgr
setenv LM_LICENSE_FILE $VTIP/license/license.dat
source $VTIP/com/setupvtip
source $RVCGR/setuprvcgr.csh
set CFLAGS="-I$CG_VKI -I$CG_RTLI -c"
set DefineList="dls_std:STANDARD dls_std:TEXTIO"
setenv VSIM_LIB ~vsim/lib

setwork
#
#
set CmdName=`basename ${0} .csh`

#
#
#	Check parameter and set file names
#
if ( $#argv != 1 ) then
	echo "Error: $CmdName requires VHDL source file as parameter"
	exit 1
endif
set VHDLFile=$1
set ModelName=`basename $VHDLFile .vhdl`
set ModelName=`basename $ModelName .vhd`
set AUXFile="${ModelName}.AUX"
set CFGFile="${ModelName}.CFG"
#
#       1) Select top level unit
#
if ( ! -f "$AUXFile" ) then
    echo "Error: temporary file $AUXFile not found"
    exit 1
endif
set ECUnits=`awk '($1 == "DesignUnit:" && $2 == "Saved") && ($3 == "Entity" || $3 == "Configuration") { print $4 }' $AUXFile`

set MinUnits=`echo $ECUnits | tr ' ' '\012' | awk -F: '{print $2 }' | awk -F. '{ print $1 }'`
set LastUnit=""
foreach unit ($MinUnits)
    set LastUnit="$unit"
end
if ($LastUnit == "") then
    echo "Error: file $VHDLFile does not contain an entity or architecture"
    exit 1
endif

set InTopUnit = `basename $LastUnit`
#
#	2) Configure the model (gather required units).
#
echo ""
echo "--Configuring ${ModelName}"
$CG_CFGE/config ${InTopUnit} -output $CFGFile -define $DefineList
if ( $status != 0 ) then
	echo "Error: configuration tool failed."
	exit 1
endif

#
#	Get Top Unit and Unit List
#
set TopUnit=`awk '/^TopUnit:/ { print $2 }' $CFGFile`
set OtherUnits=`awk '/^RequiredUnit:/ { print $2 }' $CFGFile`

set ValidUnits=""
set Objects=""
foreach unit ( $TopUnit $OtherUnits )
	if ( $unit != "dls_std:STANDARD" && \
	    $unit != "dls_std:TEXTIO") then
	    set ValidUnits="$ValidUnits $unit"
	    set unit_name=`echo $unit | awk -F: '{ print $2 }'`
	    set Objects="$Objects ${unit_name}-g.o"
	endif
end

#
#	3) Build Register Units Module (--config.c file).
#
echo ""
echo "--Building register units module"
set ConfigCFile=${ModelName}--config.c
set ConfigOFile=${ModelName}--config.o

echo "/*    File: ${ModelName}--config.c */" 	> $ConfigCFile
echo "/* Version: 3 */ " 			>> $ConfigCFile
echo '#include "VLI.h"'				>> $ConfigCFile
echo "" 					>> $ConfigCFile
set QQ='"'
echo "char TK_ModelName[] = ${QQ}${ModelName}${QQ};" 	>> $ConfigCFile
echo "" 					>> $ConfigCFile
echo "void TK_RegisterUnits()" 			>> $ConfigCFile
echo "{" 					>> $ConfigCFile
echo "/**/" 					>> $ConfigCFile


set TopUnitSpecified=0
foreach unit ($ValidUnits) 
    set RegName=`echo $unit | sed s/dls_// | sed s/:/__/|sed s/-/__/`
    if ( "$unit" == "$TopUnit" )  then
        echo "    TK_SpecifyTopUnit(VC_Reg_${RegName}());" >> $ConfigCFile
        set TopUnitSpecified=1
    else	 
        echo "    VC_Reg_${RegName}();" >> $ConfigCFile
    endif
end
if ( 0 == $TopUnitSpecified ) then
    echo "Error: top unit not specified in register unit module"
    exit 1
endif
    
echo ""					>> $ConfigCFile
echo "} /* TK_RegisterUnits */"		>> $ConfigCFile

cc $CFLAGS -o $ConfigOFile $ConfigCFile
if ( $status != 0 ) then
	echo "Error: compile of register units module failed"
	exit 1
endif

#
#	4) Link required unit's objects, registration module and test kernel.
#
echo " "
echo "--Linking model ${ModelName}"
CC -o ${ModelName}.sim -v $ConfigOFile $Objects $VSIM_LIB/EventQueue.o  -L$VSIM_LIB -lVsim -lVCGRTL -llwp
if ($status != 0) then
	echo "Error: link of simulation model failed."
	exit 1
endif

exit 0

