#!/bin/csh -f

#
# Compile.csh - compiles the single argument file in the cuurent working dir into .o files
#  file.vhdl -> files.c -> files.o
#

#
# Initialization
#
setenv CLSI /usr/local/clsi
setenv VTIP $CLSI/vtip
setenv RVCGR $CLSI/rvcgr
setenv LM_LICENSE_FILE $VTIP/license/license.dat
source $VTIP/com/setupvtip
source $RVCGR/setuprvcgr.csh
set CFLAGS="-I$CG_VKI -I$CG_RTLI -c"
#
#
# set work directory to be current working directory
setwork
#
#
#
set CmdName=`basename ${0} .csh`
#
#
#	Check parameter and set file name
#
if ( $#argv != 1 ) then
	echo "Error: $CmdName requires VHDL source file as parameter"
	exit 1
endif
set VHDLFile=$1
set ModelName=`basename $VHDLFile .vhdl`
set ModelName=`basename $ModelName .vhd`
set AuxFile="${ModelName}.AUX"
#
#	1) Analyze the VHDL source file
#
echo ""
echo "--Analyzing $1"
vhdl -aux $AuxFile $1
if ( $status > 1 ) then
	echo "Error: from analyzer"
	exit 1
endif

#
#	Get list of library units from anaylzer output file
#
set FullUnits=`awk '$1 == "DesignUnit:" && $2 == "Saved" { print $4 }' $AuxFile`
set MinUnits=`echo $FullUnits | tr ' ' '\012' | awk -F: '{print $2 }' | awk -F. '{ print $1 }'`

#
#	2) Generate 'C' code for each library unit analyzed
#
foreach unit ($MinUnits)
	echo ""
	echo "--Generating C code for $unit"
	$CG_VCG/vcg -debug $unit
	if ( $status > 1 ) then
		echo "Error: from code generator"
		exit 1
	endif
end
#
#	3) Compile the 'C' code
#
foreach unit ($MinUnits)
        set CFile="${unit}-g.c"
	echo ""
	echo "--Compiling $CFile"
	cc $CFLAGS $CFile
	if ( $status ) then
		echo "Error: from 'C' compiler"
		exit 1
	endif
end

