#ifndef VLDataP_H
#define VLDataP_H
/* ******************************************************************
* 
* Copyright 1989-1991 by CAD Language Systems, Inc.  All rights reserved. 
* 
* HEADER FILE:	VLDataP.h
* 
* DESCRIPTION:
* 	RVCG run-time library miscellaneous data-manipulation macros, types, 
*	etc.
* 
* CONSTANTS:
* 	<Constants>
*
* TYPES:
* 	<Types>
*
* GLOBAL VARIABLES:
* 	<Global Vars>
* 
* ROUTINES:
*   Module-external:
* 	<Routines>
* 
*   Module-internal:
* 	<Routines>
* 
* HISTORY:
* 	14-SEP-90 GJB	Created
*	27-FEB-91 DSB	Restructuring VKI and VLI types
*	25-APR-91 DSB	W0021:
*	16-APR-91 DSB	Removing old types from RTL and CG
*	30-APR-91 DSB	Changed old VL_tIntegerType to VK_tIntegerData/-Value
*	13-MAY-91 DSB	W0060: Added VL_Cvt{To,From}LargeTime
*	29-MAY-91 DSB	Added a few comments
*	11-JUN-91 DSB	Added VL_INIT_LARGE; VL_CopyLarge
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*
* SCCS ID: @(#)VLDataP.h	35.3 1/11/92
* 
****************************************************************** */

#include "VLConfigP.h"
#include "VKI.h"		/* for ... ?? */
#include "VLMathOpsP.h"

#ifndef NULL
#  define NULL (0)
#endif


/* *******************************************************************
*
* special
*
******************************************************************* */

extern VK_tIntegerData VL_Zero;		/* for when we need address of zero */

/* VL_INIT_LARGE - special syntactic macro for initialization of large
	integer variables (or element thereof) */
#ifdef VK_LITTLE_ENDIAN
#  define VL_INIT_LARGE(high, low) {low, high}
#else
#  ifdef VK_BIG_ENDIAN
#    define VL_INIT_LARGE(high, low) {high, low}
#  else
    #error
#  endif
#endif


#define VL_CopyLarge(target, source) \
	((target).high = (source).high, (target).low = (source).low)

/* *******************************************************************
*
* Constraint manipulation macros
*
* HISTORY:
*	16-AUG-90 DSB	Added access functions
*	04-SEP-90 DSB	Changed index constraint array from pointers to records
*	05-SEP-90 DSB	Added VL_CopyRange
*	  -MAR-91 DSB	Moved constraint types to VKI
*	12-APR-91 DSB	Cleaned up some comments
*	09-MAY-91 DSB	Added VL_RangesMatch
*	05-AUG-91 DSB	Renamed old VL_IsNullRange to VL_IsNullSlice; added
*			new VL_IsNullRange
*	05-NOV-91 DSB	Added VL_NotInLargeRange
*
******************************************************************* */

					/* general access functions */

/* VL_CopyRange - copy (non-large) integer or floating range */

#define VL_CopyRange(trg, src) \
  (trg).right = (src).right; (trg).dir = (src).dir; (trg).left = (src).left

/* VL_CopyLargeRange - copy large integer range */

#define VL_CopyLargeRange(trg, src) \
	VL_MemCpy(trg, src, sizeof(VK_tLargeRangeConstraint)
					/* copy large range */

#define VL_GetLeft(r) ((r).left)	/* get left bound of range */
#define VL_GetDir(r) ((r).dir)		/* get direction of range */
#define VL_GetRight(r) ((r).right)	/* get right bound of range */

#define VL_GetLow(r) (*((r).dir == VK_Ascending ? &((r).left) : &((r).right)))
					/* get lower bound of range */
#define VL_GetHigh(r) (*((r).dir == VK_Ascending ? &((r).right) : &((r).left)))
					/* get upper bound of range */

/* VL_GetRawLength - get length of non-null range */
#define VL_GetRawLength(r) ( (((r).right - (r).left) * (r).dir) + 1 )

/* VL_GetLength - get length of any range (including null range) */
#define VL_GetLength(r) \
	( VL_IsNullRange(r) ? 0 : ((((r).right - (r).left) * (r).dir) + 1) )
					/* get length of range*/

/* Note: The * and & are to allow large ranges to work. */

/* VL_RangesMatch - check if ranges match; valid for non-null range only */

#define VL_RangesMatch(l, r)	\
   (((l).left == (r).left) && ((l).dir == (r).dir) && ((l).right == (r).right))

/* VL_NotInRange - check for value being out of range; for scalar type
	implemented as small integer or as floating point */
/* ?? hole: does not address null ranges ?? */

#define VL_NotInRange(v, r) ((v) < VL_GetLow(r) || VL_GetHigh(r) < (v))

/* VL_NotInLargeRange - check for value being out of range; for large physical
	type implemented as large integer */
/* ?? hole: does not address null ranges ?? */

#define VL_NotInLargeRange(v, r) \
	(VL_LargeLessThan(&(v), &VL_GetLow(r)) || \
	 VL_LargeLessThan(&VL_GetHigh(r), &(v)))


/* VL_IsNullRange - check for null range (as defined by LRM 3.1) */
/* VK_tDiscreteRangePtr range; - range to examine */

#define VL_IsNullRange(range) (VL_GetRawLength(range) <= 0)
					/* simple implementation */


/* VL_IsNullSlice - check for null slice (as defined by LRM 6.5) */
/* ?? hole:  This does not yet consider VASG Issue #0013 which specifies that
	a reversed direction is an error.) */
/* VK_tDiscreteRangePtr r; - range to examine */
/* VK_tDirection d; - direction of sliced object */

#define VL_IsNullSlice(r, d) (VL_GetDir(r) != (d) || VL_IsNullRange(r))


/* VL_GET_OFFSET - get offset of non-array field in record (designated by 
	structure pointer type instead of structure type) */
#define VL_GET_OFFSET(pt, field) \
	((unsigned int) ( ( (char *) (&(((pt)NULL)->field)) ) - \
		((char *)NULL) ))


/* VL_GET_OFFSET2 - get offset of array field in record (designated by 
	structure pointer type instead of structure type) */
#define VL_GET_OFFSET2(pt, field) \
	((unsigned int) ( ( (char *) (/*&*/(((pt)NULL)->field)) ) - \
		((char *)NULL) ))

/* Note: because C SUCKS, for array fields we need a separate version that 
	doesn't have the ampersand on ->F (to avoid compiler warnings). */
/* ?? These could use better names, though VL_GET_ARRAY_OFFSET would be
	pretty long... ?? */


/* *******************************************************************
*
* 
*
******************************************************************* */

typedef VK_tDataLength VL_tLengthField;	/* length in words */
typedef VK_tDataLength VL_tWordOffsetField;
					/* record element offset in words */


/* *******************************************************************
*
* 
*
******************************************************************* */

#define VL_Alloc	VK_Alloc	/* use VKI functions */
#define VL_Free		VK_Free		/* use VKI functions */


/* *******************************************************************
*
* 
*
******************************************************************* */

#if 0 /* use function version for now */
#define VL_MemCpy(target, source, byte_size) \
	memcpy(target, source, byte_size)
#endif

void VL_Duplicate(			/* for evaluating aggregates */
#ifdef VK_ANSI_FN_PROTOS
	VK_tCompositeAddr buf_addr, int element_length, int element_count
#endif
	);

/* *******************************************************************
*
* Types for the data associated with access values...
*
******************************************************************* */

void VL_Deallocate(
#ifdef VK_ANSI_FN_PROTOS
	VK_tAccessDataPtr * access_var_addr
#endif
	);
		  
#endif /* VLDataP_H */
