#ifndef VLLogOpsP_H
#define VLLogOpsP_H
/* *******************************************************************
*
* Copyright (c) 1989 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VLLogOpsP.h
*
* DESCRIPTION:  
*	RVCG run-time library routines for predefined logical operators,
*	relational operators, and concatenation.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
* 	14-SEP-90 GJB	Created from VCGRTL.h
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*
* SCCS ID: @(#)VLLogOpsP.h	35.1 12/24/91
*
******************************************************************* */

#include "VLConfigP.h"		/* for VK_ANSI_FN_PROTOS */
#include "VLDataP.h"		/* for ?? */


typedef VK_tIntegerData * VL_tEnumArrayAddr;
				/* to represent enumeration type params. */


/* *******************************************************************
*
* Predefined operator support routines
*
* HISTORY:
*	21-AUG-90 DSB	Created section, added VL_PredefVectorNot
*	17-SEP-90 DSB	Added and, nand, or, nor, xor
*	31-JAN-91 DSB	Changed from byte size to scalar count
*
******************************************************************* */

VL_tEnumArrayAddr VL_PredefVectorNot(	/* implicitly defined NOT on vector */
#ifdef VK_ANSI_FN_PROTOS
	VL_tEnumArrayAddr result,	/* &out: result array address */
	VL_tEnumArrayAddr argument,	/* &in: operand array address */
	VK_tElementCount array_length	/* in: length of arrays */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns address of result buffer */


VL_tEnumArrayAddr VL_PredefVectorAnd(	/* implicitly defined AND on vector */
#ifdef VK_ANSI_FN_PROTOS
	VL_tEnumArrayAddr result,	/* &out: result array address */
	VL_tEnumArrayAddr left_arg,	/* &in: operand array address */
	VL_tEnumArrayAddr right_arg,	/* &in: operand array address */
	VK_tElementCount array_length	/* in: length of arrays */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns address of result buffer */


VL_tEnumArrayAddr VL_PredefVectorNand(	/* implicitly defined NAND on vector */
#ifdef VK_ANSI_FN_PROTOS
	VL_tEnumArrayAddr result,	/* &out: result array address */
	VL_tEnumArrayAddr left_arg,	/* &in: operand array address */
	VL_tEnumArrayAddr right_arg,	/* &in: operand array address */
	VK_tElementCount array_length	/* in: length of arrays */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns address of result buffer */


VL_tEnumArrayAddr VL_PredefVectorOr(	/* implicitly defined OR on vector */
#ifdef VK_ANSI_FN_PROTOS
	VL_tEnumArrayAddr result,	/* &out: result array address */
	VL_tEnumArrayAddr left_arg,	/* &in: operand array address */
	VL_tEnumArrayAddr right_arg,	/* &in: operand array address */
	VK_tElementCount array_length	/* in: length of arrays */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns address of result buffer */


VL_tEnumArrayAddr VL_PredefVectorNor(	/* implicitly defined NOR on vector */
#ifdef VK_ANSI_FN_PROTOS
	VL_tEnumArrayAddr result,	/* &out: result array address */
	VL_tEnumArrayAddr left_arg,	/* &in: operand array address */
	VL_tEnumArrayAddr right_arg,	/* &in: operand array address */
	VK_tElementCount array_length	/* in: length of arrays */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns address of result buffer */


VL_tEnumArrayAddr VL_PredefVectorXor(	/* implicitly defined NOR on vector */
#ifdef VK_ANSI_FN_PROTOS
	VL_tEnumArrayAddr result,	/* &out: result buffer address */
	VL_tEnumArrayAddr left_arg,	/* &in: operand array address */
	VL_tEnumArrayAddr right_arg,	/* &in: operand array address */
	VK_tElementCount array_length	/* in: length of arrays */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns address of result buffer */

VK_tCompositeAddr VL_PredefVectorConcat(
					/* for implicitly defined & on array */
#ifdef VK_ANSI_FN_PROTOS
	VK_tDiscreteRangePtr result_constraint,
					/* &out: result value constraint */
	VK_tDiscreteRangePtr left_constraint,
					/* &in: left argument constraint */
	VK_tCompositeAddr left_ptr,	/* &in: left argument value */
	VK_tDiscreteRangePtr right_constraint,
					/* &in: right argument constraint */
	VK_tCompositeAddr right_ptr,	/* &in: right argument value */
	VK_tDataLength element_length	/* in: word size of element */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns address of result on heap */


VK_tBoolean VL_PredefVectorRelational(	/* for impl. defined <, >, <=, >= */
#ifdef VK_ANSI_FN_PROTOS
	VK_tBoolean also_equals,	/* in: selects <= vs. < */
	VK_tElementCount left_length,	/* in: length of left vector */
	VL_tEnumArrayAddr left_ptr,	/* &in: left vector value */
	VK_tElementCount right_length,	/* in: length of right vector */
	VL_tEnumArrayAddr right_ptr	/* &in: right vector value */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns True if left </<= right */



VK_tBoolean VL_PredefCompositeEquality(	/* implicitly defined =, /= */
#ifdef VK_ANSI_FN_PROTOS
	VK_tCompositeAddr left_ptr,	/* &in: left argument value */
	VK_tCompositeAddr right_ptr,	/* &in: right argument value */
	VK_tDataLength length		/* in: whole value word length */
#endif /* VK_ANSI_FN_PROTOS */
	);				/* returns True if equal */


#endif /* VLLogOpsP_H */
