#ifndef VKBasicTypes_H
#define VKBasicTypes_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKBasicTypes.h
*
* DESCRIPTION:
*	Types for VHDL data and common types.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
*	21-FEB-91 DSB 	Split out of VKCommonP.h
*	15-DEC-89 DSB	Created
*	21-SEP-90 AWH	Changed signal handles to scalar signal numbers
*	09-OCT-90 DSB	Cleaning up: moved VK_tScalarData, etc., to here from
*			VKSignals.h;
*	...
*	26-MAR-91 DSB	Cleaning; merge VKCommonP.h stuff to here
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength
*	10-DEC-91 DSB	Removed erroneous comment that integer values for
*			VK_tDirection's enumerators may be changed
*
* SCCS ID: @(#)VKBasicTypes.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"

/* *******************************************************************
*
* 1: Basic types used throughout the VKI
*
******************************************************************* */

typedef char * VK_tCString;		/* C strings for certain parameters */


/* *******************************************************************
*
* type VK_tVoidPtr - general pointer with unknown designated type
*
******************************************************************* */

#ifdef VK_ANSI_VOID_STAR
  typedef void * VK_tVoidPtr;		/* general pointer */
#else
  typedef char * VK_tVoidPtr;		/* general pointer */
#endif


/* *******************************************************************
*
* type VK_tInteger - general integer type used to define other integer types
*
******************************************************************* */

typedef long int VK_tInteger;		/* default impl. is long int */


/* *******************************************************************
*
* types for sizes, lengths, etc.
*
******************************************************************* */

typedef VK_tInteger VK_tWordSize;	/* length in words (long int) */
					/* usage: to define more VKI types 
					   only */

typedef VK_tWordSize VK_tWordOffset;	/* offset in words */
					/* usage: to define more VKI types 
					   only */

typedef VK_tInteger VK_tByteSize;	/* size in bytes (char) */

typedef VK_tInteger VK_tListLength;	/* variable arguments n-tuple count */

typedef VK_tInteger VK_tArraySize;	/* element count of non-VHDL array */



/* *******************************************************************
*
* 2: types for VHDL data 
*
******************************************************************* */

/* *******************************************************************
*
* 2.1: basic types for values of VHDL scalar types
*
* NOTES:
*   Assumption: float, long int, and pointers are all the same length 
*	(normally 32 bits).
*
******************************************************************* */

#ifdef VK_LITTLE_ENDIAN			/* match word order to byte order */
  typedef struct {
	unsigned long int low;		/* -least significant word */
	/* signed */ long int high;	/* -most significant word */
    } VK_tLargeData;			/* VHDL large physical type data */
#else
#ifdef VK_BIG_ENDIAN
  typedef struct {
	/* signed */ long int high;	/* -most significant word */
	unsigned long int low;		/* -least significant word */
    } VK_tLargeData;			/* VHDL large physical type data */
#else
  #error Neither VK_BIG_ENDIAN nor VK_LITTLE_ENDIAN is defined
#endif
#endif


typedef long int VK_tIntegerData;	/* VHDL scalar data when known to be
					   represented as integer (integer, 
					   enumeration, or small physical 
					   type) */
					/* Usage: code, kernel */

typedef float VK_tFloatingData;		/* VHDL scalar data when known to be 
					   represented as floating point */
					/* Usage: code, kernel */

typedef long int VK_tScalarData;	/* normal-size VHDL scalar data
					   (integer or floating point) */
					/* Usage: code, kernel */


typedef long int VK_tIntegerValue;	/* VHDL integer value as parameter;
					   to parallel VK_tFloatingValue */
					/* Usage: VKI only */

typedef double VK_tFloatingValue;	/* VHDL floating value as parameter;
					   for ANSI fn. prototype parameters */
					/* Usage: VKI only */

/* *******************************************************************
*
* 2.2: types for data addresses and for composite types
*
******************************************************************* */

typedef VK_tScalarData * VK_tScalarAddr;
					/* address of scalar data */
					/* Usage: code, kernel */

typedef VK_tLargeData * VK_tLargeAddr;	/* address of large scalar data */
					/* Usage: code, kernel */

typedef VK_tScalarData * VK_tCompositeAddr;
					/* address of composite data (may 
					   include small or large elements) */
					/* Usage: code, kernel */

typedef VK_tScalarData * VK_tDataAddr;	/* address of data of unspecified
				 	   class (scalar, large, composite) */

typedef VK_tDataAddr * VK_tAddrAddr;	/* address of pointer to data */




/* *******************************************************************
*
* 2.3: types for data sizes
*
******************************************************************* */


typedef VK_tWordSize VK_tDataLength;	/* VHDL composite data size in words */

typedef VK_tWordOffset VK_tDataOffset;	/* special data offset in words */


typedef VK_tInteger VK_tElementCount;	/* VHDL array elements count (not 
					   necessarily scalar subelements) */


/* *******************************************************************
*
* 2.4: types for VHDL constraints
*
******************************************************************* */

/* *******************************************************************
*
* type VK_tDirection - direction of range
*
* USAGE: Defined by VKI; used by generated code; used by kernel.
*
* NOTE: DO NOT change values.
*
******************************************************************* */

#ifdef VK_ANSI_ENUM
  typedef enum {
	VK_BadDirection =  0,	        /* -(not required by VKI) */
	VK_Descending   = -1, 		/* -descending range ("downto") */
	VK_Ascending    =  1		/* -ascending range ("to") */
    } VK_tDirection;			/* direction of range */
#else
  typedef long int VK_tDirection;	/* direction of range */
#  define VK_Descending -1		/* -descending range ("downto") */
#  define VK_BadDirection 0		/* -(not required by VKI) */
#  define VK_Ascending 1		/* -ascending range ("to") */
#endif


/* *******************************************************************
*
* types VK_tXxxRangeConstraint - scalar constraints
*
* USAGE: Defined by VKI; used by generated code; used by kernel.
*
******************************************************************* */

typedef struct {
	VK_tIntegerData left;
	VK_tDirection dir;
	VK_tIntegerData right;
    } VK_tIntegerRangeConstraint;	/* range constraint for VHDL integer,
					   enumeration, or small physical 
					   type */

typedef struct {
	VK_tFloatingData left;
	VK_tDirection dir;
	VK_tFloatingData right;
    } VK_tFloatingRangeConstraint;	/* range constraint for VHDL floating
					   type */

typedef struct {
	VK_tLargeData left;
	VK_tDirection dir;
	VK_tLargeData right;
    } VK_tLargeRangeConstraint;		/* range constraint for VHDL large
					   physical type */


/* *******************************************************************
*
* types: VK_tDiscreteRange, VK_tDiscreteRangePtr, etc.
*
* USAGE: Defined by VKI; used by generated code; used by kernel.
*
******************************************************************* */


typedef struct {
	VK_tIntegerData left;		/* -left bound; discrete type */
	VK_tDirection dir;		/* -direction */
	VK_tIntegerData right;		/* -right bound; discrete type */
    } VK_tDiscreteRange;		/* range record for discrete type */

typedef VK_tDiscreteRange * VK_tDiscreteRangePtr;
					/* pointer to range record */

/* generate: VK_tDiscreteRangePtr <field_name>[<dimensionality>]; */
					/* index constraint (array of discrete
					   ranges) */

typedef VK_tDiscreteRange * VK_tIndexConstraintAddr;
					/* pointer to range array */

typedef VK_tVoidPtr VK_tAnyConstraintAddr;
					/* pointer to range constraint record
					   or to index constraint (array of
					   records) */


/* *******************************************************************
*
* 3: Types for specific versions of VHDL data used in VKI functions
*
******************************************************************* */

typedef VK_tIntegerData * VK_tVHDLString;
					/* for standard type STRING; is array
					   of integer (enumeration) data */

typedef VK_tElementCount VK_tStringLength;
					/* length of VHDL string */


/* *******************************************************************
*
* type VK_tBoolean - representation of VHDL type Std.Standard.BOOLEAN
*
******************************************************************* */

typedef VK_tIntegerValue VK_tBoolean;	/* for standard type BOOLEAN; is 
					   integer representing enumeration */
#ifndef True
#  define True 1
#  define False 0
#endif


/* *******************************************************************
*
* Time types - implementation of values of VHDL type STD.Standard.TIME
*
* Note: Absolute and relative times are defined separately, in case they
*	are implemented differently in the future. 
*
******************************************************************* */


typedef VK_tLargeData VK_tTimeBuffer;	/* storage for value of standard type
					   TIME: large integer representing 
					   physical value */
typedef VK_tLargeAddr VK_tTimeBufferPtr;
					/* pointer to value of type TIME */

					/* relative times (intervals) */
typedef VK_tTimeBuffer VK_tRelativeTimeBuf;
typedef VK_tTimeBuffer * VK_tRelativeTimePtr;

					/* absolute simulation times */
typedef VK_tTimeBuffer VK_tAbsoluteTimeBuf;
typedef VK_tTimeBuffer * VK_tAbsoluteTimePtr;


/* *******************************************************************
*
* 4: "Forward" declarations of various object identification types (these 
*	types logically belong with other modules, but are here because of 
*	somewhat recursive dependencies)
*
******************************************************************* */


/* *******************************************************************
*
* 4.1: signal and driver identification types
*
******************************************************************* */

/* *******************************************************************
*
* type VK_tSignalHandle - handle to whole signal
*
******************************************************************* */

typedef struct VK_sSignal * VK_tSignalHandle;


/* *******************************************************************
*
* type VK_tSignalOffset - word offset to portion of signal
*
******************************************************************* */

typedef VK_tWordOffset VK_tSignalOffset;


/* *******************************************************************
*
* type VK_tSignalLength - length in words of a signal or driver portion
*
******************************************************************* */

typedef VK_tWordSize VK_tSignalLength;


/* *******************************************************************
*
* type VK_tDriverHandle - handle to whole driver
*
******************************************************************* */

typedef struct VK_sDriver * VK_tDriverHandle;


/* *******************************************************************
*
* type VK_tDriverOffset - word offset to portion of driver
*
******************************************************************* */

typedef VK_tWordOffset VK_tDriverOffset;


/* *******************************************************************
*
* 4.2: process and sensitivity set identification types
*
******************************************************************* */

/* *******************************************************************
*
* type VK_tProcessHandle - handle to process
*
******************************************************************* */

typedef struct VK_sProcess * VK_tProcessHandle;


/* *******************************************************************
*
* type VK_tSensitivitySetHandle - handle to sensitivity set
*
******************************************************************* */

typedef struct VK_sSensitivitySet * VK_tSensitivitySetHandle;


#endif /* VKBasicTypes_H */
