#ifndef VKCodeIntf_H
#define VKCodeIntf_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKCodeIntf.h
*
* DESCRIPTION:
*	Types from the generated code needed by the kernel for calling 
*	generated code functions (for elaboration, execution, signal 
*	resolution, or port type conversion) or for debugging.
*
* CONSTANTS:
*	...
*
* TYPES:
*	...
*
* GLOBAL VARIABLES:
*	...
*
* ROUTINES:
*	...
*
* HISTORY:
*	27-FEB-91 DSB	Created
*	19-MAR-91 DSB	Renamed VKFixedP.h to VKCodeIntf.h
*	02-APR-91 DSB	Fixed missing value handling in VK_ResolveScalarSignal
*	09-MAY-91 DSB	Fixed VL_RunProcess to VK_RunProcess_Function
*	14-MAY-91 DSB	Fixed type in 09-MAY change
*	16-MAY-91 DSB	Added TK_ModelName (for test kernel only)
*	05-AUG-91 DSB	Renamed VK_RunProcess... to VC_RunProcess...
*	08-AUG-91 DSB	Removed VK_tTopIAR
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*
* SCCS ID: @(#)VKCodeIntf.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"	/* for VK_Boolean */
#include "VKSymbTypes.h"	/* for VK_tSymbInfoHandle */


/* *******************************************************************
*
* NOTES:
*   This file contains a variety of declarations.  Some are for the generated
*	code to use, some are for the kernel to use, and some are internal 
*	(for neither to use directly).
*   The kernel must not use any declarations except those designated for
*	direct use by the kernel.
*
******************************************************************* */


/* *******************************************************************
*
* control macro VK_DEBUG_RP - select certain macros vs. real RTL functions
*
* NOTE: used with #if, not #ifdef, so VK_DEBUG_RP should always be defined
*
******************************************************************* */

#ifndef VK_DEBUG_RP
#  define VK_DEBUG_RP 0			/* default to False */
#endif


/* *******************************************************************
*
* 0: TEST KERNEL IMPLEMENTATION ONLY
*
* USAGE:  This is not part of the VKI.
*
******************************************************************* */

#if TK_SPECIFIC
  extern char TK_ModelName[];		/* name of top-level entity */
	/* ?? can we remove this now that the top-level unit is registered 
		with the kernel ?? */
#endif

/* *******************************************************************
*
* ?: Types and macros for access types
*
******************************************************************* */

/* *******************************************************************
*
* Type VK_tAccessDataRec -
*
* USAGE: Used by generated code; used by later types and macros; PRIVATE with 
*	respect to kernel (kernel must not use field names directly)
*
******************************************************************* */

typedef struct {
	VK_tDataAddr data_addr;		/* address of data on heap */
	VK_tDiscreteRange constraint[1];
					/* n-dimensional constraint, or none */
    } VK_tAccessDataRec;
/* Note:  The index constraint does not match this structure declaration 
	exactly.  The constraint may not be present, or it may be more than the
	one dimension implied by the "[1]".  Generated code allocates space as
	necessary according to the number of dimensions (including zero for a 
	non-array type).  */


/* *******************************************************************
*
* Type VK_tAccessDataPtr -
*
* USAGE: Used by generated code; used by later macros; may be used by kernel 
*	for debugging (i.e., kernel may use the fact that this is a pointer
*	type, but may not use the fields from VK_tAccessDataRec, above)
*
******************************************************************* */

typedef VK_tAccessDataRec * VK_tAccessDataPtr;

/* Specification:  A VHDL access value of NULL is represented as a null 
	pointer.  The kernel may use this information.  */
/* Specification:  
For debugging, the kernel may manipulate access objects and objects designated
	by access values, but may not perform VHDL allocation and deallocation.
More specifically:
    -	The kernel may read and modify objects of access types (represented
	as C type VK_tAccessDataRec).  The kernel may copy an existing access 
	value from an access object and store it or NULL in another access 
	object.
    -	The kernel may examine and modify objects designated by access types.
	The kernel must get the address of such objects using 
	VK_GetAccessedValue (and must not use VK_tAccessDataRec's field names
	directly).
    -   The kernel must not allocate or free data of type VK_tAccessDataRec.
    -   The kernel may read the data address field (by using 
	VK_GetAccessedValue) but must not modify it.
    -	The kernel may read or modify the constraint field (by using
	VK_GetAccessedConstraint).
*/



/* *******************************************************************
*
* VK_GetAccessedValue - get access value's designated object's value buffer 
*	address
*
* DESCRIPTION:
*	Given an access value, returns the address of the value buffer (on the
*	heap) of an object designated by the access value.
*
* DECLARATION:
*	VK_tDataAddr VK_GetAccessedValue(VK_tAccessDataPtr access_value);
*
* USAGE: For kernel.
*
* NOTES:
*   *	This is provided for debugging access to the value buffer of the object
*	designated by an access value.
*   *	The kernel is responsible for checking that the access value is not 
*	null.
*
******************************************************************* */

#define VK_GetAccessedValue(access_value) \
	((access_value)->data_addr)	/* get address of value buffer of 
					   designated object */


/* *******************************************************************
*
* VK_GetAccessedConstraint - get access value's designated object's index 
*	constraint address
*
* DESCRIPTION:
*	Given an access value, returns the address of the index constraint 
*	(part of data on the heap) of an array object designated by the access
*	value.
*
* DECLARATION:
*	VK_tIndexConstraintAddr VK_GetAccessedConstraint(VK_tAccessDataPtr 
*		access_value);
*
* USAGE: For kernel.
*
* NOTES:
*   *	This is provided for debugging access to the index constraint of the
*	array object designated by an access value.
*   *	The kernel is responsible for checking that the access value is not 
*	null.
*
******************************************************************* */

#define VK_GetAccessedConstraint(access_value) \
	((access_value)->constraint)	/* get address of designated object's
					   index constraint */


/* *******************************************************************
*
* 1: Types and macros related to activation record and processes
*
******************************************************************* */

/* *******************************************************************
*
* Activation record header pointer types
*
* NOTES:
*	For use by the kernel.
*
******************************************************************* */

typedef struct VK_sIARHeader * VK_tIARHeaderPtr;
					/* interface. activation record 
					   (header) pointer */

typedef struct VK_sLARHeader * VK_tLARHeaderPtr;
					/* local activation record (header) 
					   pointer */


typedef VK_tLARHeaderPtr VK_tContextPtr;
					/* context data for generated code
					   function to be called by kernel */
/* Notes:  For a VHDL resolution or type conversion function, the context is 
	the LAR of the enclosing region of the function; for a wait evaluation
	function, the context is the LAR of the enclosing process statement or
	procedure; for a guard evaluation function, the context is the LAR of
	the associated block statement. */

/* *******************************************************************
*
* 2: Types for generated functions called by the kernel
*
******************************************************************* */


/* *******************************************************************
*
* type VK_tBindFnPtr - entity declaration default binding function pointer
*
* NOTES:
*	Used by VKI functions.
*
******************************************************************* */

typedef VK_tLARHeaderPtr (VK_tBindFn)(	/* elaboration function pointer */
#ifdef VK_ANSI_FN_PROTOS
	VK_tVoidPtr kernel_data,	/* context data from kernel */
	VK_tIARHeaderPtr entity_intf_data
					/* intf. act. rec. for entity to be
					   instantiated */
#endif
	);				/* returns intf. act. rec. address */
typedef VK_tBindFn * VK_tBindFnPtr;



/* *******************************************************************
*
* type VK_tGenericElabFnPtr - design entity elaboration function pointer
*
* NOTES:
*	Used by VKI functions.
*
******************************************************************* */

typedef void (VK_tGenericElabFn)(	/* elaboration function pointer */
#ifdef VK_ANSI_FN_PROTOS
	VK_tIARHeaderPtr IARP
#endif
	);
typedef VK_tGenericElabFn * VK_tGenericElabFnPtr;



/* *******************************************************************
*
* type VK_tGenericEvalFnPtr - VHDL function evaluation function pointer
*
* NOTES:
*	Used by VKI functions.
*
******************************************************************* */

typedef VK_tScalarData VK_tGenericEvalFn(	/* function evaluation func. */
#ifdef VK_ANSI_FN_PROTOS
	VK_tIARHeaderPtr IARP
#endif
	);
typedef VK_tGenericEvalFn * VK_tGenericEvalFnPtr;


/* *******************************************************************
*
* type VK_tExecFnPtr - process or procedure execution function
*
* NOTES:
*	Used by VKI functions.
*
******************************************************************* */

typedef void (VK_tExecFn)(		/* execution function pointer */
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr LARP		/* local activation record pointer */
#endif
	);
typedef VK_tExecFn * VK_tExecFnPtr;


/* *******************************************************************
*
* Declarations for wait statement condition evaluation
*
******************************************************************* */

/* *******************************************************************
*
* type VK_tWaitEvalFnPtr - wait condition evaluation function
*
* NOTES:
*	Used by VKI functions.
*
******************************************************************* */

typedef VK_tBoolean VK_tWaitEvalFn(	/* wait condition eval. func. */
#ifdef VK_ANSI_FN_PROTOS
	VK_tContextPtr execution_context
					/* context record pointer */
#endif
	);				/* returns boolean condition value */
typedef VK_tWaitEvalFn * VK_tWaitEvalFnPtr;


/* *******************************************************************
*
* Declarations for guard evaluation
*
******************************************************************* */

/* *******************************************************************
*
* type VK_tGuardEvalFnPtr - guard expression evaluation function
*
* NOTES:
*	Used by VKI functions.
*
******************************************************************* */

typedef VK_tBoolean (VK_tGuardEvalFn)(
					/* guard expr. eval. function ptr. */
#ifdef VK_ANSI_FN_PROTOS
	VK_tContextPtr evaluation_context
					/* context to evaluation function */
#endif
	);				/* returns boolean expression value */
typedef VK_tGuardEvalFn * VK_tGuardEvalFnPtr;


/* *******************************************************************
*
* 3:  Types and macros for process execution
*
******************************************************************* */

/* *******************************************************************
*
* Process context block types for macros
*
* USAGE: Used by generated code; used by later macros; PRIVATE with respect 
*	to kernel (kernel must not use field names directly)
*
******************************************************************* */

typedef struct {
	VK_tProcessHandle process_id;
	VK_tExecFnPtr function;
	VK_tLARHeaderPtr act_rec_ptr;
	VK_tBoolean suspended;
    } VK_tPCB, * VK_tPCBP;		/* process context block and pointer */


typedef struct {
	VK_tExecFnPtr function;
	VK_tLARHeaderPtr act_rec_ptr;
    } VK_tPCBSave;			/* caller save area */


/* *******************************************************************
*
* Process context management macros for kernel macros
*
* NOTES:
*	PRIVATE with respect to kernel; (used for kernel macros)
*
******************************************************************* */

#define VK_PCB_DidProcessSuspend(process_context_pointer) \
	((process_context_pointer)->suspended)
					/* query suspension bit */

#define VK_PCB_ResumeProcess(process_context_pointer) \
	((process_context_pointer)->suspended = False)
					/* clear susp. bit for resumption */

#define VK_PCB_GetCurrentExecFn(process_context_pointer) \
	((process_context_pointer)->function)
					/* get current execution function */


/* *******************************************************************
*
* Process context management macros for kernel
*
* NOTES:
*	Not used by generated code
*
******************************************************************* */


/* *******************************************************************
*
* VK_GetCurrentLAR - get current region activation record from process data
*
* DESCRIPTION:
*	Given the process context block of a process, returns the local 
*	activation record of the region on the top of the subprogram call
*	stack.
*
* DECLARATION:
*	VK_tLARHeaderPtr VK_GetCurrentLAR(VK_tPCBP process_context_pointer);
*
* USAGE: For kernel; for VC_RunProcess macro.
*
* NOTES:
*   *	This is provided for debugging access to the LAR of the current 
*	procedure or process statement for a process.
*
******************************************************************* */

#define VK_GetCurrentLAR(process_context_pointer) \
	((process_context_pointer)->act_rec_ptr)
					/* get top-of-stack lcl. act. rec. */

/* *******************************************************************
*
* VC_RunProcess - run a process until it suspends
*
* DESCRIPTION:
*	Given the process context block of a process, executes the process
*	until the process suspends.
*
* DECLARATION:
*	void VC_RunProcess(VK_tPCBP process_context_pointer);
*
* USAGE: For kernel only.
*
* NOTES:
*   *	The macro version is provided as an optimization.  The function
*	version is provided (in the RVCG run-time library) to help with
*	kernel implementation and debugging.
*
******************************************************************* */
					/* macro version: save call overhead */
#define VC_RunProcess(process_context_pointer) \
	do { \
	    VK_PCB_ResumeProcess(process_context_pointer); \
	    do { \
		(*(VK_PCB_GetCurrentExecFn(process_context_pointer))) \
			(VK_GetCurrentLAR(process_context_pointer)); \
	    } while (! VK_PCB_DidProcessSuspend(process_context_pointer)); \
	} while (False)
/* Note: The outer do...while statement is a trick for creating a
	syntactically robust macro.  It may be removed. */


#if VK_DEBUG_RP				/* to assist kernel debugging, use
					   real function version in RTL */
#  undef VC_RunProcess			/* cancel macro */
#  define VC_RunProcess(process_context_pointer) \
	VC_RunProcess_Function(process_context_pointer)
					/* redirect to real function version
					   in RTL */

void VC_RunProcess_Function(		/* RTL function to run process */
#ifdef VK_ANSI_FN_PROTOS
	VK_tPCBP process_context_pointer
#endif /* VK_ANSI_FN_PROTOS */
	);

#endif


/* *******************************************************************
*
* 4: Activation record header types - 
*
******************************************************************* */

/* *******************************************************************
*
* NOTE:
*	Generated IAR and LAR records types correspond to these headers, but
*	define some fields with different types.  (The generated type 
*	declarations do not use these VKI type declarations).  The kernel 
*	may use these fields in these types, for example for debugging.
*
******************************************************************* */


/* *******************************************************************
*
* type VK_tLARHeader - generic definition of header of generated LAR type
*
* DESCRIPTION:
*	VK_tLARHeader represents the standard fields of a local activation
*	record defined by the generated code.
*
* NOTES:
*   *	This declaration must not be modified.
*
* USAGE:  Provided for kernel macros; kernel must use macros instead of 
*	using predefined field names directly.
*
******************************************************************* */

typedef struct VK_sLARHeader {
	VK_KERNEL_LAR_FIELDS()		/* declare kernel fields */
	VK_tIARHeaderPtr IARP;		/* pointer to associated IAR */
    } VK_tLARHeader;			/* generic header of region's local 
					   activation record */


/* *******************************************************************
*
* type VK_tIARHeader - generic definition of header of generated IAR type
*
* DESCRIPTION:
*	VK_tIARHeader represents the standard fields of a interface activation
*	record defined by the generated code.
*
* NOTES:
*   *	This declaration must not be modified.
*
* USAGE:  Provided for kernel macros; kernel must use macros instead of 
*	using field names directly.
*
******************************************************************* */

typedef struct VK_sIARHeader {
	VK_KERNEL_IAR_FIELDS()		/* declare kernel fields */
	VK_tLARHeaderPtr caller_LARP;	/* "calling" region lcl. act. rec. */
					/* region is caller of subprogram, en-
					   closing region of concurrent state-
					   ment, or component instantiation */
	VK_tLARHeaderPtr context_LARP;
					/* lexical parent region generic LAR */
    } VK_tIARHeader;			/* generic header of region's interface
					   activation record */


/* *******************************************************************
*
* Activation record field macros for kernel
*
* USAGE:  For kernel only (kernel must not use field names directly).
*
******************************************************************* */

#define VK_GetIARP(cur_reg_LARP) \
	((cur_reg_LARP)->IARP)		/* get region's IAR (address) from 
					   region's LAR (address) */

#define VK_GetCallerLARP(cur_reg_IARP) \
	((cur_reg_IARP)->caller_LARP)	/* get caller region's IAR (address)
					   from given region's IAR (addr.) */

#define VK_GetContextLARP(cur_reg_IARP) \
	((cur_reg_IARP)->context_LARP)	/* get context region's IAR (address)
					   from given region's IAR (addr.) */


#define VK_SetIARP(cur_reg_LARP, _iarp) \
	((cur_reg_LARP)->IARP = (_iarp))
					/* set region's IAR (address) from 
					   region's LAR (address) */

#define VK_SetCallerLARP(cur_reg_IARP, LARP) \
	((cur_reg_IARP)->caller_LARP = (LARP))
					/* set caller region LAR address 
					   field in given region's IAR */

#define VK_SetContextLARP(cur_reg_IARP, LARP) \
	((cur_reg_IARP)->context_LARP = (LARP))
					/* set context region LAR address 
					   field in given region's IAR */


/* *******************************************************************
*
* 5: Resolution and type conversion declarations
*
******************************************************************* */


/* *******************************************************************
*
* type VK_tTypeGroup - type and subtype classes for code's calling protocols
*
******************************************************************* */

typedef enum {
	VK_BadTypeGroup,        	/* -(not required) */
	VK_IntegerGroup,		/* -scalar, integer type */
	VK_FloatingGroup,		/* -scalar, floating type */
	VK_LargeGroup,			/* -scalar, large integer type */
	VK_ConstrainedCompositeGroup,	/* -constrained composite type */
	VK_UnconstrainedCompositeGroup	/* -unconstrained composite type */
    } VK_tTypeGroup;			/* type classes for generated code's 
					   calling protocols */
/* Specification:  Integer values do not matter.  Values shown are test 
	kernel's choice. */


/* *******************************************************************
*
* 5.1: Resolution function interface activation record types and macros
*
******************************************************************* */

typedef VK_tGenericEvalFnPtr VK_tAnyResFnPtr;
					/* VHDL resolution functions' C 
					   evaluation function */

/* *******************************************************************
*
* VC_ResolveSignal - call resolution function to compute resolved value
*
* DESCRIPTION:
*	...
*
* USAGE:  Defined by VKI; called by kernel; provided by run-time library.
*
* NOTE:  VC_ResolveSignal is called by the kernel.
*
* HISTORY:
*	26-JUL-91 DSB	Renamed VK_ResolveSignal to VC_ResolveSignal
*	03-JAN-92 DSB	Added left_bound and direction parameters
*
******************************************************************* */
void VC_ResolveSignal(
#ifdef VK_ANSI_FN_PROTOS
				/* resolution function data: */
	VK_tAnyResFnPtr res_func, 	/* resolution function (pointer) */
	VK_tContextPtr context_data,	/* context for resolution function */
	VK_tLARHeaderPtr caller_data,	/* kernel's choice for caller_LARP */
				/* input value data: */
	VK_tIntegerValue left_bound,	/* left bound from index subtype */
	VK_tDirection direction,	/* direction from index subtype */
	VK_tElementCount vector_length,	/* input values vector length */
	VK_tCompositeAddr values, 	/* input values vector address */
				/* output data: */
	VK_tTypeGroup type_group,	/* int./float./large/composite type */
	VK_tDataLength value_length,	/* output buffer length in words */
	VK_tDataAddr output_addr	/* output buffer address */
#endif
	);				/* (no return value) */
/* Specification:  Parameter res_func is the address of the resolution function
	to call. */
/* Specification:  Parameter context_data is the local activation record of
	the appropriate instance of the encloding region of the resolution
	function's function declaration. */
/* Specification:  Parameter caller_data is the kernel's choice of a value to
	be used for the caller_LARP field in the interface activation record
	of the resolution function. (The kernel may see this value again
	during debugging.) */
/* Specification:  Parameters left_bound and direction are left bound and 
	direction of the first (and only) index subtype of the instance of
	the resolution function's parameters' one-dimensional array type. These
	are used, along with vector_length, to set the index constraint of the
	unconstrained parameter to the resolution function.  */
/* Specification:  Parameter vector_length is the number of values to be
	resolved. */
/* Specification:  Parameter values is the address of an array of values to be
	resolved. */
/* Specification:  Parameter type_group specifies whether the element type of
	the array of values, and the return type of the resolution function,
	is an enumeration, integer, or small physical type, a floating-point
	type, a large physical type, or a composite type. */
/* Specification:  Parameter value_length is the length of a value of the
	return type of the resolution function. */
/* Specification:  Parameter output_addr is the address of a buffer into
	which the resolved value should be placed by VC_ResolveSignal. */

/* Note: The kernel will have to use symbolic information and traverse
	activation records to retrieve the resolution function's parameter's 
	array type's index subtype.  */
/* Specification:
- VC_ResolveSignal will assemble an interface activation record using
	context_data, caller_data, and various other data.
- VC_ResolveSignal will call the resolution function.  (The kernel may support
	debugging of the resolution function.)
- VC_ResolveSignal and the resolution function will place the resolved signal
	into the buffer specified by output_addr and value_length.
*/


/* *******************************************************************
*
* 5.2: Type conversion function interface activation record types and macros
*
******************************************************************* */


typedef VK_tGenericEvalFnPtr VK_tAnyTypeConvFnPtr;
					/* VHDL resolution function's C 
					   evaluation function */

/* *******************************************************************
*
* VC_ConvertSignal - call type conversion function to convert value
*
* DESCRIPTION:
*	...
*
* USAGE:  Defined by VKI; called by kernel; provided by run-time library.
*
* NOTE:  VC_ConvertSignal is called by the kernel.
*
* HISTORY:
*	26-JUL-91 DSB	Renamed VK_ConvertSignal to VC_ConvertSignal
*	04-DEC-91 DSB	Removed unneeded output_constraint
*
******************************************************************* */
void VC_ConvertSignal(
#ifdef VK_ANSI_FN_PROTOS
				/* type conversion function data: */
	VK_tAnyTypeConvFnPtr type_conv_func,
				 	/* type conv. function (pointer) */
	VK_tByteSize intf_data_size,	/* byte size of function's interface
					   activation record */
	VK_tContextPtr context_data,	/* context for type conv. function */
	VK_tLARHeaderPtr caller_data,	/* kernel's choice for caller_LARP */
				/* input value data: */
	VK_tTypeGroup input_type_group,	/* type group of input type */
	VK_tArraySize input_dimensions,	/* input type dimensions */
	VK_tIndexConstraintAddr input_constraint,
					/* input value actual constraint */
	VK_tDataAddr input_addr,	/* input value address */
				/* output data: */
	VK_tTypeGroup output_type_group,
					/* type group of output type */
	VK_tArraySize output_dimensions,
					/* output type dimensions */
	VK_tIndexConstraintAddr output_constraint,
					/* required constraint of output 
					   value */
	VK_tDataLength output_length,	/* output value word length */
	VK_tDataAddr output_addr	/* output buffer address */
#endif
	);				/* (no return value) */
/* Specification notes:
- Parameter input_dimensions applies only for an array input type; 
	parameter output_dimensions applies only for an array output type.
- Parameter output_size applies only for an array output type.
- Parameter input_constraint applies only if the function's parameter is of an
	unconstrained array subtype; parameter output_constraint applies only 
	if the function's return type is an unconstrained array subtype.
- VC_ConvertSignal is similar to VC_ResolveSignal, especially for context_data,
	caller_data, output_addr, and output_length.
*/


#endif /* VKCodeIntf_H */
