#ifndef VKFiles_H
#define VKFiles_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKFiles.h
*
* DESCRIPTION:
*	File operations.
*
* CONSTANTS:
*	...
*
* TYPES:
*	...
*
* GLOBAL VARIABLES:
*	...
*
* ROUTINES:
*	...
*
* HISTORY:
*	28-FEB-91 DSB	Created
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*
* SCCS ID: @(#)VKFiles.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"	/* for VK_tCompositeAddr, etc. */
#include "VKSymbTypes.h"	/* for VK_tMode, VK_tSymbInfoHandle */
#include "VKCodeIntf.h"	/* for VK_tLARHeaderPtr */


/* *******************************************************************
*
* type VK_tFileHandle - file identification handle
*
******************************************************************* */

typedef struct VK_sFile * VK_tFileHandle;	/* file handle */


/* *******************************************************************
*
* VK_CreateFile - create VHDL file (elaborate file variable declaration)
*
******************************************************************* */
VK_tFileHandle VK_CreateFile(
#ifdef VK_ANSI_FN_PROTOS
				/* instance identification: */
	VK_tSymbInfoHandle file_info,	/* file declaration identification */
	VK_tLARHeaderPtr encl_region_data,
					/* enclosing region lcl. act. rec. */

				/* file logical name: */
	VK_tStringLength name_length,	/* length of file logical name */
	VK_tVHDLString logical_name,	/* file logical name string */

				/* mode information: */
	VK_tMode file_mode,		/* VHDL mode of file */

				/* size information: */
	VK_tDataLength element_length,	/* element length in words */
	VK_tElementCount element_count	/* element count (zero for 
					   unconstrained file) */

#endif
	);				/* returns: handle to created file */
/* Note: An element count of zero specifies that file is an unconstrained 
	file. */
/* Note:  File mode and size information is redundant.  The file mode is
	provided directly to VK_RegisterLocalFileDecl.  The kernel can 
	compute the sizes from subtype information provided to 
	VK_RegisterFileType and	VK_RegisterLocalFileDecl.  Whether a file is
	constrained is specified to VK_RegisterLocalFileDecl. */


/* *******************************************************************
*
* VK_DestroyFile - destroy file ("de-elaborate" file variable declaration)
*
******************************************************************* */
void VK_DestroyFile(
#ifdef VK_ANSI_FN_PROTOS
	VK_tFileHandle file_handle	/* file identification */
#endif
	);


/* *******************************************************************
*
* VK_WriteFile - write data to file
*
******************************************************************* */
void VK_WriteFile(
#ifdef VK_ANSI_FN_PROTOS
	VK_tFileHandle file_handle,	/* file identification */
	VK_tElementCount value_length,	/* value length in elements (not 
					   scalar subelements); for 
					   unconstrained array type only */
	VK_tDataAddr value_addr		/* value to write to file */
#endif
	);
/* Note: For an constrained file, the kernel is currently responsible for 
	checking that VK_WriteFile's value_length matches VK_CreateFile's 
	element_count */


/* *******************************************************************
*
* VK_ReadFile - read data from file
*
******************************************************************* */
VK_tElementCount VK_ReadFile(
#ifdef VK_ANSI_FN_PROTOS
	VK_tFileHandle file_handle,	/* file identification */
	VK_tElementCount buffer_length,	/* buffer length in elements 
					   (not scalar subelements) */
	VK_tDataAddr buffer_addr	/* file read result buffer */
#endif
	);				/* returns: actual value length in 
					   elements (not scalar subelements);
					   defined only for constrained file */
/* Note: For a constrained file, the kernel is currently responsible for 
	checking that VK_ReadFile's buffer_length matches VK_CreateFile's 
	element_count */
/* Note:  The result is the length of the value in the file, not of the
	possibly partial value read into the buffer. */



/* *******************************************************************
*
* VK_IsEndOfFile - query file status
*
******************************************************************* */
VK_tBoolean VK_IsEndOfFile(
#ifdef VK_ANSI_FN_PROTOS
	VK_tFileHandle file_handle	/* file identification */
#endif
	);				/* returns: True if at end of file */



#endif /* VKFiles_H */
