#ifndef VKMisc_H
#define VKMisc_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKMisc.h
*
* DESCRIPTION:  
*	Miscellaneous VHDL kernel interface routines.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Created
*	09-OCT-90 DSB	Cleaning up VK_IssueAssertionViolation parameters
*	23-AUG-91 DSB	Split out VKElaborate.h
*
* SCCS ID: @(#)VKMisc.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"		/* for -VHDLString, -CString, etc. */
#include "VKCodeIntf.h"		/* for VK_tLARHeaderPtr */


/* *******************************************************************
*
* 1:  Miscellaneous VHDL operations support
*
******************************************************************* */


/* *******************************************************************
*
* VK_ReadSimulationTime - read simulation time
*
******************************************************************* */
void VK_ReadSimulationTime(
#ifdef VK_ANSI_FN_PROTOS
	VK_tAbsoluteTimePtr buffer	/* address of time buffer */
#endif
	);


/* *******************************************************************
*
* VK_tSeverity - assertion violation severity level
*
******************************************************************* */

#ifdef VK_ANSI_ENUM
  typedef enum {
	VK_Note = 0, VK_Warning = 1, VK_Error = 2, VK_Failure = 3 
    } VK_tSeverity;			/* assertion severity */
#else
  typedef VK_tIntegerValue VK_tSeverity;
					/* assertion severity */
#  define VK_Note	0
#  define VK_Warning	1
#  define VK_Error	2
#  define VK_Failure	3
#endif
/* Specification:  Integer values MUST NOT be changed. */


/* *******************************************************************
*
* VK_IssueAssertionViolation - issue VHDL assertion violation through kernel
*
******************************************************************* */
void VK_IssueAssertionViolation(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr current_region_data,
					/* encl. region data */
	VK_tSeverity severity,		/* severity of assertion */
	VK_tStringLength message_length,
					/* length of message */
	VK_tVHDLString message		/* assertion message */
#endif
	);
/* Note:  Note that the message parameter is not a C string (a packed array 
	of characters terminated by an ASCII NUL character.)   It is (a 
	pointer to) an array of VK_tIntegerData that may contain null values.
	Each value is in the range 0 to 127, corresponding to ASCII and to 
	position values of enumeration literals	of type CHARACTER. */



/* *******************************************************************
*
* 2:  VHDL run-time error reporting
*
******************************************************************* */


/* *******************************************************************
*
* type VK_tVHDLErrorCode - codes for VHDL run-time errors
*
* DESCRIPTION:
*	Note:  Error codes beginning with "VL_TP_" denote errors defined 
*	by "VHDL Validation Suite - VHDL LRM Test Points" from Virginia Tech.
*	Error codes are named according to that document's numbering system.
*	A number AABBCD_EEFFGG encodes the chapter (AA), section (BB), 
*	subsection (C),	subsubsection(D), paragraph (EE), and sentence (FF) 
*	numbers, from the VHDL 1076-1987 LRM, plus the test point within the 
*	sentence (GG).  
*
* USAGE:  Type name is defined by VHDL and used by VKI functions; identifiers
*	are used by generated code.  Kernel may add identifiers; may implement
*	with defined macros (instead of enumeration type).
*
******************************************************************* */

typedef enum {
	VK_BadVHDLErrorCode,	/* -(not required) */

			/* specific LRM errors: */
	VK_TP_060100_050401,	/* -null access value prefix */
	VK_TP_070250_030201,	/* -negative exponent illegal for integer */
	VK_TP_070340_020301,	/* -qualified expression subtype check */
	VK_TP_081100_040301,	/* -function not completing via return */
	VK_TP_080400_070101,	/* -variable assignment subtype check (non-
					array, non-aggregate) */
	VK_TP_080400_070102,	/* -variable assignment aggregate subtype 
					check */
	VK_TP_080410_020301,	/* -variable assignment array subtype 
					conversion check (non-aggregate) */
			/* specific non-LRM errors: */
	VK_DivideByZero,	/* -divide (or mod or div) by zero */

			/* miscellaneous errors: */
	VK_OtherVHDLError,	/* -(temporary catch-all error code) */
	VK_OtherSystemError,	/* -(temporary catch-all error code) */
	VK_LastVHDLErrorCode	/* -(just for convenience) */
    } VK_tVHDLErrorCode;		/* VHDL/RVCG run-time error code */

/* NOTE: Various other error codes will be filled in.  VK_OtherError will
	be used when a needed error code is not yet defined. */

/* Specification:  Integer values do not matter. */



/* *******************************************************************
*
* VK_ReportError - report simulation error from generated code
*
*
******************************************************************* */
void VK_ReportError(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr current_region_data,
					/* encl. region data */
	VK_tVHDLErrorCode error_code,	/* error code */
	VK_tCString message,		/* main error message */
	VK_tCString extra		/* extra information */

#endif
	);
/* Note:  Parameter current_region_data may be null if the current region's 
	data is not available (e.g., if VK_ReportError is called from the
	run-time library).  In this case, the kernel can retrieve it if the
	kernel saves it from VK_TraceCall/VK_TraceBegin. */
/* Note:  Parameter current_region_data may point to an initialized but
	unregistered local activation record (i.e., kernel-defined fields
	are set to NULL).  Currently, this occurs only in the default binding
	function for an entity. */


/* *******************************************************************
*
* 3:  Memory management functions
*
******************************************************************* */


/* *******************************************************************
*
* VK_Alloc - allocate memory through kernel
*
******************************************************************* */
VK_tVoidPtr VK_Alloc(			/* allocate memory */
#ifdef VK_ANSI_FN_PROTOS
	VK_tByteSize size		/* in: number of bytes needed */
#endif
	);				/* returns address of memory */
/* Specification: Parameter size may be zero, in which case the generated code
	will not dereference the returned pointer value.  (Therefore, VK_Alloc
	may return a null pointer.) */


/* *******************************************************************
*
* VK_Free - free memory through kernel
*
******************************************************************* */
void VK_Free(				/* free memory */
#ifdef VK_ANSI_FN_PROTOS
	VK_tVoidPtr ptr			/* in: address of memory to free */
#endif
	);


#endif /* VKMisc_H */







