#ifndef VKProcesses_H
#define VKProcesses_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKProcesses.h
*
* DESCRIPTION:  
*	Process creation and sensitivity routines.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
*	15-DEC-89 DSB	Created
*	20-DEC-89 DSB	Changed sensitivity set interface
*	12-APR-91 DSB	Removed else_ parameters
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*
* SCCS ID: @(#)VKProcesses.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"	/* for VK_tBoolean, etc. */
#include "VKCodeIntf.h"		/* for VK_tLARHeaderPtr, etc. */


/* *******************************************************************
*
* VK_CreateProcess - create process for VHDL process statement
*
* HISTORY:
*	12-MAR-91 DSB	Replaced process_name parameter with process_info
*	21-MAR-91 DSB	Renamed VK_CreateProcessStatement to VK_CreateProcess
*	12-APR-91 DSB	Removed else_ parameter
*
******************************************************************* */
VK_tProcessHandle VK_CreateProcess(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle process_info,		
					/* process statement identification */
	VK_tLARHeaderPtr process_data,	/* process statement lcl. act. rec. */
	VK_tPCBP process_context	/* process context block */
#endif
	);

/* Specificiation:  VK_CreateProcess called after VK_TraceBegin is called for 
	a process statement. */


/* *******************************************************************
*
* Process sensitivity set functions 
*
******************************************************************* */


/* *******************************************************************
*
* VK_CreateSensitivitySet - create sensitivity set
*
* HISTORY:
*	27-NOV-91 DSB	Removed else_ parameter
*
******************************************************************* */
VK_tSensitivitySetHandle VK_CreateSensitivitySet(
#ifdef VK_ANSI_FN_PROTOS
	VK_tProcessHandle process_id	/* process sensitive to set */
#endif
	);				/* returns sensitivity set handle */
/* Note:  Parameter process_id is null for a guard expression sensitivity
	set. */
/* Note:  VK_CreateSensitivitySet is called during static elaboration for a
	wait statement in a process.  It is called during dynamic elaboration 
	(during simulation) for a wait statement in a procedure sensitivity 
	set. */


/* *******************************************************************
*
* VK_SensitizeSet - sensitize sensitivity set to list of signals
*
******************************************************************* */

#ifndef TK_ANSI_VAR_ARGS		/* for test kernel compilation
					   problem only */
  void VK_SensitizeSet();
#else

  void VK_SensitizeSet(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSensitivitySetHandle sens_set,
					/* set to sensitize */
	VK_tProcessHandle process_id,	/* handle of relevant process */
	VK_tListLength list_length,	/* number of signals listed */
				/* first signal: */
	VK_tSignalHandle signal_handle_1,
					/* signal handle */
	VK_tSignalOffset signal_offset_1,	/* signal offset */
        VK_tSignalLength signal_length_1,
					/* signal length */
	...			/* additional signals: */
					/* repeat triplets for each  */
		       			/* additional signal */
#endif
	); 
#endif

/* Note:  Parameter process_id is null for a guard expression sensitivity
	set. */


/* *******************************************************************
*
* VK_DestroySensitivitySet - destroy sensitivity set
*
******************************************************************* */
void VK_DestroySensitivitySet(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSensitivitySetHandle sens_set,
					/* handle of set to destroy */
	VK_tProcessHandle process_id	/* handle of relevant process */
#endif
	);
/* Note:  Called for procedure sensitivity sets only. */


/* *******************************************************************
*
* Wait statement function
*
******************************************************************* */


/* *******************************************************************
*
* VK_SetWaitConditions - set process sensitivity for wait statement
*
* HISTORY:
*	16-JUL-90 DSB	Added wait evaluation function parameters
*	16-JUL-90 DSB	Renamed VK_SetWaitSensitivity to VK_SetWaitConditions;
*			removed timed_wait parameter; using null time parameter
*
******************************************************************* */
void VK_SetWaitConditions(
#ifdef VK_ANSI_FN_PROTOS
	VK_tProcessHandle process_id,	/* id. of process to sensitize */
	VK_tSensitivitySetHandle sens_set,
					/* sensitivity list for wait */
					/* -may be NULL if no sensitivity */
	VK_tWaitEvalFnPtr wait_function,
					/* wait condition evaluation fn. */
					/* -may be NULL if no condition */
	VK_tContextPtr wait_context,	/* wait evaluation fn. context */
					/* -may be NULL if no condition */
	VK_tRelativeTimePtr time	/* "for" clause time */
					/* -is NULL if no timeout */
#endif
	);
/* Specification:  Currently, the kernel is responsible for checking that the 
	time value is non-negative (080100_050301). */

#endif /* VKProcesses_H */
