#ifndef VKRegister_H
#define VKRegister_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKRegister.h
*
* DESCRIPTION:
*	Definition registration functions for Symbolic Information Interface.
*
* CONSTANTS:
*	...
*
* TYPES:
*	...
*
* GLOBAL VARIABLES:
*	none
*
* ROUTINES:
*	...
*
* HISTORY:
*	19-FEB-91 DSB	Created
*	11-MAR-91 DSB	Refining VKI design
*	15-MAR-91 DSB	Removed type declarations to VKSymbTypes.h
*	02-APR-91 DSB	Removed byte_size_offset, scalar_count_offset for now
*	09-APR-91 DSB	Added break_table for design units
*	11-APR-91 DSB	Modified full order (item-specific groups); folded
*			stmt_pos and elab_pos names to elab_pos
*	12-APR-91 DSB	Applied VK_XxxArrayAddr to enum. and phys. types
*	22-APR-91 DSB	Added analysis time for architecture
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*	17-MAY-91 DSB	Fixed typo: VK_RegisterSubypeDecl
*	22-MAY-91 DSB	Fixed VK_RegisterInstantiationStmt, VK_Register-
*			ConfigurationSpec
*	23-MAY-91 DSB	Added VK_RegisterFunctionBody return subtype info.
*	11-JUL-91 DSB	Added VK_RegisterConfigurationDeclaration elaboration
*			function
*	23-JUL-91 DSB	Added intf_data_size to VK_RegisterEntityDecl
*	24-JUL-91 DSB	Added binding_function to VK_RegisterEntityDecl
*	03-AUG-91 DSB	Added unconstrained bit for interface objects, etc.
*
* SCCS ID: @(#)VKRegister.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"	/* for VK_tIntegerValue, etc. */
#include "VKCodeIntf.h"	/* for VK_tAnyResFnPtr */
#include "VKSymbTypes.h"	/* for symbolic information types */



/* *******************************************************************

* Patterns, Hints, Ideas:

** Coordination of parameters for registration functions:
	(Registration function parameters are coordinated to yield consistent
	partial ordering requirements.)
  * Identification information (of most items):
    *   (Item type is identified by registration function name)
    *   A primary design unit has the library name.
    *   A region extension has an extended region.  
    (*  An implementation of an extended region might need a list of 
	extensions.)
    *	Any item except a design unit has an enclosing region. (A configuration
	specification has a configured component instantiation.)
    *   An item having an enclosing region has an item number in that region.
	(This is for optimization.  An item number of -1 means not implemented
	yet.  Currently, no item numbers are implemented yet.)  (Item numbers
	might be used to allow optimization using an array instead of a list of
	information of enclosed items.)
    *	In most cases, an item has the simple name of the item.
    *   An attribute specification has an attributed item and an attribute
	declaration; an alias declaration has as aliased item.
  * Execution information (of unit, of region, of declaration/statement):
    *	A design unit has the source file name.
    *   A design unit has an execution position count, an execution position
	identification (trace) table, and an execution control (breakpoint)
	table.
    *   Most declarations and concurrent statements have an execution position
	within the unit's execution tables.  (An execution position of -1
	means not implemented yet.)
    *   Most regions have a beginning and ending execution position within
	the unit's execution tables.
  * Region information:
    *   A region has a count of enclosed items.  (An item count of -1 means
	not implemented yet.  Currently, enclosed item counts are not 
	implemented yet.  See item numbers, above.)
    (*  An implementation of a region might need a list of enclosed items.)
  * Type and subtype information (of object, type, subtype, or function):
    *   An object, array type, record element, or subtype has a base type.
	(For array type, it is element base type.)
    *   An object, type, or subtype has constraint path and offset information.
    *   A signal, array type, record element, or subtype has a resolution 
	function.
    *   A function or interface object has a bit indicating whether is was 
	declared with an unconstrained subtype.
  * Item-specific information:
   -(object information)
    *   An non-signal object (or alias) has value field offset information.
    *   A signal (or signal alias) has signal handle field offset information.
    *   A signal (or alias) has signal offset field offset information.

    *   An interface object has an interface mode.
    *   A file has a file mode.
    *   A signal (local or interface) has a signal kind.

   -(type information)
    *	An integer, physical, or floating type has bounds information.

    +(these three are for enumeration and physical literal display)
    *   An enumeration type or physical type has a count of enumeration
	literals or physical units.
    *   An enumeration type has enumeration literal strings.
    *	A physical type has physical unit strings and position values.

    +(these two plus four are for composite types)
    *	An array type has the number of dimensions.
    *	A record type has the number of record elements.

   -(misc. information)

    +(these three are for X/parent/procedure/Y checks)
    *   A process statement has a sensitivity list presence bit.
    *   A procedure body has a wait statement presence bit.
    *   (Registered separately):  A procedure has information about non-local
	references.
    *   (Registered separately):  A process or subprogram body has information
	about called subprograms.

    +(more misc.)
    *   A block, if generate, or for generate statement has the offset of its
	interface activation (IAR) record field in the enclosing region's local
	activation record (LAR); a component instantiation statement has the 
	offset if its component's IAR field in the enclosing record of the
	instantiation, and configuration specification has the offset of its
	entity's IAR field in the LAR of the configuration specification.
    *   An entity or function declaration has an interface activation record 
	size (especially for calling type conversion functions or elaborating a
	default entity).
    *   An architecture body or a configuration declaration has an elaboration
	function.
    *   A function body has an evaluation function (for signal resolution and 
	type conversion).
    *   An architecture body has an analysis time.
    *   A component instantiation has an instantiated component declaration; 
	a configuration specification has an instantiated unit.
    *	A block statement has a guard expression bit.


******************************************************************* */

/* *******************************************************************

* Protocols and common parameters:

** Static constant arguments for string parameters:
    For any VK_RegisterXxx function parameter that is a C string, the actual
	argument will be a static constant string.  (Therefore, the kernel may
	save the address of the string, without copying the string).

** Enclosed item count and item numbers.
    These are not used yet, and are not yet provided by the generated code.

** Path to constraint for subtype of item.
    A constraint for an item (an object, type, index type, or record element) 
	is stored at some offset in the interface or local activation record
	for some region.  The offset is given directly.  The region is 
	specified directly.  However, whether the offset applies to the 
	interface activation record or the local activation record is given 
	implicitly:
    If the constraint region is not the enclosing region of the item, then the
	constraint is stored in the local activation record of the indicated 
	region.  If the indicated region is the enclosing region, then for an 
	interface object the constraint is stored in the interface activation 
	record of the enclosing region, and for a local item it is stored in 
	the local activation record.
    Note:  A local object declared with a constraint is represented in the same
	way as a local object declared using a subtype declared in the same
	region.  (The kernel can not tell the difference, but it should not 
	need to.)

** Unconstrained objects.
    There are two types of unconstrained objects:  those that are eventually
	constrained (when they are elaborated) (e.g., subprogram parameters, 
	ports, deferred constants), and those that are permanently uncon-
	strained (e.g., access types, access objects, file objects).
    An non-file, non-access interface object is always constrained by the time 
	it is created.  Such an eventually-constrained object always has a 
	constraint field.  For a port or generic declared as unconstrained, 
	the eventual constraint is stored in the interface activation record, 
	as for a constrained port or generic.  For a subprogram parameter, the
	eventual constraint is also stored in the interface activation record.
	This also applies to the result value returned by an unconstrained
	function; the constraint of the actual value is stored in the interface
	activation record when the function completes.
    A permanently unconstrained type or object is represented with a null 
	constraint_region parameter.  This applies to access types, file 
	types, and file variables.  

** Field offsets.
    Each field offset parameter is a byte offset into the interface
	activation record or local activation record for a region.  For an 
	interface object, the value offset applies to the interface activation
	record.  For a local object or type, the value offset applies to the 
	local activation record.

******************************************************************* */


/* *******************************************************************
*
* 1: Registration functions for regions
*
******************************************************************* */

/* *******************************************************************
*
* 1.1: Registration functions for design units
*
******************************************************************* */

/* Note:  For a design unit, the beginning execution position index is zero 
	and the ending index is "exec_pos_count - 1". */

/* *******************************************************************
*
* VK_RegisterPackageDecl - register package declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterPackageDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
					/* (no enclosing region) */
	VK_tName library_name,		/* VHDL library name */
	VK_tName package_name,		/* package declaration simple name */
 
				/* execution information: */
	VK_tCString source_file_name,	/* VHDL source file name */
	VK_tExecPosIndex exec_pos_count,
					/* execution tables sizes */
	VK_tTraceTable trace_table,	/* elaboration/execution pos. table */
	VK_tBreakTable break_table,	/* execution control table */

				/* region information: */
	VK_tEnclItemNumber encl_item_count
					/* number of enclosed items */

				/* package-specific information: */
					/* nothing */
#endif
	);
/* Specification:  (Currently:)  Parameter library_name includes the "dls_"
	prefix.  The capitalization of the string is not defined. */


/* *******************************************************************
*
* VK_RegisterPackageBody - register package body
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterPackageBody(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle package_handle,
					/* extended package decl. ident. */
					/* (no package body name) */

				/* execution information: */
	VK_tCString source_file_name,	/* VHDL source file name */
	VK_tExecPosIndex exec_pos_count,
					/* execution tables sizes */
	VK_tTraceTable trace_table,	/* elaboration/execution pos. table */
	VK_tBreakTable break_table,	/* execution control table */

				/* region information: */
	VK_tEnclItemNumber encl_item_count
					/* number of enclosed items */
 
				/* package-body-specific information: */
					/* nothing */
#endif
	);


/* *******************************************************************
*
* VK_RegisterEntityDecl - register entity declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterEntityDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
					/* (no enclosing region) */
	VK_tName library_name,		/* VHDL library name */
	VK_tName entity_name,		/* entity declaration simple name */

				/* execution information: */
	VK_tCString source_file_name,	/* VHDL source file name */
	VK_tExecPosIndex exec_pos_count,
					/* execution tables sizes */
	VK_tTraceTable trace_table,	/* elaboration/execution pos. table */
	VK_tBreakTable break_table,	/* execution control table */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */

				/* entity-specific information: */
	VK_tByteSize intf_data_size,	/* byte size of interface activation
					   record */
	VK_tBindFnPtr binding_function	/* special function for default
					   binding indication */
#endif
	);
/* Specification:  (Currently:)  Parameter library_name includes the "dls_"
	prefix.  The capitalization of the string is not defined. */


/* *******************************************************************
*
* VK_RegisterArchitectureBody - register architecture body
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterArchitectureBody(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle extended_entity,	
					/* extended entity identification */
	VK_tName architecture_name,	/* architecture body simple name */

				/* execution information: */
	VK_tCString source_file_name,	/* VHDL source file name */
	VK_tExecPosIndex exec_pos_count,
					/* execution tables sizes */
	VK_tTraceTable trace_table,	/* elaboration/execution pos. table */
	VK_tBreakTable break_table,	/* execution control table */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */

				/* architecture-specific information: */
	VK_tGenericElabFnPtr elaboration_function,
					/* design entity elaboration fn. */
	VK_tTimeStampPtr analysis_time	/* time and date unit was analyzed */
#endif
	);


/* *******************************************************************
*
* VK_RegisterConfigurationDecl - register configuration declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterConfigurationDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle extended_architecture,
					/* extended entity/arch pair ident. */
	VK_tName configuration_name,	/* configuration body simple name */

				/* execution information: */
	VK_tCString source_file_name,	/* VHDL source file name */
	VK_tExecPosIndex exec_pos_count,
					/* execution tables sizes */
	VK_tTraceTable trace_table,	/* elaboration/execution pos. table */
	VK_tBreakTable break_table,	/* execution control table */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */

				/* configuration-specific information: */
	VK_tGenericElabFnPtr elaboration_function
					/* design entity elaboration fn. */
#endif
	);



/* *******************************************************************
*
* VK_RegisterLibraryClause - register library clause
*
******************************************************************* */
void VK_RegisterLibraryClause(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing unit identification */

	VK_tName symbol_name,		/* library symbol simple name */
 
				/* library-symbol-specific information: */
	VK_tName library_name		/* VHDL library name */

#endif
	);
/* Specification:  (Currently:)  Parameter library_name includes the "dls_"
	prefix.  The capitalization of the string is not defined. */
/* Specification:  VK_RegisterLibraryClause is called to register library
	symbols defined by library clauses for an entity declaration, an
	architecture body, or a configuration declaration. */
/* Note:  Library clause are registered for the kernel's use in performing
	default entity selection. */


/* *******************************************************************
*
* 1.2: Registration functions for main region statements
*
******************************************************************* */


/* *******************************************************************
*
* VK_RegisterBlockStmt - register block statement
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterBlockStmt(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName label_name,		/* block statement label simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* statement elaboration position */
	VK_tExecPosIndex begin_pos,	/* region begin position */
	VK_tExecPosIndex end_pos,	/* region end position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */

				/* block-specific information: */
	VK_tFieldOffset intf_data_offset,	
					/* offset of IAR field in encl. LAR */
	VK_tBoolean has_guard_expression
					/* block has guard expression */
#endif
	);


/* *******************************************************************
*
* VK_RegisterIfGenerateStmt - register if generate statement
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterIfGenerateStmt(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName label_name,		/* generate stmt. label simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* statement elaboration position */
	VK_tExecPosIndex begin_pos,	/* region begin position */
	VK_tExecPosIndex end_pos,	/* region end position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */

				/* if-generate-specific information: */
	VK_tFieldOffset intf_data_offset
					/* offset of IAR field in encl. LAR */
#endif
	);


/* *******************************************************************
*
* VK_RegisterForGenerateStmt - register for generate statement
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterForGenerateStmt(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName label_name,		/* generate stmt. label simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* statement elaboration position */
	VK_tExecPosIndex begin_pos,	/* region begin position */
	VK_tExecPosIndex end_pos,	/* region end position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */

				/* for-generate-specific information: */
	VK_tFieldOffset intf_data_offset
					/* offset of IAR field in encl. LAR */
#endif
	);
/* Notes:
* Currently there is one IAR and multiple LARs for the equivalent blocks.
* The generate parameter's value is stored temporarily in the IAR and
	permanently in each LAR.  The parameter_offset parameter is the 
	offset of the field in the IAR.
* The generate parameter's equivalent local constant in the LAR is registered 
	with VK_RegisterLocalConstantDecl.  (This is intended for debugging.)
* The generate parameter's IAR field is registered with VK_RegisterInterface-
	ConstantDecl.    (This is intended for use by VK_TraceBegin.  The 
	equivalent constant's value may not be filled in before VK_TraceBegin
	is called.) 
*/
	


/* *******************************************************************
*
* VK_RegisterProcessStmt - register process statement
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterProcessStmt(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName label_name,		/* process stmt. label simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* statement elaboration position */
	VK_tExecPosIndex begin_pos,	/* region begin position */
	VK_tExecPosIndex end_pos,	/* region end position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */

				/* process-specific information: */
	VK_tBoolean has_sensitivity_list
					/* process had sensitivity list */
#endif
	);
/* Specification:  Parameter label_name is the process statement's label
	identifier.  An empty string indicates an unlabeled process statement.
	Currently, an unlabeled process statement is given a generated name 
	starting with an underline ("_") to distinguish it. */
/* Note: Parameter has_sensitivity list is provided for check
	TP_080100_080201/TP_090200_020301. */


/* *******************************************************************
*
* 1.3: Registration functions for subprograms
*
******************************************************************* */


/* *******************************************************************
*
* VK_RegisterProcedureDecl - register procedure declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterProcedureDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName procedure_name,	/* procedure simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count
					/* number of enclosed items
					   (interface objects only) */

				/* procedure-specific information: */
					/* nothing */
#endif
	);


/* *******************************************************************
*
* VK_RegisterProcedureBody - register procedure body
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterProcedureBody(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle extended_declaration,
					/* procedure declaration ident. */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */

				/* execution information: */
	VK_tExecPosIndex begin_pos,	/* region begin position */
	VK_tExecPosIndex end_pos,	/* region end position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */
					/* (local objects only) */

				/* procedure-specific information: */
	VK_tBoolean contains_wait_stmt	/* procedure contains wait stmt. */
#endif
	);
/* Note: Parameter contains_wait_stmt list is provided for checks 
	TP_080100_080101 and TP_080100_080201/TP_090200_020301. */


/* *******************************************************************
*
* VK_RegisterFunctionDecl - register function declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterFunctionDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName function_name,		/* function simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items
					   (interface objects only) */

				/* return type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	VK_tBoolean is_unconstrained,	/* whether declared as unconstrained */

				/* function-specific information: */
	VK_tByteSize intf_data_size	/* byte size of interface activation
					   record */
#endif
	);
/* Note:  Parameter intf_data_size provides the interface activation record
	size so that VC_ConvertSignal can know the size directly. */


/* *******************************************************************
*
* VK_RegisterFunctionBody - register function body
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterFunctionBody(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle extended_declaration,
					/* function declaration ident. */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */

				/* execution information: */
	VK_tExecPosIndex begin_pos,	/* region begin position */
	VK_tExecPosIndex end_pos,	/* region end position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count,
					/* number of enclosed items */
					/* (local objects only) */

				/* function-specific information: */
	VK_tGenericEvalFnPtr evaluation_function
					/* evaluation function pointer */
#endif
	);
/* Note:  Parameter evaluation_function provides the C function address so
	that the kernel can call the function if needed for resolution or
	type conversion. */


/* *******************************************************************
*
* 1.4: Registration functions for references for parent-of-procedure checks
*
* SPECIFICATION:  The generated code must register procedure calls and
*	non-local references sufficient to detect a violation of check 
*	TP_080100_080201/TP_090200_020301 or check TP_020200_070101.  
*	The code is is not expected to register all subprogram calls and 
*	non-local references.
*
******************************************************************* */


/* *******************************************************************
*
* VK_RegisterCalledProcedures - register procedures called by subprogram or
*	process
*
******************************************************************* */
void VK_RegisterCalledProcedures(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle caller_region,
					/* calling process or subprogram body
					   identification */
	VK_tListLength procedure_count,	/* number of procedure handles */
	VK_tSymbInfoHandle called_procedure_1,
					/* called procedure declaration's 
					   identification */
	...				/* repeat for each additional called
					   procedure */
#endif
	);
/* Specification:  Currently, a called procedure may be listed repeatedly. */
/* Note: VK_RegisterCalledProcedures is provided for check TP_080100_080201/
	TP_090200_020301 and for check TP_020200_070101. */


/* *******************************************************************
*
* VK_RegisterNonLocalReferences - register non-local references from procedure
*
******************************************************************* */
void VK_RegisterNonLocalReferences(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle referring_region,
					/* procedure body making references */

				/* reference-specific information: */
	VK_tListLength object_count,	/* number of object handles */
	VK_tSymbInfoHandle referenced_object_1,
					/* referenced object identification */
	...				/* repeat for each additional object */
#endif
	);
/* Specification:  This registers references in a procedure to signals or 
	variables that are not local to the procedure. */
/* Specification:  Currently, a referenced object may be listed repeatedly. */
/* Note:  VK_RegisterNonLocalReferences is provided for check TP_020200_070101.
*/


/* *******************************************************************
*
* 2: Registration functions for component instantiation items
*
******************************************************************* */


/* *******************************************************************
*
* VK_RegisterComponentDecl - register component declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterComponentDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName component_name,	/* component simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* region information: */
	VK_tEnclItemNumber encl_item_count
					/* number of enclosed items */

				/* component-decl.-specific information: */
					/* nothing */
#endif
	);


/* *******************************************************************
*
* VK_RegisterInstantiationStmt - register component instantiation statement
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterInstantiationStmt(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName label_name,		/* comp. inst. label simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* statement elaboration position */

				/* instantiation-specific information: */
	VK_tFieldOffset intf_data_offset,	
					/* offset of IAR field in encl. LAR */
	VK_tSymbInfoHandle component_decl
					/* instantiated comp. decl. ident. */
#endif
	);
/* Note:  Parameter intf_data_offset is the offset of the component IAR field
	in the LAR of the enclosing region of the component instantiation 
	statement. */


/* *******************************************************************
*
* VK_RegisterConfigurationSpec - register configuration specification
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterConfigurationSpec(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle configured_instantiation,	
					/* "parent" comp. instantiation */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* specification elaboration pos. */

				/* configuration spec.-specific information: */
	VK_tFieldOffset intf_data_offset,	
					/* offset of IAR field in encl. LAR */
	VK_tSymbInfoHandle instantiated_unit
					/* instantiated unit ident. */
#endif
	);
/* Note:  Parameter intf_data_offset is the offset of the entity IAR field
	in the configuration specification's LAR. */
/* Note:  Parameter instantiated_unit is NULL for an entity aspect of "open",
	or the symbolic information handle of an entity declaration for 
	"entity E", of an architecture body for "entity E(A)", or of a
	configuration declaration for "configuration C". */


/* *******************************************************************
*
* VK_RegisterComponentConfiguration - register component configuration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterComponentConfiguration(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle configured_instantiation,	
					/* "parent" comp. instantiation */
	VK_tSymbInfoHandle encl_region,	/* enclosing region */
					/* ?? review: does this apply? */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
					/* ?? review: does this apply? */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* specification elaboration pos. */
					/* ?? review: does this apply? */

				/* configuration spec.-specific information: */
	VK_tFieldOffset intf_data_offset,	
					/* offset of IAR field in encl. LAR */
	VK_tSymbInfoHandle instantiated_unit
					/* instantiated unit ident. */
#endif
	);
/* Note:  Parameter intf_data_offset is the offset of the entity IAR field
	in the configuration configuration's LAR. */
/* Note:  Parameter instantiated_unit is NULL for an entity aspect of "open",
	or the symbolic information handle of an entity declaration for 
	"entity E", of an architecture body for "entity E(A)", or of a
	configuration declaration for "configuration C". */


/* *******************************************************************
*
* 3: Registration functions for type declarations
*
******************************************************************* */


/* *******************************************************************
*
* VK_RegisterEnumerationTypeDecl - register enumeration type declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterEnumerationTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* type-specific information: */
	VK_tArraySize cardinality,	/* number of enumeration literals */
	VK_tStringArrayAddr literals	/* enumeration literals string table */
#endif
	);


/* *******************************************************************
*
* VK_RegisterIntegerTypeDecl - register integer type declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterIntegerTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* type-specific information: */
	VK_tIntegerValue left_bound,	/* left bound of integer range */
	VK_tDirection direction,	/* direction of integer range */
	VK_tIntegerValue right_bound	/* right bound of integer range */
#endif
	);


/* *******************************************************************
*
* VK_RegisterPhysicalTypeDecl - register normal-size physical type declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterPhysicalTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* type-specific information: */
	VK_tIntegerValue left_bound,	/* left bound of integer range */
	VK_tDirection direction, 	/* direction of integer range */
	VK_tIntegerValue right_bound,	/* right bound of integer range */

	VK_tArraySize unit_count,	/* number of physical units */
	VK_tStringArrayAddr units,	/* physical units string table */
	VK_tIntegerArrayAddr position_values
					/* unit position values table */
#endif
	);


/* *******************************************************************
*
* VK_RegisterLargePhysicalTypeDecl - register large physical type declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterLargePhysicalTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* type-specific information: */
	VK_tLargeAddr left_bound,	/* left bound of integer range */
	VK_tDirection direction,	/* direction of integer range */
	VK_tLargeAddr right_bound,	/* right bound of integer range */

	VK_tArraySize unit_count,	/* number of physical units */
	VK_tStringArrayAddr units,	/* physical units string table */
	VK_tLargeArrayAddr position_values
					/* unit position values table */
#endif
	);

/* *******************************************************************
*
* VK_RegisterFloatingTypeDecl - register floating type declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterFloatingTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* type-specific information: */
	VK_tFloatingValue left_bound,	/* left bound of floating range */
	VK_tDirection direction,	/* direction of floating range */
	VK_tFloatingValue right_bound	/* right bound of floating range */
#endif
	);


/* *******************************************************************
*
* VK_RegisterArrayTypeDecl - register array type declaration (partial)
*
* NOTE: See VK_RegisterIndexSubtype also
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterArrayTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* element type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	VK_tSymbInfoHandle res_func_info,
					/* resolution function ident. */

				/* type-specific information: */
	VK_tInteger dimensions		/* number of dimensions */
#endif
	);
/* Note:  Parameter res_func_info is null if the element subtype is not 
	resolved. */


/* *******************************************************************
*
* VK_RegisterIndexSubtype - register index subtype of array type declaration
*
******************************************************************* */
void VK_RegisterIndexSubtype(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle parent_type,	/* containing array type */
	VK_tEnclItemNumber dimension,	/* which dimension's index subtype */
	
				/* index type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* range path indication */
	VK_tFieldOffset constraint_offset
					/* range LAR field offset */

				/* index-subtype-specific information: */
					/* nothing */
#endif
	);


/* *******************************************************************
*
* VK_RegisterRecordTypeDecl - register record type declaration (partial)
*
* NOTE: See VK_RegisterRecordElement also
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterRecordTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* type-specific information: */
	VK_tEnclItemNumber record_element_count
					/* number of record elements */
#endif
	);


/* *******************************************************************
*
* VK_RegisterRecordElementDecl - register element declaration of record type
*
******************************************************************* */
void VK_RegisterRecordElementDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle parent_type,	/* containing record type */
	VK_tEnclItemNumber record_element_number,
					/* which record element */
	VK_tName element_name,		/* record element name */

				/* element type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* element base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	VK_tSymbInfoHandle res_func_info
					/* resolution function identication */

				/* element-specific information: */
					/* nothing */
#endif
	);
/* Note:  Parameter res_func_info is null if the element subtype is not 
	resolved. */


/* *******************************************************************
*
* VK_RegisterFileTypeDecl - register file type declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterFileTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* designated type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset
					/* constraint LAR field offset */

				/* type-specific information: */
					/* nothing */
#endif
	);


/* *******************************************************************
*
* VK_RegisterAccessTypeDecl - register access type declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterAccessTypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName type_name,		/* type simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* designated type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset
					/* constraint LAR field offset */

				/* type-specific information: */
					/* nothing */
#endif
	);


/* *******************************************************************
*
* VK_RegisterSubtypeDecl - register subtype declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterSubtypeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName constraint_name,		/* subtype simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* subtype's type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	VK_tSymbInfoHandle res_func_info
					/* resolution function ident. */

				/* subtype-specific information: */
					/* nothing */
#endif
	);
/* Note:  Parameter res_func_info is null if the subtype is not resolved. */


/* *******************************************************************
*
* 4: Registration functions for object declarations
*
******************************************************************* */

/* Protocol note: for subprogram parameters, the encl_region parameter to
	VK_RegisterInterfaceXxxDecl is the handle from 
	VK_Register<Subprogram>Decl (not VK_Register<Subprogram>Body) */

/* Note: */


/* *******************************************************************
*
* VK_RegisterInterfaceConstantDecl - register interface constant declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterInterfaceConstantDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName constant_name,		/* constant simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint field offset */
	VK_tBoolean is_unconstrained,	/* whether declared as unconstrained */
	
				/* constant-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset,	/* IAR offset of value field */
				    /* other */
	VK_tMode intf_mode		/* interface mode */
#endif
	);


/* *******************************************************************
*
* VK_RegisterLocalConstantDecl - register local constant declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterLocalConstantDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName constant_name,		/* constant simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	
				/* constant-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset	/* LAR offset of value field */
#endif
	);


/* *******************************************************************
*
* VK_RegisterInterfaceVariableDecl - register interface variable declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterInterfaceVariableDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName variable_name,		/* variable simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint field offset */
	VK_tBoolean is_unconstrained,	/* whether declared as unconstrained */
	
				/* variable-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset,	/* IAR offset of value field */
				    /* other */
	VK_tMode intf_mode		/* interface mode */
#endif
	);


/* *******************************************************************
*
* VK_RegisterLocalVariableDecl - register local variable declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterLocalVariableDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName variable_name,		/* variable simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	
				/* variable-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset	/* LAR offset of value field */
#endif
	);


/* *******************************************************************
*
* VK_RegisterLocalFileDecl - register local file declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterLocalFileDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName file_name,		/* file simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	
				/* file-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset,	/* LAR offset of file handle field */
				    /* other */
	VK_tMode file_mode		/* VHDL mode of file */
#endif
	);


/* *******************************************************************
*
* VK_RegisterInterfaceSignalDecl - register interface signal declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterInterfaceSignalDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName signal_name,		/* signal simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint field offset */
	VK_tSymbInfoHandle res_func_info,
					/* resolution function ident. */
	VK_tBoolean is_unconstrained,	/* whether declared as unconstrained */

				/* signal-specific information: */
				    /* object value information: */
	VK_tFieldOffset handle_offset,	/* offset of signal handle IAR field */
	VK_tFieldOffset offset_offset,	/* offset of signal offset IAR field */
				    /* other */
	VK_tMode intf_mode,		/* interface mode */
	VK_tSignalKind signal_kind	/* signal kind */
#endif
	);
/* Note:  Parameter res_func_info is null if the signal subtype is not 
	resolved (as a whole). */


/* *******************************************************************
*
* VK_RegisterLocalSignalDecl - register local signal declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterLocalSignalDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName signal_name,		/* signal simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	VK_tSymbInfoHandle res_func_info,
					/* resolution function ident. */

				/* signal-specific information: */
				    /* object value information: */
	VK_tFieldOffset handle_offset,	/* offset of signal handle LAR field */
	VK_tFieldOffset offset_offset,	/* offset of signal offset LAR field */
				    /* other */
	VK_tSignalKind signal_kind	/* signal kind */
#endif
	);
/* Note:  Parameter res_func_info is null if the subtype is not resolved (as
	a whole). */


/* *******************************************************************
*
* VK_RegisterAttributeDecl - register attribute declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterAttributeDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName attribute_name,	/* attribute simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset
					/* constraint LAR field offset */
	
				/* attribute (decl.)-specific information: */
					/* nothing */
#endif
	);


/* *******************************************************************
*
* VK_RegisterAttributeSpec - register attribute specification
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterAttributeSpec(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tSymbInfoHandle attributed_item,
					/* item having attribute instance */
	VK_tSymbInfoHandle attribute_decl,
					/* attribute identification */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
	
				/* attribute (spec.)-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset	/* LAR offset of value field */
				    /* other */
#endif
	);


/* *******************************************************************
*
* VK_RegisterAliasDecl - register (local) alias declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterAliasDecl(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_region,	/* enclosing region identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName alias_name,		/* alias simple name */

				/* execution information: */
	VK_tExecPosIndex elab_pos,	/* declaration elaboration position */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */
				/* alias-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset,	/* offset of value pointer LAR field */
	VK_tFieldOffset handle_offset,	/* offset of signal handle LAR field */
	VK_tFieldOffset offset_offset,	/* offset of signal offset LAR field */
				    /* other */
	VK_tSymbInfoHandle aliased_object
					/* aliased object identification */
#endif
	);
/* Specification:  If parameter aliased_object identifies a signal, then
	parameter handle_offset and offset_offset are valid.  If parameter
	aliased_object identifies a constant or variable, then parameter
	value_offset is valid. */



/* *******************************************************************
*
* 5: Registration functions for loop labels and for for loop parameters
*
******************************************************************* */


/* *******************************************************************
*
* VK_RegisterLoopStmt - register loop statement and label
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterLoopStmt(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_loop_etc,
					/* enclosing loop or major region */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName label_name,		/* loop label simple name */

				/* execution information: */
	VK_tExecPosIndex loop_exec_pos,	/* loop statement execution position */
	VK_tExecPosIndex begin_pos,	/* region begin position */
	VK_tExecPosIndex end_pos	/* positions of first and last 
					   statements enclosed by for loop */

				/* loop-specific information: */
					/* nothing */
#endif
	);
/* Specification:  VK_RegisterLoopStmt is called for each loop
	statement. */
/* Specification:  Parameter encl_region is the symbolic information handle
	of the innermost enclosing loop statement, or process statement or
	subprogram body. */
/* Specification:  Parameter item_number (if and when implemented) will apply 
	to the innermost enclosing process statement or subprogram body, 
	representing the position of the implicit declaration of the loop 
	label. */
/* Specification:  Parameter label_name is the loop statement's label
	identifier.  An empty string indicates an unlabeled loop.  Currently, 
	an unlabeled loop is given a generated name starting with an underline
	("_") to distinguish it. */
/* Specification:  Parameter loop_exec_pos is the execution position of the
	loop statement. */
/* Specification:  Parameters begin_pos and end_pos specify the range of 
	statements enclosed by the loop.  For a for loop, this is the scope of
	the loop's iteration parameter. */


/* *******************************************************************
*
* VK_RegisterLoopParameter - register loop parameter effective declaration
*
******************************************************************* */
VK_tSymbInfoHandle VK_RegisterLoopParameter(
#ifdef VK_ANSI_FN_PROTOS
				/* identification: */
	VK_tSymbInfoHandle encl_loop,	/* enclosing for loop identification */
	VK_tEnclItemNumber item_number,	/* which item in enclosing region */
	VK_tName parameter_name,	/* loop parameter simple name */

				/* execution information: */
					/* nothing */

				/* object type and subtype information: */
	VK_tSymbInfoHandle base_type,	/* base type identification */
	VK_tSymbInfoHandle constraint_region,
					/* constraint path indication */
	VK_tFieldOffset constraint_offset,
					/* constraint LAR field offset */

				/* parameter-specific information: */
				    /* object value information: */
	VK_tFieldOffset value_offset	/* LAR offset of value field */
#endif
	);
/* Specification:  Parameter encl_loop is the symbolic information handle of
	the corresponding enclosing for loop statement. */
/* Specification:  Parameter item_number (if and when implemented) will apply 
	to the innermost enclosing process statement or subprogram body. */
/* Specification:  Parameter value_offset is the offset of the loop parameter
	field in the local activation record in the innermost enclosing 
	process statement or subprogram body. */
/* Specification:  Parameters constraint_region and constraint_offset specify
	the range over which the parameter value iterates.  Currently, 
	constraint_region may be null, indicating that this range is not
	provided. */



#endif /* VKRegister_H */






