#ifndef VKSignals_H
#define VKSignals_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKSignalsP.h
*
* DESCRIPTION:  
*	Signal operations.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
*	15-DEC-89 DSB	Created
*	...
*	09-APR-90 DSB	Changed resolution function interface of
*			VK_CreateExplicitSignal
*	18-SEP-90 AWH	Changed Signal Interface to use scalar signal numbers
*			and support vector signals.
*	09-OCT-90 DSB	Applying VAX port corrections (under 
*			VK_ANSI_FN_PROTOS); cleaning up type declarations;
*	09-OCT-90 DSB	Shuffling VK_CreateVectorSignal index parameters
*	22-FEB-91 DSB	Starting symbolic information interface conversion
*	10-APR-91 DSB	Split implicit signal creation routine
*	12-APR-91 DSB	Misc. cleanup
*	22-APR-91 DSB	Added VK_CreateParameterAliases
*	23-APR-91 DSB	Added constraint parameter to VK_CreateExplicitSignal,
*			VK_AssociatePort_Simple, and VK_AssociatePort_General
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*	16-MAY-91 DSB	W0053+: Removed old VK_Create{Scalar,Vector}Signal
*	21-MAY-91 DSB	Cleaning: removed old signal types
*	22-MAY-91 DSB	Added signal_length to VK_CreateExplicitSignal and
*			VK_AssociatePort_*
*	29-MAY-91 DSB	Edited comments
*	31-MAY-91 DSB	Added drivers to VK_CreateParametersAliases
*	08-OCT-91 DSB	Added VK_SetDisconnectionTime and VK_DisconnectDriver
*			for new implementation of disconnection specs.
*
* SCCS ID: @(#)VKSignals.h	42.2 1/13/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"	/* for VK_tDataAddr, etc. */
#include "VKSymbTypes.h"	/* for VK_tSymbInfoHandle */
#include "VKCodeIntf.h"	        /* for VK_tLARHeaderPtr */


/* *******************************************************************
*
* 1: Types for flavors of signals and of signal operations
*
******************************************************************* */

typedef enum {
	VK_BadDelayType,	/* -(not required) */
	VK_InertialDelay, 	/* -specifies inertial delay */
	VK_TransportDelay, 	/* -specifies transport delay */

    } VK_tDelayType;			/* signal assignment delay mode */
/* Specification:  Integer values do not matter.  Values shown are test 
	kernel's choice. */


/* *******************************************************************
*
* 2: Signal creation (elaboration) functions
*
******************************************************************* */


/* *******************************************************************
*
* VK_CreateExplicitSignal - create explicit signal 
*
******************************************************************* */
void VK_CreateExplicitSignal(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle * signal_handle_addr,
					/* created signal handle address */
	VK_tSignalOffset * signal_offset_addr,
					/* created signal offset address */
	VK_tSymbInfoHandle signal_info,	/* signal declaration identification */
	VK_tLARHeaderPtr encl_region_data,
					/* enclosing region local act. rec. */
	VK_tAnyConstraintAddr top_constraint,
					/* range or index constraint addr. */
	VK_tSignalLength signal_length,	/* length of signal to create */
	VK_tDataAddr default_value	/* signal default value */
#endif
	);
/* Specification:  Parameter signal_handle_addr is the address of a field into
	which the kernel should place the signal handle for the created signal.
	The kernel may leave this field uninitialized if signal handles are not
	used.  (In this case, the uninitialized field value may be passed back
	to the kernel in various VKI routine calls.)  */
/* Specification:  Parameter signal_offset_addr is the address of a field into
	which the kernel should place the beginning signal offset for the 
	created signal. */
/* Specification:  Parameter signal_info is the symbolic information handle
	for the signal declaration for which an instance of a signal is being
	created.  */
/* Specification:  Parameter encl_region_data is (the address of) the local 
	activation record of the instance of the region that immediately 
	encloses the signal declaration. */
/* Specification:  Parameter top_constraint is the address of the top-level 
	range constraint or index constraint for the signal.  It represents 
	type VK_tIndexConstraintAddr or a VK_t<type group>RangeConstraint 
	type.  It is null for a signal of a record type. */
/* Specification:  Parameter signal_length is the length in words of the signal
	to create. */
/* Specification:  Parameter default_value is the default value for the whole
	signal, from the value of the default expression if present, or the 
	implicit default value otherwise. */
/* Specification:  The data pointed to by default_value is valid only during 
	the call to VK_CreateExplicitSignal.  Specifically, the generated code
	may deallocate or change it later, so the kernel must copy or 
	otherwise use the value and not just save its address. */


/* *******************************************************************
*
* VK_SetDisconnectionTime - set disconnection time for signal
*
******************************************************************* */
void VK_SetDisconnectionTime(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle signal_handle,	/* signal handle */
	VK_tSignalOffset signal_offset,	/* signal offset */
	VK_tSignalLength signal_length,	/* signal length */
	VK_tRelativeTimePtr time_parameter
					/* disconnection time */
#endif
	);				/* (no return value) */
/* Specification:  Parameters signal_handle_addr, signal_offset_addr, and
	signal_length specify a signal whose disconnection time to set.
	The signal may be a subelement or slice of a subelement of a whole
	declared signal. */
/* Note:  VK_SetDisconnectionTime represents a disconnection specification.
	It will be called only for a guarded signal. */
/* Specification:  The kernel is responsible for checking that the time
	value is non-negative (050301_040102). */


/* *******************************************************************
*
* VK_CreateDriver - create driver for explicit signal
*
* NOTE: Note the special rules about the driver handle and driver offset.
*
******************************************************************* */
void VK_CreateDriver(
#ifdef VK_ANSI_FN_PROTOS
	VK_tDriverHandle * driver_handle_addr,
					/* driver handle address */
	VK_tDriverOffset * driver_offset_addr,
					/* driver offset address */
	VK_tSignalHandle signal_handle,	/* driven signal handle */
	VK_tSignalOffset signal_offset,	/* driven signal offset */
	VK_tSignalLength signal_length,	/* driven signal length */
	VK_tProcessHandle process_id	/* process holding driver */
#endif
	);
/* Specification:  For a given process and a specified (portion of a) signal 
	(consisting of some number of scalar subelements), VK_CreateDriver 
	creates scalar drivers for any scalar subelements of the given portion
	of the signal for which scalar driver do not exist yet.  That is,
	within one process, calls to create drivers for overlapping portions 
	of a signal will yield one scalar driver for each scalar subelement of
	the union of the given portions of the signal, and will not yield 
	multiple drivers for any of those elements in the intersection.  
	(Calls to create drivers for a different process will create 
	additional drivers.)  */
/* Specification:  For a given process and portions of a given whole signal, 
	different calls to VK_CreateDriver for different (scalar or composite)
	subelements of the signal must all return the same driver handle.  
	They must also all return "coordinated" driver offsets.  Specifically,
	the returned driver offset minus the given signal offset must be the 
	same for each driver.  (That is, the starting driver offset for the 
	apparent whole driver for the whole signal must be constant.)  */


/* *******************************************************************
*
* Implicit signal creation functions
*
******************************************************************* */


/* *******************************************************************
*
* VK_CreateTransactionSignal - create implicit signal S'Transaction
*
******************************************************************* */
void VK_CreateTransactionSignal(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle * signal_handle_addr,	
					/* created signal handle address */
	VK_tSignalOffset * signal_offset_addr,
					/* created signal offset address */
	VK_tSignalHandle prefix_signal_handle,
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length
					/* prefix signal S offset and length */
#endif
	);


/* *******************************************************************
*
* VK_CreateQuietSignal - create implicit signal S'Quiet(T)
*
******************************************************************* */
void VK_CreateQuietSignal(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle * signal_handle_addr,	
					/* created signal handle address */
	VK_tSignalOffset * signal_offset_addr,
					/* created signal offset address */
	VK_tSignalHandle prefix_signal_handle,
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length,
					/* prefix signal S offset and length */
	VK_tRelativeTimePtr time_parameter
					/* time value T in S'Quiet(T) */
#endif
	);
/* Specification:  The kernel is responsible for checking that the time
	value is non-negative (140100_220302). */


/* *******************************************************************
*
* VK_CreateStableSignal - create implicit signal 'Stable(T)
*
******************************************************************* */
void VK_CreateStableSignal(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle * signal_handle_addr,
					/* created signal handle address */
	VK_tSignalOffset * signal_offset_addr,
					/* created signal offset address */
	VK_tSignalHandle prefix_signal_handle,
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length,
					/* prefix signal S offset and length */
	VK_tRelativeTimePtr time_parameter
					/* time value T in S'Stable(T) */
#endif
	);
/* Specification:  The kernel is responsible for checking that the time
	value is non-negative (140100_210302). */


/* *******************************************************************
*
* VK_CreateDelayedSignal - create implicit signal S'Delayed(T)
*
******************************************************************* */
void VK_CreateDelayedSignal(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle * signal_handle_addr,
					/* created signal handle address */
	VK_tSignalOffset * signal_offset_addr,
					/* created signal offset address */
	VK_tSignalHandle prefix_signal_handle,
					/* prefix signal S handle */
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length,
					/* prefix signal S offset and length */
	VK_tRelativeTimePtr time_parameter,
					/* time value T in S'Delayed(T) */
	VK_tSymbInfoHandle prefix_type	/* base type of S for S'Delayed */
#endif
	);
/* Specification:  The kernel is responsible for checking that the time
	value is non-negative (140100_200302). */

/* Note: VK_CreateDelayedSignal returns a signal handle and a signal offset
	denoting a signal that has the same signal length (one or more) and 
	type (scalar or composite) as the prefix. */
/* Note:  Parameter prefix_type provides type information that for cases in
	which the prefix offset and length are ambiguous.  This type 
	information is needed only for display but not for simulation.  (For 
	example, S(1) and S(1 to 1) have the same offset and count, but 
	different types.  They are simulated identically, but might be
	displayed differently.) */


/* *******************************************************************
*
* VK_CreateGuardSignal - create implicit guard signal for guard expression
*
******************************************************************* */
void VK_CreateGuardSignal(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle * signal_handle,
					/* created signal handle address */
	VK_tSignalOffset * signal_offset,
					/* created signal offset address */
	VK_tLARHeaderPtr encl_region_data,
					/* block statement local act. rec. */
	VK_tGuardEvalFnPtr evaluation_function,	
					/* to evaluate guard expression */
	VK_tContextPtr evaluation_context,
					/* context for evaluation function */
	VK_tSensitivitySetHandle sens_set
					/* sensitivity set for guard
					   evaluation */
#endif
	);

/* Note:  Currently, evaluation_context is the same as encl_region_data. */


/* *******************************************************************
*
* 3: Port and signal parameter association functions
*
******************************************************************* */
 

/* *******************************************************************
*
* VK_CreateParameterAliases - create special aliases for signal 
*	parameter associated with independent subelement association
*
******************************************************************* */
void VK_CreateParameterAliases(
#ifdef VK_ANSI_FN_PROTOS
				/* result data: */
	VK_tSignalHandle * formal_signal_addr,
					/* signal alias handle address */
	VK_tSignalOffset * formal_signal_offset,
					/* signal alias offset address */
	VK_tDriverHandle * formal_driver_handle,
					/* driver alias handle address; 
					   NULL if no driver alias is needed */
	VK_tDriverOffset * formal_driver_offset,
					/* driver alias offset address;
					   NULL if no driver alias is needed */

				/* signal parameter identification data: */
	VK_tSymbInfoHandle parameter_info,
					/* signal parameter decl. ident. */
	VK_tIARHeaderPtr subprogram_intf_data,
					/* called subprogram *interface*
					   activation record */
	VK_tSignalLength formal_total_length,
					/* length of whole formal parameter */

	VK_tListLength association_element_count,
					/* number of association elements */

				/* first association element: */
	VK_tSignalLength formal_relative_offset_1,
	VK_tSignalLength formal_relative_length_1,
					/* formal portion offset and length */
	VK_tSignalHandle actual_signal_handle_1,
	VK_tSignalOffset actual_signal_offset_1,
	VK_tSignalLength actual_length_1,
	VK_tDriverHandle actual_driver_handle_1,
	VK_tSignalOffset actual_driver_offset_1,
					/* actual signal (and driver) 
					   identification */

				/* additional association elements: */
	...				/* repeat 7-tuples for each additional
					   association element */
#endif
	);

/* Specification: 
- VK_CreateParameterAliases is used for a subprogram call in which subelements
	of a formal parameter are associated independently.  (The generated
	code needs contiguous signal offsets to identify elements of the 
	formal parameter, even though the effective actual was assembled from
	separate signals.)  It creates a special signal (and driver) alias
	that can be used to denote the corresponding actual signal (and 
	driver).
- VK_CreateParameterAliases must create a driver alias for a signal 
	parameter of mode out or inout, also indicated by a non-null 
	formal_driver_offset parameter value.  (This value is NULL if the
	signal parameter is of mode in.)
*/



/* *******************************************************************
*
* VK_DestroyParameterAliases - destroy special aliases for signal parameter
*
******************************************************************* */
void VK_DestroyParameterAliases(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSignalHandle signal_handle,	/* signal handle */
	VK_tSignalOffset signal_offset,	/* formal's signal offset */
	VK_tDriverHandle driver_handle,	/* signal handle */
	VK_tDriverOffset driver_offset	/* formal's driver offset;
					   zero if there was no driver */
#endif
	);				/* (no return value) */
/* Specification:  
- VK_DestroyParameteAliases tells the kernel that a create alias is no longer
	needed, and the kernel may destroy it (to reclaim space).
- Paramater signal_handle is the signal handle returned from 
	VK_CreateParameterAliases via parameter formal_signal_addr.  The other
	parameters are similar.
*/


/* *******************************************************************
*
* VK_AssociatePort_Simple - create and associate port, for simple association
*
* DESCRIPTION:
*	VK_AssociatePort_Simple is used if there is a single association 
*	element (for the whole formal port) with no type conversion functions.
*
******************************************************************* */
void VK_AssociatePort_Simple(
#ifdef VK_ANSI_FN_PROTOS
				/* formal port results: */
	VK_tSignalHandle * formal_handle_addr,
					/* formal port handle address */
	VK_tSignalOffset * formal_offset_addr,
					/* formal port offset address */

				/* formal port data: */
	VK_tSymbInfoHandle port_info,	/* port declaration identification */
	VK_tIARHeaderPtr encl_region_intf_data,
					/* instantiated region *interface*
					   activation record */
	VK_tAnyConstraintAddr top_constraint,
					/* range or index constraint addr. */
	VK_tSignalLength signal_length,	/* length of signal to create */
	VK_tDataAddr default_value,	/* port default expression value */

				/* association data: */
	VK_tSignalHandle actual_handle,
	VK_tSignalOffset actual_offset	/* actual signal identification */

#endif
	);
/* Specification:  Parameter top_constraint is the address of the top-level 
	range constraint or index constraint for the signal.  It represents 
	type VK_tIndexConstraintAddr or a VK_t<type group>RangeConstraint 
	type.  It is null for a signal of a record type. */
/* Specification:  Parameter default_value is the default value for the whole
	signal, from the value of the default expression if present, or the 
	implicit default value otherwise.  It is NULL for a port of mode in 
	that has no default expression. */
/* Specification:  The data pointed to by default_value is valid only during 
	the call to VK_AssociatePort_Simple.  Specifically, the generated code
	may deallocate or change it later, so the kernel must copy or 
	otherwise use the value and not just save its address. */
/* Specification:  Parameter signal_length is the length of the port signal
	to create and the length of the actual signal.  (That is, the signal to
	be associated as an actual is identified by actual_handle, 
	actual_offset, and signal_length.) */


/* *******************************************************************
*
* VK_AssociatePort_General - create and associate port, for complex association
*
* DESCRIPTION:
*	VK_AssociatePort_General is used if independent subelements of a 
*	formal port are associated separately (in multiple association 
*	elements) or if there are type conversion functions.
*
* HISTORY:
*	04-DEC-91 DSB	Added formal_portion_constraint_1, actual_constraint_1
*
******************************************************************* */
void VK_AssociatePort_General(
#ifdef VK_ANSI_FN_PROTOS
				/* formal port results: */
	VK_tSignalHandle * formal_handle_addr,
					/* formal port handle address */
	VK_tSignalOffset * formal_offset_addr,
					/* formal port offset address */

				/* formal port data: */
	VK_tSymbInfoHandle port_info,	/* port declaration identification */
	VK_tIARHeaderPtr encl_region_intf_data,
					/* instantiated region *interface*
					   activation record */
	VK_tAnyConstraintAddr top_constraint,
					/* range or index constraint addr. */
	VK_tSignalLength signal_length,	/* length of signal to create */
	VK_tDataAddr default_value,	/* port default expression value */

				/* association data: */
	VK_tListLength association_element_count,
					/* number of association elements */

				    /* first association element: */

	VK_tSignalLength formal_portion_offset_1,
	VK_tSignalLength formal_portion_length_1,
					/* formal portion offset and length */

	VK_tSignalHandle actual_handle_1,
	VK_tSignalOffset actual_offset_1,
	VK_tSignalLength actual_length_1,
					/* actual signal identification */

	VK_tSymbInfoHandle in_type_conv_func_info_1,
					/* inward type conv. func. ident. */
	VK_tLARHeaderPtr in_type_conv_func_context_1,
					/* inward type conv. func. context */
	VK_tIndexConstraintAddr actual_constraint_1,
					/* actual signal index constraint */

	VK_tSymbInfoHandle out_type_conv_func_info_1,
					/* outward type conv. func. ident/ */
	VK_tLARHeaderPtr out_type_conv_func_context_1,
					/* outward type conv. func. context */
	VK_tIndexConstraintAddr formal_portion_constraint_1,
					/* formal portion index constraint */

				    /* additional association elements: */
	...				/* repeat 11-tuples for each additional
					   association element */
#endif
	);
/* Specification:  Parameter top_constraint is the address of the top-level 
	range constraint or index constraint for the signal.  It represents 
	type VK_tIndexConstraintAddr or a VK_t<type group>RangeConstraint 
	type.  It is null for a signal of a record type. */
/* Specification:  Parameter default_value is the default value for the whole
	signal, from the value of the default expression if present, or the 
	implicit default value otherwise.  It is NULL for a port of mode in 
	that has no default expression. */
/* Specification:  The data pointed to by default_value is valid only during 
	the call to VK_AssociatePort_Simple.  Specifically, the generated code
	may deallocate or change it later, so the kernel must copy or 
	otherwise use the value and not just save its address. */
/* Specification:  Parameter formal_portion_constraint_1 is the index 
	constraint of the portion of the formal parameter.  It is provided
	if the portion is an array and if there is an outward type conversion
	function.  Otherwise, it is undefined (it may be NULL or may be the
	address of a range constraint).   The index constraint corresponds to
	the type of the portion of the formal (not necessarily the type of the
	whole formal).  (The type of the portion of the formal is also the 
	parameter type of the outward type conversion function, and the result
	type of the inward type conversion function (if any).  */
/* Specification:  Parameter actual_constraint_1 is the index constraint of 
	the actual parameter.  It is provided if the actual is an array and if
	there is an inward type conversion function.  Otherwise, it is 
	undefined (it may be NULL or may be the address of a range constraint).
	The index constraint corresponds to the type of the actual, the 
	parameter type of the inward type conversion function, and the result 
	type of the outward type conversion function (if any).  */


/* *******************************************************************
*
* VK_NotePort - associate port with enclosing region
*
* DESCRIPTION:
*	VK_NotePort is a hook to help the kernel identify the hierarchical
*	name of a port instance.  (Port association is done before the local
*	activation record is available.  This can make it inconvenient for a
*	kernel)
*
* USAGE:
*	VK_NotePort is called after the local activation record for a design 
*	entity, block statement, or (equivalent block for a) configuration 
*	specification is created.  
*
******************************************************************* */
void VK_NotePort(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle port_info,	/* port declaration identification */
	VK_tLARHeaderPtr encl_region_data,
					/* enclosing region local act. rec. */
	VK_tSignalHandle formal_handle,	/* formal port signal handle */
	VK_tSignalOffset formal_offset	/* formal port signal offset */

#endif
	);				/* (no return value) */

/* Specification:  Each of the arguments for parameters formal_handle and 
	formal_offset will be in such a form that a VK_NotePort macro can take
	the address of the C object containing the argument value (in case the
	kernel wants to replace the values returned from port association 
	functions). */


/* *******************************************************************
*
* 4: Driver update functions
*
******************************************************************* */


/* *******************************************************************
*
* VK_AssignWaveform - assign waveform to driver of signal
*
******************************************************************* */
		
#ifndef TK_ANSI_VAR_ARGS		/* for test kernel compilation
					   problem only */
  void VK_AssignWaveform();
#else
  void VK_AssignWaveform(
#  ifdef VK_ANSI_FN_PROTOS
	VK_tDriverHandle driver_handle, /* in: driver handle */
	VK_tDriverOffset driver_offset,	/* in: driver offset */
	VK_tSignalLength length, 	/* in: driver length */
	VK_tSignalHandle signal_handle,	/* in: signal handle */
	VK_tSignalOffset signal_offset,	/* in: signal offset */
	VK_tProcessHandle process, 	/* in: driver's process */
	VK_tDelayType delay_type, 	/* in: transport vs. inertial */
	VK_tListLength waveform_element_count,
					/* in: number of waveform elements */
	VK_tCompositeAddr value1, 	/* in: first value pointer */
	VK_tRelativeTimePtr time1, 	/* in: first time pointer */
	...				/* in: additional value/time pairs */
	);				/* (no return value) */
#  endif
#endif
/* Specification:  Variable arguments are interpreted as follows:  
     There are pairs of values and times.  The total number of pairs (including
	value1 and time1) is given by waveform_element_count.  Each pair has 
	the following members:
		VK_tCompositeAddr value 	- in: first value pointer
		VK_tRelativeTimePtr time 	- in: first time pointer
*/
/* Specification:  A null waveform element (for a guarded signal) is repre-
	sented with a null pointer for a value parameter. */
/* Specification:  Currently, the kernel is responsible for checking that the 
	time values are non-negative (080310_030301). */
/* Specification:  Currently, the kernel is responsible for checking that the 
	time values are in ascending order (080310_050501). */
/* Note:  The process_id parameter is provided in case driver handles and 
	offsets are local to a process.  The signal_handle and signal_offset 
	parameters are provided in case a kernel needs the signal also. */


/* *******************************************************************
*
* VK_AssignWaveform_Aggregate - assign waveform for aggregate target
*
* HISTORY:
*	...
*	02-DEC-91 DSB	Added value_offset virtual parameter
*
******************************************************************* */
		
#ifndef TK_ANSI_VAR_ARGS		/* for test kernel compilation
					   problem only */
  void VK_AssignWaveform_Aggregate();
#else
  void VK_AssignWaveform_Aggregate(
#  ifdef VK_ANSI_FN_PROTOS
	VK_tProcessHandle process, 	/* in: driver's process */
	VK_tDelayType delay_type, 	/* in: transport vs. inertial */

	VK_tListLength target_count,	/* in: number of signal names in 
					   aggregate target */
	VK_tListLength waveform_element_count,
					/* in: number of waveform elements */
	...				/* in: 6-tuples for signal names, then
					   pairs for waveform elements */
	);				/* (no return value) */
#  endif
#endif
/* Specification:  Variable arguments are interpreted as follows:  
     First, there are 6-tuples of signal and driver identification.  The number
	of 6-tuples is given by target_count.  Each 6-tuple has the following 
	members:
		VK_tDataOffset value_offset	- in: values portion offset
		VK_tDriverHandle driver_handle - in: driver handle
		VK_tDriverOffset driver_offset	- in: driver offset
		VK_tSignalLength length 	- in: driver length
		VK_tSignalHandle signal_handle	- in: signal handle
		VK_tSignalOffset signal_offset	- in: signal offset
	The 6-tuples are listed in left-to-right order ...  Each driver takes
	element values starting at an offset given by value_offset and having 
	a length equal to the length of the driver.  

     Second, there are pairs of values and times.  The number of pairs is given
	by waveform_element_count.  Each pair has the following members:
		VK_tCompositeAddr value 	- in: first value pointer
		VK_tRelativeTimePtr time 	- in: first time pointer
*/
/* Specification:  A null waveform element (for a guarded signal) is repre-
	sented with a null pointer for a value parameter. */
/* Specification:  Currently, the kernel is responsible for checking that the 
	time values are non-negative (080310_030301). */
/* Specification:  Currently, the kernel is responsible for checking that the 
	time values are in ascending order (080310_050501). */
/* Note:  The process_id parameter is provided in case driver handles and 
	offsets are local to a process.  The signal_handle and signal_offset 
	parameters are provided in case a kernel needs the signal also. */


/* *******************************************************************
*
* VK_DisconnectDriver - assign null transaction to driver of guarded signal
*
******************************************************************* */
		
void VK_DisconnectDriver(
#ifdef VK_ANSI_FN_PROTOS
	VK_tDriverHandle driver_handle, /* in: driver handle */
	VK_tDriverOffset driver_offset,	/* in: driver offset */
	VK_tSignalLength length, 	/* in: driver length */
	VK_tSignalHandle signal_handle,	/* in: signal handle */
	VK_tSignalOffset signal_offset,	/* in: signal offset */
	VK_tProcessHandle process, 	/* in: driver's process */
	VK_tDelayType delay_type 	/* in: transport vs. inertial */
#endif
	);				/* (no return value) */
/* Specification:  The kernel must assign a null transaction to each scalar
	subelement of the specified driver, using the disconnection times
	provided via VK_SetDisconnectionTime.  If no disconnection time was
	provided, a value of 0 fs is to be used.  */
/* Note:  The process_id parameter is provided in case driver handles and 
	offsets are local to a process.  The signal_handle and signal_offset 
	parameters are provided in case a kernel needs the signal also. */


/* *******************************************************************
*
* 5: Signal and attribute read functions
*
******************************************************************* */


/* *******************************************************************
*
* VK_ReadSignalValue - read value of signal
*
******************************************************************* */
void VK_ReadSignalValue(
#ifdef VK_ANSI_FN_PROTOS
	VK_tAddrAddr ptr_addr,      	/* buffer pointer address */
	VK_tSignalHandle signal_handle,	/* read signal handle */
	VK_tSignalOffset signal_offset,
 	VK_tSignalLength signal_length	/* read signal offset and length */
#endif
	);


/* *******************************************************************
*
* VK_ReadActive - return boolean value of predefined attribute S'ACTIVE
*
******************************************************************* */
void VK_ReadActive(
#ifdef VK_ANSI_FN_PROTOS
	VK_tScalarAddr boolean_addr,	/* result buffer address */
	VK_tSignalHandle prefix_signal_handle,
					/* prefix signal S handle */
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length
					/* prefix signal S offset and length */
#endif
	);


/* *******************************************************************
*
* VK_ReadEvent - return boolean value of predefined attribute S'EVENT
*
******************************************************************* */
void VK_ReadEvent(
#ifdef VK_ANSI_FN_PROTOS
	VK_tScalarAddr boolean_addr,	/* result buffer address */
	VK_tSignalHandle prefix_signal_handle,
					/* prefix signal S handle */
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length
					/* prefix signal S offset and lenght */
#endif
	);


/* *******************************************************************
*
* VK_ReadLastActive - return time value of predefined attribute S'LAST_ACTIVE
*
******************************************************************* */
void VK_ReadLastActive(
#ifdef VK_ANSI_FN_PROTOS
	VK_tRelativeTimePtr time_buffer,
					/* result buffer address */
	VK_tSignalHandle prefix_signal_handle,
					/* prefix signal S handle */
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length
					/* prefix signal S offset and length */
#endif
	);


/* *******************************************************************
*
* VK_ReadLastEvent - return time value of predefined attribute S'LAST_EVENT
*
******************************************************************* */
void VK_ReadLastEvent(
#ifdef VK_ANSI_FN_PROTOS
	VK_tRelativeTimePtr time_buffer,
					/* result buffer address */
	VK_tSignalHandle prefix_signal_handle,
					/* prefix signal S handle */
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length
					/* prefix signal S offset and length */
#endif
	);


/* *******************************************************************
*
* VK_ReadLastValue - read value of predefined attribute S'LAST_VALUE
*
******************************************************************* */
void VK_ReadLastValue(
#ifdef VK_ANSI_FN_PROTOS
	VK_tAddrAddr ptr_addr,      	/* buffer pointer address */
	VK_tSignalHandle prefix_signal_handle,
					/* prefix signal S handle */
	VK_tSignalOffset prefix_signal_offset,
	VK_tSignalLength prefix_signal_length
					/* prefix signal S offset and length */
#endif
	);


/* *******************************************************************
*
* 6: Other information
*
******************************************************************* */


/*  Specification:  The kernel is responsible for checking that the driving
	value and the effective value of each scalar subelement of a signal
	belong to the subtype of subelement (120610_210101). */

#endif /* VKSignalsP_H */

