#!/bin/csh -x -f
#
# FILE: vbuild.csh
#
# DESCRIPTION:
#	This script builds a model for the given top level unit.
#
#	Steps:
#	1) Configure the model (gather required units).
#	2) Build Register Units module (--config.c file).
#	3) Link required unit's objects, registration module and test kernel.
#
#	Environment:
#	  VTIP variables and aliases: vhdl $spi_obj $spi_defs
#	  RVCG variables and aliases: $CG_VKI $CG_RTLI $CG_VCG $CG_TKO 
#		$CG_CFGE
#		
#
# HISTORY:
#	20-DEC-91 AWH 	Created.
#
# SCCS ID: @(#)vbuild.csh	1.2 92/01/13
#
############################################################################
#
#	Configuration Options
#
set CFLAGS="-I$CG_VKI -I$CG_RTLI -c "
set CmdName=`basename ${0} .csh`
set DefineList="dls_std:STANDARD dls_std:TEXTIO"
#
#
#	Check parameter and set file name
#
if ( $#argv == 1 ) then
    set InTopUnit=`basename $1`
    set ModelName=`basename $1 .vhdlview`
else if ( $#argv == 2 ) then
    set InTopUnit=`basename $1`
    set ModelName=`basename $2`
else
	echo "Usage: $CmdName <top_unit> [<model_name>]"
        echo "  <top_unit> top level unit of model to build."
        echo "  <model_name> name of model, default <top_unit>"
	exit 1
endif
set CFGFile="${ModelName}.CFG"


#
#	1) Configure the model (gather required units).
#
echo ""
echo "--Configuring ${ModelName}"
$CG_CFGE/config ${InTopUnit} -output $CFGFile -define $DefineList
if ( $status != 0 ) then
	echo "Error: configuration tool failed."
	exit 1
endif

#
#	Get Top Unit and Unit List
#
set TopUnit=`awk '/^TopUnit:/ { print $2 }' $CFGFile`
set OtherUnits=`awk '/^RequiredUnit:/ { print $2 }' $CFGFile`

set ValidUnits=""
set Objects=""
foreach unit ( $TopUnit $OtherUnits )
	if ( $unit != "dls_std:STANDARD" && \
	    $unit != "dls_std:TEXTIO") then
	    set ValidUnits="$ValidUnits $unit"
	    set unit_name=`echo $unit | awk -F: '{ print $2 }'`
	    set Objects="$Objects ${unit_name}-g.o"
	endif
end

#
#	2) Build Register Units Module (--config.c file).
#
echo ""
echo "--Building register units module"
set ConfigCFile=${ModelName}--config.c
set ConfigOFile=${ModelName}--config.o

echo "/*    File: ${ModelName}--config.c */" 	> $ConfigCFile
echo "/* Version: 3 */ " 			>> $ConfigCFile
echo '#include "VLI.h"'				>> $ConfigCFile
echo "" 					>> $ConfigCFile
set QQ='"'
echo "char TK_ModelName[] = ${QQ}${ModelName}${QQ};" 	>> $ConfigCFile
echo "" 					>> $ConfigCFile
echo "void TK_RegisterUnits()" 			>> $ConfigCFile
echo "{" 					>> $ConfigCFile
echo "/**/" 					>> $ConfigCFile


set TopUnitSpecified=0
foreach unit ($ValidUnits) 
    set RegName=`echo $unit | sed s/dls_// | sed s/:/__/|sed s/-/__/`
    if ( "$unit" == "$TopUnit" )  then
        echo "    TK_SpecifyTopUnit(VC_Reg_${RegName}());" >> $ConfigCFile
        set TopUnitSpecified=1
    else	 
        echo "    VC_Reg_${RegName}();" >> $ConfigCFile
    endif
end
if ( 0 == $TopUnitSpecified ) then
    echo "Error: top unit not specified in register unit module"
    exit 1
endif
    
echo ""					>> $ConfigCFile
echo "} /* TK_RegisterUnits */"		>> $ConfigCFile

cc $CFLAGS -o $ConfigOFile $ConfigCFile
if ( $status != 0 ) then
	echo "Error: compile of register units module failed"
	exit 1
endif

#
#	3) Link required unit's objects, registration module and test kernel.
#
echo " "
echo "--Linking model ${ModelName}"
cc -v -pg -Bstatic -o ${ModelName}.sim  $ConfigOFile $Objects -L$CG_TKO -lKernel \
	-L$CG_RTLO -lVCGRTL -L$spi_obj -Bdynamic -lSPI -Bstatic
if ($status != 0) then
	echo "Error: link of simulation model failed."
	exit 1
endif

exit 0





