#!/bin/csh -x -f
#
# FILE: vcompile.csh
#
# DESCRIPTION:
#	This script "compiles" VHDL source into objects ready
#	to link with the test simulation kernel.
#
#	Steps:
#
#	1) Analyze the VHDL source file
#	2) Generate 'C' code for each library unit analyzed
#	3) Compile the 'C' code
#
#	Environment:
#	  VTIP variables and aliases: vhdl $spi_obj $spi_defs
#	  RVCG variables and aliases: $CG_VKI $CG_RTLI $CG_VCG
#	  dls library WORK must be defined
#		
#
# HISTORY:
#	20-DEC-91 AWH 	Created.
#
# SCCS ID: @(#)vcompile.csh	1.1 91/12/12
#
############################################################################
#
#	Configuration Options
#
#set verbose=1
set CFLAGS="-I$CG_VKI -I$CG_RTLI -c"
set CmdName=`basename ${0} .csh`
#
#
#	Check parameter and set file name
#

if ( $#argv == 1 ) then
    set VHDLFile="$1"
    set ModelName=`basename $1 .vhdl`
    set ModelName=`basename $ModelName .vhd`
else if ( $#argv == 2 ) then
    set VHDLFile="$1"
    set ModelName="$2"
else
	echo "Usage: $CmdName <vhdl_file> [<model_name>]"
        echo "  <vhdl_file> VHDL file name"
        echo "  <model_name> name of model, default <vhdl_file>"
	exit 1
endif
set AuxFile="${ModelName}.AUX"

#
#	1) Analyze the VHDL source file
#
echo ""
echo "--Analyzing $1"
vhdl -aux $AuxFile $1
if ( $status > 1 ) then
	echo "Error: from analyzer"
	exit 1
endif

#
#	Get list of library units from anaylzer output file
#
set FullUnits=`awk '$1 == "DesignUnit:" && $2 == "Saved" { print $4 }' $AuxFile`
set MinUnits=`echo $FullUnits | tr ' ' '\012' | awk -F: '{print $2 }' | awk -F. '{ print $1 }'`

#
#	2) Generate 'C' code for each library unit analyzed
#
foreach unit ($MinUnits)
	echo ""
	echo "--Generating C code for $unit"
	$CG_VCG/vcg -debug $unit
	if ( $status > 1 ) then
		echo "Error: from code generator"
		exit 1
	endif
end
#
#	3) Compile the 'C' code
#
foreach unit ($MinUnits)
        set CFile="${unit}-g.c"
	echo ""
	echo "--Compiling $CFile"
	cc $CFLAGS $CFile
	if ( $status ) then
		echo "Error: from 'C' compiler"
		exit 1
	endif
end
exit 0
