#!/bin/csh -f
#
# FILE: vtest.csh 
#
# DESCRIPTION:
#	This script compiles, builds, and simulates a model defined in one
#       VHDL source file.  The top level unit is defined as the last
#       configuration or entity in the file.
#
#	Steps:
#	1) Compile source file with vcompile
#       2) Select top level unit
#       3) Build model with vbuild
#       4) Simulate
#       5) Format output
#
#	Environment:
#	  VTIP variables and aliases: vhdl $spi_obj $spi_defs
#	  RVCG variables and aliases: $CG_VKI $CG_RTLI $CG_VCG $CG_TKO 
#		$CG_CFGE
#		
#
# HISTORY:
#	11-DEC-91 AWH 	Created.
#
# SCCS ID: @(#)vtest.csh	1.1 91/12/12
#
############################################################################
#
#	Configuration Options
#
set CmdName=`basename ${0} .csh`

#
#
#	Check parameter and set file name
#
if ( $#argv != 1 ) then
	echo "Error: $CmdName requires VHDL source file as parameter"
	exit 1
endif
set VHDLFile=$1
set ModelName=`basename $VHDLFile .vhdl`
set ModelName=`basename $ModelName .vhd`
set AUXFile="${ModelName}.AUX"
set TraceFile="${ModelName}.trace"
set WaveFile="${ModelName}.wave"

#
#	1) Compile source file with vcompile
#
~vsim/profiling/vcompile.csh $VHDLFile $ModelName
if ( $status != 0 ) then
    exit 1
endif

#
#       2) Select top level unit
#
if ( ! -f "$AUXFile" ) then
    echo "Error: temporary file $AUXFile not found"
    exit 1
endif
set ECUnits=`awk '($1 == "DesignUnit:" && $2 == "Saved") && ($3 == "Entity" || $3 == "Configuration") { print $4 }' $AUXFile`

set MinUnits=`echo $ECUnits | tr ' ' '\012' | awk -F: '{print $2 }' | awk -F. '{ print $1 }'`
set LastUnit=""
foreach unit ($MinUnits)
    set LastUnit="$unit"
end
if ($LastUnit == "") then
    echo "Error: file $VHDLFile does not contain an entity or architecture"
    exit 1
endif
#
#       3) Build model with vbuild
#
~vsim/profiling/vbuild.csh $LastUnit $ModelName
if ( $status != 0 ) then
    exit 1
endif

#
#       4) Simulate
#
echo Simulating ${ModelName}
${ModelName}.sim
if ( ! -f $TraceFile ) then
    exit Error: simulation modle ${ModelName} failed"
    exit 1
endif

#
#       5) Format output
#
echo Formatting output $WaveFile
$CG_TKO/fmtwave ${ModelName}

echo "Test of ${ModelName} complete."

exit 0







