static char SCCS_ID[] = "@(#)CmdProcessor.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: CmdProcessor.c
*
* DESCRIPTION:  
*	Routines that support user specification of commands and actions
*       to be executed when commands are recognized in the input.
*
*
* CONSTANTS:
*	CVSIZE
*
* TYPES:
*	tCmdVectorEntry
*	tCmdVector
*	tTempBuf80
*	tTempBuf256
*
* GLOBAL VARIABLES:
*	UserTables
*	CurrMax
*	ControlCharTable
*
* ROUTINES:
*   External:
*
*	ParseCmd - 		process a command line in token buffer
*	ValidTableId - 		tests a user table id for validity
*	RegCmdTable - 		register a user command table
*	NextToken - 		get the next token from token_buffer
*	TokenValue - 		copy the value of token to buffer
	TokenBufferValue -	copy the value of token buffer to buffer
*	SetTokenBuffer - 	initialize a token buffer (MACRO)
*	SetTokenSeparator -	initialize a token delimiter object
*	Pushback -	       	return a token to the token buffer
*	TokenType - 		return the type of a token (MACRO)
*
*   Internal:
*	IsPrefix -		determine if string1 is a prefix of string2
*   BuildSet -			build internal representation of delimiter set
*	InSet -			determine if an element is member of delimiter set
*   CommandType -		determine if a token is a command type
*
* HISTORY:
*       09-JAN-91 RM	Created
*	09-FEB-91 RM	Added SetTokenSeparator, TokenBufferValue, Pushback.
*	02-MAY-91 DSB	Cleaning up generally
*
* SCCS ID: @(#)CmdProcessor.c	42.1 1/10/92
*
******************************************************************* */

#include "CmdProcessor.h"

#include <stdio.h>     		/* for NULL */
#include <ctype.h>		/* for character classifying routines */
#include <assert.h>             /* assert statements */
#include <varargs.h>            /* for variable arg. list function */


# define CVSIZE 20	/* nr. of command tables allowed */

typedef char tTempBuf80[80];
typedef char tTempBuf128[128];
typedef char tTempBuf256[256];

typedef struct sCmdVectorEntry {

	tCmdTable *tabptr;
	int tabsize;

} tCmdVectorEntry;


typedef tCmdVectorEntry tCmdVector[CVSIZE + 1];


static tCmdVector UserTables;

static tCmdTableId CurrMax = -1;
#define NO_TID (-1)

static char * ControlCharTable[] = {

"NUL", "SOH", "STX", "ETX", "EOT", "ENQ", "ACK", "BEL",
"BS",  "TAB", "LF",  "VT",  "FF",  "CR",  "SO",  "SI",
"DLE", "DC1", "DC2", "DC3", "DC4", "NAK", "SYN", "ETB",
"CAN", "EM",  "SUB", "ESC", "FS",  "GS",  "RS",  "US"

};


VK_tBoolean iIsPrefix();

#define IsPrefix(str1, str2, same) \
       iIsPrefix(str1, str2, &(same))


VK_tBoolean iCommandType();

#define CommandType(type)	iCommandType(type)


VK_tBoolean iInSet();

#define InSet(candidate, members) \
       iInSet(candidate, members)


void iBuildSet();

#define BuildSet(tk_sp, set, size) \
       iBuildSet(tk_sp, set, size)


#define INIT(ts) (ts)[0]='I';(ts)[1]='N';(ts)[2]='T'

#define ISINIT(ts) \
((ts)[0]=='I' && (ts)[1]=='N' && (ts)[2]=='T')

#define INIT_OFFSET 4
#define ISDEFAULT(ts)\
        ((ts)[INIT_OFFSET-1] == '1')    /* it suffices that this bit is set */
                                   /* to assume default behavior even if */
                                   /* other bits are set as well */



/* *******************************************************************
*
* ParseCmd - process contents of token_buf as follows:
*         first token must be a command whose associated routine will
*         be executed with the remaining of the buffer as argument.
*
* PRE: token_buf is a previously initialized token buffer to tokenize, tid a 
*      user command table id.
*
* POST: return value is set to :
*	PS_Error -      if unexpected errors are encountered (internal).
*       PS_NotCmd -	if next token in token_buf is not a command
*                        or if it is a non-registered command.
*	PS_AmbiguosCmd - if next token specifies more than one registered
*                        command.
*       PS_NotCmdTable - if tid is not a valid command table id.
*       PS_CmdComplete - if next token is a registered command and its
*                        associated routine has been executed and returned
*                        a True value.
*       PS_CmdIncomplete - if next token is a registered command and its
*                        associated routine has been executed and returned
*                        a False value.
*	PS_NoNext -	if no next token was found.
*
* HISTORY:
*	09-JAN-91 RM	Created
*	18-JAN-91 AWH   Added P_NoToken case.
*       08-FEB-91 RM	Added PS_CmdIncomplete case, new use of NextToken.
*
******************************************************************* */

tParseStatus eParseCmd(token_buf, tid)
tTokenBuffer token_buf;
tCmdTableId tid;
{
    tParseStatus result;
    tToken token;		/* token for command name */
    int i;			/* index */
    tCmdProc userproc;          /* user routine */
    int matches;		/* to detect ambiguos commands */
    VK_tBoolean exact;		/* true if next token exactly matches a command */
    tTokenSeparator tksp;       /* to specify what kind of tokens */
/**/
    if (token_buf == NULL) {
        return ( PS_Error);
    }
    if (! ValidTableId(tid)) {
        result = PS_NotCmdTable;
    }
    else {

        /* default behavior, white space is not important */
        
        SetTokenSeparator(tksp, NULL);
        NextToken(token_buf, token, tksp);
        if (TokenType(token) == TK_NoToken) {
            result = PS_NoNext;
	}
        else if (! CommandType(TokenType(token))) {
            result = PS_NotCmd;
	}
        else {

            tTempBuf256 cmd_name;
            /**/

            TokenValue(token, cmd_name, sizeof(cmd_name));

            /* search user table for command */

            i = 0;
            matches = 0;
            exact = False;

            while (i< UserTables[tid].tabsize && ! exact) {

                /* INV: 0<=j<i cmd_name !~ *(UserTables[tid].tabptr)[j].cmd &&
                        matches >= 0   */

                if(IsPrefix(cmd_name, (*(UserTables[tid].tabptr))[i].cmd, exact)) {

                    userproc = (*(UserTables[tid].tabptr))[i].proc;
                    matches++;
	        }
                i++;

	    } /* while */

            /* INV && i == UserTables[tid].tabsize || exact = True */

            if (matches == 0) {

                result = PS_NotCmd;
	    }
            else if (matches > 1 && ! exact) {

                result = PS_AmbiguosCmd;
	    }
            else {
                    /* call user routine with rest of command line */

                    if ((*userproc) (token_buf)) {
                        result = PS_CmdComplete;
		    }
                    else {
                        result = PS_CmdIncomplete;
		    }
	    }

            }

      }

    return (result);

} /* ParseCmd */





/* *******************************************************************
*
* ValidTableId - tests a user table id for validity.
*
* PRE: tid is a user table id.
*
* POST: returns True, if tid is a valid table id, false otherwise.
*
* HISTORY:
*	09-JAN-91 RM	Created
*
******************************************************************* */

VK_tBoolean eValidTableId(table_id)
tCmdTableId table_id;
{
/**/
    if (table_id >= 0 && table_id <= CurrMax) {
        return (True);
    }
    else {
        return (False);
    }
    
} /* ValidTableId */





/* *******************************************************************
*
* RegCmdTable - register a user command table.
*
* PRE: UserTable[k] is set, 0<=k<=CurrMax, table is a command table
*      and size is table size.
*
* POST: UserTable[k] is set, 0<=k<=CurrMax+1 , and return CurrMax+1
*       if successful, otherwise return NO_TID and UserTable unchanged.
*
* HISTORY:
*	09-JAN-91 RM	Created
*
******************************************************************* */

tCmdTableId eRegCmdTable(table, size)
tCmdTable table;
int size;
{
    tCmdTableId result;
/**/
    if (table == NULL) {
        result = NO_TID;
    }
    else if (CurrMax+1 > CVSIZE-1 || CurrMax+1 < 0) {
         result = NO_TID;
    }
    else {
        CurrMax ++;
        UserTables[CurrMax].tabptr = (tCmdTable *) &table[0];
        UserTables[CurrMax].tabsize = size;
        result = CurrMax;
    }

    return (result);

} /* RegCmdTable */





/* *******************************************************************
*
* NextToken - get the next token from token_buffer using elements of the
*             token separator set as token delimiters.
*
* PRE: token_buffer is previously initialized buffer to process, token is
*      repository for next token and token_sep is a previously initialized
*      token separator object.
*
* POST: type of token parameter is set according to the contents of next token
*       in token_buffer as follows:
*
*		TK_UserSpecified - if next token is delimited by token separator set
*       TK_Identifier - if next token is alphanumeric (includes underscore)
*       TK_Number     - if next token is a span of digits
*       TK_Dash       - if next token is '-'
*       ...
*       TK_NoToken    - if there is no next token
*       TK_NotKnown   - if next token is not any known token
*       TK_NotComplete - if next token is being scanned and end of input
*                        is found before a specified delimiter
*
*       an identifier starts with a letter/underscore and optionally
*       is followed by letters, digits and/or underscores in any order.
*
*       token value of parameter is set with the value of the
*       next token.
*
* HISTORY:
*	09-JAN-91 RM	Created
*       22-JAN-91 AWH   Changed identifiers to include leading under
*                       scores. (Need more flexible definition of
*                       the set of characters that make up identifiers ??)
*       06-FEB-91 RM	Added support for user specification of delimeters.
*
******************************************************************* */

void eNextToken(token_buffer, token, token_sep)
tTokenBuffer token_buffer;
tToken token;
tTokenSeparator token_sep;
{
    int ix;
/**/

    if (token_buffer == NULL || token == NULL || token_sep == NULL) {
        return;
    }

    if (! ISINIT(token_sep)) {
        return;
    }

    /* determine what kind of token to look for */

    if (! ISDEFAULT(token_sep)) {

        /* use specified token separators to find next token */

        tTempBuf128 set;
        /**/

        BuildSet(token_sep, set, sizeof(set));

        ix = token_buffer[0].pos;

        /* skip delimeters */

        while (InSet(token_buffer[0].buffptr[ix], set)) {
            ix++;
        }

        token[0].start = ix;
        token[0].bptr = token_buffer[0].buffptr;

        if (token_buffer[0].buffptr[ix] == '\0') {

            token[0].type = TK_NoToken;
            token[0].start = 0;
            token[0].end = 0;
        }
        else {

            /* span a token delimited by set */

            while (! InSet(token_buffer[0].buffptr[ix], set) &&
                     token_buffer[0].buffptr[ix] != '\0') {
                ix++;
            }
            
            if (token_buffer[0].buffptr[ix] == '\0') {
                token[0].type = TK_NotComplete;
            }
            else {
                token[0].type = TK_UserSpecified;
            }
            token[0].end = ix - 1;

	}

    }
    else {

/* ==> start indent 4 spaces */

    /* skip white space */

    ix = token_buffer[0].pos;

    while (token_buffer[0].buffptr[ix] == ' ' ||
           token_buffer[0].buffptr[ix] == '\t' ||
           token_buffer[0].buffptr[ix] == '\n' ) {

        ix++;
    }

    token[0].start = ix;
    token[0].bptr = token_buffer[0].buffptr;

    if (token_buffer[0].buffptr[ix] == '\0') {

        token[0].type = TK_NoToken;
        token[0].start = 0;
        token[0].end = 0;
    }
    else if (isalpha((int) token_buffer[0].buffptr[ix]) ||
             token_buffer[0].buffptr[ix] == '_') {

        /* span an alphanumeric */

        while (isalnum((int) token_buffer[0].buffptr[ix]) ||
               token_buffer[0].buffptr[ix] == '_') {

            ix++;
	} /* while */

        token[0].type = TK_Identifier;
        token[0].end = ix - 1;
    }
    else if (isdigit((int) token_buffer[0].buffptr[ix])) {

        /* span a sequence of digits */

        while (isdigit((int) token_buffer[0].buffptr[ix])) {

            ix++;
	} /* while */

        token[0].type = TK_Number;
        token[0].end = ix - 1;
    }
    else if (ispunct((int) token_buffer[0].buffptr[ix])) {

        /* recognize a punctuation character */

        switch (token_buffer[0].buffptr[ix]) {

            case '!':     token[0].type = TK_Exclamation ;
                          break;

            case '"':     token[0].type = TK_DQuote ;
                          break;

            case '#':     token[0].type = TK_Pound ;
                          break;

            case '$':     token[0].type = TK_Dollar ;
                          break;

            case '%':     token[0].type = TK_Percent ;
                          break;

            case '&':     token[0].type = TK_Ampersand ;
                          break;

            case '\'':     token[0].type = TK_SQuote ;
                          break;

            case '(':     token[0].type = TK_LParenthesis ;
                          break;

            case ')':     token[0].type = TK_RParenthesis ;
                          break;

            case '*':     token[0].type = TK_Asterisk ;
                          break;

            case '+':     token[0].type = TK_Plus ;
                          break;

            case ',':     token[0].type = TK_Comma ;
                          break;

            case '-':     token[0].type = TK_Dash ;
                          break;

            case '.':     token[0].type = TK_Period ;
                          break;

            case '/':     token[0].type = TK_FSlash ;
                          break;

            case ':':     token[0].type = TK_Colon ;
                          break;

            case ';':     token[0].type = TK_SemiColon ;
                          break;

            case '<':     token[0].type = TK_LAngle ;
                          break;

            case '=':     token[0].type = TK_Equal ;
                          break;

            case '>':     token[0].type = TK_RAngle ;
                          break;

            case '?':     token[0].type = TK_Question ;
                          break;

            case '|':     token[0].type = TK_Bar ;
                          break;

            case '@':     token[0].type = TK_At ;
                          break;

            case '[':     token[0].type = TK_LSquare ;
                          break;

            case '\\':     token[0].type = TK_BSlash ;
                          break;

            case ']':     token[0].type = TK_RSquare ;
                          break;

            case '^':     token[0].type = TK_UpArrow ;
                          break;

            case '_':     token[0].type = TK_Underline ;
                          break;

            case '`':     token[0].type = TK_Tick ;
                          break;

            case '{':     token[0].type = TK_LCurly ;
                          break;

            case '}':     token[0].type = TK_RCurly ;
                          break;

            case '~':     token[0].type = TK_Tilde ;
                          break;

            default:      token[0].type = TK_NotKnown;

        } /* switch */

        token[0].end = ix;
        ix++;
    }
    else if (iscntrl((int) token_buffer[0].buffptr[ix])) {

        token[0].type = TK_Control;
        token[0].end = ix;
        ix++;
    }
    else {

        /* really not recognizable input, recognize one char only */

        token[0].type = TK_NotKnown;
        token[0].end = ix;
        ix++;
    }

/* ==> end indent 4 spaces */
    }

    token_buffer[0].pos = ix;

} /* NextToken */





/* *******************************************************************
*
* TokenValue - copy the value of token to buffer.
*
* PRE: token is a valid token, buffer is place holder for value of token,
*      max_size is maximun size for value of token, max_size > 1.
*
* POST: value of token is transferred to buffer with at most max_size
*       characters (buffer[0 .. max_size-1]), including the null character
*       at the end.
*
* HISTORY:
*	09-JAN-91 RM	Created
*
******************************************************************* */

void eTokenValue(token, buffer, max_size)
tToken token;
char * buffer;
int max_size;
{
    int ix;			/* index into token buffer */
    int count = 0;			/* index into new buffer */
/**/
    if (buffer == NULL) {
        return;
    }
    if (max_size <= 1) {
        return;
    }
    strcpy(buffer, "");
    if (token == NULL) {
        return;
    }

    if (token[0].bptr == NULL) {
        return;
    }

    if (token[0].type == TK_Control) {
    
        int c;
        char *str;
        tTempBuf80 tbuf;
        /**/
        c = token[0].bptr[token[0].start];
        if (c >= 0x00 && c<= 0x1F) {
            str = ControlCharTable[c];
        }
        else if (c == 0x7F) {
            str = "DEL";
        }
        else {
            str = "UNKNOWN CONTROL CHAR";
        }
        
        sprintf(tbuf, "<%s>", str);
        strncpy(buffer, tbuf, max_size-2);
        buffer[max_size-1] = '\0';
    }
    else {

        ix = token[0].start;
        count = 0;
        while(ix <= token[0].end && count < max_size-1) {

            buffer[count] = (token[0].bptr)[ix];
            ix++;
            count ++;
        
        } /* while */

        buffer[count] = '\0';

    }
} /* TokenValue */





/* *******************************************************************
*
* IsPrefix - determine if string1 is a prefix of string2.
*
* PRE: string1 and string2 are null terminated strings.
*
* POST: return value is True if string1 is a prefix of string2, same
*       argument is True if string1 is the same as string2;
*       otherwise return False and same argument is set to False.
*
* HISTORY:
*	14-JAN-91 RM	Created
*
******************************************************************* */

static VK_tBoolean iIsPrefix(str1, str2, same)
char str1[];
char str2[];
VK_tBoolean *same;
{
    int i = 0;
    VK_tBoolean prefix;
/**/

    if (str1 == NULL || str2 == NULL) {
        return (False);
    }

    prefix = False;
    *same = False;
    i = 0;
    while (str1[i] == str2[i] && str1[i] != '\0' && str2[i] != '\0') {
        i++;
    }

    if (i==0 && str1[i] != '\0') {
        prefix = False;
    }

    else if (str1[i] != '\0' && str2[i] == '\0') {
        prefix = False;
    }

    else if (str1[i] != '\0' && str2[i] != '\0') {
        prefix = False;
    }

    else {
        prefix = True;
        if (str1[i] == '\0' && str2[i] == '\0') {
            *same = True;
	}
    }

    return (prefix);

} /* IsPrefix */





/* *******************************************************************
*
* Pushback - put back token to the token_buffer.
*
* PRE: token is the most recent obtained token from tokenbuffer.
*
* POST: token is in token_buffer, token has a null value (type TK_NoToken).
*
* NOTES: if token is not the most recent obtained token, any other
*        tokens obtained after token will also be restored to the
*        token buffer, also the delimiters skipped before recognizing
*        token (if any) are not restored.
*
* HISTORY:
*	05-FEB-91 RM	Created
*
******************************************************************* */

void ePushback(token_buffer, token)
tTokenBuffer token_buffer;
tToken token;
{
/**/

    if (token_buffer == NULL || token == NULL) {
        return;
    }

    if (token[0].bptr != token_buffer[0].buffptr) {
        return;
    }

    token_buffer[0].pos = token[0].start;
    token[0].type = TK_NoToken;
    token[0].start = 0;
    token[0].end = 0;

} /* Pushback */





/* *******************************************************************
*
* CommandType - decide if token is a command.
*
* PRE: token_type is a valid token type.
*
* POST: return True if token_type is of command type, False otherwise.
*
* HISTORY:
*	05-FEB-91 RM	Created
*
******************************************************************* */

static VK_tBoolean iCommandType(token_type)
tTokenKind token_type;
{
    VK_tBoolean result = False;
/**/

    /* put a more robust check here */

    switch (token_type) {

    case TK_Identifier:
    case TK_Exclamation:
    case TK_Pound:
    case TK_At:
                  result = True;
                  break;
    default:
                  result = False;

    } /* switch */

    return (result);

}  /* CommandType */





/* *******************************************************************
*
* SetTokenSeparator - initialize the token separator set with the user
*                     specifications.
*
* PRE: tksp is a token separator object.
*
* POST: tksp is contains user specified options.
*
* HISTORY:
*	06-FEB-91 RM	Created
*
******************************************************************* */

void eSetTokenSeparator(tksp, va_alist)
tTokenSeparator tksp;
va_dcl
{
    va_list lptr;       /* pointer to traverse argument list */
    int arg;

/**/
    for (arg=0;arg<TSSize;arg++) {   /* mark it as unset */
        tksp[arg] = '0';
    }

    INIT(tksp);                /* mark it as initialized */

    /* unload any number of arguments */

    va_start(lptr);

    arg = va_arg(lptr, int);

    /* set default if no arguments were specified */
    /* white space is not significant */

    if ((char *) arg == NULL) {
        tksp[INIT_OFFSET-1] = '1';
    }

    while ((char *) arg != NULL) {

        assert(arg+INIT_OFFSET < TSSize || arg == TS_WhiteSpace);

        if (arg == TS_WhiteSpace) {
             tksp[TS_SP+INIT_OFFSET] = '1';
             tksp[TS_NL+INIT_OFFSET] = '1';
             tksp[TS_CR+INIT_OFFSET] = '1';
             tksp[TS_HT+INIT_OFFSET] = '1';
	}
        else {
            tksp[arg+INIT_OFFSET] = '1';
        }

        arg = va_arg(lptr, int);

    }

    va_end(lptr);

} /* SetTokenSeparator */





/* *******************************************************************
*
* InSet - determine if c is present among members of set.
*
* PRE: c is a char and set is a null terminated array.
*
* POST: returns True if c is in set, False otherwise.
*
* HISTORY:
*	06-FEB-91 RM	Created
*
******************************************************************* */

static VK_tBoolean iInSet(c, set)
char c;
char *set;
{
/**/
    assert(set != NULL);
    
    while (c != *set && *set) {
        set++;
    }

    return (*set != '\0');

} /* InSet */




/* *******************************************************************
*
* BuildSet - construct an array of characters specified by the contents
*            of object tk_sp.
*
* PRE: tk_sp is a token separator object.
*
* POST: set is a null terminated string consisting of characters specified
*       by tk_sp, the length of the string will be < size or zero.
*
* NOTE: this allows a null character to be included in the set, in which
*       case any other delimiters following the null will be ignored by
*       other routines that assume set is null terminated; also it is
*       not strictly a set. (more than one element can be present)
*
* HISTORY:
*	06-FEB-91 RM	Created
*
******************************************************************* */

static void iBuildSet(tk_sp, set, size)
tTokenSeparator tk_sp;
char set[];
int size;
{
    int i, j;
/**/
    if (tk_sp == NULL || set == NULL) {
        return;
    }
    j = 0;
    i = 0;
    while (j<size && i< TSSize) {

        if (tk_sp[i+INIT_OFFSET] == '1') {
            set[j++] = i;
	}
        i++;
    }

    set[j] = '\0';

} /* BuildSet */




/* *******************************************************************
*
* TokenBufferValue - copy the contents of the token buffer to buf.
*
* PRE: tk_buf is a token buffer object, buf is a char buffer of
*      size > 0.
*
* POST: buf is a null terminated string of length at most size-1,
*       containing the current value of tk_buf.
*
* HISTORY:
*	07-FEB-91 RM	Created
*
******************************************************************* */

void eTokenBufferValue(tk_buf, buf, size)
tTokenBuffer tk_buf;
char *buf;
int size;
{
/**/
    if (tk_buf == NULL) {
        return;
    }

    if (tk_buf[0].buffptr == NULL) {
        return;
    }

    if (size <= 0) {
        return;
    }

    if (size > 1) {  
        strncpy(buf, &tk_buf[0].buffptr[tk_buf[0].pos], size-1);
    }
    buf[size-1] = '\0';
  

}  /* TokenBufferValue */
