static char SCCS_ID[] = "@(#)FWMain.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: FWnMain.c
*
* DESCRIPTION:  
*	Main module for waveform formatter (fmtwave).
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	15-MAY-91 DSB	Created from FmtWave.c
*	02-JUL-91 JBR	Moved iAllocateMemory and iFreeMemory to FWUtils.c.
*	02-JUL-91 JBR	Replaced TKCommon.h with FWCommon.h.
*	02-JUL-91 JBR	Added explicit SPI.h (was in TKCommon.h).
*	12-SEP-91 MNP	Added preview option, copied from VLK kernel.
*			Added test mode. 
*	24-OCT-91 MNP	Added names mode.
*			
* SCCS ID: @(#)FWMain.c	42.1 1/10/92
*
******************************************************************* */
#include "Version.inc"
#include "FWCommon.h"			/* for FWTempBuf, etc. */
#include "ParseCommand.h"	/* for PCL_ stuff */
#include "TKIO.h"		/* for ControlFileExtension */
#include "SPI.h"		/* for OpenDLS and CloseDLS */

#include "FWMisc.h"		/* for VersionText, GoodExit, etc. */
#include "FWTrace.h"		/* for FWT_* */

extern FILE * F_WaveFile;	/* output waveform file */

/* *******************************************************************
*
* Parse Command Line Delcarations	?? spelling error ??
*
******************************************************************* */
#define F_ParamRootName "ROOTNAME"
#define F_QualTemplate 	"TEMPLATE"
#define F_QualControl 	"CONTROL"
#define F_QualPreview 	"PREVIEW"
#define F_QualTest 	"TEST"
#define F_QualNames	"NAMES"

static PCL_tParameterTable F_Parameters =
{

     {F_ParamRootName, 	PCL_Fixed|PCL_Required, "model name",	NULL},	
     {NULL, 	0, 	NULL, 		NULL}
};

static PCL_tQualifierTable F_Qualifiers =
{
    {F_QualTemplate, 	PCL_Nonnegatable|PCL_Variable, 	1},
					/* generate control file */
    {F_QualControl,	PCL_Nonnegatable|PCL_Variable,	1},
					/* use control file */   
    {F_QualPreview,	PCL_Nonnegatable|PCL_Variable,  1},
				/* produce preview only */   
    {F_QualTest,	PCL_Negatable,  0},
				/* don't print a version banner */   
    {F_QualNames,	PCL_Negatable,  0},
    				/* print signal names across top */
    {NULL, 		0, 		0}
};


/* *******************************************************************
*
* ParseCommandLine - Parse Command Line using PCL from the tool box 
*
* HISTORY:
*	14-OCT-90 AWH	Created.
*	02-JUL-91 JBR	Removed references to VK_tCString.
*	13-SEP-91 MNP	Added preview from vlk. Added test mode.
*
******************************************************************* */
static VK_tBoolean ParseCommandLine(argc, argv, root_name, template, control,
	control_name, preview)
int argc;
char * argv[];
FWString root_name;
VK_tBoolean *template;
VK_tBoolean *control;
FWString control_name;
Integer4 * preview;
{
   FWString valueP ;
/**/
	
     /* set up PCL */
    if (PCL_SetTable(F_Parameters, F_Qualifiers) != PCL_Normal) {
	fprintf(stderr, ",%s (%s)\n", PCL_ErrorMessage(),
		PCL_ErrorString());
	return False;
    }

    if (PCL_ParseArgs(argc, argv) != PCL_Normal) {
	fprintf(stderr, "%s (%s)\n", PCL_ErrorMessage(),
		PCL_ErrorString());
	return False;
    }

    /* prefix */
    switch (PCL_GetValue(F_ParamRootName, valueP)) {
	case PCL_Normal:
	    strcpy(root_name, valueP);
	    break;
	default:
	    fprintf(stderr, "%s (%s)\n", PCL_ErrorMessage(),
		PCL_ErrorString());
	    return FALSE;
    } /*switch*/
 
    /* template */
    switch(PCL_IsPresent(F_QualTemplate)) {
	case PCL_Present:
	    *template = True;
	    switch(PCL_GetValue(F_QualTemplate, valueP)) {
		case PCL_Normal:
		case PCL_More:
		    strcpy(control_name, valueP);
		    break;
		case PCL_Absent:
		    strcpy(control_name, root_name);
		    strcat(control_name, ControlFileExtension);
		    break;
		default:
	    	    fprintf(stderr, "%s (%s)", PCL_ErrorMessage(),
			PCL_ErrorString());
	    	    return FALSE;
 	    } /*switch*/
	    break;
	case PCL_Absent:
	case PCL_Negated:
	    *template = False;
	    break;
	default:
	    fprintf(stderr, "%s (%s)\n", PCL_ErrorMessage(),
		PCL_ErrorString());
	    return False;
     }/* switch */

 
    /* control */
    switch(PCL_IsPresent(F_QualControl)) {
	case PCL_Present:
	    *control = True;
	    switch(PCL_GetValue(F_QualControl, valueP)) {
		case PCL_Normal:
		case PCL_More:
		    strcpy(control_name, valueP);
		    break;
		case PCL_Absent:
		    strcpy(control_name, root_name);
		    strcat(control_name, ControlFileExtension);
		    break;
		default:
	    	    fprintf(stderr, "%s (%s)\n", PCL_ErrorMessage(),
			PCL_ErrorString());
	    	    return FALSE;
 	    } /*switch*/
	    break;
	case PCL_Absent:
	case PCL_Negated:
	    *control = False;
	    break;
	default:
	    fprintf(stderr, "%s (%s)\n", PCL_ErrorMessage(),
		PCL_ErrorString());
	    return False;
     }/* switch */

    /* preview */
    switch(PCL_IsPresent(F_QualPreview)) {
	case PCL_Present:
	    switch(PCL_GetValue(F_QualPreview, valueP)) {
		case PCL_Normal:
		case PCL_More:
		    *preview = atoi(valueP);
       		    break;
		case PCL_Absent:
		    *preview = 1;	/* default value */
		    break;
		default:
	    	    fprintf(stderr,"%s (%s)\n", PCL_ErrorMessage(),
			PCL_ErrorString());
	    	    return FALSE;
 	    } /*switch*/
	    break;
	case PCL_Absent:
	case PCL_Negated:
	    *preview = NO_PREVIEW;
	    break;
 
	default:
	    fprintf(stderr,"%s (%s)\n", PCL_ErrorMessage(),
		PCL_ErrorString());
	    return False;
    }/* switch */

    /* test */
    switch (PCL_IsPresent(F_QualTest)) {
	case PCL_Present:
	    TestMode = True;
	    break;
	case PCL_Absent:
	case PCL_Negated:
	    TestMode = False;
	    break;
   	default:
	    fprintf(stderr,"%s (%s)\n", PCL_ErrorMessage(),
		PCL_ErrorString());
	    return False;
    } /* switch */

   /* names */
   NamesMode = True;
   if (TestMode) {
       NamesMode = False;		/* Test mode implies Names. */
   }
   else {
       switch (PCL_IsPresent(F_QualNames)) {
	 case PCL_Present:
	 case PCL_Absent:
	   break;
	 case PCL_Negated:
	   NamesMode = False;
	   break;
	 default:
	   fprintf(stderr,"%s (%s)\n", PCL_ErrorMessage(),
		   PCL_ErrorString());
	   return False;
       } /* switch */
   }

   /* Skip columns if test mode and names mode are off */
   SkipColumns = (TestMode || !NamesMode) ? 0 : 1;

   return True;

} /* ParseCommandLine */


/* *******************************************************************
*
* DisplayPreviewMessage
*
* HISTORY:
*	16-MAY-91 AWH	Created.
*
******************************************************************* */

void DisplayPreviewMessage()
{
static char message[] = "\n\
This is a preview of the formatted output waveform file. With the current \n\
format control file, only the above-specified signals will be formatted. \n\
Enter the following command to generate the entire formatted output waveform\n\
file:\n\
\n\
                           format <top_level_entity>\n\
\n";


/**/
    fprintf(F_WaveFile, message, Fmt_SkipTraceLine);

} /* DisplayPreviewMessage */


/* *******************************************************************
*
* main - 
*
* HISTORY:
*	15-NOV-89 DSB	Created.
*
******************************************************************* */

int main(argc, argv)
int argc;
char **argv;
{
    FWTempBuf80 prefix;
    FWTempBuf80 control_file;
    VK_tBoolean template;
    VK_tBoolean control;
    Integer4 preview;
/**/

#ifdef VCG_VLK
    fprintf(stderr, "%%FMT-I-Ident, VHDL Learning Kit Version %s.\n",
	    VLK_SYSTEM_VERSION_4);
#else
    fprintf(stderr, "%%FMT-I-Ident, Retargetable VHDL Code Generator Waveform Formatter Version %s.\n",
	    RVCG_SYSTEM_VERSION_4);
#endif

#if 000
    fprintf(stdout,
"***************************************************************************\n");
    fprintf(stdout,
 "    CLSI Retargetable VHDL Code Generator %s Waveform Formatter.\n",
	VersionText);
    fprintf(stdout,
"***************************************************************************\n");
#endif /* 000 */

    OpenDLS();

    if (! ParseCommandLine(argc, argv, prefix, &template, &control, 
		control_file, &preview)) {
        CloseDLS();
	exit(BadExit);
    }
 
    if (template) {
	ControlGenerate(prefix, control_file);
    }
    else {
	Format(prefix, control, control_file, preview);
    }


   CloseDLS();
   exit(GoodExit);
   return 0;			/* gcc: must return from non-void */
} /* main */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created.
*
******************************************************************* */

<static> Fntype RtnName(parm)				/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif
