static char SCCS_ID[] = "@(#)FWMisc.c	42.2 1/13/92";
/* *******************************************************************
*
* Copyright (c) 1991-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: FWMisc.c
*
* DESCRIPTION:  
*	<(usually) short description of module or its purpose>
*
* CONSTANTS:
*	<constants>	<constants defined in this .h file >
*
* TYPES:
*	<types>		<types defined in this .h file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this .h file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	21-MAY-91 DSB	Cleaned up old types; renamed to remove some VK_
*	02-JUL-91 JBR	Removed #include of LibFnsPub.h, SPI.h, and VKConfig.h.
*	02-JUL-91 JBR	Replace TKCommon.h with FWCommon.h.
*	19-SEP-91 MNP	Add preview from vlk kernel. Added test mode.
*	26-SEP-91 MNP	Handle trace version error ungracefully: quit.
*	 4-NOV-91 MNP	Add simple names to wave.
*	11-NOV-91 MNP	Split WriteHeader and WriteTimeSlice for VLK to share.
*
* SCCS ID: @(#)FWMisc.c	42.2 1/13/92
*
******************************************************************* */

#include "FWMisc.h"		/* include module's own header file */

/* *******************************************************************
*
* NOTE: fmtwave qualifiers and parameters
*       
*   Usage:
*	fmtwave <model> [-template [<tfile>] | -control [<cfile>]]
*
*	<model> : top level entity name in the model simulated. Used
*		as the prefix to the wave file name and the trace
*		file name.
*	
*	-template [<tfile>] : produces an  example control file with the
*			name <tfile>. The default file name is the model
*			name with the .control suffix.
*
*	-control [<cfile>] : directs the format of the wave file based on
*			the commands found in the file <cfile>. The default
*			name for teh <cfile> is the model name with the
*			.control suffix.
*
*	-preview [<count>] : limits output to only <count> number of lines
*			default is unlimited
*
* NOTE: fmtwave uses qqEffectiveValue and qqPreviousValue as
*	POINTERS to string representation of value. (Test kernel uses
*	them as value or position value.)
*
******************************************************************** */
#include "Version.inc"
#define   VK_NO_VERIFY	1	/* causes TKMain to link */
#include "TKTemp.h"
#include "FWCommon.h"		/* for Boolean and FWTempBuf# */
#include "TKStructs.h" 
#include "TKIO.h"		/* for codes */
#include "TKData.h"		/* for TIME macros */
#include "TKListFns.h"
#include "TKRecord.h"		/* for RD_ stuff */

#include "FWTrace.h"		/* for FWT_ stuff */
#include "FWUtils.h"		/* for mem stuff */
#include "ParseCommand.h"	/* for PCL_ stuff */

/* (The Format strings have moved to FWTrace.h) */

/*****************************************************************************
*
* Globals
*
*****************************************************************************/

/* extern */ FILE * F_TraceFile;	/* input simulation trace file */
/* extern */ FILE * F_WaveFile;		/* output waveform file */
/* extern */ FILE * F_ControlFile;	/* input/output formate control file */

Integer4 SkipColumns;		/* To skip or not to skip... See .h file */
Integer4 WaveWidth;		/* The print width of wave file */
VK_tBoolean TestMode;		/* suppress version banner */
VK_tBoolean NamesMode;		/* suppress name printing */

/* *******************************************************************
*
* Control File Declarations
*
* NOTE:	The control file used to control the output of fmtwave. The
*	file is currently not robustly defined. Just something quick.
*
* Control File Records:
* DISPLAY <scalar signal number> [<scalar signal number>]
*	 # display signal or range of signals
*
******************************************************************* */
#define CF_DisplaySignal "DISPLAY"



/* *******************************************************************
*
* ReadControlMapSignal - 
*
* HISTORY:
*	15-OCT-90 AWH	Created.
*
******************************************************************* */

static void ReadControlMapSignal(traceline)
char * traceline;
{
    int signal_first; 
    int signal_last;

/**/
#if 000
    int index;

/* ?? fix indentation ?? */
    switch (sscanf(traceline, "%d %d", &signal_first, &signal_last)) {
	case 1:
		signal_last = signal_first;
		break;
	default:			/* ?? fix this ?? */
		signal_first = 0;
		signal_last = 0;
		break;
	} /* switch */

    for (index = signal_first; index <= signal_last; index++) {
	FWT_SetSignalDisplay(index);
    } /*for*/
#endif

    /* Above not used: there's now one line for each signal element. */

    if (sscanf(traceline, "%d", &signal_first)) {
	FWT_SetSignalDisplay(signal_first);
    }
    else {    
	fprintf(stderr, 
		"fmtwave: unknown signal spec in control file.\n");
	fprintf(stderr, "\t%s", traceline);
    }

	      
} /* ReadControlMapSignal */



/* *******************************************************************
*
* ReadControlFile - 
*
* HISTORY:
*	15-OCT-90 AWH	Created.
*
******************************************************************* */

static void ReadControlFile()
{
    FWTempBuf80 tokenbuf;
    FWTempBuf256 linebuf;
    FWTempBuf256 bcbuf;
    char * ptr;
    int len;
/**/
/* ?? fix indentation ?? */

    if (NULL != F_ControlFile) {
        while (! feof(F_ControlFile)) {
		tokenbuf[0]=0;
		bcbuf[0]=0;
		ptr = fgets(linebuf, sizeof(linebuf), F_ControlFile);
		if (ptr != NULL) {
		    len = strlen(ptr);
		    if (ptr[len] == '\n') {
		    	ptr[len] = 0;
		    }	
		sscanf(linebuf, "%[^!#\t ] %[^!#\n]", tokenbuf,
			bcbuf); 
		if ( '\0' != tokenbuf[0] ) {
		    
		    if (0 == strncmp(CF_DisplaySignal, tokenbuf, 2)) {
		    	ReadControlMapSignal(bcbuf);
		    }
		    else {
			fprintf(stderr, 
			   "fmtwave: unknown line in control file.\n");
			fprintf(stderr, "\t%s", linebuf);
		    }
		}
	    } /* while */
	}
    }
    else {
	/* mark each signal to display */ 
	FWT_SetAllSignalDisplay();
    }
} /* ReadControlFile */



/*****************************************************************************
*
* WriteWaveSignalHeader - Writes version and signal map to wave file.
*
* DESCRIPTION:
*
* HISTORY:
*	 6-NOV-91 MNP	Created, split off from WriteHeader.
*
*****************************************************************************/

WriteWaveSignalHeader(testing)
VK_tBoolean testing;		/* in: test mode on */
{
    TempItem scan1;
    FWT_tSignalGroupHandle group;
/**/
    /* TestMode is set by -test command line switch. It inhibits 
       printing of the next two lines. MNP */

    if (!testing)
    {
	fprintf(F_WaveFile, "CLSI Retargetable VHDL Code Generator %s\n",
		VersionText);
	fprintf(F_WaveFile, "\n");
    }

    fprintf(F_WaveFile, "Formatted Test Kernel Simulation Output\n");

    fprintf(F_WaveFile, "\n");
    fprintf(F_WaveFile, "Signal       Signal\n");
    fprintf(F_WaveFile, "Number       Path Name\n");
    fprintf(F_WaveFile, 
"----------   -------------------------------------------------------------------\n");
    for (scan1=TempFirstItem(FWT_SignalGroupList);
		! TempNullItem(scan1); scan1 = TempNextItem(scan1)) {

	group = FWT_SigGrpValue(scan1);
	if (FWT_CheckSignalDisplay(group)) {
            switch (qqGroupClass(group)) {
	      case gc_ScalarSignal:
	        fprintf(F_WaveFile, "       %-3d : %s\n", 
			qqSignalStart(group), qqName(group));
	        break; 
	      case gc_VectorSignal:
	      case gc_CompositeSignal:
	        fprintf(F_WaveFile, " %3d - %-3d : %s\n", 
			qqSignalStart(group), 
			qqSignalStart(group) + qqSignalLength(group) - 1, 
			qqName(group));
	        break;
	      default:
		fprintf(F_WaveFile, "Unknown signal class\n");
		break;
	     } /*switch*/
	 } /* if  */
    }	/* for  */
    fprintf(F_WaveFile, "\n");

} /* WriteWaveSignalHeader */


/* *******************************************************************
*
* WriteHeader - Writes to wave file the signal header.
*
* HISTORY:
*	03-OCT-89 DSB	Created.
*	...
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	13-SEP-91 MNP	Added test mode.
*	 4-NOV-91 MNP	added simple names
*	 6-NOV-91 MNP	Factored out pieces into FormatWaveHeader and 
*			WriteWaveSignalHeader, for WaveTool to share.
*
******************************************************************* */

void WriteHeader()
{
    char *buf;	
    int bufsize;
/**/

    WriteWaveSignalHeader(TestMode);		/* version, signal map */

    bufsize = WaveWidth * 3 + 9;		/* 3 lines + some fudge.*/
    AllocateMemory(buf, char, bufsize);     
    
    FWT_FormatWaveHeader(buf, bufsize, NamesMode, SkipColumns);
    fprintf(F_WaveFile, buf);
    FreeMemory(buf);
    
} /* WriteHeader */



/* *******************************************************************
*
* WriteTimeSlice - 
*
* DESCRIPTION:
*	Formats and prints a time slice line. 
*
* HISTORY:
*	03-OCT-89 DSB	Created.
*	 6-NOV-91 MNP	Split real work off to FormatTimeSlice.
*
******************************************************************* */

void WriteTimeSlice()
{
    static char *buf = NULL;
    static int bufsize;
/**/
    /* First time through, allocate the buffer */
    if (NULL == buf)  {
	bufsize = WaveWidth + 3;
	AllocateMemory(buf, char, bufsize);     
    }
    
    FWT_FormatTimeSlice(buf, bufsize, SkipColumns);
    fprintf(F_WaveFile, "%s\n", buf);

} /* WriteTimeSlice */


/* *******************************************************************
*
* DisplayTraceLinesSkipped 
*
* HISTORY:
*	31-DEC-90 AWH	Created.
*
******************************************************************* */

void DisplayTraceLinesSkipped()
{
/**/
    fprintf(F_WaveFile, "%s", Fmt_SkipTraceLine);
} /* DisplayTraceLinesSkipped */



/* *******************************************************************
*
* DisplayAssertion - 
*
* HISTORY:
*	??-SEP-89 DSB	Created.
*	11-MAY-90 DSB	Added process pathname
*	28-DEC-90 AWH	Changed to use TKRecord
*	31-DEC-90 AWH	Reorganized FmtWave
*	02-JUL-91 JBR	Changed line_number to int to remove SPI dependency.
*
******************************************************************* */

void DisplayAssertion(process_name, source_file, line_number, message, 
	severity)
VK_tCString process_name;
VK_tCString source_file;
int line_number;
VK_tCString message;
VK_tCString severity;
{
    FWTempBuf80 time_buffer;
/**/
    UTIME_FORMAT(time_buffer, FWT_SimTime);
    fprintf(F_WaveFile, 
	"%s%sAssertion Violation: at time = %s %s + %d,\n", 
	Fmt_TimeFill, Fmt_TimeSepFill, time_buffer, BaseUnit, FWT_DeltaCount);
    fprintf(F_WaveFile, 
	"%s%s  in process %s,\n", 	Fmt_TimeFill, Fmt_TimeSepFill, 
		process_name);
    fprintf(F_WaveFile, 
	"%s%s  from line %d of file %s,\n", 
	Fmt_TimeFill, Fmt_TimeSepFill, 
	line_number, 
	source_file);
    fprintf(F_WaveFile, 
	"%s%s  %s: \"%s\".\n", 	Fmt_TimeFill, Fmt_TimeSepFill, severity, 
	message);

} /* DisplayAssertion */


/* *******************************************************************
*
* DisplayError - 
*
* HISTORY:
*	31-DEC-90 AWH	Created from ProcessTraceError
*	02-JUL-91 JBR	Changed line_number to int to remove SPI dependency.
*
******************************************************************* */

void DisplayError(message, extra, source_file, line_number)
  VK_tCString message;
  VK_tCString extra;
  VK_tCString source_file;
  int line_number;
{
    FWTempBuf80 time_buffer;
/**/
    UTIME_FORMAT(time_buffer, FWT_SimTime);
    fprintf(F_WaveFile,
	"%s%sError: at time = %s %s + %d,\n",
	Fmt_TimeFill, Fmt_TimeSepFill, time_buffer, BaseUnit, FWT_DeltaCount);
    fprintf(F_WaveFile,
	"%s%s from line %d of file %s,\n",
	Fmt_TimeFill, Fmt_TimeSepFill,
	line_number, source_file);
    fprintf(F_WaveFile,
	"%s%s %s\n",
	Fmt_TimeFill, Fmt_TimeSepFill, message);

} /* DisplayError */


/* *******************************************************************
*
* InitializeFmtWave - 
*
* HISTORY:
*	??-SEP-89 DSB	Created.
*	24-APR-90 DSB	Converted to combined map/trace file
*	19-SEP-91 MNP	Added preview from vlk kernel.
*
******************************************************************* */

static void InitializeFmtWave(prefix, control, control_file, preview)
VK_tCString prefix;
VK_tBoolean control;
VK_tCString control_file;
Integer4 preview;
{
    FWTempBuf256 file_name_buf;
/**/
    sprintf(file_name_buf, "%s%s", prefix, TraceFileExtension);
    F_TraceFile = fopen(file_name_buf, "r");
    if (F_TraceFile == NULL) {
	fprintf(stderr, 
		"fmtwave error: simulation trace file %s not found\n", 
		file_name_buf);
	exit(BadExit);
    }
    RD_SetInput(F_TraceFile);

    /* This preview stuff is from the VLK kernel. Let's see if it 
       works here. 910913 MNP */

#if 000
    sprintf(file_name_buf, "%s%s", prefix, WaveFileExtension);
    F_WaveFile = fopen(file_name_buf, "w");
    if (F_WaveFile == NULL) {
	fprintf(stderr, 
		"fmtwave error: simulation wave file %s not opened\n", 
		file_name_buf);
	exit(BadExit);
    }
#else
    F_WaveFile = NULL;
    if (NO_WAVE_FILE != preview) {
    	sprintf(file_name_buf, "%s%s", prefix, WaveFileExtension);
    	F_WaveFile = fopen(file_name_buf, "w");
    	if (F_WaveFile == NULL) {
	    fprintf(stderr, 
		"fmtwave error: simulation wave file %s not opened\n", 
		file_name_buf);
	    exit(BadExit);
    	}
    	fprintf(stderr, 
       	   "%%FMT-I-Create, Creating formatted wave file: %s.\n",
	    file_name_buf);
    }
#endif /* 000 */

    if (control) {
	F_ControlFile = fopen(control_file, "r");
	if (F_WaveFile == NULL) {
	    fprintf(stderr, 
		"fmtwave error: control file %s not found\n", 
		control_file);
	    exit(BadExit);
	}
    }
    FWT_Initialize();
} /* InitializeFmtWave */


/* *******************************************************************
*
* TerminateFmtWave - 
*
* HISTORY:
*	??-SEP-89 DSB	Created.
*	24-APR-90 DSB	Converted to combined map/trace file
*
******************************************************************* */

static void TerminateFmtWave()
{
/**/
    fclose(F_TraceFile);
    fclose(F_WaveFile);
} /* TerminateFmtWave */


/* *******************************************************************
*
* Format - 
*
* HISTORY:
*	??-SEP-89 DSB	Created.
*	24-APR-90 DSB	Converted to combined map/trace file
*	19-SEP-91 MNP	Added preview from vlk.
*
******************************************************************* */

void Format(prefix, control, control_file, preview)
VK_tCString prefix;
VK_tBoolean control;
VK_tCString control_file;
Integer4 preview;
{
    Integer4 total_signals;
    Integer4 displayed_signals;
    Integer4 wave_width;
   
/**/
    FWT_PreviewLineCount = preview;

    InitializeFmtWave(prefix, control, control_file, preview);

    FWT_SignalMap();

    ReadControlFile();


    /* display width information information message */
    FWT_CalculateWidth(NamesMode, SkipColumns, &total_signals,
		       &displayed_signals, &wave_width);
    wave_width += strlen(Fmt_TimeFill) +       /* add width of time */
	          strlen(Fmt_TimeSepFill);     /* add width of time seperator*/
   
    WaveWidth = wave_width;
    
    if (NO_WAVE_FILE == preview) {
      fprintf(stderr, 
         "%%FMT-I-Width, %d signal(s) will be formatted, requiring %d columns.\n", 
	 displayed_signals, wave_width);
     }
    else {
      fprintf(stderr, 
         "%%FMT-I-Width, %d signal(s) formatted, requiring %d columns.\n", 
	 displayed_signals, wave_width);
 	
         WriteHeader();

         FWT_SignalActivity();

    }


    TerminateFmtWave();
} /* Format */


/* *******************************************************************
*
* ControlProcessTraceSignalDecl - DEAD
*
* HISTORY:
*	15-OCT-90 AWH	Created.
*	31-DEC-90 AWH	Changed to use TKRecord
*
******************************************************************* */

static void ControlProcessTraceSignalDecl(record)
RD_tRecord * record;
{
/**/
    if (record->signal_declaration.group == gc_ScalarSignal ) {
    	fprintf(F_ControlFile, "%s %d # %s\n", CF_DisplaySignal,
		 record->signal_declaration.start, 
		 record->signal_declaration.name);
    }
    else {
	fprintf(F_ControlFile, "%s %d %d # %s\n", CF_DisplaySignal,
		record->signal_declaration.start,
		record->signal_declaration.length + 
		   record->signal_declaration.start - 1, 
		record->signal_declaration.name);
    }
} /* ControlProcessTraceSignalDecl */


/*****************************************************************************
*
* ControlProcessTraceSignalElementDecl - 
*
* DESCRIPTION:
*
* HISTORY:
*	 4-NOV-91 MNP	Created, from ControlProcessTraceSignalDecl
*
*****************************************************************************/

static void ControlProcessTraceSignalElementDecl(record)
RD_tRecord * record;
{
    FWT_tSignalGroupHandle group;
/**/
    group = FindSignalGroupById(record->signal_element_declaration.group_id);
    if (NULL != group) {
	fprintf(F_ControlFile, "%s %d # %s%s\n", CF_DisplaySignal,
		record->signal_element_declaration.start, 
		qqName(group),
		record->signal_element_declaration.subel_name);
    }

} /* ControlProcessTraceSignalElementDecl */


/* *******************************************************************
*
* ControlReadTraceRecords - 
*
* HISTORY:
*	15-OCT-90 AWH	Created.
*	26-SEP-91 MNP	Handle trace version error ungracefully: quit.
*	 5-NOV-91 MNP	Now reads all signal group and el. decls into
*			the lists as in format mode.
*
******************************************************************* */

static void ControlReadTraceRecords()
{
    RD_tRecord record;
    VK_tBoolean done;
/**/

    done = False;
    while ((!done) && RD_ReadNextRecord(&record)) {
	switch(record.type) {

	    /* NEW: Uses the same Process* funs the format mode
	       uses. These just put group and element records into
	       the lists for use when finished reading. MNP 911105 */

	    case RD_SignalGroupDeclaration:
	    	ProcessTraceSignalGroupDecl(&record);
	        break;
	     
	    case RD_SignalElementDeclaration:
		ProcessTraceSignalElementDecl(&record);
	    	break;

	    case RD_EndOfSignalMap:
		done = True;
	    	break;

	      case RD_VersionError:
		fprintf(stderr,
 "%%FMT-F-BadVer, Bad trace file version %d.%d (%d.%d expected).\n", 
			record.version_error.first, 
			record.version_error.second,
			RD_VERSION_MAJOR, RD_VERSION_MINOR);
		exit(1);
		break;

	    default:
		break;

	} /* switch */
	RD_FreeRecord(&record);
    } /* while */
} /* GenerateReadTraceRecords */


/* *******************************************************************
*
* ControlInitialize - 
*
* HISTORY:
*	14-OCT-90 AWH	Created.
*
******************************************************************* */

static void ControlInitialize(prefix, control_file)
VK_tCString prefix;
VK_tCString control_file;
{
    FWTempBuf256 file_name_buf;
/**/
    sprintf(file_name_buf, "%s%s", prefix, TraceFileExtension);
    F_TraceFile = fopen(file_name_buf, "r");
    if (F_TraceFile == NULL) {
	fprintf(stderr, 
		"fmtwave error: simulation trace file %s not found\n", 
		file_name_buf);
	exit(BadExit);
    }
    RD_SetInput(F_TraceFile);

    sprintf(file_name_buf, "%s%s", prefix, WaveFileExtension);
    F_ControlFile = fopen(control_file, "w");
    if (F_ControlFile == NULL) {
	fprintf(stderr, 
		"fmtwave error: simulation control file %s could not open\n", 
		control_file);
	exit(BadExit);
    }

    FWT_Initialize ();	/* added so keeps a sig grp list */

} /* ControlInitialize */




/*****************************************************************************
*
* ControlWriteControlFile - Writes body of control file out.
*
* DESCRIPTION:
*	For each group in group list, write a group comment header
* 	(for humans), then one line for each element.
*
* HISTORY:
*	 5-NOV-91 MNP	Created
*
*****************************************************************************/

ControlWriteControlFile()
{
    TempItem grpscan;
    FWT_tSignalGroupHandle group;
    FWT_tSignalElementHandle element;
    int	eltindex;
/**/

    /* For all groups .. */

    for (grpscan = TempFirstItem(FWT_SignalGroupList);
	 ! TempNullItem(grpscan);
	 grpscan = TempNextItem(grpscan)) {

	group = FWT_SigGrpValue(grpscan);

	switch (qqGroupClass(group)) {
	    
	  case gc_ScalarSignal:
	    /* Just write one line */
	    fprintf(F_ControlFile, "%s %d # %s\n", CF_DisplaySignal,
		    qqSignalStart(group),
		    qqName(group));
	    break;

	  case gc_VectorSignal:
	    /* A comment line... */
	    fprintf(F_ControlFile, "### %s \n", qqName(group));

	    /* then one line for each element. */
	    for (eltindex = qqSignalStart(group);
		 eltindex < qqSignalStart(group) + qqSignalLength(group);
		 eltindex++) {
		element = FindSignalByNumber(eltindex);
		if (NULL != element) {
		    fprintf(F_ControlFile, "%s %d # %s%s\n", 
			    CF_DisplaySignal,
			    qqNumber(element),
			    qqSimpleName(group), 
			    qqSubelName(element));
		} /* if elt */
	    } /* for all elements in group */
	    break;
	    
	  default:	
	    fprintf(stderr, "Bad switch in ControlWriteControlFile");
	    return;

	} /* switch */
    } /* For all groups */

} /* ControlWriteControlFile */


/* *******************************************************************
*
* ControlTerminate - 
*
* HISTORY:
*	14-OCT-90 AWH	Created.
*
******************************************************************* */

static void ControlTerminate()
{
/**/
    fclose(F_TraceFile);
    fclose(F_ControlFile);
} /* ControlTerminate */


/* *******************************************************************
*
* ControlGenerate - Generate control file
*
* HISTORY:
*	14-OCT-90 AWH	Created.
*	 5-NOV-91 MNP	Now reads all signal group and el. decls into
*			the lists first, as in format mode, then writes
*			the control file out.
*
******************************************************************* */

void ControlGenerate(prefix, control_file)
VK_tCString prefix;
VK_tCString control_file;
{
/**/
    ControlInitialize(prefix, control_file);

    ControlReadTraceRecords();
    
    ControlWriteControlFile();

    ControlTerminate();
} /* GenerateControl */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created.
*
******************************************************************* */
<static> Fntype RtnName(parm)	/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */
#endif

