static char SCCS_ID[] = "@(#)FWTrace.c	42.2 1/13/92";
/* ******************************************************************* 
* 
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: FWTrace.c
*
* DESCRIPTION:  
*	Module that manages data structures that represent trace file
*       information over time 
*	<then describe contents> 
* 
* CONSTANTS: 
*	<constants>	<constants defined in this header file > 
* 
* TYPES: 
*	<types>		<types defined in this header file > 
* 
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	31-DEC-90 AWH	Created from portions of FmtWave.c
*	18-MAR-91 HK	Added references to TIME macros.
*	01-APR-91 WAG	W0015.  Removed inclusion of TKMisc.h; added
*			iAllocateMemory	routine.
*	02-APR-91 WAG	W0015.  Moved the declaration of iAllocateMemory
*			in here and made the routine static.
*	02-MAY-91 DSB	Cleaning up generally
*	06-MAY-91 DSB	Cleaning up signal changes
* 	05-SEP-91 MNP	Added FWT_SignalActivityOnce(), from VLK's FWTrace.c.
*	13-SEP-91 MNP	Added preview functionality, from VLK. 
*	26-SEP-91 MNP	Handle trace version error ungracefully: quit.
*	19-NOV-91 DKC	Changed to ignore group size in SG decl. 
*                       - counts elements instead.
*
* SCCS ID: @(#)FWTrace.c	42.2 1/13/92
*
******************************************************************* */

#include "FWTrace.h"
#include "TKRecord.h"	/* for: RD_tRecord, RD_<...> */
#include "TKData.h"	/* for TIME macros */
#include "FWMisc.h"	/* for WriteTimeSlice ... */

#define FWT_ElementValue(item) ((FWT_tSignalElementHandle)(TempValue(item)))

/* *******************************************************************
*
* Global Variables
*
******************************************************************* */
/* extern */ tAbsTimeBuf FWT_SimTime;
/* extern */ tSimCycleCount FWT_SimCycleCount;
/* extern */ Integer4 FWT_DeltaCount;
/* extern */ VK_tBoolean FWT_DoTimeSlice; /* Write TimeSlice this cycle */

/* extern */ TempList FWT_SignalElementList;
/* extern */ TempList FWT_SignalGroupList;
/* extern */ TempList FWT_TypeList;

/* extern */ Integer4 FWT_PreviewLineCount = FWT_NO_PREVIEW;
/* extern */ Integer4 FWT_LineCount;


/* *******************************************************************
*   FWTrace Global Variables
******************************************************************* */
static VK_tBoolean Simulating = False;

void CatSprintf(
#ifdef VK_ANSI_FN_PROTOS
        char *buf,
        int  bufsize,
        char *format,
        ...
#endif
    );


/*****************************************************************************
*
* CatSprintf - Appends sprintf output to the end of a buffer.
*
* DESCRIPTION:
*	1. Minimal error checks. No warnings, but tries not to overwrite.
*	2. 1k limit on what gets appended.
*	3. Very Slow. About O(3n).
*
* HISTORY:
*	 6-NOV-91 MNP	Created
*
*****************************************************************************/

static void CatSprintf(buf,bufsize,format VA_ALIST)
char *buf;			/* in/out: what to append to */
int  bufsize;			/* in: size shouldn't be exceeded */
char *format;			/* in: printf format string */
VA_DCL				/* in: printf args */
{
    va_list args;
    int buf_end, new_len;
    FWTempBuf1024 new;
/**/
    VA_START(args,format);
    vsprintf(new, format, args);
    
    /* Safety: sprintf might have already nuked something, but this 
       might avoid more damage later... */
    new[sizeof(new) - 1] = '\0';	

    buf_end = strlen(buf);
    
    strncat(&buf[buf_end], new, bufsize - buf_end - 1);
    va_end(args);

}  /* CatSprintf */


/* *******************************************************************
*
* IsPreviewDone - check if preview is done
*
* HISTORY:
*	16-MAY-91 AWH	Created.
*	13-SEP-91 MNP	Copied from VLK kernel.
*
******************************************************************* */
static VK_tBoolean IsPreviewDone()
{
    if (FWT_NO_PREVIEW == FWT_PreviewLineCount) {
	return False;
    }
    else {
	return FWT_LineCount > FWT_PreviewLineCount;
    }
    
} /* IsPreviewDone */


/* *******************************************************************
*
* ProcessPreviewDone
*
* HISTORY:
*	16-MAY-91 AWH	Created
*	13-SEP-91 MNP	Copied from VLK kernel.
*
******************************************************************* */

static void ProcessPreviewDone(record)
RD_tRecord * record;
{
/**/
    Simulating = False;
    DisplayPreviewMessage();
    
} /* ProcessPreviewDone */


/*****************************************************************************
*
* FindGroupById - search group list for group having ID
*
* DESCRIPTION:
*	The ID is an integer which was written to the trace file, so elements
*	could point to their groups. Given a group ID, find the group, or
*	return NULL handle.
*
* HISTORY:
*	 5-NOV-91 MNP	Created
*
*****************************************************************************/

FWT_tSignalGroupHandle FindSignalGroupById(id)
int id;
{
    TempItem scan;
/**/

    scan = TempFirstItem(FWT_SignalGroupList);
    while (! TempNullItem(scan) && qqGroupID(FWT_SigGrpValue(scan)) != id)
	scan = TempNextItem(scan);
    
    return  TempNullItem(scan) ? NULL : FWT_SigGrpValue(scan);

} /* FindSignalGroupById */


/* *******************************************************************
*
* FindSignalByNumber - 
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*
******************************************************************* */

FWT_tSignalElementHandle FindSignalByNumber(signalnumber)
int signalnumber;
{
    TempItem scan;
    FWT_tSignalElementHandle element = NULL;
/**/
    scan = TempFirstItem(FWT_SignalElementList);
    while (! TempNullItem(scan)) {
	element = FWT_SigElemValue(scan);
	if (signalnumber == qqNumber(element)) {
	    scan = TempNoItem();
	}
	else {
	    scan = TempNextItem(scan);
	}
    }
    return element;
} /* FindSignalByNumber */


/* *******************************************************************
*
* ProcessTraceMarker - 
*
* HISTORY:
*	24-APR-90 DSB	Created
*
******************************************************************* */

static void ProcessTraceMarker(record)
RD_tRecord * record;
{
/**/
   Simulating = True;
} /* ProcessTraceMarker */


/* *******************************************************************
*
* ProcessTraceSimCycle - 
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*
******************************************************************* */

static void ProcessTraceSimCycle(record)
RD_tRecord * record;
{
static VK_tBoolean skip = False; /* skipped writing time slice */
/**/
    if (FWT_DoTimeSlice) {
	if (skip) {
	    DisplayTraceLinesSkipped();
	}
	skip = False;
	FWT_DoTimeSlice = False;
        WriteTimeSlice();
	FWT_LineCount++;
    }
    else {
	skip = True;
    }

    FWT_SimCycleCount++;
    FWT_DeltaCount++;
} /* ProcessTraceSimCycle */


/* *******************************************************************
*
* ProcessTraceTimeAdvance - 
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*
******************************************************************* */

static void ProcessTraceTimeAdvance(record)
RD_tRecord * record;
{
/**/
    TIME_COPY(FWT_SimTime, record->time_advance.time);
    FWT_DeltaCount = 0;
} /* ProcessTraceTimeAdvance */



/* *******************************************************************
*
* ProcessTraceTypeDecl - 
*
* HISTORY:
*	24-APR-90 DSB	Created from LoadMap
*	28-DEC-90 AWH	Changed to use TKRecord
*	02-MAY-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*	03-JUN-91 JBR	W0019.  Disabled routine.
*
******************************************************************* */
#if 0    

static void ProcessTraceTypeDecl(record)
RD_tRecord * record;
{
    tTypeHandle  type;
/**/
    AllocateMemory(type, tTypeRec, 1);

    AddqqName(type, CopyPrivStr(record->type_declaration.name));
    AddqqNumber(type, record->type_declaration.number);

    TempAddLast(FWT_TypeList, type);
} /* ProcessTraceTypeDecl */

#endif


/* *******************************************************************
*
* ProcessTraceSignalDecl - 
*
* HISTORY:
*	24-APR-90 DSB	Created from LoadMap
*	28-DEC-90 AWH	Changed to use TKRecord
*	01-APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*	24-OCT-91 MNP	added simplename bookkeeping
*	03-NOV-91 MNP	split into group and element funs, this is obsolete
*
******************************************************************* */

static void ProcessTraceSignalDecl(record)
RD_tRecord * record;
{
#if 000
    FWT_tSignalElementHandle element;
    FWT_tSignalGroupHandle group;
    int index, width;
    VK_tCString val_ptr;
    char tempbuf[40];
/**/

    /* group record */
    AllocateMemory(group, FWT_tSignalGroupRec, 1);
    AddqqRecordKind(group, FWT_rt_SignalGroup);

    AddqqName(group, CopyPrivStr(record->signal_declaration.name));
    AddqqSimpleName(group, CopyPrivStr(record->signal_declaration.simplename));
    AddqqSignalStart(group, record->signal_declaration.start);
    AddqqSignalLength(group, record->signal_declaration.length);
    AddqqGroupClass(group, record->signal_declaration.group);
    AddqqElementClass(group, record->signal_declaration.class);

    /* Calculate print width. For vectors, We want the larger of the
       value and (name, index, parens). For scalars, just the name. */

    if (NamesMode) {
	width = strlen(qqSimpleName(group));
	if (gc_VectorSignal == qqGroupClass(group)) {
    	    sprintf(tempbuf, "%d", qqSignalLength(group));
	  	width += strlen(tempbuf) + 2;
	} 
	AddqqWidth(group, MAX(width, record->signal_declaration.width));
    }
    else {
	AddqqWidth(group, record->signal_declaration.width);
    }

    TempAddLast(FWT_SignalGroupList, group);

     /* for each element */
     for (index = 0; index <= record->signal_declaration.length-1; index++ ) {
 	AllocateMemory(element, FWT_tSignalElementRec, 1);
	AddqqRecordKind(element, FWT_rt_SignalElement);
	
    	AddqqNumber(element, record->signal_declaration.start + index);
	AddqqGroup(element, group);
    
	AllocateMemory(val_ptr, char, FWT_VALUE_BUFFER_SIZE);
    	AddqqEffectiveValue(element, val_ptr);

	AllocateMemory(val_ptr, char, FWT_VALUE_BUFFER_SIZE);
    	AddqqPreviousValue(element, val_ptr);

    	AddqqLastActiveCycle(element, -1);	/* ?? */
	AddqqDisplay(element, False);

    	TempAddLast(FWT_SignalElementList, element);
     } /* for */

#endif /* 000 */
} /* ProcessTraceSignalDecl */


/* *******************************************************************
*
* ProcessTraceSignalGroupDecl - 
*
* HISTORY:
*	24-APR-90 DSB	Created from LoadMap
*	28-DEC-90 AWH	Changed to use TKRecord
*	01-APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*	24-OCT-91 MNP	added simplename bookkeeping
*	 1-NOV-91 MNP	Split off from ProcessTraceSignalDecl
*
******************************************************************* */

void ProcessTraceSignalGroupDecl(record)	/* used in fwmisc */
RD_tRecord * record;
{
    FWT_tSignalGroupHandle group;
/**/

    AllocateMemory(group, FWT_tSignalGroupRec, 1);
    AddqqRecordKind(group, FWT_rt_SignalGroup);

    AddqqName(group, CopyPrivStr(record->signal_group_declaration.name));
    AddqqSimpleName(group,
		    CopyPrivStr(record->signal_group_declaration.simple_name));
    AddqqSignalStart(group, record->signal_group_declaration.start); 
    AddqqGroupID(group, record->signal_group_declaration.group_id); 
    AddqqSignalLength(group, 0);
    AddqqGroupClass(group, record->signal_group_declaration.group);

    TempAddLast(FWT_SignalGroupList, group);

} /* ProcessTraceSignalGroupDecl */


/* *******************************************************************
*
* ProcessTraceSignalElementDecl - 
*
* HISTORY:
*	24-APR-90 DSB	Created from LoadMap
*	28-DEC-90 AWH	Changed to use TKRecord
*	01-APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*	24-OCT-91 MNP	added simplename bookkeeping
*	 1-NOV-91 MNP	Split off from ProcessTraceSignalDecl
*
******************************************************************* */

void ProcessTraceSignalElementDecl(record)  /* used in fwmisc, and here */
RD_tRecord * record;
{
    FWT_tSignalElementHandle element;
    FWT_tSignalGroupHandle group;
    int name_width, field_width;
    VK_tCString val_ptr;
    char tempbuf[40];
/**/
    /* Locate the group record of this element, using its ID. */

    group = FindSignalGroupById(record->signal_element_declaration.group_id);
     
    if(NULL == group) {  /* err if grp not found */
	fprintf(stderr,"Warning, group id not found in PTSED().");
	return;
    }
    
    /* Create a Signal Element Record */

    AllocateMemory(element, FWT_tSignalElementRec, 1);
    AddqqRecordKind(element, FWT_rt_SignalElement);
    AddqqNumber(element, record->signal_element_declaration.start);
    AddqqGroup(element, group);
    SetqqSignalLength(group, qqSignalLength(group) +1);
    AddqqSubelName(element,
		   CopyPrivStr(record->signal_element_declaration.subel_name));

    AllocateMemory(val_ptr, char, FWT_VALUE_BUFFER_SIZE);
    AddqqEffectiveValue(element, val_ptr);
    AllocateMemory(val_ptr, char, FWT_VALUE_BUFFER_SIZE);
    AddqqPreviousValue(element, val_ptr);

    AddqqLastActiveCycle(element, -1);	/* ?? */
    AddqqDisplay(element, False);

    AddqqValueWidth(element, record->signal_element_declaration.width);

    /* FieldWidth assignment moved to FWT_CalculateWidth. */

    TempAddLast(FWT_SignalElementList, element);
} /* ProcessTraceSignalElementDecl */


/* *******************************************************************
*
* ProcessTraceSignalInitial - 
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*	28-DEC-90 AWH	Changed to use TKRecord
*
******************************************************************* */

static void ProcessTraceSignalInitial(record)
RD_tRecord * record;
{
    FWT_tSignalElementHandle element;
/**/
    element = FindSignalByNumber(record->signal_initialization.signal);
    if (element != NULL) {
    	strcpy((VK_tCString) qqEffectiveValue(element), record->signal_initialization.value);
    	strcpy((VK_tCString) qqPreviousValue(element), record->signal_initialization.value);
    }
} /* ProcessTraceSignalInitial */



/* *******************************************************************
*
* ProcessTraceSignalTransaction - 
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*	28-DEC-90 AWH	Changed to use TKRecord
*
******************************************************************* */

static void ProcessTraceSignalTransaction(record)
RD_tRecord * record;
{
     FWT_tSignalElementHandle element;
/**/

    element = FindSignalByNumber(record->signal_transaction.signal);
    if (element != NULL) {
    	SetqqLastActiveCycle(element, FWT_SimCycleCount);
	if (qqDisplay(element)) {
            FWT_DoTimeSlice = True;
	}
    }
} /* ProcessTraceSignalTransaction */



/* *******************************************************************
*
* ProcessTraceSignalEvent - 
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*	28-DEC-90 AWH	Changed to use TKRecord
*
******************************************************************* */

static void ProcessTraceSignalEvent(record)
RD_tRecord * record;
{
    FWT_tSignalElementHandle element;
/**/
    element = FindSignalByNumber(record->signal_event.signal);
    if (element != NULL) {
    	strcpy((VK_tCString) qqEffectiveValue(element), record->signal_event.value);
	if (qqDisplay(element)) {
	    FWT_DoTimeSlice = True;
	}
    }
} /* ProcessTraceSignalEvent */



/* *******************************************************************
*
* ProcessTraceAssertionViolation - 
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*	11-MAY-90 DSB	Added process pathname
*	28-DEC-90 AWH	Changed to use TKRecord
*
******************************************************************* */

static void ProcessTraceAssertionViolation(record)
RD_tRecord * record;
{
    VK_tCString sevstr;
/**/

    switch (record->assertion.severity) {
	case VK_Note:		sevstr = "Note";	break;
	case VK_Warning:	sevstr = "Warning";	break;
	case VK_Error:		sevstr = "Error";	break;
	case VK_Failure:	sevstr = "Failure";	break;
	default:		sevstr = "<bad value>";	break;
    } /* switch */

    DisplayAssertion(
	record->assertion.process_name,
	record->assertion.source_file,
	record->assertion.line_number,
	record->assertion.message,
	sevstr
	);

    FWT_DoTimeSlice = True;
} /* ProcessTraceAssertionViolation */


/* *******************************************************************
*
* ProcessTraceError - 
*
* HISTORY:
*	11-APR-90 DSB	Created.
*	28-DEC-90 AWH	Changed to use TKRecord
*
******************************************************************* */

static void ProcessTraceError(record)
RD_tRecord * record;
{
/**/
    DisplayError(
	record->error.message,
	record->error.extra,
	record->error.source_file,
	record->error.line_number
	);

    FWT_DoTimeSlice = True;
} /* ProcessTraceError */

/* *******************************************************************
*
* ProcessTerminated - 
*
* HISTORY:
*	02-JAN-91 AWH	Created
*
******************************************************************* */

static void ProcessTerminated(record)
RD_tRecord * record;
{
/**/
    Simulating = False;
} /* ProcessTerminated */



/* *******************************************************************
*
* FWT_Initialize - 
*
* HISTORY:
*	31-DEC-90 AWH	Created to use TKRecord
*
******************************************************************* */

void FWT_Initialize()
{
/**/

    FWT_SignalElementList = TempNewList();
    FWT_SignalGroupList = TempNewList();
    FWT_TypeList = TempNewList();
 
    TIME_ZERO(FWT_SimTime);
    FWT_DeltaCount = 0;                   /* */
    FWT_SimCycleCount = 0;
    FWT_DoTimeSlice = True;

    FWT_LineCount = 0;

} /* FWT_Initialize */


/* *******************************************************************
*
* FWT_CalculateWidth - Calculate Width of wave file
*
* HISTORY:
*	16-MAY-91 AWH 	Created
*	19-SEP-91 MNP	Copied from kernel vlk.
*	12-NOV-91 MNP	Added name and column args.
*
******************************************************************* */

void FWT_CalculateWidth(names_mode, skip_columns, total_signals,
	displayed_signals, wave_width)
VK_tBoolean names_mode;		/* in: True if names to be printed */
Integer4   skip_columns;	/* in: how many cols between signals */
Integer4 * total_signals;	/* out */
Integer4 * displayed_signals;	/* out */
Integer4 * wave_width;		/* out */
{
    TempItem sigscan;
    FWT_tSignalElementHandle element;
    Integer4 count;
    Integer4 width;
    Integer4 total;
/**/
    count = 0;
    width = 0;
    total = 0;
    
    for (sigscan=TempFirstItem(FWT_SignalElementList);
		! TempNullItem(sigscan); sigscan = TempNextItem(sigscan)) {
	element = FWT_ElementValue(sigscan);

	total++;
	
	/* Calculate print width: the larger of the value and name.
	   Better to do this as it's read in, but the environment is
	   here and not there.. */

	if (names_mode) {
	    AddqqFieldWidth(element, MAX(qqValueWidth(element)+1, 
			strlen(qqSimpleName(qqGroup(element))) +
			strlen(qqSubelName(element))));
	}
	else {
	    AddqqFieldWidth(element, qqValueWidth(element));
	}

	if (FWT_CheckSignalDisplay(element)) {
      	    count++;
	    width += (1 + skip_columns + qqFieldWidth(element));
       	}
    
    } /* for */

    *total_signals = total;
    *displayed_signals = count;
    *wave_width = width;
    
} /* FWT_CalculateWidth */


/* *******************************************************************
*
* FWT_CheckSignalDisplay - Check if signal is set to be desplayed
*
* HISTORY:
*	03-JAN-91 AWH 	Created
*	07-JAN-91 DSB	Fixed bug: added qqRecordKind on switch expression
*
******************************************************************* */

VK_tBoolean FWT_CheckSignalDisplay(handle) 	/* True if element set to display */
    void * handle; 
{
    FWT_tSignalElementHandle element;
    FWT_tSignalGroupHandle group;
    Integer4 index;
    VK_tBoolean result;
/**/
   result = False;
   switch (qqRecordKind((FWT_tSignalElementHandle) handle)) {
	case FWT_rt_SignalGroup:
	    group = (FWT_tSignalGroupHandle) handle;
	    for (index = qqSignalStart(group) ; 
		 index <= qqSignalStart(group) + qqSignalLength(group) - 1;
		 index ++ ){
		element = FindSignalByNumber(index);
		if (NULL != element && qqDisplay(element)) {
		    result = True;
		    break; /* for */
		}
	    } /* for */
	    break;
	case FWT_rt_SignalElement:
	    element = (FWT_tSignalElementHandle) handle;
	    result = qqDisplay(element);
	    break;
	default:
	    break;
	    
   } /* switch */
   return result;

} /* FWT_CheckSignalDisplay */ 



/* *******************************************************************
*
* FWT_SetAllSignalDisplay - Set all signals to display
*
* HISTORY:
*	31-DEC-90 AWH 	Created
*
******************************************************************* */

void FWT_SetAllSignalDisplay()
{
    FWT_tSignalElementHandle element;
    TempItem scan;
/**/
    for (scan=TempFirstItem(FWT_SignalElementList);
		! TempNullItem(scan); scan = TempNextItem(scan)) {
     	element = FWT_SigElemValue(scan);
        SetqqDisplay(element, True); /* display all elements */
     } /* for */
} /* FWT_SetAllSignalDisplay */


/* *******************************************************************
*
* FWT_SetSignalDisplay - Set signal to display
*
* HISTORY:
*	31-DEC-90 AWH 	Created
*
******************************************************************* */

void FWT_SetSignalDisplay(signal)
Integer4 signal;
{
    FWT_tSignalElementHandle element;

/**/
    element = FindSignalByNumber(signal);
    if (NULL != element) {
	SetqqDisplay(element, True);
    }
} /* FWT_SetSignalDisplay */


/* *******************************************************************
*
* FWT_SignalMap - Process Trace File until end of signal map
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*	24-APR-90 DSB	Converted to combined map/trace file
*	28-DEC-90 AWH	Changed to use TKRecord
*	31-DEC-90 AWH 	Changed to use Display call backs
*	03-JUN-91 JBR	W0019. Remove case for RD_TypeDeclaration.
*	26-SEP-91 MNP	Handle trace version error ungracefully: quit.
*
******************************************************************* */

void FWT_SignalMap()
{
    RD_tRecord record;
    VK_tBoolean done;
/**/
   done = False;

    while ((!done) && RD_ReadNextRecord(&record)) {
	switch (record.type) {
	    case RD_SignalDeclaration:			/* obsolete */
		ProcessTraceSignalDecl(&record);
		break;
	    case RD_SignalGroupDeclaration:
		ProcessTraceSignalGroupDecl(&record);
		break;
	    case RD_SignalElementDeclaration:
		ProcessTraceSignalElementDecl(&record);
		break;
	    case RD_EndOfSignalMap:
		ProcessTraceMarker();    
		done = True;
		break;
	    case RD_Assertion:
		ProcessTraceAssertionViolation(&record);
		break;
	    case RD_Error:
		ProcessTraceError(&record);
		break;
	    case RD_UnknownType:
		fprintf(stderr, "%%FMT-F-BadTRec, Bad Trace Record:\n %s\n\n", 
			record.unknown_type.text);
		exit(1);
		break;
	      case RD_VersionError:
		fprintf(stderr,
 "%%FMT-F-BadVer, Bad trace file version %d.%d (%d.%d expected).\n", 
			record.version_error.first, 
			record.version_error.second,
			RD_VERSION_MAJOR, RD_VERSION_MINOR);
		exit(1);
		break;
	    default:
		break;
        } /*switch*/
	RD_FreeRecord(&record);
    } /* while */
} /* FWT_SignalMap */


/* *******************************************************************
*
* FWT_SignalActivity - Process signal activity in trace file
*
* HISTORY:
*	__-SEP-89 DSB	Created.
*	24-APR-90 DSB	Converted to combined map/trace file
*	28-DEC-90 AWH	Changed to use TKRecord
*	31-DEC-90 AWH 	Changed to use Display/Write call backs
*	13-SEP-91 MNP	Added previewing, as in VLK kernel.
*
******************************************************************* */

VK_tBoolean FWT_SignalActivity()  	/* return false when simulation 
					   completed */
{
    RD_tRecord record;
    VK_tBoolean preview_done = False;
    static RD_tRecordKinds prev_type = RD_UnknownType;
/**/

    while (RD_ReadNextRecord(&record) && !preview_done) {
	switch (record.type) {
	    case RD_SignalInitialization:
		ProcessTraceSignalInitial(&record);
		break;
	    case RD_SignalTransaction:
	        ProcessTraceSignalTransaction(&record);
		break;
	    case RD_SignalEvent:
	        ProcessTraceSignalEvent(&record);
		break;
	    case RD_Assertion:
	        ProcessTraceAssertionViolation(&record);
		break;
	    case RD_Error:
	        ProcessTraceError(&record);
		break;
	    case RD_TimeAdvance:
	        ProcessTraceTimeAdvance(&record);
		break;
	    case RD_SimulationCycle:

		/* NOTE - This is messed up!! It's a temporary fix 
		   to counteract an extra SC (immediately following a
		   TA) in the trace file. This prevents dots. MNP */

		if (prev_type == RD_TimeAdvance)
		{
		    FWT_DoTimeSlice = True;
		}
		ProcessTraceSimCycle(&record);
		break;
	    case RD_Terminated:
	        ProcessTerminated();
 	        break;
	    case RD_UnknownType:
	        fprintf(stderr, "Bad Trace Record:\n %s\n\n", 
	     	    record.unknown_type.text);
		break;
	    default:
		break;
        } /* switch */

	prev_type = record.type;
	RD_FreeRecord(&record);

	preview_done = IsPreviewDone();

    } /* while */

    /* Copied from vlk kernel - Should this happen before or after
       the if stmnt below? MNP */

    if (preview_done) {
	ProcessPreviewDone();
    }

/* ?? Redesign this for VLK: added to get last time slice even if 
	assertion or error terminates simulation: ?? */
    if (FWT_DoTimeSlice) {
	ProcessTraceSimCycle(NULL);
    }
				
    return Simulating;
} /* FWT_SignalActivity */


/* *******************************************************************
*
* FWT_SignalActivityOnce - Same as above, but returns after only 1 slice.
*
* HISTORY:
*	21-AUG-89 MNP	Created.
*
******************************************************************* */

VK_tBoolean FWT_SignalActivityOnce()  	/* return false when simulation 
					   completed */
{
    RD_tRecord record;
    VK_tBoolean preview_done = False;
    static RD_tRecordKinds prev_type = RD_UnknownType;
/**/
    if (RD_ReadNextRecord(&record) && !preview_done) {
	switch (record.type) {
	    case RD_SignalInitialization:
		ProcessTraceSignalInitial(&record);
		break;
	    case RD_SignalTransaction:
	        ProcessTraceSignalTransaction(&record);
		break;
	    case RD_SignalEvent:
	        ProcessTraceSignalEvent(&record);
		break;
	    case RD_Assertion:
	        ProcessTraceAssertionViolation(&record);
		break;
	    case RD_Error:
	        ProcessTraceError(&record);
		break;
	    case RD_TimeAdvance:
	        ProcessTraceTimeAdvance(&record);
		break;
	    case RD_SimulationCycle:

		/* NOTE - This is messed up!! It's a temporary fix 
		   to counteract an extra SC (immediately following a
		   TA) in the trace file. This prevents dots. MNP */

		if (prev_type == RD_TimeAdvance)
		{
		    FWT_DoTimeSlice = True;
		}
		ProcessTraceSimCycle(&record);
		break;
	    case RD_Terminated:
	        ProcessTerminated();
 	        break;
	    case RD_UnknownType:
	        fprintf(stderr, "Bad Trace Record:\n %s\n\n", 
	     	    record.unknown_type.text);
		break;
	    default:
		break;
        } /* switch */

	prev_type = record.type;
	RD_FreeRecord(&record);

	preview_done = IsPreviewDone();

    } /* if */

    if (preview_done) {
	ProcessPreviewDone();
    }
    
    return Simulating;
} /* FWT_SignalActivityOnce */



/*****************************************************************************
*
* FWT_FormatWaveHeader - Fill a buffer with several signal header lines.
*
* DESCRIPTION:
*	                           1   2   3    4   5 		; names
*	T(fs) + Delta              x   y cin cout sum 
*	------------------- --   --- --- --- ---- --- 
*
*
*	              
*	T(fs) + Delta              1   2   3    4   5		 ; !names
*	------------------- --   --- --- --- ---- --- 
*
* HISTORY:
*	 6-NOV-91 MNP	Created
*
*****************************************************************************/

void FWT_FormatWaveHeader(buf, bufsize, names, skip_columns)
char *buf;
Integer4 bufsize;
VK_tBoolean names;
Integer4 skip_columns;
{
    TempItem scan1;
    FWT_tSignalElementHandle element;
    FWT_tSignalGroupHandle group;
    FWTempBuf80 tempbuf, tempbuf2;
    int i, n1;
/**/
    /* T(fs) + Delta  */

    buf[0] = '\0';
    
    if (names) {
	CatSprintf(buf, bufsize, Fmt_TimeFill);
	CatSprintf(buf, bufsize, Fmt_TimeSepFill);
    }
    else {
	CatSprintf(buf, bufsize, Fmt_TimeLbl, BaseUnit);
	CatSprintf(buf, bufsize, Fmt_TimeSepLbl);
    }	

    /* 1   2   3   4 ...  */

    n1 = 0;
    for (scan1=TempFirstItem(FWT_SignalElementList);
		! TempNullItem(scan1); scan1 = TempNextItem(scan1)) {
	n1++;
	element = FWT_SigElemValue(scan1);
	
	if (! FWT_CheckSignalDisplay(element)) {
		/* do not display this element */
		continue; /* for */
	}

	CatSprintf(buf, bufsize, Fmt_TransLbl);
	CatSprintf(buf, bufsize, Fmt_SigValLbl,
		       qqFieldWidth(element), n1); 
	CatSprintf(buf, bufsize, Fmt_ColSepLbl);
    } /* for */
    CatSprintf(buf, bufsize, "\n");

    /* Print simple names (adds one row, if the option is enbabled) */

    if (names) {
	CatSprintf(buf, bufsize, Fmt_TimeLbl, BaseUnit);
	CatSprintf(buf, bufsize, Fmt_TimeSepLbl);
	
	for (scan1=TempFirstItem(FWT_SignalElementList);
		! TempNullItem(scan1); 
	     	scan1 = TempNextItem(scan1)) {

	    element = FWT_SigElemValue(scan1);
	    if (FWT_CheckSignalDisplay(element)) {

		CatSprintf(buf, bufsize, Fmt_TransLbl);

		switch(qqGroupClass(qqGroup(element))) {
		  case gc_VectorSignal:
		  case gc_CompositeSignal:
		  case gc_ScalarSignal:
		    strncpy(tempbuf2, qqSimpleName(qqGroup(element)),
			sizeof(tempbuf2));
		    strncat(tempbuf2, qqSubelName(element),
			sizeof(tempbuf2) - strlen(tempbuf2));
		    CatSprintf(buf, bufsize, Fmt_SigValIR,
			qqFieldWidth(element), tempbuf2);
		    break;

		  default:
		    fprintf(stderr, "Warning, bad switch in WriteHeader()");

		} /* switch */

		CatSprintf(buf, bufsize, Fmt_ColSepLbl);

	    } /* if display */
	} /* for scan */
	CatSprintf(buf, bufsize, "\n");

    } /* if names */


    /* print "---------- --   --- --- --- --- ..." */

    CatSprintf(buf, bufsize, "%s%s", Fmt_TimeLine, Fmt_TimeSepLine);

    for (scan1=TempFirstItem(FWT_SignalElementList);
		! TempNullItem(scan1); scan1 = TempNextItem(scan1)) {

	element = FWT_SigElemValue(scan1);
	
	if (! FWT_CheckSignalDisplay(element)) {
		/* do not display this element */
		continue; /* for */
	}

	CatSprintf(buf, bufsize, Fmt_TransLine);

	for (i=1; i<=qqFieldWidth(element); i++) {
					/* Special for transaction mark */
	    tempbuf[i-1] = '-';
	}
	tempbuf[i-1] = '\0';
	CatSprintf(buf, bufsize, tempbuf);
	CatSprintf(buf, bufsize, Fmt_ColSepLine, n1);
    }
    CatSprintf(buf, bufsize, "\n");

} /* FWT_FormatWaveHeader */


/*****************************************************************************
*
* FWT_FormatTimeSlice - Fill a buffer with a time slice.
*
* DESCRIPTION:
*
* HISTORY:
*	 7-NOV-91 MNP	Created
*
*****************************************************************************/

void FWT_FormatTimeSlice(buf, bufsize, skip_columns)
char *buf;			/* inout: Where to stick it*/
Integer4 bufsize;		/* in: max buffer size, in chars */
Integer4 skip_columns;		/* in: how many to skip (1 or 0) */
{
    TempItem sigscan;
    FWT_tSignalElementHandle element;
    VK_tCString previous, current;
    VK_tBoolean transaction;
    FWTempBuf80 time_buffer;    
    int width;
/**/
    buf[0] = '\0';
    
    /* write time data */
    UTIME_FORMAT(time_buffer, FWT_SimTime);
    CatSprintf(buf, bufsize, Fmt_Time, time_buffer, FWT_DeltaCount);
    CatSprintf(buf, bufsize, Fmt_TimeSep);

    for (sigscan=TempFirstItem(FWT_SignalElementList);
		! TempNullItem(sigscan); sigscan = TempNextItem(sigscan)) {
	element = FWT_SigElemValue(sigscan);
	
	if (! FWT_CheckSignalDisplay(element)) {
		continue; /* for */
	}

	transaction = qqLastActiveCycle(element) == FWT_SimCycleCount;

	previous = (VK_tCString) qqPreviousValue(element);
	current = (VK_tCString) qqEffectiveValue(element);

	CatSprintf(buf, bufsize, transaction ? Fmt_TransMark :
		       Fmt_NoTransMark); 

	width = qqFieldWidth(element) - skip_columns;
	
	if (tc_Enumeration == qqElementClass(element)) {
	    CatSprintf(buf, bufsize, Fmt_SigValE, width, current);
	}
	else {
	    CatSprintf(buf, bufsize, Fmt_SigValIR, width, current);
	}

	CatSprintf(buf, bufsize, Fmt_ColSep);
	strcpy(	(VK_tCString) qqPreviousValue(element), 
		(VK_tCString) qqEffectiveValue(element));
    }
    
} /* FWT_FormatTimeSlice */

