#ifndef FWTrace_H
#define FWTrace_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: FWTrace.h
*
* DESCRIPTION:  
*	Private interface file for FWTrace.h  <then describe contents>
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	31-DEC-90 AWH	Created from portions of FmtWave.c
*	04-APR-91 WAG	W0023.  Removed unneeded #include of VKI.h;
*			Modified to conform to coding style standards.
*	02-MAY-91 DSB	Cleaning up generally
*	06-MAY-91 DSB	Cleaning up signal changes
*	13-SEP-91 MNP	Added preview functionality, from VLK. 
*
* SCCS ID: @(#)FWTrace.h	42.2 1/13/92
*
******************************************************************* */

#include "FWCommon.h"
#include "TKStructs.h"		/* for tGroupClass */


/* *******************************************************************
*
* Data Structure Declarations
*
******************************************************************* */
typedef enum {
	FWT_rt_BadRecordKind,
	FWT_rt_SignalGroup,
	FWT_rt_SignalElement
    } FWT_tRecordKind;

typedef struct FWT_sSignalGroup * FWT_tSignalGroupHandle;
typedef struct FWT_sSignalGroup {
	FWT_tRecordKind fRecordKind;	/* FWT_rt_SignalGroup */
	VK_tCString fName;		/* Complete name of signal */
	VK_tCString fSimpleName;	/* Simple name of signal */
	tGroupClass fGroupClass;	/* Vector or Group */
	Integer4 fSignalStart;		/* first signal in group */
	Integer4 fSignalLength;		/* length of signal group */
	Integer4 fGroupID;		/* group id, read from trace file */
    } FWT_tSignalGroupRec;

/* ?? HEY: FWTrace should not be using TKStruct.h's qqXx macros to
	access these fields.  It should define its own, or use
	fields directly. (Renaming fields in TKStruct.h and its macros
	can cause FWTrace.c to break.) ?? */

typedef struct FWT_sSignalElement * FWT_tSignalElementHandle;
typedef struct FWT_sSignalElement {
	FWT_tRecordKind fRecordKind;	/* FWT_rt_SignalElement */
	Integer4 	fNumber;	/* scalar signal number */
	FWT_tSignalGroupHandle fGroup;	/* group record */
	VK_tCString 	fEffectiveValue;/* effective value */
	VK_tCString 	fPreviousValue;	/* previous value */
	tSimCycleCount fLastActiveCycle;/* cycle of last activity */
	VK_tBoolean 	display;	/* display signal in wave file */
	Integer4	fValueWidth; 	/* max display width of elem. value */
	Integer4 	fFieldWidth;	/* width of larger of name or value */
	tTypeClass  	fElementClass;	/* Class of the element */	
	VK_tCString 	fSubelName;	/* Subelement name: append to
					   simple name from group */
    } FWT_tSignalElementRec;

/* Access macros for the above structures of questionable lineage. */

#define qqSimpleName(N)	((N)->fSimpleName)
#define AddqqSimpleName(N, V) ((N)->fSimpleName = V)
#define SetqqSimpleName(N, V) ((N)->fSimpleName = V)

#define qqGroupID(N)	((N)->fGroupID)
#define AddqqGroupID(N, V) ((N)->fGroupID = V)
#define SetqqGroupID(N, V) ((N)->fSimpleName = V)

#define qqSubelName(N)	((N)->fSubelName)
#define AddqqSubelName(N, V) ((N)->fSubelName = V)
#define SetqqSubelName(N, V) ((N)->fSubelName = V)

#define qqEffectiveValue(N)   ((N)->fEffectiveValue)
#define AddqqEffectiveValue(N, V) ((N)->fEffectiveValue = V)
#define SetqqEffectiveValue(N, V) ((N)->fEffectiveValue = V)

#define qqPreviousValue(N)    ((N)->fPreviousValue)
#define AddqqPreviousValue(N, V) ((N)->fPreviousValue = V)
#define SetqqPreviousValue(N, V) ((N)->fPreviousValue = V)

#define qqFieldWidth(N)	((N)->fFieldWidth)
#define AddqqFieldWidth(N, V) ((N)->fFieldWidth = V)
#define SetqqFieldWidth(N, V) ((N)->fFieldWidth = V)

#define qqValueWidth(N)	((N)->fValueWidth)
#define AddqqValueWidth(N, V) ((N)->fValueWidth = V)
#define SetqqValueWidth(N, V) ((N)->fSubelName = V)

#define FWT_SigGrpValue(item) ((FWT_tSignalGroupHandle)\
		(TempValue(item)))

#define FWT_SigElemValue(item) ((FWT_tSignalElementHandle)\
		(TempValue(item)))

#define FWT_VALUE_BUFFER_SIZE 80

#define FWT_NO_PREVIEW -1

/* *******************************************************************
*
* Format Strings
*
* HISTORY:
*	10-JUN-91 DSB	(Tentatively) widened time field for 64-bit time
*	 4-NOV-91 MNP	Moved from FWMisc.c
*
******************************************************************* */

	/* time */

#if 1

#  define Fmt_TimeLbl	"      Time(%s) + Cycle"  /* label line */
#  define Fmt_TimeLine	"----------------------"  /* line under label line */
#  define Fmt_Time	"%19s+%2d"		  /* data line */
#  define Fmt_TimeFill 	"                      "  /* continuation line */

#else

#  define Fmt_TimeLbl	"T(%s) + Delta"		/* label line */
#  define Fmt_TimeLine	"---------- --"		/* line under label line */
#  define Fmt_Time	"%10s+%2d"		/* data line */
#  define Fmt_TimeFill 	"             "		/* continuation line */

#endif

	/* separation after time */
#define Fmt_TimeSepLbl	"  "
#define Fmt_TimeSepLine	"  "
#define Fmt_TimeSep	": "
#define Fmt_TimeSepFill	"  "

/* skip_columns is a local var in some FWT_Format* fns, same name in each */

	/* transaction marks */
#define Fmt_TransLbl	(skip_columns ? "" : " ")
#define Fmt_TransLine	(skip_columns ? "" : " ")	
#define Fmt_TransMark	"*"
#define Fmt_NoTransMark	" "

	/* separation between signal columns */
#define Fmt_ColSepLbl	(skip_columns ? " " : "")
#define Fmt_ColSepLine	(skip_columns ? " " : "")
#define Fmt_ColSep	(skip_columns ? " " : "")

	/* signal values */
#define Fmt_SigValLbl	"%*d"

#define Fmt_SigValE	"%-*s"	/* if enumeration: left justified */
#define Fmt_SigValIL	"%-*s"	/* if integer: left justified case (special) */
#define Fmt_SigValIR	"%*s"	/* if integer: normally right justified */

	/* seperation between lines */
#define Fmt_SkipTraceLine "...\n"

 	/* padding source buffer */
#define Fmt_PadChars	"                                                  "


/* *******************************************************************
*
* Global Variables
*
******************************************************************* */
extern tAbsTimeBuf FWT_SimTime;
extern tSimCycleCount FWT_SimCycleCount;
extern Integer4 FWT_DeltaCount;
extern VK_tBoolean FWT_DoTimeSlice;/* Write TimeSlice this cycle */

extern TempList	FWT_SignalElementList;
extern TempList	FWT_SignalGroupList;
extern TempList	FWT_TypeList;

extern Integer4	FWT_PreviewLineCount;
extern Integer4	FWT_LineCount;

/* NOTE:
	Only write a time slice line ( the line of signal values) when
	there has been interesting activity.
	Interesting activity: events and transactions on signals marked
	to be displayed, assertions, errors, and for the first cycle.
*/
				

FWT_tSignalElementHandle FindSignalByNumber(
#ifdef VK_ANSI_FN_PROTOS
	int signalnumber
#endif
	);

FWT_tSignalGroupHandle FindSignalGroupById(
#ifdef VK_ANSI_FN_PROTOS
	int id
#endif
	);

VK_tBoolean FWT_CheckSignalDisplay(
#ifdef VK_ANSI_FN_PROTOS
    void * handle
#endif
	);

VK_tBoolean FWT_SignalActivity (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

VK_tBoolean FWT_SignalActivityOnce (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void FWT_SignalMap (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void FWT_SetAllSignalDisplay (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void FWT_Initialize (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void FWT_SetSignalDisplay (
#ifdef VK_ANSI_FN_PROTOS
	Integer4 signal
#endif
	);

void FWT_CalculateWidth(
#ifdef VK_ANSI_FN_PROTOS
	VK_tBoolean names_mode,
	Integer4   skip_columns,
	Integer4 * total_signals,
	Integer4 * displayed_signals,
	Integer4 * wave_width
#endif
	);

void FWT_FormatTimeSlice(
#ifdef VK_ANSI_FN_PROTOS
	char *buf,			
	Integer4 bufsize,
	Integer4  skip_columns
#endif
	);

void FWT_FormatWaveHeader(
#ifdef VK_ANSI_FN_PROTOS
	char *buf,
	Integer4 bufsize,
	VK_tBoolean names,
	Integer4  skip_columns		     
#endif
	);

#endif /* FWTrace_H */
