/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKCommand.c
*
* DESCRIPTION:  
*	User debugging features.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* TODO:
*	- Break line and break unit cmds should not set two breaks to same
*	  location. Should scan breakpoint list to check for dupes.
*
*	- A cmd to assign values to variables and signals.
*
* HISTORY:
*	16-JAN-90 DSB	Split from TKDebug.c
*	18-MAR-91 HK	Added references to TIME macros
*	04-APR-91 WAG	W0023:  Changed all abbreviations of number of the
*			for "nr" to the form "num"
*	25-APR-91 AWH	Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	08-JUN-91 AWH   Merged in VLK 1.0 commands
*	01-OCT-91 MNP	Renamed break line to break unit; added new brk. ln.
*
* SCCS ID: @(#)TKCommand.c	42.2 1/13/92
*
******************************************************************* */
static char SCCS_ID[] = "@(#)TKCommand.c	42.2 1/13/92";

#include "Version.inc"		/* for: VLK_SYSTEM_VERSION_4 */

#include "TKCommon.h"
#include "TKGlobals.h"		/* for: GF. */
#include "TKDebug.h"		/* for: DB. and tLineNumCode */
#include "TKDebugUtils.h"	/* for Verify*Line... */
#include "TKSim.h"		/* for: CurrentProcess */
#include "TKCommand.h"
#include "TKData.h"		/* for TIME macros */
#include "TKStructs.h"		/* for symbolic information types */
#include "TKSymbUtils.h"	/* for TK_GetRegionInfo, etc. */
#include "TKIO.h"		/* for DisplayResponse */
#include "CmdProcessor.h"

#include <signal.h>		/* for DoExamine... signal handling */
#include <setjmp.h>

/* ?? why aren't these all declared with "static" as the function bodies are? 
	?? */
VK_tBoolean DoBreakLine();
VK_tBoolean DoBreakUnit();
VK_tBoolean DoBreakProcess();
VK_tBoolean DoBreakTime();
VK_tBoolean DoCancelBreakpoint();
VK_tBoolean DoComment();
VK_tBoolean DoDo();
VK_tBoolean ParseDump();
VK_tBoolean DoElaborate();
VK_tBoolean DoExit();
VK_tBoolean DoEnd();
VK_tBoolean DoGo();
VK_tBoolean DoHelp();
VK_tBoolean DoInitialize();
VK_tBoolean DoPaceTime();
VK_tBoolean DoSetMovie();
VK_tBoolean DoSetPace();
VK_tBoolean DoDumpSymbolic();
VK_tBoolean DoShowBreakpoints();
VK_tBoolean DoShowHierarchy();
VK_tBoolean DoShowProcesses();
VK_tBoolean DoShowSignals();
VK_tBoolean DoStep();
VK_tBoolean DoStepCycle();
VK_tBoolean DoStepInto();
VK_tBoolean DoStepOver();
VK_tBoolean DoStepProcess();
VK_tBoolean DoStepSuspend();
VK_tBoolean DoRecord();
VK_tBoolean DoExamine();
VK_tBoolean DoUnimplemented();
VK_tBoolean DoVersion();
VK_tBoolean ParseBreak();
VK_tBoolean ParseCancel();
VK_tBoolean ParseSet();
VK_tBoolean ParseShow();
VK_tBoolean ParseDump();

#define TableSize(T) (sizeof(T)/sizeof(T[0]))

static tCmdTable TopCmdTbl = 
{
    {"break", ParseBreak},
    {"cancel", ParseCancel},
    {"#", DoComment},
    {"cycle", DoStepCycle}, 		/* ?? remove; renamed to step cycle */
    {"do", DoDo},
    {"dump", ParseDump},
    {"elaborate", DoElaborate},
    {"end", DoEnd},
    {"ex", DoExamine},
    {"examine", DoExamine},
    {"exit", DoExit},
    {"go", DoGo},
    {"help", DoHelp},
    {"initialize", DoInitialize},
    {"into", DoStepInto},
    {"next", DoStepOver},
    {"over", DoStepOver},
    {"quit", DoExit},
    {"record", DoRecord},
    {"set", ParseSet},
    {"show", ParseShow},
    {"s", DoStep},
    {"step", DoStep},
    {"suspend", DoStepSuspend},		/* ?? remove; renamed to step suspend */
/*    {"test", DoTest}, ?? */
    {"version", DoVersion}
};
static tCmdTableId TopCmdTblId;


/* ?? these name are bad form;  e.g., CmdTable_Break would follow precedents
	much better */

static tCmdTable breakCmdTbl =
{
    {"line", DoBreakLine},
    {"time", DoBreakTime},
    {"process", DoBreakProcess},
    {"unit", DoBreakUnit}

};
static tCmdTableId breakCmdTblId;


static tCmdTable cancelCmdTbl =
{
    {"breakpoint", DoCancelBreakpoint }

};
static tCmdTableId cancelCmdTblId;


static tCmdTable setCmdTbl =
{
    {"movie", DoSetMovie},
    {"pace",  DoSetPace}

};
static tCmdTableId setCmdTblId;



static tCmdTable dumpCmdTbl = 
{
    {"symbolic_information", DoDumpSymbolic}
};
static tCmdTableId dumpCmdTblId;

static tCmdTable showCmdTbl = 
{
    {"breakpoints", DoShowBreakpoints},
    {"hierarchy", DoShowHierarchy},
    {"processes", DoShowProcesses},
    {"signals", DoShowSignals}
};
static tCmdTableId showCmdTblId;

VK_tBoolean AnotherCommand;    

/* Default recording file name. */
#define RECORDING_FILE	"vlk.script"
static FILE	*RecordFP = NULL;
static FILE	*SourceFP = NULL;
static Boolean1	RecordLine = False;

/* Error strings used by some fns. The function name forms the prefix. */

#define DBL_NOCUR \
    "Error: Current position not yet established, must specify completely."

#define DBL_USAGE \
    "FILE`LINE or LINE expected; or . for current position."

#define DBU_USAGE \
   "Error: LIBRARY`PRIMARY`SECONDARY`LINE expected, or . for current position."


/* Used by break line and break unit */
#define ID_MAX 3
#define ID_LIBRARY 0
#define ID_PRIMARY 1
#define ID_SECONDARY 2
    
#ifdef VK_ANSI_FN_PROTOS
  extern char *strchr(char *, int);
#endif

/* *******************************************************************
*
* DoBreakLine - command :  break line [file_name`]line_number | .
*
* HISTORY:
*	01-OCT-91 MNP	Renamed old DoBreakLine to DoBreakUnit
*	01-OCT-91 MNP	Created
*
******************************************************************* */

static VK_tBoolean  DoBreakLine(token_buffer)
tTokenBuffer token_buffer;
{
    Integer4 line;
    tTempBuf256 specified_file;

    VK_tCString library, primary, secondary, source_file; 
    VK_tLineNumber following_line, previous_line, smallest, largest;
    VK_tSymbInfoHandle unit_info;

    tTempBuf1024 buffer;
    tToken token;
    tTempBuf80 value_buffer;

    tLineNumCode status;
    tTokenSeparator tk_sp; 
    char *tick_pos, *badptr;
    int remains;
/**/
    GetLastBreakpoint(&library, &primary, &secondary, &source_file, &line);

    SetTokenSeparator(tk_sp, TS_WhiteSpace, NULL);
    NextToken(token_buffer, token, tk_sp);
    TokenValue(token, buffer, sizeof(buffer));
    
    if (TokenType(token) == TK_NoToken) {
	DisplayResponse(DBL_USAGE);
	return True;
    }
    else {
     	TokenValue(token, buffer, sizeof(buffer));
	
	if (!strcmp(".", buffer)){		/* check for . */
	    if ('\0' == *library || '\0' == *primary ||'\0' ==
			*secondary || 0 == line) {
		DisplayResponse(DBL_NOCUR);	/* err, no defaults */
		return True;
	    }
	    else {
		strncpy(specified_file, source_file,
			sizeof(specified_file) - 1); 
	    }
	}
					/* check for file`line */
	else if (tick_pos = strchr(buffer, '`')) {
	    strncpy(specified_file, buffer, tick_pos-buffer);
	    specified_file[tick_pos-buffer] = '\0';  /* safety */
	    line = atoi(tick_pos + 1);
	}
					/* Check for number only  */
	else {
	    remains = sscanf(buffer, "%d %s", &line, value_buffer);
	    
	    switch (remains) {
	      case 1:		/* found line */
		if (NULL != source_file && '\0' != source_file) {
		    strncpy(specified_file, source_file,
			sizeof(specified_file) - 1); 
		}
		else {
		    DisplayResponse(DBL_USAGE);		
		    return True;
		}
		break;
		
	      case 2:		/* found line + garbage */
		sprintf(buffer, "Warning: extra parameters \"%s\"ignored.",
			value_buffer);
		DisplayResponse(buffer);
		break;
		
	      default:		/* nothing found */
		DisplayResponse(DBL_USAGE);		
		return True;
	    } /* switch */
	} /* if dot */
    } /* if token */
    
    /* Validate Line Number */
    status = VerifyFileLine(specified_file, line, &previous_line,
			    &following_line, &smallest, &largest, &unit_info);
    switch (status) {
      case L_NoValue:
      case L_InvParam:
	ReportInternalError("invalid parameters to VerifyLineNum", "", 
			    __FILE__, __LINE__);
	break;
	
      case L_InvSrcFile:
	if (NULL == specified_file || '\0' == *specified_file) {
	    DisplayResponse("Error: current position not yet established, source file must be specified." ); 
	}
	else {
	    PrintfResponse("Error: invalid file name \"%s\".",
			specified_file );
	}
	break;
	
      case L_InvLine:
	DisplayResponse("Error: invalid line number");
	break;
	
      case L_InRange:
	PrintfResponse(
                "Error: statement at %d not found; previous:%d following:%d",
		line, previous_line, following_line);
	break;
	
      case L_OutOfRange:
	PrintfResponse(
		"Error: statement at %d not found; out of range %d-%d",
		line, smallest, largest);
	break;
	
      case L_Same:
	su_GetUnitNames(unit_info, &library, &primary, &secondary,
		  &source_file); 
	SetLineBreakpoint((VK_tProcessHandle) NULL, 	/* process */
			  library, primary, secondary, line,
			  source_file);
	break;
	
      default:
	ReportInternalError("unexpected case in switch", "",
			    __FILE__, __LINE__);
	break;
	
    } /* switch */

    return True;
} /* DoBreakLine */


/* *******************************************************************
*
* DoBreakUnit - break unit [[lib`]prim_unit`[sec_unit`]] number | .
*
* HISTORY:
*	16-JAN-91 AWH	Created
*	01-OCT-91 MNP	Renamed from DoBreakLine
*
******************************************************************* */

static VK_tBoolean  DoBreakUnit(token_buffer)
tTokenBuffer token_buffer;
{
    VK_tBoolean more;
    VK_tBoolean line_found;
    VK_tBoolean line_started;
    VK_tBoolean tick_expected;

    VK_tProcessHandle process;
    Integer4 line;

    tTempBuf80  ids[ID_MAX];
    VK_tCString idp[ID_MAX];
    
    Integer4 id_count;
    Integer4 id_pointer;

    tTempBuf1024 buffer;
    
    tToken token;
    tTempBuf80 value_buffer;

    Integer4	previous_line;
    Integer4	following_line;
    tLineNumCode	status;
    tTokenSeparator tk_sp; 
    VK_tCString source_file;
/**/

    previous_line = 0;
    following_line = MaxInt4;
    more = True;
    line_started = False;
    line_found = False;
    tick_expected = False;
    id_count = 0;
    process = NULL;

    GetLastBreakpoint(&idp[ID_LIBRARY], &idp[ID_PRIMARY],
		      &idp[ID_SECONDARY], &source_file, &line);
    
    /* parse line specifier */
    while (more)  {
	
        SetTokenSeparator(tk_sp, NULL);
    	NextToken(token_buffer, token, tk_sp);
    
   	switch (TokenType(token)) {
      	    case TK_NoToken:
	        if (tick_expected || line_started)  {
		    DisplayResponse(
			 "Error: invalid line specifier");
	        }
	    	else {
	 	    DisplayResponse(DBU_USAGE);
		}
		
	   	more = False; 
		break;
	
     	    case TK_Identifier:
	        if (tick_expected || id_count >= ID_MAX )  {
		    DisplayResponse(
			 "Error: invalid line specifier");
		    more = False;
	        }
	    	else {
		    TokenValue(token, ids[id_count] , sizeof(ids[id_count]));
		    id_count ++;
		    line_started = True;
		    tick_expected = True;
		}
		break;
	    
	    case TK_Number:
	        if (tick_expected )  {
		    DisplayResponse(
			 "Error: invalid line specifier");
		    more = False;
	        }
	    	else {
		    TokenValue(token, value_buffer, sizeof(value_buffer));
		    line = atoi(value_buffer);
		    line_found = True;
		    more = False;
		}
		break;
		
	    case TK_Tick:
	        if (! tick_expected) {
		    DisplayResponse(
			 "Error: invalid line specifier");
		    more = False;
	        } 
		else {
		    tick_expected = False;
		} 
		break;
		    
      	    case TK_Period:
		if (line_started)	{
		    DisplayResponse("Error: invalid line specifier");
		}
		else {
		    line_found = True;
		}
		more = False;
		break;	

      	     default:
		DisplayResponse("Error: unrecoginzed parameter");
		break;    
   	 } /* switch */
    } /* while */

    /* process line */
    if (line_found)  {

	/* set library etc ... */
        id_pointer = ID_MAX - 1;
     
        while (id_count > 0 ) {
            id_count -- ;
	    idp[id_pointer] = ids[id_count];
	    id_pointer --;
	} /* while (id_count > 0) */

        /* Validate Line Number */
	status = VerifyUnitLine(idp[ID_LIBRARY], idp[ID_PRIMARY],
		     idp[ID_SECONDARY], line, &previous_line,
		     &following_line, &source_file);
	switch (status) {
	  case L_NoValue:
	  case L_InvParam:
	  	ReportInternalError("invalid parameters to VerifyUnitLine","", 
			      __FILE__, __LINE__);
		break;

	  case L_InvLine:
		DisplayResponse("Error: invalid line number");
                break;

          case L_NoLibrary:
		if (NULL == idp[ID_LIBRARY] || '\0' == *idp[ID_LIBRARY]) {
 			DisplayResponse("Error: current position not yet established, library must be specified." ); 
		}
		else {
		    PrintfResponse("Error: library %s not found",
					idp[ID_LIBRARY]);
		}
                break;

          case L_NoUnit:
		PrintfResponse("Error: unit %s`%s`%s not found",
				idp[ID_LIBRARY], idp[ID_PRIMARY],
				idp[ID_SECONDARY]);
                break;

          case L_InRange:
		PrintfResponse(
                  "Error: statement at %d not found; previous:%d following:%d",
		   line, previous_line, following_line);
                break;
          case L_OutOfRange:
		PrintfResponse(
		    "Error: statement at %d not found; out of range %d-%d.",
		    line, previous_line, following_line);
                break;

          case L_Same:
		SetLineBreakpoint(process, 
                  idp[ID_LIBRARY], idp[ID_PRIMARY],   
       		  idp[ID_SECONDARY], line, source_file);
                break;
          default:
             	ReportInternalError("unexpected case in switch", "",
                             __FILE__, __LINE__);
                break;

	} /* switch */
    } /* if (line_found) */
	    
    else if ('\0' == *idp[ID_LIBRARY]) {	
	DisplayResponse(DBL_NOCUR);
    }
    else {
	ReportInternalError("Internal Error in DoBreakUnit...", "",
			    __FILE__, __LINE__); 
    }

    return True;
} /* DoBreakUnit */

/* *******************************************************************
*
* DoBreakProcess -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoBreakProcess(token_buffer)
tTokenBuffer token_buffer;
{ 
    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;
/**/
    SetTokenSeparator(tk_sp, NULL);
    NextToken(token_buffer, token, tk_sp);
    
    switch(TokenType(token)) {
      case TK_NoToken:
	DisplayResponse("Error: requires process name or '*' for all processes");
	break;
	
      case TK_Identifier:
	TokenValue(token, value_buffer, sizeof(value_buffer));
	SetProcessBreakpoint(value_buffer);
	break;

      case TK_Asterisk:
	/* All Processes */
	SetProcessAllBreakpoint();
	DisplayResponse("Done: Set break before any process");
	break;

      case TK_Period:
	if (IsUserProcess(CurrentProcess)) {
	    SetProcessBreakpoint(qqName(CurrentProcess));
	}
	else {
	    DisplayResponse("Error: not currently in user process");
	}
	break;
	
      default:
	DisplayResponse("Error: unrecoginzed parameter");
	break;    
    } /* switch */
   
    return True;
} /* DoBreakProcess */



/* *******************************************************************
*
* DoBreakTime -
*
* HISTORY:
*	23-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoBreakTime(token_buffer)
tTokenBuffer token_buffer;
{
    VK_tBoolean more;
    VK_tBoolean time_found;
    VK_tBoolean error_found;

    tAbsTimeBuf time_buffer;
    tAbsTimeBuf time_base;

    tTempBuf256 buffer;

    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;
/**/

    more = True;
    time_found = False;
    strcpy(value_buffer, "1");
    TIME_CONVERT(time_base, value_buffer);
    error_found = False;
    
    /* parse time specifier */
    while (more)  {
	
        SetTokenSeparator(tk_sp, NULL);
    	NextToken(token_buffer, token, tk_sp);
    
   	switch (TokenType(token)) {
      	    case TK_NoToken:
	        if (! time_found)  {
		    DisplayResponse(
			 "Error: time parameter required");
		    error_found = True;
	        }
	   	more = False; 
		break;
	
     	    case TK_Identifier:
	        if (! time_found )  {
		    DisplayResponse(
			 "Error: unrecognized parameter");
		    more = False;
		    error_found = True;
	        }
	    	else {
		    TokenValue(token, value_buffer , sizeof(value_buffer));

		    if (StringMatch(TU_PS, value_buffer)) {	
			TIME_COPY(time_base, tu_ps_factor);
		    }
		    else if (StringMatch(TU_NS, value_buffer)) {
			TIME_COPY(time_base, tu_ns_factor);
		    }
		    else if (StringMatch(TU_US, value_buffer)) {
			TIME_COPY(time_base, tu_us_factor);
		    }
		    else {
			DisplayResponse("Error: unrecognized time unit");
		    	error_found = True;
		    }
		    more = False;
		}
		break;
	    
	    case TK_Number:
	        if (time_found)  {
		    DisplayResponse(
			 "Error: invalid time parameter");
		    more = False;
		    error_found = True;
	        }
	    	else {
		    TokenValue(token, value_buffer, sizeof(value_buffer));
		    TIME_CONVERT(time_buffer, value_buffer);
		    time_found = True;
		}
		break;

      	     default:
		DisplayResponse("Error: unrecoginzed parameter");
		error_found = True;
		more = False;
		break;    
   	 } /* switch */
    } /* while */

    if (! error_found) {

	TIME_MULT(time_buffer, time_buffer, time_base);
	
	if (True /* ?? check for overflow from multiplication ?? */) {
	    if (IsFutureTime(time_buffer)) {
		SetTimeBreakpoint(&time_buffer);
		UTIME_FORMAT(value_buffer, time_buffer);
		sprintf(buffer, "Done: break set at time %s %s", 
			value_buffer, BaseUnit);
		DisplayResponse(buffer);
	    }
	    else {
		DisplayResponse("Error: time already passed");
	    }
	}
        else {
		DisplayResponse("Error: time to large");
	}
	
    }
    
    return True;
} /* DoBreakTime */


/* *******************************************************************
*
* DoCancelBreakpoint -
*
* HISTORY:
*	21-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoCancelBreakpoint(token_buffer)
tTokenBuffer token_buffer;
{
    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;
/**/
    SetTokenSeparator(tk_sp, NULL);
    NextToken(token_buffer, token, tk_sp);
    
    switch(TokenType(token)) {
      case TK_NoToken:
	DisplayResponse(
	  "Error: requires breakpoint number or '*' for all breakpoints");
	break;
	
      case TK_Number:
	TokenValue(token, value_buffer, sizeof(value_buffer));
	if (CancelBreakpointN(atoi(value_buffer))) {
	    DisplayResponse("Done: breakpoint canceled");
	}
	else {
	    DisplayResponse("Error: breakpoint not canceled ");
	}
	break;

      case TK_Asterisk:
	/* All Breakpoints */
	DoUnimplemented();
	break;
	
      default:
	DisplayResponse("Error: unrecoginzed parameter");
	break;    
    } /* switch */
    
    return True;

} /* DoCancelBreakpoint */



/* *******************************************************************
*
* DoStepCycle -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoStepCycle()
{
/**/

    if (DB.is_after_sim) {
       DisplayResponse("Error: simulation has already completed");
    }
    else {
	SetCycleBreakpoint();
    	AnotherCommand = False;
    }
    return True;
} /* DoStepCycle */


/* *******************************************************************
*
* DoComment -
*
* HISTORY:
*	17-MAY-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoComment()
{
/**/
    /* nothing to do */
    return True;
} /* DoComment */


/* *******************************************************************
*
* DoDo -
*
* HISTORY:
*	17-MAY-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoDo(token_buffer)
tTokenBuffer token_buffer;    
{
    tToken token;
    tTempBuf80 value_buffer;
    tTempBuf256 message;
    tTokenSeparator tk_sp;
    StringN file_name;
/**/
    /* Parse Optional Parameter */
    SetTokenSeparator(tk_sp, TS_WhiteSpace, NULL);
    NextToken(token_buffer, token, tk_sp);
    if (TokenType(token) == TK_NoToken) {
	file_name = RECORDING_FILE;
    }
    else {
     	TokenValue(token, value_buffer, sizeof(value_buffer));
	file_name = value_buffer;
    }
    
 
    if (NULL != RecordFP) {
	DisplayResponse("Error: cannot use do command in a script");
	}

    /* Open Command File */    
    else if (NULL == SourceFP) {
	/* open command file */
	SourceFP = fopen(file_name, "r");
	if (NULL != SourceFP ) {
	    sprintf(message,"Done: executing commands in file: %s.", file_name);
	    DisplayResponse(message);
	}
	else 	{
	    sprintf(message, "Error: unable to open file: %s.", file_name);
	    DisplayResponse(message);
	}
    }
    else {
	DisplayResponse("Error: already executing do command.");
    }
    
    return True;


} /* DoDo */



/* *******************************************************************
*
* DoEnd -
*
* HISTORY:
*	17-MAY-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoEnd()
{
/**/
 
    if (NULL != RecordFP) {
	fclose(RecordFP);
	RecordFP = NULL;
       	DisplayResponse("Done: recording of commands ended.");
	}
    else {
	DisplayResponse("Error: not currently recording commands.");
    }
    RecordLine = False;
   
    return True;
    
} /* DoEnd */


/* *******************************************************************
*
* DoElaborate -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoElaborate()
{
/**/

    if (DB.is_after_elab) {
       DisplayResponse("Error: elaboration already complete");
    }
    else {
	SetElabBreakpoint();
    	AnotherCommand = False;
    }
    return True;
} /* DoElaborate */



/* *******************************************************************
*
* DoExit -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoExit()
{
/**/
    TerminateKernel();
    return True;
} /* DoExit */


/* *******************************************************************
*
* DoHelp - display help information
*
* HISTORY:
*	21-DEC-90 AWH	Create
* 	01-OCT-91 MNP	break-line -> break unit, added new break line
*
******************************************************************* */
static VK_tBoolean DoHelp()
{
/**/
    /* Display Text */
    DisplayText("VLK Simulation System Shell Mode Commands:");
    DisplayText("  Basic commands:");
    DisplayText("    elaborate               - run to end of elaboration");
    DisplayText("    initialize              - run to end of initialization");
    DisplayText("    go                      - run to next breakpoint");
    DisplayText("    next                    - step to next statement, w/o entering subprograms");
    DisplayText("    step                    - step to next statement, entering subprograms");
    DisplayText("    step cycle              - continue to end of simulation cycle");
    DisplayText("    step suspend            - continue until process suspension");
    DisplayText("    exit, quit              - terminate simulator");
    DisplayText("  Breakpoint commands:");
    DisplayText("    break process <p>       - before running process <p>; \"*\" for all processes");
    DisplayText("    break line <f>`<l>      - at file <f> and line <l>; \".\" for current line");
    DisplayText("    break unit <u>`<l>      - at unit <u> and line <l>; \".\" for current line");
    DisplayText("    break time <t>          - at a given time");
    DisplayText("    cancel breakpoint <n>   - remove breakpoint <n>");
    DisplayText("  Show commands:");
    DisplayText("    show hierarchy          - display static structure");
    DisplayText("    show breakpoints        - display breakpoints");
    DisplayText("    show processes          - display explicit and implicit processes");
    DisplayText("    show signals            - display signal map");
    DisplayText("    examine <simple_name>   - examine values of objects");
    DisplayText("    examine *               - examine values of objects");
    DisplayText(" ");
    DisplayText("All commands may be abbreviated to the smallest unique strings.");
    DisplayText("See Appendix 1 of \"Designing with VHDL\" for the full command set.");

    return True;
} /* DoHelp */


/* *******************************************************************
*
* DoGo -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoGo()
{
/**/
    if (DB.is_after_sim ) {
	DisplayResponse("Error: simulation already completed");
    }
    else {
	AnotherCommand = False;
    }
    return True;
} /* DoGo */



/* *******************************************************************
*
* DoInitialize -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean DoInitialize()
{
/**/

    if (DB.is_after_init) {
       DisplayResponse("Error: initialization already completed");
    }
    else {
	SetInitBreakpoint();
    	AnotherCommand = False;
    }
    return True;
} /* DoInitialize */



/* *******************************************************************
*
* DoPaceTime -
*
* HISTORY:
*	25-JAN-91 AWH	Created
*
******************************************************************* */
#if 0
static VK_tBoolean  DoPaceTime(token_buffer)
tTokenBuffer token_buffer;
{
    VK_tBoolean more;
    VK_tBoolean time_found;
    VK_tBoolean error_found;

    tAbsTimeBuf time_buffer;
    tAbsTimeBuf time_base;

    tTempBuf256 buffer;

    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;
/**/

    more = True;
    time_found = False;
    strcpy(value_buffer, "1");
    TIME_CONVERT(time_base, value_buffer);
    error_found = False;
    
    /* parse time specifier */
    while (more)  {
	
        SetTokenSeparator(tk_sp, NULL);
    	NextToken(token_buffer, token, tk_sp);
    
   	switch (TokenType(token)) {
      	    case TK_NoToken:
	        if (! time_found)  {
		    DisplayResponse(
			 "Error: time parameter required");
		    error_found = True;
	        }
	   	more = False; 
		break;
	
     	    case TK_Identifier:
	        if (! time_found )  {
		    DisplayResponse(
			 "Error: unrecognized parameter");
		    more = False;
		    error_found = True;
	        }
	    	else {
		    TokenValue(token, value_buffer , sizeof(value_buffer));

		    if (StringMatch(TU_PS, value_buffer)) {	
			TIME_COPY(time_base, tu_ps_factor);
		    }
		    else if (StringMatch(TU_NS, value_buffer)) {
			TIME_COPY(time_base, tu_ns_factor);
		    }
		    else if (StringMatch(TU_US, value_buffer)) {
			TIME_COPY(time_base, tu_us_factor);
		    }
		    else {
			DisplayResponse("Error: unrecognized time unit");
		    	error_found = True;
		    }
		    more = False;
		}
		break;
	    
	    case TK_Number:
	        if (time_found)  {
		    DisplayResponse(
			 "Error: invalid time parameter");
		    more = False;
		    error_found = True;
	        }
	    	else {
		    TokenValue(token, value_buffer, sizeof(value_buffer));
		    TIME_CONVERT(time_buffer, value_buffer);
		    time_found = True;
		}
		break;

      	     default:
		DisplayResponse("Error: unrecoginzed parameter");
		error_found = True;
		more = False;
		break;    
   	 } /* switch */
   } /* while */

    if (! error_found) {

	TIME_MULT(time_buffer, time_buffer, time_base);
	
	if (True /* ?? check for overflow from multiplication ?? */) {
		SetPaceTime(&time_buffer);
		UTIME_FORMAT(value_buffer, time_buffer);
		sprintf(buffer, 
	 "Done: each %s %s of simulation time will take at least one second", 
			value_buffer, BaseUnit);
		DisplayResponse(buffer);
	}
        else {
		DisplayResponse("Error: time too large");
        }
    }
    return True;
} /* DoPaceTime */

#endif

/* *******************************************************************
*
* DoRecord -
*
* HISTORY:
*	17-MAY-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoRecord(token_buffer)
tTokenBuffer token_buffer;
{
    tToken token;
    tTempBuf80 value_buffer;
    tTempBuf256 message;
    tTokenSeparator tk_sp;
    StringN file_name;
/**/
    /* Parse Optional Parameter */
    SetTokenSeparator(tk_sp, TS_WhiteSpace, NULL);
    NextToken(token_buffer, token, tk_sp);
    if (TokenType(token) == TK_NoToken) {
	file_name = RECORDING_FILE;
    }
    else {
     	TokenValue(token, value_buffer, sizeof(value_buffer));
	file_name = value_buffer;
    }
     
 
    /* Open Command File */    
    if (NULL == RecordFP) {
	/* open command file */
	RecordFP = fopen(file_name, "w");
	if (NULL != RecordFP ) {
	    long secs = time(NULL);

	    fprintf(RecordFP, "#\n# VHDL Learning Kit Version %s\n",
		VLK_SYSTEM_VERSION_4);
	    fprintf(RecordFP, "# Command Recording File\n#\n");
	    fprintf(RecordFP, "# Date: %s", ctime(&secs));
	    fprintf(RecordFP, "# User: %s\n#\n",  getenv("USER"));

	    sprintf(message,"Done: recording commands to file: %s.", file_name);
	    DisplayResponse(message);
	}
	else 	{
	    sprintf(message, "Error: unable to open file: %s.", file_name);
	    DisplayResponse(message);
	}
    }
    else {
	DisplayResponse("Error: already recording commands.");
    }
    
    RecordLine = False;
    return True;
    
} /* DoRecord */



/* *******************************************************************
*
* DoSetPace -
*
* HISTORY:
*	25-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoSetPace(token_buffer)
tTokenBuffer token_buffer;
{
    VK_tInteger seconds;
    
    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;

    tTempBuf256 buffer;
    
/**/
    seconds = -1;
    SetTokenSeparator(tk_sp, NULL);
    NextToken(token_buffer, token, tk_sp);
    
    switch (TokenType(token)) {
      	    case TK_NoToken:
	 	seconds = 1;
		break;
	
     	    case TK_Identifier:
		DisplayResponse(
			 "Error: unrecognized parameter");
	    	break;
	    
	    case TK_Number:
		TokenValue(token, value_buffer, sizeof(value_buffer));
		seconds = atoi(value_buffer);
		break;

      	     default:
		DisplayResponse("Error: unrecoginzed parameter");
		break;    
   	 } /* switch */

	if ( -1 != seconds ) {
		SetPaceTime(seconds);
		sprintf(buffer, 
	 "Done: each possible step point will now take at least %.1f seconds.",
			((float)seconds/10.0));
		
		DisplayResponse(buffer);
	    }
    
    return True;
} /* DoSetPace */


/* *******************************************************************
*
* DoSetMovie -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoSetMovie(token_buffer)
tTokenBuffer token_buffer;
{
    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;
/**/
   SetTokenSeparator(tk_sp, NULL);
   NextToken(token_buffer, token, tk_sp);
    switch(TokenType(token)) 
    {
      case TK_NoToken:
	DisplayResponse("Error: qualifier required");
	break;
      case TK_Identifier:
	TokenValue(token, value_buffer, sizeof(value_buffer));
	if (StringMatch(value_buffer, "o")) {
	    DisplayResponse("Error: ambigious option");
	}
       	else if (StringMatchPrefix(value_buffer, "on")) {
	    if (DB.movie_flag) {
		DisplayResponse("Warning: movie mode already on");
	    }
	    else {
		DB.movie_flag = True;
	   	DisplayResponse("Done: movie mode on");
	    }
	}
       	else if (StringMatchPrefix(value_buffer, "off")) {
	    if (! DB.movie_flag) {
		DisplayResponse("Warning: movie mode already off");
	    }
	    else {
		DB.movie_flag = False;
	   	DisplayResponse("Done: movie mode off");
	    }
	}
	else  {
           DisplayResponse("Error: unrecognized command");
        }
	break;
      default:
	  DisplayResponse("Error: unrecognized command");
	break;
    } /* switch */
	    
    return True;
	
} /* DoSetMovie */


/* *******************************************************************
*
* DoDumpSymbolic -
*
* HISTORY:
*	22-JUL-91 DSB	Created
*
******************************************************************* */

static VK_tBoolean DoDumpSymbolic()
{
/**/
    DumpSymbInfo();
    return True;
} /* DoDumpSymbolic */


/* *******************************************************************
*
* DoShowBreakpoints -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoShowBreakpoints()
{
/**/
    ShowBreakpoints();
    return True;
} /* DoShowBreakpoints */


/* *******************************************************************
*
* DoShowHierarchy -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoShowHierarchy()
{
/**/
    ShowHierarchy();
    return True;
} /* DoShowHierarchy */


/* *******************************************************************
*
* DoShowProcesses -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoShowProcesses()
{
/**/

    ShowProcesses();
    return True;

} /* DoShowProcesses */


/* *******************************************************************
*
* DoShowSignals -
*
* HISTORY:
*	17-MAY-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoShowSignals()
{
/**/

    ShowSignals();
    return True;

} /* DoShowSignals*/


/* *******************************************************************
*
* DoStep -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoStep(token_buffer)
tTokenBuffer token_buffer;
{
    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;
/**/

    if (DB.is_after_sim) {
       DisplayResponse("Error: simulation has already completed");
    }
    else {
        SetTokenSeparator(tk_sp, NULL);
	NextToken(token_buffer, token, tk_sp);
	if (TokenType(token) == TK_NoToken) {
	    /* step */
	    SetStepBreakpoint(NULL);
	    AnotherCommand = False;
	}
	else if (TokenType(token) == TK_Identifier) {
	    TokenValue(token, value_buffer, sizeof(value_buffer));
	    if (StringMatchPrefix(value_buffer, "process")) {
		DoStepProcess(token_buffer);
	    }
	    else if (StringMatchPrefix(value_buffer, "over")) {
		DoStepOver(token_buffer);
	    }
	    else if (StringMatchPrefix(value_buffer, "suspend")) {
		DoStepSuspend(token_buffer);
	    }
	    else if (StringMatchPrefix(value_buffer, "cycle")) {
		DoStepCycle(token_buffer);
	    }
	    else  {
		DisplayResponse("Error: unrecognized command");
	    }
	    
	}
	else {
	    DisplayResponse("Error: unrecognized command");
	    
	}
	
	
   }
    return True;

} /* DoStep */




/* *******************************************************************
*
* DoStepInto -
*
* HISTORY:
*	14-MAR-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoStepInto()
{
/**/

    if (DB.is_after_sim) {
       DisplayResponse("Error: simulation has already completed");
    }
    else {
       SetStepBreakpoint(NULL);
       AnotherCommand = False;
    }
    return True;
} /* DoStepInto */



/* *******************************************************************
*
* DoStepOver -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoStepOver(token_buffer)
tTokenBuffer token_buffer;
{
/**/

    if (DB.is_after_sim) {
       DisplayResponse("Error: simulation has already completed");
    }
    else {
     	SetStepOverBreakpoint(CurrentProcess);
	AnotherCommand = False;
    }
    return True;

} /* DoStepOver */



/* *******************************************************************
*
* DoStepProcess -
*
* HISTORY:
*	25-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoStepProcess(token_buffer)
tTokenBuffer token_buffer;
{
/**/

    if (DB.is_after_sim) {
       DisplayResponse("Error: simulation has already completed");
    }
    else {
     	SetStepBreakpoint(CurrentProcess);
	AnotherCommand = False;
    }
    return True;

} /* DoStepProcess */



/* *******************************************************************
*
* DoStepSuspend -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoStepSuspend()
{
/**/

    if (DB.is_after_sim) {
       DisplayResponse("Error: simulation has already completed");
    }
    else {
	SetSuspendBreakpoint();
    	AnotherCommand = False;
    }
    return True;
} /* DoStepSuspend */




/* *******************************************************************
*
* DoVersion -
*
* HISTORY:
*	17-MAY-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoVersion()
{
    tTempBuf80 buffer;
/**/
    sprintf(buffer, "VHDL Learning Kit Version %s", 
	    VLK_SYSTEM_VERSION_4);
    DisplayText(buffer);
    return True;
    
} /* DoVersion */


static jmp_buf JumpBuffer;	/* used for exit handler */


/* *******************************************************************
*
* ExamineHandler - handle expected signals for uninitialized dta
*
* HISTORY:
*	07-JUN_91 DSB	Created
*
******************************************************************* */
static void ExamineHandler()
{
/**/
    longjmp(JumpBuffer, 1);
} /* ExamineHandler */



typedef struct {
	VK_tDataAddr start_addr;	/* starting address of value */
	StringN buf_ptr;		/* buffer for value */
	Integer4 buf_size;		/* size of buffer */
	VK_tBoolean comma_flag;		/* flag to control comma insertion */
	VK_tBoolean qualify_flag;	/* flag to control qualified exprs. */
    } tFGVUserData;


/* *******************************************************************
*
* FGVUserFn1 - user of TraverseSubtype for composite value display
*
* HISTORY:
*	25-OCT-91 DSB	Created
*
******************************************************************* */
tUserStatus FGVUserFn1(user_data, trav_data)
tFGVUserData * user_data;	
tTravDataPtr trav_data;
{
    VK_tSymbInfoHandle type_info;
    VK_tInteger relative_offset;
    tTypeClass type_class;
    tTempBuf80 temp_buf;
/**/
	/* : if scalar (or access or file) type, format value */
    if (IsMember(tf_ScalarType, TravGet_Flags(trav_data))) {
	type_info = TravGet_SymbInfo(trav_data);
	relative_offset = TravGet_Offset(trav_data);

	if (user_data->qualify_flag) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, 
		qqName(TravGet_SymbInfo(trav_data)));
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, "'(");
					/* format as qualified expression
					   to show type name */
	}

	type_class = su_GetTypeInfoTypeClass(type_info);
	if (tc_Access == type_class) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, "<ptr>");
	}
	else if (tc_File == type_class) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, "<file>");
	}
	else {
	    FormatScalarValue(temp_buf, sizeof(temp_buf), type_class, 
		type_info, user_data->start_addr + relative_offset);
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, temp_buf);
	}
	user_data->comma_flag = True;

	if (user_data->qualify_flag) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, ")");
	}

    }

	/* : if record type, append "(", format elements, append ")" */
    if (IsMember(tf_RecordType, TravGet_Flags(trav_data))) {
	if (user_data->qualify_flag) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, 
		qqName(TravGet_SymbInfo(trav_data)));
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, "'");
					/* format as qualified expression
					   to show type name */
	}

	TempStrCat(user_data->buf_ptr, user_data->buf_size, "(");
					/* start record type */
	user_data->comma_flag = False;

	TraverseSubtype_Recurse(trav_data);
					/* recursively process record type */

	TempStrCat(user_data->buf_ptr, user_data->buf_size, ")");
					/* end record type */
    }
	/* : if array type, append "(", format elements, append ")" */
    if (IsMember(tf_ArrayType, TravGet_Flags(trav_data))) {
	if (user_data->qualify_flag) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, 
		qqName(TravGet_SymbInfo(trav_data)));
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, "'");
					/* format as qualified expression
					   to show type name */
	}

	TempStrCat(user_data->buf_ptr, user_data->buf_size, "(");
					/* start array type */
	user_data->comma_flag = False;

	TraverseSubtype_Recurse(trav_data);
					/* recursively process array type */

	TempStrCat(user_data->buf_ptr, user_data->buf_size, ")");
					/* end array type */
    }
	/* : if array dimension, append "(", format elements, append ")" */
    if (IsMember(tf_PseudoType, TravGet_Flags(trav_data))) {

	TempStrCat(user_data->buf_ptr, user_data->buf_size, "(");
					/* start array subdimension */
	user_data->comma_flag = False;

	TraverseSubtype_Recurse(trav_data);
					/* recursively process array type */

	TempStrCat(user_data->buf_ptr, user_data->buf_size, ")");
					/* end array subdimension */
    }
	/* : if record element, append element name and "=>", format value */
    if (IsMember(tf_RecordElem, TravGet_Flags(trav_data))) {
	if (user_data->comma_flag) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, ", ");
	}

	TempStrCat(user_data->buf_ptr, user_data->buf_size, 
		qqName(TravGet_SymbInfo(trav_data)));
	TempStrCat(user_data->buf_ptr, user_data->buf_size, " => ");
					/* start record element */

					/* return to process element value and
					   succeeding record elements */
/* ?? don't remove experiment until "unexpected state" bug is found ?? */
	if (user_data->comma_flag) {	/* ?? experimenting ?? */
					/* on first call, use return protocol;
					   on later calls recurse here */
	    TraverseSubtype_Recurse(trav_data);
					/* recursively process element  */
	}

	user_data->comma_flag = True;	/* */

    }
	/* : if record element, append index and "=>", format value */
    if (IsMember(tf_ArrayElem, TravGet_Flags(trav_data))) {
	VK_tIntegerData dim_index_value;
	VK_tSymbInfoHandle dim_subtype, dim_type_info;
	tTypeClass dim_type_class;
	tIndexArrayPtr index_array;
    /**/
	if (user_data->comma_flag) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, ", ");
	}

	index_array = TravGet_IndexArray(trav_data);
	dim_index_value = index_array[TravGet_Dimension(trav_data)-1];
	dim_subtype = SymbInfoValue(TempNthItem(TravGet_Dimension(trav_data), 
		qqChildren(TravGet_SymbInfo(trav_data))));
	dim_type_info = qqBaseTypeInfo(dim_subtype);

	dim_type_class = su_GetTypeInfoTypeClass(dim_type_info);
	FormatScalarValue(temp_buf, sizeof(temp_buf), dim_type_class, 
		dim_type_info, &dim_index_value);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, temp_buf);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, " => ");
					/* start array element */

/* ?? don't remove experiment until "unexpected state" bug is found ?? */
	if (user_data->comma_flag) {	/* ?? experimenting ?? */
					/* on first call, use return protocol;
					   on later calls recurse here */
	    TraverseSubtype_Recurse(trav_data);
					/* recursively process element */
	}

	user_data->comma_flag = True;	/* */
					/* return to process element value and
					   succeeding array elements */
    }
	/* : if pseudo-element, append index and "=>", format value */
    if (IsMember(tf_PseudoElem, TravGet_Flags(trav_data))) {
	VK_tIntegerData dim_index_value;
	VK_tSymbInfoHandle dim_subtype, dim_type_info;
	tTypeClass dim_type_class;
	tIndexArrayPtr index_array;
    /**/
	if (user_data->comma_flag) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, ", ");
	}

	index_array = TravGet_IndexArray(trav_data);
	dim_index_value = index_array[TravGet_Dimension(trav_data)-1];
	dim_subtype = SymbInfoValue(TempNthItem(TravGet_Dimension(trav_data), 
		qqChildren(TravGet_SymbInfo(trav_data))));
	dim_type_info = qqBaseTypeInfo(dim_subtype);

	dim_type_class = su_GetTypeInfoTypeClass(dim_type_info);
	FormatScalarValue(temp_buf, sizeof(temp_buf), dim_type_class, 
		dim_type_info, &dim_index_value);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, temp_buf);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, " => ");
					/* start array pseudo-element */

	if (user_data->comma_flag) {	/* ?? experimenting ?? */
					/* on first call, use return protocol;
					   on later calls recurse here */
	    TraverseSubtype_Recurse(trav_data);
					/* recursively process pseudo-element */
	}

	user_data->comma_flag = True;	/* */
					/* return to process pseudo-element 
					   value and succeeding array pseudo-
					   elements */
    }

    /*    printf("<<< %s >>>\n", user_data->buf_ptr); */

    return us_Searching;

} /* FGVUserFn1 */


/* *******************************************************************
*
* FormatGeneralValue - format value of arbitrary type into string
*
* DESCRIPTION:
*	Displays record and array values using aggregates with named named 
*	association.  Displays values of scalar type using FormatScalarValue.
*	Currently displays values of an access type (pointers) as "<ptr>", and
*	values of a file type as "<file>".
*
* HISTORY:
*	18-OCT-91 DSB	Created
*
******************************************************************* */
void FormatGeneralValue(buffer, buf_size, type_info, type_region_data, 
	constraint_addr, value_address)
StringN buffer;				/* buffer to hold string */
Integer4 buf_size;			/* total size of buffer */
VK_tSymbInfoHandle type_info;		/* type of value */
VK_tLARHeaderPtr type_region_data;	/* type's encl. region instance */
VK_tIndexConstraintAddr constraint_addr;
					/* array values's index constraint */
VK_tDataAddr value_address;		/* address of value */
{
    tFGVUserData user_data;
/**/
    user_data.start_addr = value_address;
    user_data.comma_flag = False;	/* initialize comma flag */

    user_data.qualify_flag = False;	/* disable this option currently */

    user_data.buf_ptr = buffer;
    strcpy(user_data.buf_ptr, "");	/* initialize buffer */
    user_data.buf_size = buf_size;

    TraverseSubtype(type_info, type_region_data, constraint_addr,
	tf_ALL, FGVUserFn1, &user_data);
					/* go to it! */
} /* FormatGeneralValue */



/* *******************************************************************
*
* DoExamineOneObjectsSub - 
*
* DESCRIPTION:
*
* HISTORY:
*	25-OCT-91 DSB	Split 
*	17-DEC-91 DKC	Added loop-parameter handling
*
******************************************************************* */
void DoExamineOneObject(object_info, current_region_data, current_region_info,
	any_names_p, any_objects_p)
VK_tSymbInfoHandle object_info;
VK_tLARHeaderPtr current_region_data;
VK_tSymbInfoHandle current_region_info;
VK_tBoolean * any_names_p;
VK_tBoolean * any_objects_p;
{
    tSymbInfoKind item_kind;

    VK_tBoolean found_something;
    VK_tBoolean is_interface_object;
    VK_tBoolean is_signal;
    VK_tBoolean is_composite;

    VK_tSymbInfoHandle type_info;
    tTypeClass type_class;
    VK_tLARHeaderPtr constraint_act_rec;
    VK_tIndexConstraintAddr constraint_address;

    VK_tSymbInfoHandle index_type_info;
    tTypeClass index_type_class = tc_BadTypeClass;

    VK_tDataLength length;

    VK_tLARHeaderPtr act_rec;

    VK_tSignalHandle signal_handle;
    VK_tSignalOffset signal_offset;
    VK_tDataAddr value_address;

    tTempBuf1024 buffer;		/* ?? THIS CAN EASILY OVERFLOW ?? */
    tTempBuf80   constraint;
    Integer4 len;
    VK_tBoolean first;
    Integer4 i;
    void (*save_main_handler_1)();
    void (*save_main_handler_2)();

    VK_tLARHeaderPtr type_region_data;
/**/

	/* 1: determine object characteristics for use in getting object value
		and constraint */
	
    item_kind = qqSymbInfoKind(object_info);

    is_interface_object = False;
    is_signal           = False;
    found_something     = False; 

    switch (item_kind) {
      case LoopStmtInfo: /* get loop parameter info */
                         object_info = SymbInfoValue(TempFirstItem(
                                                     qqChildren(object_info)));
                         item_kind   = qqSymbInfoKind(object_info);
                         VERIFY_ERROR(LoopParameterInfo == item_kind);
                         /* drop through */
      case LocalConstantDeclInfo:
      case LocalVariableDeclInfo:
      case LocalFileDeclInfo:          found_something     = True;
                                       break;

      case LocalSignalDeclInfo:        found_something     = True;
                                       is_signal           = True;
                                       break;

      case InterfaceVariableDeclInfo:
      case InterfaceConstantDeclInfo:  found_something     = True;
                                       is_interface_object = True;
                                       break;

      case InterfaceSignalDeclInfo:    found_something     = True;
	                               is_interface_object = True;
	                               is_signal           = True;
                                       break;

      default: break;  /* not displayable or not implemented */
    }

    if (found_something) {
	*any_objects_p = True;

	FormatLexicalName(buffer, sizeof(buffer), object_info);
					/* ?? Note: we will also want to 
					   differentiate different activations
					   by stack levels and by hierarchical
					   position ?? */



	save_main_handler_1 = signal(SIGBUS, ExamineHandler);
	save_main_handler_2 = signal(SIGSEGV, ExamineHandler);
					/* catch examine errors more softly;
					   remember previous settings */
	if (setjmp(JumpBuffer)) {
	    TempStrCat(buffer, sizeof(buffer), 
		"  <unexpected error in display routines>");
	}
	else {
            type_region_data = NULL;
	    type_info        = qqBaseTypeInfo(object_info);
	    type_class       = su_GetTypeInfoTypeClass(type_info);

			/* 2: get constraint information for object */

	    if (NULL != qqConstraintRegionInfo(object_info)) {
	    	constraint_act_rec = su_TraverseToData(
			qqConstraintRegionInfo(object_info), 
			current_region_info, current_region_data);

		/* special handling: if an interface object's constraint is in
			the same region as the object, then the constraint is
			in the interface activation record of the region */

		if (is_interface_object && qqConstraintRegionInfo(object_info)
                                             == current_region_info) {
		    constraint_act_rec = (VK_tLARHeaderPtr) 
			                        VK_GetIARP(constraint_act_rec);
		}

		constraint_address = (VK_tIndexConstraintAddr) 
			(qqConstraintOffset(object_info) + 
				(char *) constraint_act_rec);
	    }
	    else {
	    	constraint_address = NULL;
	    }

            if (NULL != constraint_address && 
                (tc_Array == type_class || LoopParameterInfo == item_kind)) {

		if (constraint_address->dir != VK_Ascending &&
		    constraint_address->dir != VK_Descending) {
		    goto abort_label;	/* looks uninitialized */
		}

	        if (tc_Array == type_class) {
    	            /* ?: if array object, format index constraint */
		    index_type_info  = qqBaseTypeInfo(SymbInfoValue(
			                TempFirstItem(qqChildren(type_info))));
		    index_type_class = su_GetTypeInfoTypeClass(index_type_info);
                }
                else { /* loop parameter */

		    index_type_info  = type_info;
                    index_type_class = type_class;
                }

		strcpy(constraint,"("); /* begin index constraint */

					/* ?? this should be something like
					   FormatDiscreteRange ?? */
		len = strlen(constraint);
		FormatScalarValue(&constraint[len], sizeof(constraint)-len,
			index_type_class, index_type_info, 
			&constraint_address->left);

		if (VK_Ascending == constraint_address->dir) {
		    TempStrCat(constraint, sizeof(constraint), " to ");
		}
		else {
		    TempStrCat(constraint, sizeof(constraint), " downto ");
		}

		len = strlen(constraint);
		FormatScalarValue(&constraint[len], sizeof(constraint)-len,
			index_type_class, index_type_info, 
			&constraint_address->right);

		TempStrCat(constraint, sizeof(constraint), ")");
					/* end index constraint */
	    }
	    else {
                *constraint = '\0';
            }

	    act_rec = (! is_interface_object) ? current_region_data :
		(VK_tLARHeaderPtr) VK_GetIARP(current_region_data);

	    if (setjmp(JumpBuffer)) {
					/* ?? AGH! a goto ?? */
		goto abort_label;	/* expect uninitialized 
						   pointers to hit here (before
						   next setjmp() call)*/
	    }
	    if (is_signal) {
			/* : if signal, read signal to get value address */
		length = su_GetSubtypeLength(type_info, type_region_data,
			constraint_address);

		signal_handle = * (VK_tSignalHandle *) 
			(qqHandleOffset(object_info) + (char *) act_rec);
		signal_offset = * (VK_tSignalOffset *) 
			(qqOffsetOffset(object_info) + (char *) act_rec);
		VK_ReadSignalValue(&value_address, 
			signal_handle, signal_offset, length);
	    }
	    else {
			/* : if constant or vaiable, get address directly */
	 	is_composite = tc_Array == type_class || 
				tc_Record == type_class;
		if (is_composite) {
		    value_address = * (VK_tCompositeAddr *) 
				(qqValueOffset(object_info) + 
					(char *) act_rec);

			/* : dereference address to check for bad (e.g.,
				uninitialized) pointer; catch signal under 
				specific JumpBuffer setting */
		    {
		    	VK_tScalarData d = *value_address; 
		    /**/
		    	if (d)
			    ;
		    }
		}
		else {
		    value_address = (VK_tScalarAddr) (
			qqValueOffset(object_info) + (char *) act_rec);
		}
		    
	    }

	    TempStrCat(buffer, sizeof(buffer), constraint);
	    TempStrCat(buffer, sizeof(buffer), " = ");

		/* : now format object value */

	    type_region_data = su_TraverseToData(qqEnclReg(type_info), 
		current_region_info, current_region_data);
	    len = strlen(buffer);
	    FormatGeneralValue(&buffer[len], sizeof(buffer)-len,
		type_info, type_region_data, constraint_address, 
		value_address); 

	} /* setjmp else */
					/* ?? AGH! a goto ?? */
	goto skip_abort;		/* go around abort_label code */
					/* ?? I need to use longjump instead 
					   of a goto ?? */
    abort_label:			/* expected error */
	TempStrCat(buffer, sizeof(buffer), 
		"  <can't display; may not be elaborated yet>");
    skip_abort:
	DisplayResponse(buffer);
				/* ?? DisplayResponse or -Text? ?? */

	signal(SIGBUS, save_main_handler_1);
	signal(SIGSEGV, save_main_handler_2);
					/* restore to normal handler */
    } /* found_something */

} /* DoExamineOneObject */



/* *******************************************************************
*
* DoExamineObjectsSub - 
*
* DESCRIPTION:
*
* HISTORY:
*	05-JUN-91 DSB	Created
*	07-JUN-91 DSB	Added LocalFileDeclInfo case; cleaned up NIY messages
*	07-JUN-91 DSB	WRITING A BUNCH OF GARBAGE CODE THAT I NEED TO CLEAN
*	11-JUN-91 DSB	Specifically reject large physical types
*
******************************************************************* */
void DoExamineObjectsSub(simple_name, current_region_data, current_region_info,
	any_names_p, any_objects_p)
StringN simple_name;
VK_tLARHeaderPtr current_region_data;
VK_tSymbInfoHandle current_region_info;
VK_tBoolean * any_names_p;
VK_tBoolean * any_objects_p;
{
    TempItem child_scan;
    VK_tSymbInfoHandle child;
/**/
	/* : scan region for object(s?) matching simple_name */
    for (child_scan=TempFirstItem(qqChildren(current_region_info)); 
		! TempNullItem(child_scan); 
		child_scan=TempNextItem(child_scan)) {
	child = SymbInfoValue(child_scan);

	if ((NULL != qqName(child)) && (
		StringMatch(simple_name, "*") ||
		StringMatch(simple_name, qqName(child)))) {
					/* if name matches (or wildcard) */
	    *any_names_p = True;

	    DoExamineOneObject(child, current_region_data, current_region_info,
		&*any_names_p, &*any_objects_p);

	}
	else {
		/* not displayable object, ignore currently */
	}
	
    } /* scan children */
} /* DoExamineObjectsSub */



/* *******************************************************************
*
* DoExamineObjects - display values of objects
*
* DESCRIPTION:
*
* HISTORY:
*	05-JUN-91 DSB	Created
*
******************************************************************* */
/* ?? not static because VK_TraceEnd calls it for dump mode?? */
void DoExamineObjects(simple_name)
StringN simple_name;
{
    VK_tLARHeaderPtr current_region_data;
    VK_tSymbInfoHandle current_region_info;

    VK_tBoolean any_names, any_objects;
    TempItem package_scan;

    tTempBuf256 buffer;
    VK_tIARHeaderPtr temp_iarp;
/**/
    GetLastTracepoint(&current_region_data);
					/* start at last-noted region */
	/* Note: eventually, separate starting region from last-noted region,
		to implement "up" and "down" */

    any_names = False;
    any_objects = False;

    /* ?? THIS IS EXPERIMENTAL CODE!  IT SHOULD BE REDESIGNED AND REWRITTEN,
	ESPECIALLY THE PACKAGE-SCAN STUFF ?? */

/* Traversals/Algorithm: 
	1a: traverse regions from last-traced region up context_LARP links
		to top architecture (or package body)
	1b: then traverse all packages

	2: scan all items in region
	3: display constant, variable, and signal objects; no files;
		scalar types only;  display object name and scalar value
*/


    while (NULL != current_region_data) {
	current_region_info = TK_GetRegionInfo(current_region_data);
	DoExamineObjectsSub(simple_name, current_region_data, 
		current_region_info, &any_names, &any_objects);
	if (NULL != qqExtReg(current_region_info)) {
	    if (PackageBodyInfo == qqSymbInfoKind(current_region_info)) {
		DoExamineObjectsSub(simple_name, 
			qqRegionData(qqExtReg(current_region_info)), 
			qqExtReg(current_region_info), 
			&any_names, &any_objects);
	    }
	    else {
		DoExamineObjectsSub(simple_name, current_region_data, 
			qqExtReg(current_region_info), 
			&any_names, &any_objects);
	    }
	}
		/* : go up to context region */
	temp_iarp = VK_GetIARP(current_region_data);
	current_region_data = (NULL == temp_iarp) ? NULL : 
		VK_GetContextLARP(temp_iarp);
					/* Note: going up context to display
					   all potentially-visible regions */
    } /* while */


    for (package_scan = TempFirstItem(SymbInfoList); 
		! TempNullItem(package_scan);
		package_scan = TempNextItem(package_scan)) {
	if (PackageDeclInfo == qqSymbInfoKind(SymbInfoValue(package_scan))) {
	    current_region_info = SymbInfoValue(package_scan);
	    DoExamineObjectsSub(simple_name, 
		qqRegionData(current_region_info), current_region_info,
		&any_names, &any_objects);
	}
    }

    if (! any_names) {
	sprintf(buffer, "No match found for '%s'", simple_name);
	DisplayResponse(buffer);
    }
    else if (! any_objects) {
	sprintf(buffer, "No object found for '%s'", simple_name);
	DisplayResponse(buffer);
    }
} /* DoExamineObjects */


/* *******************************************************************
*
* DoExamine -
*
* HISTORY:
*	03-JUN-91 DSB	Created
*
******************************************************************* */

VK_tBoolean DoExamine(token_buffer)
tTokenBuffer token_buffer;
{ 
    tToken token;
    tTempBuf80 value_buffer;
    tTokenSeparator tk_sp;

    tTempBuf80 tempbuf;
/**/
    SetTokenSeparator(tk_sp, NULL);
    NextToken(token_buffer, token, tk_sp);
    
    switch(TokenType(token)) {
      case TK_NoToken:
	DisplayResponse("Error: 'examine' requires object name or '*'");
	break;
	
      case TK_Identifier:
	TokenValue(token, value_buffer, sizeof(value_buffer));
	DoExamineObjects(value_buffer);
	break;

      case TK_Asterisk:
	DoExamineObjects("*");
	break;

      default:
	DisplayResponse("Error: unrecoginzed parameter");
	break;    
    } /* switch */
   
    return True;
} /* DoExamine */


/* *******************************************************************
*
* ParseBreak -
*
* HISTORY:
*	17-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  ParseBreak(token_buffer)
tTokenBuffer token_buffer;
{
/**/
     switch(ParseCmd(token_buffer, breakCmdTblId)) {
     	  case PS_NotCmdTable:
	      ReportInternalError("Bad parse table", "", __FILE__, __LINE__);
	      break;

	  case PS_CmdComplete:
 	    break;
	
          case PS_NotCmd:
       	    DisplayResponse("Error: unrecognized command");    
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;

	  case PS_NoNext:
	  case PS_AmbiguosCmd:
       	    DisplayResponse("Error: ambiguous command");
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;
	    
	  default:
	    ReportInternalError("Unknown status from parse", "", __FILE__, __LINE__);
	    break;
	} /* switch */

    return True;
} /* ParseBreak */


/* *******************************************************************
*
* ParseCancel -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  ParseCancel(token_buffer)
tTokenBuffer token_buffer;
{
/**/
     switch(ParseCmd(token_buffer, cancelCmdTblId)) {
     	  case PS_NotCmdTable:
	      ReportInternalError("Bad parse table", "", __FILE__, __LINE__);
	      break;

	  case PS_CmdComplete:
 	    break;
	
          case PS_NotCmd:
       	    DisplayResponse("Error: unrecognized command");    
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;

	  case PS_NoNext:
	  case PS_AmbiguosCmd:
       	    DisplayResponse("Error: ambiguous command");
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;
	    
	  default:
	    ReportInternalError("Unknown status from parse", "", __FILE__, __LINE__);
	    break;
	} /* switch */

    return True;
} /* ParseCancel */



/* *******************************************************************
*
* ParseSet -
*
* HISTORY:
*	25-JAN-91 AWH	Created
*	17-MAY-91 AWH	Changed from ParsePace
*
******************************************************************* */

static VK_tBoolean  ParseSet(token_buffer)
tTokenBuffer token_buffer;
{
/**/
     switch(ParseCmd(token_buffer, setCmdTblId)) {
     	  case PS_NotCmdTable:
	      ReportInternalError("Bad parse table","",  __FILE__, __LINE__);
	      break;

	  case PS_CmdComplete:
 	    break;
	
          case PS_NotCmd:
       	    DisplayResponse("Error: unrecognized command");    
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;

	  case PS_NoNext:
	  case PS_AmbiguosCmd:
       	    DisplayResponse("Error: ambiguous command");
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;
	    
	  default:
	    ReportInternalError("Unknown status from parse", "", __FILE__, __LINE__);
	    break;
	} /* switch */

    return True;
} /* ParseSet */



/* *******************************************************************
*
* ParseDump -
*
* HISTORY:
*	22-JUL-91 DSB	Created
*
******************************************************************* */

static VK_tBoolean  ParseDump(token_buffer)
tTokenBuffer token_buffer;
{
/**/
     switch(ParseCmd(token_buffer, dumpCmdTblId)) {
     	  case PS_NotCmdTable:
	      ReportInternalError("Bad parse table", "", __FILE__, __LINE__);
	      break;

	  case PS_CmdComplete:
 	    break;
	
          case PS_NotCmd:
       	    DisplayResponse("Error: unrecognized command");    
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;

	  case PS_NoNext:
	  case PS_AmbiguosCmd:
       	    DisplayResponse("Error: ambiguous command");
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;
	    
	  default:
	    ReportInternalError("Unknown status from parse", "", __FILE__, __LINE__);
	    break;
	} /* switch */
    return True;

} /* ParseDump */


    

/* *******************************************************************
*
* ParseShow -
*
* HISTORY:
*	18-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  ParseShow(token_buffer)
tTokenBuffer token_buffer;
{
/**/
     switch(ParseCmd(token_buffer, showCmdTblId)) {
     	  case PS_NotCmdTable:
	      ReportInternalError("Bad parse table", "", __FILE__, __LINE__);
	      break;

	  case PS_CmdComplete:
 	    break;
	
          case PS_NotCmd:
       	    DisplayResponse("Error: unrecognized command");    
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;

	  case PS_NoNext:
	  case PS_AmbiguosCmd:
       	    DisplayResponse("Error: ambiguous command");
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;
	    
	  default:
	    ReportInternalError("Unknown status from parse", "", __FILE__, __LINE__);
	    break;
	} /* switch */
    return True;

} /* ParseShow */


    

/* *******************************************************************
*
* DoUnimplemented -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */

static VK_tBoolean  DoUnimplemented()
{
/**/
   DisplayResponse("Unimplemented\n"); /* ?? fix this */   
    return True;
} /* DoUnimplemented */

    

/* *******************************************************************
*
* InitializeCommand -
*
* HISTORY:
*	16-JAN-91 AWH	Created
*
******************************************************************* */


void  InitializeCommand()
{
/**/
    TopCmdTblId = RegCmdTable(TopCmdTbl, TableSize(TopCmdTbl));
    breakCmdTblId = RegCmdTable(breakCmdTbl, TableSize(breakCmdTbl));
    setCmdTblId = RegCmdTable(setCmdTbl, TableSize(setCmdTbl));
    cancelCmdTblId = RegCmdTable(cancelCmdTbl, TableSize(cancelCmdTbl));
    showCmdTblId = RegCmdTable(showCmdTbl, TableSize(showCmdTbl));
    dumpCmdTblId = RegCmdTable(dumpCmdTbl, TableSize(dumpCmdTbl));
    

} /* InitializeCommand */


/* *******************************************************************
*
* GetCommands - get and process interactive commands
*
* HISTORY:
*	15-JUN-90 DSB	Created
*	18-JUN-90 DSB	Added tokenizing and prefix matching
*	09-NOV-90 AWH	Make it work with the demo
*	26-DEC-90 DSB	Added is_after_sim check for step command
*	16-JAN-91 AWH	Change to use CmdProcessor module
*
******************************************************************* */
void GetCommands()			/* return value desc. if not void */
{
    tTempBuf256 input_line;
    tTokenBuffer token_buffer;
    char * result;
/**/
    AnotherCommand = True;		/* until step, go, etc. */
    while (AnotherCommand) {
    	DisplayPrompt("Command> ");
	fflush(stdout);
    	result = fgets(input_line, sizeof(input_line), stdin);
	if (result == NULL) {
	    fprintf(stderr, "Error: Error reading command input\n");
	    exit(4);
	}

	SetTokenBuffer(token_buffer, input_line);

	switch(ParseCmd(token_buffer, TopCmdTblId)) {
	  case PS_NotCmdTable:
	    ReportInternalError("Bad parse table", "", __FILE__, __LINE__);
	    break;

	  case PS_CmdComplete:
	  case PS_NoNext:
 	    break;
	
          case PS_NotCmd:
       	    DisplayResponse("Error: unrecognized command");    
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;

	  case PS_AmbiguosCmd:
       	    DisplayResponse("Error: ambiguous command");
	    if (! GF.slave) {
		DoHelp();
	    }
	    break;
	    
	  default:
	    ReportInternalError("Unknown status from parse", "", __FILE__, __LINE__);
	    break;
	} /* switch */
    } /* while*/
} /* GetCommands */




#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */


<static> Fntype RtnName(parm)				/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif

/* TKCommand.c */


