static char SCCS_ID[] = "@(#)TKData.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKData.c
*
* DESCRIPTION:  
*	General operations on VKI data types
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	18-MAR-91 HK	Created to initialize data defined in TKData.h
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	14-MAY-91 DSB	W0060: Change resolution limit (BaseUnit) back to fs
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	06-JUN-91 DSB	Added CvtVHDLToCString
*	10-JUN-91 DSB	Removed CvtFromLargeTime, CvtToLargeTime; added math
*			functions for time
* 	15-OCT-91 MNP	Renamed iIntNtoInt4 and iCheckIntN to x... to
*			avoid the same names in SPI.
*       29-OCT-91 DKC   Changed above to z... to avoid clash with VLMath 
*	14-NOV-91 DKC	Extended MinTime8Const down 1 (full range)
*
* SCCS ID: @(#)TKData.c	42.1 1/10/92
*
******************************************************************* */

#include <ctype.h>		/* for character classifying routines */

#include "TKCommon.h"
#include "TKData.h"


/* *******************************************************************
*
*
*
******************************************************************* */
	/* The following ten variables are initialized in InitData */

/* extern */ tAbsTimeBuf TimeLow;		/* ?? remove this ?? */
/* extern */ tAbsTimeBuf ZeroTime;
/* extern */ tAbsTimeBuf MaxTime;

/* extern */ tRelTimeBuf UZeroTime;

/* ?? these are global, so some part of the name should be mixed case; e.g.,
	TimeUnitFactor_ps
?? */
/* extern */ tAbsTimeBuf tu_fs_factor;
/* extern */ tAbsTimeBuf tu_ps_factor;
/* extern */ tAbsTimeBuf tu_ns_factor;
/* extern */ tAbsTimeBuf tu_us_factor;
/* extern */ tAbsTimeBuf tu_ms_factor;
/* extern */ tAbsTimeBuf tu_sec_factor;
/* extern */ tAbsTimeBuf tu_min_factor;
/* extern */ tAbsTimeBuf tu_hr_factor;
 

/* *******************************************************************
*
* CvtVHDLToCString - <description of function or procedure>
*
* HISTORY:
*	06-JUN-91 DSB	Created
*
******************************************************************* */

void CvtVHDLToCString(c_buffer, buf_size, VHDL_string, length)
VK_tCString c_buffer;			/* &out: packed character buffer */
Integer4 buf_size;			/* in: size of character buffer */
	/* ?? which type should we be using for integers? ?? */
VK_tVHDLString VHDL_string;		/* &in: unpacked word buffer */
VK_tStringLength length;			/* in: length of word buffer */
{
    int i;
/**/
    for (i=0; i<=length-1; i++) {
	c_buffer[i] = VHDL_string[i];
    }
    c_buffer[i] = '\0';			/* terminate C string */
} /* CvtVHDLToCString */


/* ?? Note: We need to replace the following routines with the run-time
	library's newer modified version of these modified SPI functions. ?? */

#ifdef VK_OS_VMS
#  include <stdlib.h>		/* for strtol, HUGE_VAL */
#endif



#if MaxInt4 == 2147483647
#  define MaxInt4Div10 214748364 /* avoid run-time divide (if not optimized) */
#  define MinInt4Div10 -214748364
#else
#  define MaxInt4Div10 MaxInt4 / 10
#  define MinInt4Div10 MinInt4 / 10
#endif

/* *******************************************************************
*
*
*
******************************************************************* */

/* If you're looking for some typedefs, they moved to TKData.h MNP */

static VK_tBoolean Time_MathError = False;

#define ClearMathStatus() (Time_MathError = False)
#define SetMathStatus(a, b) (Time_MathError = True)
#define MATHERROR Time_MathError

#define MinTime8	iMinTime8()
#define MaxTime8	iMaxTime8()

#define Time8BAdd	iTime8BAdd
#define Time8BSub	iTime8BSub

#define EnableMathErrors	iEnableMathErrors
#define Time8BZero	iTime8BZero
#define Time8IsZero	iTime8IsZero
#define DisableMathErrors	iDisableMathErrors
#define Time8IsNeg	iTime8IsNeg
#define Time8Compare	iTime8Compare
#define Int4ToTime8B	iInt4ToTime8B

/* These two have i... names in the SPI, the RTL has x... versions */
#define IntNToInt4	zIntNToInt4
#define CheckIntN	zCheckIntN

static Integer4 iGetExponent();
#define GetExponent(strn)	iGetExponent(strn)

static VK_tBoolean iTime8BMult10();
#define Time8BMult10(buf_addr, mcand) \
	iTime8BMult10(buf_addr, mcand)

static Integer4 iTime8BDivMod10();
#define Time8BDivMod10(buf_addr)	iTime8BDivMod10(buf_addr)

#define IntNToTime8B(buffer, IN) iIntNToTime8B(buffer, IN);

#define Time8ToIntNB iTime8ToIntNB 

/* *******************************************************************
*
* FourShort and tInteger8-related constants and types
*
******************************************************************* */


#define ShortCount 4			/* number of short words per tInteger8 */
#define ShortIndex (ShortCount-1)
#define ShortBits 16			/* number of bits per short word */
#define ShortShift (ShortBits-1)	/* for shifting high bit down */
#define ShortMax 65535			/* maximum positive short (if signed) */
#define ShortMaxP1 65536		/* unsigned rep. of (signed short) -1 */
#define ShortMask 0x0000FFFF		/* low short portion of long int */
#define ShortHighMask 0xFFFF0000	/* high short portion of long int */

typedef unsigned short (FourShort)[ShortCount];
typedef FourShort FourShortPtr;



#ifdef VK_LITTLE_ENDIAN
#define R4(i) (i)		/* don't reverse index; for four-element array */
#define R2(i) (i)		/* don't reverse index; for two-element array */
#else
#ifdef VK_BIG_ENDIAN
#define R4(i) (3-(i))		/* reverse index; for four-element array */
#define R2(i) (1-(i))		/* reverse index; for two-element array */
#else
  #error
#endif
#endif

#define IndexI8AsFS(Time8, ix) ((*(FourShort *)(Time8))[R4(ix)])
				/* to index tInteger8 value (with host packing)
				   as FourShort with x[0] being low-order
				   short word */


/* *******************************************************************
*
* tInteger8 value constants
*
******************************************************************* */

#ifdef VK_LITTLE_ENDIAN
    static const tLocalInteger8 MaxTime8Const = {0xFFFFFFFF, 0x7FFFFFFF};
    static const tLocalInteger8 MinTime8Const = {0x00000000, 0x80000000};
#else
#ifdef VK_BIG_ENDIAN
    static const tLocalInteger8 MaxTime8Const = {0x7FFFFFFF, 0xFFFFFFFF};
    static const tLocalInteger8 MinTime8Const = {0x80000000, 0x00000000};
#else
  #error
#endif
#endif


/* *******************************************************************
*
* Time8BZero - return null tInteger8 value (zero)
*
* HISTORY:
*	22-DEC-87 DSB	Created
*
******************************************************************* */
tInteger8 iTime8BZero(buf_addr)
tInteger8 buf_addr;
{
/**/
    buf_addr->high = 0;
    buf_addr->low = 0;
    return buf_addr;
} /* iTime8BZero */


/*********************************************************************
*
* iTimeAdd1 - add positive relative time to absolute simulation time yielding
*	absolute scheduled time
*
* NOTE: No overflow check is needed
*
* HISTORY:
*	10-JUN-91 DSB	Derived from from SPI's iTime8BAdd
*
*********************************************************************/
void iTimeAdd1(sched_time_addr, abs_time_addr, rel_time_addr)
tInteger8U sched_time_addr;		/* full-range unsigned sim. time */
tInteger8U abs_time_addr;		/* lower-half unsigned sim. time */
tInteger8 rel_time_addr;		/* positive signed relative time */
{
    Integer4 carry;
    tLocalInteger8U sum;
/**/
/*    VERIFY_ERROR(! STIME_NEGATIVE(*rel_time_addr)); */
/*    VERIFY_ERROR(! STIME_LESSTHAN(MaxTime, *abs_time_addr)); */

    sum[0].low = abs_time_addr->low + rel_time_addr->low;
					/* Note: we are allowing unsigned long
					   int to overflow */

    if (sum[0].low < abs_time_addr->low) {
	carry = 1;
    }
    else {
	carry = 0;
    }

    sum[0].high = abs_time_addr->high + rel_time_addr->high + carry;

	/* no overflow check */

    sched_time_addr->high = sum[0].high;
    sched_time_addr->low = sum[0].low;

} /* iTimeAdd1 */


/*********************************************************************
*
* tTimeSub1 - subtract simulation time from (scheduled) simulation time
*	yielding positive relative time
*
* HISTORY:
*	10-JUN-91 DSB	Derived from from SPI's iTime8BSub
*
*********************************************************************/
void iTimeSub1(rel_time_addr, left, right)
tInteger8 rel_time_addr;
const tInteger8U left;
tInteger8U right;
{
    Integer4 borrow;
    tLocalInteger8 diff;
/**/
    diff[0].low = left->low - right->low;
					/* Note: we are allowing unsigned long
					   int to overflow */

    if (diff[0].low > left->low) {
	borrow = 1;
    }
    else {
	borrow = 0;
    }

    diff[0].high = left->high - right->high - borrow;

	/* no overflow check */

    rel_time_addr->high = diff[0].high;
    rel_time_addr->low = diff[0].low;

} /* iTimeSub1 */



/* *******************************************************************
*
* MaxTime8 -
*
* HISTORY:
*	15-DEC-88 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */

const tInteger8 iMaxTime8()		/* returns maximum tInteger8 value */
{
/**/
    return (const tInteger8)MaxTime8Const;
} /* iMaxTime8 */


/* *******************************************************************
*
* MinTime8 -
*
* HISTORY:
*	15-DEC-88 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */

const tInteger8 iMinTime8()		/* returns minimum tInteger8 value */
{
/**/
    return (const tInteger8)MinTime8Const;
} /* iMinTime8 */


/* *******************************************************************
*
* Time8IsNeg - test whether tInteger8 value is negative
*
* HISTORY:
*	22-DEC-87 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************** */
VK_tBoolean iTime8IsNeg(Time8)	/* was Boolean1-why? */
tInteger8 Time8;
{
/**/
    return ((long int) Time8->high) < 0;	/* consider high word (signed) */
} /* iTime8IsNeg*/



/* *******************************************************************
*
* Time8Compare -
*
* HISTORY:
*	04-FEB-88 DSB	Created
*	05-JAN-89 DSB	Fixed >128 to >=128
*	12-JUN-89 PBG	Fixed problems with negative numbers
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */
NumericOrder iTime8Compare(left, right)
tInteger8 left;
tInteger8 right;
{
/**/
    if (left->high != right->high) {
	if (left->high < right->high) {
	    return LessThan;
	}
	else {
	    return GreaterThan;
	}
    }

    if (left->low != right->low) {
	if (left->low < right->low) {
	    return LessThan;
	}
	else {
	    return GreaterThan;
	}
    }

    return EqualTo;

} /* iTime8Compare */



/* *******************************************************************
*
* Time8IsZero -
*
* HISTORY:
*	22-DEC-87 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */
VK_tBoolean iTime8IsZero(Time8) 	/* was Boolean1-why? */
tInteger8 Time8;
{
/**/
    return Time8->low == 0 && Time8->high == 0;
} /* iTime8IsZero */


/*********************************************************************
*
* Time8BAdd - add tInteger8 values
*
* HISTORY:
*	22-DEC-87 DSB	Created
*	21-JAN-88 DSB	Rewritten in C only (no VAX calls)
*	12-JAN-90 PBG	Only two adds now
*	10-JUN-91 DSB	Copied from SPI
*
*********************************************************************/
VK_tBoolean iTime8BAdd(buf_addr, left, right)
tInteger8 buf_addr;
tInteger8 left;
tInteger8 right;
{
    VK_tBoolean okay;
    tLocalInteger8 sum;
    tInteger8 sumP;
    Integer4 carry;
/**/
    sumP = sum;

    /* In this algorithm, we are intentionally allowing long unsigned 	*/
    /* overflow to occur.  Will this cause trouble on any machines???	*/

    sumP->low = left->low + right->low;

    if (sumP->low < left->low) {
	carry = 1;
    }
    else {
	carry = 0;
    }

    sumP->high = left->high + right->high + carry;

    /* Overflow rules in addition:					*/
    /* 1) non-negative plus non-negative is ok iff result is 		*/
    /*		non-negative						*/
    /* 2) negative plus negative is ok iff result is negative		*/
    /* 3) non-negative plus negative is always ok			*/
    /* 4) negative plus non-negative is always ok			*/
    /* 5) negative plus negative may overflow if tInteger8 bounds are	*/
    /* 		symmetric (may overflow to -2**63)			*/

    okay = True;
    if (left->high >= 0 && right->high >= 0) {
	if (sumP->high >= 0) {
	    /* ok */
	}
	else {
	    SetMathStatus(Math_Overflow, "Time8BAdd");
	    okay = False;
	}
    }
    else if (left->high < 0 && right->high < 0) {
	if (sumP->high < 0) {
	    if (Time8Compare(sum, MinTime8) == LessThan) {
		SetMathStatus(Math_Overflow, "Time8BAdd");
	    }
	    else {
		/* ok */
	    }
	}
	else {
	    SetMathStatus(Math_Overflow, "Time8BAdd");
	    okay = False;
	}
    }
    else if (left->high >= 0 && right->high < 0) {
	/* ok */
    }
    else if (left->high < 0 && right->high >= 0) {
	/* ok */
    }

    /* Since buf_addr can be the same as left or right, don't set it 	*/
    /* until after testing left and right.				*/

    buf_addr->low = sumP->low;
    buf_addr->high = sumP->high;

    return okay;
} /* iTime8BAdd */



/*********************************************************************
*
* Time8BSub - subtract tInteger8 values
*
* HISTORY:
*	22-DEC-87 DSB	Created
*	21-JAN-88 DSB	Rewritten in C only (no VAX calls)
*	12-JAN-90 PBG	Only two subtracts now
*	07-MAR-91 PBG	Made left a const, to allow ZeroTime 
*	10-JUN-91 DSB	Copied from SPI
*
*********************************************************************/
VK_tBoolean iTime8BSub(buf_addr, left, right)
tInteger8 buf_addr;
const tInteger8 left;
tInteger8 right;
{
    VK_tBoolean okay;
    tLocalInteger8 diff;
    tInteger8 diffP;
    Integer4 borrow;
/**/
    ClearMathStatus(); 			/* until not */
    diffP = diff;

    diffP->low = left->low - right->low;

    if (diffP->low > left->low) {
	borrow = 1;
    }
    else {
	borrow = 0;
    }

    diffP->high = left->high - right->high - borrow;

    /* Overflow rules in subtraction:					*/
    /* 1) non-negative minus non-negative is always ok			*/
    /* 2) negative minus negative is always ok				*/
    /* 3) non-negative minus negative is ok iff result is non-negative	*/
    /* 4) negative minus non-negative is ok iff result is negative	*/
    /* 5) negative minus positive may overflow if tInteger8 bounds are	*/
    /* 		symmetric (may overflow to -2**63)			*/

    if (left->high >= 0 && right->high >= 0) {
	/* ok */
    }
    else if (left->high < 0 && right->high < 0) {
	/* ok */
    }
    else if (left->high >= 0 && right->high < 0) {
	if (diffP->high >= 0) {
	    /* ok */
	}
	else {
	    SetMathStatus(Math_Overflow, "Time8BSub");
	    okay = False;
	}
    }
    else if (left->high < 0 && right->high >= 0) {
	if (diffP->high < 0) {
	    if (Time8Compare(diff, MinTime8) == LessThan) {
		SetMathStatus(Math_Overflow, "Time8BSub");
		okay = False;
	    }
	    else {
		/* ok */
	    }
	}
	else {
	    SetMathStatus(Math_Overflow, "Time8BSub");
	    okay = False;
	}
    }

    /* Since buf_addr can be the same as left or right, don't set it 	*/
    /* until after testing left and right.				*/

    buf_addr->low = diffP->low;
    buf_addr->high = diffP->high;

    return okay;
} /* iTime8BSub */



/*********************************************************************
*
* Time8BDivMod10 - divide tInteger8 value by ten, returning remainder
*
* NOTE: Intended for use with Time8ToStrN routine
*
* HISTORY:
*	10-FEB-88 DSB	Created
*	12-APR-88 DSB	Optimizing
*	10-JUN-91 DSB	Copied from SPI
*
*********************************************************************/

static Integer4 iTime8BDivMod10(buf_addr)		/* */
tInteger8 buf_addr; 			/* number to be divided by ten */
{
    Boolean1 negative; 			/* for remember negative quotient */
    unsigned long int temp2;
    unsigned long int remainder;
    VK_tBoolean okay;
/**/
    ClearMathStatus(); 			/* until not */

    negative = False; 			/* */
    if (Time8IsNeg(buf_addr)) {
	negative = True;
	okay = Time8BSub(buf_addr, (tInteger8)&ZeroTime, buf_addr);
        if (! okay) {
	    return -1;
	}
    }

    temp2 = 	    		       IndexI8AsFS(buf_addr, 3);
    IndexI8AsFS(buf_addr, 3) = temp2 / 10;
    remainder = temp2 % 10;

    temp2 = (remainder << ShortBits) + IndexI8AsFS(buf_addr, 2);
    IndexI8AsFS(buf_addr, 2) = temp2 / 10;
    remainder = temp2 % 10;

    temp2 = (remainder << ShortBits) + IndexI8AsFS(buf_addr, 1);
    IndexI8AsFS(buf_addr, 1) = temp2 / 10;
    remainder = temp2 % 10;

    temp2 = (remainder << ShortBits) + IndexI8AsFS(buf_addr, 0);
    IndexI8AsFS(buf_addr, 0) = temp2 / 10;
    remainder = temp2 % 10;


    if (negative) {
	okay = Time8BSub(buf_addr, (tInteger8)&ZeroTime, buf_addr);
	if (! okay) {
	    return -1;
    	}
    }

    return remainder;

} /* iTime8BDivMod10 */



/* *******************************************************************
*
* Time8ToIntNB - convert signed tInteger8 value to string value
*
* ALGORITHM:
*	This routine fills in a buffer from the right to the left,
*	returning the address of the first digit in the buffer.
*	This saves having to shift or swap digits to return
*	the starting address of the whole buffer.
*
* HISTORY:
*	10-JUN-91 DSB	??
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */
tIntegerN iTime8ToIntNB(userbuf, Time8val)
tIntegerN userbuf; 		/* in: buffer in which to put string */
tInteger8 Time8val; 		/* in: tInteger8 value to convert */
{
#define Time8StrLen 21		/* sign+'\0'+~19 digits */
    static char tempbuf[Time8StrLen];
    tLocalInteger8 cur_val;
    Boolean1 negative;
    Integer4 ox; 			/* output buffer index */
/**/
    ox = sizeof(tempbuf)-1; 		/* point to end of buffer */
    tempbuf[ox] = '\0'; 		/* terminate string */

    TIME_COPY(cur_val[0], *Time8val);
    negative = Time8IsNeg(cur_val);
    if (negative) {
	if (Time8Compare(cur_val, MinTime8) == EqualTo) {
	    strcpy(userbuf, "-9223372036854775808");
					/* only if (-MaxTime8)-1 */
					/* ?? DO THIS RIGHT ?? */
	    return userbuf;
	}
	else {
	    Time8BSub(cur_val, (tInteger8)&ZeroTime, cur_val);
					/* negate negative input */
	}
    }
    do {		/* do..while won't leave null string for zero */
        ox--;
	tempbuf[ox] = (char) ('0' + Time8BDivMod10(cur_val));
    } while (! Time8IsZero(cur_val));

    if (negative) {
	ox--;
	tempbuf[ox] = '-'; 		/* put in negative sign if negative */
    }
    strcpy(userbuf, &tempbuf[ox]);
    return userbuf;
} /* iTime8ToIntNB */



/* *******************************************************************
*
* CheckIntN -
*
* HISTORY:
*	30-AUG-88 DSB	Created
*	01-SEP-88 DSB	Added reporterror flag and function
*	10-JUN-91 DSB	Copied from SPI
* 	15-OCT-91 MNP	Renamed from i... to x...
*	29-OCT-91 DKC	Renamed x... to z...
*
******************************************************************* */

VK_tBoolean zCheckIntN(intn, reporterror)		/* was Boolean1 */
tIntegerN intn; 				/* in: ... */
VK_tBoolean reporterror;		/* was Boolean1 */
{
    Boolean1 result;
    Integer4 i;
/**/
    result = True; 			/* until found to be False */

    i = 0;
    if (intn[i] == '+') {		/* allow optional "+" sign */
	i = i + 1;
    }
    else if (intn[i] == '-') {		/* or optional "-" sign */
	i = i + 1;
    }
    if (! isdigit(intn[i])) {		/* require at least one digit */
	if (reporterror) {
/*  ??
	    SetErrorStatus(SPI_MathSyntax, "tIntegerN", "no digits", "digits",
		"CheckIntN");
?? */
	}
	result = False;
    }
    else {
	while (isdigit(intn[i])) {	/* allow any number of digits */
	    i = i + 1;
	}
	if (intn[i] == 'e' || intn[i] == 'E') {	/* allow positive exponent */
	    i = i + 1;
	    if (intn[i] == '+') {	/* allow optional "+" on exponent */
		i = i + 1;
	    }
	    if (! isdigit(intn[i])) {	/* require digit after "E" or "e" */
		if (reporterror) {
/* ??
		    SetErrorStatus(SPI_MathSyntax, "tIntegerN",
			&intn[i], "digits after 'E'", "CheckIntN");
*/
		}
		result = False;
	    }
	    else {
		while (isdigit(intn[i])) {
		    i = i + 1;
		}
	    }

	}
    	if (intn[i] != '\0') {		/* disallow junk after valid number */
	    if (reporterror) {
/* ??
	    	SetErrorStatus(SPI_MathSyntax, "tIntegerN", &intn[i],
			"end of string",
			"CheckIntN");
*/
	    }
	    result = False;
    	}
    }
    return result;
} /* xCheckIntN */

#ifndef VK_OS_VMS

/* *******************************************************************
*
* MyStrToL - <description of function or procedure>
*
* HISTORY:
*	03-MAY-89 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */

#define MyStrToL(instr, outval, endptr, overflow) \
	iMyStrToL(instr, &(outval), &(endptr), &(overflow))
static void iMyStrToL(instr, outval, endptr, overflow)
char * instr;				/* (&)in: input string */
long int * outval;			/* &out: output value */
char * * endptr;			/* &out: ending string ptr. */
Boolean1 * overflow;				/* &out: boolean overflow flag */
{
    register char * scan;		/* string scan pointer */
    register int digit;			/* digit value from string */
    register long int accum;		/* value accumulator */
    register long int temp;		/* temp. value */
    int neg;				/* boolean negative flag */
    register int error;			/* boolean overflow flag */
/**/
    scan = instr;
    accum = 0;
    error = FALSE;

    if (*scan == '-') {			/* if negative sign */
	neg = TRUE;
	scan++;
    }
    else if (*scan == '+') {		/* if positive sign */
	neg = FALSE;
	scan++;
    }
    else {				/* else no sign */
	neg = FALSE;
    }
    if (neg) {				/* if negative, ... */
	while (isdigit(*scan) && ! error) {
	    if (accum < MinInt4Div10) {
		error = TRUE;
	    }
	    else {
		temp = accum * 10;
		digit = *scan - '0';
		if (temp < MinInt4 + digit) {
		    error = TRUE;
		}
		else {
		    accum = temp - digit;
		    scan++;
		}
	    }
	}
    }
    else {
	while (isdigit(*scan) && ! error) {	/* stop at nondigit or overflow */
	    if (accum > MaxInt4Div10) {	/* check *10 overflow */
		error = TRUE;
	    }
	    else {
		temp = accum * 10; 	/* check +digit overflow */
		digit = *scan - '0';
		if (temp > MaxInt4 - digit) {
		    error = TRUE;
		}
		else {
		    accum = temp + digit;
		    scan++;
		}
	    }
	}
    }

    *outval = accum;
    *overflow = error;
    if (endptr != NULL) {
	*endptr = scan;
    }
} /* iMyStrToL */
#endif


/* *******************************************************************
*
* IntNToInt4 - convert string value to Integer4 value
*
* HISTORY:
*	01-SEP-88 DSB	Added handling of exponents
*	26-SEP-88 DSB	Renamed from IntNToInt4 to StrNToInt4
*	10-JUN-91 DSB	Copied from SPI
* 	15-OCT-91 MNP	Renamed from i..x to x...
*	29-OCT-91 DKC	Changed x... to z...
*
******************************************************************* */

Integer4 zIntNToInt4(intn)
tIntegerN intn;
{
    Integer4	int4val;
    char * 	endptr; 		/* non-numeric char. from strtol */
    Integer4	exp, i;
    Boolean1    overflow;
/**/
    if (! CheckIntN(intn, True)) {
	return 0;
    }
#ifdef VK_OS_VMS
    errno = 0;
    int4val = strtol(intn, &endptr, 10); 	/* 10=base */
    if (errno == ERANGE) {
#else
    MyStrToL(intn, int4val, endptr, overflow);
    if (overflow) {
#endif
        SetMathStatus(Math_Overflow, "IntNToInt4");
	return int4val; 		/* VK_OS_VMS strtol yields LONG_MAX or LONG_MIN */
    }
    else {
    					/* allow positive exponent */
	exp = GetExponent(intn);

	for (i=1; i<=exp; i++) {	/* ?? this is not optimized ?? */
	    if (int4val > MaxInt4Div10) {
		SetMathStatus(Math_Overflow, "IntNToInt4");
		return 0;
	    }
	    else {
	    	int4val = int4val * 10;
	    }
	}
	return int4val;
    }
} /* xIntNToInt4 */


/* *******************************************************************
*
* GetExponent -
*
* HISTORY:
*	31-AUG-88 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */

static Integer4 iGetExponent(strn)		/* returns value of exponent */
StringN strn;
{
    Integer4 epos;
    Integer4 exp;
/**/
    epos = strcspn(strn, "Ee"); 		/* see if there is exponent */
    if (epos == strlen(strn)) {		/* no match -> strlen() */
	exp = 0; 				/* no exponent; use zero */
    }
    else {
	exp = IntNToInt4(&strn[epos+1]); 	/* get exponent as number */
    }
    return exp;
} /* iGetExponent */



/*********************************************************************
*
* Time8BMult10 - multiply tInteger8 value by 10
*
* HISTORY:
*	15-DEC-88 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
*********************************************************************/

static VK_tBoolean iTime8BMult10(buf_addr, mcand)		/* */
tInteger8 buf_addr;
tInteger8 mcand;
{
    VK_tBoolean okay;
    tLocalInteger8 twobuf, eightbuf;
/**/
    ClearMathStatus();

    okay = Time8BAdd(twobuf, mcand, mcand);
					/* twobuf = 2 * in */
    if (okay) {
	okay = Time8BAdd(eightbuf, twobuf, twobuf);
					/* eightbuf = 4 * in */
	if (okay) {
	    okay = Time8BAdd(eightbuf, eightbuf, eightbuf);
					/* eightbuf = 8 * in */
	    if (okay) {
		okay = Time8BAdd(buf_addr, eightbuf, twobuf);
					/* buf = (8 + 2) * in */
	    }
	}
    }
    return okay;
} /* iTime8BMult10 */




/* *******************************************************************
*
* Int4ToTime8B - convert Integer4 value to tInteger8 value
*
* HISTORY:
*	22-DEC-87 DSB	Created
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */
tInteger8 iInt4ToTime8B(Time8buf, int4)
tInteger8 Time8buf;
Integer4 int4;
{
/**/
    ClearMathStatus();

    Time8buf->low = int4;

    if (int4 < 0) {			/* handle sign extension: */
	Time8buf->high = -1;
    }
    else {
	Time8buf->high = 0;
    }
    return Time8buf;
} /* iInt4ToTime8B */



/* *******************************************************************
*
* IntNToTime8B - convert string value to tInteger8 value
*
* HISTORY:
*	01-SEP-88 DSB	Added handling of exponents
*	15-DEC-88 DSB	Switch from Time8BMult to Time8BMult10
*	10-JUN-91 DSB	Copied from SPI
*
******************************************************************* */
VK_tBoolean iIntNToTime8B(buf_addr, intn)
tInteger8 buf_addr;
tIntegerN intn;
{
    VK_tBoolean okay;
    tLocalInteger8	Zero, val_buf;
    Integer4	pos;
    Boolean1	neg;
    Integer4	exp, i;
/**/
    if (! CheckIntN(intn, True)) {
	Time8BZero(buf_addr);
	return False;
    }

    Time8BZero(buf_addr); 		/* initialize output*/

    Time8BZero(Zero); 			/* assign constant 0 */
    pos = 0;

    neg = False; 			/* unless '-' */
    if (intn[pos] == '-') {
	neg = True;
	pos = pos+1;
    }
    else if (intn[pos] == '+') {
	pos = pos+1;
    }

    while (isdigit(intn[pos])) {
	okay = Time8BMult10(buf_addr, buf_addr);
	if (! okay) {
	    Time8BZero(buf_addr);
	    return False;
	}
	Time8BAdd(buf_addr, buf_addr, Int4ToTime8B(val_buf, intn[pos]-'0'));
	if (! okay) {
	    Time8BZero(buf_addr);
	    return False;
	}
	pos = pos+1;
    }
    /* allow positive exponent */
    exp = GetExponent(intn);

#ifdef SPI_TRYEXP
    if (exp != 0) {
	Time8BExp(toexp, ten, exp);
	okay = Time8BMult(buf_addr, buf_addr, toexp);	/* shift by exponent */
    }
#else
    for (i=1; i<=exp; i++) {	/* ?? this is not optimized ?? */
	okay = Time8BMult10(buf_addr, buf_addr);
	if (! okay) {
	    Time8BZero(buf_addr);
	    return False;
	}
    }
#endif

    if (neg) {
	okay = Time8BSub(buf_addr, Zero, buf_addr); 	/* negate result */
	if (! okay) {
	    Time8BZero(buf_addr);
	    return False;
	}
    }

    return True;
} /* iIntNToTime8B */



/*********************************************************************
*
* Time8BMult - multiply tInteger8 values
*
* HISTORY:
*	22-DEC-87 DSB	Created
*	06-JUN-89 PBG	Optimized
*	10-JUN-91 DSB	Copied from SPI
*
*********************************************************************/

VK_tBoolean iTime8BMult(buf_addr, mpyer, mcand)
tInteger8 buf_addr;
tInteger8 mpyer;
tInteger8 mcand;
{
    VK_tBoolean okay;
    tLocalInteger8 mcand_work,
		mpyer_work,
		prod_work; 		/* so Time8BMult(A, A, A) works */
    unsigned long int prod, sum, carry, prodlow, prodhigh;
    Integer4 ax, bx, cx; 		/* loop indices */
    Boolean1 negative;
/**/

    ClearMathStatus(); 			/* until not */
    mcand_work[0].high = mcand->high;
    mcand_work[0].low = mcand->low;	/* copy multiplicand to work area */
    mpyer_work[0].high = mpyer->high;
    mpyer_work[0].low = mpyer->low;	/* copy multiplier to work area */

    Time8BZero(prod_work);
    negative = False; 			/* */
    if (Time8IsNeg(mpyer_work)) {
	negative = ! negative;
	okay = Time8BSub(mpyer_work, (tInteger8)&ZeroTime, mpyer_work);
        if (! okay) {
	    return False;
	}
    }
    if (Time8IsNeg(mcand_work)) {
	negative = ! negative;
	okay = Time8BSub(mcand_work, (tInteger8)&ZeroTime, mcand_work);
        if (! okay) {
	    return False;
	}
    }

    /* use host machine to multiply pairs of shorts;
       then add these partial sums together */

    /*    a3   a2   a1   a0
          b3   b2   b1   b0
       sum is
                       a0b0
                  a0b1
       	          a1b0
             a0b2
             a1b1
             a2b0
        a0b3
        a1b2
        a2b1
        a3b0

	terms such as a3b1, etc., must be zero or there is an overflow */


    /* First test for overflow due to the a3b1, etc. terms being non-	*/
    /* zero.  Testing for overflow here means that the multiplication	*/
    /* loop is somewhat simpler.					*/

    for (ax = 0; ax <= ShortIndex; ax++) {
	for (bx = ShortIndex - ax + 1; bx <= ShortIndex; bx++) {
	    if (IndexI8AsFS(mcand_work, ax) != 0 &&
		IndexI8AsFS(mpyer_work, bx) != 0) {

		SetMathStatus(Math_Overflow, "Time8BMult");
		return False;
	    }
	}
    }

    /* Now do the multiplication.					*/

    for (ax = 0; ax <= ShortIndex; ax++) {
	for (bx = 0; bx <= ShortIndex - ax; bx++) {
	    prod = IndexI8AsFS(mcand_work, ax) * IndexI8AsFS(mpyer_work, bx);

	    /* We don't have to do any work if prod == 0.		*/

	    if (prod != 0) {

		prodlow = prod & ShortMask;
		prodhigh = (prod & ShortHighMask) >> ShortBits;

		/* Now add the partial product into the result, 	*/
		/* propagating any carry.  If the carry propagates 	*/
		/* beyond the end of the product work space, we have an */
		/* overflow.						*/

		cx = ax + bx;
		sum = IndexI8AsFS(prod_work, cx) + prodlow;
		carry = (sum >> ShortBits) & 0x01;
		IndexI8AsFS(prod_work, cx) = sum & ShortMask;

		if (prodhigh != 0 ||
		    carry != 0) {
		    if (cx < ShortIndex) {

			cx++;
			sum = IndexI8AsFS(prod_work, cx) + prodhigh + carry;
			carry = (sum >> ShortBits) & 0x01;
			IndexI8AsFS(prod_work, cx) = sum & ShortMask;
		    }
		    else {
			SetMathStatus(Math_Overflow, "Time8BMult");
			return False;
		    }
		}

		while (carry != 0 &&
		       cx < ShortIndex) {

		    cx++;
		    sum = IndexI8AsFS(prod_work, cx) + carry;
		    carry = (sum >> ShortBits) & 0x01;
		    IndexI8AsFS(prod_work, cx) = sum & ShortMask;
		}

		if (carry != 0) {
		    SetMathStatus(Math_Overflow, "Time8BMult");
		    return False;
		}
	    }
	}
    }

    if (Time8IsNeg(prod_work)) {
	SetMathStatus(Math_Overflow, "Time8BMult");
	Time8BZero(buf_addr);
	return False;
    }

    if (negative) {
	okay = Time8BSub(prod_work, (tInteger8)&ZeroTime, prod_work);

	/* Here we catch the result overflowing to -2**63, which	*/
	/* is an overflow if the range of tInteger8 is symmetric.	*/

    	if (! okay) {
	    return False;
    	}
    }

    TIME_COPY(*buf_addr, prod_work[0]);
    return True;
} /* iTime8BMult */




/*********************************************************************
*
* Time8BDiv - divide tInteger8 values for quotient.
*
* NOTE:  The arithmetic in this routine is unsigned, since the result of
* multiplying a short by a short is not always a positive signed long.
* This algorithm is based on the routine InternalIntNDiv in MathIntN.
*
* HISTORY:
*	22-DEC-87 DSB	Created
*	07-JUN-89 PBG	Use algorithm from IntNDiv, break out
*			remainder and modulus to separate routines
*	10-JUN-91 DSB	Copied from SPI
*
*********************************************************************/
VK_tBoolean iTime8BDiv(buf_addr, dividend, divisor)
tInteger8 buf_addr;
tInteger8 dividend;
tInteger8 divisor;
{
    VK_tBoolean okay;
    tLocalInteger8 divisor_work; 	/* (positive) copy of divisor */
    tLocalInteger8 dividend_work;	/* (positive) copy of dividend */
    Integer4 i, j;
    Boolean1 negative; 			/* to remember negative quotient */
    long int dividend_vector[ShortCount + 1];
    long int divisor_vector[ShortCount];
    Integer4 dividend_size;
    Integer4 divisor_size;
    Integer4 quotient_size;
    static long int Time8_base = 1 << ShortBits;
    long int scale;
    unsigned long int utemp, digit, bigdigit, estimate;
    long int stemp, carry;
/**/
    Time8BZero(buf_addr); 		/* initialize output */

    ClearMathStatus();

    if (Time8IsZero(divisor)) {
	SetMathStatus(Math_DivideZero, "Time8BDiv");
	return False;
    }

    TIME_COPY(divisor_work[0], *divisor);   /* copy divisor to work area */
    TIME_COPY(dividend_work[0], *dividend); /* copy dividend to work area */

    negative = False;

    if (Time8IsNeg(dividend_work)) {
	negative = ! negative;
 	okay = Time8BSub(dividend_work, (tInteger8)&ZeroTime, dividend_work);
        if (! okay) {
	    Time8BZero(buf_addr);
	    return False;
	}
    }

    if (Time8IsNeg(divisor_work)) {
	negative = ! negative;
    	okay = Time8BSub(divisor_work, (tInteger8)&ZeroTime, divisor_work);
        if (! okay) {
	    Time8BZero(buf_addr);
	    return False;
	}
    }

    dividend_size = ShortCount;
    while (dividend_size >= 0 &&
	   IndexI8AsFS(dividend_work, dividend_size - 1) == 0) {
	dividend_size--;
    }

    divisor_size = ShortCount;
    while (divisor_size > 0 &&
	   IndexI8AsFS(divisor_work, divisor_size - 1) == 0) {
	divisor_size--;
    }

    /* Three cases:  							*/
    /* 	(1) The divisor is obviously bigger than the dividend.  The 	*/
    /* quotient is zero.						*/
    /* 	(2) The divisor is really a short.  Do an optimized divide in 	*/
    /* this case.							*/
    /* 	(3) The general case.  Scale the divisor and dividend such that	*/
    /* the high order digit of the divisor is at least Time8_base / 2. 	*/
    /* Then do the division, computing one short quotient digit per 	*/
    /* loop.								*/

    if (divisor_size > dividend_size) {
	Time8BZero(buf_addr);
    }
    else if (divisor_size == 1) {
	digit = IndexI8AsFS(divisor_work, 0);
	utemp = 0;
	for (i = ShortCount - 1; i >= 0; i--) {
	    bigdigit = Time8_base * utemp + IndexI8AsFS(dividend_work, i);
	    IndexI8AsFS(buf_addr, i) = bigdigit / digit;
	    utemp = bigdigit % digit;
	}
    }
    else {

	/* Two steps here. First, copy the divisor and dividend into 	*/
	/* their Integer4 vectors.  This allows us to treat the four  	*/
	/* (short) digits as positive, base 2**16.  Note that these 	*/
	/* vectors have [0] as the high order component, while the 	*/
	/* tInteger8s have [0] as the low order component.  This is to	*/
	/* be consistent with the algorithm in IntNDiv, upon which this	*/
	/* routine is modeled.  Second, multiply the dividend and 	*/
	/* divisor by a scaling factor to ensure that the high order 	*/
	/* digit of the divisor is at least (2**16) / 2.		*/

	quotient_size = dividend_size - divisor_size + 1;

	for (i = 0; i <= dividend_size - 1; i++) {
	    dividend_vector[dividend_size - i] = IndexI8AsFS(dividend_work, i);
	}
	dividend_vector[0] = 0;

	for (i = 0; i <= divisor_size - 1; i++) {
	    divisor_vector[divisor_size - i - 1] = 
		IndexI8AsFS(divisor_work, i);
	}

	/* Since scaling may increase the size of the dividend by one	*/
	/* digit, increase its size here.				*/

	dividend_size++;

	scale = Time8_base / (divisor_vector[0] + 1);

	if (scale != 1) {

	    /* Scale the divisor.					*/

	    utemp = 0;
	    for (i = divisor_size - 1; i >= 0; i--) {
		utemp = divisor_vector[i] * scale + utemp;
		divisor_vector[i] = utemp % Time8_base;
		utemp = utemp / Time8_base;
	    }

	    /* Scale the dividend.					*/

	    utemp = 0;
	    for (i = dividend_size - 1; i >= 0; i--) {
		utemp = dividend_vector[i] * scale + utemp;
		dividend_vector[i] = utemp % Time8_base;
		utemp = utemp / Time8_base;
	    }
	}

	for (i = 0; i <= quotient_size - 1; i++) {

	    utemp = dividend_vector[i] * Time8_base + dividend_vector[i + 1];

	    /* Estimate the quotient digit, then decrement it if it  	*/
	    /* seems too high.  This guarantees that the quotient digit */
	    /* is either correct or one too high.  The latter case is 	*/
	    /* handled below. 				 		*/

	    estimate = utemp / divisor_vector[0];

	    if (estimate >= Time8_base) {
		estimate = Time8_base - 1;
	    }

	    /* This loop checks the following condition:		*/
	    /* 								*/
	    /* estimate * 						*/
	    /* (divisor_vector[0] * Time8_base + divisor_vector[1])	*/
	    /* 	<							*/
	    /* (dividend_vector[i] * Time8_base ** 2 +			*/
	    /*		dividend_vector[i + 1] * Time8_base +		*/
	    /* 		dividend_vector[i + 2])				*/
	    /* 								*/
	    /* It has been rearranged to avoid overflow.		*/

	    {
		unsigned long int temp1, temp2, temp3, temp4, temp5;
	    /**/
		while (True) {
		    temp1 = divisor_vector[1] * estimate;
		    temp2 = estimate * divisor_vector[0];
		    temp3 = utemp - temp2;
		    if ((float) temp3 * Time8_base + dividend_vector[i + 2] > 
				MaxInt4) {
			break;
		    }
		    temp4 = temp3 * Time8_base;
		    temp5 = temp4 + dividend_vector[i + 2];
		    if (temp1 > temp5) {
			estimate--;
		    }
		    else {
			break;
		    }
		}
	    }



/* 
	    while (divisor_vector[1] * estimate >
		   ((utemp - estimate * divisor_vector[0]) * Time8_base +
		    dividend_vector[i + 2])) {
		estimate--;
	    }
*/


	    carry = 0;
	    for (j = divisor_size - 1; j >= 0; j--) {

		stemp = dividend_vector[i + j + 1] -
		                estimate * divisor_vector[j] + carry;

		carry = stemp / Time8_base;
		stemp = stemp % Time8_base;

		if (stemp < 0) {
		    stemp = stemp + Time8_base;
		    carry = carry - 1;
		}
		dividend_vector[i + j + 1] = stemp;
	    }
	    dividend_vector[i] += carry;

	    /* If the quotient estimate was too large, then the 	*/
	    /* dividend is now negative.  Correct for this by 		*/
	    /* decrementing the quotient estimate and adding the 	*/
	    /* divisor back to the dividend.				*/

	    if (dividend_vector[i] < 0) {
		estimate--;
		carry = 0;

		for (j = divisor_size - 1; j >= 0; j--) {

		    utemp = dividend_vector[i + j + 1] +
		           divisor_vector[j] +
			   carry;

		    if (utemp > Time8_base) {
			utemp = utemp - Time8_base;
			carry = 1;
		    }
		    else {
			carry = 0;
		    }
		    dividend_vector[i + j + 1] = utemp;
		}
		dividend_vector[i] += carry;
	    }

	    /* Store the estimate.					*/

	    IndexI8AsFS(buf_addr, quotient_size - i - 1) = estimate;

	}
    }

    if (negative) {
	okay = Time8BSub(buf_addr, (tInteger8)&ZeroTime, buf_addr);
	if (! okay) {
	    Time8BZero(buf_addr);
	    return False;
	}
    }

    return True;
} /* iTime8BDiv */



/* ********************************************************************
*
* StrNToTime8B - Convert tIntegerN value to corresponding tInteger8 value
*
******************************************************************** */

VK_tBoolean iStrNToTime8B(buffer, IN)
tInteger8 buffer;
tIntegerN IN;
{
/**/
    return IntNToTime8B(buffer, IN);
}  /* iStrNToTime8B */



/* ********************************************************************
*
* Time8ToStrNB - Convert tInteger8 value to corresponding tIntegerN value
*
******************************************************************** */

tIntegerN iTime8ToStrNB(buffer, Time8)
tLocalIntegerN buffer;
tInteger8 Time8;
{
/**/
    return Time8ToIntNB(buffer, Time8);	/* ?? why is this marked? */
}  /* iTime8ToStrNB */



/* *******************************************************************
*
* InitData - 
*
* HISTORY:
*	18-MAR-91 HK	Created  
*	10-JUN-91 DSB	Implemented large values
*
******************************************************************* */

void InitData()
{
/**/
    TIME_COPY(TimeLow, *MinTime8);
    TIME_COPY(MaxTime, *MaxTime8);
    TIME_ZERO(ZeroTime);
    TIME_ZERO(UZeroTime);

    TIME_CONVERT(tu_fs_factor,                    "1");
    TIME_CONVERT(tu_ps_factor,                 "1000");
    TIME_CONVERT(tu_ns_factor,              "1000000");
    TIME_CONVERT(tu_us_factor,           "1000000000");
    TIME_CONVERT(tu_ms_factor,        "1000000000000");
    TIME_CONVERT(tu_sec_factor,    "1000000000000000");
    TIME_CONVERT(tu_min_factor,   "60000000000000000");
    TIME_CONVERT(tu_hr_factor,  "3600000000000000000");
} /* InitData */

