#ifndef TKData_h
#define TKData_h
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKData.h
*
* DESCRIPTION:  
*	Definitions internal to the test kernel for operations on VHDL
*	data.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	07-MAR-91 DSB	Reincarnated. (Old TKData.* because TKStructs.*.)
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	06-MAY-91 DSB	Added tSimCycleCount
*	14-MAY-91 DSB	W0060: Change resolution limit (BaseUnit) back to fs
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	06-JUN-91 DSB	Added CvtVHDLToCString
*	10-JUN-91 DSB	Implemented 64-bit time
*	13-JUN-91 DSB	Adding casts for STIME_LESSTHAN to force to signed
* 	15-OCT-91 MNP	Proto maint.
*       29-OCT-91 DKC   Added tScalarUnion for handling large physical types
*                       and COPY_WORDS
*
* SCCS ID: @(#)TKData.h	42.1 1/10/92
*
******************************************************************* */

#include "VKBasicTypes.h"	/* for time types, etc. */
#include "SPI.h"		/* for NumericOrder type */

/* ******************************************************************* 
*
*
*
*******************************************************************  */

typedef VK_tInteger tSimCycleCount;	/* simulation cycle number */

/* Design note: A primary use of the simulation cycle count is to implement 
	certain boolean conditions efficiently.  Setting a cycle count to the
	current cycle count indicates that something applies to the current 
	cycle.	When the current cycle count changes, then all previous cycle 
	counts are not equal; effectively, all indicators have been cleared to
	False.
*/

/* ******************************************************************* 
*
* type tLargeUnsigned - representation of unsigned abolute time
*
******************************************************************* */

#ifdef VK_LITTLE_ENDIAN			/* match word order to byte order */
  typedef struct {
	unsigned long int low;		/* -least significant word */
	unsigned int high;		/* -most significant word */
    } tLargeUnsigned;			/* unsigned large physical type data */
#else
#ifdef VK_BIG_ENDIAN
  typedef struct {
	unsigned int high;		/* -most significant word */
	unsigned long int low;		/* -least significant word */
    } tLargeUnsigned;			/* unsigned large physical type data */
#else
  #error Neither VK_BIG_ENDIAN nor VK_LITTLE_ENDIAN is defined
#endif
#endif

#ifndef __STDC__		/* Where does this belong? Its effect is */
#  define const			/* needed in this file. MNP */
#endif

/* ******************************************************************* 
*
*
*
*******************************************************************  */

typedef VK_tLargeData 	tRelTimeBuf;
typedef tRelTimeBuf * 	tRelTimePtr;

typedef tLargeUnsigned 	tAbsTimeBuf;
typedef tAbsTimeBuf * 	tAbsTimePtr;

typedef char * 		tIntegerN;
typedef char * 		tLocalIntegerN;

typedef tRelTimePtr 	tInteger8;
typedef tAbsTimePtr 	tInteger8U;
typedef tRelTimeBuf 	tLocalInteger8[1];
typedef tAbsTimeBuf 	tLocalInteger8U[1];
 					/* one-element array for ease of use */

typedef union {
        VK_tIntegerData   t_int;
        VK_tFloatingData  t_float;
        VK_tLargeData     t_large;
        VK_tIntegerData   t_array[2];
    } tScalarUnion;                     /* stucture for holding element and
                                           driver values */

typedef tScalarUnion * tScalarUnionPtr;

extern tAbsTimeBuf tu_fs_factor;
extern tAbsTimeBuf tu_ps_factor;
extern tAbsTimeBuf tu_ns_factor;
extern tAbsTimeBuf tu_us_factor;
extern tAbsTimeBuf tu_ms_factor;
extern tAbsTimeBuf tu_sec_factor;
extern tAbsTimeBuf tu_min_factor;
extern tAbsTimeBuf tu_hr_factor;
					/* fs, ms, sec, min, and hr factors
					   are not initialized yet, so do
					   not use them */
/* ?? These global identifiers should be mixed case; or mixed case plus
	special, e.g.,  TimeFactor_us ?? */
 
extern tAbsTimeBuf TimeLow;		/* low for signed */	
extern tAbsTimeBuf ZeroTime;		/* zero for signed and unsigned */

extern tAbsTimeBuf MaxTime;		/* high for signed */
	/* ?? rename MaxTime to TimeHigh ?? */

extern tRelTimeBuf UZeroTime;		/* zero for signed and unsigned */

/* ******************************************************************* 
*
* Exported function headers.
*
*******************************************************************  */

void CvtVHDLToCString(
#ifdef VK_ANSI_FN_PROTOS
	VK_tCString c_buffer,		/* &out: packed character buffer */
	Integer4 buf_size,		/* in: size of character buffer */
	VK_tVHDLString VHDL_string,	/* &in: unpacked word buffer */
	VK_tStringLength length		/* in: length of word buffer */
#endif
	);

tInteger8 iTime8BZero(
#ifdef VK_ANSI_FN_PROTOS
	tInteger8 buf_addr
#endif
	);

void iTimeAdd1(
#ifdef VK_ANSI_FN_PROTOS
	tInteger8U sched_time_addr,
	tInteger8U abs_time_addr,
	tInteger8 rel_time_addr
#endif
	);

void iTimeSub1(
#ifdef VK_ANSI_FN_PROTOS
    	tInteger8 rel_time_addr,
    	const tInteger8U left,
    	tInteger8U right
#endif
	);

const tInteger8 iMaxTime8(
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

const tInteger8 iMinTime8(
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

VK_tBoolean iTime8IsNeg(
#ifdef VK_ANSI_FN_PROTOS
    	tInteger8 Time8
#endif
	);

NumericOrder iTime8Compare(
#ifdef VK_ANSI_FN_PROTOS
    	tInteger8 left,
    	tInteger8 right
#endif
	);

VK_tBoolean iTime8IsZero(
#ifdef VK_ANSI_FN_PROTOS
 	tInteger8 Time8
#endif
	);

VK_tBoolean iTime8BAdd(
#ifdef VK_ANSI_FN_PROTOS
 	tInteger8 buf_addr,
 	tInteger8 left,
 	tInteger8 right
#endif
	);

VK_tBoolean iTime8BSub(
#ifdef VK_ANSI_FN_PROTOS
 	tInteger8 buf_addr,
 	const tInteger8 left,
 	tInteger8 right
#endif
	);

tIntegerN iTime8ToIntNB(
#ifdef VK_ANSI_FN_PROTOS
 	tIntegerN userbuf,
 	tInteger8 Time8val
#endif
	);

tIntegerN iTime8ToStrNB(
#ifdef VK_ANSI_FN_PROTOS
	tLocalIntegerN buffer,
	tInteger8 Time8
#endif
	);

VK_tBoolean iStrNToTime8B(
#ifdef VK_ANSI_FN_PROTOS
	tRelTimePtr bufaddr,
	tIntegerN intn
#endif
	);
	
VK_tBoolean iTime8BDiv(
#ifdef VK_ANSI_FN_PROTOS
	tInteger8 buf_addr,
	tInteger8 dividend,
	tInteger8 divisor
#endif
	);

VK_tBoolean iTime8BMult(
#ifdef VK_ANSI_FN_PROTOS
	tInteger8 buf_addr,
	tInteger8 mpyer,
	tInteger8 mcand
#endif
	);


VK_tBoolean zCheckIntN(		
#ifdef VK_ANSI_FN_PROTOS
    	tIntegerN intn,
    	VK_tBoolean reporterror
#endif
	);

Integer4 zIntNToInt4(
#ifdef VK_ANSI_FN_PROTOS
	tIntegerN intn
#endif
	);

void InitData();


/* *******************************************************************
*
* In the following TIME macros, parameter 'buffer' is of type (char *),
* and all other parameters are of type TK_tTimeBuffer??.  A macro may be
* called with the same actual parameter for any two formal parameters
* of type TK_tTimeBuffer.  TIME_ADD, TIME_SUB, TIME_MULT, and TIME_DIV
* will check for overflow when 64-bit time values are handled.
*
* ?? UPDATE THIS
*
* NOTE: Time computations for simulation usually don't need overflow checks.
*	Absolute time is unsigned (using all bits), and relative time is 
*	signed.  ...
*
*
*
******************************************************************* */

/* ?? HEY: These macros should have more parentheses ?? */

/* TIME_COPY - copy signed or unsigned time */
#  define TIME_COPY(target, source)	((target).low = (source).low, \
					 (target).high = (source).high)

#  define TIME_ZERO(time)		TIME_COPY(time, ZeroTime)
#  define STIME_HIGH(time)		TIME_COPY(time, MaxTime)
#  define STIME_LOW(time)		TIME_COPY(time, TimeLow)

/* TIME_EQUAL - compare two signed times or two unsigned times */
#  define TIME_EQUAL(left, right)	((left).low == (right).low && \
					 (left).high == (right).high)

#  define UTIME_LESSTHAN(left, right)	\
    ((((unsigned long int) (left).high) < (unsigned long int) (right).high) ||\
     (((left).high == (right).high) && ((left).low < (right).low)))
	/* unsigned less than: if left high word is (unsigned) less than right
		high word, or if high words are equal and left low word is
		less than right low word */

#  define STIME_LESSTHAN(left, right)	\
    ((((long int) (left).high) < (long int) (right).high) || \
     (((left).high == (right).high) && ((left).low < (right).low)))
	/* signed less than: if left high word is (signed) less than right
		high word, or if high words are equal and left low word is
		less than right low word */

/* STIME_NEGATIVE - check if negative signed time */
#  define STIME_NEGATIVE(time)	(((/* unsigned */long int)(time).high) < 0)

/* UTIME_FORMAT - format unsigned time to string */
#  define UTIME_FORMAT(buffer, time)\
    iTime8ToStrNB((tLocalIntegerN)buffer, (tInteger8)&time)
					/* ?? need unsigned version ?? */

/* STIME_FORMAT - format signed time to string */
#  define STIME_FORMAT(buffer, time)\
    iTime8ToStrNB(buffer, (tInteger8)&time)

/* TIME_CONVERT - convert string to signed time */
#  define TIME_CONVERT(time, string)\
    iStrNToTime8B((tInteger8)&time, string)
 	/* buffer may be a string literal */

#  define UTIME_ADD(sched, sim, rel) \
    iTimeAdd1((tInteger8U)&sched, (tInteger8U)&sim, (tInteger8)&rel)
 	/* add relative time to simulation time giving scheduled sim. time */

#  define TIME_SUB(result, left, right)	\
    iTimeSub1((tInteger8)&result, (const tInteger8U)&left, (tInteger8U)&right)
	/* subtract sim. from sim. or scheduled time giving relative time */

/* TIME_MULT - multiply times (?? probably should be unsigned ??) 
 * The &'s were not down there, trying it for consistency - MNP 910917
 */
#  define TIME_MULT(result, left, right) \
    iTime8BMult((tInteger8)&result, (tInteger8)&left, (tInteger8)&right)

/* TIME_DIV - divide times (?? probably should be unsigned ??) 
 * The &'s were not down there, trying it for consistency - MNP 910917
 */
#  define TIME_DIV(result, left, right)	\
    iTime8BDiv((tInteger8)&result, (tInteger8)&left, (tInteger8)&right)

/* TIME_POS - get position value in two 4-byte integers */
#  define TIME_POS(high, low, time)	(high = time.high, low = time.low)/* ?? OLD ?? */


#define TU_FS "fs"
#define TU_PS "ps"
#define TU_NS "ns"
#define TU_US "us"
#define TU_MS "ms"
#define TU_SEC "sec"
#define TU_MIN "min"
#define TU_HR "hr"
#define BaseUnit TU_FS


#define IsFutureTime(T) (UTIME_LESSTHAN(SimTime, T))


/* ***********************************************************************
*
* The following macro is for in-line copying of word-aligned data 
*
************************************************************************ */

#define COPY_WORDS(from,to,words) {long *f=(long *)(from),*t=(long *)(to);\
                                   int  w=(words); while (w--)\
                                                          *t++ = *f++;}

#endif /* TKData_h */


