#ifndef TKDebug_H
#define TKDebug_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKDebug.h
*
* DESCRIPTION:  
*	Private interface file for TKDebug.c.  <then describe contents>
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Created
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	11-OCT-91 MNP	Header fixup.
*	21-OCT-91 MNP	Added sourcefile field to tBP_Line and
*			SetLineBreakpoint
*
* SCCS ID: @(#)TKDebug.h	42.1 1/10/92
*
******************************************************************* */

#include "VKBasicTypes.h"			/* for VK_tBoolean */
#include "VKTrace.h"
#include "SPI.h"
#include "TKData.h"

typedef struct {
	VK_tBoolean next_step_flag;	/* stop on next debug callback */
	VK_tBoolean movie_flag;		/* movie mode: show source line for all
					   steps even if not stepping */
	VK_tBoolean break_any_process;	/* stop before resuming any process */
	VK_tBoolean break_after_elab;	/* step after elaboration, before
					   simulation initialization */
	VK_tBoolean break_after_init;	/* stop after initialization, before
					   simulation */
	VK_tBoolean is_after_elab;
	VK_tBoolean is_after_init;
	VK_tBoolean is_after_sim;
    } tDebugData;
extern tDebugData DB;

/* break point class */
typedef enum {
	bp_NoBreakPoint,
	bp_AfterElaboration,
	bp_AfterInitialization,
	bp_Step,
	bp_StepOver,
	bp_Cycle,
	bp_Time,
	bp_Process,
	bp_ProcessAll,
	bp_Interrupt,
	bp_Line,
	bp_Suspend,
	bp_Call,
	bp_Return
    } tBreakPointClass;

#define bp_AtLine bp_Step		/* ?? let's remove this soon ?? */

/* ?? breakpoints records should not use qqRecordKind from TKStructs.h ?? */

/* ?? indent line with closing brace one level (4 spaces) from start, not
	8 spaces (tBreakPointClass is right (as I write this)) ?? */

/* break point record types */
typedef struct sBP_AfterElaboration {
	tBreakPointClass fRecordKind;		/* record type */
	} tBP_AfterElaboration;
	
typedef struct sBP_AfterInitialization {
	tBreakPointClass fRecordKind;		/* record type */
	} tBP_AfterInitialization;

typedef struct sBP_Step {
	tBreakPointClass fRecordKind;		/* record type */
	VK_tProcessHandle process;	
	} tBP_Step;

typedef struct sBP_StepOver {
	tBreakPointClass fRecordKind;		/* record type */
	VK_tProcessHandle process;
	Integer4 depth;			/* subprogram call level */
						/* >= 0 same or higher then */
						/* level when breakpoint set */
	} tBP_StepOver;

typedef struct sBP_Cycle {
	tBreakPointClass fRecordKind;		/* record type */
	} tBP_Cycle;
	
typedef struct sBP_Time {
	tBreakPointClass fRecordKind;		/* record type */
	tAbsTimeBuf fTime;
	} tBP_Time;

typedef struct sBP_ProcessAll 	{
	tBreakPointClass fRecordKind;		/* record type */
	} tBP_ProcessAll;

typedef struct sBP_Process {
	tBreakPointClass fRecordKind;		/* record type */
	VK_tProcessHandle process;	
	} tBP_Process;

typedef struct sBP_Interrupt {
	tBreakPointClass fRecordKind;		/* record type */
	} tBP_Interrupt;

typedef struct sBP_Line {
	tBreakPointClass fRecordKind;		/* record type */
	VK_tProcessHandle process;
	VK_tCString library;
	VK_tCString primary;
	VK_tCString secondary;
	Integer4 line;
	VK_tCString sourcefile;
	} tBP_Line;

typedef struct sBP_Suspend {
	tBreakPointClass fRecordKind;		/* record type */
	} tBP_Suspend;

#define qqDepth(N) (N)->depth
#define AddqqDepth(N, V) ((N)->depth = V)
#define AddqqDepth(N, V) ((N)->depth = V)

#define qqLibrary(N) (N)->library
#define AddqqLibrary(N, V) ((N)->library = V)
#define SetqqLibrary(N, V) ((N)->library = V)

#define qqLine(N) (N)->line
#define AddqqLine(N, V) ((N)->line = V)
#define SetqqLine(N, V) ((N)->line = V)

#define qqPrimary(N) (N)->primary
#define AddqqPrimary(N, V) ((N)->primary = V)
#define SetqqPrimary(N, V) ((N)->primary = V)

#define qqProcess(N) (N)->process
#define AddqqProcess(N, V) ((N)->process = V)
#define SetqqProcess(N, V) ((N)->process = V)

#define qqSecondary(N) (N)->secondary
#define AddqqSecondary(N, V) ((N)->secondary = V)
#define SetqqSecondary(N, V) ((N)->secondary = V)

#define qqSourcefile(N) (N)->sourcefile
#define AddqqSourcefile(N, V) ((N)->sourcefile = V)
#define SetqqSourcefile(N, V) ((N)->sourcefile = V)

void GetCommands();
void DebugInterrupt();

void GetLastBreakpoint(
#ifdef VK_ANSI_FN_PROTOS
	VK_tCString * library,
	VK_tCString * primary,
	VK_tCString * secondary,
	VK_tCString * file,
	Integer4 * line
#endif
	);

void GetLastTracepoint(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr * region_data	/* region LAR pointer address */
#endif
	);

VK_tBoolean CancelBreakpointN(
#ifdef VK_ANSI_FN_PROTOS
	Integer4 number
#endif
	);

void SetLineBreakpoint(
#ifdef VK_ANSI_FN_PROTOS
	VK_tProcessHandle process,
	VK_tCString library,
	VK_tCString primary,
	VK_tCString secondary,
	Integer4 line,
	VK_tCString sourcefile
#endif
	);

void SetTimeBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
	tAbsTimePtr time
#endif
	);


void SetStepBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
	VK_tProcessHandle process		/* current process */
#endif
	);

void SetSuspendBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void SetProcessAllBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void SetCycleBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void SetInitBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);

void SetElabBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);


void SetStepOverBreakpoint (
#ifdef VK_ANSI_FN_PROTOS
	VK_tProcessHandle process		/* current process */
#endif
	);

VK_tBoolean IsUserProcess(
#ifdef VK_ANSI_FN_PROTOS
	VK_tProcessHandle process
#endif
	);

void SetProcessBreakpoint(
#ifdef VK_ANSI_FN_PROTOS
	VK_tCString process_name		/* in: process simple name */
#endif
	);

void FormatBreakpoint(
#ifdef VK_ANSI_FN_PROTOS
	char * buffer,
	int buf_size,
	VK_tVoidPtr handle
#endif
	);

void ShowBreakpoints(
#ifdef VK_ANSI_FN_PROTOS
#endif
	);


void ShowProcesses(
#ifdef VK_ANSI_FN_PROTOS
#endif
	);


void ShowSignals(
#ifdef VK_ANSI_FN_PROTOS
#endif
	);


void InitializeDebug (
#ifdef VK_ANSI_FN_PROTOS
#endif
	);


void InternalDebug(
#ifdef VK_ANSI_FN_PROTOS
 	tBreakPointClass breakpoint_class,
 	tAbsTimePtr time
#endif
	);


/* Really defined in TKCommand.h, but then it doesn't belong there, */
/* either. -MNP */

extern void DoExamineObjects(
#ifdef VK_ANSI_FN_PROTOS
	StringN simple_name
#endif
	);

#endif /* TKDebug_H */

