/*****************************************************************************
*
* Copyright (c) 1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* FILE: TKDebugUtils.c - secondary support functions for TKDebug
*
* DESCRIPTION:
*
* TYPES:
*
* CONSTANTS:
*
* FUNCTIONS:
*	ScanTraceTable
*	VerifyUnitLine
*	VerifyFileLine
*	DumpTraceTables		- for debug 
*
* HISTORY:
*	16-OCT-91 MNP	Created
*
* SCCS ID: @(#)TKDebugUtils.c	42.1 1/10/92
*
*****************************************************************************/
#include "Copyright.h"
static char SCCS_ID[] = "@(#)TKDebugUtils.c	42.1 1/10/92";

#include "TKDebugUtils.h"
#include "TKCommon.h"
#include "TKData.h"		/* for TIME macros */
#include "TKStructs.h"		/* for symbolic information types */
#include "TKSymbUtils.h"	/* for TK_GetRegionInfo, etc. */
#include "TKGlobals.h"		/* for SymbInfoList */
#include "TKFileUtils.h"	/* for FilenamesEqual... */

/* *******************************************************************
*
* ScanTraceTable - Searches a trace table for a line number.
*
* Returns statistics on the trace table and if the specified line was
* found in it. The returned tLineNumCode is informative.
*	
* HISTORY:
*	08-OCT-91 MNP	Created
*
******************************************************************* */

tLineNumCode ScanTraceTable(syminfo, wanted_line, previous, following,
			    smallest, largest) 
VK_tSymbInfoHandle syminfo;	/* in: sym info holding table to be searched */
VK_tLineNumber wanted_line;	/* in: line number to look for */
VK_tLineNumber *previous;	/* out: largest line less than wanted_line */
VK_tLineNumber *following;	/* out: smallest ln greater than wanted_line */
VK_tLineNumber *smallest;	/* out: smallest line number in table */
VK_tLineNumber *largest;	/* out: largest line number in table */
{
    VK_tExecPosIndex 	tablesize, i;
    VK_tTraceTableEntry *ttp;
    VK_tBoolean		found;		
/**/
    /* Check parameters */

    if (syminfo == NULL || previous == NULL || following == NULL ||
	smallest == NULL || largest == NULL)
	return L_InvParam;
    if (wanted_line < 0)
	return L_InvLine;

    found = False;
    ttp = qqTraceTable(syminfo);
    tablesize = qqExecTableSize(syminfo);

    /* for all table entries */

    for (i=0; i<tablesize && ttp != NULL && ttp->line_number != -1; i++) {

	if (ttp->line_number == wanted_line)
	    found = True;
	if (ttp->line_number > *largest) 
	    *largest = ttp->line_number;
	if (ttp->line_number < *smallest) 
	    *smallest = ttp->line_number;
	if (ttp->line_number > wanted_line && ttp->line_number < *following)
	    *following = ttp->line_number;
	if (ttp->line_number < wanted_line && ttp->line_number > *previous)
	    *previous = ttp->line_number;

	ttp++;	/* next tracetable entry */
    }

    /* determine return code */

    if (found)
    	return L_Same;
    else if (wanted_line > *largest || wanted_line < *smallest)
	return L_OutOfRange;
    else 
	return L_InRange;

} /* ScanTraceTable */


/* *******************************************************************
*
* VerifyUnitLine - Verify that a line in a unit is a valid exec pos.
*
* HISTORY:
*	08-OCT-91 MNP	Created
*	21-OCT-91 MNP	added source param for DoBreakUnit to see
*
******************************************************************* */

tLineNumCode VerifyUnitLine(lib_name, entity_name, arch_name, line_num,
			   prev, next, source) 
VK_tCString lib_name;   	/* in: name of the library */
VK_tCString entity_name;	/* in: name of the entity */
VK_tCString arch_name;  	/* in: name of the architecture */
Integer4 line_num;     		/* in: line number to verify */
Integer4 *prev;     		/* out: highest line number <= line_num */
Integer4 *next;     		/* out: smallest line number > line_num */
VK_tCString *source; 		/* out: src file where found, if found */
{
    TempItem 		package_scan;
    VK_tSymbInfoHandle 	unit_info;
    VK_tName 		library, primary, secondary, file;
    VK_tBoolean 	lib_found, prim_found, sec_found;
    VK_tLineNumber	smallest, largest;
/**/
    /* Check params */
    if ((! lib_name) || (! *lib_name) ) {
        return L_NoLibrary;
    }
    else if ((!entity_name) || (! *entity_name)) {
        return L_NoUnit;
    }
    else if ((! arch_name) || (! *arch_name)) {
        return L_NoUnit;
    }
    else if (line_num < 0) {
        return L_InvLine;
    }

    /* Search all syminfo records, until an exact L`S`P match. Also 
       gather line info for all L`P matches. */

    /* NOTE - This is not as fast as it could be, with all these string 
       compares. Searching a hierarchical list of Lib->Sec->Prim would
       surely be better. ??? -MNP */
      
    lib_found = False;
    prim_found = False;
    sec_found = False;

    for (package_scan = TempFirstItem(SymbInfoList); 
	 ! TempNullItem(package_scan);
	 package_scan = TempNextItem(package_scan)) {

	unit_info = SymbInfoValue(package_scan);

	switch (qqSymbInfoKind(unit_info)) {
	  case PackageDeclInfo:
	  case EntityDeclInfo:
	  case ConfigurationDeclInfo:
	  case PackageBodyInfo:
	  case ArchitectureBodyInfo:
	    su_GetUnitNames(unit_info, &library, &primary, &secondary, &file);

	    if (!strcmp(library, lib_name)) {
		lib_found = True;
	    }
	    if (!strcmp(primary, entity_name)) {
		prim_found = True;
	    }
	    if (!strcmp(secondary, arch_name)) {
		sec_found = True;
	    }
	    
	    if (lib_found && prim_found && sec_found) {

		/* Everything else matches, just check the line now.
		   Will return same, in range, or out of range */

		*source = qqSourceFileName(unit_info);
		return ScanTraceTable (unit_info, line_num, prev,
		      next, &smallest, &largest);
	    }

	    /* all didn't match; keep scanning */
	    break;		

	  default:
	    break;
	} /* switch */
    } /* for */
	
    /* Didn't match all of (lib, prim, sec). Find which and report. */
    
    if (!lib_found) {
	return L_NoLibrary;
    }
    else {
	return L_NoUnit;
    }
    
} /* VerifyUnitLine */


/* *******************************************************************
*
* VerifyFileLine - Verify that a line in a src file is a valid exec pos.
*
* HISTORY:
*	03-OCT-91 MNP	Created
*
******************************************************************* */
tLineNumCode VerifyFileLine(source_file, line_number, previous_line,
			    following_line, smallest, largest, unit_info)
VK_tCString source_file;	/* in: vhdl file to look in */
VK_tLineNumber line_number;	/* in: line number to verify */
VK_tLineNumber *previous_line;	/* out: largest line less than line_num */
VK_tLineNumber *following_line; /* out: smallest ln greater than line_num */
VK_tLineNumber *smallest;	/* out: smallest line number in file */
VK_tLineNumber *largest;	/* out: largest line number in file */
VK_tSymbInfoHandle *unit_info;	/* out: unit in which the line was found */
{
    TempItem 		package_scan;
    VK_tBoolean		found_source, found_num;
    VK_tCString		sym_srcfile;
    tLineNumCode	result;
/**/

    /* Check some args */

    if (NULL == source_file || *source_file == '\0') {
	return L_InvSrcFile;
    }
    
    /* Search all symbolic info records of proper type, looking for 
       the desired source file. */

    found_source = False;
    found_num = False;
    
    /* stats to be returned */

    *largest = 0;
    *following_line = MaxInt4;
    *smallest = MaxInt4;
    *previous_line = 0;

    for (package_scan = TempFirstItem(SymbInfoList); 
	 ! TempNullItem(package_scan);
	 package_scan = TempNextItem(package_scan)) {

	switch(qqSymbInfoKind(SymbInfoValue(package_scan)))
	{
	  case EntityDeclInfo:
	  case PackageDeclInfo:
	  case PackageBodyInfo:
	  case ArchitectureBodyInfo:
	  case ConfigurationDeclInfo:
	    sym_srcfile = qqSourceFileName(SymbInfoValue(package_scan));
	    if (FilenamesEqual(sym_srcfile, source_file)) {

		/* We found requested source file. Now search its
		   trace table, looking for the requested line, and
		   also recording min and max line nums. */ 

		found_source = True;

		result = ScanTraceTable(SymbInfoValue(package_scan),
					line_number, previous_line,
					following_line, smallest,
					largest);
		switch (result) 
		{
		  case L_Same:		/* found it! */
		    *unit_info =  SymbInfoValue(package_scan);
		    return L_Same;
		    break;
		  case L_InRange:	/* Not yet, look at another record...*/
		  case L_OutOfRange:	/* ...but remember the limits. */
		    break;
		  default:
		    return result;	/* it was an error of some kind */
		}
	    } /* if FilenamesEqual */
	    break;
	    
	  default:
	    break;

	} /* switch syminfo kind */
    } /* for all syminfos */
    
    /* determine return code, it's an error */

    if (!found_source)
	return L_InvSrcFile;
    else if (line_number < *largest && line_number > *smallest)
	return L_InRange;
    else
	return L_OutOfRange;

} /* VerifyFileLine */



/*****************************************************************************
*
* DumpTraceTables - for src debugging
*
* DESCRIPTION: 
*	Walks syminfolist, prints tt's of records having them.
*
* HISTORY:
*	17-OCT-91 MNP	Created
*
*****************************************************************************/

void DumpTraceTables()
{
    tSymbInfoKind kind;
    TempItem package_scan;
    VK_tTraceTableEntry *tt;
    int size,i;

    for (package_scan = TempFirstItem(SymbInfoList); 
	 ! TempNullItem(package_scan);
	 package_scan = TempNextItem(package_scan)) 
    {
	kind = qqSymbInfoKind(SymbInfoValue(package_scan));
	switch(kind)
	{
	  case PackageDeclInfo:
	  case PackageBodyInfo:
	  case EntityDeclInfo:
	  case ArchitectureBodyInfo:
	  case ConfigurationDeclInfo:
	    fprintf(stderr,"SymbInfoKind\tname\tsourcefile\n%d\t%s\t%s\n",
		    (int)kind,
		    qqName(SymbInfoValue(package_scan)),
		    qqSourceFileName(SymbInfoValue(package_scan)));
	    size = qqExecTableSize(SymbInfoValue(package_scan));
	    tt = qqTraceTable(SymbInfoValue(package_scan));
	    fprintf(stderr, "\tent #\tline #\texec_pos_kind\n");
	    for (i=0; i<size; i++)
	    {
		fprintf(stderr, "\t%d\t%d\t%d\n", i, tt->line_number,
			(int)tt->exec_pos_kind);
		tt++;
	    }
	    break;
	  default:
	    break;
	}
    }
} /* DumpTraceTables */
