static char SCCS_ID[] = "@(#)TKElaborate.c	42.2 1/13/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKElaborate.c
*
* DESCRIPTION:  
*	Functions for VKI files VKElaborate.c
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Split from VKI.c
*	18-MAR-91 HK	Added references to TIME macros
*	29-MAR-91 WAG	Added AllocateMemory, ReallocateMemory,
*			and FreeMemory routines
*	02-APR-91 WAG	W0015.  Moved the TerminateKernel, EarlyTerminate, and
*			ExitProgram routines here from VKMain.c
*	03-APR-91 WAG	W0015.  Created VK_Alloc and VK_Free routines from
*			AllocateMemory and FreeMemory
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 DSB	Moved non-VKI stuff to TKTemp.c
*	16-MAY-91 DSB	Converting to unsigned absolute time
*
* SCCS ID: @(#)TKElaborate.c	42.2 1/13/92
*
******************************************************************* */

#include "TKTemp.h"
#include "TKCommon.h"
#include "TKSim.h"		/* for CurrentProcess */
#include "TKData.h"		/* for TIME macros */
#include "TKGlobals.h"		/* for GF */
#include "TKDebug.h"		/* for DB */
#include "TKSymbUtils.h"	/* for su_TraverseToData, etc. */


/* ?? remodularization: the function for doing elaboration should be
	moved to a separate module; actual, the VKI should probably have
	a separate model for the four or so VKI functions; probably
	VKElaborate.h and TKElaborate.[ch]
?? */

/* extern */ VK_tSymbInfoHandle SpecifiedTopUnit = NULL;

typedef struct { 
	VK_tSymbInfoHandle comp_inst_info;
	VK_tSymbInfoHandle comp_decl_info;
	VK_tIARHeaderPtr inst_intf_data;
    } tBindingData, * tBindingDataPtr;


/* *******************************************************************
*
* FindLatestArchitecture - find most-recently-analyzed architecture for entity
*
* HISTORY:
*	22-JUL-91 DSB	Created
*	31-JUL-91 DSB	Fixed bug: comparison was backwards
*
******************************************************************* */
static VK_tSymbInfoHandle FindLatestArchitecture(entity_info)
VK_tSymbInfoHandle entity_info;	/* identification of entity */
{
    VK_tSymbInfoHandle latest;
    TempItem scan;
    VK_tSymbInfoHandle current;
    VK_tTimeStampPtr latest_date, current_date;
    VK_tBoolean greater;
/**/
    latest = NULL;
    for (scan = TempFirstItem(qqExtensions(entity_info)); ! TempNullItem(scan);
		scan = TempNextItem(scan)) {
	current = SymbInfoValue(scan);
	if (NULL == latest) {
	    latest = current;
	}
	else {
	    latest_date = qqAnalysisTime(latest);
	    current_date = qqAnalysisTime(current);
/* ?? clean: timestamp comparison should be a function in TKData ?? */
	    if (current_date->year > latest_date->year) {
		greater = True;
	    }
	    else if (current_date->year < latest_date->year) {
		greater = False;
	    }
	    else if (current_date->month > latest_date->month) {
		    greater = True;
	    }
	    else if (current_date->month < latest_date->month) {
		    greater = False;
	    }
	    else if (current_date->day > latest_date->day) {
		greater = True;
	    }
	    else if (current_date->day < latest_date->day) {
		greater = False;
	    }
	    else if (current_date->hour > latest_date->hour) {
		greater = True;
	    }
	    else if (current_date->hour < latest_date->hour) {
		greater = False;
	    }
	    else if (current_date->minute > latest_date->minute) {
		greater = True;
	    }
	    else if (current_date->minute < latest_date->minute) {
		greater = False;
	    }
	    else if (current_date->second > latest_date->second) {
		greater = True;
	    }
	    else if (current_date->second < latest_date->second) {
		greater = False;
	    }
	    else if (current_date->hundredth > latest_date->hundredth) {
		greater = True;
	    }
	    else if (current_date->hundredth < latest_date->hundredth) {
		greater = False;
	    }
	    else {			/* time stamps are equal */
/* ?? TBD: what is policy for equal dates ?? */
	        greater = False;	/* for now, use first architecture */
	    }
	    
	    if (greater) {
		latest = current;
	    }
	}
    }
    return latest;
} /* FindLatestArchitecture */


/* *******************************************************************
*
* VK_ElaborateDefaultArchitecture - elaborate default arch. for entity aspect 
*	of "entity E"
*
* HISTORY:
*	22-JUL-91 DSB	Created
*	15-AUG-91 DSB	Check for lack of architecture
*
******************************************************************* */
void VK_ElaborateDefaultArchitecture(entity_info, intf_data)
VK_tSymbInfoHandle entity_info;	/* identification of entity */
VK_tIARHeaderPtr intf_data;	/* interface activation record for 
				   elaboration function */
{
    VK_tSymbInfoHandle latest;
    VK_tGenericElabFnPtr elab_fn;
    tTempBuf256 ent_buf;
/**/
/* LRM does not say anything beyond that "entity E" implies "any" architecture.
	Test kernel policy is to use latest architecture. */

    latest = FindLatestArchitecture(entity_info);
					/* get latest architecture */
    if (NULL == latest) {		/* error if no architecture for E */
	sprintf(ent_buf, "%s.%s", qqName(entity_info), 
		qqName(qqEnclReg(entity_info)));
					/* qqEnclReg gets entity's library */
	ReportKernelError("no architecture for entity", ent_buf, "", 0);
    }

    elab_fn = qqElaborationFunction(latest);
					/* get design entity's elaboration 
					   function from architecture*/
    (*elab_fn)(intf_data);		/* call elaboration function */
} /* VK_ElaborateDefaultArchitecture */


/* *******************************************************************
*
* FindDefaultEntity - implement default entity aspect of default binding
*
* ALGORITHM:
*	Given component instantiation CI in architecture A of entity E.
*	For each library symbol LS defined in a library clause of A or E, 
*	search the library denoted by LS.  (Account for multiple symbols LS
*	denoting the same library L.)  In those libraries, note each entity
*	whose simple name matches the simple name of the component C.
*	If there is one such matching entity, use it.  If there are zero,
*	report an error.  If there are two or more, report an error.
*
* HISTORY:
*	24-JUL-91 DSB	Created
*	15-AUG-91 DSB	Added more precise default entity searching
*	16-AUG-91 DSB	Added less precise default entity searching
*
******************************************************************* */
static VK_tSymbInfoHandle FindDefaultEntity(instance_info)
VK_tSymbInfoHandle instance_info;	/* component instantiation id. */
{
    VK_tSymbInfoHandle comp_decl_info;
    VK_tSymbInfoHandle match;
    TempList match_list;
    VK_tSymbInfoHandle ext_unit_info;

    TempItem lib_scan;
    VK_tSymbInfoHandle library_symbol;
    VK_tSymbInfoHandle library_region;
    TempItem scan;
    VK_tSymbInfoHandle current;
/**/
    VERIFY_ERROR(NULL != instance_info);
    VERIFY_ERROR(InstantiationStmtInfo == qqSymbInfoKind(instance_info));

    comp_decl_info = qqCompDeclInfo(instance_info);

    match = NULL;
    match_list = TempNewList();

					/* scan libraries specified for this
					   instantiation */
/* ?? hole:  Implicit component configurations under explicit block 
	configurations for architectures are not addressed yet. ?? */

	/* : find symbolic information record for design unit containing
		component instantiation */

    ext_unit_info = qqEnclReg(instance_info);
    while (ArchitectureBodyInfo != qqSymbInfoKind(ext_unit_info)) {
	ext_unit_info = qqEnclReg(ext_unit_info);
    }

    while (NULL != ext_unit_info) {	/* scan architecture and entity for
					   all library symbols */

    	for (lib_scan = TempFirstItem(qqChildren(ext_unit_info));
		! TempNullItem(lib_scan); lib_scan = TempNextItem(lib_scan)) {
					/* scan one design unit for library
					   symbols */
	    library_symbol = SymbInfoValue(lib_scan);
	    if (LibrarySymbolInfo == qqSymbInfoKind(library_symbol)) {
		library_region = qqLibraryInfo(library_symbol);

/* ?? Note:  What should we do if a configuration or package matches the
	default entity name?  Should we ignore non-entities, or does it count
	as a match for purposes of ambiguous macthing?  ?? */

		for (scan = TempFirstItem(qqChildren(library_region)); 
			! TempNullItem(scan); scan = TempNextItem(scan)) {
					/* scan one library for entities */
		    current = SymbInfoValue(scan);

		    if (EntityDeclInfo == qqSymbInfoKind(current)) {
					/* consider entities only */
			if (StringMatch(qqName(comp_decl_info), 
				qqName(current))) {
					/* entity name matches comp. name */

					/* (avoid duplicates, because we don't
					   yet avoid duplicate libraries for
					   different library symbols) */
			    if (TempNullItem(TempFindItem(match_list, 
					current))) {
				TempAddLast(match_list, current);
					/* accumulate list of ~matching 
					   entities */
			    }
			}
		    }
		}
	    }

        }
	ext_unit_info = qqExtReg(ext_unit_info);

    }

    if (0 == TempListLength(match_list)) {
	ReportKernelError("no matching default entity",
		qqName(instance_info), "", 0);
	match = NULL;
    }
    else if (1 == TempListLength(match_list)) {
	match = SymbInfoValue(TempFirstItem(match_list));
    }
    else {				/* more than one */
/* Note: by (partial) visibility rules, neither is visible.  We can report this
	as no visible entity or as a multiple, ambiguous match. */
/* ?? When we have an interactive mode, we can allow the user to choose which
	entity to use.  By then, we should have an interactive configuration
	tool, which will report choices to the kernel. ?? */


    	ReportKernelError("non-unique matching default entity",
		qqName(instance_info), "", 0);

    }

    return match;
} /* FindDefaultEntity */


/* *******************************************************************
*
* VK_GetActualGeneric - get default binding's actual value
*
* ??? WORK ON NAME; maybe VK_GetActual_Generic, VK_GetGenericActual,
*	VK_GetLocalGeneric, VK_GetLocal_Generic, ... ?? 
*
* DESCRIPTION:
*
* HISTORY:
*	24-JUL-91 DSB	Created
*	26-JUL-91 DSB	Handle top-level generics
*
******************************************************************* */
void VK_GetActualGeneric(kernel_data, formal_info, formal_constraint_addr,
	actual_constraint_addr_addr, actual_value_addr_addr)
VK_tVoidPtr             kernel_data;	/* blind pointer to kernel data */
VK_tSymbInfoHandle      formal_info;	/* generic constant identification */
VK_tAnyConstraintAddr   formal_constraint_addr;
				        /* NULL or formal's constraint */
VK_tIndexConstraintAddr *actual_constraint_addr_addr;
				        /* to return actual's constraint */
VK_tDataAddr            *actual_value_addr_addr;
				        /* to return actual's value */
{
    VK_tBoolean        looping;
    TempItem           local_scan;
    VK_tSymbInfoHandle candidate;
    VK_tSymbInfoHandle local;
    VK_tVoidPtr        constraint_act_rec_base;
    VK_tDataAddr       value_address;
    VK_tSymbInfoHandle type_info;
    tTypeClass         type_class;
    VK_tBoolean        is_composite;
    tBindingDataPtr    binding_data = (tBindingDataPtr)kernel_data;
/**/

/* ?? WHAT A MESS; THIS NEEDS A LOT OF CLEANUP! ?? */
/* fold code that is common into separate functions */

    if (NULL == binding_data) {		/* null binding_data means that we are
					   doing top-level default binding */

		/* : specify actual of "open"; see if generated code accepts */
/* ?? note: eventually, register whether generic can be left unassociated ?? */
	if (NULL != actual_constraint_addr_addr) {
	    *actual_constraint_addr_addr = NULL;
	}
	if (NULL != actual_value_addr_addr) {
	    *actual_value_addr_addr = NULL;
	}
    }
    else {				/* non-null kernel data means that we
					   are doing VHDL default binding */

	/* 1: search instantiated component for local generic matching formal 
		by name */

/* ?? begin indent ?? */
    local = NULL;
    looping = True;
    local_scan = TempFirstItem(qqChildren(binding_data->comp_decl_info));
    while (looping) {
	if (TempNullItem(local_scan)) {
	    looping = False;
	}
	else {
	    candidate = SymbInfoValue(local_scan);
	    if (InterfaceConstantDeclInfo == qqSymbInfoKind(candidate) &&
			StringMatch(qqName(candidate), qqName(formal_info))) {
					/* look for generic matching by name */
		local = candidate;
		looping = False;
	    }
	    else {
		local_scan = TempNextItem(local_scan);
	    }
	}
    } /* while */

	/* 2: return local's data */

    if (NULL == local) {		/* no local matching formal */
		/* 2.1: return indication of "open" */

	if (NULL != actual_constraint_addr_addr) {
	    *actual_constraint_addr_addr = NULL;
	}
	if (NULL != actual_value_addr_addr) {
	    *actual_value_addr_addr = NULL;
	}
    }
    else {				/* for local matching formal */
		/* 2.2: return constraint from local */

	if (NULL != actual_constraint_addr_addr) {
					/* caller requested constraint */

	    if (qqConstraintRegionInfo(local) == qqEnclReg(local)) {
					/* constraint is in intf. act. rec. of
					   instantiation */
		constraint_act_rec_base = (VK_tVoidPtr) 
			                         binding_data->inst_intf_data;
	    }
	    else {			/* constraint is in local act. rec. of
					   some package */
		constraint_act_rec_base = (VK_tVoidPtr) 
			qqRegionData(qqConstraintRegionInfo(local));
					/* get package's local act. rec. */
	    }

	    *actual_constraint_addr_addr = (VK_tIndexConstraintAddr) 
		(qqConstraintOffset(local) + (char *) constraint_act_rec_base);
					/* compute constraint's address */
	}

		/* 2.3: return value from local */

	if (NULL != actual_value_addr_addr) {
	    type_info = qqBaseTypeInfo(local);
	    type_class = su_GetTypeInfoTypeClass(type_info);
	    is_composite  = type_class == tc_Array || type_class == tc_Record;
	    if (is_composite) {
		value_address = * (VK_tCompositeAddr *) 
			(qqValueOffset(local) + 
				(char *)binding_data->inst_intf_data);
	    }
	    else {
		value_address = (VK_tScalarAddr) (qqValueOffset(local)
				+ (char *)binding_data->inst_intf_data);
	    }
	    *actual_value_addr_addr = value_address;
	}
    }
/* ?? end indent ?? */
    }
} /* VK_GetActualGeneric */


/* *******************************************************************
*
* VK_GetActualPort - get default binding's actual signal
*
* DESCRIPTION:
*
* HISTORY:
*	24-JUL-91 DSB	Created
*	26-JUL-91 DSB	Handle top-level ports
*
******************************************************************* */
void VK_GetActualPort(kernel_data, formal_info, formal_constraint_addr, 
	actual_constraint_addr_addr, actual_handle_addr, actual_offset_addr)
VK_tVoidPtr             kernel_data;	/* blind pointer to kernel data */
VK_tSymbInfoHandle      formal_info;	/* port identification */
VK_tAnyConstraintAddr   formal_constraint_addr;
				        /* NULL or formal's constraint */
VK_tIndexConstraintAddr *actual_constraint_addr_addr;
				        /* to return actual's constraint */
VK_tSignalHandle        *actual_handle_addr;
				        /* to return actual's handle */
VK_tSignalOffset        *actual_offset_addr;
				        /* to return actual's base offset */
{
    VK_tBoolean        looping;
    TempItem           local_scan;
    VK_tSymbInfoHandle candidate;
    VK_tSymbInfoHandle local;
    VK_tVoidPtr        constraint_act_rec_base;
    tBindingDataPtr    binding_data = (tBindingDataPtr)kernel_data;
/**/

/* ?? WHAT A MESS; THIS NEEDS A LOT OF CLEANUP! ?? */

    if (NULL == binding_data) {		/* no binding_data means that we are
					   doing top-level default binding */

		/* : specify actual of "open"; see if generated code accepts */
/* ?? note: eventually, register whether port can be left unassociated ?? */
	if (NULL != actual_constraint_addr_addr) {
	    *actual_constraint_addr_addr = NULL;
	}
	if (NULL != actual_handle_addr) {
	    *actual_handle_addr = NULL;
	}
	if (NULL != actual_offset_addr) {
	    *actual_offset_addr = NULL;
	}
    }
    else {				/* non-null kernel data means that we
					   are doing VHDL default binding */

	/* 1: search instantiated component for local port matching formal by 
		name */

/* ?? begin indent ?? */
    local = NULL;
    looping = True;
    local_scan = TempFirstItem(qqChildren(binding_data->comp_decl_info));
    while (looping) {
	if (TempNullItem(local_scan)) {
	    looping = False;
	}
	else {
	    candidate = SymbInfoValue(local_scan);
	    if (InterfaceSignalDeclInfo == qqSymbInfoKind(candidate) &&
			StringMatch(qqName(candidate), qqName(formal_info))) {
					/* look for port matching by name */
		local = candidate;
		looping = False;
	    }
	    else {
		local_scan = TempNextItem(local_scan);
	    }
	}
    } /* while */

	/* 2: return local's data */

    if (NULL == local) {		/* no local matching formal */
		/* 2.1: return indication of "open" */

	if (NULL != actual_constraint_addr_addr) {
	    *actual_constraint_addr_addr = NULL;
	}
	if (NULL != actual_handle_addr) {
	    *actual_handle_addr = NULL;
	}
	if (NULL != actual_offset_addr) {
	    *actual_offset_addr = NULL;
	}
    }
    else {				/* for local matching formal */
		/* 2.2: return constraint from local */

	if (NULL != actual_constraint_addr_addr) {
					/* caller requested constraint */

	    if (qqConstraintRegionInfo(local) == qqEnclReg(local)) {
					/* constraint is in intf. act. 
					   rec. of instantiation */
		constraint_act_rec_base = (VK_tVoidPtr) 
			                         binding_data->inst_intf_data;
	    }
	    else {			/* constraint is in local act. rec. of
					   some package */
		constraint_act_rec_base = (VK_tVoidPtr) 
			qqRegionData(qqConstraintRegionInfo(local));
					/* get package's local act. rec. */
	    }

	    *actual_constraint_addr_addr = (VK_tIndexConstraintAddr) 
			(qqConstraintOffset(local) + 
				(char *) constraint_act_rec_base);
					/* compute constraint's address */
	}

		/* 2.3: return signal identification from local */

	if (NULL != actual_handle_addr) {
	    *actual_handle_addr = * (VK_tSignalHandle *)
		(qqHandleOffset(local) + (char *)binding_data->inst_intf_data);
	}
	if (NULL != actual_offset_addr) {
	    *actual_offset_addr = * (VK_tSignalOffset *)
		(qqOffsetOffset(local) + (char *)binding_data->inst_intf_data);
	}
    }
/* ?? end indent ?? */
    }
} /* VK_GetActualPort */


/* *******************************************************************
*
* CheckModes - Check that modes of formal port and actual signal agree
*
* HISTORY:
*	14-AUG-91 DSB	Created
*
******************************************************************* */
static void CheckModes(formal_info, actual_info)
VK_tSymbInfoHandle formal_info;	/* in: formal port symb. info. handle */
VK_tSymbInfoHandle actual_info;	/* in: actual port or signal symb. info. 
					handle */
{
    VK_tMode formal_mode;
    VK_tMode actual_mode;
/**/
    formal_mode = qqIntfMode(formal_info);
    actual_mode = qqIntfMode(actual_info);


    if (VK_LinkageMode == formal_mode) {
	ReportKernelError("cannot simulate linkage port", 
		qqName(formal_info), "", 0);
    }
    else if (VK_InternalMode == actual_mode) {
	/* is okay */
    }
    else {
/* ?? begin indent ?? */

/* ?? when error reporting is redesigned, would like to report context of
	these error; specifically, e.g., "during default binding ... ?? */

/* ?? clean: this could be done a little more neatly and compactly ?? */
/* LRM: section 1.1.1.2 defines mode checks */
    switch (formal_mode) {
	case VK_InMode:	
		switch (actual_mode) {
		    case VK_InMode:
		    case VK_InoutMode:
		    case VK_BufferMode:
			/* is okay */
			break;
		    case VK_OutMode:
			ReportKernelError(
"port mode mismatch: formal port of mode in, actual port of mode out",
				qqName(formal_info), "", 0);
			break;

		    case VK_LinkageMode:
		    default:
			ReportInternalError("Unexpected case in CheckModes",
				"in => linkage", "", 0);
			break;
		} /* switch */
	    break;
	case VK_InoutMode:	
		switch (actual_mode) {
		    case VK_InoutMode:
			/* is okay */
			break;
		    case VK_InMode:
			ReportKernelError(
"port mode mismatch: formal port of mode inout, actual port of mode in",
				qqName(formal_info), "", 0);
			break;
		    case VK_OutMode:
			ReportKernelError(
"port mode mismatch: formal port of mode inout, actual port of mode out",
				qqName(formal_info), "", 0);
			break;
		    case VK_BufferMode:
			ReportKernelError(
"port mode mismatch: formal port of mode inout, actual port of mode buffer",
				qqName(formal_info), "", 0);
			break;

		    case VK_LinkageMode:
		    default:
			ReportInternalError("Unexpected case in CheckModes",
				"inout => linkage", "", 0);
			break;
		} /* switch */
	    break;
	case VK_OutMode:	
		switch (actual_mode) {
		    case VK_OutMode:
		    case VK_InoutMode:
			/* is okay */
			break;
		    case VK_InMode:
			ReportKernelError(
"port mode mismatch: formal port of mode out, actual port of mode in",
				qqName(formal_info), "", 0);
			break;

		    case VK_BufferMode:
			ReportKernelError(
"port mode mismatch: formal port of mode out, actual port of mode buffer",
				qqName(formal_info), "", 0);
			break;

		    case VK_LinkageMode:
		    default:
			ReportInternalError("Unexpected case in CheckModes",
				"in => linkage", "", 0);
			break;
		} /* switch */
	    break;
	case VK_BufferMode:	
		switch (actual_mode) {
		    case VK_BufferMode:
			/* is okay */
			break;
		    case VK_InMode:
			ReportKernelError(
"port mode mismatch: formal port of mode buffer, actual port of mode in",
				qqName(formal_info), "", 0);
			break;

		    case VK_InoutMode:
			ReportKernelError(
"port mode mismatch: formal port of mode buffer, actual port of mode inout",
				qqName(formal_info), "", 0);
			break;

		    case VK_OutMode:
			ReportKernelError(
"port mode mismatch: formal port of mode buffer, actual port of mode out",
				qqName(formal_info), "", 0);
			break;

		    case VK_LinkageMode:
		    default:
			ReportInternalError("Unexpected case in CheckModes",
				"in => linkage", "", 0);
			break;
		} /* switch */
	    break;
	default:
	    break;
    } /* switch */
/* ?? end indent ?? */
    }
} /* CheckModes */


/* *******************************************************************
*
* VK_ElaborateDefaultEntity - implement default binding indication
*
* HISTORY:
*	22-JUL-91 DSB	Created
*	24-JUL-91 DSB	Adding most of default entity processing
*	15-AUG-91 DSB	Check for lack of architecture
*	19-NOV-91 DKC	Added region for instance and tidied memory allocation
*
******************************************************************* */
void VK_ElaborateDefaultEntity(instance_info, instance_intf_data)
VK_tSymbInfoHandle instance_info;	/* identification of component
					   instantiation */
VK_tIARHeaderPtr instance_intf_data;	/* interface activation record for 
					   elaboration function */
{
    tBindingDataPtr binding_data;		/* ?? rename type ?? */
    VK_tSymbInfoHandle entity_info;
    TempItem local_scan, formal_scan;
    VK_tBoolean looping;
    VK_tSymbInfoHandle local;
    VK_tSymbInfoHandle candidate;
    VK_tSymbInfoHandle formal;
    VK_tLARHeaderPtr equiv_lcl_data;    
    VK_tIARHeaderPtr entity_intf_data;
    VK_tBindFnPtr bind_fn;
    VK_tSymbInfoHandle arch_info;
    VK_tGenericElabFnPtr elab_fn;
    tTempBuf256 ent_buf;
/**/
/* Note:  General approach:  Generated code contains special default-binding 
	function for each entity; binding function is registered with kernel; 
	kernel calls binding function to set up associations; binding function
	calls kernel to get actuals; kernel uses symbolic information to read 
	component instantiation's IAR; binding function creates and assembles
	IAR for entity (reading is easier for kernel than assembling IAR); 
	binding functions returns with completed IAR for design entity; kernel
	can then call elaboration function for design entity or configuration
	*/
/*
Steps for VK_ElaborateDefaultEntity:
1.  Select entity declaration (not well-defined by LRM; use simplified rules)
2.  Check generic and port type and mode matching rules (using symbolic 
	information)
3.  Call default binding function for entity
	- when it calls back, provide actuals from component instantiation's
		IAR (using symbolic information)
4.  Select latest architecture body
5.  Elaborate design entity
*/

/* : 
- get component declaration from component instantiation
- check matching rules:
	- match generic and ports by name
	- types must match 
	- modes must be compatible according to rules
- if using additional rules:
	- allocate LAR to represent binding indication
	- allocate IAR for entity and copy from component instantiation's
		IAR (use intf_data_size)
- otherwise (full VHDL options):
	- allocate local activation record for implicit binding indication
		(fixed size)
	- allocate interface activation record for design entity (size is
		registered)
	- fill in generic constraints (probably by calling special function)
	- associate generics:
		- copy in constraints if unconstrained
		- check input vs. generic constraints if constrained
		- copy in scalar values or pointers
	- fill in port constraints (probably by calling special function)
	- associate ports:
		- copy in constraints if unconstrained
		- check input vs. generic constraints if constrained
		- compute default values (probably by calling special function)
		- create formal ports and associate
- fill in standard interface activation record fields (caller_LARP,
	context_LARP, config_func)
- call elaboration function with interface activation record
*/

	/* 1: select default entity */

    entity_info = FindDefaultEntity(instance_info);

	/* 2: check that generics and ports match appropriately */
/* ?? clean: this should be a separate function, as FindDefaultEntity is ?? */

    for (local_scan = TempFirstItem(qqChildren(qqCompDeclInfo(instance_info)));
		! TempNullItem(local_scan); 
		local_scan = TempNextItem(local_scan)) {
					/* for each interface object of
					   component declaration */
	local = SymbInfoValue(local_scan);

		/* 2.1: check that entity has corresponding formal */

	formal = NULL;
	looping = True;
	formal_scan = TempFirstItem(qqChildren(entity_info));
	while (looping) {
	    if (TempNullItem(formal_scan)) {
		looping = False;
	    }
	    else {
		candidate = SymbInfoValue(formal_scan);
		if ((InterfaceSignalDeclInfo == qqSymbInfoKind(candidate) ||
			InterfaceConstantDeclInfo == qqSymbInfoKind(
				candidate)) &&
			StringMatch(qqName(candidate), qqName(local))) {
		    formal = candidate;
		    looping = False;
		}
		else {
		}
	    }
	    formal_scan = TempNextItem(formal_scan);
	} /* while */
	if (NULL == formal) {
	    ReportKernelError("default binding: no formal matching local", 
		qqName(local), "", 0);
	}

		/* 2.2: check that formal is of correct type */

	if (qqBaseTypeInfo(formal) != qqBaseTypeInfo(local)) {
	    ReportKernelError(
		"default binding: local and formal types don't match", 
		qqName(local), "", 0);
	}

		/* 2.3: for port, check that modes are compatible */
	
	if (InterfaceSignalDeclInfo == qqSymbInfoKind(local)) {
	    CheckModes(formal, local);
	}
    }

	/* 3: allocate various activation record and other data */

	/* 3.1: allocate local activation record for implicit binding 
		indication */

    AllocateMemoryZero(equiv_lcl_data, VK_tLARHeader, 1);

    TK_SetRegionHandle(equiv_lcl_data,
           CreateComponentInstantiation(
                    TK_GetRegionHandle(VK_GetCallerLARP(instance_intf_data)),
	                                qqName(instance_info)));

    VK_SetIARP(equiv_lcl_data,instance_intf_data);

	/* 3.2 : allocate interface activation record for entity */
 
    AllocateBlockZero(entity_intf_data, VK_tIARHeader, 
                      qqIntfDataSize(entity_info));

    VK_SetCallerLARP (entity_intf_data, equiv_lcl_data);
    VK_SetContextLARP(entity_intf_data, NULL);

	/* 3.3 : set up context data for calls to VK_GetActualGeneric, etc. */

    AllocateMemory(binding_data, tBindingData, 1);
    binding_data->comp_decl_info = qqCompDeclInfo(instance_info);
    binding_data->comp_inst_info = instance_info;
    binding_data->inst_intf_data = instance_intf_data;

	/* 4: call default binding indication association function */

    bind_fn = qqBindingFunction(entity_info);
    (*bind_fn)(binding_data, entity_intf_data);

/* ?? (REVIEW: Is this the interface we want for the default binding function?
	Can we simplify what the kernel has to know about? Maybe we can
	provide an RTL routine.  Maybe we can provide part the test kernel as
	utility functions an integrator would be expected to use) ?? */

	/* 5: select latest architecture */

    arch_info = FindLatestArchitecture(entity_info);
    if (NULL == arch_info) {		/* error if no architecture for E */
	sprintf(ent_buf, "%s.%s", qqName(qqEnclReg(entity_info)), 
		qqName(entity_info));	/* qqEnclReg gets entity's library */
	ReportKernelError("no architecture for entity", ent_buf, "", 0);
    }

	/* 6: get and call architecture's elaboration function */

    elab_fn = qqElaborationFunction(arch_info);
    (*elab_fn)(entity_intf_data);
} /* VK_ElaborateDefaultEntity */


/* *******************************************************************
*
* TK_SpecifyTopUnit - function to specify top architecture, configuration 
*	declaration, or entity declaration
*
* NOTE: TEST KERNEL ONLY
*
* HISTORY:
*	24-JUL-91 DSB	Created
*
******************************************************************* */
void TK_SpecifyTopUnit(unit_info)
VK_tSymbInfoHandle unit_info;	/* symb. info. handle to identify top unit */
{
/**/
    VERIFY_ERROR(NULL != unit_info);

    SpecifiedTopUnit = unit_info;
} /* TK_SpecifyTopUnit */


/* *******************************************************************
*
* ElaborateModel - elaborate VHDL model starting at specified top unit
*
* DESCRIPTION:
*	Starts at specified top unit (entity, architecture, or configuration).
*	If entity was specified, uses latest architecture of that entity.
*
* HISTORY:
*	24-JUL-91 DSB	Split out
*	25-JUL-91 DSB	Removed call to VK_ElaborateModel
*	26-JUL-91 DSB	Handle top-level generics and ports
*	15-AUG-91 DSB	Check for lack of architecture
*	16-DEC-91 DKC	Allocated top_context permanent space
*
******************************************************************* */
void ElaborateModel()
{
    VK_tSymbInfoHandle   entity_info;
    VK_tSymbInfoHandle   top_unit_info;
    VK_tIARHeaderPtr     entity_intf_data;
    VK_tBindFnPtr        bind_fn;
    VK_tGenericElabFnPtr elab_fn;
    VK_tLARHeaderPtr     top_context;
    tTempBuf256          ent_buf;
/**/
    if (NULL == SpecifiedTopUnit) {
	ReportKernelError(
		"No top-level unit registered (VK_RegisterUnits)", 
		"", "", 0);
    }

	/* 1: determine entity declaration for specified unit */

    if (EntityDeclInfo == qqSymbInfoKind(SpecifiedTopUnit)) {
					/* entity was specified */
	entity_info = SpecifiedTopUnit;
	top_unit_info = FindLatestArchitecture(SpecifiedTopUnit);
					/* assume latest architecture */
	if (NULL == top_unit_info) {	/* error if no architecture for E */
	    sprintf(ent_buf, "%s.%s", qqName(SpecifiedTopUnit), 
		qqName(qqEnclReg(SpecifiedTopUnit)));
					/* qqEnclReg gets entity's library */
	    ReportKernelError("no architecture for entity", ent_buf, "", 0);
	}
    }
    else if (ArchitectureBodyInfo == qqSymbInfoKind(SpecifiedTopUnit)) {
					/* architecture was specified */
	entity_info = qqExtReg(SpecifiedTopUnit);
	top_unit_info = SpecifiedTopUnit;
    }
    else if (ConfigurationDeclInfo == qqSymbInfoKind(SpecifiedTopUnit)) {
					/* configuration was specified */
	entity_info = qqExtReg(qqExtReg(SpecifiedTopUnit));
	top_unit_info = SpecifiedTopUnit;
    }
    else {
	/* ERROR */
	entity_info = NULL;
    }



	/* 2: allocate interface activation record for top design entity */

    AllocateBlockZero(entity_intf_data,VK_tIARHeader,
                                       qqIntfDataSize(entity_info));

    AllocateMemoryZero(top_context,VK_tLARHeader,1); /* required for ports */

    VK_SetCallerLARP  (entity_intf_data, top_context);
    TK_SetRegionInfo  (top_context,      entity_info);

/* ?? note: need note is VKI spec. that all fields not known to kernel must
	be initialized to zero ?? */

	/* 3: call default binding indication association function */

    bind_fn = qqBindingFunction(entity_info);
    (*bind_fn)(NULL, entity_intf_data);
					/* pass NULL for kernel_data to 
					   indicate top-level elaboration to
					   VK_GetActualGeneric, etc. */

	/* 4: call elaboration function to elaborate design entity */

    elab_fn = qqElaborationFunction(top_unit_info);
    VERIFY_ERROR(NULL != elab_fn);
    (*elab_fn)(entity_intf_data);		/* call elaboration function */

} /* ElaborateModel */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */
<static> Fntype RtnName(parm)				/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif
