/*****************************************************************************
*
* Copyright (c) 1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* FILE: TKFileUtils.c - Utilities dealing with filenames and
*	environment variables.
*
* DESCRIPTION:
*	Based upon work from STSrcDisplay in the VLK, by GJB.
*
* TYPES:
*
* CONSTANTS:
*
* FUNCTIONS:
*
* HISTORY:
*	22-OCT-91 MNP	Created
*
* SCCS ID: @(#)TKFileUtils.c	42.1 1/10/92
*
*****************************************************************************/
#include "Copyright.h"
static char SCCS_ID[] = "@(#)TKFileUtils.c	42.1 1/10/92";

#include <sys/param.h>				/* for MAXPATHLEN */
#include "TKFileUtils.h"			/* this module's header */
#include "TKTemp.h"				/* For string and mem fns */

#define PATH_SEP_CHAR		'/'		/* Path separator char */
#define PATH_SIZE		MAXPATHLEN	/* largest expected path */


/*****************************************************************************
*
* CopyStringN - return allocated copy of N chars of a string
*
* DESCRIPTION:
*
* HISTORY:
*	22-OCT-91 MNP	Created
*
*****************************************************************************/

static char * CopyStringN(s, n)
char *s;
int n;
{
    char *t;
/**/	
    AllocateMemory(t, char, n+1);
    strncpy(t, s, n);
    t[n] = '\0';
    return t;
} /* CopyStringN */


/*****************************************************************************
*
* AppendCharacter - appends a character to a char pointer and updates it 
*
* DESCRIPTION:
*
* HISTORY:
*	22-OCT-91 MNP	Created
*
*****************************************************************************/

static void AppendCharacter(c, p)
char c;
char **p;
{
    *(*p)++ = c;
} /* AppendCharacter */;


/*****************************************************************************
*
* AppendVariable - appends the expansion of a variable to a char pointer
*
* DESCRIPTION:
*
* HISTORY:
*	??-???-90 GJB	Created
*	22-OCT-91 MNP	Moved from VLK STSrcDisplay
*
*****************************************************************************/

static tFileUtilStatus AppendVariable(var_name, p)
char * var_name;		/* in: environment var to expand */
char **p;			/* in/out: where to put expansion */
{
    extern char * getenv();
    char * expanded_var;
    tFileUtilStatus status;
/**/    
    if ((expanded_var = getenv(var_name)) == NULL)
    {
	status = UnknownVarInPath;
    }
    else
    {
	*(*p) = '\0';
	strcat((*p), expanded_var);		/* bad idea ?? */
	(*p) += strlen(expanded_var);
	status = FileSuccess;
    }
    return status;
} /* AppendVariable */


/*****************************************************************************
*
* ExpandPathElement - Expand one element of a path
*
* DESCRIPTION:
*
* HISTORY:
*	??-???-90 GJB	Created
*	22-OCT-91 MNP	Moved from VLK STSrcDisplay
*
*****************************************************************************/

static tFileUtilStatus
    ExpandPathElement(orig_path_elm, new_path_elm, bad_var_name)
char **orig_path_elm;		/* in: element to expand */
char **new_path_elm;		/* out: expanded element */
char **bad_var_name;		/* out: errorful piece to yell about */
{
    int first_char;
    char var_name[81], * var_name_p;
    tFileUtilStatus status;
    int len;
/**/
    first_char = 1;
    status     = FileSuccess;
    while (status == FileSuccess &&
	   **orig_path_elm && **orig_path_elm != '/')
    {
	switch(**orig_path_elm)
	{
	  case '~':
	    if (first_char && *(*orig_path_elm+1) == '/')
	    {
				/* "~/" translates to $HOME */
		status        = AppendVariable("HOME", new_path_elm);
		*bad_var_name = "HOME";
	    }
	    else
	    {
				/* "~something" is not supported... */
		status        = BadTildeInPath;
		var_name_p    = strchr(*orig_path_elm, '/');
		len           = (var_name_p == NULL) ?
		    			strlen(*orig_path_elm) :
					    var_name_p - *orig_path_elm;
		*bad_var_name = CopyStringN(*orig_path_elm, len);
	    }
	    (*orig_path_elm)++;
	    break;
	    
	  case '$':
	    (*orig_path_elm)++;
	    var_name_p = var_name;
	    while (**orig_path_elm		&&
		   **orig_path_elm != '/'	&&
		   **orig_path_elm != '$')
		*var_name_p++ = *(*orig_path_elm)++;
	    *var_name_p   = '\0';
	    status        = AppendVariable(var_name, new_path_elm);
	    *bad_var_name = CopyPrivStr(var_name);
	    break;
	    
	  default:
	    AppendCharacter(*(*orig_path_elm)++, new_path_elm);
	    break;
	}
	first_char = 0;
    }
    return status;
} /* ExpandPathElement */


/*****************************************************************************
*
* ExpandPathName - expands tildes and variables in a pathname  
*
* DESCRIPTION:
*
* HISTORY:
*	??-???-90 GJB	Created
*	22-OCT-91 MNP	Moved from VLK STSrcDisplay
*
*****************************************************************************/

tFileUtilStatus ExpandPathName(old_file_name, new_file_name, bad_var_or_tilde)
char * old_file_name;		/* in: filename to look at */
char ** new_file_name;		/* out: the expanded name */
char ** bad_var_or_tilde;	/* out: if error, talk about this */
{
    char * old_file_name_p;
    char * new_file_name_p;
    tFileUtilStatus status;
/**/
    if (!old_file_name || *old_file_name == '\0')
	return IntError;
    
/*  *new_file_name = (char *) malloc(PATH_SIZE); */
    AllocateMemory(*new_file_name, char, PATH_SIZE);

    new_file_name_p = *new_file_name;
    old_file_name_p = old_file_name;
    status = FileSuccess;
    while (status == FileSuccess && old_file_name_p && *old_file_name_p)
    {
	status = ExpandPathElement(&old_file_name_p, &new_file_name_p,
				   bad_var_or_tilde);
	if (status == FileSuccess && *old_file_name_p == '/')
	{
	    AppendCharacter(*old_file_name_p++, &new_file_name_p);
	}
    }
    *new_file_name_p++ = '\0';
    return status;
} /* ExpandPathName */


/*****************************************************************************
*
* FilenameBase - Like unix 'basename' 
*
* DESCRIPTION:
*
* HISTORY:
*	22-OCT-91 MNP	Created
*
*****************************************************************************/

VK_tCString FilenameBase(fname)
VK_tCString fname;
{
    VK_tCString  p;
/**/

    p = strrchr(fname, PATH_SEP_CHAR);
    return (p ? p+1 : fname);

} /* FilenameBase */


/*****************************************************************************
*
* FilenamesEqual - Compare filenames.
*
* DESCRIPTION:
*	 if A exactly matches B -> True
*	 else if base of A exactly matches B -> True
*	 else if A exactly matches base of B -> True
*	 else if base of A exactly matches base of B -> True
*
* HISTORY:
*	22-OCT-91 MNP	Created
*
*****************************************************************************/

VK_tBoolean FilenamesEqual(a, b)
VK_tCString a, b;
{
/**/
    if (!strcmp(a, b) || !strcmp(FilenameBase(a), b)
	    || !strcmp(a, FilenameBase(b))
	    || !strcmp(FilenameBase(a), FilenameBase(b)))
	return True;
    else 
	return False;

} /* FilenamesEqual */

