#include "VKConfig.h"
static char SCCS_ID[] = "@(#)TKFiles.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1991-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKFiles.c
*
* DESCRIPTION:  
*	This module contains routines that implement VHDL file I/O.
*
*	NOTE: THIS WHOLE DAMN MODULE NEEDS TO BE REVIEWED.  
*
*	Style and integration problems: error-checking if statements should 
*	have elses; automatic variable initialization is counted on; 
*	lower-case names should have ynderlines between words; SPI lists ARE 
*	NOT TO BE USED! - we have TempList, etc. <<done>>; hard-coded "1024"; 
*	"constraint" for "constrained"
*
* TYPES:
*	tHeaderRecord -		first record in host file.
*	tSizeRecord -		record preceding each data block in host file.
*	tFileSignature -	signature to identify host file.
*
* GLOBAL VARIABLES:
*	FileList - 		list of created files.
*
* ROUTINES:
*   External:
*	VK_ReadFile -		read data from file.
*	VK_WriteFile -		write data to file.
*	VK_DestroyFile -	destroy object referenced by file_handle.
*	VK_CloseFile -		close file.
*	VK_IsEndOfFile -	detect end of file condition.
*	InitFileIO -		initialization routine.
*
*   Internal:
*	ReadHeaderRecord -	reads the first record from host file.
*	WriteHeaderRecord -	writes the first record to host file.
*	ReadAhead -		initialize file for further read operations.
*	FileExists -		tell if a file already exists.
*	Signature -		create a signature for file.
*
* HISTORY:
*	15-MAR-91 RGM	Created
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	23-SEP-91 MNP	Converted to TK List fns from SPI ones.
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*
* SCCS ID: @(#)TKFiles.c	42.1 1/10/92
*
******************************************************************* */

#include "TKFiles.h"

#include "SPI.h"			/* for SPI types and list routines */

#include "TKRegister.h"			/* for SIH_Std__TextIO__Text */

typedef Integer4 tFileSignature;	/* an integer to identify host file */

typedef Integer4 tSizeRecord;		/* size of following data block in */
 					/* host file */

typedef struct sHeaderRecord {		/* first record in host file */
	VK_tDataLength length;		/* size of elements in file in words */
	VK_tElementCount count;		/* count of elements in file */
	tFileSignature signature;	/* sig. to identify file */
    } tHeaderRecord;


static TempList FileList;		/* global list of created files */


#define DEFAULT_DIR		"./"	/* string representing default dir. */


/* NOTE on conversion from SPI lists to TKLists - prepend "Temp" to the
 * following SPI List thingies to get the TK versions:
 *
 *   	Item			Node			Value(I)
 *	FirstItem(L)		NullItem(I)		NextItem(I)
 *	RemoveItem(I, N)
 *
 *   Except for 
 *
 *   	NewList(O) -> TempNewList()
 *
 *  The following two macros show diffs, also.   23-SEP-91 MNP
 */

/* Add a file to global list of  created files. */

#define AddFileList(fh) TempAddLast(FileList, fh)	
 /* AddLast(FileList, NewIntegerValue4((Integer4)(fh))) */ 

/* Return file handle, given a TempNode. */

#define FILE_HANDLE(N) ((VK_tFileHandle) (N))	
 /* ((VK_tFileHandle) qIntVal4((N))) */


/* kernel internal error handler */

void iReadAhead();
#define ReadAhead(handle)		iReadAhead(handle)

void iReadHeaderRecord();
#define ReadHeaderRecord(handle)	iReadHeaderRecord(handle)

void iWriteHeaderRecord();
#define WriteHeaderRecord(handle)	iWriteHeaderRecord(handle)

VK_tBoolean iFileExists();
#define FileExists(name, mode, length, count, dup)	\
       iFileExists(name, mode, length, count, &(dup))

tFileSignature iSignature();
#define Signature(i)			iSignature(i)




/* *******************************************************************
*
* InitFileIO - initialization routine.
*
* HISTORY:
*	21-MAR-91 RGM	Created
*
******************************************************************* */

void InitFileIO()
{
/**/
    FileList = TempNewList();		/* NewList(Object);*/

} /* InitFileIO */




/* *******************************************************************
*
* ReadAhead - initialize file for further read operations.
*
* PRE:  Parameter handle points to a file object created for reading.
*
* POST: File object is updated with size of next data block in host file,
*       if EOF is found the state of the file object is marked as at EOF.
*       Host file pointer points to next block of data.
*
* HISTORY:
*	20-MAR-91 RGM	Created
*	06-JUN-91 DSB	Added text-mode files
*
******************************************************************* */

static void iReadAhead(file_handle)
VK_tFileHandle file_handle;
{
    Integer4 n;
    tSizeRecord size;			/* size of record in bytes */
    char ch;
/**/
    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null file object handle", __FILE__, __LINE__);
    }

    if (file_handle->file_mode != VK_InMode) {
        ReportInternalError("Internal inconsistency:", 
		"file is not open for reading", __FILE__, __LINE__);
    }

    if (file_handle->host_fp == stdin) {
	/* don't read ahead on standard input */
    }
    else if (file_handle->is_text_file) {
	ch = fgetc(file_handle->host_fp);
	if (EOF == ch) {
            file_handle->file_state = fs_Eof;
	}
	else {
	    ungetc(ch, file_handle->host_fp);
	}
    }
    else {
/* ?? begin indent ?? */

    n = fread((char *)&size, sizeof(size), 1, file_handle->host_fp);

    if (n != 1) {

        /* check to see if we have an empty file */

        if (feof(file_handle->host_fp)) {
            file_handle->file_state = fs_Eof;
            file_handle->next_element_size = 0;
	}
        else {
            ReportKernelError("system routine (fread) failed", 
		file_handle->file_name, "", 0);
        }
    }
    else {

        file_handle->next_element_size = size;	/* size is in bytes */
    }
/* ?? end indent ?? */

    }

} /* ReadAhead */




/* *******************************************************************
*
* ReadHeaderRecord - reads the first record of host file.
*
* PRE:  Parameter handle points to a file object created for reading.
*
* POST: Data read from header record in host file has been checked against
*       data in file object pointed by handle as follows:
*       if element count > 0 (both files are constraint) then they must be
*       the same (element count), otherwise no restriction on element count,
*       element sizes must be the same.
*
* HISTORY:
*	18-MAR-91 RGM	Created
*
******************************************************************* */

static void iReadHeaderRecord(file_handle)
VK_tFileHandle file_handle;
{
    tHeaderRecord hrec;
    int n;
/**/
    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null file object handle", __FILE__, __LINE__);
    }

    if (file_handle->file_mode != VK_InMode) {
        ReportInternalError("Internal inconsistency:", 
		"file is not open for reading", __FILE__, __LINE__);
    }

    if (file_handle->element_count < 0) {
        ReportInternalError("Internal inconsistency:",
	        "element count of file object has illegal value",
		__FILE__, __LINE__);
    }

    n = fread((char *)&hrec, sizeof(hrec), 1, file_handle->host_fp);
    if (n != 1) {

        /* PA: may want to check for premature end of file here, */
        /*     or for a smaller header -> not a VHDL file */

        ReportKernelError("system routine (fread) failed",
		file_handle->file_name, "", 0);
    }

    if (hrec.signature != Signature(hrec.length)) {
        ReportKernelError("Host file is not a VHDL binary data file", 
		file_handle->file_name, "", 0);
    }

    if (hrec.length < 0 || hrec.count < 0) {
        ReportKernelError(
		"Illegal element size or count in VHDL binary data file",
		file_handle->file_name, "", 0);
    }

    /* check that element_length is the same as the one in file */

    if (file_handle->element_length != hrec.length) {
        ReportKernelError(
		"Incompatible element size", 
		file_handle->file_name, "", 0);
    }

    if (file_handle->element_count > 0) {

        if (hrec.count > 0) {

            /* both files are constraint, make sure they are the same size */

           if (file_handle->element_count != hrec.count) {
		ReportKernelError(
"VHDL data file has different element count than current designated subtype", 
			file_handle->file_name, "", 0);
	   }
	}
        /* else constraint file vs. unconstraint file */
     }
     /* else unconstraint file vs. either constraint or unconstraint files */


} /* ReadHeaderRecord */




/* *******************************************************************
*
* WriteHeaderRecord - writes the first record to host file.
*
* PRE:  Parameter handle points to a file object created for writing.
*
* POST: A header record with element length, count and signature obtained
*       from file object is written to host file.
*
* HISTORY:
*	18-MAR-91 RGM	Created
*
******************************************************************* */

static void iWriteHeaderRecord(file_handle)
VK_tFileHandle file_handle;
{
    tHeaderRecord rec;
    int n;
/**/
    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null file object handle", __FILE__, __LINE__);
    }

    if (file_handle->file_mode != VK_OutMode) {
        ReportInternalError("Internal inconsistency:", 
		"file is not open for writing", __FILE__, __LINE__);
    }

    rec.length = file_handle->element_length;		/* length in words */
    rec.count = file_handle->element_count;
    rec.signature = Signature(file_handle->element_length);

    n = fwrite((char *)&rec, sizeof(rec), 1, file_handle->host_fp);
    if (n != 1) {
	ReportKernelError("system routine (fwrite) failed", 
		file_handle->file_name, "", 0);
    }

} /* WriteHeaderRecord */




/* *******************************************************************
*
* FileExists - tell if a file already exists
*
* PRE:  Parameters name, mode, size and count are data of a file not in
*       the global list of created files.
*
* POST: Return value is True if an existing file in the global list of
*       created files has the same data as the parameters, this file can be
*       either mode (Read or Write). If such a file exists, parameter dup
*       is set to the address of the file object. Otherwise, return value
*       is False.
*
* HISTORY:
*	19-MAR-91 RGM	Created
*
******************************************************************* */

static VK_tBoolean iFileExists(name, mode, length, count, dup)
VK_tCString name;		/* name of file */
VK_tMode mode;			/* mode of file */
VK_tDataLength length;		/* length of element in words */
VK_tElementCount count;		/* count of elements */
VK_tFileHandle *dup;		/* handle to an already existing file */
{
    TempItem I;			/* temporary list item */
    VK_tBoolean found;		/* True if file already exists */
    VK_tFileHandle file_handle;	/* temporary file handle */
/**/
    *dup = (VK_tFileHandle) NULL;
    found = False;
    
    for (I=TempFirstItem(FileList); ! TempNullItem(I); I=TempNextItem(I)) {
	/*  for (I=FirstItem(FileList); ! NullItem(I); I=NextItem(I)) { */

        file_handle = FILE_HANDLE(TempValue(I));
        if (strcmp(file_handle->file_name, name) == 0) {
	    if (file_handle->file_mode != mode) {
			/* ?? CLEAN UP WORDING ?? */
		ReportKernelError(
	"Two files with same (translated) name have incompatible modes",
			name, "", 0);
	    }
	    else if (file_handle->element_length == length && 
			file_handle->element_count == count) {
                *dup = file_handle;
                found = True;
	    }
            else {
                /* files have different type */
                /* PA: to allow arrays of same elements but different lengths */
                /*     (constraint, unconstraint) change the if condition   */

                if (mode == VK_OutMode) {
                    ReportKernelError(
			"Can create files with same name but different modes", 
			file_handle->file_name, "", 0);
	        }
                /* else duplicates are allowed for read mode files */
	    }
	 } /* if */
    } /* for */

    return found;
} /* FileExists */




/* *******************************************************************
* 
* VK_CreateFile - create a file object and return a handle to it.
* 
* PRE:  Parameters specify what kind of file object to create.
*
* POST: Returns pointer to a new file object such that:
*       If file_mode is VK_InMode, file object is initialized and data
*       is checked against data found in host file. This host file must
*       be a file previoulsy written by this routine. Element lengths are
*       required to be the same, duplicates are allowed.
*       If file_mode is VK_OutMode, file object is initialized with
*       parameter information. Duplicate files are allowed only if
*       element length and count are the same.
*
* HISTORY:
* 	15-MAR-91 RGM	Created
*	06-JUN-91 DSB	Removed old else_ parameter
*	06-JUN-91 DSB	Added text-mode files
*	14-AUG-91 DSB	Recognize STD_INPUT and STD_OUTPUT
*	21-AUG-91 DSB	Fixed bug: std. file logical names not recognized
* 
******************************************************************* */

VK_tFileHandle VK_CreateFile(file_info, encl_region, name_length,
	logical_name, file_mode, element_length, element_count)
VK_tSymbInfoHandle file_info;	/* lexical file identification */
VK_tLARHeaderPtr encl_region;	/* encl. region loc. act. rec. */
VK_tStringLength name_length;	/* length of logical name */
VK_tVHDLString logical_name;	/* file logical name string */
VK_tMode file_mode;		/* VHDL mode of file */
VK_tDataLength element_length;		/* element length in words */
VK_tElementCount element_count;	/* element count */
{
    VK_tFileHandle file_handle;	/* handle of new object */
    VK_tFileHandle dup;		/* handle of a duplicate file object */
    tTempBuf1024 buf;		/* C representation of logical name */
    VK_tCString tlogname;	/* translated logical name */
    VK_tCString fname;		/* complete host file name */
    int i, j = 0;		/* loop and array indices */
    int name_len;		/* length of C name */
    VK_tBoolean exists;		/* to find files with same name and modes */
    SeverityLevel old_sev;	/* to save current severity level */
/**/

    /* NOTE: */
    /* parameters file_info and encl_region are not used at this time */

    if (logical_name == NULL) {
        ReportInternalError("Internal inconsistency:", "null logical name", 
		__FILE__, __LINE__);
    }

    if (element_count < 0 || element_length < 0 || name_length < 0) {

        ReportInternalError("Internal inconsistency:",
	   "illegal value for element count or size or length of logical name"
		, __FILE__, __LINE__);
    }

    /* convert logical name to a C string */
    /* NOTE: */
    /* this conversion will work for ASCII and is subject to change */

    for (i=0;i<=name_length - 1;i++) {
        buf[j] = logical_name[i];
        if (buf[j] == '\0') {
            /* Cannot report warnings at this point */
	    /* ReportKenrelError("Warning, null character in VHDL string", "", __FILE__, __LINE__); */
	}
        j++;
    } /* for */

    buf[j] = '\0';
    if (j ==  0) {
        ReportKernelError("null translation to C of VHDL file name", 
		logical_name, "", 0);
    }
    
    /* translate logical name */

    old_sev = SetReportLevel(Error);    /* need to worry about error handlers ?? */
    tlogname = SysTranslate(buf);
    SetReportLevel(old_sev);

    if (strcmp(tlogname, "") == 0) {

        /* if no translation was found use the name */
        /* this may change in the future */
        
        tlogname = buf;
    }

    name_len = strlen(tlogname);

    /* NOTE: */
    /* for file names that are relative names use the current */
    /* directory as default for now */

    if (tlogname[0] != '/') {

        fname = (VK_tCString) malloc(name_len+strlen(DEFAULT_DIR)+1);
        if (fname == NULL) {
            ReportKernelError("system routine (malloc) failed", "", "", 0);
	}
        strcpy(fname, "");
        strcat(fname, DEFAULT_DIR);
    }
    else {
        fname = (VK_tCString) malloc(name_len+1);
        if (fname == NULL) {
            ReportKernelError("system routine (malloc) failed", "", "", 0);
	}
        strcpy(fname, "");
    }

    strcat(fname, tlogname);

    /* determine if another file object with same name already exists */

    exists = FileExists(fname, file_mode, element_length, element_count, dup);

    file_handle = (VK_tFileHandle) malloc(sizeof(tFileRec));
    if (file_handle == NULL) {
        ReportKernelError("system routine (malloc) failed", 
		file_handle->file_name, "", 0);
    }

    file_handle->record_kind = FileRecord;
#if 0
    file_handle->file_info = file_info;
    file_handle->encl_region_LARP = encl_region;
#endif
    file_handle->logical_name = tlogname;
    file_handle->file_name = fname;
    file_handle->file_mode = file_mode;
    file_handle->element_length = element_length;
    file_handle->element_count = element_count;

    file_handle->is_text_file = SIH_Std__TextIO__Text == qqBaseTypeInfo(file_info);
					/* treat a file of type STD.TEXTIO.TEXT
					   as a text file, with different
					   characteristics */


    /* create host file if needed */

    switch (file_mode) {

        case VK_InMode:
	    if (StringMatch("STD_INPUT", tlogname)) {
		file_handle->host_fp = stdin;
		/* ?? what to do about ReadAhead ?? */
	    }
	    else {

            file_handle->host_fp = fopen(fname, "r");
            if (file_handle->host_fp == NULL) {

                /* PA: check here to see if file exists at all */
                /*     to distinguish from other problems (permissions) */

                ReportKernelError("Host file could not be opened for reading",
			file_handle->file_name, "", 0);
	    }
            else {
		if (! file_handle->is_text_file) {
		    ReadHeaderRecord(file_handle);
		}
                ReadAhead(file_handle);
	    }
	    }
            break;

	case VK_OutMode:
            if (exists) {

                /* file object associated with this name exists, use the */
                /* existing file pointer, data size must be the same */

                file_handle->host_fp = dup->host_fp;
	    }
            else {
		if (StringMatch("STD_OUTPUT", tlogname)) {
		    file_handle->host_fp = stdout;
		}
		else {
	                /* create a new host file */
		    file_handle->host_fp = fopen(fname, "w");
		    if (file_handle->host_fp == NULL) {
			ReportInternalError(
				"Host file could not be opened for writing", 
				file_handle->file_name, "", 0);
		    }
                    else {
			if (! file_handle->is_text_file) {
			    WriteHeaderRecord(file_handle);
		    	}
		    }
		}
	    }

            break;

	default:
            ReportInternalError("Internal inconsistency:",
	            "corrupted value for file mode", __FILE__, __LINE__);
            break;
    } /* switch */        

    file_handle->file_state = fs_Open;

    AddFileList(file_handle);

    return file_handle;
} /* VK_CreateFile */



/* *******************************************************************
* 
* VK_ReadFile - read data from file.
* 
* PRE:  Parameter file_handle points to an existing file object
*       created for reading; buffer_length >= 0.
*
* POST: Result value is set to number of words of data transferred from
*       host file to buffer_addr. At most buffer_length words of data are
*       transferred. File object is updated with the size of next block of
*       data for subsequent reads.
*
* HISTORY:
* 	21-MAR-91 RGM	Created
*	06-JUN-91 DSB	Removed old else_ parameter
*	12-JUN-91 DSB	Fixed bug: wrong index for newline check
*	21-AUG-91 DSB	Fixed bug: allow zero-length records
* 
******************************************************************* */

VK_tElementCount VK_ReadFile(file_handle, buffer_length, buffer_addr)
VK_tFileHandle file_handle;
VK_tElementCount buffer_length;		/* size of buffer */
VK_tCompositeAddr buffer_addr;		/* buffer to hold data */
{
    VK_tElementCount result;		/* holds return value */
    int n;				/* return status from system call */
    tTempBuf1024 buf;			/* to read data from disk */
    char * temp_buf = buf;		/* to index buf */
    char * user_buf;			/* to index buffer_addr */
    int buf_size, c;
    char * status;
    int line_length;
    char ch;
    int min_length;
    int i;
/**/
    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null file object handle", __FILE__, __LINE__);
    }

    if (file_handle->file_mode != VK_InMode) {
        ReportInternalError("Internal inconsistency:", 
		"file is not open for reading", __FILE__, __LINE__);
    }

    if (buffer_length < 0) {
        ReportInternalError("Internal inconsistency:", 
		"illegal value of buffer length", __FILE__, __LINE__);
    }

    if (buffer_addr == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null value for data buffer", __FILE__, __LINE__);
    }

    /* LRM page 3-17: required to report error if eof */

    if (file_handle->file_state == fs_Eof) {
        ReportKernelError("Attempt to read past the end of file", 
		file_handle->file_name, "", 0);
    }

    if (file_handle->host_fp == NULL) {
        ReportInternalError("Internal inconsistency:",
	        "corrupted value for host file pointer", __FILE__, __LINE__);
    }

    if (file_handle->element_length < 0) {
        ReportInternalError("Internal inconsistency:",
	        "illegal value for file object element size", __FILE__, __LINE__);
    }

    if (file_handle->is_text_file) {	/* for text file, read "line" delimited
					   by newline character */

				/* ?? THIS NEEDS TO BE REVISITED ?? */
	status = fgets(buf, sizeof(buf), file_handle->host_fp);
	if (NULL == status) {
	    ReportKernelError("End of input file", file_handle->file_name, 
		"", 0);
	}
	line_length = strlen(buf);
	if ('\n' != buf[line_length-1]) {
					/* file line was longer than internal
					   buffer */
		/* : discard remaining file character up to next newline */
	    do {
		ch = fgetc(file_handle->host_fp);
	    } while ('\n' != ch && EOF != ch);
	}
 	else {
	    buf[line_length-1] = '\0';
	    line_length--;		/* don't consider newline as part of
					   record */
	}
	min_length = (line_length < buffer_length) ? line_length : 
		buffer_length;		/* get shorter of line length and
					   buffer length */
	for (i = 0; i <= min_length - 1; i++) {
	    buffer_addr[i] = buf[i];	/* unpack internal character buffer
					   into VHDL word buffer */
	}
	result = min_length;
    }
    else {
/* ?? begin indent ?? */
    

    if (file_handle->next_element_size < 0) {
        ReportInternalError("Internal inconsistency:",
	        "illegal value for next element size", __FILE__, __LINE__);
    }

    if ((file_handle->next_element_size % 4) != 0) {
        ReportInternalError("Internal inconsistency:",
	        "illegal value for next element size", __FILE__, __LINE__);
    }

    /* for constraint object read request must have same size */
    
    if (file_handle->element_count > 0) {
        if (buffer_length != file_handle->element_count) {
	    ReportKernelError("constrained file subtype mismatch", 
		file_handle->file_name, "", 0);
	}
    }

    /* read from file file_handle->next_element_size bytes */

    if (file_handle->next_element_size > 1024) {
					/* ?? WHY IS THIS NOT A SIZEOF?! ?? */
        temp_buf = (char *) malloc(file_handle->next_element_size);
        if (temp_buf == NULL) {
            ReportInternalError("System error detected:",
	            "system routine (malloc) failed", __FILE__, __LINE__);
	}
    }

    if (0 != file_handle->next_element_size) {
    	n = fread(temp_buf, file_handle->next_element_size, 1, 
		file_handle->host_fp);
    	if (n != 1) {
        /* PA: may want to check for premature EOF here */
            ReportInternalError("System error detected:",
	        "system routine (fread) failed", __FILE__, __LINE__);
    	}
    }

    /* transfer data read to user buffer: */
    /* when constraint and unconstraint files are mixed, copy as many bytes */
    /* as possible, that is even if there is not enough data or if there */
    /* to much data to fit in user buffer */

    user_buf = (char *) buffer_addr;

    /* convert buffer length to words and then to bytes */ 
    buf_size = BYTES(buffer_length * file_handle->element_length);
    c = 0;
    while (c < buf_size && c < file_handle->next_element_size) {
        user_buf[c] = temp_buf[c];
        c++;
    }


    if (file_handle->next_element_size > 1024) {
        free(temp_buf);
    }

    /* size of next element should always be multiple of word size */

    result = (VK_tElementCount) (WORDS(file_handle->next_element_size) /
	file_handle->element_length);

    /* read the next record size for subsequent read */


/* ?? end indent ?? */
    }

    ReadAhead(file_handle);

    return result;
} /* VK_ReadFile */



/* *******************************************************************
* 
* VK_WriteFile - write data to file.
* 
* PRE:  Parameter file_handle points to a file object created for
*       writing; value_length >= 0.
*
* POST: value_length words of data from value_addr are written to host
*       file.
*
* HISTORY:
* 	22-MAR-91 RGM	Created
*	06-JUN-91 DSB	Removed old else_ parameter
*	06-JUN-91 DSB	Added text-mode output
*	21-AUG-91 DSB	Fixed bug: allow zero-length records
* 
******************************************************************* */

void VK_WriteFile(file_handle, value_length, value_addr)
VK_tFileHandle file_handle;
VK_tElementCount value_length;		/* length of value, used for */
					/* unconstraint arrays */
VK_tCompositeAddr value_addr;		/* address of data to write */
{
    tSizeRecord size;
    int n, code;
    int vx;
    char ch;
/**/
    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null file object handle", __FILE__, __LINE__);
    }

    if (file_handle->file_mode != VK_OutMode) {
        ReportInternalError("Internal inconsistency:", 
		"file is not open for writing", __FILE__, __LINE__);
    }

    if (value_length < 0) {
        ReportInternalError("Internal inconsistency:", 
		"illegal value of buffer length", __FILE__, __LINE__);
    }

    if (file_handle->element_length <= 0) {
					/* ?? why isn't size of zero allowed 
					   ?? */
        ReportInternalError("Internal inconsistency:", 
		"illegal value of element size", __FILE__, __LINE__);
    }

    if (file_handle->element_count > 0) {

        /* must be constraint array, check if sizes match */

        if (file_handle->element_count != value_length){

	    ReportKernelError(
		"Different element count for constrained file",
		file_handle->file_name, "", 0);
	}
    }

    if (file_handle->is_text_file) {	/* text mode: write characters */
    	for (vx = 0; vx <= value_length-1; vx++) {
	    ch = (char) value_addr[vx];
	    code = fputc(ch, file_handle->host_fp);
	    if (EOF == code) {
		ReportKernelError("system routine (fputc) failed",
			file_handle->file_name, "", 0);
	    }
	}
	code = fputc('\n', file_handle->host_fp);
					/* terminate "record" with newline */
	if (EOF == code) {
	    ReportKernelError("system routine (fputc) failed", 
		file_handle->file_name, "", 0);
	}
	
    }
    else {				/* binary mode: write words */
    	size = BYTES(file_handle->element_length) * value_length;

    	/* write the size record in bytes */

    	n = fwrite((char *) &size, sizeof(size), 1, file_handle->host_fp);
    	if (n != 1) {
            ReportKernelError("system routine (fwrite) failed",
		file_handle->file_name, "", 0);
    	}

	/* write data record */

    	if (0 != size) {
	    n = fwrite((char *) value_addr, size, 1, file_handle->host_fp);
	    if (n != 1) {
	        ReportKernelError("system routine (fwrite) failed", 
			file_handle->file_name, "", 0);
	    }
    	}
    }

} /* VK_WriteFile */




/* ?? WHO SAID THERE WAS TO BE A VK_CloseFile?! ?? */
/* *******************************************************************
* 
* VK_CloseFile - close file.
* 
* PRE:  Parameter file_handle is a pointer to a file object.
*
* POST: File state is set to closed, associated host file is closed except
*       that if file is open for writing, host file is closed only if this
*       is the last file associated with host file.
*
* HISTORY:
* 	21-MAR-91 RGM	Created
* 
******************************************************************* */

void VK_CloseFile(file_handle)
VK_tFileHandle file_handle;
{
    TempItem I; 		/* temporary item */
    VK_tFileHandle fh;		/* temporary file handle */
    VK_tBoolean found;		/* True if a file with same size is open */
    int n;			/* for return status of system call */
/**/
    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", "null file handle", __FILE__, __LINE__);
    }

    if (file_handle->file_state == fs_Closed) {
        ReportInternalError("Internal inconsistency:", "file already closed", __FILE__, __LINE__);
    }

    file_handle->file_state = fs_Closed;

    /* check global list to see if this is the last file to close */
    /* and if so close the host file (only for out modes) */

    found = False;
    if (file_handle->file_mode == VK_OutMode) {

        I=TempFirstItem(FileList);
        while(!found && !TempNullItem(I)) {

            fh = FILE_HANDLE(TempValue(I));	
            if (fh->file_mode == VK_OutMode && fh != file_handle) {
                if (strcmp(fh->file_name, file_handle->file_name) == 0) {

                    found = True;
	        }
	     } /* if */

             I = TempNextItem(I);

        } /* while */

    } /* if */

    if ((!found && file_handle->file_mode == VK_OutMode) ||
       file_handle->file_mode == VK_InMode) {
        n = fclose(file_handle->host_fp);
        if (n != 0) {
            ReportKernelError("system routine (fclose) failed", 
		file_handle->file_name, "", 0);
        }
    }

} /* VK_CloseFile */




/* *******************************************************************
* 
* VK_DestroyFile - destroy object pointed by file_handle.
* 
* PRE:  Parameter file_handle is a pointer to a file object already
*       closed.
*
* POST: file_handle is deleted from the global list of created files,
*       file object and data associated are deleted and its space freed.
*
* HISTORY:
* 	21-MAR-91 RGM	Created
*	07-JUN-91 DSB	Close file to avoid "destroying unclosed file" error
* 
******************************************************************* */

void VK_DestroyFile(file_handle)
VK_tFileHandle file_handle;
{
    TempItem I;		 	/* temporary to traverse global list */
    TempNode N;	  		/* temporary to remove list item */
    VK_tBoolean found;
/**/
    VK_CloseFile(file_handle);	/* close the file first, because there wasn't
				   supposed to be a VK_CloseFile */

    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null file object handle", __FILE__, __LINE__);
    }

    if (file_handle->file_state != fs_Closed) {
        ReportInternalError("Internal inconsistency:",
	        "attempting to destroy an open file", __FILE__, __LINE__);
    }

    /* delete this file from global list */

    found = False;
    I=TempFirstItem(FileList);
    while(!found && !TempNullItem(I)) {

        if (FILE_HANDLE(TempValue(I)) == file_handle) {
            TempRemoveItem(I, N);		/* free space of N ?? */
            found = True;
	}
        else {
            I = TempNextItem(I);
	}

    } /* while */

    if (! found) {		/* consistency check */
        ReportInternalError("Internal inconsistency: file not found in table",
		file_handle->file_name,  __FILE__, __LINE__);
    }
    
    file_handle->record_kind = (tRecordKind) -1;
    file_handle->file_info = (VK_tSymbInfoHandle) 0;
    file_handle->encl_region_LARP = (VK_tLARHeaderPtr) 0;
    /* free SPI temporary ?? */
    file_handle->logical_name = (VK_tCString) 0;
    file_handle->file_mode = (VK_tMode) 0;
    file_handle->element_length = (VK_tDataLength) -1;

    file_handle->element_count = (VK_tElementCount) -1;
    free(file_handle->file_name);
    file_handle->file_name = (VK_tCString) 0;
    file_handle->host_fp = (FILE*) 0;
    file_handle->file_state = (tFileState) -1;
    file_handle->next_element_size = (VK_tDataLength) -1;


/* ??    n = */ free(file_handle);
/* ??
    if (n !=1 ) {
        ReportKernelError("system routine (free) failed", "", "", 0);
    }
?? */
} /* VK_DestroyFile */




/* *******************************************************************
* 
* VK_IsEndOfFile - detect end of file condition.
*
* PRE:  Parameter file_handle is a pointer to a file object.
*
* POST: Return value is True if next read operation will find data,
*       otherwise return False.
*
* HISTORY:
* 	21-MAR-91 RGM	Created
* 
******************************************************************* */

VK_tBoolean VK_IsEndOfFile(file_handle)
VK_tFileHandle file_handle;
{
/**/
    if (file_handle == NULL) {
        ReportInternalError("Internal inconsistency:", 
		"null file object handle", __FILE__, __LINE__);
    }

    /* required by LRM */

    if (file_handle->file_mode == VK_OutMode) {
        return True;
    }

    return file_handle->file_state == fs_Eof;

} /* VK_IsEndOfFile */



/* *******************************************************************
*
* Signature - return number of set bits in argument
*
* PRE:  Parameter i is an integer.
*
* POST: Return value is set to the number of '1' bits in the binary 
*       representation of i.
*
* HISTORY:
*	21-MAR-91 RGM	Created
*
******************************************************************* */

static tFileSignature iSignature(i)
unsigned i;
{
    Integer4 c = 0;
/**/
    while (i != 0) {
        if ((i&1) == 1) {
            c++;
	}
        i >>= 1;
    }
    return (tFileSignature) c;
} /* Signature */




#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */
<static> Fntype RtnName(parm)	/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */
#endif
