#ifndef TKFiles_h
#define TKFiles_h
/* *******************************************************************
*
* Copyright (c) 1991-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKFiles.h
*
* DESCRIPTION:  
*	Interface file for module TKFiles.c.
*
* TYPES:
*	tHeaderRecord -		first record in host file.
*	tSizeRecord -		record preceding each data block in host file.
*	tFileSignature -	signature to identify host file.
*
* GLOBAL VARIABLES:
*	FileList - 		list of created files.
*
*
* ROUTINES:
*   External:
*	VK_ReadFile -		read data from file.
*	VK_WriteFile -		write data to file.
*	VK_DestroyFile -	destroy object referenced by file_handle.
*	VK_CloseFile -		close file.
*	VK_IsEndOfFile -	detect end of file condition.
*	InitFileIO -		initialization routine.
*
* HISTORY:
*	15-MAR-91 RGM	Created
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-OCT-91 MNP	added some protos
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*
* SCCS ID: @(#)TKFiles.h	42.1 1/10/92
*
******************************************************************* */

#include <stdio.h>		/* for FILE type */



#include "VKBasicTypes.h"	/* for VK_tCompositeAddr, etc. */
#include "VKSymbTypes.h"	/* for VK_tMode, VK_tSymbInfoHandle */

#include "TKStructs.h"		/* for tRecordKind */
#define FileRecord BadRecord	/* ?? Not in TKStructs yet */

#include "SPI.h"		/* for SPI types */

typedef enum { fs_Open, fs_Closed, fs_Eof } tFileState;
				/* ?? these should not have "VK_" ?? */

/* *******************************************************************
*
* type tFileRec
*
* HISTORY:
*	06-JUN-91 DSB	Renamed VK_tFile to tFileRec; added is_text_file
*
******************************************************************* */
/* typedef struct VK_sFile * VK_tFileHandle */
typedef struct VK_sFile {
    	tRecordKind record_kind;
    	VK_tSymbInfoHandle file_info;	/* lexical file identification */
    	VK_tLARHeaderPtr encl_region_LARP;
					/* encl. region loc. act. rec. */
    	VK_tCString logical_name;	/* file logical name string */
    	VK_tMode file_mode;		/* VHDL mode of file */
	VK_tBoolean is_text_file;	/* is type STD.TextIO.TEXT */
    	VK_tDataLength element_length;	/* element length in words */
    	VK_tElementCount element_count;	/* element count */
    	VK_tCString file_name;		/* name of the host file */
    	FILE * host_fp;			/* assoc. host file */
    	tFileState file_state;	/* status of file */
    	VK_tByteSize next_element_size;
					/* next element size in bytes */
    } tFileRec;




void InitFileIO(
#ifdef VK_ANSI_FN_PROTOS
#endif
	);


#endif /* TKFiles_h */

