#ifndef TKGlobals_H
#define TKGlobals_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKGlobals.h
*
* DESCRIPTION:  
*	Interface file for TKGlobals.c. 
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
* 	07-Sep-90 RM	Created from TKCommon module.
*	27-MAR-91 WAG	Split the ProcessList into two lists: KernelProcessList
*			and UserProcessList.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	06-MAY-91 DSB	Cleaning up signal changes, etc.
*	14-MAY-91 DSB	W0053+: Split out DelayedProcessList
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	23-JUL-91 DSB	Rename .errors to .error_occurred, etc.
*	14-AUG-91 DSB	Added LibraryInfoList (not used yet)
*
* SCCS ID: @(#)TKGlobals.h	42.1 1/10/92
*
******************************************************************* */

#include "VKI.h"		/* get all public definitions */
#include "TKStructs.h"		/* for tSignalNumberTable */
#include "TKSignals.h"		/* for tDriverNumberTable */

typedef struct {
	VK_tBoolean interactive;	/* use model interactively */
	VK_tBoolean slave;		/* use model from another program */
	VK_tBoolean dump;		/* temporary: dump to test 
					   symbolic information intf. */
	VK_tBoolean verbose;		/* display information messages */
	VK_tBoolean error_occurred;	/* error encountered */
	VK_tBoolean warning_occurred;	/* warning encountered */
    } tGlobalFlags;

extern tGlobalFlags GF;


extern TempList SymbInfoList;		/* list of symbolic info. records */
extern TempList LibraryInfoList;	/* list of library symb. info. recs. */

extern TempList RegionList;		/* list of regions */

extern TempList TypeList;		/* list of types (not needed ...) */
extern TempList AssocList;		/* list of port associations */

extern TempList SensitivitySetList;	/* list of sensitivity sets  */
extern TempList KernelProcessList;	/* list of kernel processes */
extern TempList DelayedProcessList;	/* list of second-phase 'Delayed 
					   processes */
extern TempList UserProcessList;	/* list of user processes */

extern tSignalNumber		ScalarSignalCount;
extern tSignalNumberTable	SignalNumberTable;
extern TempList			SignalGroupList;

extern TempList			DriverGroupList;

extern tSimCycleCount SimCycleCount;	/* current simulation cycle number */

extern tAbsTimeBuf SimTime;	/* current simulation time */
extern int DeltaCount;			/* current relative cycle number
					   with current simulation time */

extern int EventCount;
extern int TotalAssertionCount;
extern int LocalAssertionCount;
extern int TotalErrorCount;
extern int LocalErrorCount;
extern int TotalWarningCount;
extern int LocalWarningCount;

void InitGlobals();

/* ******************************************************************** 
*  Program Exit Status Constents
********************************************************************* */
#ifdef SPI_OS_VMS

#define VK_GoodExit 0x10000001   	 	/* success exit status */
#define VK_WarningExit  0x10000000 		/* failure exit status */
#define VK_ErrorExit  0x10000002  		/* failure exit status */

#else
#ifdef SPI_OSC_UNIX

#define VK_GoodExit  0  			/* success exit status */
#define VK_WarningExit 1 			/* failure exit status */
#define VK_ErrorExit  2 			/* failure exit status */

#endif
#endif


#endif /* TKGlobals_H */


