static char SCCS_ID[] = "@(#)TKListFns.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1987-1989 by CAD LanguageSystems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKListFns.c
*
* DESCRIPTION:  
*	Routines that provide access, manipulation and alteration
*	of TempList structures.
*
* CONTAINS:
*   User-visible:
*	TempNewList	- returns a new, empty, constrained list	
*	TempIsEmpty	- tell whether list is empty (has not items)
*	TempAddFirst 	- adds a list item for a node at the head of a list
*	TempAddLast	- adds a list item for a node at the tail of a list
*	Temp(AddFirstOrLast	- combine AddFirst and AddLast)
*	TempAddNext  	- adds a list item for a node after a given item
*	TempAddPrev  	- adds a list item for a node before a given item
*	TempRemoveItem	- deletes a list item	
*       TempRemoveLast 	- deletes the last item of a list 
*       TempRemoveFirst	- deletes the first item of a list
*	TempFirstItem	- returns the first item of a list
*       TempLastItem 	- returns the last item of a list
*	TempNthItem	- returns the Nth item of a list
*	TempPrevItem	- returns the predecessor of a given item
*	TempNextItem	- returns the successor of a given item
*	TempListOf	- returns the list containing an item
*	TempValue   	- returns the node of an item 
*       TempFindItem 	- searches a list for an item whose value is node N
*	TempListParent	- return parent of list L
*
*	TempSetValue 	- sets the value field of a list item
*	TempInsertAfter	- common routine for adding an item
*	TempMakeGluon	- allocates a new list item	
*
*	
* HISTORY:
* 	02-Apr-90 RM	Converted to TempList version.
* 	22-Aug-90 RM	Cleaned up Daniel's functions.
* 	05-Sep-90 RM	Allow TempNode to be NULL.
* 	17-Sep-90 RM	Renamed module from TempListFns.c to VKListFns.c
*	02-APR-91 WAG	W0015.  Included TKMisc.h.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*
* SCCS ID: @(#)TKListFns.c	42.1 1/10/92
*
******************************************************************* */
#include "TKErrors.h"		/* for KE_ ... */
#include "TKListFns.h"
#include "TKTemp.h"		/* for AllocateMemory and FreeMemory */


#ifndef True
#  define True 1
#  define False 0
#endif

#ifndef NULL
#  define NULL 0
#endif

#define TempNoNode() NULL


typedef struct xList {
        int size;          /* size of the list */
        TempItem first;           /* first element in the list */
        TempItem last;            /* last element in the list */
} xListHeader;


typedef struct pGluon {
        TempItem next;           /* next in list */
        TempItem prev;           /* prev in list */
        TempNode itemnode;       /* node of item */
        TempList hdr;            /* list where item belongs */
} xGluon;



static TempItem TempInsertAfter();


void TempFreeHeader();
#define TempFreeHeader(L)	       TempxFreeHeader(&(L))

TempItem TempMakeGluon();

void TempxFreeGluon();
#define TempFreeGluon(G)			TempxFreeGluon(&(G))



/* *******************************************************************
*
* TempNewList - returns a new, empty, constrained list
*
* HISTORY:
*	22-DEC-88 DSB	Optimized: Combined MakeHeader into NewList 
*	12-JAN-89 DSB	Fixed: added return statement
* 	22-Aug-90 RM	Cleaned up comments and indentation.
* 	05-Sep-90 RM	Added check for out of space condition.
*	02-APR-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*			Removed explicit check for out of space condition.
*
******************************************************************* */

TempList TempNewList()
{
    TempList header;
/**/

    AllocateMemory(header, xListHeader, 1);

    header->first = TempNoItem();
    header->last = TempNoItem();
    header->size = 0;
    return header;

} /* TempNewList */



/* *******************************************************************
*
* TempIsEmpty - Returns true if list pointed to by L is empty (has no nodes)
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempBoolean TempIsEmpty(L)
TempList L;
{
/**/

    if (TempNullList(L)) { 
        return True;
    }
    else {
        return TempNullItem(L->first);
    }
} /* TempIsEmpty */



/* *******************************************************************
*
* TempListLength - return size of list L
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

int TempListLength(L)
TempList L;
{
/**/
    if (TempNullList(L))  {		/* list is not initialized */
	return 0;	
    }   
    else
        return L->size;
} /* Temp ListLength */




/* *******************************************************************
*
* TempSetValue - sets the node field of a list item, returns old value
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
* 	05-Sep-90 RM	Allow node to be NULL.
*
******************************************************************* */

TempNode TempSetValue(I, N)
TempItem I; 
TempNode N;
{
    TempNode result;
/**/
    result = TempNoNode();

    if (TempNullItem(I))  {		/* list item is a null pointer */
        SetErrorStatus(KE_NullListItem, "TempSetValue");
	return result;
    }

    /* node may be NULL */

    else {
        result = I->itemnode;
        I->itemnode = N;
    }

    return result;

}  /* TempSetValue */



/* *******************************************************************
*
* TempValue - return the node value of an item 
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempNode TempValue(LI)
TempItem LI;
{
/**/
    if (TempNullItem(LI))  {		/* list item is a null pointer */
        SetErrorStatus(KE_NullListItem, "TempValue");
	return TempNoNode();
    }
    else
	return LI->itemnode;
}  /* TempValue */



/* *******************************************************************
*
* TempAddFirstOrLast - check for error conditions and append list item
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempAddFirstOrLast(L, N, lastflag)
TempList L; 
TempNode N; 
TempBoolean lastflag;
{
    TempItem result;
/**/        
    if (TempNullList(L)) {		/* list is not initialized */
        SetErrorStatus(KE_NullList, "TempAddFirstOrLast");
	return TempNoItem();	
    }   
    else  {
	if (lastflag) {
	    result = TempInsertAfter(L, L->last, N);
	}
	else {
	    result = TempInsertAfter(L, TempNoItem(), N);
	}
    }
    return result;
} /* TempAddFirstOrLast */



/* *******************************************************************
*
* TempAddNext - Insert a node in a list
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempAddNext(I, N)
TempItem I; 
TempNode N;
{
    TempItem result;
/**/
    if (TempNullItem(I)) {
        SetErrorStatus(KE_NullListItem, "TempAddNext");
	return TempNoItem();
    }   
    else {
	result = TempInsertAfter(I->hdr, I, N);
    }
    return result;
} /* TempAddNext */



/* *******************************************************************
*
* TempAddPrev - Insert a node in a list
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempAddPrev(I, N)
TempItem I; 
TempNode N;
{
    TempItem result;
/**/
    if (TempNullItem(I)) {
        SetErrorStatus(KE_NullListItem, "TempAddPrev");
	return TempNoItem();
    }   
    else  {
	result = TempInsertAfter(I->hdr, I->prev, N);
    }
    return result;
} /* TempAddPrev */



/* *******************************************************************
*
* TempInsertAfter - add node to list after given item or as first item
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
* 	05-Sep-90 RM	Allow node to be NULL.
*
******************************************************************* */

static TempItem TempInsertAfter(L, LI, N)
				/* returns True if okay, False if error */
TempList L; 
TempItem LI; 
TempNode N;
{
    TempItem G;			/* new gluon for new node */
/**/

    /* node may be NULL */


      G = TempMakeGluon(L, N);
      if (L->size == 0) {		     /* first (only) item for list */
	G->prev = TempNoItem();	
	G->next = TempNoItem();	
	L->first = G;			/* becomes first (and last) */
	L->last = G;
      }
      else if (TempNullItem(LI)) {		/* Add before first item    */
	G->prev = TempNoItem();
	G->next = L->first;
	L->first->prev = G;
	L->first = G;
      }
      else {				/* insert gluon after given */
	G->prev = LI;		 	/* hook up gluon  */
	G->next = LI->next;
	LI->next = G;			/* hook up neighbors */
	
	/* If adding at the end of the list, set last to point	*/
	/* to G.  Otherwise, make sure that the next item 	*/
	/* points back to G.					*/
	
	if (TempNullItem(G->next)) {
	  G->hdr->last = G;	
	}
	else {
	  G->next->prev = G;	
	}
      }
      L->size = L->size + 1;	/* adjust list size */
      
      return G;

} /* TempInsertAfter */



/* *******************************************************************
*
* TempRemoveItem - deletes a list item
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

void TempxRemoveItem(LI, N)
TempItem *LI; 
TempNode *N;
{
    TempItem newitem;
    TempItem I;
/**/
    *N = TempNoNode();
    I = *LI;
    if (TempNullItem(I))  {
        SetErrorStatus(KE_NullListItem, "TempRemoveItem");
        return;
    }

    *N = I->itemnode;		/* get the node value */

    newitem = I->next;		/* new I value */


    if (TempSameItem(I->hdr->first, I->hdr->last)) {  /* if this is only item */
	    I->hdr->first = NULL;	/* list is now empty */
	    I->hdr->last = NULL;
    }
    else {
        if (! TempNullItem(I->next)) {
            I->next->prev = I->prev;     /* disconnect neighbors */
	}
	if (! TempNullItem(I->prev)) {
	    I->prev->next = I->next;	
	}
	    
	/* if deleted item was first (or last) item, set first 	*/
	/* (or last) to new first (or last) item.		*/
				 /* note: there will always be a next
					   or a previous item, and given item
					   will never be both first item and
					   last item, because the one-item case
					   is handled above */
	    
	if (TempSameItem(I->hdr->first, I)) {
	    I->hdr->first = I->next;
	}
	else if (TempSameItem(I->hdr->last, I)) {
	    I->hdr->last = I->prev;
	}
    }
    I->hdr->size = I->hdr->size - 1;	/* adjust list size */
	
    I->hdr = TempNoList();		/* clear to sense deleted item */
    TempFreeGluon(I);			/* free the gluon */
    *LI = newitem;			/* update item to successor or NULL */


} /* TempRemoveItem */



/* *******************************************************************
*
* TempRemoveLast - deletes the last item of list L
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

void TempxRemoveLast(L, N)
TempList L; 
TempNode *N;
{
    TempItem i;			/* temp. to retrieve items */
/**/
    *N = TempNoNode();  			/* return NoNode if error */

    if (TempNullList(L))  {			/* list is not initialized */
        SetErrorStatus(KE_NullList, "TempRemoveLast");
	return;	
    }
    else if (TempIsEmpty(L)) {		/* list has no elements */
        SetErrorStatus(KE_EmptyList, "TempRemoveLast");
	return;
    }
    else  {
	i = TempLastItem(L);
	TempRemoveItem(i, *N);
    }	

} /* TempRemoveLast */



/* *******************************************************************
*
* TempRemoveFirst - deletes the first item of list L
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

void TempxRemoveFirst(L, N)
TempList L; 
TempNode *N;
{
    TempItem i;			/* temp. to get items in a list  */
/**/
    *N = TempNoNode();  	/* return no node if something goes wrong */

    if (TempNullList(L))  {		/* list is not initialized */
        SetErrorStatus(KE_NullList, "TempRemoveFirst");
    }
    else if (TempIsEmpty(L)) {	/* list has no elements */
        SetErrorStatus(KE_EmptyList, "TempRemoveFirst");
	return;
    }
    else {
	i = TempFirstItem(L);
	TempRemoveItem(i, *N);
    }
} /* TempRemoveFirst */



/* *******************************************************************
*
* TempFirstItem - returns the first node of list L
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempFirstItem(L)
TempList L;
{
/**/

    if (TempNullList(L))  {		/* list is not initialized */
        SetErrorStatus(KE_NullList, "TempFirstItem");
	return TempNoItem();	
    }
    else {
        return L->first;
    }
} /* TempFirstItem */



/* *******************************************************************
*
* TempLastItem - returns the last item of list L
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempLastItem(L)
TempList L;
{
/**/
    if (TempNullList(L))  {			/* list is not initialized */
        SetErrorStatus(KE_NullList, "TempLastItem");
	return TempNoItem();	
    }   
    else {
	return L->last;
    }
} /* TempLastItem */



/* *******************************************************************
*
* TempNthItem - returns the nth item of list L
*
* NOTES:
*   parameter N:
*	1 to length : item counting from start of list
*	-1 to -length : item counting backwards from end of list
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempNthItem(N, L)   	/* returns indicated item */
int N; 
TempList L;
{
    int i, net;
    TempItem scan;
    char buf1[40];		/* for error message */
/**/

    if (TempNullList(L))  {			/* list is not initialized */
        SetErrorStatus(KE_NullList, "TempNthItem");
	return TempNoItem();	
    }   
    else if (N > L->size || N < -L->size || N == 0) {
	sprintf(buf1, " (ListLength is %d)", L->size);
        SetErrorStatus(KE_OutOfRange, "TempNthItem");
	return TempNoItem();
    }
    else {
	if (N > 0) {
	    net = N;			/* positive means count from first */
	}
	else {
	    net = L->size - (-N - 1);	/* negative means count from last */
	}
	if (net < L->size-net) {       /* if closer to front, scan from front*/
	    scan = L->first;
	    for (i=2; i<=net; i++) {
		scan = scan->next;
	    }
	}
	else {				/* else scan from back */
	    scan = L->last;
	    for (i=2; i<=L->size-net+1; i++) {
		scan = scan->prev;
	    }
	}
	return scan;
    }
} /* TempNthItem */



/* *******************************************************************
*
* TempPrevItem - returns the previous node of list L
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempPrevItem(LI)
TempItem LI;
{
/**/
    if (TempNullItem(LI))  {		/* list item is null */
        SetErrorStatus(KE_NullListItem, "TempPrevItem");
	return TempNoItem();	
    }   
    else {
	return LI->prev;
    }
} /* TempPrevItem */



/* *******************************************************************
*
* TempNextItem - returns the next node of list L
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempItem TempNextItem(LI)
TempItem LI;
{
/**/
    if (TempNullItem(LI)) {			/* list item is null */
        SetErrorStatus(KE_NullList, "TempNextItem");
	return TempNoItem();	
    }   
    else { 	/* ?? check for LI->hdr being NULL ?? */
	return LI->next;
    }
} /* TempNextItem */



/* *******************************************************************
*
* TempListOf - returns LI's list
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*
******************************************************************* */

TempList TempListOf(LI)
TempItem LI;
{
/**/
    if (TempNullItem(LI))  {		/* item is null */
        SetErrorStatus(KE_NullListItem, "TempListOf");
	return TempNoList();	
    }   
    else
        return LI->hdr;
} /* TempListOf */



/* *******************************************************************
*
* TempFreeHeader - frees space of a list header 
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*	02-APR-91 WAG	W0015.  Replaced free call with FreeMemory.
*
******************************************************************* */

void TempxFreeHeader(L)
TempList * L;
{
/**/
		    /* deallocate header pointed to by L */

    (*L)->first = TempNoItem();
    (*L)->last = TempNoItem();
    (*L)->size = 0;

    FreeMemory(*L);
} /* TempFreeHeader */



/* *******************************************************************
*
* TempMakeGluon - allocates a new list item 
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*	02-APR-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*			Replaced explicit out of space check.
*
******************************************************************* */

TempItem TempMakeGluon(L, N)
TempList L; 
TempNode N;
{
    TempItem gluon;			/* temporary */
/**/
    AllocateMemory(gluon, xGluon, 1);

    gluon->hdr = L;
    gluon->prev = TempNoItem();
    gluon->next = TempNoItem();
    gluon->itemnode = N;
    return gluon;
} /* TempMakeGluon */



/* *******************************************************************
*
* TempFreeGluon - frees space of a list item 
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
*	02-APR-91 WAG	W0015.  Replaced free call with FreeMemory.
*
******************************************************************* */

void TempxFreeGluon(G)
TempItem *G;
{
/**/
    FreeMemory(*G);
} /* TempFreeGluon */




/* *******************************************************************
*
* TempFindItem - return item in list L whose node is N
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
* 	05-Sep-90 RM	Allow node to be NULL.
*
******************************************************************* */

TempItem TempFindItem(L, N)
TempList L; 
TempNode N;
{
    TempItem scan;
/**/
    if (TempNullList(L))  {		/* list is not initialized */
        SetErrorStatus(KE_NullList, "TempFindItem");
	return TempNoItem();
    }

    /* allow node to be NULL */

    else {
    	scan = TempNoItem();		/* initial value */
    	scan = TempFirstItem(L);
    	while (! TempNullItem(scan) && ! (scan->itemnode == N)) {
	    scan = TempNextItem(scan);
    	} 
    	return scan;		/* will be NoItem from NextItem if no match */
    }
} /* TempFindItem */



/* *******************************************************************
*
* TempConcatenate - Remove nodes from second list, add them to the first list.
*
* Note: L2 may be a null list pointer.
*
* HISTORY:
* 	22-Aug-90 RM	Cleaned up comments and indentation.
* 	06-Sep-90 RM	Cleaned up unused variables.
*
******************************************************************* */

void TempxConcatenate(L1, L2)
TempList L1; 
TempList * L2;
{
    TempItem li;		/* temp to traverse list */ 
    int i, l;	/* index variables */
/**/

   if (TempNullList(L1)) { 
       SetErrorStatus(KE_NullList, "TempConcatenate");
	return;
    }
    else if (TempNullList(*L2)) {	/* if L2 is null, then nothing to do */
	return;
    }	
    else if (TempSameList(L1, *L2))  {
        SetErrorStatus(KE_NullList, "TempConcatenate");
        return;
    }

   	if (TempIsEmpty(*L2)) {		/* if L2 is NULL or empty */
	     				/* no moving to do, just */
	    (*L2)->first = TempNoItem();
	    (*L2)->last = TempNoItem();
	    (*L2)->size = 0;
          /* UNTIL TempFreeList is implemented */
            (*L2) = TempNoList();
	  /*  TempFreeList(*L2, False); */    /* free header, gluons; (no nodes) */
    	}

    	else  {


	    li = TempFirstItem(*L2);   /* start scan of second list */
	    i = TempListLength(*L2);
	    for (l=1; l<=i; l++) {	/* move 2nd list's items */

		li->hdr = L1;	/* point item to L1 */
		li = li->next; 	/* can't use NextItem here */
	    }  

	    if (TempIsEmpty(L1))  {	/* need to set .first, .last only */
		L1->first = (*L2)->first;
		L1->last = (*L2)->last;
	    }
	    else {				/* else hook L2 to L1 */
		L1->last->next = (*L2)->first;
		(*L2)->first->prev = L1->last;
		L1->last = (*L2)->last;
	    }
	    L1->size = L1->size + (*L2)->size;   /* adjust new size */
		    /* clear L2's header to avoid dangling references */
	    (*L2)->first = TempNoItem();
	    (*L2)->last = TempNoItem();
	    (*L2)->size = 0;

          /* UNTIL TempFreeList is implemented */
            (*L2) = TempNoList();
	/*  TempFreeList(*L2, False); */	/* free header, gluons; (no nodes) */

	  }

} /* TempConcatenate */


