#ifndef TKListFns_H
#define TKListFns_H
/* *******************************************************************
*
* Copyright (c) 1987-1989 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKListFns.h
*
* DESCRIPTION:  
*	Interface file for TKListFns.c; abstract list functions.
*
*
* ROUTINES:
*   User-visible:
*	TempNewList	- returns a new, empty, constrained list	
*	TempIsEmpty	- tell whether list is empty (has not items)
*	TempAddFirst 	- adds a list item for a node at the head of a list
*	TempAddLast	- adds a list item for a node at the tail of a list
*	Temp(AddFirstOrLast	- combine AddFirst and AddLast)
*	TempAddNext  	- adds a list item for a node after a given item
*	TempAddPrev  	- adds a list item for a node before a given item
*	TempRemoveItem	- deletes a list item	
*       TempRemoveLast 	- deletes the last item of a list 
*       TempRemoveFirst	- deletes the first item of a list
*	TempFirstItem	- returns the first item of a list
*       TempLastItem 	- returns the last item of a list
*	TempNthItem	- returns the Nth item of a list
*	TempPrevItem	- returns the predecessor of a given item
*	TempNextItem	- returns the successor of a given item
*	TempListOf	- returns the list containing an item
*	TempValue   	- returns the node of an item 
*       TempFindItem 	- searches a list for an item whose value is node N
*	TempListParent	- return parent of list L
*
*	TempSetValue 	- sets the value field of a list item
*	TempInsertAfter	- common routine for adding an item
*	TempMakeGluon	- allocates a new list item	
*
* HISTORY:
*	20-JUL-90 DSB	Created from PCListFns created from SPI ListFns.
* 	22-Aug-90 RM	Cleaned up Daniel's routines.
* 	17-Sep-90 RM	Renamed from TempListFns.h to VKListFns.h
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*
* SCCS ID: @(#)TKListFns.h	42.1 1/10/92
*
******************************************************************* */


typedef void * TempNode;
typedef int TempBoolean;

#ifdef NULL
#  define NULL 0
#endif

/* TempList type */

typedef struct xList *TempList;

/* TempItem type */

typedef struct pGluon *TempItem;



#define TempNoList() ((TempList) 0)
#define TempNullList(L) ((L) == TempNoList())

#define TempNoItem() ((TempItem) 0)
#define TempNullItem(I) ((I) == TempNoItem())

#define TempSameList(L1, L2) ((L1) == (L2))
#define TempSameItem(I1, I2) ((I1) == (I2))

TempBoolean TempIsEmpty();

int TempListLength();

TempNode TempSetValue();

TempNode TempValue();

TempNode TempListParent();

TempItem TempAddFirstOrLast();

TempItem TempAddNext();

TempItem TempAddPrev();

void TempxRemoveItem();
#define TempRemoveItem(LI, N) \
	TempxRemoveItem(&(LI), &(N))

void TempxRemoveLast();
#define TempRemoveLast(L, N) \
	TempxRemoveLast(L, &(N))

void TempxRemoveFirst();
#define TempRemoveFirst(L, N) \
	TempxRemoveFirst(L, &(N))

TempItem TempFirstItem();

TempItem TempLastItem();

TempItem TempNthItem();

TempItem TempPrevItem();

TempItem TempNextItem();

TempList TempListOf();

TempItem TempFindItem();

void TempSetListParent();

void TempxConcatenate();
#define TempConcatenate(L1, L2) \
        TempxConcatenate(L1, &(L2))


TempList TempNewList();

TempList TempMakeList();

TempItem	TempAddFirst();

TempItem 	TempAddLast();

#define TempAddLast(L, N)\
        TempAddFirstOrLast(L, N, True)

#define TempAddFirst(L, N)\
        TempAddFirstOrLast(L, N, False)

#endif

