char static SCCS_ID[] = "@(#)TKMain.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKMain.c
*
* DESCRIPTION:  
*	Main module for test kernel.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	09-OCT-89 DSB	Created
*	02-APR-91 WAG	W0015.  Moved the Terminate, EarlyTerminate, and
*			ExitProgram routines to TKMisc.c and included
*			TKMisc.h.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	18-MAY-91 AWH	Added uncalled refrence to VC_ResolveSignal
*	12-SEP-91 AWH	Merging VLK changes, Error messages
*
* SCCS ID: @(#)TKMain.c	42.1 1/10/92
*
******************************************************************* */

#include "Version.inc"
#include "VKI.h"
#include "TKStructs.h"
#include "TKGlobals.h"          /* for list init. */
#include "TKSim.h"		/* for EventTimeQueue */
#include "TKRegions.h"		/* for CreateRegion */
#include "TKDebug.h"		/* for DB record */
#include "TKErrors.h"		/* for KE_<...> kernel error codes */
#include "TKTemp.h"		/* for EarlyTerminate(), TerminateKernel() */
#include "TKRecord.h"		/* for RD_SetVirtualFileMode */
#include "TKParams.h"		/* for GetParams */
#include "TKElaborate.h"	/* for ElaborateModel */
#include "TKCommand.h"		/* for InitializeCommand.. */
#include "TKIO.h"		/* for *FraceFile... */
#include "TKFiles.h"		/* for InitFileIO */

#include <signal.h>
#include <setjmp.h>

static jmp_buf JumpBuffer;	/* used for exit handler */

/* *******************************************************************
*
* InitializeKernel - initialize kernel
*
* HISTORY:
*	27-SEP-89 DSB	Created
* 	07-Sep-90 RM	Splitted into calls to other init routines.
*	01-MAY-91 DSB	Added InitFileIO call 
*	16-MAY-91 DSB	Open trace file in non-slave mode too
*
******************************************************************* */

void InitializeKernel()
{
/**/
    /* Note: can't open trace file yet because we must elaborate to get
	     top-level unit name to name the trace file */

    OpenTraceFile();

    InitializeCommand();
    InitGlobals();
    InitKernelRegion();
    InitializeDebug();
    InitData();
    InitSim();
    InitFileIO();

/* order dependency: InitData must come before InitSim; ?? this should be
	fixed ?? */
} /* InitializeKernel */


/* *******************************************************************
*
* PostElaborate - complete any elaboration or initialization
*
* HISTORY:
*	11-APR-90 DSB	Created
*
******************************************************************* */

void PostElaborate()
{
/**/
/*    PostElaborateSignals(); ?? if needed */
	/* this is currently handled in Simulate_Initialize */
} /* PostElaborate */



/* *******************************************************************
*
* SignalHandler - handle unexpected signals
*
* HISTORY:
*	22-OCT-90 AWH	Created
*	04-JAN-91 AWH   Added Count of SignalHandler Calls
*
******************************************************************* */
static void SignalHandler()
{
#define MAX_CALLS_TO_DUMP_HANDLER 2
    static Integer4 count = 0;	/* count calls to SignalHandler */
/**/
    if (count++ > MAX_CALLS_TO_DUMP_HANDLER) {
	ExitProgram();	/* repeated calls to SignalHandler, exit now */
    }
    else {
       longjmp(JumpBuffer, 1);
    }
} /* SignalHandler */

	

/* *******************************************************************
*
* main - 
*
* HISTORY:
*	29-SEP-89 DSB	Created
*	07-JUN-91 DSB	Moved registration call to before first GetCommands
*	11-JUN-91 AWH	Added setting of slave mode for RD_ module
*	23-JUL-91 DSB	Renamed .errors to .error_occurred
*	24-JUL-91 DSB	Split out ElaborateModel
*	06-JAN-92 DKC	Removed reference to VC_ResolveSignal (an optimising
*                       compiler would too!)
*
******************************************************************* */

int main(argc, argv)
int argc;
char *argv[];
{
    VK_tBoolean status;
/**/
    /* Set up unexpected signal handler */

    signal(SIGBUS, SignalHandler);
    signal(SIGSEGV, SignalHandler);
    if (setjmp(JumpBuffer)) {
        ReportInternalError("Caught internal error. (SIGBUS or SIGSEGV)", "",
		__FILE__, __LINE__);
        ExitProgram();
    }

    /* Set up interrupt signal handler */
    signal(SIGINT, DebugInterrupt);	/* trap ^C */
 
  
    OpenDLS();
    SetTraceback(True);

    status = GetParams(argc, argv, &GF.interactive, &GF.slave, &GF.dump,
		       &GF.verbose);

    InitializeErrors(GF.slave);

    RD_SetVirtualFileMode(GF.slave);

    /* Process GetParams Error Status */
    if (! status) {
	GF.error_occurred = True;
	ExitProgram();
    }

    InitializeKernel();			/* initialize kernel */


#ifdef VCG_VLK
    SetErrorStatus(KE_Ident, "VHDL Learning Kit", VLK_SYSTEM_VERSION_4);
#else
    SetErrorStatus(KE_Ident, "Retargetable VHDL Code Generator Test Kernel", 
		   RVCG_SYSTEM_VERSION_4);
#endif


    TK_RegisterUnits();		/* call registration routine in
					   generated configuration file */
    if (GF.dump) {
	DumpSymbInfo();
    }


    if (GF.interactive) {
	GetCommands();
    }
    SetErrorStatus(KE_Elab);
    if (GF.slave) {
	RD_WriteStatus(ss_Elaborating);
    }

    ElaborateModel();

    DB.is_after_elab = True;
    InternalDebug(bp_AfterElaboration, NULL);


    PostElaborate();

    SimulateModel();			/* simulate model */

    TerminateKernel();

    return 0;				/* main must have a return */
} /* main */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */

<static> Fntype RtnName(parm)				/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif
