static char SCCS_ID[] = "@(#)TKMisc.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKMisc.c
*
* DESCRIPTION:  
*	Functions for VKI files VKMisc.c
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Split from VKI.c
*	18-MAR-91 HK	Added references to TIME macros.
*	29-MAR-91 WAG	Added AllocateMemory, ReallocateMemory,
*			and FreeMemory routines.
*	02-APR-91 WAG	W0015.  Moved the TerminateKernel, EarlyTerminate, and
*			ExitProgram routines here from VKMain.c.
*	03-APR-91 WAG	W0015.  Created VK_Alloc and VK_Free routines from
*			AllocateMemory and FreeMemory.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 DSB	Moved non-VKI stuff to TKTemp.c
*	16-MAY-91 DSB	Converting to unsigned absolute time
*
* SCCS ID: @(#)TKMisc.c	42.1 1/10/92
*
******************************************************************* */

#include "TKTemp.h"
#include "TKCommon.h"
#include "TKSim.h"		/* for CurrentProcess */
#include "TKData.h"		/* for TIME macros */
#include "TKGlobals.h"		/* for GF */
#include "TKDebug.h"		/* for DB */
#include "TKSymbUtils.h"	/* for TraverseToData, etc. */
#include "TKIO.h"		/* for FormatHierarchicalName, etc. */


/* *******************************************************************
*
* VK_ReadSimulationTime - read current simulation time
*
* HISTORY:
*	04-OCT-89 DSB	Created
*	10-JUN-91 DSB	Applied 64-bit time
*
******************************************************************* */
void VK_ReadSimulationTime(buffer)
VK_tAbsoluteTimePtr buffer;
{
/**/
    TIME_COPY(*buffer, SimTime);
} /* VK_ReadSimulationTime */


/* *******************************************************************
*
* VK_IssueAssertionViolation - issue VHDL assertion violation
*
* HISTORY:
*	03-OCT-89 DSB	Created
*	...
*	01-MAY-90 DSB	Copied formatting from Formatter.c
*	11-MAY-90 DSB	Added process pathname to message
*	25-JUL-90 DSB	Added density parameter for general strings
*	09-OCT-90 DSB	Removed density parameter; cleaned up parameter types
*	03-JAN-91 DSB	Changed source_file string to unit_info pointer
*	14-MAY-91 AWH	Changed to support new protocol
*	17-MAY-91 DSB	Applied su_IdentifyPosition
*	02-AUG-91 DSB	Fixed bug: why was big_buf[bx] set to \0?
*
******************************************************************* */
void VK_IssueAssertionViolation(current_region_data, severity, 
	message_length, message) 
VK_tLARHeaderPtr current_region_data; 	/* encl. region data */
VK_tSeverity severity;			/* severity of assertion */
VK_tStringLength message_length;	/* length of message */
VK_tVHDLString message;			/* assertion message */
{
    static tStaticStrBuf big_buf = STATIC_STR_INITIAL;

    VK_tCString library;
    VK_tCString primary;
    VK_tCString secondary;
    VK_tCString source_file;
    VK_tSymbInfoHandle region_info;

    VK_tCString sevstr;
    tTempBuf80  sim_time,
                temp_buff;
    tTempBuf256 proc_buf;
    char        *cp,
                ch;
    int         line_number,
                temp_left;
/**/
    su_IdentifyPosition(current_region_data, &library, &primary, &secondary, 
	&source_file, &line_number, &region_info);
					/* identify position from stuffed
					   LAR fields */

	/* 1: format internal form message to C string */

/* ?? this code should be a separate function so TKFiles.c can use it too;
	maybe PackVHDLString(buffer, buf_size, string_length, string_addr), 
	etc.; separate function of packing string from displaying unprintable
	characters
?? */

    StaticStrCpy(&big_buf,"");

    for (cp = temp_buff,temp_left=sizeof(temp_buff); message_length--;) {

        ch = *message++;

        if (0 < --temp_left && (ch >= ' ' && ch <= 0x7e)) {
            *cp++ = ch;
        }
        else {
            *cp = '\0';
            StaticStrCat(&big_buf,temp_buff);
            if (ch >= ' ' && ch <= 0x7e) {
                cp        = temp_buff;
                *cp++     = ch;
                temp_left = sizeof(temp_buff) -1;
            }
            else {
                sprintf(temp_buff, "<0x%02x>", ch);	
                cp        = &temp_buff[6];
                temp_left = sizeof(temp_buff) -7;
            }
        }
    }

    *cp = '\0';
    StaticStrCat(&big_buf,temp_buff);

	/* 2: do miscellaneous processing */

    switch (severity) {
	case VK_Note:
	    sevstr = "Note";	
	    break;
	case VK_Warning:
 	    sevstr = "Warning";
  	    break;
	case VK_Error:
	    sevstr = "Error";
	    GF.warning_occurred = True;	/* severe assertion counts as kernel
					   warning */
 	    break;
	case VK_Failure:
	    sevstr = "Failure";
	    GF.warning_occurred = True;	/* severe assertion counts as kernel
					   warning */
            break;
	default:
	    sevstr = "<bad value>";
	    break;
    } /* switch */

    FormatHierarchicalName(proc_buf, sizeof(proc_buf), CurrentProcess);

	/* 3: report assertion violation */

    if (! GF.slave ) {
	UTIME_FORMAT(sim_time, SimTime);
        fprintf(stdout, 
    	    "               Assertion Violation: at time = %s %s + %d,\n", 
    	    sim_time, BaseUnit, DeltaCount);
        fprintf(stdout, 
	    "                 in process %s,\n", proc_buf);
        fprintf(stdout, 
	    "                 from line %d of file %s,\n", 
	    line_number, source_file);
        fprintf(stdout, 
	    "                 %s: \"%s\".\n", sevstr, big_buf.string);
    }

    TotalAssertionCount++;
    LocalAssertionCount++;

	/* 4: log assertion to trace file */

    TraceAssertion(severity,big_buf.string,source_file,line_number,proc_buf);

    if (GF.dump) {
	TryTraceback(current_region_data);
    }

    EarlyTerminate(FALSE);		/* terminate based on global flags */

} /* VK_IssueAssertionViolation */


/* *******************************************************************
*
* VK_ReportError - report simulation error from generated code
*
* NOTE: As of 03-JAN-91, VK_ReportError and ReportKernelError are
*	virtually identical. 
*
* HISTORY:
*	11-APR-90 DSB	Created
*	09-OCT-90 DSB	Added extra data and source file parameters
*	14-OCT-90 AWH	Added processing of source_file and extra parameters
*	03-JAN-91 DSB	Split VK_ReportError from ReportKernelError;
*			changed source_file string to unit_info pointer
*	14-MAY-91 AWH	W0054: Changed to support new definition
*	17-MAY-91 DSB	Applied su_IdentifyPosition
*	17-MAY-91 DSB	Trying temporary tracebacks
*	28-MAY-91 DSB	Added space ("%s(%s)" => "%s (%s)")
*	13-JUL-91 DSB	Check for null current_region_data
*
******************************************************************* */
void VK_ReportError(current_region_data, error_code, message, extra)
VK_tLARHeaderPtr current_region_data;	/* encl. region data */
VK_tVHDLErrorCode error_code;		/* error code */
VK_tCString message;			/* main error message */
VK_tCString extra;			/* extra information */
{
    VK_tLARHeaderPtr use_region_data;
    VK_tCString library;
    VK_tCString primary;
    VK_tCString secondary;
    VK_tCString source_file;
    VK_tInteger line_number;	/* ?? (this should be VK_tLineNumber) ?? */
    VK_tSymbInfoHandle region_info;

    tTempBuf80 sim_time_buf;
/**/
/* ?? clean: this section needs to be reworked ?? */

    if (NULL == current_region_data) {	/* if no data from region */
	GetLastTracepoint(&use_region_data);
					/* use last registered data */
    }
    else {
	if (NULL != TK_GetRegionInfo(current_region_data)) {
					/* if data is initialized */
	    use_region_data = current_region_data;
					/* use given region data */
	}
	else {				/* (not initialized) */
	    if (NULL != VK_GetIARP(current_region_data) && NULL != 
			VK_GetCallerLARP(VK_GetIARP(current_region_data))) {
		use_region_data = VK_GetCallerLARP(
			VK_GetIARP(current_region_data));
					/* use caller region's data */
	    }
	    else {
		GetLastTracepoint(&use_region_data);
					/* use last traced region */
	    }
	}
    }
	
    su_IdentifyPosition(use_region_data, &library, &primary, &secondary, 
	&source_file, (VK_tLineNumber *)&line_number, &region_info);
					/* identify position from stuffed
					   LAR fields */
    if (NULL == region_info) {
	GetLastTracepoint(&use_region_data);
	su_IdentifyPosition(use_region_data, &library, &primary, &secondary, 
		&source_file, (VK_tLineNumber *)&line_number, &region_info);
    }



    if (NULL == message ) {
	message = "<NULL>";
    }
    if (NULL == extra) {
	extra = "";
    }

    /* Report error */
    if (! GF.slave) {
	UTIME_FORMAT(sim_time_buf, SimTime);
        fprintf(stdout, 
	    "               Error: at time = %s %s + %d,\n", 
	    sim_time_buf, BaseUnit, DeltaCount);

        fprintf(stdout, 
	    "                 from line %d of file %s,\n", 
	    line_number, (NULL == source_file) ? "<NULL>" : source_file);

        if ('\0' == extra[0]) {
            fprintf(stdout, 
	        "                 %d:%s\n",  error_code, message);
        }
        else {
	    fprintf(stdout, 
	        "                 %d: %s (%s)\n",  error_code, message, extra);
        }
    } /* if (! GF.slave) */
  
    TotalErrorCount++;
    LocalErrorCount++;

    TraceError(message, extra, source_file, line_number, error_code 
		/* , proc_buf */);
	/* ?? need to add additional data to trace call and trace record ?? */

    /* set error exit indication */
    GF.warning_occurred = True;		/* fatal VHDL run-time check counts
					   as kernel warning */

    if (GF.dump) {
	TryTraceback(use_region_data);
    }

    EarlyTerminate(FALSE);		/* terminate based on global flags */
} /* VK_ReportError */


/* *******************************************************************
*
* VK_Alloc - allocate memory through kernel
*
* HISTORY:
*	03-APR-91 WAG	Created
*
******************************************************************* */
VK_tVoidPtr VK_Alloc(size)
long int size;
{
    VK_tVoidPtr pointer;				
/**/
    pointer = (VK_tVoidPtr) malloc(size);
    
    if (NULL == pointer) {
	ReportInternalError("Out of memory", "", "",0);
    }
    return pointer;
    
} /* VK_Alloc */



/* *******************************************************************
*
* VK_Free - Free memory through kernel.
*
* HISTORY:
*	03-APR-91 WAG	Created
*
******************************************************************* */
void VK_Free(pointer)
VK_tVoidPtr pointer;
{
/**/
    free(pointer);
        
} /* VK_Free */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */
<static> Fntype RtnName(parm)				/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif
