static char SCCS_ID[] = "@(#)TKProcesses.c	42.2 1/13/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKProcesses.c
*
* DESCRIPTION:  
*	<then describe contents>
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	15-DEC-89 DSB	Split from VKUnsplit.c
*	18-MAR-91 HK	Added references to TIME macros.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*	06-MAY-91 DSB	Cleaning up signal changes
*	14-MAY-91 DSB	W0053+: Split out 'Delayed processes; removed 
*			obsolete Create*Process parameters
*	21-MAY-91 DSB	Cleaned up old types; renamed to remove some VK_
*
* SCCS ID: @(#)TKProcesses.c	42.2 1/13/92
*
******************************************************************* */
#include "VKI.h"

#include "VKConfig.h"		/* for varargs decls */

#include "TKCommon.h"
#include "TKProcesses.h"
#include "TKStructs.h"		/* for tProcessRec */
#include "TKSim.h"		/* for NeverTime and ZeroTime */
#include "TKTemp.h"		/* for VERIFY and AllocateMemory */
#include "TKData.h"		/* for TIME macros */
#include "TKIO.h"		/* for FormatHierarchicalName... */

#include "TKListFns.h"		/* for TempAddLast */
#include "TKGlobals.h"		/* for SignalNumberTable */
#include "TKSymbUtils.h"	/* for TK_GetRegionHandle */

static Integer4 ProcessCount = 0;  /* process identifier */
                                   /* - used to order processing */

/* *******************************************************************
*
* CreateKernelProcess - create internal process
*
* HISTORY:
*	02-OCT-89 DSB	Created from VK_CreateProcessStatement
*	20-OCT-89 DSB	Adding timed wait support
*	13-JUL-90 DSB	Added WaitFn, WaitContext
* 	28-Sep-90 GJB	Change SetqqTriggerCycle to SetqqTimeoutTriggerCycle
*	18-MAR-91 WAG	Created from CreateProcess (which was subsequently)
*			renamed "CreateUserProcess").  Now puts process records
*			on KernelProcessList.
*	20-MAR-91 WAG	Removed the priority parameter; set the fKernelProcess
*			field in the process record to True always.
*	01-APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*	14-MAY-91 DSB	Removed unneeded parameters (e.g., process name)
*
******************************************************************* */

VK_tProcessHandle CreateKernelProcess(parent_region, process_kind, 
	proc_data)
TK_tRegionHandle parent_region;	/* parent region handle */
tProcessKind process_kind;
VK_tVoidPtr proc_data;		/* process data */
{
    VK_tProcessHandle process;
/**/
    AllocateMemoryZero(process, tProcessRec, 1);
    AddqqRecordKind(process, ProcessRecord);

    TempAddLast(KernelProcessList, (TempNode) process);
					  /* add to global list */
    AddqqNumber(process, ProcessCount++);	  /* unique process number */ 
    AddqqName  (process, "");

    SetqqProcessKind    (process, process_kind);
    SetqqKernelProcess  (process, TRUE);
    AddqqProcessData    (process, (VK_tPCBP) proc_data);
    AddqqSensitivityList(process, TempNewList());

    AddqqParentRegion(process, parent_region);
    if (! (NULL == parent_region)) {
    	TempAddLast(qqChildren(parent_region), (TempNode) process);
					/* ?? */
    }

    return process;

} /* CreateKernelProcess */ 


/* *******************************************************************
*
* CreateDelayedProcess - create internal user process
*
* HISTORY:
*	14-MAY-91 DSB	Split from CreateUserProcess, removed unneeded 
*			parameters (e.g., process name)
*
******************************************************************* */

VK_tProcessHandle CreateDelayedProcess(parent_region, proc_data)
TK_tRegionHandle parent_region;		/* parent region handle */
tProcData_Implicit_Ptr proc_data;	/* process data */
{
    VK_tProcessHandle process;
/**/
    AllocateMemoryZero(process, tProcessRec, 1);
    AddqqRecordKind   (process, ProcessRecord);

    TempAddLast(DelayedProcessList, (TempNode) process);
					/* add to global list */

    AddqqNumber(process, ProcessCount++); /* unique process number */ 
    AddqqName  (process, "");

    SetqqProcessKind    (process, pc_Delayed);
    SetqqKernelProcess  (process, True);
    AddqqProcessData    (process, (VK_tPCBP) proc_data);
    AddqqSensitivityList(process, TempNewList());

    AddqqParentRegion(process, parent_region);
    if (NULL != parent_region) {
    	TempAddLast(qqChildren(parent_region), (TempNode) process);  /* ?? */
    }

    return process;

} /* CreateDelayedProcess */ 


/* *******************************************************************
*
* CreateUserProcess - create internal user process
*
* HISTORY:
*	02-OCT-89 DSB	Created from VK_CreateProcessStatement
*	20-OCT-89 DSB	Adding timed wait support
*	13-JUL-90 DSB	Added WaitFn, WaitContext
* 	28-Sep-90 GJB	Change SetqqTriggerCycle to SetqqTimeoutTriggerCycle
*	18-MAR-91 WAG	Renamed; old name was "CreateProcess".  Now puts
*			process records on UserProcessList.
*	20-MAR-91 WAG	Removed the priority parameter; set the fKernelProcess
*			field in the process record to False always.
*	01-APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*	14-MAY-91 DSB	Removed unneeded parameters
*	17-MAY-91 DSB	Removed process name (cleaning for W0052)
*
******************************************************************* */

static VK_tProcessHandle CreateUserProcess(parent_region, proc_data)
TK_tRegionHandle parent_region;	/* parent region handle */
VK_tPCBP proc_data;		/* process data */
{
    VK_tProcessHandle process;
/**/
    AllocateMemoryZero(process, tProcessRec, 1);
    AddqqRecordKind   (process, ProcessRecord);

    TempAddLast(UserProcessList, (TempNode) process);  /* add to global list */
    AddqqNumber(process, ProcessCount++);	  /* unique process number */ 
    AddqqName  (process, "<use process statement REGION now>");

    SetqqProcessKind    (process, pc_ProcessStatement);
    SetqqKernelProcess  (process, False);
    AddqqProcessData    (process, proc_data);
    AddqqSensitivityList(process, TempNewList());

    AddqqParentRegion(process, parent_region);
    if (! (NULL == parent_region)) {
    	TempAddLast(qqChildren(parent_region), (TempNode) process);
					/* ?? */
    }

    return process;

} /* CreateUserProcess */ 


/* *******************************************************************
*
* VK_CreateProcess - create process for process statement
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	02-OCT-89 DSB	Split out CreateProcess
*	18-MAR-91 WAG	Changed CreateProcess call to CreateUserProcess.
*	20-MAR-91 WAG	Removed value for the (now defunct) priority field.
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*	17-MAY-91 DSB	W0052: Updated to new VKI
*
******************************************************************* */

VK_tProcessHandle VK_CreateProcess(process_info, process_data, process_context)
VK_tSymbInfoHandle process_info;	/* process statement identification */
VK_tLARHeaderPtr process_data;	/* process statement lcl. act. rec. */
VK_tPCBP process_context;	/* process context block */
{
    TK_tRegionHandle process_region;
/**/
	/* process_info is already used by VK_TraceBegin */

    process_region = TK_GetRegionHandle(process_data);
					/* get old region handle from process
					   statement's LAR */

    return CreateUserProcess(process_region, process_context);
					/* create internal process; store 
					   process context data so kernel
					   can run process */

/* ?? This needed to be reworked once VK_TraceBegin is stable.
	The process data probably should point to the process statement's
	LAR instead of the process statement's internal/old region handle.
?? */
} /* VK_CreateProcessStatement */ 



/* *******************************************************************
*
* VK_CreateSensitivitySet - 
*
* HISTORY:
*	20-DEC-89 DSB	Created
*	02-APR-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*
******************************************************************* */

VK_tSensitivitySetHandle VK_CreateSensitivitySet(process_id)
VK_tProcessHandle process_id;
{
    VK_tSensitivitySetHandle sens_set;
/**/
    AllocateMemory(sens_set, tSensitivitySetRec, 1);
    AddqqRecordKind(sens_set, SensitivitySetRecord);

    TempAddLast(SensitivitySetList, (TempNode) sens_set);
					/* add to global list */

    AddqqEnabled(sens_set, False);
    AddqqSensitiveProcess(sens_set, process_id);
    AddqqSensitivityList(sens_set, TempNewList());

    return sens_set;
} /* VK_CreateSensitivitySet */


/* *******************************************************************
*
* VK_DestroySensitivitySet - 
*
* HISTORY:
*	20-DEC-89 DSB	Created
*	16-JUL-90 DSB	Partially implemented
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*
******************************************************************* */

void VK_DestroySensitivitySet(sens_set, process_id)
VK_tSensitivitySetHandle sens_set;
					/* handle of set to destroy */
VK_tProcessHandle process_id;		/* will we need this param.? */
{
/**/
    if (NULL == sens_set) {
 	ReportInternalError("VK_DestroySensitivitySet: Input handle is null", "",
		__FILE__, __LINE__);
	return;
    }

    if (NULL != process_id) {
	if (qqCurrentSensSet(process_id) == sens_set) {
	    if (NULL != qqCurrentSensSet(process_id)) {
	    	SetqqEnabled(qqCurrentSensSet(process_id), False);
				/* disable previous sensitivity set */
	    }
	    SetqqCurrentSensSet(process_id, NULL);
	}

    }

    /* ?? need to remove from list of sensitivity sets ?? */
    /* ?? need to undo all other structures involving it ?? */
    /* ?? need to deallocate list of signals, and base of list ?? */
	
} /* VK_DestroySensitivitySet */


/* *******************************************************************
*
* VK_SensitizeSet - 
*
* HISTORY:
*	20-DEC-89 DSB	Created
*	05-OCT-90 AWH	Added signal vectors.	
*       29-OCT-91 DKC   Modified to handle large physical types
*
******************************************************************* */

void VK_SensitizeSet(sens_set, process_id, list_length, signal_handle_1, 
	signal_offset_1, signal_length_1 VA_ALIST)
VK_tSensitivitySetHandle sens_set;
VK_tProcessHandle process_id;
VK_tListLength list_length;
VK_tSignalHandle signal_handle_1;
VK_tSignalOffset signal_offset_1;
VK_tSignalLength signal_length_1;
VA_DCL
{
    va_list varlistptr;         /* ptr for traversing var arg list */
    VK_tSignalHandle signal_handle;
/*  VK_tSignalOffset signal_offset; */
    tSignalNumber signal_start;
    VK_tSignalLength signal_length;
    tSignalElementHandle signal_element_handle;
    TempList L;
    int sg_index;		/* signal group index */
    int ve_index;		/* vector element index */
/**/
    VA_START(varlistptr, signal_length_1);
  
    L = qqSensitivityList(sens_set);

    for (sg_index = 1; sg_index <= list_length; sg_index++) {
	if (1 == sg_index) {		/* process first signal */
	    signal_handle = signal_handle_1;
	    signal_start = signal_offset_1 + qqSignalStart(signal_handle);
	    signal_length = signal_length_1;
	}
	else {				/* process additional signals */
	    signal_handle = (VK_tSignalHandle) 
		va_arg(varlistptr, VK_tSignalHandle *);
	    signal_start = (VK_tSignalOffset) 
		va_arg(varlistptr, VK_tSignalOffset *) +  
		qqSignalStart(signal_handle);
	    signal_length = (VK_tSignalLength)
		va_arg(varlistptr, VK_tSignalLength *);
	}
 
        ve_index = 0;
	while (ve_index <= signal_length - 1) {
	    signal_element_handle = SignalElementHandle(signal_start + ve_index);
	    TempAddLast(L, signal_element_handle);
					/* add signal to set's signals */
					/* (point sens. set to signals to find
					   them when destroying sensitivity 
					   set) */
	    TempAddLast(qqTriggerEventSets(signal_element_handle), sens_set);
					/* add sensitivity set to signal */
  
            ve_index += ScalarUnionWordLength(qqElementClass(signal_element_handle));
	} /* for ve_index */
    } /* for sg_index */

    va_end(varlistptr);                  /* end retrieving args */
} /* VK_SensitizeSet */


/* *******************************************************************
*
* VK_SetWaitConditions - set process sensitivity for wait statement
*
* HISTORY:
*	20-DEC-89 DSB	Created
*	13-JUL-90 DSB	Added WaitFn, WaitContext
*	16-JUL-90 DSB	Added wait evaluation function parameters
*	16-JUL-90 DSB	Renamed VK_SetWaitSensitivity to VK_SetWaitConditions;
*			removed timed_wait parameter; using null time parameter
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*	13-MAY-91 DSB	W0052. Removed else_ parameter
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	10-JUN-91 DSB	Applying 64-bit time
*
******************************************************************* */

void VK_SetWaitConditions(process_id, sens_set, wait_fn, wait_context, time)
VK_tProcessHandle process_id;		/* process id. */
VK_tSensitivitySetHandle sens_set;
					/* sensitivity list for wait */
					/* -may be NULL if no sensitivity */
VK_tWaitEvalFnPtr wait_fn;		/* wait condition evaluation fn. */
					/* -may be NULL if no condition */
VK_tContextPtr wait_context;		/* wait evaluation fn. context */
					/* -may be NULL if no condition */
VK_tRelativeTimePtr time;		/* "for" clause time */
					/* -is NULL if no timeout */
{
    tAbsTimeBuf timeout_time;
    tTempBuf256 msg_buf;
    tTempBuf80 proc_buf;
    tTempBuf80 time_buf;
/**/
    /* NOTE: Each of sens_set, time, and wait_fn may be NULL */

    if ((time != NULL) && STIME_NEGATIVE(*time)) {
	FormatHierarchicalName(proc_buf, sizeof(proc_buf), process_id);
	STIME_FORMAT(time_buf, *time);
	sprintf(msg_buf, 
		"Negative time (%s %s) in 'wait for' in process %s\n",
		time_buf, BaseUnit, proc_buf);
	ReportKernelError(msg_buf, "", "", 0);
	
	return;
    }

    if (sens_set != qqCurrentSensSet(process_id)) {
	if (NULL != qqCurrentSensSet(process_id)) {
	    SetqqEnabled(qqCurrentSensSet(process_id), False);
				/* disable previous sensitivity set */
	}
	SetqqCurrentSensSet(process_id, sens_set);
	if (NULL != qqCurrentSensSet(process_id)) {
	    SetqqEnabled(qqCurrentSensSet(process_id), True);
				/* enable new sensitivity set */
	}
    }

    if (NULL != qqScheduled(process_id)) {
        /* cancel previous wakeup request */
        DequeueEvent(qqScheduled(process_id));
    }

    if ((time != NULL)) {
	UTIME_ADD(timeout_time, *time, SimTime);
	SetqqScheduled(process_id,
                       QueueEvent(&timeout_time,ev_ProcessWakeUp,process_id));
    }

    SetqqWaitFn     (process_id, wait_fn);   /* note that these may be NULL */
    SetqqWaitContext(process_id, wait_context);

} /* VK_SetWaitConditions */



#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */


<static> Fntype RtnName(parm)				/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif
