static char SCCS_ID[] = "@(#)TKRecord.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKRecord.c
*
* DESCRIPTION:  
*	<then describe contents>
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	20-DEC-90 AWH	Created
*	18-MAR-91 HK	Added references to TIME macros.
*	02-APR-91 WAG	W0015.  Included TKMisc.h.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	21-MAY-91 DSB	Cleaned up old types; renamed to remove some VK_
*	31-MAY-91 JBR	W0019: Changed file I/O methods for trace file 1.0.
*	03-JUN-91 JBR	W0019: Removed WriteTypeDeclaration.
*	21-AUG-91 DSB	Don't use in-channel "<NULL>" for empty or null string
* 	10-SEP-91 MNP	Mem fix in RD_ReadNextRecord.
*	23-SEP-91 MNP	Added Version support. See the CurrentState global.
*
* SCCS ID: @(#)TKRecord.c	42.1 1/10/92
*
******************************************************************* */

#include <stdio.h>		/* for *scanf, *printf */
#include "TKCommon.h"		/* for other system protos */
#include "TKRecord.h"
#include "TKData.h"		/* for TIME macros */
#include "TKGlobals.h"		/* for: GF */
#include "TKTemp.h"		/* for: AllocateMemory and FreeMemory */

/*****************************************************************************
*	Module Types & Constants 
*****************************************************************************/

#define TagLength			2
#define c_AccessType			"AT"
#define c_Assertion 			"AV"
#define c_BreakPoint			"BP"
#define c_CommandResponse		"CR"
#define c_Error 			"ER"
#define c_EnumerationType		"ET"
#define c_SignalEvent 			"EV"
#define c_Terminated 			"EX"
#define c_FloatingType			"FT"
#define c_IntegerType			"IT"
#define c_TextLine 			"LI"
#define c_EndOfSignalMap 		"MM"
#define c_PhysicalType			"PT"
#define c_LargePhysicalType		"LT"	
#define c_RecordType			"RT"
#define c_SimulationCycle 		"SC"
#define c_SignalDeclaration 		"SD"
#define c_SignalElementDeclaration	"SE"
#define c_SignalGroupDeclaration 	"SG"
#define c_SignalInitialization 		"SI"
#define c_Status 			"SS"
#define c_Subtype			"ST"
#define c_TimeAdvance 			"TA"
#define c_SignalTransaction 		"TR"
#define c_TimeUnit			"TU"
#define c_UserPrompt 			"UP"
#define c_Version			"VN"
#define c_VersionError			"VE"
#define c_ArrayType			"VT"
/* let's make these three characters so they make sense (VT, etc.) */

#define TagMatch(s1, s2) ((s1[0] == s2[0]) && (s1[1] == s2[1]))

#define StoreString(STRUCTURE, MEMBER, STRING) \
	STRUCTURE.MEMBER = ProcessString(STRING)
/* ?? this should be STORE_STRING (because it is a macro does things than
	a function call should not do; also, the parameters should not be
	uppercase ?? */
#define SaveString(STRUCTURE, MEMBER, STRING) \
	STRUCTURE.MEMBER = STRING
/* ?? WHY? Why do we need a macro to do structure.member? ?? */
/* ?? One more time: macro parameters should NOT be upper case ?? */

#define DefineNull(S) (((S) && *(S)) ? S : "")

#define IsWhitespace(C) ((C) == ' ' || (C) == '\t' || (C) == '\n')

/*****************************************************************************
*	Module Objects
*****************************************************************************/

static FILE * OutputFP;
static FILE * InputFP;
static Boolean1 SlaveMode = False;

static int NextType = 1;		/* trace file number for next type or
					   subtype */

/* CurrentState
   Used to detect condition where the first record read is not a version
   one, or a wrong version. The first record RD_ReadNextRecord returns is an
   RD_Version if good, or RD_VersionError if bad. If bad, RD_ReadNextRecord
   returns True on the first call, and False on subsequent calls. - MNP */

static enum {
    BeforeHeader,		/* Header not read yet. */
    AfterHeader,		/* Header read, it's ok. */
    HeaderError			/* Bad header (old version, or trash). */
} CurrentState;				

/* *******************************************************************
*
* ProcessString - if not null; duplicate a string 
*
* HISTORY:
*	20-DEC-90 AWH	Created
*	02-APR-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*	21-AUG-91 DSB	Removed non-designed mapping of "<NULL>" to null ptr.
*
******************************************************************* */
static RD_tString ProcessString(string)
RD_tString string;
{
    RD_tInteger size;
    RD_tString new_string;
/**/
    if (NULL == string ) {
	new_string = NULL;
    }
    else {
    	size = strlen(string);
        AllocateMemory(new_string, char, size + 1);

	if (NULL != new_string) {
    	    strcpy(new_string, string);
        }
    }

    return new_string;

} /* ProcessString */


/* *******************************************************************
*
* ReadString - Read a string from the trace file
*
* HISTORY:
*	28-MAY-91 JBR	Created
*	16-DEC-91 DKC	Changed to use resizing buffer
*
******************************************************************* */

RD_tString ReadString(stream)		/* the processed string */
FILE * stream;				/* in: the input file */
{
    static char *buffer  = "";		/* input string */
    static int  buff_len = 0;
#define BUFR_INIT 500
#define BUFR_INCR 100

    int i = 0;				/* number of characters read */
    int byte;				/* integer form of one character of
					   input */
/**/
    /* skip leading whitespace */
    byte = getc(stream);
    if (EOF == byte) {
	return (RD_tString) NULL;
    }					/* ?? WHERE IS ELSE? ?? */
    while (IsWhitespace(byte)) {
	byte = getc(stream);
	if (EOF == byte) {
	    return (RD_tString) NULL;	/* ?? RETURN FROM OUTSIDE LOOP SO
					   THERE CAN BE AN ELSE ?? */
	}
    }	/* while (IsWhitespace(byte)) */

    /* verify initial " */
    if ('"' != (char) byte) {
	/* ?? report error ?? */
	return (RD_tString) NULL;
    }					/* ?? WHERE IS ELSE ?? */


    /* continue reading until closing " is found */
    for (;;) {
/* ?? style note: The CLSI style guide says to use "while (True) {" for
	this situation. ?? */

        if (i >= buff_len) { /* check for sufficient buffer */
            if (0 == buff_len) {
                buffer = (char *)malloc(BUFR_INIT);
                buff_len  = BUFR_INIT;
            }
            else {
                buff_len += BUFR_INCR;
                buffer = (char *)realloc(buffer,buff_len);
            }
        }

	/* get character and check for end of file */
        byte = getc(stream);
	if (EOF == byte) {
	    buffer[i] = '\0';
	    return ProcessString(buffer);
	}

	/* process character */
	switch ((char) byte) {
	    case '\"':
		/* end of string */
		buffer[i] = '\0';
		return ProcessString(buffer);
		break;

	    case '\\':
		/* get the escaped character */
		byte = getc(stream);
		if (EOF == byte) {
		    buffer[i] = '\0';
		    return ProcessString(buffer);
		}
		buffer[i++] = (char) byte;
		break;

	    default:
		/* save character */
		buffer[i++] = (char) byte;
		break;
	}   /* switch ((char) byte) */
    }	/* for (;;) */
/* ?? DON'T RETURN FROM MULTIPLE PLACES; IT'S NOT STRUCTURED! ?? */
} /* ReadString */



/* *******************************************************************
*
* WriteString - Format and write a string to the trace file
*
* HISTORY:
*	28-MAY-91 JBR	Created
*
******************************************************************* */

void WriteString(str, stream)
RD_tString str;				/* in: the string to write */
FILE * stream;				/* in: the input file */
{
/**/
    /* write the initial " */
    putc('\"', stream);

    /* check for trivial call */
    if (NULL == str) {
	putc('\"', stream);
	return;
    }

    while (*str != '\0') {
	switch (*str) {
	    case '\"':
	    case '\n':
	    case '\\':
		putc('\\', stream);
		putc(*str, stream);
		break;

	    default:
		putc(*str, stream);
		break;
	}   /* switch (*str) */

	str++;
    }	/* while (*str != '\0') */

    /* write the closing " */
    putc('\"', stream);
} /* WriteString */


/* *******************************************************************
*
* ReadAssertion - 
*
* HISTORY:
*	...
*       11-MAY-90 DSB   Added process pathname
*	28-DEC-90 AWH	Moved from FmtWave
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*
******************************************************************* */

static void ReadAssertion(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString msg_buf;
    RD_tString source_file;
    RD_tString proc_name;
/**/
    record->type = RD_Assertion;

    source_file = ReadString(stream);
    fscanf(stream, "%d", &record->assertion.line_number);
    proc_name = ReadString(stream);
    fscanf(stream, "%d", (int *) &record->assertion.severity);
    msg_buf = ReadString(stream);

    SaveString(record->assertion, message, msg_buf);
    SaveString(record->assertion, process_name, proc_name);
    SaveString(record->assertion, source_file, source_file);

} /* ReadAssertion */



/* *******************************************************************
*
* ReadBreakPoint - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*	03-JUN-91 JBR	W0019: more changes in file I/O.
*
******************************************************************* */

static void ReadBreakPoint(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString library_name;
    RD_tString primary_unit;
    RD_tString secondary_unit;
    RD_tString process_name;
    RD_tString source_file;
/**/
    record->type = RD_BreakPoint;
    fscanf(stream, "%d", &record->break_point.bp_class);
    library_name = ReadString(stream);
    primary_unit = ReadString(stream);
    secondary_unit = ReadString(stream);
    process_name = ReadString(stream);
    source_file = ReadString(stream);
   fscanf(stream, "%d", &record->break_point.line_number);

   SaveString(record->break_point, library, library_name);
   SaveString(record->break_point, primary, primary_unit);
   SaveString(record->break_point, secondary, secondary_unit);
   SaveString(record->break_point, process, process_name);
   SaveString(record->break_point, source_file, source_file);

} /* ReadBreakPoint */



/* *******************************************************************
*
* ReadError - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*	03-JUN-91 JBR	W0019: more changes in style of file I/O.
*
******************************************************************* */

static void ReadError(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString source_file_buf;
    RD_tString msg_buf;
/**/
    record->type = RD_Error;
    source_file_buf = ReadString(stream);
    fscanf(stream, "%d", &record->error.line_number);
    msg_buf = ReadString(stream);

    SaveString(record->error, source_file, source_file_buf);
    SaveString(record->error, message, msg_buf);
    StoreString(record->error, extra, NULL); /* Not Implemented yet ??? */

} /* ReadError */



/* *******************************************************************
*
* ReadIntegerType - 
*
* HISTORY:
*	04-JUN-91 JBR	Created
*
******************************************************************* */

static void ReadIntegerType(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString name_buf;
    char c;
/**/
    record->type = RD_IntegerType;
    fscanf(stream, "%d", &record->integer_type.number);
    name_buf = ReadString(stream);
    fscanf(stream, "%d %c %d",
	&record->integer_type.left,
	&c,
	&record->integer_type.right);

    if (c == 'A') {
	record->integer_type.dir = VK_Ascending;
    }
    else if (c == 'D') {
	record->integer_type.dir = VK_Descending;
    }
    else {
	record->integer_type.dir = VK_BadDirection;
    }

    SaveString(record->integer_type, name, name_buf);

} /* ReadIntegerType */



/* *******************************************************************
*
* ReadFloatingType - 
*
* HISTORY:
*	04-JUN-91 JBR	Created
*
******************************************************************* */

static void ReadFloatingType(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString name_buf;
    char c;
/**/
    record->type = RD_FloatingType;
    fscanf(stream, "%d", &record->integer_type.number);
    name_buf = ReadString(stream);
/* ?? Why is this "integer_type"? ?? */
    fscanf(stream, "%f %c %f",
	&record->integer_type.left,
	&c,
	&record->integer_type.right);

    if (c == 'A') {
	record->integer_type.dir = VK_Ascending;
    }
    else if (c == 'D') {
	record->integer_type.dir = VK_Descending;
    }
    else {
	record->integer_type.dir = VK_BadDirection;
    }

    SaveString(record->integer_type, name, name_buf);

} /* ReadFloatingType */



/* *******************************************************************
*
* ReadEnumerationType - 
*
* HISTORY:
*	06-JUN-91 JBR	Created
*
******************************************************************* */

static void ReadEnumerationType(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString name_buf;
    int i;
/**/
    record->type = RD_EnumerationType;
    fscanf(stream, "%d", &record->enumeration_type.number);
    name_buf = ReadString(stream);
    SaveString(record->enumeration_type, name, name_buf);

    fscanf(stream, "%d", &record->enumeration_type.cardinality);
    AllocateMemory(record->enumeration_type.literals, VK_tName,
	record->enumeration_type.cardinality);

/* ?? style note: the CLSI style guide says to use "for (i = left; i <= right;
		...)" instead of "for (i = left; i < right+1; ...)" ?? */
    for (i = 0; i < record->enumeration_type.cardinality; i++) {
	name_buf = ReadString(stream);
	SaveString(record->enumeration_type, literals[i], name_buf);
    }

} /* ReadEnumerationType */



/* *******************************************************************
*
* ReadPhysicalType - 
*
* HISTORY:
*	06-JUN-91 JBR	Created
*	07-JUN-91 JBR	Wasn't reading base unit name.
*
******************************************************************* */

static void ReadPhysicalType(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString name_buf;
    char c;
    int i;
/**/
    record->type = RD_PhysicalType;
    fscanf(stream, "%d", &record->physical_type.number);
    name_buf = ReadString(stream);
    SaveString(record->physical_type, name, name_buf);
    fscanf(stream, "%d %c %d",
	&record->physical_type.left,
	&c,
	&record->physical_type.right);

    if (c == 'A') {
	record->physical_type.dir = VK_Ascending;
    }
    else if (c == 'D') {
	record->physical_type.dir = VK_Descending;
    }
    else {
	record->physical_type.dir = VK_BadDirection;
    }

    name_buf = ReadString(stream);
    fscanf(stream, "%d", &record->physical_type.cardinality);
    AllocateMemory(record->physical_type.literals, VK_tName,
	record->physical_type.cardinality);
    AllocateMemory(record->physical_type.positions, RD_tInteger,
	record->physical_type.cardinality);
    SaveString(record->physical_type, literals[0], name_buf);
    SaveString(record->physical_type, positions[0], 1);
    for (i = 1; i < record->physical_type.cardinality; i++) {
	name_buf = ReadString(stream);
	SaveString(record->physical_type, literals[i], name_buf);

	fscanf(stream, "%d", &(record->physical_type.positions[i]));
    }

} /* ReadPhysicalType */



/* *******************************************************************
*
* ReadLargePhysicalType - 
*
* HISTORY:
*	11-JUN_91 DSB	Created from ReadPhysicalType ???? NOT FINISHED ??
*
******************************************************************* */
#ifdef NOT_USED

static void ReadLargePhysicalType(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString name_buf;
    char c;
    int i;
/**/
    record->type = RD_LargePhysicalType;
    fscanf(stream, "%d", &record->large_physical_type.number);
    name_buf = ReadString(stream);
    SaveString(record->large_physical_type, name, name_buf);
    fscanf(stream, "%d %c %d",
	&record->large_physical_type.left,	/* ???? */
	&c,
	&record->large_physical_type.right);	/* ???? */

    if (c == 'A') {
	record->large_physical_type.dir = VK_Ascending;
    }
    else if (c == 'D') {
	record->large_physical_type.dir = VK_Descending;
    }
    else {
	record->large_physical_type.dir = VK_BadDirection;
    }

    name_buf = ReadString(stream);
    fscanf(stream, "%d", &record->large_physical_type.cardinality);
    AllocateMemory(record->large_physical_type.literals, VK_tName,
	record->large_physical_type.cardinality);
    AllocateMemory(record->large_physical_type.positions, RD_tInteger,
	record->large_physical_type.cardinality);
    SaveString(record->large_physical_type, literals[0], name_buf);
    record->large_physical_type.positions[0] = 1;
    for (i = 1; i < record->large_physical_type.cardinality; i++) {
	name_buf = ReadString(stream);
	SaveString(record->large_physical_type, literals[i], name_buf);

	fscanf(stream, "%d", &(record->large_physical_type.positions[i]));
					/* ???? */
    }

} /* ReadLargePhysicalType */

#endif

/* *******************************************************************
*
* ReadSignalDeclaration - 
*
* HISTORY:
*	...
*	24-APR-90 DSB	Created from LoadMap
*	20-DEC-90 AWH	Moved from TKIO
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*	11-JUN-91 DSB	Added 'L' for large physical type
*	23-OCT-91 MNP	Added simple name
*
******************************************************************* */

static void ReadSignalDeclaration(record, stream)
RD_tRecord * record;
FILE * stream;
{
    char type_char;
    char group_char;
    RD_tString signal_name;
    RD_tString simple_name;
/**/
    record->type = RD_SignalDeclaration;
    fscanf(stream, " %d %d %c %c %d", 
	&record->signal_declaration.start,
	&record->signal_declaration.length,
	&group_char,
	&type_char,
	&record->signal_declaration.width);

    signal_name = ReadString(stream);
    simple_name = ReadString(stream);

    switch (group_char) {
	case 'S': record->signal_declaration.group = gc_ScalarSignal; break;
	case 'V': record->signal_declaration.group = gc_VectorSignal; break;
	case 'C': record->signal_declaration.group = gc_CompositeSignal; break;
	case '?': record->signal_declaration.group = gc_BadGroupClass; break;
    } /* switch */

    switch (type_char) {
	case 'E': record->signal_declaration.class = tc_Enumeration;	break;
	case 'I': record->signal_declaration.class = tc_Integer;	break;
	case 'P': record->signal_declaration.class = tc_Physical;	break;
	case 'L': record->signal_declaration.class = tc_LargePhysical;	break;
	case 'F': record->signal_declaration.class = tc_Floating;	break;
	case 'A': record->signal_declaration.class = tc_Array;		break;
	case 'R': record->signal_declaration.class = tc_Record;		break;
	case '?': record->signal_declaration.class = tc_BadTypeClass;	break;
     } /* switch */

     SaveString(record->signal_declaration, name, signal_name);
     SaveString(record->signal_declaration, simplename, simple_name);

} /* ReadSignalDeclaration */

/* *******************************************************************
*
* ReadSignalGroupDeclaration - 
*
* HISTORY:
*	...
*	24-APR-90 DSB	Created from LoadMap
*	20-DEC-90 AWH	Moved from TKIO
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*	11-JUN-91 DSB	Added 'L' for large physical type
*	23-OCT-91 MNP	Added simple name
*	 1-NOV-91 MNP	Split from ReadSignalDeclaration()
*
******************************************************************* */

static void ReadSignalGroupDeclaration(record, stream)
RD_tRecord * record;
FILE * stream;
{
    char group_char;
    RD_tString name;
    RD_tString simple_name;
/**/
    record->type = RD_SignalGroupDeclaration;
    fscanf(stream, " %d %d %d %c", 
	&record->signal_group_declaration.start,
	&record->signal_group_declaration.group_id,
	&record->signal_group_declaration.length,
	&group_char);

    name = ReadString(stream);
    simple_name = ReadString(stream);

    switch (group_char) {
      case 'S':
	record->signal_group_declaration.group = gc_ScalarSignal;
	break;
      case 'V':
	record->signal_group_declaration.group = gc_VectorSignal;
	break;
      case 'C':
	record->signal_group_declaration.group = gc_CompositeSignal;
	break;
      case '?':	
	record->signal_group_declaration.group = gc_BadGroupClass;
	break;
    } /* switch */

    SaveString(record->signal_group_declaration, name, name);
    SaveString(record->signal_group_declaration, simple_name, simple_name);

} /* ReadSignalGroupDeclaration */


/* *******************************************************************
*
* ReadSignalElementDeclaration - 
*
* HISTORY:
*	...
*	24-APR-90 DSB	Created from LoadMap
*	20-DEC-90 AWH	Moved from TKIO
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*	11-JUN-91 DSB	Added 'L' for large physical type
*	23-OCT-91 MNP	Added simple name
*	 1-NOV-91 MNP	Split from ReadSignalDeclaration()
*
******************************************************************* */

static void ReadSignalElementDeclaration(record, stream)
RD_tRecord * record;
FILE * stream;
{
    char type_char;
    RD_tString subel_name;
/**/
    record->type = RD_SignalElementDeclaration;
    fscanf(stream, " %d %d %c %d", 
	&record->signal_element_declaration.start,
	&record->signal_element_declaration.group_id,
	&type_char,
	&record->signal_element_declaration.width);

    subel_name = ReadString(stream);

    switch (type_char) {
      case 'E': record->signal_element_declaration.class = tc_Enumeration;
	break;
      case 'I': record->signal_element_declaration.class = tc_Integer;
	break;
      case 'P': record->signal_element_declaration.class = tc_Physical;
	break;
      case 'L': record->signal_element_declaration.class = tc_LargePhysical;
	break;
      case 'F': record->signal_element_declaration.class = tc_Floating;
	break;
      case 'A': record->signal_element_declaration.class = tc_Array;
	break;
      case 'R': record->signal_element_declaration.class = tc_Record;
	break;
      case '?': record->signal_element_declaration.class = tc_BadTypeClass;
	break;
     } /* switch */

    SaveString(record->signal_element_declaration, subel_name, subel_name);    

} /* ReadSignalElementDeclaration */


/* *******************************************************************
*
* ReadSignalEvent - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*
******************************************************************* */

static void ReadSignalEvent(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString signal_value;
/**/
    record->type = RD_SignalEvent;
    fscanf(stream, " %d", &record->signal_event.signal);
    signal_value = ReadString(stream);
    SaveString(record->signal_event, value, signal_value);

} /* ReadSignalEvent */


/* *******************************************************************
*
* ReadSignalInitialization- 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*	03-JUN-91 JBR	W0019: more changes in style of file I/O.
*
******************************************************************* */

static void ReadSignalInitialization(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tString signal_value;
/**/
    record->type = RD_SignalInitialization;

    fscanf(stream, " %d", &record->signal_initialization.signal);

    signal_value = ReadString(stream);
    SaveString(record->signal_initialization, value, signal_value);

} /* ReadSignalInitialization */


/* *******************************************************************
*
* ReadVersion - Read a version record.
*
* Checks against current version, returns an RD_VersionError if bad.
*
* HISTORY:
*	23-SEP-91 MNP	Created 
*
******************************************************************* */

static void ReadVersion(record, stream)
RD_tRecord * record;
FILE * stream;
{
    RD_tInteger maj, min;
/**/
    fscanf(stream, " %d %d", &maj, &min);
    
    if (maj == RD_VERSION_MAJOR && min == RD_VERSION_MINOR) {
	record->type = RD_Version;
	record->version.first = maj;
	record->version.second = min;
	CurrentState = AfterHeader;
    }
    else {
	record->type = RD_VersionError;
	record->version_error.first = maj;
	record->version_error.second = min;
	CurrentState = HeaderError;
    }
} /* ReadVersion */



/* *******************************************************************
*
* RD_FreeRecord - free allocated memory associated with the record
*			but not the record itself.
*
* HISTORY:
*	20-DEC-90 AWH	Created
*	02-APR-91 WAG	W0015.  Replaced free call with FreeMemory.
*	07-JUN-91 JBR	Rewrote to free each record kind separately.
*	23-SEP-91 MNP	Added version.
*
******************************************************************* */
void RD_FreeRecord(record)				
RD_tRecord * record;
{
    int i;
/**/
    switch (record->type) {
	case RD_UnknownType:
	    FreeMemory(record->unknown_type.text);
	    break;
	case RD_TimeUnit:
	    FreeMemory(record->time_unit.unit);
	    break;
	case RD_IntegerType:
	    FreeMemory(record->integer_type.name);
	    break;
	case RD_FloatingType:
	    FreeMemory(record->floating_type.name);
	    break;
	case RD_EnumerationType:
	    FreeMemory(record->enumeration_type.name);
	    for (i = 0; i < record->enumeration_type.cardinality; i++) {
		FreeMemory(record->enumeration_type.literals[i]);
	    }
	    FreeMemory(record->enumeration_type.literals);
	    break;
	case RD_AccessType:
	    FreeMemory(record->access_type.name);
	    break;
	case RD_RecordType:
	    FreeMemory(record->record_type.name);
	    for (i = 0; i < record->record_type.elem_count; i++) {
		FreeMemory(record->record_type.elements[i]);
	    }
	    FreeMemory(record->record_type.elements);
	    FreeMemory(record->record_type.elem_type);
	    break;
	case RD_PhysicalType:
	    FreeMemory(record->physical_type.name);
	    for (i = 0; i < record->physical_type.cardinality; i++) {
		FreeMemory(record->physical_type.literals[i]);
	    }
	    FreeMemory(record->physical_type.literals);
	    FreeMemory(record->physical_type.positions);
	    break;
	case RD_LargePhysicalType:
	    FreeMemory(record->large_physical_type.name);
	    for (i = 0; i < record->large_physical_type.cardinality; i++) {
		FreeMemory(record->large_physical_type.literals[i]);
	    }
	    FreeMemory(record->large_physical_type.literals);
	    FreeMemory(record->large_physical_type.positions);
	    break;
	case RD_ArrayType:
	    FreeMemory(record->array_type.name);
	    FreeMemory(record->array_type.index);
	    break;
	case RD_Subtype:
	    FreeMemory(record->subtype.name);
	    break;
	case RD_FloatingSubtype:
	    FreeMemory(record->floating_subtype.name);
	    break;
	case RD_SignalDeclaration:
	    FreeMemory(record->signal_declaration.name);
	    break;
	case RD_EndOfSignalMap:
	    break;
	case RD_SignalInitialization:
	    FreeMemory(record->signal_initialization.value);
	    break;
	case RD_SignalTransaction:
	    break;
	case RD_SignalEvent:
	    FreeMemory(record->signal_event.value);
	    break;
	case RD_Assertion:
	    FreeMemory(record->assertion.message);
	    FreeMemory(record->assertion.process_name);
	    FreeMemory(record->assertion.source_file);
	    break;
	case RD_Error:
	    FreeMemory(record->error.message);
	    FreeMemory(record->error.extra);
	    FreeMemory(record->error.source_file);
	    break;
	case RD_TimeAdvance:
	    FreeMemory(record->time_advance.time);	/* MNP */
	    break;
	case RD_SimulationCycle:
	    break;
	case RD_UserPrompt:
	    FreeMemory(record->user_prompt.text);
	    break;
	case RD_TextLine:
	    FreeMemory(record->text_line.text);
	    break;
	case RD_BreakPoint:
	    FreeMemory(record->break_point.library);
	    FreeMemory(record->break_point.primary);
	    FreeMemory(record->break_point.secondary);
	    FreeMemory(record->break_point.process);
	    FreeMemory(record->break_point.source_file);
	    break;
	case RD_CommandResponse:
	    FreeMemory(record->command_response.text);
	    break;
	case RD_Terminated:
	case RD_Version:
	case RD_VersionError:
	case RD_Status:
	default:
	    break;
    }	/* switch ((*record).type) */

} /* RD_FreeRecord */


/* *******************************************************************
*
* RD_ReadNextRecord - read and parse next record
*
* The first record returned is an RD_Version if the version is Ok, or
* RD_VersionError if not. If not, RD_ReadNextRecord will  return True
* on the first call, and False on subsequent calls. - MNP 
*
* HISTORY:
*	20-DEC-90 AWH	Created
*	31-MAY-91 JBR	W0019.  Changed style of file I/O.
*	03-JUN-91 JBR	W0019.  More I/O changes.
*	07-JUN-91 JBR	Removed a reference to StoreString.
*	13-JUN-91 AWH	Changed the trailing white space eater to end
*			at newlines.
* 	10-SEP-91 MNP	Fixed UnknownType condition, uses auto memory.
*	23-SEP-91 MNP	Added Version support. See above.
*
******************************************************************* */

VK_tBoolean RD_ReadNextRecord(record)				
RD_tRecord * record;
{
    char buffer[TagLength + 1];	/* Only need to hold two characters */
    RD_tString buffer2;
    RD_tString time_buffer;
    RD_tInteger len;
    VK_tBoolean more;
    char tmpbuf[81];		/* for error reporting  */
    int newline;

/**/
    /* Return False if the last call was errorful. */
					/* ^ That is not a word! */
    if (CurrentState == Error)
	return False;		
	
    more = True;
  
    if (feof(InputFP)) {
	clearerr(InputFP);	/* allow fgets to try again on reading */
    }
				/* NOTE: this kludge has been used to support
					processessing the trace file as it
					is new input is added */

    if (NULL == fgets(buffer, sizeof(buffer), InputFP)) {
	more = False;
    }
    else {

	/* clean up new lines */
	len = strlen(buffer);
	if ('\n' == buffer[len - 1]) {
		buffer[len - 1] ='\0';
        }

	/* initialize the record */
	record->type = RD_UnknownType;

	if 	(TagMatch(buffer, c_IntegerType)) {
	   ReadIntegerType(record, InputFP);
	} 
	else if (TagMatch(buffer, c_FloatingType)) {
	    ReadFloatingType(record, InputFP);
	}
	else if (TagMatch(buffer, c_EnumerationType)) {
	    ReadEnumerationType(record, InputFP);
	}
	else if (TagMatch(buffer, c_PhysicalType)) {
	    ReadPhysicalType(record, InputFP);
	}
	else if (TagMatch(buffer, c_SignalDeclaration)) {
	    ReadSignalDeclaration(record, InputFP);
	}
	else if (TagMatch(buffer, c_SignalGroupDeclaration)) {
	    ReadSignalGroupDeclaration(record, InputFP);
	}
	else if (TagMatch(buffer, c_SignalElementDeclaration)) {
	    ReadSignalElementDeclaration(record, InputFP);
	}
	else if (TagMatch(buffer, c_EndOfSignalMap)) {
	   record->type = RD_EndOfSignalMap;
	}
	else if (TagMatch(buffer, c_SignalInitialization)) {
	   ReadSignalInitialization(record, InputFP);
	}
	else if (TagMatch(buffer, c_SignalTransaction)) {
	   record->type = RD_SignalTransaction;
	   fscanf(InputFP, "%d", &record->signal_transaction.signal);
	}
	else if (TagMatch(buffer, c_SignalEvent)) {
	   ReadSignalEvent(record, InputFP);
	}
	else if (TagMatch(buffer, c_Assertion)) {
	   ReadAssertion(record, InputFP); 
	}
	else if (TagMatch(buffer, c_Error)) {
	   ReadError(record, InputFP); 
	}
	else if (TagMatch(buffer, c_TimeAdvance)) {
	   record->type = RD_TimeAdvance;
	   time_buffer = ReadString(InputFP);
	   TIME_CONVERT(record->time_advance.time, time_buffer);
	}
	else if (TagMatch(buffer, c_SimulationCycle)) {
	   record->type = RD_SimulationCycle;
	}
	else if (TagMatch(buffer, c_UserPrompt)) {
	   record->type = RD_UserPrompt;
	   buffer2 = ReadString(InputFP);
	   SaveString(record->user_prompt, text, buffer2);
	}
	else if (TagMatch(buffer, c_BreakPoint)) {
	   ReadBreakPoint(record, InputFP);
	}
	else if (TagMatch(buffer, c_Terminated)) {
	   record->type = RD_Terminated;
	   fscanf(InputFP, "%d", &record->terminated.reason);
	}
	else if (TagMatch(buffer, c_TextLine)) {
	   record->type = RD_TextLine;
	   buffer2 = ReadString(InputFP);
	   SaveString(record->text_line, text, buffer2);
	}
	else if (TagMatch(buffer, c_Status)) {
	   record->type = RD_Status;
	   fscanf(InputFP, "%d", &record->status.status);
	}
	else if (TagMatch(buffer, c_CommandResponse)) {
	   record->type = RD_CommandResponse;
	   buffer2 = ReadString(InputFP);
	   SaveString(record->command_response, text, buffer2);
        }
	else if (TagMatch(buffer, c_Version)) {
	    ReadVersion(record, InputFP);   /* Sets CurrentState  */
	}
	else if (TagMatch(buffer, c_VersionError)) {
	    ReadVersion(record, InputFP);   /* Sets CurrentState */	
	}
	else {
	   record->type = RD_UnknownType;

	   /* Only report a reasonable amount, say 80 char, since we
	      don't know when the next newline is; fgets could want
	      several megs since this is an error condition. MNP */
	   
	   tmpbuf[0] = buffer[0];
	   tmpbuf[1] = buffer[1];
	   fgets(&tmpbuf[2], sizeof(tmpbuf) - 2, InputFP);
	   StoreString(record->unknown_type, text, (RD_tString)tmpbuf);
        }
    }

    /* If the first record read was not a version one, complain */

    if (CurrentState == BeforeHeader && record->type !=	RD_UnknownType) {
	RD_FreeRecord(record);
	record->type = RD_VersionError;
	record->version_error.first = RD_OLD_VERSION_MAJOR;
	record->version_error.second = RD_OLD_VERSION_MINOR;
	CurrentState = Error;
    }

    /* Skip over the closing whitespace */
    /* AWH: Careful here.. this function needs to return exactily at
       the end of line. If the code waits for one more character
       it will cause the simtool to hang! */

    if (more) {
	newline = getc(InputFP);
	while (newline != '\n' && newline != EOF && 
	       IsWhitespace((char) newline)) {
	    newline = getc(InputFP);
	}
	if (newline != '\n' &&  newline != EOF) {
	    ungetc(newline, InputFP);
	}
    }

    return more;
} /* RD_ReadNextRecord */



/* *******************************************************************
*
* RD_SetInput - set input file pointer
*
* HISTORY:
*	20-DEC-90 AWH	Created
*	23-SEP-91 MNP	Added version.
*
******************************************************************* */
void RD_SetInput(fp)				
FILE * fp;			/* input file pointer */
{
/**/
    InputFP = fp;
    CurrentState = BeforeHeader; /* reading new file; must see version */
} /* RD_SetInput */



/* *******************************************************************
*
* RD_SetOutput - set output file pointer, and write version
*
* HISTORY:
*	20-DEC-90 AWH	Created
*	23-SEP-91 MNP	Added version.
*
******************************************************************* */
void RD_SetOutput(fp)				
FILE * fp;			/* output file pointer */
{
/**/
    setlinebuf(fp);		
    OutputFP = fp;

    RD_WriteVersion();		/* So RD users don't have to... */

} /* RD_SetOutput */


/* *******************************************************************
*
* RD_SetVirtualFileMode - output additional data; "virtual trace file"
*
* HISTORY:
*	11-JUN-90 AWH	Created
*
******************************************************************* */
void RD_SetVirtualFileMode(slave)				
Boolean1 slave;		      	/* slave mode */
{
/**/
    SlaveMode = slave;

} /* RD_SetVirtualFileMode */



/* ?? style note: shouldn't have two spaces before function name (only one)
	?? */
/* *******************************************************************
*
*  RD_WriteIntegerType - 
*
* HISTORY:
*	04-JUN-91 JBR	Created
*
******************************************************************* */
void RD_WriteIntegerType(name, left, dir, right, trace_num)
VK_tName name;
VK_tInteger left;
VK_tDirection dir;
VK_tInteger right;
int *trace_num;
{
/**/
    (*trace_num) = NextType++;
    fprintf(OutputFP, "%s %d ",
	c_IntegerType,		/* record type */
	(*trace_num)
	);

    WriteString(DefineNull(name), OutputFP);

    fprintf(OutputFP, " %d %c %d\n",
	left,
	dir == VK_Ascending ? 'A' : (dir == VK_Descending ? 'D' : '?'),
	right
	);

    fflush(OutputFP);
    
} /* RD_WriteIntegerType */




/* *******************************************************************
*
*  RD_WriteFloatingType - 
*
* HISTORY:
*	04-JUN-91 JBR	Created
*
******************************************************************* */
void RD_WriteFloatingType(name, left, dir, right, trace_num)
VK_tName name;
VK_tFloatingData left;
VK_tDirection dir;
VK_tFloatingData right;
int *trace_num;
{
/**/
    (*trace_num) = NextType++;
    fprintf(OutputFP, "%s %d ",
	c_FloatingType,		/* record type */
	(*trace_num)
	);

    WriteString(DefineNull(name), OutputFP);
/* ?? %f is not what we want; it writes many digits that are not significant;
	maybe use whatever format that FormatScalarValue currently uses ?? */
    fprintf(OutputFP, " %f %c %f\n",
	left,
	dir == VK_Ascending ? 'A' : (dir == VK_Descending ? 'D' : '?'),
	right
	);
    fflush(OutputFP);

} /* RD_WriteFloatingType */




/* *******************************************************************
*
*  RD_WriteEnumerationType - 
*
* HISTORY:
*	04-JUN-91 JBR	Created
*
******************************************************************* */
void RD_WriteEnumerationType(name, cardinality, literals, trace_num)
VK_tName name;
VK_tListLength cardinality;
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h,
	and VK_RegisterEnumerationTypeDecl.cardinality ?? */
VK_tName * literals;
int *trace_num;
{
    int i;				/* loop iterator */
/**/
    (*trace_num) = NextType++;
    fprintf(OutputFP, "%s %d ",
	c_EnumerationType,		/* record type */
	(*trace_num)
	);

    WriteString(DefineNull(name), OutputFP);

    fprintf(OutputFP, " %d", cardinality);

    for (i = 0; i < cardinality; i++) {
	putc(' ', OutputFP);
	WriteString(DefineNull(literals[i]), OutputFP);
    }
    putc('\n', OutputFP);

} /* RD_WriteEnumerationType */




/* *******************************************************************
*
*  RD_WritePhysicalType - 
*
* HISTORY:
*	06-JUN-91 JBR	Created
*
******************************************************************* */
void RD_WritePhysicalType(name, left, dir, right, unit_count, units,
	positions, trace_num)
VK_tName name;
VK_tScalarData left;
VK_tDirection dir;
VK_tScalarData right;
VK_tListLength unit_count;
VK_tName * units;
VK_tIntegerArray positions;
int *trace_num;
{

#if 0 /* ?? DAC MANIA: Don't need it, can't fix it, ifdefit */

    int i;				/* loop iterator */
/**/
    (*trace_num) = NextType++;
    fprintf(OutputFP, "%s %d ",
	c_PhysicalType,		/* record type */
	(*trace_num)
	);

    WriteString(DefineNull(name), OutputFP);

    fprintf(OutputFP, " %d %c %d ",
	left,
	dir == VK_Ascending ? 'A' : (dir == VK_Descending ? 'D' : '?'),
	right
	);

    WriteString(DefineNull(units[0]), OutputFP);

    fprintf(OutputFP, " %d", unit_count);

    for (i = 1; i < unit_count; i++) {
	putc(' ', OutputFP);
	WriteString(DefineNull(units[i]), OutputFP);
	fprintf(OutputFP, " %d", positions[i]);
    }

    putc('\n', OutputFP);

    fflush(OutputFP);

#endif

} /* RD_WritePhysicalType */



/* *******************************************************************
*
* RD_WriteLargePhysicalType - 
*
* HISTORY:
*	11-JUN-91 DSB	Created from RD_WritePhysicalType ???? NOT FINISHED ??
*
******************************************************************* */
void RD_WriteLargePhysicalType(name, left, dir, right, unit_count, units,
	positions, trace_num)
VK_tName name;
VK_tLargeAddr left;
VK_tDirection dir;
VK_tLargeAddr right;
VK_tListLength unit_count;
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h ?? */
VK_tName * units;
VK_tIntegerArray positions;
int *trace_num;
{

#if 0 /* ?? DAC MANIA: Don't need it, can't fix it, ifdefit */

    int i;				/* loop iterator */
/**/

    (*trace_num) = NextType++;
    fprintf(OutputFP, "%s %d ",
	c_LargePhysicalType,		/* record type */
	(*trace_num)
	);

    WriteString(DefineNull(name), OutputFP);

    fprintf(OutputFP, " %d %c %d ",
	left,	/* ???? */
	dir == VK_Ascending ? 'A' : (dir == VK_Descending ? 'D' : '?'),
	right	/* ???? */
	);

    WriteString(DefineNull(units[0]), OutputFP);

    fprintf(OutputFP, " %d", unit_count);

    for (i = 1; i < unit_count; i++) {
	putc(' ', OutputFP);
	WriteString(DefineNull(units[i]), OutputFP);
	fprintf(OutputFP, " %d", positions[i]); /* ???? */
    }

    putc('\n', OutputFP);
    fflush(OutputFP);

#endif
} /* RD_WriteLargePhysicalType */




/* *******************************************************************
*
*  RD_WriteSignalDeclaration - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	04-APR-91 WAG	W0023.  Modified to conform to coding style standards.
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*	11-JUN-91 DSB	Added 'L' for large physical type
*	23-OCT-91 MNP	Added simple name.
*
******************************************************************* */
void RD_WriteSignalDeclaration(name, simplename, class, start,
			       length, group, width)
RD_tString name;
RD_tString simplename;
tTypeClass class;
tSignalNumber start;
VK_tSignalLength length;
tGroupClass group;
RD_tInteger width;
{
    char class_ch;
    char group_ch;
  
/**/

   /* encode the enumerations */
   switch (group) {
	case gc_ScalarSignal:		group_ch = 'S'; break;
	case gc_VectorSignal:		group_ch = 'V'; break;
	case gc_CompositeSignal:	group_ch = 'C'; break;
	default:			group_ch = '?'; break;
   } /* switch */

   switch (class) {
	case tc_Enumeration:	class_ch = 'E'; break;
	case tc_Integer:	class_ch = 'I'; break;
	case tc_Physical:	class_ch = 'P'; break;
	case tc_LargePhysical:	class_ch = 'L'; break;
	case tc_Floating:	class_ch = 'F'; break;
	case tc_Array:		class_ch = 'A'; break;
	case tc_Record:		class_ch = 'R'; break;
	default:		class_ch = '?'; break;
    } /* switch */

    fprintf(OutputFP, "%s %d %d %c %c %d ",
	c_SignalDeclaration,		/* record type */
	start,				/* first scalar signal number */
	length,				/* signal group length */
	group_ch,			/* class of group */
	class_ch,			/* signal type class */
	width				/* approximate maximum width of
					   signal value */
  	);

    WriteString(DefineNull(name), OutputFP); /* signal path name */
    fprintf(OutputFP, " ");
    WriteString(DefineNull(simplename), OutputFP); /* signal simple name */
					
    putc('\n', OutputFP);
    fflush(OutputFP);

} /* RD_WriteSignalDeclaration */


/* One SignalGroupDeclaration and one or more GroupElementDeclarations
 * appear for each signal. (Together they replace Signal Declarations.)
 * To form the simple name of an element, append the <subelement name> of
 * the element to the <simple name> of the element's group.
 * 
 *     SG <signal> <group id> <length> <group class> <hier name> <simple name> 
 *         <signal>            (integer)   Unique identifer of first signal
 *                                         in the group.
 *         <group id>          (integer)   Unique group identifier.
 *         <length>            (integer)   Number of signals in the group.
 *         <group class>       (char)      V:vector S:scalar
 *         <hier name>         (string)    Hierarchical name of the group.
 *         <simple name>       (string)    Simple name, subelement not incl. 
 *
 *     SE <start> <group id> <type class> <value size> <subelement name>
 *         <signal>            (integer)   Unique identifier of signal. 
 *         <group id>          (integer)   Group to which element belongs.
 *         <type class>        (char)      E:enumeration I:intger F:floating
 *                                         P:physical A:array R:record
 *         <value size>        (integer)   Number of characters in string.
 *                                         representing the longest value
 *                                         of this signal's type.
 *         <subelement name>   (string)    Subelement name.
 */

/* *******************************************************************
*
*  RD_WriteSignalGroupDeclaration - 
*
* DESCRIPTION:
*	See comment above.
*
* HISTORY:
*	23-OCT-91 MNP	Created, half of RD_WriteSignalDeclaration
*	 6-NOV-91 DKC	Changed to use element formatter id
*
******************************************************************* */
void RD_WriteSignalGroupDeclaration(name, simple_name, start,
			       group_id, length, group_class)
RD_tString name;		/* in: hier name */
RD_tString simple_name;		/* in: simple name, no subelement info */
tSignalNumber start;		/* in: 1st element's signal number */
tSignalNumber group_id;		/* in: group id */
VK_tSignalLength length;	/* in: how many elements (1 for sclr) */
tGroupClass group_class;	/* in: class of the group, vctr or sclr */
{
    char group_ch;
/**/

  /* encode the enumerations */
    switch (group_class)  {
	case gc_ScalarSignal:		group_ch = 'S'; break;
	case gc_VectorSignal:		group_ch = 'V'; break;
	case gc_CompositeSignal:	group_ch = 'C'; break;
	default:			group_ch = '?'; break;
   } /* switch */

    /* The start field written below used to be 
       qqFormatId(SignalElementHandle(start)), this deref was moved into
       TKIO so TKRecord could be used without TKIO. MNP 911113 */

    fprintf(OutputFP, "%s %d %d %d %c ", c_SignalGroupDeclaration, start,
         group_id, length, group_ch);

    WriteString(DefineNull(name), OutputFP);
    fprintf(OutputFP, " ");
    WriteString(DefineNull(simple_name), OutputFP);
					
    putc('\n', OutputFP);
    fflush(OutputFP);

} /* RD_WriteSignalGroupDeclaration */


/* *******************************************************************
*
*  RD_WriteSignalElementDeclaration - 
*
* DESCRIPTION:
*	See comment above RD_WriteSignalGroupDeclaration.
*
* HISTORY:
*	23-OCT-91 MNP	Created, half of RD_WriteSignalDeclaration
*	 6-NOV-91 DKC	Removed type_class parameter
*
******************************************************************* */
void RD_WriteSignalElementDeclaration(signal, group_id, class,
	width, subel_name) 
tSignalNumber signal;		/* in: sig number */
tSignalNumber group_id;		/* in: what group this belongs to */
tTypeClass class;		/* in: element class */
RD_tInteger width;		/* in: print width */
RD_tString subel_name;		/* in: subelement name */
{
    char class_ch;
/**/
    switch (class) {
      case tc_Enumeration:	class_ch = 'E'; break;
      case tc_Integer:		class_ch = 'I'; break;
      case tc_Physical:		class_ch = 'P'; break;
      case tc_LargePhysical:	class_ch = 'L'; break;
      case tc_Floating:		class_ch = 'F'; break;
      case tc_Array:		class_ch = 'A'; break;
      case tc_Record:		class_ch = 'R'; break;
      default:			class_ch = '?';
    } /* switch */

    fprintf(OutputFP, "%s %d %d %c %d ",
	c_SignalElementDeclaration, signal, group_id, class_ch,	width);

    WriteString(DefineNull(subel_name), OutputFP);
    putc('\n', OutputFP);
    fflush(OutputFP);

} /* RD_WriteSignalElementDeclaration */


/* *******************************************************************
*
*  RD_WriteEndOfSignalMap - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*
******************************************************************* */

void RD_WriteEndOfSignalMap()
{
/**/
    fprintf(OutputFP, "%s \n", 
	c_EndOfSignalMap		/* record type */
	);

    fflush(OutputFP);

} /* WriteEndOfSignalMap */


/* *******************************************************************
*
*  RD_WriteVersion - Write the version record.
*
*  Uses the current TKRecord.h definition of what the version is.
*
* HISTORY:
*	23-SEP-91 MNP	Created
*
******************************************************************* */

void RD_WriteVersion()
{
/**/
    fprintf(OutputFP, "%s %d %d\n", 
	c_Version,		/* record type */
	RD_VERSION_MAJOR,
	RD_VERSION_MINOR
	);

    fflush(OutputFP);

} /* WriteVersion */


/* *******************************************************************
*
*  RD_WriteSignalInitialization - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	04-APR-91 WAG	W0023.  Modified to conform to coding style standards.
*	03-JUN-91 JBR	W0019: changed style of file I/O.
*
******************************************************************* */
void RD_WriteSignalInitialization(signal, value)
tSignalNumber signal;
VK_tCString value;
{
    fprintf(OutputFP, "%s %d ",
	c_SignalInitialization,		/* record type */
	signal);

    WriteString(DefineNull(value), OutputFP);
    putc('\n', OutputFP);
    fflush(OutputFP);

} /* RD_WriteSignalInitialization */


/* *******************************************************************
*
*  RD_WriteSignalTransaction - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	04-APR-91 WAG	W0023.  Modified to conform to coding style standards.
*
******************************************************************* */
void RD_WriteSignalTransaction(signal)
tSignalNumber signal;
{
    fprintf(OutputFP, "%s %d\n", 
	c_SignalTransaction,		/* record type */
	signal
	);
    fflush(OutputFP);
} /* RD_WriteSignalTransaction */



/* *******************************************************************
*
*  RD_WriteSignalEvent - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	04-APR-91 WAG	W0023.  Modified to conform to coding style standards.
*	31-MAY-91 JBR	W0019: changed style of file I/O.
*
******************************************************************* */
void RD_WriteSignalEvent(signal, value)
tSignalNumber signal;
VK_tCString value;
{
    fprintf(OutputFP, "%s %d ",
	c_SignalEvent,		/* record type */
	signal);

    WriteString(DefineNull(value), OutputFP);
    putc('\n', OutputFP);
    fflush(OutputFP);
} /* RD_WriteSignalEvent */



/* *******************************************************************
*
*  RD_WriteAssertion - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*
******************************************************************* */
void RD_WriteAssertion(severity, message, source_file, line_number, proc_name)
VK_tSeverity severity;
char * message;
char * source_file;
int line_number;
char * proc_name;
{
/**/
    fprintf(OutputFP, "%s ", c_Assertion);
    WriteString(DefineNull(source_file), OutputFP);
    fprintf(OutputFP, " %d ", line_number);
    WriteString(DefineNull(proc_name), OutputFP);
    fprintf(OutputFP, " %d ", (int) severity);
    WriteString(DefineNull(message), OutputFP);
    putc('\n', OutputFP);
    fflush(OutputFP);
} /* RD_WriteAssertion */


/* *******************************************************************
*
*  RD_WriteError - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	03-JUN-91 JBR	W0019: changed style of file I/O.
*
******************************************************************* */
void RD_WriteError(message, source_file, line_number)
VK_tCString message;
VK_tCString source_file;
Integer4 line_number;
{
/**/
    fprintf(OutputFP, "%s ", c_Error);
    WriteString(DefineNull(source_file), OutputFP);
    fprintf(OutputFP, " %d ", line_number);
    WriteString(DefineNull(message), OutputFP);
    putc('\n', OutputFP);
    fflush(OutputFP);
} /* RD_WriteError */
												  

/* *******************************************************************
*
*  RD_WriteTimeAdvance - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*	04-APR-91 WAG	W0023.  Modified to conform to coding style standards.
*
******************************************************************* */
void RD_WriteTimeAdvance(time)
tAbsTimePtr time;
{
    tTempBuf80 buffer;
/**/
    UTIME_FORMAT(buffer, *time);
    fprintf(OutputFP, "%s ", c_TimeAdvance);
    WriteString(buffer, OutputFP);
    putc('\n', OutputFP);
    fflush(OutputFP);
} /* RD_WriteTimeAdvance */




/* *******************************************************************
*
*  RD_WriteSimulationCycle - 
*
* HISTORY:
*	28-DEC-90 AWH	Created
*
******************************************************************* */

void RD_WriteSimulationCycle()
{
    fprintf(OutputFP, "%s \n", 
	c_SimulationCycle		/* record type */
	);
    fflush(OutputFP);
} /* RD_WriteSimulationCycle */



/* *******************************************************************
*
* RD_WriteUserPrompt - write prompt record to output
*
* HISTORY:
*       19-DEC-90 AWH   Created
*	03-JUN-91 JBR	W0019: Changed style of I/O.
*
******************************************************************* */
void RD_WriteUserPrompt(prompt)
VK_tCString prompt;
{
/**/
    if (SlaveMode) {
    	fprintf(OutputFP, "%s ", c_UserPrompt);
	WriteString(DefineNull(prompt), OutputFP);
	putc('\n', OutputFP);
	fflush(OutputFP);
    }
} /* RD_WriteUserPrompt */



/* *******************************************************************
*
* RD_WriteBreakPoint - write break point to output
*
* HISTORY:
*       19-DEC-90 AWH   Created
*	03-JUN-91 JBR	W0019:  Changed style of file I/O.
*
******************************************************************* */
void RD_WriteBreakPoint(class, library, primary, secondary, 
			process, file, line)
tBreakPointClass class;
VK_tCString library;
VK_tCString primary;
VK_tCString secondary;
VK_tCString process;
VK_tCString file;
Integer4 line;
{
/**/
     if (SlaveMode) {
	fprintf(OutputFP, "%s %d ", c_BreakPoint, class);
	WriteString(DefineNull(library), OutputFP);
	putc(' ', OutputFP);			/* library name */
	WriteString(DefineNull(primary), OutputFP);
	putc(' ', OutputFP);			/* primary unit name */
	WriteString(DefineNull(secondary), OutputFP);
	putc(' ', OutputFP);			/* secondary unit name */
	WriteString(DefineNull(process), OutputFP);
	putc(' ', OutputFP);			/* process name */
	WriteString(DefineNull(file), OutputFP);/* source file name */
	fprintf(OutputFP, " %d \n", line);	/* line number */
    }
    fflush(OutputFP);
} /* RD_WriteBreakPoint */




 
/* *******************************************************************
*
* RD_WriteTextLine - write text line to output
*
* HISTORY:
*       19-DEC-90 AWH   Created
*	03-JUN-91 JBR	W0019.  Changed style of file I/O.
*
******************************************************************* */
void RD_WriteTextLine(text_line)
VK_tCString text_line;
{
 /**/
     if (SlaveMode) {
	fprintf(OutputFP, "%s ", c_TextLine);
	WriteString(DefineNull(text_line), OutputFP);
	putc('\n', OutputFP);
     }
    fflush(OutputFP);
 } /* RD_WriteTextLine */




/* *******************************************************************
*
* RD_WriteStatus - write status record 
*
* HISTORY:
*	20-DEC-90 AWH	Created
*
******************************************************************* */
void RD_WriteStatus(status)				
tSimulationStatus status;		/* status */
{
/**/
    if (NULL != OutputFP) {
	fprintf(OutputFP, "%s %d\n", c_Status, status);
    }
    fflush(OutputFP);
} /* RD_WriteStatus */




/* *******************************************************************
*
* RD_WriteCommandResponse - write command response record 
*
* HISTORY:
*	19-JAN-91 AWH	Created
*	03-JUN-91 JBR	W0019. Change style of file I/O.
*
******************************************************************* */
void RD_WriteCommandResponse(message)				
RD_tString message;		/* text */
{
/**/
    if (NULL != OutputFP) {
	fprintf(OutputFP, "%s ", c_CommandResponse);
	WriteString(DefineNull(message), OutputFP);
	putc('\n', OutputFP);
    }
} /* RD_WriteCommandResponse */




/* *******************************************************************
*
* RD_WriteTerminated - write terminated record 
*
* HISTORY:
*	20-DEC-90 AWH	Created
*
******************************************************************* */
void RD_WriteTerminated(status)				
RD_tInteger status;			/* status descrition */
{
/**/
    if (NULL != OutputFP) {
	fprintf(OutputFP, "%s %d\n", c_Terminated, status);
    }
    fflush(OutputFP);
} /* RD_WriteTerminated */




/* *******************************************************************
*
* RD_WriteRecord - write record of any type
*
* HISTORY:
*	02-JAN-91 AWH	Created
*	03-JUN-91 JBR	W0019. Removed case for RD_TypeDeclaration.
*	13-NOV-91 MNP	Added: SignalGroupDeclaration, 
*			SignalElementDeclaration.
*			Removed SignalDeclaration.
*
******************************************************************* */

void RD_WriteRecord(record)				
RD_tRecord * record;			/* status descrition */
{
/**/
    switch(record->type) {
        case RD_Version:
		/* This integrity check might not be needed since it's done
		   in RD_ReadNextRecord also. */
	    if (record->version.first != RD_VERSION_MAJOR ||
		record->version.second != RD_VERSION_MINOR) {
		fprintf(stderr,
		"Warning: Internal version mismatch in RD_WriteVersion()\n");
	    }
	    RD_WriteVersion();
	    break;
	case RD_SignalGroupDeclaration:
	    RD_WriteSignalGroupDeclaration (
		record->signal_group_declaration.name,
		record->signal_group_declaration.simple_name,
		record->signal_group_declaration.start,
		record->signal_group_declaration.group_id,
		record->signal_group_declaration.length,
		record->signal_group_declaration.group
		);
	    break;
	case RD_SignalElementDeclaration:
	    RD_WriteSignalElementDeclaration (
		record->signal_element_declaration.start,
		record->signal_element_declaration.group_id,
		record->signal_element_declaration.class,
		record->signal_element_declaration.width,
		record->signal_element_declaration.subel_name
		);
	    break;
	case RD_EndOfSignalMap:
	    RD_WriteEndOfSignalMap();
	    break;
	case RD_SignalInitialization:
	    RD_WriteSignalInitialization(
		record->signal_initialization.signal,
		record->signal_initialization.value
		);
	    break;
	case RD_SignalTransaction:
	    RD_WriteSignalTransaction(
		record->signal_transaction.signal
		);
	    break;
	case RD_SignalEvent:
	    RD_WriteSignalEvent(
		record->signal_event.signal,
		record->signal_event.value
		);
	    break;
	  case RD_EnumerationType:
	    RD_WriteEnumerationType(
		record->enumeration_type.name,
		record->enumeration_type.cardinality,
		record->enumeration_type.literals,
		&record->enumeration_type.number
		);
	    break;
	case RD_IntegerType:
	   RD_WriteIntegerType( 
		record->integer_type.name,
		record->integer_type.left,
		record->integer_type.dir,
		record->integer_type.right,
		&record->integer_type.number
		);
	    break;
	  case RD_FloatingType:
	    RD_WriteFloatingType(
		record->floating_type.name,
		record->floating_type.left,
		record->floating_type.dir,
		record->floating_type.right,
		&record->floating_type.number
		);
	    break;
	case RD_PhysicalType:
	    RD_WritePhysicalType(
		record->physical_type.name,
		record->physical_type.left,
		record->physical_type.dir,
		record->physical_type.right,
		record->physical_type.cardinality,
		record->physical_type.literals,
		record->physical_type.positions,
		&record->physical_type.number
	    );
	    break;
	case RD_LargePhysicalType:
	    RD_WriteLargePhysicalType(
		record->large_physical_type.name,
		record->large_physical_type.left,
		record->large_physical_type.dir,
		record->large_physical_type.right,
		record->large_physical_type.cardinality,
		record->large_physical_type.literals,
		record->large_physical_type.positions,
		&record->large_physical_type.number
	    );
	    break;
	case RD_Assertion:
	    RD_WriteAssertion(
		record->assertion.severity,
		record->assertion.message,
		record->assertion.source_file,
		record->assertion.line_number,
		record->assertion.process_name
		);
	    break;
	case RD_Error:
	    RD_WriteError(
		record->error.message,
		record->error.source_file,
		record->error.line_number
		);
	    break;
	case RD_TimeAdvance:
	    RD_WriteTimeAdvance(
		&record->time_advance.time
		);
	    break;
	case RD_SimulationCycle:
	    RD_WriteSimulationCycle();
	    break;
	case RD_UserPrompt:
	    RD_WriteUserPrompt(
		record->user_prompt.text
		);
	    break;
	case RD_BreakPoint:
	    RD_WriteBreakPoint(
	        record->break_point.bp_class,
		record->break_point.library,
		record->break_point.primary,
		record->break_point.secondary,
		record->break_point.process,
	        record->break_point.source_file,
	        record->break_point.line_number
	    );
	    break;
	case RD_Terminated:
	    RD_WriteTerminated(
	        record->terminated.reason
	    );
	    break;
	case RD_TextLine:
	    RD_WriteTextLine(
	        record->text_line.text
	    );
	    break;
	case RD_Status:
	    RD_WriteStatus(
		record->status.status
	    );
	    break;
	case RD_CommandResponse:
	    RD_WriteCommandResponse(
		record->command_response.text
	    );
	    break;
	default:
	    fprintf(stderr, "Bad switch in RD_WriteRecord()\n");
	    break;
    } /* switch */
} /* RD_WriteRecord */





#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */


<static> Fntype RtnName(parm)				/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif

