#ifndef TKRecord_H
#define TKRecord_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKRecord.h
*
* DESCRIPTION:  
*	Interface file for TKRecord.c.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	20-DEC-90 AWH	Created
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	03-JUN-91 JBR	W0019: Added RD_Version, RD_TimeUnit, RD_*Type,
*			RD_Subtype and associated records and procedures.
*			Removed RD_TypeDeclaration.
*	07-JUN-91 JBR	Removed sc fields from structures.
*	23-SEP-91 MNP	Added Version record support. TKRecord.c has details.
*	11-OCT-91 MNP	Header cleanup.
*	23-OCT-91 MNP	added simplname field for signal decls, bumped version.
* 	29-OCT-91 MNP	Added SignalGroupDeclaration, SignalElementDeclaration.
*			The SignalDeclaration record is now obsolete. New ver.
*	13-NOV-91 MNP	Proto maint
*
* SCCS ID: @(#)TKRecord.h	42.1 1/10/92
*
******************************************************************* */
#include "VKI.h"	/* for: interface declaration VK_<...> */
#include "TKStructs.h"	/* for: tGroupClass */
#include "TKDebug.h"	/* for: tBreakPointClass */
#include "TKSim.h"	/* for: tSimulationStatus */


/* The version of the trace file spoken by this code. To be
   incremented manually when appropriate. */

#define RD_VERSION_MAJOR	0	
#define RD_VERSION_MINOR	3

/* If a trace file is opened and it has no version, or an unreadable
   one, this is returned as the version in an RD_VersionError record. */

#define RD_OLD_VERSION_MAJOR	0
#define RD_OLD_VERSION_MINOR	0

/* Supported Field Types */
typedef char * 		RD_tString;
typedef long int  	RD_tInteger;
typedef char		RD_tCharacter;
typedef int		RD_tEnumeration;		/* special handling */
typedef tAbsTimeBuf	RD_tTime;

/* Record Types */
typedef enum {
   RD_UnknownType,
   RD_Version,
   RD_VersionError,
   RD_TimeUnit,
   RD_IntegerType,
   RD_FloatingType,
   RD_EnumerationType,
   RD_AccessType,		/* ?? there is no need for access types ?? */
   RD_RecordType,
   RD_PhysicalType,
   RD_LargePhysicalType,
   RD_ArrayType,
   RD_Subtype,
   RD_FloatingSubtype,
   RD_SignalDeclaration,
   RD_SignalGroupDeclaration,
   RD_SignalElementDeclaration,
   RD_EndOfSignalMap,
   RD_SignalInitialization,
   RD_SignalTransaction,
   RD_SignalEvent,
   RD_Assertion,
   RD_Error,
   RD_TimeAdvance,
   RD_SimulationCycle,
   RD_UserPrompt,
   RD_BreakPoint,
   RD_Terminated,
   RD_TextLine,
   RD_Status,
   RD_CommandResponse
    } RD_tRecordKinds;

/*
* NOTES:
*	The type member in each structure in the union and as 
*	the first member of the union. This redundancy is intended
*	and is provided for easy coding of the ReadNextRecord loop
*
*/

/* Type Declaration */
typedef struct RD_sUnknownTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			text;	/* text line from unknown record */
   } RD_tUnknownTypeRecord;

/* Version */
typedef struct RD_sVersionRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tInteger			first;	/* major version number */
   RD_tInteger			second;	/* minor version number */
   } RD_tVersionRecord;
/* Note: major and minor conflict with system files */

/* Version Error */
typedef struct RD_sVersionErrorRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tInteger			first;	/* major version number */
   RD_tInteger			second;	/* minor version number */
   } RD_tVersionErrorRecord;

/* Time Unit */
typedef struct RD_sTimeUnitRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tTime			unit;	/* time unit (was RD_tString - MNP) */
   } RD_tTimeUnitRecord;

/* Integer Type */
typedef struct RD_sIntegerTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   VK_tScalarData		left;	/* left bound */
   VK_tDirection		dir;	/* direction */
   VK_tScalarData		right;	/* right bound */
   } RD_tIntegerTypeRecord;

/* Floating Type */
typedef struct RD_sFloatingTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   VK_tFloatingData		left;	/* left bound */
   VK_tDirection		dir;	/* direction */
   VK_tFloatingData		right;	/* right bound */
   } RD_tFloatingTypeRecord;

/* Enumeration Type */
typedef struct RD_sEnumerationTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   VK_tListLength		cardinality;	/* number of literals */
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h,
	and VK_RegisterEnumerationTypeDecl.cardinality ?? */
   VK_tName			* literals;	/* literals string table */
   } RD_tEnumerationTypeRecord;

/* Access Type */
typedef struct RD_sAccessTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   RD_tInteger			accessed; /* type accessed */
   } RD_tAccessTypeRecord;

/* Record Type */
typedef struct RD_sRecordTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   RD_tInteger			elem_count;	/* number of elements */
   VK_tName			* elements;	/* element name table */
   RD_tInteger			* elem_type;	/* element type table */
   } RD_tRecordTypeRecord;

/* Physical Type */
typedef struct RD_sPhysicalTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   VK_tScalarData		left;	/* left bound */
   VK_tDirection		dir;	/* direction */
   VK_tScalarData		right;	/* right bound */
   RD_tInteger			cardinality;	/* number of literals */
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h,
	and VK_RegisterEnumerationTypeDecl.cardinality ?? */
   VK_tName			* literals;	/* unit name table */
   RD_tInteger			* positions;	/* unit positions table */
   } RD_tPhysicalTypeRecord;

/* Large Physical Type */
typedef struct RD_sLargePhysicalTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   VK_tLargeData		left;	/* left bound */
   VK_tDirection		dir;	/* direction */
   VK_tLargeData		right;	/* right bound */
   RD_tInteger			cardinality;	/* number of literals */
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h,
	and VK_RegisterEnumerationTypeDecl.cardinality ?? */
   VK_tName			* literals;	/* unit name table */
   int				* positions;	/* unit positions table */
   } RD_tLargePhysicalTypeRecord;

/* Array Type */
typedef struct RD_sArrayTypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type number */
   RD_tInteger			dimensions;	/* number of dimensions */
   RD_tInteger			* index;	/* index types */
   } RD_tArrayTypeRecord;

/* Subtype */
typedef struct RD_sSubtypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type/subtype number */
   RD_tInteger			of_type;/* subtype on which this is based */
   VK_tScalarData		* left;	/* left bound(s) */
   VK_tDirection		* dir;	/* direction(s) */
   VK_tScalarData		* right;/* right bound(s) */
   } RD_tSubtypeRecord;

/* Floating Subtype */
typedef struct RD_sFloatingSubtypeRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			name;	/* type name */
   RD_tInteger			number; /* type/subtype number */
   RD_tInteger			of_type;/* subtype on which this is based */
   VK_tFloatingData		left;	/* left bound */
   VK_tDirection		dir;	/* direction */
   VK_tFloatingData		right;	/* right bound */
   } RD_tFloatingSubtypeRecord;

/* Signal Declaration */
/*
 * Now obsoleted by the two below!! 
 */
typedef struct RD_sSignalDeclarationRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   VK_tCString			name;	/* signal name, hier */
   VK_tCString			simplename; /* signal name, simple */
   tTypeClass			class;	/* signal type class */
   tSignalNumber		start;  /* first scalar signal number */
   VK_tSignalLength		length;	/* length of signal */
   tGroupClass			group;	/* signal grouping */
   RD_tInteger			width;	/* widest value */
   } RD_tSignalDeclarationRecord;

/* Signal Group Declaration */
typedef struct RD_sSignalGroupDeclarationRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   tSignalNumber		start;  /* first scalar signal number */
   tSignalNumber		group_id; /* unique group identity */
   VK_tSignalLength		length;	/* length of signal */
   tGroupClass			group;	/* signal grouping */
   VK_tCString			name;	/* hier name */
   VK_tCString			simple_name; /* simplename, no subelements */
   } RD_tSignalGroupDeclarationRecord;

/* Signal Element Declaration */
typedef struct RD_sSignalElementDeclarationRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   tSignalNumber		start;  /* first scalar signal number */
   tSignalNumber		group_id; /* unique group identity */
   tTypeClass			class;	/* signal type class */
   RD_tInteger			width;	/* widest value */
   VK_tCString			subel_name; /* name of this element */
   } RD_tSignalElementDeclarationRecord;

/* End of Signal Map */
typedef struct RD_sEndOfSignalMapRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   } RD_tEndOfSignalMapRecord;

/* Signal Initialization */
typedef struct RD_sSignalInitializationRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			value;  /* initial value */
   RD_tInteger			signal; /* scalar signal number */
   } RD_tSignalInitializationRecord;

/* Signal Transaction */
typedef struct RD_sSignalTransactionRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tInteger			signal; /* scalar signal number */
   } RD_tSignalTransactionRecord;

/* Signal Event */
typedef struct RD_sSignalEventRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			value;	/* new signal value */
   RD_tInteger			signal; /* scalar signal number */
   } RD_tSignalEventRecord;

/* Assertion */
typedef struct RD_sAssertionRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			message;
   RD_tString			process_name;
   RD_tString			source_file;
   RD_tInteger			line_number;
   RD_tEnumeration		severity;
   } RD_tAssertionRecord;

/* Error */
typedef struct RD_sErrorRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			message;
   RD_tString			extra;
   RD_tString			source_file;
   RD_tInteger			line_number;
   } RD_tErrorRecord;

/* Time Advance */
typedef struct RD_sTimeAdvanceRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tTime			time;	/* simulation time */
   } RD_tTimeAdvanceRecord;

/* Simulation Cycle */
typedef struct RD_sSimulationCycleRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   } RD_tSimulationCycleRecord;

/* User Prompt */
typedef struct RD_sUserPromptRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			text;   /* prompt for the user */
   } RD_tUserPromptRecord;

/* Text Line */
typedef struct RD_sTextLineRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			text;	/* message for the user */
   } RD_tTextLineRecord;

/* Break Point */
typedef struct RD_sBreakPointRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			library;	/* design library */
   RD_tString			primary;	/* design unit */
   RD_tString			secondary;	/* design unit */
   RD_tString			process;	/* process name */
   RD_tString			source_file;	
   RD_tInteger			line_number;	
   tBreakPointClass		bp_class;	/* class of break point */
   } RD_tBreakPointRecord;

/* Terminated */
typedef struct RD_sTerminatedRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tEnumeration		reason; /* terminate reason */
   } RD_tTerminatedRecord;

/* Status */
typedef struct RD_sStatusRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   tSimulationStatus		status; /* simulation status */
   } RD_tStatusRecord;

/* CommandResponse */
typedef struct RD_sCommandResponseRecord {
   RD_tRecordKinds		type;	/* first element of every record */
   RD_tString			text; 	/* message for the user */
   } RD_tCommandResponseRecord;


/* Record Structure */
typedef union RD_sRecord {
    RD_tRecordKinds 			type;
    RD_tUnknownTypeRecord	 	unknown_type;
    RD_tVersionRecord			version;
    RD_tVersionErrorRecord		version_error;
    RD_tTimeUnitRecord			time_unit;
    RD_tIntegerTypeRecord		integer_type;
    RD_tFloatingTypeRecord		floating_type;
    RD_tEnumerationTypeRecord		enumeration_type;
    RD_tAccessTypeRecord		access_type;
    RD_tRecordTypeRecord		record_type;
    RD_tPhysicalTypeRecord		physical_type;
    RD_tLargePhysicalTypeRecord		large_physical_type;
    RD_tArrayTypeRecord			array_type;
    RD_tSubtypeRecord			subtype;
    RD_tFloatingSubtypeRecord		floating_subtype;
    RD_tSignalDeclarationRecord 	signal_declaration;
    RD_tSignalGroupDeclarationRecord	signal_group_declaration;
    RD_tSignalElementDeclarationRecord	signal_element_declaration;
    RD_tEndOfSignalMapRecord		end_of_signal_map;
    RD_tSignalInitializationRecord	signal_initialization;
    RD_tSignalTransactionRecord		signal_transaction;
    RD_tSignalEventRecord		signal_event;
    RD_tAssertionRecord			assertion;
    RD_tErrorRecord			error;
    RD_tTimeAdvanceRecord		time_advance;
    RD_tSimulationCycleRecord		simulate_cycle;
    RD_tUserPromptRecord		user_prompt;
    RD_tBreakPointRecord		break_point;
    RD_tTerminatedRecord 		terminated;
    RD_tTextLineRecord			text_line;
    RD_tStatusRecord			status;
    RD_tCommandResponseRecord		command_response;
    } RD_tRecord;



/* *******************************************************************
*
*  RD_FreeRecord - 
*
******************************************************************* */

void RD_FreeRecord(
#ifdef ANSI_FN_PROTOS
	RD_tRecord * record
#endif
);

/* *******************************************************************
*
*  RD_ReadNextRecord - 
*
******************************************************************* */

VK_tBoolean RD_ReadNextRecord(
#ifdef ANSI_FN_PROTOS
	RD_tRecord * record
#endif
);


/* *******************************************************************
*
*  RD_SetInput - 
*
******************************************************************* */

void RD_SetInput(
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);

/* *******************************************************************
*
*  RD_SetVirtualFileMode -
*
******************************************************************* */

void RD_SetVirtualFileMode(
#ifdef ANSI_FN_PROTOS
	Boolean1 slave
#endif
);


/* *******************************************************************
*
*  RD_SetOutput - 
*
******************************************************************* */

void RD_SetOutput(
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteVersion - 
*
******************************************************************* */

void RD_WriteVersion (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteTimeUnit - 
*
******************************************************************* */

void RD_WriteTimeUnit (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteIntegerType - 
*
******************************************************************* */

void RD_WriteIntegerType (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteFloatingType - 
*
******************************************************************* */

void RD_WriteFloatingType (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteEnumerationType - 
*
******************************************************************* */

void RD_WriteEnumerationType (
#ifdef ANSI_FN_PROTOS
	VK_tName name,
	VK_tListLength cardinality,
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h,
	and VK_RegisterEnumerationTypeDecl.cardinality ?? */
	VK_tName * literals,
	int *trace_num
#endif
);


/* *******************************************************************
*
*  RD_WriteAccessType - 
*
******************************************************************* */

void RD_WriteAccessType (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteRecordType - 
*
******************************************************************* */

void RD_WriteRecordType (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WritePhysicalType - 
*
******************************************************************* */

void RD_WritePhysicalType (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WritePhysicalType - 
*
******************************************************************* */

void RD_WritePhysicalType (
#ifdef ANSI_FN_PROTOS
	VK_tName name,
	VK_tLargeAddr left,
	VK_tDirection dir,
	VK_tLargeAddr right,
	VK_tListLength unit_count,
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h,
	and VK_RegisterEnumerationTypeDecl.cardinality ?? */
	VK_tName * units,
	VK_tIntegerArray positions,
	int *trace_num
#endif
	);


/* *******************************************************************
*
*  RD_WriteArrayType - 
*
******************************************************************* */

void RD_WriteArrayType (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteSubtype - 
*
******************************************************************* */

void RD_WriteSubtype (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteFloatingSubtype - 
*
******************************************************************* */

void RD_WriteFloatingSubtype (
#ifdef ANSI_FN_PROTOS
#endif
);


/* *******************************************************************
*
*  RD_WriteSignalDeclaration - 	OBSOLETE
*
******************************************************************* */

void RD_WriteSignalDeclaration (
#ifdef ANSI_FN_PROTOS
	RD_tString name,
	RD_tString simplename,
	tTypeClass class,
	tSignalNumber start,
	VK_tSignalLength length,
	tGroupClass group,
	RD_tInteger width
#endif
	);


/* *******************************************************************
*
*  RD_WriteSignalGroupDeclaration -
*
******************************************************************* */

void RD_WriteSignalGroupDeclaration (
#ifdef ANSI_FN_PROTOS
	RD_tString name,
	RD_tString simple_name,
	tSignalNumber start,
	tSignalNumber group_id,
	VK_tSignalLength length
tGroupClass group_class;	/* in: class of the group, vctr or sclr */
#endif
	);


/* *******************************************************************
*
*  RD_WriteSignalElementDeclaration -
*
******************************************************************* */

void RD_WriteSignalElementDeclaration (
#ifdef ANSI_FN_PROTOS
	tSignalNumber signal,
	tSignalNumber group_id,
	tTypeClass class,
	RD_tInteger width,
	RD_tString subel_name
#endif
	);


/* *******************************************************************
*
*  RD_WriteEndoOfSignalMap - 
*
******************************************************************* */

void RD_WriteEndOfSignalMap (
#ifdef ANSI_FN_PROTOS
#endif
	);


/* *******************************************************************
*
*  RD_WriteSignalInitialization - 
*
******************************************************************* */

void RD_WriteSignalInitialization (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteSignalTransaction - 
*
******************************************************************* */

void RD_WriteSignalTransaction (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteSignalEvent - 
*
******************************************************************* */

void RD_WriteSignalEvent (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteAssertion - 
*
******************************************************************* */

void RD_WriteAssertion (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteError - 
*
******************************************************************* */

void RD_WriteError (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteTimeAdvance - 
*
******************************************************************* */

void RD_WriteTimeAdvance (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteSimulationCycle - 
*
******************************************************************* */

void RD_WriteSimulationCycle (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteUserPrompt - 
*
******************************************************************* */

void RD_WriteUserPrompt (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteBreakPoint - 
*
******************************************************************* */

void RD_WriteBreakPoint (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteTerminated - 
*
******************************************************************* */

void RD_WriteTerminated (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteTextLine - 
*
******************************************************************* */

void RD_WriteTextLine (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteStatus - 
*
******************************************************************* */

void RD_WriteStatus (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);


/* *******************************************************************
*
*  RD_WriteCommandResponse - 
*
******************************************************************* */

void RD_WriteCommandResponse (
#ifdef ANSI_FN_PROTOS
	FILE * fp
#endif
);

/* *******************************************************************
*
*  RD_WriteLargePhysicalType
*
******************************************************************* */

void RD_WriteLargePhysicalType (
#ifdef ANSI_FN_PROTOS
	VK_tName name,
	VK_tLargeAddr left,
	VK_tDirection dir,
	VK_tLargeAddr right,
	VK_tListLength unit_count,
/* ?? this should be VK_tArraySize, not VK_tListLength; see VKBasicTypes.h,
	and VK_RegisterEnumerationTypeDecl.cardinality ?? */
	VK_tName * units,
	VK_tIntegerArray positions,
	int *trace_num
#endif
);


#endif /* TKRecord_H */



