static char SCCS_ID[] = "@(#)TKRegions.c	42.1 1/10/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKRegions.c
*
* DESCRIPTION:  
*	Private interface file for TKRegions.c.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Split from VKI.c
*	19-APR-90 DSB	Added VK_CreatePackageRegion
*	02-JAN-91 DSB	Added region routines for generate statements
*	02-APR-91 WAG	W0015.  Included TKMisc.h.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 JBR	W0046.  Added CreateRegion.  Removed VK_ from most
*			if not all routine names.
*	17-MAY-91 DSB	Added CreatePackageBody
*	21-MAY-91 DSB	Renamed VK_tRegionHandle to TK_tRegionHandle
*
* SCCS ID: @(#)TKRegions.c	42.1 1/10/92
*
******************************************************************* */

#include "TKCommon.h"
#include "TKStructs.h"		/* for tRegionKind, etc. */
#include "TKGlobals.h"		/* for RegionList */
#include "TKTemp.h"		/* for AllocateMemory */

#include "TKListFns.h"		/* for TempAddLast, ... */

#include <string.h>		/* for ?? */


TK_tRegionHandle KernelRegionHandle;	/* region holding internal processes
					   implementing VHDL kernel process */
/* ?? This should be in TKGlobals (unless we distributed globals). */



/* *******************************************************************
*
* CreateRegionSub - create base region object and initialize standard fields
* 
* HISTORY:
*	02-JAN-91 DSB	Split out of CreateRegion (for generate stmts.)
*	02-APR-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*	21-MAY-91 DSB	Renamed VK_tRegionHandle to TK_tRegionHandle
*
******************************************************************* */

TK_tRegionHandle CreateRegionSub(parent_region, region_kind, region_name)
TK_tRegionHandle parent_region;	/* parent region */
tRegionKind region_kind;	/* region type */
VK_tName region_name;	/* identifier part of name */
{
    TK_tRegionHandle region;
/**/
    AllocateMemory(region, tRegionRec, 1);

    AddqqRecordKind(region, RegionRecord);

    TempAddLast(RegionList, (TempNode) region);	/* add to global list */

    AddqqName(region, region_name);

    SetqqRegionKind(region, region_kind);
    AddqqSecName(region, "");

    SetqqParentRegion(region, parent_region);
    if (! (NULL == parent_region)) {
    	TempAddLast(qqChildren(parent_region), (TempNode) region);
    }
    AddqqChildren(region, TempNewList());

    return region;
} /* CreateRegionSub */


/* *******************************************************************
*
* InitKernelRegion -
*
* HISTORY:
* 	07-Sep-90 RM	Moved from TKMain module.
*
******************************************************************* */

void InitKernelRegion()
{
/**/
    KernelRegionHandle = CreateRegionSub(NULL, KernelRegion, "_Kernel");
} /* InitKernelRegion */


/* *******************************************************************
*
* CreatePackage - create region handle for package
*
*	17-MAY-91 JBR	W0046.  Removed VK_ from name.
*
******************************************************************* */

TK_tRegionHandle CreatePackage(parent_region, package_name)
TK_tRegionHandle parent_region;	/* parent hierarchical item - NULL? */
VK_tName package_name;		/* package name identifier */
{
/**/
    return CreateRegionSub(parent_region, PackageRegion, package_name);
} /* CreatePackage */


/* *******************************************************************
*
* CreatePackageBody - create region handle for package body
*
* HISTORY:
*	17-MAY-91 DSB	Created
*
******************************************************************* */

TK_tRegionHandle CreatePackageBody(extended_region)
TK_tRegionHandle extended_region;
				/* package declaration's region handle */
{
/**/
    return CreateRegionSub(extended_region, PackageBodyRegion, "");
} /* CreatePackageBody */


/* *******************************************************************
*
* CreateDesignEntity - create region handle for design entity
*
* HISTORY:
*	23-FEB-90 DSB	Created
*	02-JAN-90 DSB	Reworking
*	17-MAY-91 JBR	W0046.  Removed VK_ from name.
*
******************************************************************* */

TK_tRegionHandle CreateDesignEntity(parent_region, entity_name,
	architecture_name)
TK_tRegionHandle parent_region;	/* parent hierarchical item -
					   component instantiation */
VK_tName entity_name;		/* entity declaration identifier */
VK_tName architecture_name;	/* architecture body identifier */
{
    TK_tRegionHandle region;
/**/
#ifdef REMOVE_WHEN_VERIFIED
	/* : take entity name of top-level design entity as trace file name */
    if (parent_region == NULL) {	/* region with no parent should be 
						top-level design entity */
	strcpy(FileNamePart, entity_name);
	OpenTraceFile();		/* can open trace file at this point */
    }
#endif

    region = CreateRegionSub(parent_region, DesignEntityRegion, 
		entity_name);
    AddqqSecName(region, architecture_name);

    return region;
} /* CreateDesignEntity */


/* *******************************************************************
*
* CreateBlock - create region handle for block statement region
*
* HISTORY:
*	17-MAY-91 JBR	W0046.  Removed VK_ from name.
*
******************************************************************* */

TK_tRegionHandle CreateBlock(parent_region, label_name)
TK_tRegionHandle parent_region;	/* parent hierarchical item -
					   component instantiation */
VK_tName label_name;		/* block label identifier */
{
/**/
    return CreateRegionSub(parent_region, BlockRegion, label_name);
} /* CreateBlock */


/* *******************************************************************
*
* CreateIfGenerate - create region handle for if generate statement
*
* HISTORY:
*	02-JAN-91 DSB	Created
*	17-MAY-91 JBR	W0046.  Removed VK_ from name.
*
******************************************************************* */

TK_tRegionHandle CreateIfGenerate(parent_region, label_name, 
	condition_value)
TK_tRegionHandle parent_region;	/* parent hierarchical item -
					   component instantiation */
VK_tName label_name;		/* generate statement label identifier */
VK_tBoolean condition_value;	/* value of expression */
{
    TK_tRegionHandle region;
/**/
    region = CreateRegionSub(parent_region, BlockRegion, label_name);

    region->if_value = condition_value;

    return region;
} /* CreateIfGenerate */


/* *******************************************************************
*
* CreateForGenerateGroup - create group region handle for for generate
*	statement
*
* HISTORY:
*	02-JAN-91 DSB	Created
*	04-APR-91 WAG	Now assign to TK_tRegionHandle fields via the
*			appropriate qqX macro instead of directly.
*	17-MAY-91 JBR	W0046.  Removed VK_ from name.
*
******************************************************************* */

TK_tRegionHandle CreateForGenerateGroup(parent_region, label_name, 
	instance_count, range_left, range_dir, range_right, type_class, 
	type_symb_info)
TK_tRegionHandle parent_region;	/* parent hierarchical item -
				   architecture, block, or generate */
VK_tName label_name;		/* label name identifier */
int instance_count;		/* simple left of for range */
VK_tScalarData range_left;	/* left bound of for range */
VK_tDirection range_dir;	/* direction of for range */
VK_tScalarData range_right;	/* right bound of for range */
tTypeClass type_class;	/* type class of for parameter */
VK_tSymbInfoHandle type_symb_info;	/* parameter enumeration type info. */
{
    TK_tRegionHandle region;
/**/
    region = CreateRegionSub(parent_region, ForGenerateGroupRegion, 
	label_name);
    AddqqForCount(region, instance_count);
    AddqqForRangeLeft(region, range_left);
    AddqqForRangeDir(region, range_dir);
    AddqqForRangeRight(region, range_right);
    AddqqForTypeClass(region, type_class);
    AddqqForTypeSymbInfo(region, type_symb_info);
    return region;
} /* CreateForGenerateGroup */


/* *******************************************************************
*
* CreateForGenerateBlock - create region handle for generated block
*
* HISTORY:
*	02-JAN-91 DSB	Created
*	17-MAY-91 JBR	W0046.  Removed VK_ from name.
*
******************************************************************* */
TK_tRegionHandle CreateForGenerateBlock(group_region, position_value,
	index_value)
TK_tRegionHandle group_region;	/* block's generate group */
int position_value;		/* relative index value (0 based) */
VK_tScalarData index_value;	/* VHDL index value */
{
    TK_tRegionHandle region;
/**/
    region = CreateRegionSub(qqParentRegion(group_region), 
	ForGenerateRegion, qqName(group_region));
    region->for_index_value = index_value;
    region->for_group_data = group_region;
    
    return region;
} /* CreateForGenerateBlock */


/* *******************************************************************
*
* CreateComponentInstantiation - create region handle for component 
*	instantiation
*
* HISTORY:
*	17-MAY-91 JBR	W0046.  Removed VK_ from name.
*
******************************************************************* */

TK_tRegionHandle CreateComponentInstantiation(parent_region, label_name)
TK_tRegionHandle parent_region;	/* parent hierarchical item -
				   architecture, block, or generate */
VK_tName label_name;		/* component instantiation label identifier */
{
/**/
    return CreateRegionSub(parent_region, ComponentInstantiationRegion, 
	label_name);
} /* CreateComponentInstantiation */


/* *******************************************************************
*
* CreateRegion - generic region builder
*
* HISTORY:
*	15-MAY-91 JBR	Created to build process's region.
*
******************************************************************* */

TK_tRegionHandle CreateRegion(parent_region, region_kind, region_name)
TK_tRegionHandle parent_region;	/* parent hierarchical item -
				   architecture, block, or generate */
tRegionKind region_kind;
VK_tName region_name;
{
/**/
    return CreateRegionSub(parent_region, region_kind, region_name);
} /* CreateRegion */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */


<static> Fntype RtnName(parm)	/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif


