#ifndef TKRegions_H
#define TKRegions_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKRegions.h
*
* DESCRIPTION:  
*	Private interface file for TKRegions.c.  <then describe contents>
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Created
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	17-MAY-91 JBR	W0041. Added Create<Region> functions.
*	17-MAY-91 DSB	Added PackageBodyRegion
*	17-MAY-91 DSB	Removed unused VK_tRegionIndex
*	21-MAY-91 DSB	Cleaned up old types; renamed to remove some VK_
*	16-OCT-91 MNP	Added protos.
*
* SCCS ID: @(#)TKRegions.h	42.1 1/10/92
*
******************************************************************* */

#include "VKTrace.h"
#include "TKStructs.h"		/* for tTypeClass, etc. */


extern TK_tRegionHandle KernelRegionHandle;
					/* region holding internal processes
					   implementing VHDL kernel process */

/* *******************************************************************
*
* ...
*
* HISTORY:
*	23-JUL-91 DSB	Removed "VK_ prefix"
*
******************************************************************* */

/* ?? To follow kernel enumeration type precedents, PackageRegion, etc., 
	should probably be renamed to rk_Package ?? */

typedef enum { 
	KernelRegion,			/* to represent kernel process, etc. */
	PackageRegion, 			/* package declaration region */
	PackageBodyRegion, 		/* package body region */
	DesignEntityRegion,		/* combined design entity region */
	BlockRegion, 			/* block statement region */
	IfGenerateRegion, 		/* if generate stmt. equiv. block */
	ForGenerateRegion, 		/* for generate stmt. equiv. block */
	ForGenerateGroupRegion,		/* for generate stmt. special region */
	ComponentInstantiationRegion,	/* component inst. equivalent block */
	ProcessRegion, 			/* process statement region */
	ProcedureRegion,		/* procedure region (if used) */
	FunctionRegion			/* function region (if used) */
    } tRegionKind;


/* *******************************************************************
*
* ... -
*
******************************************************************* */


TK_tRegionHandle CreateRegion(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle parent_region,	/* parent region */
	tRegionKind region_kind,	/* region type */
	VK_tName region_name		/* identifier part of name */	
#endif
	);

/* *******************************************************************
*
* CreatePackage - create region handle for package
*
******************************************************************* */

TK_tRegionHandle CreatePackage(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle parent_region,	/* parent hierarchical item -
					   NULL for now */
					/* [Note: will need library] */
	VK_tName package_name		/* package name identifier */
#endif
	);


/* *******************************************************************
*
* CreatePackageBody - create region handle for package
*
******************************************************************* */

TK_tRegionHandle CreatePackageBody(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle extended_region
					/* package decl.'s region handel */
					/* [Note: will need library] */
#endif
	);


/* *******************************************************************
*
* CreateDesignEntity - create region handle for design entity
*
******************************************************************* */

TK_tRegionHandle CreateDesignEntity(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle parent_region,	/* parent hierarchical item -
					   component instantiation */
					/* [Note: will need library] */
	VK_tName entity_name,		/* entity declaration identifier */
	VK_tName architecture_name	/* architecture body identifier */
#endif
	);


/* *******************************************************************
*
* CreateBlock - create region handle for block statement
*
******************************************************************* */

TK_tRegionHandle CreateBlock(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle parent_region,	/* parent hierarchical item -
					   architecture, block, or generate */
	VK_tName block_name		/* block name identifier */
#endif
	);


/* *******************************************************************
*
* CreateIfGenerate - create region handle for if generate statement
*
******************************************************************* */

TK_tRegionHandle CreateIfGenerate(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle parent_region,	/* parent hierarchical item -
					   architecture, block, or generate */
	VK_tName label_name,		/* label name identifier */
	VK_tBoolean condition_value	/* value of expression */
#endif
	);
/* Note: Is expected that generated code call this whether or not the condition
	value is true.
*/

#if 0 /* disable for now to avoid circular .h file dependency through
	tTypeClass and TK_tRegionHandle */
/* *******************************************************************
*
* CreateForGenerateGroup - create group region handle for for generate
*	statement
*
******************************************************************* */

TK_tRegionHandle CreateForGenerateGroup(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle parent_region,	/* parent hierarchical item -
					   architecture, block, or generate */
	VK_tName label_name,		/* label name identifier */
	int instance_count,		/* simple left of for range */
	VK_tScalarData range_left,	/* left bound of for range */
	VK_tDirection range_dir,	/* direction of for range */
	VK_tScalarData range_right,	/* right bound of for range */
	tTypeClass type_class,		/* type class of for parameter */
	VK_tSymbInfoHandle type_symb_info
					/* parameter enumeration type info. */
#endif
	);
/* Note: Is expected that generated code call this whether or not the range
	is null.
*/


/* *******************************************************************
*
* CreateForGenerateBlock - create region handle for generated block
*
******************************************************************* */

TK_tRegionHandle CreateForGenerateBlock(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle group_region,	/* block's generate group */
	int position_value,		/* relative index value (0 based) */
	VK_tScalarValue index_value	/* VHDL index value */
#endif
	);
#endif



/* *******************************************************************
*
* CreateComponentInstantiation - create region handle for comp. inst. stmt.
*
******************************************************************* */

TK_tRegionHandle CreateComponentInstantiation(
#ifdef ANSI_FN_PROTOS
	TK_tRegionHandle parent_region,	/* parent hierarchical item -
					   architecture, block, or generate */
	VK_tName comp_inst_name	/* component inst. name identifier */
#endif
	);


void InitKernelRegion(
#ifdef ANSI_FN_PROTOS
#endif
	);

#endif /* TKRegions_H */



