static char SCCS_ID[] = "@(#)TKSignals.c	42.3 1/13/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKSignals.c
*
* DESCRIPTION:  
*	<then describe contents>
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Split from VKI.c
*	04-OCT-90 AWH	Added vector signals
*	22-FEB-91 DSB	Starting symbolic information interface conversion
*	18-MAR-91 HK	Added references to TIME macros.
*	26-MAR-91 HK	Added routine GetElementTypeInfo and
*			GetIndexByElementNumber
*	03-APR-91 WAG	W0023.  Renamed all _Eval_* routines to Evaluate*;
*			renamed _Eval_Explicit to EvaluateExplicit.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	30-APR-91 DSB	Removed duplicate XInfo and XDeclInfo enumerators
*	02-MAY-91 DSB	Cleaning up generally
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*	06-MAY-91 DSB	Cleaning up signal changes
*	14-MAY-91 DSB	W0053+: Use qqProcessKind instead of qqExecFn
*	16-MAY-91 DSB	W0053+: Removed old VK_Create{Scalar,Vector}Signal
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	21-MAY-91 DSB	Cleaned up old types; renamed to remove some VK_
*	30-MAY-91 DSB	Moved su_GetTypeInfoTypeClass from here from TKSymbInfo.c
*	05-JUN-91 DSB	Moved su_TraverseToData from TKSignals to TKSymbInfo
*       29-OCT-91 DKC   Various Changes for large physical type handling
*	 6-NOV-91 DKC	Changed CreateRootSignal to use type traversal,
*                       added some record handling.
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*	10-DEC-91 DKC	Created common buffer for value/time pairs for simple 
*                       and aggregate waveform assignment (and disconnection)
*                       - number of elements is now unlimited
*
* SCCS ID: @(#)TKSignals.c	42.3 1/13/92
*
******************************************************************* */
#include "VKI.h"
#include "TKCommon.h"
#include "TKSignals.h"
#include "TKSim.h"		/* for implicit signal act. record types */
#include "TKStructs.h"		/* tSignalRec */
#include "TKRegister.h"		/* for SIH_* */
#include "TKSymbUtils.h"	/* for su_GetTypeInfoTypeClass */
#include "TKTemp.h"		/* for VERIFY */
#include "TKListFns.h"
#include "TKGlobals.h"
#include "TKData.h"		/* for TIME macros */
#include "TKProcesses.h"	/* for Create*Process routines */
#include "TKData.h"		/* for ?? */
#include "TKIO.h"		/* for FormatHierarchicalName... */


static VK_tLargeData  LargeZero   = {0, 0}; /* for initialising data */
static VK_tBoolean    TrueBuffer  = True;

static TempList       ResDataList = NULL;

int ElementCount = 0;           /* count gives element id for formatter */

int OptimisePorts = FALSE;      /* make no attempt to remove simple ports */

char *TransactionImplicit = "'Transaction",
     *DelayedImplicit     = "'Delayed",
     *QuietImplicit       = "'Quiet",
     *StableImplicit      = "'Stable",
     *GuardSignal         = ".GUARD";

static tStaticStrBuf LongName    = STATIC_STR_INITIAL;
static tStaticStrBuf ShortName   = STATIC_STR_INITIAL;

typedef struct sDriverData {
        tDriverNumber        driver_start;
        VK_tSignalLength     driver_length;
        VK_tDataOffset       data_offset;
        tSignalElementHandle element;

    } tDriverDataRec;

typedef struct sWaveData {
        VK_tCompositeAddr   value_ptr;
        VK_tRelativeTimePtr time_ptr;

    } tWaveDataRec, *tWaveform;

static tWaveform   TempWave       = NULL;
static VK_tInteger TempWaveLength = 0;

#define CHECK_WAVE(l) {if (l > TempWaveLength) {\
                        if (0 > TempWaveLength) { FreeMemory(TempWave);}\
                        AllocateMemory(TempWave,tWaveDataRec,\
                                       TempWaveLength = l);}}


/* *******************************************************************
*
* Address Table Declarations
*
* Note: An address table is an array of void  pointers used to map an
* 	integer sequence to a collection of objects.
*
******************************************************************* */
typedef void * tAddress;	/* Address of any object */
typedef tAddress * tAddressTable; 
				/* Array of object addresses */
typedef long tAddressIndex;    	/* Index of address array */


/* *******************************************************************
*
* AllocateAddressEntry - Allocate an entry in an address table
*
******************************************************************* */
static tAddressIndex AllocateAddressEntry(
#ifdef ANSI_FN_PROTOS
     tAddressTable * table, 	/* inout: address table */
     tAddressIndex * count, 	/* inout: count of entries in table */	
     tAddress address		/* in: new address  */
#endif
  );


/* *******************************************************************
*
* AllocateSignalNumber - allocate entry in SignalNumberTable 
*	(MACRO)
*
******************************************************************* */
#define AllocateSignalNumber(HANDLE)\
	(tSignalNumber) AllocateAddressEntry(\
		(tAddressTable *) &SignalNumberTable, \
		(tAddressIndex *) &ScalarSignalCount, \
		(tAddress) HANDLE) 


/* *******************************************************************
*
* AllocateDriverNumber - allocate entry in DriverNumberTable 
*	(MACRO)
*
******************************************************************* */
#define AllocateDriverNumber(PROCESS,HANDLE)\
	(tDriverNumber) AllocateAddressEntry(\
		(tAddressTable *) &qqDriverTable(PROCESS), \
		(tAddressIndex *) &qqDriverCount(PROCESS), \
		(tAddress) HANDLE) 


/* *******************************************************************
*
*  ScalarUnionWordLength - returns number of words used for scalar type
*
*  HISTORY:
*        29-OCT-91 DKC   Created for handling large (and small) physical
*                        types.
*
******************************************************************* */
VK_tInteger ScalarUnionWordLength(data_class)
tTypeClass data_class;
{
/**/
    switch (data_class) {
        case tc_Enumeration:
	case tc_Physical:
	case tc_Floating:
        case tc_Integer:       return 1;

        case tc_LargePhysical: return 2;

	default:
	    ReportInternalError(
		"Unexpected TypeClass for ScalarUnionWordLength", "",
		__FILE__, __LINE__);
	    break;
    } /* switch */
} /* ScalarUnionWordLength */

/*****************************************************************************
*
* FormatSignalPortionName - Name a slice/sub element
*
* DESCRIPTION:
*
* NOTES:       This routine can be called during the construction of
*              the name segment tree - indicated by a segment data size of (0).
*
* HISTORY:
*	13-NOV-91 DKC	Created
*
*****************************************************************************/

char *FormatSignalPortionName(str_buff,start,length,and_group,and_root)
StaticStrBufPtr str_buff;
tSignalNumber   start,
                length;
int             and_group;
{
    tSignalElementHandle element;
    tNameSegmentHandle   segment;
    VK_tSignalHandle     group;
    tSignalNumber        scan,
                         target;
    tArrayDescriptor     *array_desc;
/**/
    element = SignalElementHandle(start);
    group   = qqRootSignal(qqGroup(element));
    segment = qqChildren(group);

    if (and_group) {
        FormatGroupName (str_buff, group, FALSE, and_root, start, length);
    }
    else {
        StaticStrCpy(str_buff,"");
    }

    if (NULL != segment && (0 == length || length != qqSignalLength(group))) {

        scan   = qqSignalStart(group);
        target = start;

        while (NULL != segment) {

            array_desc = segment->array_desc;

            if (NULL != array_desc) {
                int                     subs,
                                        subl,
                                        rem,
                                        span;
                VK_tIndexConstraintAddr constraint;
                tTempBuf80              s_index;

                constraint = array_desc->constraint;

                if (array_desc->index == 1) {
                    StaticStrCat (str_buff,"(");
                }
                else {
                    StaticStrCat (str_buff,",");
                }

                if (0 == qqDataLength(segment)) {
                    sprintf (s_index,"%d",constraint->left);
                    StaticStrCat (str_buff,s_index);
                    if (array_desc->index >= array_desc->indices) {
                        StaticStrCat (str_buff,")");
                    }
                    segment = qqChildren(segment);
                    continue;
                }

                subs = target - scan;
                if (0 != length) {
                    subl = length -1;
                }
                else {
                    subl = 0;
                }
                if (0 != subs) {
                    subs /= qqDataLength(segment);
                    rem   = (target - scan) - subs * qqDataLength(segment);
                }
                else {
                    rem   = 0;
                }

                if (0 != subl) {
                    subl /= qqDataLength(segment);
                }        
                
                sprintf (s_index,"%d",constraint->left 
                                          + subs * constraint->dir);
                StaticStrCat (str_buff,s_index);
                if (0 != subl) {
                    if (constraint->dir > 0) {
                        StaticStrCat (str_buff," to ");
                    }
                    else {
                        StaticStrCat (str_buff," downto ");
                    }
                    sprintf (s_index,"%d",constraint->left 
                                          + (subs + subl) * constraint->dir);
                    StaticStrCat (str_buff,s_index);
                    StaticStrCat (str_buff,")");
                    goto done;
                }

                if (0 != length && length == qqDataLength(segment)) {
                   StaticStrCat (str_buff,")");
                   goto done;
	        }

                if (array_desc->index >= array_desc->indices) {
                    StaticStrCat (str_buff,")");
                }

                scan   = 0;
                target = rem;
            }
            else {
                while (0 != qqDataLength(segment) &&
                       scan + qqDataLength(segment) <= target) {
                    scan   += qqDataLength(segment);
                    segment = qqSibling(segment);
                }

                StaticStrCat (str_buff,".");
                StaticStrCat (str_buff,qqName(segment));

                if (0 != length && length == qqDataLength(segment)) {
                    goto done;
                }
            }

            segment = qqChildren(segment);
        }
    }

done:
    return (str_buff->string);

} /* FormatSignalPortionName */


/*****************************************************************************
*
* FormatGroupName - add group (and root) to string.
*
* DESCRIPTION:
*
* HISTORY:
*	12-NOV-91 DKC	Created
*	08-JAN-92 DKC	Re-implemented as imp. data had moved
*
*****************************************************************************/

FormatGroupName(str_buff,group,simple,and_root,use_signal,length)
StaticStrBufPtr  str_buff;
VK_tSignalHandle group;
int              simple,
                 and_root;
tSignalNumber    use_signal,
                 length;
{
    tTempBuf256          root_name;
    char                 *implicit,
                         *guard;
    int                  time_parameter;
    tPrefixDataHandle    implicit_data;
/**/
    guard          = NULL;
    implicit       = NULL;
    time_parameter = FALSE;

    StaticStrCpy (str_buff,"");

    switch (qqSignalClass(group))
    {
        case sc_Transaction:
            implicit   = TransactionImplicit;
            use_signal = 0;
            break;

        case sc_Quiet:
            implicit = QuietImplicit;
            use_signal = 0;
            time_parameter = TRUE;
            break;

        case sc_Stable:
            implicit = StableImplicit;
            use_signal = 0;
            time_parameter = TRUE;
            break;

        case sc_Delayed:
            implicit = DelayedImplicit;
            time_parameter = TRUE;
            break;

	case sc_Guard:
            guard = GuardSignal;
            break;
    }

    if (NULL != implicit) {
        implicit_data = qqImplicitData(group);
        if (use_signal) {
            FormatSignalPortionName(str_buff,
                                    PrefixSignalStart(
                                          SignalElementHandle(use_signal)),
                                    length,
                                    TRUE,and_root);
        }
        else {
            FormatSignalPortionName(str_buff,
                                    qqPrefixSignalStart(implicit_data),
                                    qqPrefixSignalLength(implicit_data),
                                    TRUE,and_root);         
	}

        if (!simple) {
            StaticStrCat (str_buff,implicit);
            if (time_parameter) {
                tTempBuf80 time_buf;

                StaticStrCat (str_buff, "(");
    	        UTIME_FORMAT (time_buf, qqTime(implicit_data));
                StaticStrCat (str_buff, time_buf);
                StaticStrCat (str_buff, " ");
                StaticStrCat (str_buff, BaseUnit);
                StaticStrCat (str_buff, ")");
            }
        }
    }
    else if (and_root && NULL != qqRegionData(group)) {
        if (NULL != TK_GetRegionHandle(qqRegionData(group))) {
            FormatHierarchicalName(root_name,sizeof(root_name),
                                   TK_GetRegionHandle(qqRegionData(group)));
            StaticStrCpy (str_buff,root_name);
        }
        else if (NULL != TK_GetRegionInfo(qqRegionData(group))) {
            StaticStrCpy (str_buff,"/");
            StaticStrCat (str_buff,
                          qqName(TK_GetRegionInfo(qqRegionData(group))));
        }

        if (NULL != guard) {
            StaticStrCat (str_buff, guard);
        }
        else if (NULL != qqSymbInfo(group)) {
            StaticStrCat (str_buff, ".");
            StaticStrCat (str_buff,qqName(qqSymbInfo(group)));
        }
    }
    else if (NULL == guard && NULL != qqSymbInfo(group)) {
        StaticStrCpy (str_buff,qqName(qqSymbInfo(group)));
    }

} /* FormatGroupName */


/*****************************************************************************
*
* FormatElementName - Build name of element
*
* DESCRIPTION: 
*
* HISTORY:
*	12-NOV-91 DKC	Created
*
*****************************************************************************/

FormatElementName(str_buff,element,and_group,and_root)
StaticStrBufPtr      str_buff;
tSignalElementHandle element;
int and_group,
    and_root;
{
/**/
    FormatSignalPortionName(str_buff,qqNumber(element),0,and_group,and_root);
} /* FormatElementName */


/*****************************************************************************
*
* ElementFormatId - sequntial id of element;
*
* DESCRIPTION:
*
* HISTORY:
*	30-DEC-91 DKC	Created
*
*****************************************************************************/

VK_tInteger ElementFormatId(element)
tSignalElementHandle element;
{
    VK_tSignalHandle     group;
    VK_tInteger          format_id,
                         offset;
    tSignalNumber        signal_start;
    tSignalElementHandle scan;
/**/
    group        = qqGroup(element);
    format_id    = qqFormatId(group);
    signal_start = qqSignalStart(group);

    if (qqIsMixed(group)) {
        for (scan = qqElements(group); scan != element;) {
            format_id++;
            scan += qqDataLength(scan);
        }
    }
    else {
        offset = qqNumber(element) - signal_start;
        if (1 == qqDataLength(element)) {
            format_id += offset;
        }
        else {
            format_id += offset/qqDataLength(element);
        }
    }
    
    return (format_id);

} /* ElementFormatId */


/* *******************************************************************
*
* InitSignalElement - Create signal element record, set elements to
*	defaults.
*
* HISTORY:
*	25-SEP-90 AWH	Created for use with vector signals
*	01_APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*	03-APR-91 WAG	W0023.  Adjusted statement indentation to conform
*			to standards.
*	15-MAY-91 DSB	W0049: Changed qqResCallerFn to qqResEvalFn
*	08-OCT-91 DSB	Initialized new qqDiscTime
*	 6-DEC-91 DKC	Moved creation to CreatSignalGroup
*	08-JAN-92 DKC	Resolution data handling removed
*
******************************************************************* */

static void InitSignalElement(element,initial_value_ptr,
                                       elem_class,symb_info,
                                       evaluating,declare)
tSignalElementHandle element;
VK_tDataAddr         initial_value_ptr;
tTypeClass           elem_class;
tSymbInfoPtr         symb_info;
int                  evaluating, /* at initialisation */
                     declare;
{
    VK_tSignalHandle group;
    tScalarUnion     initial_value;
    tSignalNumber    signal_number;
/**/
    AddqqRecordKind  (element,SignalElementRecord);
    AddqqElementClass(element,elem_class);
    AddqqDataLength  (element,ScalarUnionWordLength(elem_class)); 
    AddqqEvaluating  (element,evaluating); 

    group         = qqRootSignal(qqGroup(element));
    signal_number = qqNumber(element);

    if (initial_value_ptr) {
        SetDefaultValue  (element,initial_value_ptr);
        SetDrivingValue  (element,initial_value_ptr);
        SetEffectiveValue(element,initial_value_ptr);
    } 

    AddqqTriggerActive   (element, TempNewList());
    AddqqTriggerEvent    (element, TempNewList());
    AddqqTriggerEventSets(element, TempNewList());

    ElementCount++;
    if (declare) {
        FormatElementName (&ShortName, element, FALSE, FALSE);

        RD_WriteSignalElementDeclaration(ElementCount, group, 
                                         elem_class,
                                         ComputeFieldWidth(symb_info),
                                         ShortName.string);
    }

} /* InitSignalElement */


/* *******************************************************************
*
* CreateSignalGroup - Create signal group record, set elements to
*	defaults.
*
* NOTES:
*       InitSignalElement must be called for each element
*
* HISTORY:
*	25-SEP-90 AWH	Created for use with vector signals
*	02-MAR-91 HK	Renamed class, kind, and region parameters, and
*			deleted the name parameter.
*	01_APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*	03-APR-91 WAG	W0023.  Adjusted statement indentation to conform
*			to standards.
*	21-NOV-91 DKC	Removed index and element type parameters
*	 6-DEC-91 DKC	Added creation of signal elements and data buffer
*                       (in one block of memory).
*	13-DEC-91 DKC	Moved AddqqRecordKind for element to InitSignalElement
*
******************************************************************* */
static VK_tSignalHandle CreateSignalGroup(symb_info, encl_region, signal_kind,
                                          signal_class, group_class, length)
tSymbInfoPtr       symb_info;
VK_tLARHeaderPtr   encl_region;
VK_tSignalKind     signal_kind;
tSignalClass       signal_class;
tGroupClass        group_class;
VK_tSignalLength   length;
{
    VK_tSignalHandle     group;
    tSignalElementHandle elements,
                         element;
    tSignalNumber        signal_number;
    int                  index,
                         data_space;
    VK_tDataAddr         values_ptr;
/**/
    /* allocate space */
    AllocateBlockZero(group, tSignalGroupRec, 
                             sizeof(tSignalGroupRec) 
                             + (SIGNAL_DATA_BUFFERS * BYTES(length))
                             + (length * sizeof(tSignalElementRec)));

    /* common data */
    AddqqRecordKind(group, SignalGroupRecord);
    AddqqGroupClass(group, group_class); 
    AddqqFormatId  (group, ElementCount +1);

    if (0 < length) {
        AddqqSignalBuffer(group,(VK_tDataAddr)&group[1]);
        values_ptr  = qqSignalBuffer(group);
        values_ptr += (SIGNAL_DATA_BUFFERS * length);
        elements    = (tSignalElementHandle)values_ptr;
        AddqqElements(group,elements);
        element       = &elements[0];
        signal_number = AllocateSignalNumber(element);
        AddqqSignalStart (group, signal_number);
        AddqqSignalLength(group, length);
        for (index = 0;;) {
            AddqqGroup          (element, group);
            AddqqNumber         (element, signal_number);

	    AddqqLastActiveTime (element, NeverTime);
	    AddqqLastActiveCycle(element, NeverCycle);
	    AddqqLastEventTime  (element, NeverTime);
	    AddqqLastEventCycle (element, NeverCycle);

            values_ptr  = qqSignalBuffer(group);
            values_ptr += index;
            AddqqEffectiveValuePtr(element,(tScalarUnionPtr)values_ptr);
            values_ptr += length;
            AddqqDrivingValuePtr  (element,(tScalarUnionPtr)values_ptr);
            values_ptr += length;
            AddqqDefaultValuePtr  (element,(tScalarUnionPtr)values_ptr);
            values_ptr += length;
            AddqqPreviousValuePtr (element,(tScalarUnionPtr)values_ptr);
 
            if (++index == length) break;

                ++element;
                signal_number = AllocateSignalNumber(element);
        }
    }

    AddqqSymbInfo(group, symb_info);

    if (gc_Alias != group_class) {
        /* add to global list */
        TempAddLast(SignalGroupList, group);

        AddqqSignalKind(group, signal_kind);
        AddqqSignalClass(group, signal_class);
 
        AddqqRegionData(group, encl_region);
        if (NULL != encl_region && 
            NULL != TK_GetRegionHandle(encl_region)) {
            AddqqNoted (group,TRUE);
	    TempAddLast(qqChildren(TK_GetRegionHandle(encl_region)),
                        (TempNode) group); /* ?? or scalar signal number */
        }

        AddqqImplicitSignals(group, TempNewList());

        AddqqRootSignal(group,group); /* default to self */
    }

    return group;
} /* CreateSignalGroup */


/*****************************************************************************
*
* FindCurrentValue - (associated with an element) in buffer for driver group
*                    or port
*
* DESCRIPTION:
*       Returns address of current value (which will be the address of
*       the element's driving value if it is not a resolved type).
*
* HISTORY:
*	24-DEC-91 DKC	Created
*
*****************************************************************************/

VK_tDataAddr FindCurrentValue(element,local_start,buffer)
tSignalElementHandle element;
tSignalNumber        local_start;
VK_tDataAddr         buffer;
{
    VK_tDataAddr value_ptr;
/**/
    value_ptr = buffer;

    if (qqGroupResolved(element)) {
        value_ptr += qqNumber(element) - qqSignalStart(qqGroup(element));
    }
    else {
        value_ptr += qqNumber(element) - local_start;
    }

    return (value_ptr);

} /* FindCurrentValue */


/*****************************************************************************
*
* MakeDrivingBuffer - Creates buffer for source of element if it is resolved,
*                     otherwise returns address of elements driving value.
*
* DESCRIPTION:
*
* HISTORY:
*	24-DEC-91 DKC	Created
*
*****************************************************************************/

VK_tScalarAddr MakeDrivingBuffer(first_element,signal_length)
tSignalElementHandle first_element;
VK_tSignalLength     signal_length;
{
    VK_tScalarAddr       buffer;
    tSignalElementHandle group_start;
    VK_tInteger          byte_length;
/**/
    if (qqResolved(first_element)) {
        if (qqGroupResolved(first_element)) {
            
            signal_length = qqSignalLength(qqGroup(first_element));
            byte_length   = BYTES(signal_length);

            AllocateBlock(buffer,VK_tScalarData,byte_length);

            group_start = SignalElementHandle(qqSignalStart(
                                                    qqGroup(first_element)));

            bcopy(qqDefaultValuePtr(group_start),buffer,byte_length);
        }
        else {
            byte_length = BYTES(signal_length);
            AllocateBlock(buffer,VK_tScalarData,byte_length);
            bcopy(qqDefaultValuePtr(first_element),buffer,byte_length);
        }
    }
    else {
        buffer = (VK_tScalarAddr)qqDrivingValuePtr(first_element);
    }

    return buffer;

} /* MakeDrivingBuffer */


/* *******************************************************************
*
* CreateDriverGroup - Create driver group record, set elements to defaults.
*
* HISTORY:
*	06-MAY-91 DSB	Created from CreateSignalGroup
*	 6-DEC-91 DKC	Driver group now holds current value of its elements
*
******************************************************************* */
static VK_tDriverHandle CreateDriverGroup(driver_base,signal_start,
                                                      driven_length)
tDriverNumber    driver_base;
tSignalNumber    signal_start;
VK_tSignalLength driven_length;
{
    VK_tDriverHandle group;
/**/

    /* allocate space */
    AllocateMemoryZero(group, tDriverGroupRec, 1);

    /* common data */
    AddqqRecordKind  (group, DriverGroupRecord);
    AddqqDriverStart (group, driver_base);
    AddqqDriverLength(group, driven_length);
    AddqqSignalStart (group, signal_start);

    /* add to global list */
    TempAddLast(DriverGroupList, group);

    return group;
} /* CreateDriverGroup */


/*****************************************************************************
*
* AddElementSource - adds source and checks count of maximum sources
*                    for resolved signal
*
* DESCRIPTION:
*
* HISTORY:
*	23-DEC-91 DKC	Created
*
*****************************************************************************/

void AddElementSource(element,source)
tSignalElementHandle element;
TempNode             source;
{
   VK_tInteger sources,
               data_space;
/**/
   if (NULL == qqSourceList(element)) {
       AddqqSourceList(element, TempNewList());
   }

   TempAddLast(qqSourceList(element), source);

   if (qqResolved(element)) {

       sources = TempListLength(qqSourceList(element));
       if (sources > MaxSourceCount) {
            MaxSourceCount = sources;
       }

       if (qqGroupResolved(element)) {
           data_space = qqSignalLength(qqGroup(element));
       }
       else {
           data_space = qqDataLength(element);
       }
       data_space *= sources;
       if (data_space > MaxResolvedData) {
           MaxResolvedData = data_space;
       }
   }
} /* AddElementSource */


/* *******************************************************************
*
* CreateDriverElement - Create driver for signal element and process
*
* HISTORY:
*	13-OCT-90 AWH	Created for use with vector signals
*	01_APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*       29-OCT-91 DKC   Added driver_number parameter
*	21-NOV-91 DKC	Use of Alloc...Zero to fix resolution bug
*	 6-DEC-91 DKC	Changed so that parent group holds current value
*
******************************************************************* */

tDriverElementHandle CreateDriverElement(element,process,driver_number)
tSignalElementHandle element;	/* in: signal of drive r*/
VK_tProcessHandle    process;	/* in: process of driver */
tDriverNumber        driver_number;    /* 0 or number to use */
{
    tDriverElementHandle driver;
    VK_tSignalLength     data_length;
/**/
    /* multiple drivers, no resolution function */
    if (!qqResolved(element) && !TempIsEmpty(qqSourceList(element)) ) {
        TempItem scan;

        for (scan=TempFirstItem(qqSourceList(element)); 
	    ! TempNullItem(scan); scan = TempNextItem(scan)) {
            driver = DrvElemValue(scan);
            if (DriverElementRecord == qqRecordKind(driver)) {
	        ReportKernelError(
		           "multiple sources for non-resolved signal", 
                           FormatSignalPortionName(&LongName,qqNumber(element),
                                                   0,TRUE,TRUE),
		           "", 0);
            }
        }
    }
	  
    AllocateMemoryZero(driver, tDriverElementRec, 1);
    AddqqRecordKind(driver, DriverElementRecord);

    data_length = qqDataLength(element);

    AddqqDataLength   (driver, data_length);
    AddqqParentElement(driver, qqNumber(element));

    AddqqLastActiveCycle(driver, NeverCycle);
    AddqqWaveformValues (driver, TempNewList());
    AddqqWaveformTimes  (driver, TempNewList());
    AddqqParentElement  (driver, qqNumber(element));
    AddqqOwnerProcess   (driver, process);

    if (0 == driver_number) {
        driver_number = AllocateDriverNumber(process,driver);

        while (--data_length) { /* allocate numbers as for SignalElement */
            AllocateDriverNumber(process,NULL);
        }
    }
    else {
        qqDriverTable(process)[driver_number] = driver;  
    }

    AddqqNumber(driver, driver_number);

    if (qqGroupResolved(element)) { /* add source to base element in group */
                                    /* - if not already attached           */
        TempItem             srcscan;
        tDriverElementHandle source_d;

        element = qqElements(qqGroup(element));

	if (NULL != qqSourceList(element)) {
            for (srcscan=TempFirstItem(qqSourceList(element)); 
		    ! TempNullItem(srcscan); srcscan = TempNextItem(srcscan)) {

	        source_d = DrvElemValue(srcscan);

	         if (qqRecordKind(source_d)   == DriverElementRecord &&
                     qqOwnerProcess(source_d) == process) {
                     goto done;
	         }
            }
	}
    }

    AddElementSource(element, (TempNode) driver);

done:
    return driver;

} /* CreateDriverElement */


/* *******************************************************************
*
* GetDriverByElementProcess - Get driver handle given signal element and
*	parent process.
*
* HISTORY:
*	13-OCT-90 AWH	Created for vector signals
*	03-APR-91 WAG	W0023.  Simplified program flow; removed extraneous
*			variable "result_driver".
*	 6-DEC-91 DKC	Changed verify to test for handling port assoc.
*
******************************************************************* */
static tDriverElementHandle GetDriverByElementProcess(element, process)
tSignalElementHandle element;
VK_tProcessHandle process;
{
    tDriverElementHandle driver;
    TempItem scan;
/**/

    if (NULL != qqSourceList(element)) {
        for (scan=TempFirstItem(qqSourceList(element));
		        ! TempNullItem(scan); scan=TempNextItem(scan)) {

	    driver = DrvElemValue(scan);
	    VERIFY_ERROR(NULL != driver);
	    if (DriverElementRecord == qqRecordKind(driver) &&
                qqNumber(process) == qqNumber(qqOwnerProcess(driver))) {

	        return driver; /* return the found driver handle */
	    }
        }
    }

    return NULL;			/* no valid drive handle found; */
					/* return NULL. */
} /* GetDriverByElementProcess */


/* ?? New algorithms for implicit signal creation and matching:
- For implicit signals 'Transaction, 'Stable(T), 'Quiet(T) effectively same

- For request to create 'Delayed(T) for some subelement(s) of S:
	- search S's list for group for S'Delayed(T) (match S and T, but not
		subelements of S; return group, not just elements)
	- create group S'Delayed(T) if is does not exist
		- (put qqPrefixSignalXx on groups instead of elements)
	- for each requested element:
		- if element exists already, do nothing
		- otherwise, create element (hook to element of S, etc.)
	- return handle of group, plus offset and length of whole request
- The point:  to fold S(2 to 3)'Delayed and S(1 to 2)'Delayed efficiently (and
	logically too).
- Apply this to drivers too (eliminate driver aliases).

?? */

	
/* *******************************************************************
*
* MatchImplicitSignal - find existing implicit signal
*
* DESCRIPTION:
*	Returns starting scalar signal number of matching implicit signal;
*	otherwise returns zero.
*
* HISTORY:
*	26-SEP-90 AWH	Created from create implicit signal code
*	03-APR-91 WAG	W0023.  Modified to conform to coding style standards.
*	12-DEC-91 DKC	Changed to match exact copy only
*	08-JAN-92 DKC	Returns group handle now
*
******************************************************************* */
static VK_tSignalHandle MatchImplicitSignal(prefix_start, prefix_length,
                                            class, time_parameter)
				    /* base signal of signal to match */
tSignalNumber    prefix_start;	    /* first scalar signal number in vector */
VK_tSignalLength prefix_length;	    /* number of signals in vector */
tSignalClass     class;		    /* class of signal to match */
				    /*  any but GUARD */
tRelTimePtr      time_parameter;    /* time for 'Stable, 'Quiet, 'Delayed */
{
    TempItem             scan;
    VK_tSignalHandle     group;
    tSignalElementHandle element;
/**/
    VERIFY_ERROR(class != sc_Guard);
    VERIFY_ERROR(prefix_start > 0);

    for (scan=
	TempFirstItem(qqImplicitSignals(qqGroup(SignalElementHandle(prefix_start))));
        ! TempNullItem(scan); scan=TempNextItem(scan)) {
					/* for each item in implicit signals */
					/* list */

	group = SigGrpValue(scan);
	VERIFY_ERROR(NULL != group);
	VERIFY_ERROR(qqSignalStart(group) > 0 );
	element = SignalElementHandle(qqSignalStart(group));

	/* this is way too complex: need to simplify, with a table ?? */

	if (qqSignalClass(group) == class) {
					/* same class */

	    if (prefix_start  == PrefixSignalStart(element) &&
                prefix_length == PrefixSignalLength(element)) {
					/* same base signal slice */
		switch (class) { 
		    case sc_Transaction:
           		return group; /* match */
			break;
		    case sc_Delayed:
		    case sc_Stable:
		    case sc_Quiet:
			if ((TIME_EQUAL(ImplicitTime(element), 
                                          *time_parameter))) {
			    return group; /* match */
			}
			break;
		    default:
			break;
		} /* switch */
	    } /* match base signal */
	} /* match class */
    } /* for */

  return NULL;  /* No Match */

} /* MatchImplicitSignal */


/*****************************************************************************
*
* tSubGroupParam is used to pass data between CreateRootSignal and
* CreateSubGroup through TraverseSubtype
*
*****************************************************************************/

typedef struct sBuildSegment {
        struct sBuildSegment    *parent;
        tNameSegment            **parent_child,
                                *last;
        tTypeClass              type_class;
        short                   index,
                                indices,
                                status;
        int                     data_start,
                                count;
        tResolutionDataPtr      resolution_data;
        VK_tIndexConstraintAddr constraint; /* for array  */
        TempList                field_list; /* for record */
        TempItem                field_item;
    } tBuildSegment;

#define INITIALISE  0
#define NEEDS_SIZE  1
#define COMPLETE    2

#define DO_BUILD    0
#define DONT_BUILD  1
#define ABORT_BUILD 2
        
typedef struct sSubGroupParam {
        VK_tSignalHandle   group,
                           root_signal;
        VK_tLARHeaderPtr   parent_region;
        tSymbInfoPtr       signal_info;
        VK_tLARHeaderPtr   signal_data;
        VK_tSignalKind     signal_kind;
        VK_tInteger        data_index;
        VK_tScalarAddr     data_buffer,
                           data_default;
        VK_tSignalLength   signal_length;
        VK_tBoolean        is_port,
                           is_resolved,
                           has_large,
                           has_small,
                           group_resolved;
        tBuildSegment      *current,
                           *resolving_segment;
    } tSubGroupParam;

static tSubGroupParam *Param;


/*****************************************************************************
*
* BuildNamePath - Create a branch/leaf in the name tree
*
* DESCRIPTION:
*
* HISTORY:
*	 7-NOV-91 DKC	Created
*
*****************************************************************************/

int BuildNamePath(segment,child)
tBuildSegment *segment,
              *child;
{
    tSymbInfoPtr            field_info;
    tNameSegment            *new_segment;
    VK_tIndexConstraintAddr constraint;
    tArrayDescriptor        *array_desc;
    int                     *count,
                            build = DO_BUILD;
/**/
    if (NULL == segment) {
        return (DO_BUILD);
    }

    switch (segment->type_class) {
        case tc_Record:
            if (INITIALISE == segment->status) {
                build = BuildNamePath(segment->parent,segment);
                if (DO_BUILD != build) {
                    return (build);
                }
                segment->field_item = TempFirstItem(segment->field_list);
            }
            else {
                segment->field_item = TempNextItem(segment->field_item);
            }

            field_info = (tSymbInfoPtr)TempValue(segment->field_item);
            
            AllocateMemoryZero(new_segment, tNameSegment, 1);
            SetqqName(new_segment,qqName(field_info));
            if (!segment->status) {
                VERIFY_ERROR(NULL == *segment->parent_child);
                *segment->parent_child = new_segment;
            }
            else {
                SetqqDataLength(segment->last,Param->data_index
                                            - segment->data_start);
                SetqqSibling(segment->last,new_segment);
            }
            segment->last = new_segment;
            segment->status = NEEDS_SIZE;
            segment->data_start = Param->data_index;
            child->parent_child = &new_segment->fChildren;
            break;

        case tc_Array:
            if (INITIALISE == segment->status) {

                if (1 < segment->index) {
                    if (1 < child->indices) {
                        segment->indices = child->indices;
                    }
                    else {
                        segment->indices = segment->index;
                    }
                    /* propagates index count up unless at outer index */
                }

                build = BuildNamePath(segment->parent,segment);
                if (DO_BUILD != build) {
                    return (build);
                }

                if (child->indices > 1) {
                    segment->indices = child->indices; /* sets outer index */
                }

                segment->status = NEEDS_SIZE;
                constraint      = segment->constraint;
                segment->count  = constraint->left;

                AllocateMemoryZero(new_segment, tNameSegment,     1);
                AllocateMemoryZero(array_desc,  tArrayDescriptor, 1);

                new_segment->array_desc = array_desc;
                array_desc->indices     = segment->indices;
                array_desc->index       = segment->index;
                array_desc->constraint  = constraint;

                child->parent_child = &new_segment->fChildren;
                segment->last = new_segment;
                segment->data_start = Param->data_index;
                VERIFY_ERROR(NULL == *segment->parent_child);
                *segment->parent_child = new_segment;
            }
            else {
                if (NEEDS_SIZE == segment->status) {
                    segment->status = COMPLETE;
                    SetqqDataLength(segment->last,Param->data_index
                                                  - segment->data_start);
                }
                constraint = segment->constraint;
                count      = &segment->count;
                if (*count == constraint->right) {
                    *count = constraint->left;
                }
                else {
                    *count += constraint->dir;
                }

                build = DONT_BUILD; 
                        /* already have a copy of sub element data */
            }
            break;

	default: /* Should be Scalar data */
            build = BuildNamePath (segment->parent,segment);
            break;
    }

    return (build);

} /* BuildNamePath */


/*****************************************************************************
*
* MakeResolvingSeg - mark segment as resolving
*
* DESCRIPTION:
*
* HISTORY:
*	26-DEC-91 DKC	Created
*
*****************************************************************************/

void MakeResolvingSeg(param,segment,res_func_info)
tSubGroupParam *param;
tBuildSegment  *segment;
tSymbInfoPtr   res_func_info;
{
    VK_tAnyResFnPtr    res_eval_fn;
    VK_tLARHeaderPtr   resolution_context;
    tResolutionDataPtr resolution_data;
    VK_tInteger        dimensions;
    VK_tTypeGroup      group_type;
    TempItem           scan;
/**/
    res_eval_fn = qqEvaluationFunction(SymbInfoValue(TempFirstItem(
                                       qqExtensions(res_func_info))));
    resolution_context = su_TraverseToData(qqEnclReg(res_func_info),
		                           qqEnclReg(param->signal_info),
                                                     param->signal_data);

    VERIFY_ERROR(NULL != res_eval_fn);

    if (NULL == resolution_context) {
	ReportKernelError(
	    "Impl. limitation: can't handle this resolution case",
	    qqName(param->signal_info), "", 0);
	    res_eval_fn = NULL;
    }    

    group_type = GroupType(res_func_info,NULL,&dimensions,0,0);

    if (NULL == ResDataList) {
        ResDataList     = TempNewList();
    	resolution_data = NULL;
    }
    else {
        for (scan=TempFirstItem(ResDataList);; scan=TempNextItem(scan)) {
            if (TempNullItem(scan)) {
    	        resolution_data = NULL;
                break;
            }
            else {
    	        resolution_data = ResDataValue(scan);
                if (qqResolutionContext(resolution_data) == resolution_context
                    && qqResEvalFn(resolution_data)      == res_eval_fn
                    && qqGroupType(resolution_data)      == group_type) {
              
                    break; /* data matches previously stored data */
                }
            }
        }
    }

    if (NULL == resolution_data) {
        tSymbInfoPtr            parm_info,
                                type_info,
                                index_info;
        VK_tLARHeaderPtr        constraint_act_rec;
        VK_tIndexConstraintAddr constraint;

        AllocateMemory(resolution_data,tResolutionDataRec,1);
        AddqqResEvalFn(resolution_data,res_eval_fn);
        AddqqResolutionContext(resolution_data,resolution_context);
        AddqqGroupType(resolution_data,group_type);

        TempAddFirst(ResDataList,resolution_data);

        parm_info  = SymbInfoValue(TempFirstItem(qqChildren(res_func_info)));
        type_info  = qqBaseTypeInfo(parm_info);
        index_info = SymbInfoValue(TempFirstItem(qqChildren(type_info)));

	constraint_act_rec = su_TraverseToData(
                                      qqConstraintRegionInfo(index_info), 
                                      qqEnclReg(param->signal_info),
                                                param->signal_data);

        constraint = (VK_tIndexConstraintAddr)(qqConstraintOffset(index_info) 
                                               + (char *) constraint_act_rec);

        VERIFY_ERROR(constraint->dir == VK_Ascending ||
                     constraint->dir == VK_Descending);

        AddqqLeftBound(resolution_data,constraint->left);
        AddqqDirection(resolution_data,constraint->dir);
    }

    segment->resolution_data = resolution_data;
    param->resolving_segment = segment;
    param->group_resolved    = FALSE;

} /* MakeResolvingSeg */


/*****************************************************************************
*
* CreateSubGroup - Callback for creating groups and elements of a root signal
*
* DESCRIPTION:
*
* HISTORY:
*	31-OCT-91 DKC	Created From CreateRootSignal
*	14-NOV-91 DKC	Added Trace Declarations and Name Path generation
*	19-NOV-91 DKC	Removed outer index description from signal decl.
*	 6-DEC-91 DKC	Modified to implement complex resolution
*
*****************************************************************************/

tUserStatus CreateSubGroup(param, trav_data_ptr)
tSubGroupParam *param;
tTravDataPtr   trav_data_ptr;
{
    tTraversalFlag flag;
    tSymbInfoPtr   symb_info;
    tTypeClass     elem_type_class;
    VK_tBoolean    enter_subgroup,
                   group_resolved;
    tGroupClass    group_class;
    tBuildSegment  name_segment;
/**/
    flag = TravGet_Flags(trav_data_ptr);
    
    if (IsMember(flag,tf_ScalarType)) {

        symb_info                = TravGet_SymbInfo(trav_data_ptr);
	group_class              = gc_ScalarSignal;
	name_segment.type_class  = su_GetTypeInfoTypeClass(symb_info);
        name_segment.indices     = 0;
        elem_type_class          = name_segment.type_class;
        enter_subgroup           = FALSE;
    }
    else if (IsMember(flag,tf_ArrayType)) {

        symb_info                = TravGet_SymbInfo(trav_data_ptr);
	group_class              = gc_VectorSignal;
	name_segment.type_class  = su_GetTypeInfoTypeClass(symb_info);
        name_segment.index       = TravGet_Dimension(trav_data_ptr);
        name_segment.constraint  = TravGet_ConstraintAddr(trav_data_ptr);
        name_segment.indices     = 0;
        name_segment.status      = FALSE;

        elem_type_class          = tc_BadTypeClass;
        enter_subgroup           = TRUE;
        group_resolved           = FALSE;

        if (NULL == param->resolving_segment) {
            if (NULL != qqResFuncInfo(symb_info)) {
                MakeResolvingSeg(param,&name_segment,qqResFuncInfo(symb_info));
            }
        }
        else if (1 == name_segment.index) {
            group_resolved = TRUE;
        }
    }
    else if (IsMember(flag,tf_RecordType)) {

        symb_info                = TravGet_SymbInfo(trav_data_ptr);
        group_class              = gc_CompositeSignal;
        name_segment.indices     = 0;
	name_segment.type_class  = su_GetTypeInfoTypeClass(symb_info);
        name_segment.field_list  = qqChildren(symb_info);
        name_segment.status      = FALSE;
      	elem_type_class          = tc_BadTypeClass;
        enter_subgroup           = TRUE;

        if (NULL != param->resolving_segment) {
            group_resolved = TRUE;
        }
        else {
            group_resolved = FALSE;
            if (NULL != qqResFuncInfo(symb_info)) {
                MakeResolvingSeg(param,&name_segment,
                                       qqResFuncInfo(symb_info));
            }
        }
        
    }
    else if (IsMember(flag,tf_RecordElem)) {
        symb_info = TravGet_SymbInfo(trav_data_ptr);
        if (NULL           == param->resolving_segment ||
            param->current == param->resolving_segment) {
                if (NULL == qqResFuncInfo(symb_info)) {
                    param->resolving_segment      = NULL;
                }
                else {
                    MakeResolvingSeg(param,param->current,
                                           qqResFuncInfo(symb_info));
                }
        }
        symb_info = (tSymbInfoPtr)NULL;   
    }
    else {
        symb_info = (tSymbInfoPtr)NULL;   
    }

    if (symb_info)
    {
        VK_tSignalHandle   group,
                           subgroup;
        VK_tBoolean        has_large,
                           has_small,
                           new_group,
                           make_subgroup;
        tResolutionDataPtr resolution_data;

        name_segment.parent       = param->current;
        name_segment.parent_child = NULL;
        group = param->group;
        if (NULL == group) {
            /* create and initialize signal group record */
            group = CreateSignalGroup(param->signal_info, 
                                      param->parent_region,
		                      param->signal_kind, 
                                      sc_Explicit, group_class,
                                      param->signal_length);

            AddqqIsPort(group,param->is_port);

            param->group = group;

            if (NULL == param->root_signal) {
                param->data_buffer        = qqSignalBuffer(group);
                param->root_signal        = group;
                param->signal_length      = 0; /* => no buffer creation in 
                                                                 subgroups */
                name_segment.parent_child = &group->fChildren;

                FormatGroupName(&LongName,  group, FALSE, TRUE, 0,0);
                FormatGroupName(&ShortName, group, TRUE, FALSE, 0,0);
                RD_WriteSignalGroupDeclaration(LongName.string,
                        ShortName.string, ElementCount +1,
                        (int)group,qqSignalLength(group),qqGroupClass(group));
            }
            else {
                SetqqSignalBuffer(group, &param->data_buffer
                                                 [param->data_index]);
                SetqqRootSignal  (group, param->root_signal);
                SetqqSignalStart (group, qqSignalStart(param->root_signal)
                                         + param->data_index);
                SetqqSignalLength(group, param->data_index);
            }

            new_group = TRUE;
        }
        else {
            VERIFY_ERROR(NULL != name_segment.parent);
            new_group = FALSE;
        }

        if (enter_subgroup) {
            make_subgroup = (!param->group_resolved && group_resolved);
            if (make_subgroup) {
                param->group_resolved = TRUE; 
                if (!new_group) {
                    param->group = NULL;
                }
                has_large = param->has_large;
                has_small = param->has_small;
                param->has_large = FALSE;
                param->has_small = FALSE;
            }

            param->current = &name_segment;

            TraverseSubtype_Recurse(trav_data_ptr);

            param->current = name_segment.parent;

            if (name_segment.status == NEEDS_SIZE) {
                SetqqDataLength(name_segment.last,param->data_index
                                                  - name_segment.data_start);
            }

            if (make_subgroup) {

                param->group_resolved  = FALSE;

                if (!new_group) {
                    subgroup     = param->group;
                    param->group = group;
                    SetqqSignalLength(subgroup,
                               param->data_index - qqSignalLength(subgroup));
                    SetqqElements(subgroup,
                               SignalElementHandle(qqSignalStart(subgroup))); 
                }
                else {
                    subgroup = group;
                }

                AddqqActiveCount(subgroup,qqSignalLength(subgroup));
                AddqqIsMixed    (param->group,param->has_large && 
                                              param->has_small);
                param->has_large |= has_large;
                param->has_small |= has_small;
           } 
        }
        else {
            VK_tProcessHandle      signal_process;
            VK_tScalarAddr         default_value;
            tSignalElementHandle   element;

            if (ABORT_BUILD == BuildNamePath(&name_segment,NULL)) {
                return;
            }

      	    /* initialize signal element record */

            if (NULL == param->data_default) {
                default_value = NULL;
	    }
            else {
                default_value = &param->data_default[param->data_index];
            }

       	    element = SignalElementHandle(qqSignalStart(param->root_signal)
                                          + param->data_index);

      	    InitSignalElement(element,default_value,
                              elem_type_class,symb_info,TRUE,TRUE);

            if (1 == qqDataLength(element)) {
               param->has_small = TRUE;
            }
            else {
               param->has_large = TRUE;
            }

            if (NULL != param->resolving_segment) {

                AddqqResolved (element,TRUE);

                param->is_resolved = TRUE;

                if (param->group_resolved) {
                    SetqqGroup        (element,group);
                    AddqqGroupResolved(element,TRUE);
                }
   
                if (NULL == qqResolutionData(group)) {
                    AddqqResolutionData(group,
                                    param->resolving_segment->resolution_data);
                }
                else {
                    VERIFY_ERROR(qqResolutionData(group) ==
                                    param->resolving_segment->resolution_data);
                }
            }

      	    /* create and initialize evaluation process */
       	    signal_process = CreateKernelProcess(KernelRegionHandle, 
                                 pc_Explicit, (VK_tVoidPtr)qqNumber(element));
					/* create evaluation process */

      	    AddqqEvalProcess(element, signal_process);

            param->data_index += qqDataLength(element);
        }
    }


    if (&name_segment == param->resolving_segment) {
        param->resolving_segment = NULL;
    }

    return FALSE; /* not finished */

} /* CreateSubGroup */


/*****************************************************************************
*
* SignalTypeData - 
*
* DESCRIPTION:
*
* HISTORY:
*	17-DEC-91 DKC	Created
*
*****************************************************************************/

VK_tLARHeaderPtr SignalTypeData(signal_info,encl_region_data,is_port)
tSymbInfoPtr     signal_info;
VK_tLARHeaderPtr encl_region_data;
int              is_port;
{
    VK_tLARHeaderPtr type_region_data;
    tSymbInfoPtr     type_info;
/**/
    VERIFY_ERROR(NULL != encl_region_data);

    type_info = qqBaseTypeInfo(signal_info);

    if (!is_port) {
        type_region_data  = su_TraverseToData(qqEnclReg(type_info),
                                   qqEnclReg(signal_info), encl_region_data);
    }
    else if (EntityDeclInfo == qqSymbInfoKind(qqEnclReg(signal_info))) {
        type_region_data = su_TraverseToData(qqEnclReg(type_info),NULL,NULL);
    }
/* kludge for port association (VKI deficiency) */
    else if (ComponentDeclInfo == qqSymbInfoKind(qqEnclReg(signal_info))) {
        type_region_data = su_TraverseToData(qqEnclReg(type_info),
                                       TK_GetRegionInfo(encl_region_data),
                                                        encl_region_data);
    }
/* ******************************************* */
    else {
        type_region_data = su_TraverseToData(qqEnclReg(type_info),
                                             qqEnclReg(qqEnclReg(signal_info)),
                                                        encl_region_data);
    }

    return (type_region_data);

} /* SignalTypeData */


/* *******************************************************************
*
* CreateRootSignal - create explicit signal or unconnected port
*
* NOTES:  Currently, this implements scalar signals and vector-of-scalar 
*	signals only, with scalar resolution only.
*
* HISTORY:
*	15-MAY-91 DSB	Created
*	16-MAY-91 DSB	Fixing bugs: put element type class in group
*	17-MAY-91 DSB	Fixed bug: pass group_class vs. gc_VectorSignal
*	22-MAY-91 DSB	Applied signal_length from VKI; restored arbitrary
*			types for signals (but only partial resolution)
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	30-MAY-91 DSB	Fixed bug: null res_func_info means don't check for
*			complex resolution
*	31-MAY-91 DSB	Slightly rearranged non-vector resolution check
*	11-JUN-91 DSB	Added large physical type check
*	18-JUL-91 DSB	Added missing kludge for partial record element display
*	26-AUG-91 DSB	Split CreateRootSignal from VK_CreateExplicitSignal
*	 6-NOV-91 DKC	Split to use TraverseSubType for handling
*                       complex structures.
*
******************************************************************* */
void CreateRootSignal(signal_handle_addr, signal_offset_addr,
	signal_info, encl_region_data, next_encl_region_data, top_constraint, 
	signal_length, default_value)
VK_tSignalHandle *signal_handle_addr;	/* signal handle address */
VK_tSignalOffset *signal_offset_addr;	/* signal offset address */
tSymbInfoPtr     signal_info;		/* signal declaration identification */
VK_tLARHeaderPtr encl_region_data;	/* enclosing region local act. rec. */
VK_tLARHeaderPtr next_encl_region_data;	/* enclosing region's enclosing region
					   local act. rec. (this is special
					   for unconnected ports) */
/* ?? let's find a better way to do this! ?? */
VK_tAnyConstraintAddr top_constraint;	/* range or index constraint addr. */
VK_tSignalLength signal_length;		/* length of signal to create */
VK_tDataAddr default_value;		/* signal default expression value */
{
    tSymbInfoPtr       type_info;
    tSubGroupParam     root_data;
    tBuildSegment      root_segment;
    VK_tLARHeaderPtr   type_region_data;
    tNameSegmentHandle segment;
/**/
    Param = &root_data;

    bzero (Param,sizeof(root_data));

    type_info               = qqBaseTypeInfo(signal_info);
    root_data.signal_info   = signal_info;
    root_data.signal_data   = encl_region_data;
    root_data.signal_kind   = qqSignalKind(signal_info);
    root_data.signal_length = signal_length;
    root_data.data_default  = default_value;

    if (NULL != qqResFuncInfo(signal_info)) {
        MakeResolvingSeg(&root_data,&root_segment,qqResFuncInfo(signal_info));
    }

    if (NULL != encl_region_data) {
        root_data.parent_region = encl_region_data;
    }
    else {
        root_data.parent_region = next_encl_region_data;
    }

    root_data.is_port = (NULL == encl_region_data);

    type_region_data  = SignalTypeData(signal_info,root_data.parent_region,
                                       root_data.is_port);

    TraverseSubtype(type_info, type_region_data, top_constraint,
	            tf_RecordType|tf_RecordElem|tf_ArrayType|tf_ScalarType,
                    CreateSubGroup, &root_data);

    AddqqIsResolved (root_data.group,root_data.is_resolved);
    AddqqIsMixed    (root_data.group,root_data.has_large && 
                                     root_data.has_small);
    AddqqActiveCount(root_data.group,qqSignalLength(root_data.group));

    /* return handle */

    *signal_handle_addr = root_data.group;
    *signal_offset_addr = 0;		/* whole signal has offset of zero */

} /* CreateRootSignal */


/* *******************************************************************
*
* VK_CreateExplicitSignal - create explicit signal 
*
* NOTES:  Currently, this implements scalar signals and vector-of-scalar 
*	signals only, with scalar resolution only.
*
* HISTORY:
*	26-AUG-91 DSB	Split CreateRootSignal from VK_CreateExplicitSignal
*	
******************************************************************* */
void VK_CreateExplicitSignal(signal_handle_addr, signal_offset_addr,
	signal_info, encl_region_data, top_constraint, signal_length,
	default_value)
VK_tSignalHandle      *signal_handle_addr; /* signal handle address        */
VK_tSignalOffset      *signal_offset_addr; /* signal offset address        */
tSymbInfoPtr          signal_info;	   /* signal declaration info      */
VK_tLARHeaderPtr      encl_region_data;	   /* enclosing region LAR         */
VK_tAnyConstraintAddr top_constraint;	   /* range/index constraint addr. */
VK_tSignalLength      signal_length;	   /* length of signal to create   */
VK_tDataAddr          default_value;	   /* signal default value         */
{
    VK_tLARHeaderPtr next_encl_region_data;
/**/
    if (NULL != VK_GetIARP(encl_region_data)) {
	next_encl_region_data = 
		VK_GetContextLARP(VK_GetIARP(encl_region_data));
    }
    else {
	next_encl_region_data = NULL;
    }
    CreateRootSignal(signal_handle_addr, signal_offset_addr, signal_info, 
	encl_region_data, next_encl_region_data, top_constraint, 
	signal_length, default_value);
} /* VK_CreateExplicitSignal */


/* *******************************************************************
*
* VK_SetDisconnectionTime - set disconnection time for signal
*
* HISTORY:
*	08-OCT-91 DSB	Created
*
******************************************************************* */
void VK_SetDisconnectionTime(signal_handle, signal_offset, signal_length,
	time_parameter)		/* returns nothing */
VK_tSignalHandle signal_handle;	/* in: signal handle */
VK_tSignalOffset signal_offset;	/* in: signal offset */
VK_tSignalLength signal_length;	/* in: signal length */
VK_tRelativeTimePtr time_parameter;
				/* in: disconnection time */
{
    int index;
    tSignalNumber signal_start;
    tSignalElementHandle element;
/**/
    signal_start = qqSignalStart(signal_handle) + signal_offset;

    VERIFY_ERROR((signal_start > 0) && 
		((signal_start + (signal_length - 1)) <= ScalarSignalCount));

    index = 0;
    while (index < signal_length) {
	element = SignalElementHandle(signal_start + index);
	SetqqDiscTime(element, *time_parameter);
        index  += qqDataLength(element);
    } /* while */
} /* VK_SetDisconnectionTime */


/*****************************************************************************
*
* GetSignalTypeInfo - get signal info from handle
*
* DESCRIPTION:
*
* HISTORY:
*	17-DEC-91 DKC	Created
*
*****************************************************************************/

void GetSignalTypeInfo(signal_handle,info_ptr,region_ptr,constraint_ptr)
VK_tSignalHandle        signal_handle;
tSymbInfoPtr            *info_ptr;
VK_tLARHeaderPtr        *region_ptr;
VK_tIndexConstraintAddr *constraint_ptr;
{
   tSymbInfoPtr       signal_info;
   tNameSegmentHandle top_seg;
   tArrayDescriptor   *top_bound;
/**/
   signal_info = qqSymbInfo(signal_handle);

   if (NULL == qqBaseTypeInfo(signal_info)) {
       *info_ptr       = signal_info;
       *constraint_ptr = NULL;
       *region_ptr     = NULL;
   }
   else {
       *info_ptr   = qqBaseTypeInfo(signal_info);
       *region_ptr = SignalTypeData(signal_info,qqRegionData(signal_handle),
                                                qqIsPort(signal_handle));

       top_seg     = qqChildren(signal_handle);

       if (NULL != top_seg && NULL != top_seg->array_desc) {
           top_bound       = top_seg->array_desc;
           *constraint_ptr = top_bound->constraint;
       }
       else {
           *constraint_ptr = NULL;
       }
   }

} /* GetSignalTypeInfo */


/*****************************************************************************
*
* GetSignalSubtypeInfo - Get symb. info. for slice of a signal.
*
* DESCRIPTION:
*
* HISTORY:
*	19-DEC-91 DKC	Created
*
*****************************************************************************/

tSymbInfoPtr GetSignalSubtypeInfo(signal_start,signal_length,dimensions)
tSignalNumber     signal_start;       
VK_tSignalLength  signal_length;
VK_tInteger       *dimensions;
{
    tSignalElementHandle    element;
    VK_tSignalHandle        signal_handle;
    tSymbInfoPtr            signal_info;
    VK_tLARHeaderPtr        region_data;
    VK_tIndexConstraintAddr constraint;
    tPrefixDataHandle       imp_data;
/**/
    element       = SignalElementHandle(signal_start);
    signal_handle = qqRootSignal(qqGroup(element));
 
    if (sc_Delayed == qqSignalClass(signal_handle)) {
        imp_data     = qqImplicitData(signal_handle);
        signal_start = qqPrefixSignalStart(imp_data) 
                         + (signal_start - qqSignalStart(signal_handle));

        return (GetSignalSubtypeInfo(signal_start,signal_length,dimensions));
                                                  /* recurse */
    }
    else {
        GetSignalTypeInfo(signal_handle,&signal_info,&region_data,&constraint);

        return su_GetSubtypeInfo(signal_info,region_data,constraint,
                                 signal_start - qqSignalStart(signal_handle),
                                 signal_length,dimensions);
    }
} /* GetSignalSubtypeInfo */


/* *******************************************************************
*
* CreateImplicitSignal - create implicit signal (other than GUARD)
*
* HISTORY:
*	02-OCT-89 DSB	Added 'Stable processing
*	03-OCT-89 DSB	Finished 'Stable processing
*	03-OCT-89 DSB	Added 'Quiet processing
*	...
*	03-MAY-90 DSB	Fixed 'Delayed to get type info from BASE signal
*	24-SEP-90 AWH	Changed signal handle to number
*	18-MAR-91 WAG	Changed CreateProcess calls to CreateKernelProcess
*			except for the user process creation call for 
*			'DELAYED, which changed to CreateUserProcess.
*	20-MAR-91 WAG	Removed the value for the priority parameter from
*			the CreateKernelProcess and CreateUserProcess calls.
*       26-SEP-90 AWH   Changed to support vector signals
*	28-MAR-91 HK	Corrected the information inherited by a 'Delayed
*			signal from its base signal.
*	02-APR-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*	14-MAY-91 AWH	WOO51: Change name from VK_CreateImplicitSignals
*	14-MAY-91 DSB	Applied split DelayedProcessList
*	17-MAY-91 DSB	Fixed bug: index added to offset an extra time
*	29-MAY-91 DSB	Fixed bug: prefix_signal_start neglected offset
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	03-JUN-91 DSB	Fixed bug: set offset for 'Stable, 'Quiet records
*	07-JUN-91 DSB	Fixed bug: Stable's field for Quiet's pointer
*	10-JUN-91 DSB	Applying 64-bit time
*       29-OCT-91 DKC   Changes for large physical types
*	31-OCT-91 DKC	Split 'Delayed element and driver generation into
*                       seperate loops to fix bug.
*	14-NOV-91 DKC	Added Trace Declarations
*	 6-DEC-91 DKC	Modified to use PrefixData structure attached 
*                       to implicit signal group
*
******************************************************************* */

static void CreateImplicitSignal(signal_handle_addr, signal_offset_addr, 
	prefix_signal_handle, prefix_signal_offset, prefix_signal_length, 
	which, time_parameter) 
VK_tSignalHandle * signal_handle_addr;	/* signal handle address */
VK_tSignalOffset * signal_offset_addr;	/* signal offset address */
VK_tSignalHandle prefix_signal_handle;   	/* for all but GUARD */
VK_tSignalOffset prefix_signal_offset;   	/* for all but GUARD */
VK_tSignalLength prefix_signal_length;
tSignalClass which;			/* any but GUARD */
VK_tRelativeTimePtr time_parameter;	/* for 'Stable, 'Quiet, 'Delayed */
{
    tSignalNumber             prefix_signal_start;
    VK_tSignalHandle          group, 
                              prefix_group;
    tSignalElementHandle      element;
    tSignalElementHandle      prefix_element,
                              first_element;	/* ?? first element only! ?? */
    tProcData_Implicit_Ptr    proc_data;
    tProcData_Delayed_Ptr     proc_data_d;
    VK_tProcessHandle         signal_process;
    VK_tSignalLength          index;
    tTypeClass                index_type_class;
    tSymbInfoPtr              index_type_symb_info,
                              prefix_element_symb_info;
    VK_tScalarData            left_index, 
                              right_index;
    tGroupClass               group_class;
    VK_tSignalLength          data_length;
    tPrefixDataHandle         imp_data;
    VK_tInteger               dimensions;
/**/
    prefix_signal_start = 
	qqSignalStart(prefix_signal_handle) + prefix_signal_offset;
    VERIFY_ERROR((prefix_signal_start > 0) && 
	   ((prefix_signal_start + (prefix_signal_length - 1)) <= 
		ScalarSignalCount));

    first_element = SignalElementHandle(prefix_signal_start);

    AllocateMemory         (imp_data, tPrefixData, 1);
    AddqqPrefixSignalStart (imp_data, prefix_signal_start);
    AddqqPrefixSignalLength(imp_data, prefix_signal_length);	
    if (NULL != time_parameter) {
        AddqqTime          (imp_data, *time_parameter);
    }

    first_element = SignalElementHandle(prefix_signal_start);

/* ?? this should take prefix_signal_handle soon; plus folding overlaps of
	'Delayed; be careful with partial overlaps */

	/* : check for duplicate implicit signal */
    if (group = MatchImplicitSignal(prefix_signal_start, 
		           prefix_signal_length, which, time_parameter )) {
	*signal_handle_addr = group;
	*signal_offset_addr = 0;	/* whole signal has offset of zero */
	return;
				/* ?? DON"T RETURN FROM MIDDLE, ESPECIALLY
				   WITHOUT ELSE! ?? */
    }
 
    /* Note: this function is too big needs to be factored ?? */

    /* create signal based on class */
    switch (which) {
    case sc_Transaction:
      /* Note: 	For a 'Transaction of a composite signal, create 
		signal with one element. */
   
      /* create and initialize signal group record */
      group = CreateSignalGroup(SIH_Std__Standard__Bit,NULL,VK_UnguardedKind,
                                which, gc_ScalarSignal, 1);

      AddqqSimpleType  (group,TRUE);
      AddqqImplicitData(group,imp_data);


      FormatGroupName (&LongName,  group, FALSE, TRUE, 0,0);

      element =qqElements(group);
      RD_WriteSignalGroupDeclaration(LongName.string, "", qqNumber(element),
                        (int)group, prefix_signal_length, qqGroupClass(group));

      /* create and initialize signal element record */
      InitSignalElement(element,&TrueBuffer,tc_Enumeration,
                        SIH_Std__Standard__Bit,FALSE,TRUE);

      /* element members */
      AddqqIsImplicit  (element, TRUE);
      SetDrivingValue  (element, &TrueBuffer);
      SetEffectiveValue(element, &TrueBuffer);
      SetPreviousValue (element, &TrueBuffer);

/* ?? Note:  Prefix signal data (handle, start/offset, length, and time) can
	be on group instead of element.  It's common to all elements of
	'Delayed group (only individual start/offset and length varies, and
	that can be computed as start (or offset) = group start (or offset) + 
	index, length = 1) For 'Stable, 'Quiet, and 'Transaction, there's only
	one element, and the element is evaluated specially so it doesn't
	even need to know directly
?? */

      /* Create signal process */

      AllocateMemoryZero(proc_data, tProcData_Implicit_Rec, 1);
					/* create activation record */

      signal_process = CreateKernelProcess(KernelRegionHandle, pc_Transaction, 
                                		(VK_tVoidPtr) proc_data);
					/* create process for S'Transaction */
      AddqqEvalProcess(element, signal_process);
      index = 0;
      while (index < prefix_signal_length) {
          prefix_element = SignalElementHandle(prefix_signal_start + index);
      	  TempAddLast(qqTriggerActive(prefix_element), signal_process);
					/* sensitize S'Transaction process to 
					   activity on S */
          index += qqDataLength(prefix_element);
      } /* while */


      SetqqImplicitSignalHandle(proc_data,group);
      break;

    case sc_Stable:
      /* Note: 	For a 'Stable of a composite signal, create 
		signal with one element. */
  
      /* create and initialize signal group record */
      group = CreateSignalGroup(SIH_Std__Standard__Boolean,NULL,
                                VK_UnguardedKind, which, 
	                        gc_ScalarSignal, 1);
    
      AddqqSimpleType  (group,TRUE);
      AddqqImplicitData(group,imp_data);

      FormatGroupName (&LongName,  group, FALSE, TRUE, 0,0);
      element = qqElements(group);
      RD_WriteSignalGroupDeclaration(LongName.string, "", qqNumber(element),
                        (int)group, prefix_signal_length, qqGroupClass(group));

      /* create and initialize signal element record */
      InitSignalElement(element, &TrueBuffer,tc_Enumeration,
                        SIH_Std__Standard__Boolean,FALSE,TRUE);

      /* element members */
      AddqqIsImplicit  (element,TRUE);

      /* Create signal process */
      AllocateMemory(proc_data, tProcData_Implicit_Rec, 1);
					/* crude: could use optimized driver */
      signal_process = CreateKernelProcess(KernelRegionHandle, pc_Stable, 
                                  		(VK_tVoidPtr)proc_data);
      AddqqEvalProcess(element, signal_process);
      index = 0;
      while (index < prefix_signal_length) {
          prefix_element = SignalElementHandle(prefix_signal_start + index);
          TempAddLast(qqTriggerEvent(prefix_element), signal_process);
					/* sensitize S'Stable process to event
					   on S */
          index += qqDataLength(prefix_element);
      } /* while */

      SetqqImplicitSignalHandle(proc_data,group);
      break;

    case sc_Quiet:
      /* Note: 	For 'Quiet of a composite signal, create signal with one 
		element. */
     
      /* create and initialize signal group record */
      group = CreateSignalGroup(SIH_Std__Standard__Boolean,NULL,
                                VK_UnguardedKind, which, 
                                gc_ScalarSignal, 1);
    
      AddqqSimpleType  (group,TRUE);
      AddqqImplicitData(group,imp_data);

      FormatGroupName (&LongName,  group, FALSE, TRUE, 0,0);

      element = qqElements(group);
      RD_WriteSignalGroupDeclaration(LongName.string, "", qqNumber(element),
                        (int)group, prefix_signal_length, qqGroupClass(group));

      /* create and initialize signal element record */

      InitSignalElement(element,&TrueBuffer,tc_Enumeration,
                        SIH_Std__Standard__Boolean,FALSE,TRUE);

      /* element members */
      AddqqIsImplicit  (element,TRUE);

      /* Create signal process */
      AllocateMemory(proc_data, tProcData_Implicit_Rec, 1);
					/* crude: could use optimized driver */
      signal_process = CreateKernelProcess(KernelRegionHandle, pc_Quiet, 
		                           (VK_tVoidPtr) proc_data);
      AddqqEvalProcess(element, signal_process);
      index = 0; 			 
      while (index <= prefix_signal_length-1) {
          prefix_element = SignalElementHandle(prefix_signal_start + index);
       	  TempAddLast(qqTriggerActive(prefix_element), signal_process);
					/* sensitize S'Quiet process to 
					   activity on S */
          index += qqDataLength(prefix_element);
      } /* while */

      SetqqImplicitSignalHandle(proc_data,group);
      break;

    case sc_Delayed:

      /* Note: For 'Delayed of a composite signal, create a 'delayed
		element for each scalar subelement of composite signal*/
      /* ?? Note: When the prefix is a single signal element, we assume that
		prefix signal is a scalar (indexed name).  But, the prefix
		signal may be a vector (one-element slice name).  Fix this
		when type information is registered for 'Delayed.  ?? */

      /* create and initialize signal group record */
      prefix_group = qqGroup(first_element);
      VERIFY_ERROR(qqRootSignal(prefix_group) == prefix_signal_handle);
					/* ?? use prefix_signal_handle ?? */

      index_type_class     = tc_BadTypeClass;
      index_type_symb_info = NULL;
      left_index           = 0;
      right_index          = 0;
      group_class          = gc_ScalarSignal; /* default to scalar */

      if (gc_VectorSignal    == qqGroupClass(prefix_group) || 
	  gc_CompositeSignal == qqGroupClass(prefix_group)) {
					/* composite signal */
          element   = SignalElementHandle(prefix_signal_start);
          data_length = qqDataLength(element); 
          if (data_length < prefix_signal_length) {
                                       /* multi - element */
	      group_class = qqGroupClass(prefix_group);
          }
      }

      group = CreateSignalGroup(GetSignalSubtypeInfo(prefix_signal_start,
                                                     prefix_signal_length,
                                                     &dimensions),
                                qqRegionData(prefix_group),
                                VK_UnguardedKind, 
			        sc_Delayed, group_class, prefix_signal_length);

      AddqqImplicitData(group,imp_data);
      element = qqElements(group);
      FormatGroupName (&LongName, group, FALSE, TRUE, 0,0);
      RD_WriteSignalGroupDeclaration(LongName.string, "",
                               qqNumber(element), (int)group,
                               prefix_signal_length,
		               qqGroupClass(group));

      /* for each element in base signal */
      index   = 0;
      while (index < prefix_signal_length) {

          /* create and initialize signal element record */
          prefix_element = SignalElementHandle(prefix_signal_start + index);
          prefix_element_symb_info = GetElementTypeInfo(prefix_element);
          InitSignalElement(element, NULL, qqElementClass(prefix_element),
                            prefix_element_symb_info, TRUE,TRUE);

          AddqqIsImplicit  (element,TRUE);

	  /* : create equivalent process for half of 'Delayed evaluation */

          signal_process = CreateKernelProcess(KernelRegionHandle, 
                               pc_Explicit, (VK_tVoidPtr) qqNumber(element));
					/* create evaluation process */
          AddqqEvalProcess(element, signal_process);

          /* create delayed process */
	  AllocateMemory(proc_data_d, tProcData_Delayed_Rec, 1);

          SetqqImplicitSignalHandle(proc_data_d,group);
          SetqqImplicitSignalOffset(proc_data_d,index);
					/* save 'Delayed() signal */

          signal_process = CreateDelayedProcess(KernelRegionHandle,
                                                proc_data_d);

          TempAddLast(qqTriggerEvent(
                         SignalElementHandle(prefix_signal_start + index)),
		      signal_process);
					/* sensitize S'Delayed special process 
					   to event on S */
          /* note that S'Delayed has two processes, one to set
			S'Delayed driver <= S after T; and a normal process
			to set S'Delayed value according to the driver value */
			
	  VK_CreateDriver(&proc_data_d->delayed_driver, 
		          &proc_data_d->delayed_driver_offset, 
		          qqImplicitSignalHandle(proc_data_d), 
		          qqImplicitSignalOffset(proc_data_d), 1, 
		          signal_process);
					/* create driver (after signal_process
					   is set */

          data_length = qqDataLength(element);
          index      += data_length;
          element    += data_length;
      } /* while */

      /* point group to elements */

      break;

    default:
	*signal_handle_addr = NULL;
 	*signal_offset_addr = 0;
	return;
      break;
    } /*switch*/

    /* Add new implicit signal to base's list */
    TempAddLast(qqImplicitSignals(qqGroup(
	        SignalElementHandle(prefix_signal_start))), (TempNode) group);
  
    *signal_handle_addr = group;
    *signal_offset_addr = 0;		/* whole signal has offset of zero */

} /* CreateImplicitSignal */


/* *******************************************************************
*
* VK_CreateTransactionSignal - create implicit signal S'Transaction
*
* HISTORY:
* 	14-MAY-91 AWH	Created
*
******************************************************************* */
void VK_CreateTransactionSignal(signal_handle_addr, signal_offset_addr,
    prefix_signal_handle, prefix_signal_offset, prefix_signal_length)
VK_tSignalHandle * signal_handle_addr;  /* created signal handle address */
VK_tSignalOffset * signal_offset_addr; 	/* created signal offset address */
VK_tSignalHandle prefix_signal_handle;
VK_tSignalOffset prefix_signal_offset;
VK_tSignalLength prefix_signal_length;	/* prefix signal S offset and length */
{
/**/
    CreateImplicitSignal(signal_handle_addr, signal_offset_addr,
           prefix_signal_handle, prefix_signal_offset, 
	   prefix_signal_length, sc_Transaction, &LargeZero);
} /* VK_CreateTransactionSignal */


/* *******************************************************************
*
* VK_CreateQuietSignal - create implicit signal S'Quiet(T)
*
* HISTORY:
* 	14-MAY-91 AWH	Created
*
******************************************************************* */
void VK_CreateQuietSignal(signal_handle_addr, signal_offset_addr,
         prefix_signal_handle, prefix_signal_offset, prefix_signal_length,
	 time_parameter)
VK_tSignalHandle * signal_handle_addr;	/* created signal handle address */
VK_tSignalOffset * signal_offset_addr;	/* created signal offset address */
VK_tSignalHandle prefix_signal_handle;
VK_tSignalOffset prefix_signal_offset;
VK_tSignalLength prefix_signal_length;	/* prefix signal S offset and length */
VK_tRelativeTimePtr time_parameter;	/* time value T in S'Quiet(T) */
{
/**/
    CreateImplicitSignal(signal_handle_addr, signal_offset_addr,
           prefix_signal_handle, prefix_signal_offset, 
	   prefix_signal_length, sc_Quiet, time_parameter);
} /* VK_CreateQuiteSignal */


/* *******************************************************************
*
* VK_CreateStableSignal - create implicit signal 'Stable(T)
*
* HISTORY:
* 	14-MAY-91 AWH	Created
*
******************************************************************* */
void VK_CreateStableSignal(signal_handle_addr, signal_offset_addr,
        prefix_signal_handle, prefix_signal_offset, prefix_signal_length,
        time_parameter)
VK_tSignalHandle * signal_handle_addr;	/* created signal handle address */
VK_tSignalOffset * signal_offset_addr;	/* created signal offset address */
VK_tSignalHandle prefix_signal_handle;
VK_tSignalOffset prefix_signal_offset;
VK_tSignalLength prefix_signal_length;	/* prefix signal S offset and length */
VK_tRelativeTimePtr time_parameter;	/* time value T in S'Stable(T) */
{
/**/
    CreateImplicitSignal(signal_handle_addr, signal_offset_addr,
           prefix_signal_handle, prefix_signal_offset, 
	   prefix_signal_length, sc_Stable, time_parameter);
} /* VK_CreateSTableSignal */


/* *******************************************************************
*
* VK_CreateDelayedSignal - create implicit signal S'Delayed(T)
*
* HISTORY:
* 	14-MAY-91 AWH	Created
*
******************************************************************* */
void VK_CreateDelayedSignal(signal_handle_addr, signal_offset_addr,
        prefix_signal_handle, prefix_signal_offset, prefix_signal_length,
        time_parameter, prefix_type)
VK_tSignalHandle   *signal_handle_addr;	 /* created signal handle address   */
VK_tSignalOffset   *signal_offset_addr;	 /* created signal offset address   */
VK_tSignalHandle    prefix_signal_handle;/* prefix signal S handle          */
VK_tSignalOffset    prefix_signal_offset;/* prefix signal offset and length */
VK_tSignalLength    prefix_signal_length;
VK_tRelativeTimePtr time_parameter;	 /* time value T in S'Delayed(T)    */
tSymbInfoPtr        prefix_type;	 /* base type of S for S'Delayed    */
{
/**/
/* ?? prefix_type: Test kernel does not use prefix_type at this time */

    CreateImplicitSignal(signal_handle_addr, signal_offset_addr,
           prefix_signal_handle, prefix_signal_offset, 
	   prefix_signal_length, sc_Delayed, time_parameter);
} /* VK_CreateDelayedSignal */


/* *******************************************************************
*
* VK_CreateGuardSignal - create implicit guard signal for guard expression
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	06-FEB-90 DSB	Modified interface
*	07-FEB-90 DSB	Implementing
*	24-SEP-90 AWH	Change signal handel to number.
*	26-SEP-90 AWH   Change to support elements and groups
*	18-MAR-91 WAG	Changed CreateProcess call to CreateKernelProcess.
*	20-MAR-91 WAG	Removed the value for the priority parameter from
*			the CreateKernelProcess call.  Removed priority
*			calculation code and local variables.
*	02-APR-91 WAG	W0015.  Replaced malloc call with AllocateMemory.
*	03-APR-91 WAG	W0023. Modified comments to conform to coding standards.
*	14-MAY-91 AWH	W0055: Changed name from VK_CreateImplicitGuard
*
******************************************************************* */
void VK_CreateGuardSignal(signal_handle_addr, signal_offset_addr,
        encl_region_data, evaluation_function, evaluation_context, sens_set)
VK_tSignalHandle * signal_handle_addr;	/* signal handle address */
VK_tSignalOffset * signal_offset_addr;	/* created signal offset address */
VK_tLARHeaderPtr encl_region_data;	/* block statement local act. rec. */
VK_tGuardEvalFnPtr evaluation_function;	/* to evaluate guard expression */
VK_tContextPtr evaluation_context;	/* context for evaluation function */
VK_tSensitivitySetHandle sens_set;	/* sensitivity set for guard evaluation */
{
    tSignalNumber          signal_number;
    tSignalElementHandle   element;
    VK_tSignalHandle       group;
    tProcData_Guard_Ptr    proc_data_guard;
    tProcData_Implicit_Ptr proc_data;
    VK_tProcessHandle      signal_process;
    VK_tBoolean            TrueBuffer = True;
/**/
    group = CreateSignalGroup(SIH_Std__Standard__Boolean, encl_region_data, 
	                      VK_UnguardedKind, sc_Guard, gc_ScalarSignal, 1);
					/* create and initialize signal grp */
					/* record */
    AddqqSimpleType(group,TRUE);
    element = qqElements(group);

    FormatGroupName(&LongName, group, FALSE, TRUE, 0,0);

    RD_WriteSignalGroupDeclaration(LongName.string, "", qqNumber(element),
                                   (int)group, 1, qqGroupClass(group));

    InitSignalElement(element,&TrueBuffer,tc_Enumeration,
                      SIH_Std__Standard__Boolean,TRUE,TRUE);
					/* create and initialize signal */
					/* element record. */

    AddqqIsGuard(element,TRUE);

    AllocateMemory(proc_data_guard, tProcData_Guard_Rec, 1);
					/* set up guard evaluation process */
					/* create process activation record */
    proc_data = (tProcData_Implicit_Ptr)proc_data_guard;
    proc_data->fImplicitSignalHandle = group;
    AllocateMemory(qqSignalBuffer(proc_data->fImplicitSignalHandle), 
	           VK_tIntegerData, 1);

    proc_data_guard->evaluation_function = evaluation_function;
    proc_data_guard->evaluation_context  = evaluation_context;
					/* fill in process activation record */


    signal_process = CreateKernelProcess(KernelRegionHandle, pc_Guard, 
	                                 (VK_tVoidPtr) proc_data);

    SetqqEnabled(sens_set, True);
    SetqqSensitiveProcess(sens_set, signal_process);
					/* sensitize S'Guard process to given
					   sensitivity set */
    AddqqEvalProcess(element, signal_process);

    *signal_handle_addr = group;
    *signal_offset_addr = 0;		/* whole signal has offset of zero */

} /* VK_CreateGuardSignal */
			

/*****************************************************************************
*
* GetDriverBuffer - find buffer for driver, and return remaining length
*
* DESCRIPTION:
*
* HISTORY:
*	27-DEC-91 DKC	Created
*
*****************************************************************************/

VK_tDataLength GetDriverBuffer(process_id,
                               driven_element,driver_number,buffer_addr)
VK_tProcessHandle    process_id;
tSignalElementHandle driven_element;
tDriverNumber        driver_number;
VK_tAddrAddr         buffer_addr;
{
    VK_tDataLength       length,
                         data_length;
    tDriverElementHandle driver;
    tSignalElementHandle element;
    VK_tSignalHandle     group;
/**/
    if (qqResolved(driven_element)) {
        if (qqGroupResolved(driven_element)) {
            group   = qqGroup(driven_element);
            element = SignalElementHandle(qqSignalStart(group));
            driver_number -= qqNumber(driven_element) - qqNumber(element);
            length = qqSignalLength(group);
            while (0 < length) {
                driver = qqDriverTable(process_id)[driver_number];
                if (NULL != driver) {
                    *buffer_addr = (VK_tDataAddr)qqCurrentValue(driver);
                    break;
                }
                data_length    = qqDataLength(element);
                element       += data_length;
                driver_number += data_length;
                length        -= data_length;
            }
            if (0 >= length) {
               *buffer_addr = MakeDrivingBuffer(driven_element,length);
            }
            length = qqSignalLength(group) - (qqNumber(driven_element)
                                                - qqSignalStart(group));
      }
      else {
          length = qqDataLength(driven_element);
          driver = qqDriverTable(process_id)[driver_number];
          if (NULL == driver) {
              *buffer_addr = MakeDrivingBuffer(driven_element,length);
          }
          else {
              *buffer_addr = (VK_tDataAddr)qqCurrentValue(driver);
          }
      }
   }
   else {
       *buffer_addr = (VK_tDataAddr)qqDrivingValuePtr(driven_element);
       length       = qqDataLength(driven_element);
   }

   return (length);

} /* GetDriverBuffer */


/* *******************************************************************
*
* VK_CreateDriver - create driver for explicit signal
* 
* HISTORY:
*	27-SEP-89 DSB	Created
*	28-SEP-90 AWH	Added vector signals
*	13-OCT-90 AWH	Check for duplicate drivers
*	03-APR-91 WAG	W0023. Modified comments to conform to coding standards.
*       29-OCT-91 DKC   Modified to handle large physical types and merge
*                       driver elements.
*
******************************************************************* */
void VK_CreateDriver(driver_handle_addr, driver_offset_addr, signal_handle, 
	signal_offset, group_length, process_id) 
VK_tDriverHandle * driver_handle_addr;
VK_tDriverOffset * driver_offset_addr;
VK_tSignalHandle signal_handle;
VK_tSignalOffset signal_offset;
VK_tSignalLength group_length;
VK_tProcessHandle process_id;
{
    VK_tDriverHandle     driver_group;
    tDriverElementHandle driver;
    tDriverNumber        driver_number,
                         first_driver_number;
    tSignalElementHandle element;
    tSignalNumber        signal_start; 
    VK_tSignalLength     length,
                         signal_length,
                         data_length,
                         buffer_length;
    VK_tDataAddr         driver_buffer;
/**/
    signal_start  = qqSignalStart(signal_handle);
    signal_length = qqSignalLength(signal_handle);

    VERIFY_ERROR(gc_Alias != qqGroupClass(signal_handle));
    VERIFY_ERROR((signal_start > 0) && 
		((signal_start + (signal_length - 1)) <= ScalarSignalCount));
    VERIFY_ERROR(ProcessRecord == qqRecordKind(process_id));

    element             = SignalElementHandle(signal_start);
    first_driver_number = 0;

    for (length = signal_length ;; element += data_length) {

   	driver = GetDriverByElementProcess(element, process_id);
	if (NULL != driver) {
            first_driver_number  = qqNumber(driver) 
                                    - (qqNumber(element) - signal_start);
            first_driver_number += signal_offset;
            break;
        }

        data_length = qqDataLength(element);
        length     -= data_length;
        if (0 == data_length || 0 >= length) { 
                                        /* element length may be zero if
                                           signal is partially initialized */
            break;
        }
    }


    if (0 == first_driver_number) /* no drivers from process to signal */
    { 
        first_driver_number = signal_offset +
                              AllocateDriverNumber(process_id, NULL);

        /* allocate for rest of signal */
        for (length = signal_length -1; 0 < length ; length--) {
             AllocateDriverNumber(process_id, NULL);
        }
    }

    signal_start += signal_offset;
    driver_group  = CreateDriverGroup(first_driver_number,
                                      signal_start,group_length);
 
    element       = SignalElementHandle(signal_start);
    driver_number = first_driver_number;
    driver_buffer = NULL;
    buffer_length = 0;

    for (length = group_length; 0 < length;) {
	       			/* for each element in the group */
        if (0 == buffer_length) {
            buffer_length = GetDriverBuffer(process_id,
                                            element,driver_number,
                                            &driver_buffer);
            signal_start  = qqNumber(element);
        }

        driver = qqDriverTable(process_id)[driver_number];
	if (NULL == driver) {
	    driver = CreateDriverElement(element,process_id,driver_number);

            AddqqCurrentValue   (driver,driver_buffer);
            AddqqCurrentValuePtr(driver,
                                 FindCurrentValue(element,signal_start,
                                                  driver_buffer));
	}

        data_length    = qqDataLength(element);
        element       += data_length;
        driver_number += data_length;
        length        -= data_length;
        buffer_length -= data_length;

    } /* for index */

    *driver_handle_addr = driver_group;	/* return new driver handle */
    *driver_offset_addr = 0;		/* return new driver offset */

} /* VK_CreateDriver */


/* *******************************************************************
*
* VK_CreateParameterAliases - create special aliases for signal 
*	parameter associated with independent subelement association
*
* HISTORY:
*	23-MAY-91 DSB	Created
*	28-MAY-91 DSB	Use ReportKernelError vs. ReportInternalError
*	31-MAY-91 DSB	Added drivers to VK_CreateParametersAliases
*	17-SEP-91 MNP	Added ANSI stdargs header
*	04-DEC-91 DSB	Fixed parameter indentation
*	 6-DEC-91 DKC	Added implementation - group has PortMap which is
*                       as port association.
*
******************************************************************* */

void VK_CreateParameterAliases(formal_signal_addr, formal_signal_offset,
	formal_driver_handle, formal_driver_offset, parameter_info,
	subprogram_intf_data, formal_total_length, association_element_count,
	formal_relative_offset_1, formal_relative_length_1,
	actual_signal_handle_1, actual_signal_offset_1, actual_length_1, 
	actual_driver_handle_1, actual_driver_offset_1 VA_ALIST)
				/* result data: */
VK_tSignalHandle *formal_signal_addr;	/* signal alias handle address */
VK_tSignalOffset *formal_signal_offset;
					/* signal alias offset address */
VK_tDriverHandle *formal_driver_handle;
					/* driver alias handle address; 
					   NULL if no driver alias is needed */
VK_tDriverOffset *formal_driver_offset;
					/* driver alias offset address;
					   NULL if no driver alias is needed */
				/* signal parameter identification data: */
tSymbInfoPtr     parameter_info;	/* signal parameter decl. ident. */
VK_tIARHeaderPtr subprogram_intf_data;	/* called subprogram *interface*
					   activation record */
VK_tSignalLength formal_total_length;	/* length of whole formal parameter */

VK_tListLength   association_element_count;
					/* number of association elements */

				/* first association element: */
VK_tSignalLength formal_relative_offset_1;
VK_tSignalLength formal_relative_length_1;
					/* formal portion offset and length */
VK_tSignalHandle actual_signal_handle_1;
VK_tSignalOffset actual_signal_offset_1;
VK_tSignalLength actual_length_1;
VK_tDriverHandle actual_driver_handle_1;
VK_tSignalOffset actual_driver_offset_1;
					/* actual signal (and driver) 
					   identification */
VA_DCL				/* additional association elements: */
					/* repeat 7-tuples for each additional
					   association element */

{
    va_list              varlistptr;  /* ptr. for traversing var. arg. list */
    int                  index;
    tSignalNumber        formal_start,
                         actual_start;
    tSignalElementHandle actual_element,
                         formal_elements,
                         formal_element;
    VK_tSignalHandle     actual_driver_element;
    int                  data_size;
    tPortMap             port_map;
    tPortAssocHandle     port;
/**/
    VA_START(varlistptr, formal_driver_offset_1);

    *formal_signal_offset = 0;
    *formal_signal_addr   = CreateSignalGroup(parameter_info, 
                                              subprogram_intf_data, 
                                              VK_UnguardedKind, 
                                              sc_BadSignalClass, gc_Alias,
                                              formal_total_length);
    if (NULL != formal_driver_handle) {
       VK_tDriverHandle group;

       AllocateMemoryZero(group,tDriverGroupRec,1);

       AddqqSignalStart(group,qqSignalStart(*formal_signal_addr));

       *formal_driver_handle = group;
       *formal_driver_offset = 0;
    }

    if (association_element_count > 0) {
        AllocateMemoryZero(port_map,tPortAssocHandle,formal_total_length);
        SetqqPortMap(*formal_signal_addr, port_map);
    }

    while (association_element_count-- > 0) {

        formal_start = qqSignalStart(*formal_signal_addr) 
                                    + formal_relative_offset_1;
        actual_start = qqSignalStart(actual_signal_handle_1)
                                    + actual_signal_offset_1;

        AllocateMemoryZero(port,tPortAssocRec,1);

        SetqqRecordKind      (port, PortAssocRecord);
        SetqqSignalDownStart (port, formal_start);
        SetqqSignalDownLength(port, formal_relative_length_1);
        SetqqSignalUpStart   (port, actual_start);
        SetqqSignalUpLength  (port, actual_length_1);
        if (NULL != actual_driver_handle_1) {
            SetqqDriverStart (port, qqDriverStart(actual_driver_handle_1)
                                    + actual_driver_offset_1);
        }

        VERIFY_ERROR(actual_length_1 == formal_relative_length_1);

        formal_element = SignalElementHandle(formal_start);

        for (index = 0; index < formal_relative_length_1 ; index ++) {
            AddqqElementClass(formal_element,tc_Alias);
            port_map[formal_relative_offset_1 + index] = port;
            SetqqUsed(port,1 + qqUsed(port));
            formal_element++;
        } 

        if (association_element_count > 0) {
            formal_relative_offset_1 = va_arg(varlistptr,VK_tSignalLength);
            formal_relative_length_1 = va_arg(varlistptr,VK_tSignalLength);
            actual_signal_handle_1   = va_arg(varlistptr,VK_tSignalHandle);
            actual_signal_offset_1   = va_arg(varlistptr,VK_tSignalOffset);
            actual_length_1          = va_arg(varlistptr,VK_tSignalLength);
            actual_driver_handle_1   = va_arg(varlistptr,VK_tDriverHandle);
            actual_driver_offset_1   = va_arg(varlistptr,VK_tSignalOffset);
        }   
    }
    va_end(varlistptr); /* end retrieving args */
} /* VK_CreateParameterAliases */


/* *******************************************************************
*
* VK_DestroyParameterAliases - destroy special aliases for signal parameter
*
* HISTORY:
*	23-MAY-91 DSB	Created
*	28-MAY-91 DSB	Use ReportKernelError vs. ReportInternalError
*	 6-DEC-91 DKC	Added implementation
*
******************************************************************* */
void VK_DestroyParameterAliases(signal_handle, signal_offset, driver_handle,
	driver_offset)
VK_tSignalHandle signal_handle;		/* signal handle */
VK_tSignalOffset signal_offset;		/* formal's signal offset */
VK_tDriverHandle driver_handle;		/* signal handle */
VK_tDriverOffset driver_offset;		/* formal's driver offset;
					   zero if there was no driver */
{
    tPortAssocHandle *scan_port,
                     port;
    VK_tSignalLength length,
                     scan;             
/**/
    VERIFY_ERROR(0 == signal_offset);
    scan_port = qqPortMap(signal_handle);
    length    = qqSignalLength(signal_handle);
    for (; 0 < length ; scan_port++,length--) {
        port = *scan_port;
        if (NULL != port) {
           SetqqUsed(port,qqUsed(port) - 1);
           if (!qqUsed(port)) FreeMemory(port);
        }
    }
    FreeMemory(qqPortMap(signal_handle));
    FreeMemory(signal_handle);

    if (NULL != driver_handle) {
        FreeMemory(driver_handle);
    }

} /* VK_DestroyParameterAliases */


/* *******************************************************************
*
* VK_AssociatePort_Simple - create and associate port, for simple association
*
* HISTORY:
*	23-MAY-91 DSB	Created
*	28-MAY-91 DSB	Use ReportKernelError vs. ReportInternalError
*	26-AUG-91 DSB	Implemented unconnected prots
*	 6-DEC-91 DKC	Full but non-optimising implementation 
*                       (see general case)
*
******************************************************************* */
void VK_AssociatePort_Simple(formal_handle_addr, formal_offset_addr,
	port_info, encl_region_intf_data, top_constraint, signal_length,
	default_value, actual_handle, actual_offset)
				/* formal port results: */
VK_tSignalHandle *formal_handle_addr;	/* formal port handle address */
VK_tSignalOffset *formal_offset_addr;	/* formal port offset address */
				/* formal port data: */
tSymbInfoPtr     port_info;		/* port declaration identification */
VK_tIARHeaderPtr encl_region_intf_data;	/* instantiated region *interface*
					   activation record */
VK_tAnyConstraintAddr top_constraint;	/* range or index constraint addr. */
VK_tSignalLength signal_length;		/* length of signal to create */
VK_tDataAddr     default_value;		/* port default expression value */
				/* association data: */
VK_tSignalHandle actual_handle;
VK_tSignalOffset actual_offset;		/* actual signal identification */
{
    int                  port_required;
    tSignalNumber        formal_start,
                         actual_start,
                         formal_offset;
    tSignalElementHandle actual_element,
                         formal_element;
    void                 (*copy_fn)();
    tPortMap             port_map;
    tPortAssocHandle     port;
    VK_tDataLength       data_length;
/**/
    if (NULL == actual_handle) {
       VERIFY_ERROR(0 == actual_offset);
       /* actual is "open"; port is unconnected */
       port_required = TRUE;
    }
    else if (OptimisePorts) {
        VK_tInteger index;        /* needs to be smarter than this (DKC) */
                                  /* - should compare default values     */
        port_required = FALSE;
        actual_start  = qqSignalStart(actual_handle);

        for (index = 0; index < signal_length ;) {

            actual_element = SignalElementHandle(actual_start +index);
            if (qqResolved(actual_element)) {
                port_required = TRUE;
                break;
            }
            else {
                index += qqDataLength(actual_element);
	    }
        }
    }
    else {
       port_required = TRUE;
    }

    if (port_required) {

        CreateRootSignal(formal_handle_addr, formal_offset_addr, port_info,
		         NULL, VK_GetCallerLARP(encl_region_intf_data), 
             	         top_constraint, signal_length, default_value);
                         /* create port signal */

        if (NULL != actual_handle) {

            AllocateMemoryZero(port_map,tPortAssocHandle,signal_length);
            SetqqPortMap(*formal_handle_addr, port_map);
            AllocateMemoryZero(port,tPortAssocRec,1);

            formal_start = qqSignalStart(*formal_handle_addr);
            actual_start = qqSignalStart(actual_handle) + actual_offset;

            VERIFY_ERROR(qqSignalLength(actual_handle) - actual_offset
                                                      >= signal_length);

            AddqqRecordKind      (port, PortAssocRecord);
            AddqqSimplePort      (port, TRUE);
            AddqqSignalDownStart (port, formal_start);
            AddqqSignalDownLength(port, signal_length);
            AddqqSignalUpStart   (port, actual_start);
            AddqqSignalUpLength  (port, signal_length);

            actual_element  = SignalElementHandle(actual_start);
            formal_element  = SignalElementHandle(formal_start);

            AddqqCurrentValue(port,MakeDrivingBuffer(actual_element,
                                                         signal_length));
            AddqqCurrentValuePtr(port,FindCurrentValue(actual_element,
                                                       actual_start,
                                                       qqCurrentValue(port)));
            formal_offset = 0;
            while (0 < signal_length) {

                AddqqMapped(formal_element,TRUE);

                port_map[formal_offset] = port;

                /* Add assoc rec to actual's driver list */
                AddElementSource(actual_element, (TempNode) port);

                data_length     = qqDataLength(actual_element);
                signal_length  -= data_length;
                formal_offset  += data_length;
                actual_element += data_length;
                formal_element += data_length;
            } 
        }   
    }
    else {
        *formal_handle_addr = actual_handle;
        *formal_offset_addr = actual_offset;
    }
} /* VK_AssociatePort_Simple */


/*****************************************************************************
*
* ConvertSignalOut - 
*
* DESCRIPTION:
*
* HISTORY:
*	27-NOV-91 DKC	Created
*
*****************************************************************************/

ConvertSignalOut(port)
tPortAssocHandle port;
{
    tConvFnDataPtr fn_data = qqConvOutData(port);
/**/
    VC_ConvertSignal(qqConvFn(fn_data),
                     qqDataSize(fn_data),
	             qqContext(fn_data),
	             qqCaller(fn_data),
                     qqInType(fn_data),
                     qqInDimensions(fn_data),
                     qqInConstraint(fn_data),
	             (VK_tDataAddr)qqDrivingValuePtr(SignalElementHandle(
                                                     qqSignalDownStart(port))),
                     qqOutType(fn_data),
                     qqOutDimensions(fn_data),
                     qqOutConstraint(fn_data),
                     qqSignalUpLength(port),
	             (VK_tDataAddr)qqCurrentValuePtr(port));
} /* ConvertSignalOut */


/*****************************************************************************
*
* ConvertSignalIn - 
*
* DESCRIPTION:
*
* HISTORY:
*	27-NOV-91 DKC	Created
*	16-DEC-91 DKC	Added condition - driving sub-nets 
*                       do not necessarily need an "in" conversion
*
*****************************************************************************/

void ConvertSignalIn(port)
tPortAssocHandle port;
{
    tConvFnDataPtr fn_data = qqConvInData(port);
/**/
    if (NULL == fn_data) {
        /* ?? should do check here perhaps ?? */
    }
    else {
        VC_ConvertSignal(qqConvFn(fn_data),
                         qqDataSize(fn_data),
	                 qqContext(fn_data),
	                 qqCaller(fn_data),
                         qqInType(fn_data),
                         qqInDimensions(fn_data),
                         qqInConstraint(fn_data),
	                 (VK_tDataAddr)qqEffectiveValuePtr(SignalElementHandle(
                                                       qqSignalUpStart(port))),
                         qqOutType(fn_data),
                         qqOutDimensions(fn_data),
                         qqOutConstraint(fn_data),
                         qqSignalDownLength(port),
	                 (VK_tDataAddr)qqDrivingValuePtr(SignalElementHandle(
                                                    qqSignalDownStart(port))));
    }
} /* ConvertSignalIn */


/*****************************************************************************
*
* FindFunction - locate function ptr.
*
* DESCRIPTION:
*
* HISTORY:
*	29-NOV-91 DKC	Created
*
*****************************************************************************/

static VK_tAnyTypeConvFnPtr FindFunction(func_info)
tSymbInfoPtr func_info;
{
    tSymbInfoPtr body_info;
    TempItem     scan;
/**/
    scan = TempFirstItem(qqExtensions(func_info));

    body_info = SymbInfoValue(scan);

    return qqEvaluationFunction(body_info);

} /* FindFunction */


/*****************************************************************************
*
* GroupType - Works out group type for VC_ConvertSignal
*
* DESCRIPTION:
*
* HISTORY:
*	 3-DEC-91 DKC	Created
*
*****************************************************************************/

int GroupType(func_info_out,func_info_in,dimensions,
              signal_start,signal_length)
tSymbInfoPtr     func_info_out,       /* function returning type */
                 func_info_in;        /* function whose parameter is type */
VK_tInteger      *dimensions;
tSignalNumber    signal_start;        /* for reporting error */
VK_tSignalLength signal_length;
{
    tSymbInfoPtr type_info,   /* for dimensions and base type */
                 parm_info;   /* for IsUnconstrained          */
/**/
    if (NULL != func_info_in) {
        parm_info   = SymbInfoValue(TempFirstItem(qqChildren(func_info_in)));
        type_info   = qqBaseTypeInfo(parm_info);
        *dimensions = qqDimensions(type_info);
    }
    else if (NULL != func_info_out) { /* use other function return */
        parm_info   = func_info_out;
        type_info   = qqBaseTypeInfo(func_info_out);
        *dimensions = qqDimensions(type_info);
    }
    else {
        type_info = NULL;
    }

    if (NULL != type_info) switch (qqSymbInfoKind(type_info)) {

        case RecordTypeDeclInfo:
            return VK_ConstrainedCompositeGroup;

        case ArrayTypeDeclInfo:
            return qqIsUnconstrained(parm_info) 
                       ? VK_UnconstrainedCompositeGroup
                       : VK_ConstrainedCompositeGroup;

        case FloatingTypeDeclInfo:
            return VK_FloatingGroup;

        case EnumerationTypeDeclInfo:
        case IntegerTypeDeclInfo:
	case PhysicalTypeDeclInfo:
            return VK_IntegerGroup;

        case LargePhysicalTypeDeclInfo:
            return VK_LargeGroup;

	default: break;
    }

    if (0 < signal_start) {
        ReportKernelError("Impl. limitation: can't determine port type for conversion",
                          FormatSignalPortionName(&LongName,signal_start,
                                              signal_length,TRUE,TRUE),
                          "", 0);
    }
    else {
        ReportKernelError("Impl. limitation: can't determine function return type",
                          qqName(func_info_out),
                          "", 0);
    }

    return 0; /* not reached */

} /* GroupType */


/* *******************************************************************
*
* VK_AssociatePort_General - create and associate port, for complex association
*
* DESCRIPTION:
*	VK_AssociatePort_General is used if independent subelements of a 
*	formal port are associated separately (in multiple association 
*	elements) or if there are type conversion functions.
*
* HISTORY:
*	28-MAY-91 DSB	Use ReportKernelError vs. ReportInternalError
*	17-SEP-91 MNP	Added ansi stdarg header
*	04-DEC-91 DSB	Fixed parameter indentation
*	04-DEC-91 DSB	Added formal_portion_constraint_1, actual_constraint_1
*	 6-DEC-91 DKC	Implementation: A PortMap as added to the signal
*                       - an array of pointers to PortAssoc records - indexed
*                       by the element offset in the group. The PortAssoc
*                       record indicates how the slices are connected (simple
*                       port assoc. is the degenerate case where only one 
*                       PortAssoc record is required.
*
******************************************************************* */

void VK_AssociatePort_General(formal_handle_addr, formal_offset_addr,
	port_info, encl_region_intf_data, top_constraint, signal_length,
	default_value, association_element_count, 
	formal_portion_offset_1, formal_portion_length_1, 
	actual_handle_1, actual_offset_1, actual_length_1, 
	in_type_conv_func_info_1, in_type_conv_func_context_1, 
	actual_constraint_1,
	out_type_conv_func_info_1, out_type_conv_func_context_1, 
	formal_portion_constraint_1 VA_ALIST)
				/* formal port results: */
VK_tSignalHandle * formal_handle_addr;	/* formal port handle address */
VK_tSignalOffset * formal_offset_addr;	/* formal port offset address */
				/* formal port data: */
tSymbInfoPtr     port_info;		/* port declaration identification */
VK_tIARHeaderPtr encl_region_intf_data;	/* instantiated region *interface*
					   activation record */
VK_tAnyConstraintAddr top_constraint;	/* range or index constraint addr. */
VK_tSignalLength signal_length;		/* length of signal to create */
VK_tDataAddr default_value;		/* port default expression value */
				/* association data: */
VK_tListLength association_element_count;
					/* number of association elements */
				    /* first association element: */
VK_tSignalLength formal_portion_offset_1;
VK_tSignalLength formal_portion_length_1;
					/* formal portion offset and length */
VK_tSignalHandle actual_handle_1;
VK_tSignalOffset actual_offset_1;
VK_tSignalLength actual_length_1;	/* actual signal identification */
tSymbInfoPtr     in_type_conv_func_info_1;
					/* inward type conv. func. ident. */
VK_tLARHeaderPtr in_type_conv_func_context_1;
					/* inward type conv. func. context */
VK_tIndexConstraintAddr actual_constraint_1;
					/* actual signal index constraint */
tSymbInfoPtr     out_type_conv_func_info_1;
					/* outward type conv. func. ident/ */
VK_tLARHeaderPtr out_type_conv_func_context_1;
					/* outward type conv. func. context */
VK_tIndexConstraintAddr formal_portion_constraint_1;
					/* formal portion index constraint */

VA_DCL				    /* additional association elements: */
				/* repeat 11-tuples for each additional
					   association element */

{
    va_list                 varlistptr;  /* ptr. for traversing arg. list */
    int                     port_required,
                            in_dimensions,
                            out_dimensions;
    tSignalNumber           formal_start,
                            actual_start;
    tSignalElementHandle    actual_element,
                            formal_element;
    tPortMap                port_map;
    tPortAssocHandle        port;
    tConvFnDataPtr          fn_data;
    VK_tContextPtr          context;	
    VK_tTypeGroup           type_up,
                            type_down;
    VK_tArraySize           dimensions_up,
                            dimensions_down;
    VK_tIndexConstraintAddr constraint_up,
                            constraint_down;
    VK_tDataLength          data_length;
/**/
    VA_START(varlistptr, formal_portion_constraint_1); 

    CreateRootSignal(formal_handle_addr, formal_offset_addr, port_info,
		     NULL, VK_GetCallerLARP(encl_region_intf_data), 
             	     top_constraint, signal_length, default_value);
                     /* create port signal */

    context = VK_GetContextLARP(encl_region_intf_data); /* ??? */

    if (association_element_count > 0) {
        AllocateMemoryZero(port_map,tPortAssocHandle,signal_length);
        SetqqPortMap(*formal_handle_addr, port_map);
    }

    while (association_element_count-- > 0) {


        if (NULL != actual_handle_1) /* check not left open */
        {
            formal_start = qqSignalStart(*formal_handle_addr) 
                                        + formal_portion_offset_1;
            actual_start = qqSignalStart(actual_handle_1)
                                       + actual_offset_1;

            AllocateMemoryZero(port,tPortAssocRec,1);

            SetqqRecordKind      (port, PortAssocRecord);
            SetqqSignalDownStart (port, formal_start);
            SetqqSignalDownLength(port, formal_portion_length_1);
            SetqqSignalUpStart   (port, actual_start);
            SetqqSignalUpLength  (port, actual_length_1);

            if (NULL == out_type_conv_func_info_1 &&
                NULL == in_type_conv_func_info_1) {
                SetqqSimplePort(port, TRUE);

                VERIFY_ERROR(formal_portion_length_1 == actual_length_1);
            }
            else {
	        AddqqActiveCount(port, actual_length_1); 
                /* - this is set for the initialisation phase */
                /*         - NB LastActiveCycle must be 0 too */

                type_down = GroupType(in_type_conv_func_info_1,
                                      out_type_conv_func_info_1,
                                      &in_dimensions,
                                      formal_start,formal_portion_length_1);

                VERIFY_ERROR(VK_UnconstrainedCompositeGroup != type_down ||
                             NULL != formal_portion_constraint_1);

                type_up   = GroupType(out_type_conv_func_info_1, 
                                      in_type_conv_func_info_1,
                                      &out_dimensions,
                                      actual_start,actual_length_1);

                VERIFY_ERROR(VK_UnconstrainedCompositeGroup != type_up ||
                             NULL != actual_constraint_1);

                if (NULL != out_type_conv_func_info_1) {
                    AllocateMemoryZero(fn_data, tConvFnData, 1);
                    SetqqDataSize    (fn_data,qqIntfDataSize(
                                                   out_type_conv_func_info_1));
                    SetqqInType      (fn_data,type_down);
                    SetqqInConstraint(fn_data,formal_portion_constraint_1);
                    SetqqInDimensions(fn_data,in_dimensions);
                    SetqqOutType     (fn_data,type_up);
                    SetqqOutConstraint(fn_data,actual_constraint_1);
                    SetqqOutDimensions(fn_data,out_dimensions);
                    SetqqConvFn      (fn_data,
                                      FindFunction(out_type_conv_func_info_1));
                    SetqqCaller      (fn_data,out_type_conv_func_context_1);
                    SetqqContext     (fn_data,context);
                    SetqqConvOutData(port,fn_data);
                }

                if (NULL != in_type_conv_func_info_1) {
                    AllocateMemoryZero(fn_data, tConvFnData, 1);
                    SetqqDataSize    (fn_data,qqIntfDataSize(
                                                    in_type_conv_func_info_1));
                    SetqqInType      (fn_data,type_up);
                    SetqqInConstraint(fn_data,actual_constraint_1);
                    SetqqInDimensions(fn_data,out_dimensions);
                    SetqqOutType     (fn_data,type_down);
                    SetqqOutConstraint(fn_data,formal_portion_constraint_1);
                    SetqqOutDimensions(fn_data,in_dimensions);
                    SetqqConvFn      (fn_data,
                                  FindFunction(in_type_conv_func_info_1));
                    SetqqCaller      (fn_data,in_type_conv_func_context_1);
                    SetqqContext     (fn_data,context);
                    SetqqConvInData(port,fn_data);
                }
            }

            /* set up buffering */
            actual_element = SignalElementHandle(actual_start);
            AddqqCurrentValue(port,MakeDrivingBuffer(actual_element,
                                                     actual_length_1));
            AddqqCurrentValuePtr(port,FindCurrentValue(actual_element,
                                                       actual_start,
                                                       qqCurrentValue(port)));
            /* Add assoc rec to actuals' driver list */
            while (0 < actual_length_1) {
                AddElementSource(actual_element, (TempNode) port);
                data_length      = qqDataLength(actual_element);
                actual_length_1 -= data_length;
                actual_element  += data_length;
            }

            /* fill out port map */
            formal_element = SignalElementHandle(formal_start);
            while (0 < formal_portion_length_1) {
                AddqqMapped(formal_element,TRUE);
                port_map[formal_portion_offset_1] = port;
                data_length              = qqDataLength(formal_element);
                formal_portion_length_1 -= data_length;
                formal_portion_offset_1 += data_length;
                formal_element          += data_length;
            } 
        }

        if (association_element_count > 0) {
            formal_portion_offset_1      = va_arg(varlistptr,VK_tSignalLength);
            formal_portion_length_1      = va_arg(varlistptr,VK_tSignalLength);
            actual_handle_1              = va_arg(varlistptr,VK_tSignalHandle);
            actual_offset_1              = va_arg(varlistptr,VK_tSignalOffset);
            actual_length_1              = va_arg(varlistptr,VK_tSignalLength);
            in_type_conv_func_info_1     = va_arg(varlistptr,tSymbInfoPtr);
            in_type_conv_func_context_1  = va_arg(varlistptr,VK_tLARHeaderPtr);
            actual_constraint_1          = va_arg(varlistptr,
                                                  VK_tIndexConstraintAddr); 
            out_type_conv_func_info_1    = va_arg(varlistptr,tSymbInfoPtr);
            out_type_conv_func_context_1 = va_arg(varlistptr,VK_tLARHeaderPtr);
            formal_portion_constraint_1  = va_arg(varlistptr,
                                                  VK_tIndexConstraintAddr);
        }   
    }
    va_end(varlistptr); /* end retrieving args */
} /* VK_AssociatePort_General */



/* *******************************************************************
*
* VK_NotePort - associate port with enclosing region
*
* DESCRIPTION:
*	The purpose of VK_NotePort is to associate a port with a region after
*	the local activation record for the region is available.
*
* ?? review: does this mean that we should create the region before creating
*	signals in the region (as we used to do)? ??
*
* HISTORY:
*	23-MAY-91 DSB	Created stub
*	26-AUG-91 DSB	Implemented (for simple (unconnected) ports)
*	27-AUG-91 DSB	Fixed bug: don't override flattened signals
*
******************************************************************* */
void VK_NotePort(port_info, encl_region_data, formal_handle, formal_offset)
tSymbInfoPtr     port_info;		/* port declaration identification */
VK_tLARHeaderPtr encl_region_data;	/* enclosing region local act. rec. */
VK_tSignalHandle formal_handle;		/* formal port signal handle */
VK_tSignalOffset formal_offset;		/* formal port signal offset */
{
    TK_tRegionHandle encl_region_handle;
/**/
    VERIFY_ERROR(NULL != port_info);
    VERIFY_ERROR(NULL != encl_region_data);
    VERIFY_ERROR(NULL != formal_handle);

    encl_region_handle = TK_GetRegionHandle(encl_region_data);
					/* get region handle from LARP */

    if (!qqNoted(formal_handle)) {
       AddqqNoted (formal_handle,TRUE);
       TempAddLast(qqChildren(encl_region_handle), (TempNode) formal_handle);
					/* add signal group as child of 
					   region */
    }
} /* VK_NotePort */



/* *******************************************************************
*
* AssignOneWaveform - current internal routine to assign waveform
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	28-SEP-90 AWH	Added vector signals
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*	16-MAY-91 DSB	Added negative and non-increasing time checks
*	17-MAY-91 DSB	Fixed bug: make sure first time gets queued
*	10-JUN-91 DSB	Applying 64-bit time
*	14-JUN-91 JBR	The parameter to AllocateTimeBuffer must be
*			tAbsTimePtr NOT tAbsTimeBuf.
*       29-OCT-91 DKC   Added data_length parameter for handling large physicals
*
******************************************************************* */
static void AssignOneWaveform(driver, delay_type, element_count, 
	index, data_length) 
tDriverElementHandle driver;
VK_tDelayType delay_type;
VK_tArraySize element_count;
VK_tSignalLength index;
VK_tDataLength data_length;
{
    tRelTimeBuf          rel_time_buf;
    tRelTimeBuf          prev_time;
    Node                 temp_node;
    tAbsTimePtr          scheduled_time;
    tAbsTimeBuf          first_time;
    TempItem             scant, scanv;
    int                  i;
    TempList             temp_list;
    TempItem             temp_item;
    int                  null_count;
    VK_tBoolean          done;
    tTempBuf256          msg_buf;
    tTempBuf80           proc_buf;
    tTempBuf80           time_buf;
    tSignalElementHandle element;
    VK_tBoolean          diff;
    VK_tCompositeAddr    scan_data, 
                         new_data;
    tScalarUnionPtr      value;
    tWaveform            waveform       = TempWave;
/**/
    TIME_COPY(rel_time_buf,*waveform->time_ptr);
    
    UTIME_ADD(first_time, rel_time_buf, SimTime);

    temp_list = qqWaveformTimes(driver);
					/* truncate waveform: remove 
					   transactions after earliest new
					   transaction */
    while (! (TempIsEmpty(temp_list)) && 
       (UTIME_LESSTHAN(first_time,
		*(tAbsTimePtr) TempValue(TempLastItem(temp_list))) ||
        TIME_EQUAL(first_time,
		*(tAbsTimePtr) TempValue(TempLastItem(temp_list))))) {

	TempRemoveLast(qqWaveformTimes(driver), temp_node);
        DeAllocTempSpace(temp_node);

	TempRemoveLast(qqWaveformValues(driver), temp_node);
        DeAllocTempSpace(temp_node);
    }

    scant = TempLastItem(qqWaveformTimes(driver));
    scanv = TempLastItem(qqWaveformValues(driver));

    if (VK_InertialDelay == delay_type) {
					/* remove equal items from end of
					   portion of waveform list before
					   first new waveform element */
	done = False;
	null_count = 0;

    	while (! done ) {
	    if (TempNullItem(scant)) {
	     	done = True; 		/* end of list */
	    } 
	    else  {

                scan_data  = (VK_tCompositeAddr) TempValue(scanv);
                new_data   = (VK_tCompositeAddr) waveform->value_ptr;

		if (NULL == new_data) {
		    null_count++;
	        }

		if (NULL == scan_data) {
		    null_count++;
		}

		if (1 == null_count) {
		    done = True;	/* one value null */
		}
		else {

                    new_data += index;

                    for (diff = data_length; diff ;)
		    {
                        if (*scan_data != *new_data) {
			    break;
                        }
			else if (--diff)  {
			    scan_data++;
			    new_data++;
	                } 
    		    } 
                     
                    if (0 == null_count && diff) {
        	        done = True;	/* values not equal */
	            }
	            else {
  	    	        scant = TempPrevItem(scant);
	    	        scanv = TempPrevItem(scanv);
					/* values equal */
		    }
		}
	    }
    	} /* while (! done) */

    	while (! TempNullItem(scant)) {
	    temp_item = scant;
	    scant = TempPrevItem(scant);
	    TempRemoveItem(temp_item, temp_node);
            DeAllocTempSpace(temp_node);

	    temp_item = scanv;
	    scanv = TempPrevItem(scanv);
	    TempRemoveItem(temp_item, temp_node);
            DeAllocTempSpace(temp_node);
    	}
    }
    else {
        /* Transport */
    }

    TIME_COPY(prev_time, EarlyTime);	/* initialize for non-incr. check */

    for (i = element_count; i-- ; waveform++) {

	TIME_COPY(rel_time_buf, *waveform->time_ptr);
		/* : check for negative delays */
    	if (STIME_NEGATIVE(rel_time_buf)) {
	    FormatHierarchicalName(proc_buf, sizeof(proc_buf), CurrentProcess);
	    STIME_FORMAT(time_buf, rel_time_buf);
	    sprintf(msg_buf, 
		"Negative time (%s %s) in signal assignment in process %s\n",
		time_buf, BaseUnit, proc_buf);
	    ReportKernelError(msg_buf, "", "", 0);
	    return;	
 	}
		/* : check for non-increasing delays */
    	if (! STIME_LESSTHAN(prev_time, rel_time_buf)) {
	    FormatHierarchicalName(proc_buf, sizeof(proc_buf), CurrentProcess);
	    STIME_FORMAT(time_buf, rel_time_buf);
	    sprintf(msg_buf, 
		"Non-increasing time %s %s in signal assignment in process %s\n",
		time_buf, BaseUnit, proc_buf);
	    ReportKernelError(msg_buf, "", "", 0);
	    return;	
 	}
	TIME_COPY(prev_time, rel_time_buf);
					/* update for non-incr. check */

        scheduled_time = (tAbsTimePtr)AllocTempSpace(sizeof(tAbsTimeBuf));
	UTIME_ADD(*scheduled_time, SimTime, rel_time_buf);

        if (TempIsEmpty(qqWaveformTimes(driver))) {

            if (NULL != qqScheduled(driver)) {
                DequeueEvent(qqScheduled(driver));
            }

            /* this is the next event for this driver - tell kernel */
            SetqqScheduled(driver,
                           QueueEvent(scheduled_time, ev_DriverEvent, driver));
        }

	TempAddLast(qqWaveformTimes(driver), scheduled_time);
	if (NULL == waveform->value_ptr) {
	    TempAddLast(qqWaveformValues(driver), NULL);
					/* handled null waveform elements */
	}
	else {
            value  = (tScalarUnionPtr)AllocTempSpace(sizeof(tScalarUnion));
            *value = *(tScalarUnionPtr)(&(waveform->value_ptr)[index]);

	    TempAddLast(qqWaveformValues(driver), value);
	}
    }
    return;
} /* AssignOneWaveform */


/*****************************************************************************
*
* RealElementAndDriver - trace through alias to real element/driver pair
*
* DESCRIPTION:
*
* HISTORY:
*	 3-DEC-91 DKC	Created
*
*****************************************************************************/

tSignalElementHandle RealElementAndDriver(process, element,driver)
VK_tProcessHandle    process;
tSignalElementHandle element;
tDriverElementHandle *driver;
{
    tPortMap             port_map;
    tPortAssocHandle     port;
    int                  offset;
    VK_tSignalHandle     group;
/**/
    group    = qqGroup(element);
    offset   = qqNumber(element) - qqSignalStart(group);
    port_map = qqPortMap(group);
    port     = port_map[offset];

    *driver  = qqDriverTable(process)[DriverOut(element,port)];
    return     SignalElementHandle(SignalOut(element,port));

} /* RealElementAndDriver */


/* *******************************************************************
*
* VK_AssignWaveform - assign waveform to driver
*
* HISTORY:
*	30-SEP-89 DSB	Created
*	24-SEP-90 AWH	Added vector signals
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*			Replaced assignment statement to time_ptrs[i] with
*			the TIME_COPY macro.
*       29-OCT-91 DKC   Modified to handle large physical types
*	 6-DEC-91 DKC	Modified to handle drivers of aliased signals
*
******************************************************************* */

void VK_AssignWaveform(driver_handle, driver_offset, length, signal_handle, 
	signal_offset, process, delay_type, waveform_element_count, value1, 
	time1 VA_ALIST)
VK_tDriverHandle driver_handle;
VK_tDriverOffset driver_offset;
VK_tSignalLength length;
VK_tSignalHandle signal_handle;
VK_tSignalOffset signal_offset;
VK_tProcessHandle process;
VK_tDelayType delay_type;
VK_tListLength waveform_element_count;
VK_tCompositeAddr value1;
VK_tRelativeTimePtr time1;
VA_DCL
{
    va_list              varlistptr;  /* ptr. for traversing var. arg. list */
    int                  i;
    VK_tSignalLength     index;
    tSignalNumber        signal_start;
    tSignalElementHandle element,
                         real_el;
    tDriverElementHandle driver;
    tDriverNumber        driver_start;
    VK_tSignalLength     data_length;
    tWaveDataRec         *next_wave;
/**/
    VA_START(varlistptr, time1);

    CHECK_WAVE(waveform_element_count);

    next_wave = TempWave;

    next_wave->value_ptr = value1;
    next_wave->time_ptr  = time1;

    for (i = waveform_element_count ; --i ;) {
        next_wave++;
        next_wave->value_ptr = va_arg(varlistptr, VK_tCompositeAddr);
        next_wave->time_ptr  = va_arg(varlistptr, VK_tRelativeTimePtr);
    }

    va_end(varlistptr);                  /* end retrieving args */

    element = SignalElementHandle(qqSignalStart(driver_handle)
                                               + driver_offset);

    /* driver specifies signal => signal info is not required
                                  - use for check only */
    VERIFY_ERROR(qqNumber(element) == qqSignalStart(signal_handle) 
                                                  + signal_offset);

    if (tc_Alias == qqElementClass(element)) {
        /* for each element of composite signal */
        for (index = 0; index < length;) {
            real_el = element;
            while (tc_Alias == qqElementClass(real_el)) {
                real_el = RealElementAndDriver (process, real_el, &driver);
            }
	    data_length = qqDataLength(real_el);
	    AssignOneWaveform(driver,
                              delay_type,waveform_element_count,
                              index, data_length);
            index   += data_length;
            element += data_length;
        } /* for */
    }
    else {
        driver_start = qqDriverStart(driver_handle) + driver_offset;
        VERIFY_ERROR((driver_start > 0) && 
	            ((driver_start + (length - 1)) <= qqDriverCount(process)));
    
        /* for each element of composite signal */
        for (index = 0; index < length;) {
            driver      = qqDriverTable(process)[driver_start + index];
	    data_length = qqDataLength(element);
	    AssignOneWaveform(driver,
                              delay_type,waveform_element_count,
                              index, data_length);
            index   += data_length;
            element += data_length;
        } /* for */
    }


/* ?? Note:  This should be cleaned up so the actions performed by
	VK_AssignWaveform are clear. ?? */

    return;
} /* VK_AssignWaveform */


/* *******************************************************************
*
* VK_AssignWaveform_Aggregate - assign waveform for aggregate target
*
* HISTORY:
*	02-DEC-91 DSB	Created stub
*	10-DEC-91 DKC	Implementation
*
******************************************************************* */

void VK_AssignWaveform_Aggregate(process, delay_type, target_count, 
	waveform_element_count VA_ALIST)
VK_tProcessHandle process; 		/* in: driver's process */
VK_tDelayType     delay_type; 		/* in: transport vs. inertial */
VK_tListLength    target_count;		/* in: number of signal names in 
					   aggregate target */
VK_tListLength    waveform_element_count;
                                     	/* in: number of waveform elements */
VA_DCL					/* in: 6-tuples for signal names, then
					   pairs for waveform elements */
{
    static tDriverDataRec *driver_data;
    static VK_tInteger    data_count = 0;

    va_list              varlistptr; /* ptr. for traversing var. arg. list */
    VK_tInteger          i;
    VK_tDataOffset       value_offset;
    VK_tDriverHandle     driver_handle;
    VK_tDriverOffset     driver_offset;
    VK_tSignalLength     length;
    VK_tSignalHandle     signal_handle;
    VK_tSignalOffset     signal_offset;
    VK_tCompositeAddr    value;
    VK_tRelativeTimePtr  time;
    tDriverDataRec       *next_driver;
    VK_tSignalLength     index;
    tDriverNumber        driver_start;
    VK_tSignalLength     data_length;
    tWaveDataRec         *next_wave;
    tSignalElementHandle element,
                         real_el;
    tDriverElementHandle driver;
/**/
    VA_START(varlistptr,waveform_element_count);

    if (target_count > data_count) {
        if (0 < data_count) {
            FreeMemory(driver_data);
        }
        AllocateMemory(driver_data,tDriverDataRec,target_count);
        data_count = target_count;
    }

    next_driver = driver_data;

    for (i = target_count ; i-- ; next_driver++) {
         value_offset  = va_arg(varlistptr, VK_tDataOffset);
         driver_handle = va_arg(varlistptr, VK_tDriverHandle);
         driver_offset = va_arg(varlistptr, VK_tDriverOffset);
         length        = va_arg(varlistptr, VK_tSignalLength);
         signal_handle = va_arg(varlistptr, VK_tSignalHandle);
         signal_offset = va_arg(varlistptr, VK_tSignalOffset);

         next_driver->driver_start = qqDriverStart(driver_handle)
                                                 + driver_offset;
         next_driver->driver_length = length;
         next_driver->data_offset   = value_offset;
         next_driver->element       = SignalElementHandle(
                                          qqSignalStart(driver_handle)
                                                       + driver_offset);

         /* driver specifies signal => signal info is not required
                                  - use for check only */

         VERIFY_ERROR(qqNumber(next_driver->element) ==
                                qqSignalStart(signal_handle) + signal_offset);
    }

    CHECK_WAVE(waveform_element_count);

    next_wave = TempWave;

    for (i = waveform_element_count ; i-- ; next_wave++) {
        next_wave->value_ptr = va_arg(varlistptr, VK_tCompositeAddr);
        next_wave->time_ptr  = va_arg(varlistptr, VK_tRelativeTimePtr);
    }

    va_end(varlistptr);                  /* end retrieving args */

    next_driver = driver_data;

    for (i = target_count ; i-- ; next_driver++) {

        element      = next_driver->element;
        value_offset = next_driver->data_offset;
        length       = next_driver->driver_length;

        if (tc_Alias == qqElementClass(element)) {
            /* for each element of composite signal */
            for (index = 0; index < length;) {
                real_el = element;
                while (tc_Alias == qqElementClass(real_el)) {
                    real_el = RealElementAndDriver (process, real_el, &driver);
                }
	        data_length = qqDataLength(real_el);
	        AssignOneWaveform(driver,
                                  delay_type, waveform_element_count,
                                  value_offset + index, data_length);
                index   += data_length;
                element += data_length;
            } /* for */
        }
        else {
            driver_start = next_driver->driver_start;
            VERIFY_ERROR((driver_start > 0) && 
	            ((driver_start + (length - 1)) <= qqDriverCount(process)));
    
            /* for each element of composite signal */
            for (index = 0; index < length;) {
                driver      = qqDriverTable(process)[driver_start + index];
	        data_length = qqDataLength(element);
	        AssignOneWaveform(driver,
                                  delay_type,waveform_element_count,
                                  value_offset + index, data_length);
                index   += data_length;
                element += data_length;
            } /* for */
        }
    }
} /* VK_AssignWaveform_Aggregate */


/* *******************************************************************
*
* VK_DisconnectDriver - disconnect driver using stored disconnection time
*
* HISTORY:
*	08-OCT-91 DSB	Created
*	 6-DEC-91 DKC	Changed to handle aliassed signals - now calls 
*                       AssignOneWaveform instead of VK_Ass...
*
******************************************************************* */
void VK_DisconnectDriver(driver_handle, driver_offset, length, signal_handle,
	signal_offset, process, delay_type)
				/* returns nothing */
VK_tDriverHandle driver_handle; /* in: driver handle */
VK_tDriverOffset driver_offset;	/* in: driver offset */
VK_tSignalLength length; 	/* in: driver length */
VK_tSignalHandle signal_handle;	/* in: signal handle */
VK_tSignalOffset signal_offset;	/* in: signal offset */
VK_tProcessHandle process; 	/* in: driver's process */
VK_tDelayType delay_type; 	/* in: transport vs. inertial */
{
    tDriverNumber        driver_start;
    int                  index;
    VK_tSignalLength     data_length;
    tDriverNumber        current_driver_offset;
    tSignalNumber        current_signal_offset;
    tDriverElementHandle driver;
    tSignalElementHandle element,
                         real_el;
/**/
    element = SignalElementHandle(qqSignalStart(driver_handle)
                                              + driver_offset);

    /* driver specifies signal => signal info is not required
                                  - use for check only */
    VERIFY_ERROR(qqNumber(element) == qqSignalStart(signal_handle) 
                                                  + signal_offset);
    CHECK_WAVE(1);  

    TempWave->value_ptr = NULL;

    if (tc_Alias == qqElementClass(element)) {
        /* for each element of composite signal */
        for (index = 0; index < length;) {
            real_el   = element;
	    data_length = qqDataLength(real_el);
            while (tc_Alias == qqElementClass(real_el)) {
                real_el = RealElementAndDriver (process, real_el, &driver);
            }

            TempWave->time_ptr = &qqDiscTime(element);

	    AssignOneWaveform(driver,
                              delay_type, 1,
                              index, data_length);

            index   += data_length;
            element += data_length;
        } /* for */
    }
    else {
        driver_start = qqDriverStart(driver_handle) + driver_offset;

        VERIFY_ERROR((driver_start > 0) && 
		    ((driver_start + (length - 1)) <= qqDriverCount(process)));

	/* : for each scalar driver subelement, assign 'null after T',
		where T is disconnection time for that subelement */

        index = 0;
        while (index < length) {
	    current_driver_offset = driver_offset + index;

            driver      = qqDriverTable(process)[driver_start + index];
	    data_length = qqDataLength(element);

            TempWave->time_ptr = &qqDiscTime(element);

	    AssignOneWaveform(driver,
                              delay_type, 1,
                              index, data_length);

            index    += data_length;
            element  += data_length;
        } /* for */
    }
} /* VK_DisconnectDriver */


/*****************************************************************************
*
* RealElement - trace through alias.
*
* DESCRIPTION: called through GetRealElement macro - overall return is NULL
*              if length implies a non-contigous block of elements.
*
* HISTORY:
*	 2-DEC-91 DKC	Created
*
*****************************************************************************/

tSignalElementHandle RealElement(element)
tSignalElementHandle element;
{
    tPortMap           port_map;
    tPortAssocHandle   port;
    int                offset;
    tSignalNumber      signal_number;
    VK_tSignalHandle   group;
/**/
    group         = qqGroup(element);
    signal_number = qqNumber(element);
    offset        = signal_number - qqSignalStart(group);
    port_map      = qqPortMap(group);
    port          = port_map[offset];

    element = SignalElementHandle(SignalOut(element,port));

    return GetRealElement(element);
        
} /* RealElement */


/*****************************************************************************
*
* RealDataSegment - trace through alias to data block - or return NULL.
*
* DESCRIPTION: called through GetRealData macro - overall return is NULL
*              if length implies a non-contigous block of elements.
*
* HISTORY:
*	 2-DEC-91 DKC	Created
*
*****************************************************************************/

tSignalElementHandle RealDataSegment(element,length)
tSignalElementHandle element;
VK_tSignalLength     length;
{
    tPortMap           port_map;
    tPortAssocHandle   port;
    int                offset;
    tSignalNumber      signal_number;
    VK_tSignalHandle   group;
/**/
    group         = qqGroup(element);
    signal_number = qqNumber(element);
    offset        = signal_number - qqSignalStart(group);
    port_map      = qqPortMap(group);
    port          = port_map[offset];

    if (0 != length && qqSignalDownLength(port) 
                       - (signal_number - qqSignalDownStart(port)) < length) {
        return (NULL);
    }

    element = SignalElementHandle(SignalOut(element,port));

    return GetRealData(element,length);
        
} /* RealDataSegment */


/*****************************************************************************
*
* ReadRealElement - Get Real Elements' value
*
* DESCRIPTION: Called if the element data required is disjoint when mapped
*
* HISTORY:
*	 3-DEC-91 DKC	Created
*
*****************************************************************************/

VK_tDataAddr ReadRealElement(element,length,last_value)
tSignalElementHandle element;
VK_tSignalLength     length;
int                  last_value;
{
    VK_tSignalLength     data_length;
    VK_tSignalHandle     group;
    VK_tDataAddr         signal_buffer,
                         return_value,
                         next_value;
    tSignalElementHandle real_el;
/**/
    group         = qqGroup(element);
    signal_buffer = qqSignalBuffer(group);
    return_value  = &signal_buffer[qqNumber(element) - qqSignalStart(group)];

    if (last_value) {
        return_value += qqSignalLength(group);
        next_value    = return_value;
        while (0 < length) {
            real_el = RealElement(element,0);
            GetPreviousValue(real_el,next_value);
            data_length = qqDataLength(real_el);
            length     -= data_length;
            element    += data_length;
            next_value += data_length;
        }
    }
    else {
        next_value = return_value;
        while (0 < length) {
            real_el = RealElement(element,0);
            GetEffectiveValue(real_el,next_value);
            data_length = qqDataLength(real_el);
            length     -= data_length;
            element    += data_length;
            next_value += data_length;
        }
    }

    return (return_value);

} /* ReadRealElement */


/* *******************************************************************
*
* VK_ReadSignalValue - read value of signal (implicit or explicit)
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	28-SEP-90 AWH	Vector signals
*	20-OCT-90 DSB	Added null element check
*	 6-DEC-91 DKC	Modified for new data structuring and aliasses
*
******************************************************************* */
void VK_ReadSignalValue(ptr_addr, signal_handle, signal_offset, signal_length)
VK_tAddrAddr ptr_addr;
VK_tSignalHandle signal_handle;
VK_tSignalOffset signal_offset;
VK_tSignalLength signal_length;
{
    tSignalElementHandle element,
                         real_el;
/**/
    GetElementHandle(element,signal_handle,signal_offset,signal_length);

    real_el = GetRealData(element,signal_length);

    if (NULL == real_el) {
        *ptr_addr = ReadRealElement(element,signal_length,FALSE);
    }    
    else {
        *ptr_addr = (VK_tDataAddr)qqEffectiveValuePtr(real_el);
    } 

} /* VK_ReadSignalValue */


/* *******************************************************************
*
* VK_ReadActive - read 'Active of signal
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	28-SEP-90 AWH	Vector signals
*
******************************************************************* */
void VK_ReadActive(boolean_addr, signal_handle, signal_offset, signal_length)
VK_tScalarAddr boolean_addr;
VK_tSignalHandle signal_handle;
VK_tSignalOffset signal_offset;
VK_tSignalLength signal_length;
{
    VK_tBoolean          result;
    int                  index;
    VK_tSignalLength     data_length;
    tSignalNumber        signal_start;
    tSignalElementHandle element,
                         real_el;
/**/
    GetElementHandle(element,signal_handle,signal_offset,signal_length);

    result = False;			/* unless True */
    index  = 0;
    while (index < signal_length) {
        real_el = GetRealElement(element);
    	if (SimCycleCount == qqLastActiveCycle(real_el)) {
					/* element had event this cycle */
	    result = True;
	    break;			/* BREAK */
	}
        data_length = qqDataLength(element);
        index      += data_length;
        element    += data_length;
    } /* while */

    *boolean_addr = result;
} /* VK_ReadActive */


/* *******************************************************************
*
* VK_ReadEvent - read 'Event of signal
*
* DESCRIPTION:
*   LRM 14.1.S'EVENT:
*	For a scalar signal S, S'EVENT returns the value TRUE if an
*	event has occurred on S during the current simulation cycle; 
*	otherwise, it returns the value FALSE.
*
*	For a composite signal S, S'EVENT returns TRUE if an event has 
*	occurred on any scalar subelement of S during the current 
*	simulation cycle; otherwise, it returns FALSE.
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	28-SEP-90 AWH	Added vector signals
*
******************************************************************* */
void VK_ReadEvent(boolean_addr, signal_handle, signal_offset, signal_length)
VK_tScalarAddr boolean_addr;
VK_tSignalHandle signal_handle;
VK_tSignalOffset signal_offset;
VK_tSignalLength signal_length;
{
    VK_tBoolean          result;
    int                  index;
    VK_tSignalLength     data_length;
    tSignalElementHandle element,
                         real_el;
/**/
    GetElementHandle(element,signal_handle,signal_offset,signal_length);

    result = False;			/* unless True */
    index = 0;
    while (index < signal_length) {
        real_el = GetRealElement(element);
     	if (SimCycleCount == 
		qqLastEventCycle(real_el)) {
					/* element had event this cycle */
	    result = True;
	    break;			/* BREAK */
	}
        data_length = qqDataLength(element);
        index      += data_length;
        element    += data_length;
    } 

    *boolean_addr = result; 
} /* VK_ReadEvent */



/* *******************************************************************
*
* VK_ReadLastActive - read 'Last_Active of signal
*
* DESCRIPTION:
*
* NOTE: LRM definition has errors.  
*   LRM 14.1.S'Last_Active:
*       Correct part:
*       "... Result: The amount of time that has elapsed since the last time at
*       which signal S was active."
*       Incorrect part:
*       "For a scalar signal S, S'LAST_ACTIVE returns the largest value T of
*       type Time for which S'DELAYED(T)'QUIET would return TRUE, if such a
*       value for T exists; otherwise it returns 0 ns."
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	03-MAY-90 DSB	Trying to implement LRM precisely
*	07-MAY-90 DSB	Implementing "Sense of the VASG" interpretation
*	28-SEP-90 AWH	Vector signals
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*	10-JUN-91 DSB	Applying 64-bit time
*
******************************************************************* */
void VK_ReadLastActive(time_buffer, prefix_signal_handle, 
	prefix_signal_offset, prefix_signal_length)
VK_tRelativeTimePtr time_buffer;	/* &out: result buffer address */
VK_tSignalHandle prefix_signal_handle;	/* in: prefix signal S handle */
VK_tSignalOffset prefix_signal_offset;	/* in: prefix signal S offset */
VK_tSignalLength prefix_signal_length;	/* in: prefix signal S length */
{
    tSignalElementHandle element,
                         real_el;
    int                  index;
    VK_tSignalLength     data_length;
    tRelTimeBuf          latest_time;	/* signed to accept EarlyTime */
/**/
    GetElementHandle(element,prefix_signal_handle,
                             prefix_signal_offset,prefix_signal_length);

    TIME_COPY(latest_time, EarlyTime);
    index = 0; 
    while (index < prefix_signal_length) { /* for each element in signal */
	real_el = GetRealElement(element);
	if (STIME_LESSTHAN(latest_time, qqLastActiveTime(real_el))) {
	    TIME_COPY(latest_time, qqLastActiveTime(real_el));
	}
        
        data_length = qqDataLength(element);
        index      += data_length;
        element    += data_length;
    } /* while */

    if (TIME_EQUAL(latest_time, EarlyTime)) {
					/* No activity yet */

	STIME_HIGH(*time_buffer);	/* return TIME'HIGH */
    }
    else {
	TIME_SUB(*time_buffer, SimTime, latest_time);
					/* return "NOW - <time of last change>"
					*/
    }
} /* VK_ReadLastActive */



/* *******************************************************************
*
* VK_ReadLastEvent - read 'Last_Event of signal
*
* HISTORY:
*	27-SEP-89 DSB	Created
*	03-MAY-90 DSB	Trying to implement LRM precisely
*	07-MAY-90 DSB	Implementing "Sense of the VASG" interpretation
*	28-SEP-90 AWH	Vector signals
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*	10-JUN-91 DSB	Applying 64-bit time
*
******************************************************************* */
void VK_ReadLastEvent(time_buffer, prefix_signal_handle, prefix_signal_offset,
	prefix_signal_length)
VK_tRelativeTimePtr time_buffer;	/* &out: result buffer address */
VK_tSignalHandle prefix_signal_handle;	/* in: prefix signal S handle */
VK_tSignalOffset prefix_signal_offset;	/* in: prefix signal S offset */
VK_tSignalLength prefix_signal_length;	/* in: prefix signal S length */
{
    tSignalElementHandle element,
                         real_el;
    int                  index;
    VK_tSignalLength     data_length;
    tRelTimeBuf          latest_time;	/* signed to accept EarlyTime */
/**/
    GetElementHandle(element,prefix_signal_handle,
                             prefix_signal_offset,prefix_signal_length);

    /* NOTE: LRM definition has errors.  
       LRM 14.1.S'Last_Event:
       Correct part:
       "... Result: The amount of time that has elapsed since the last event
       occurred on signal S."
       Incorrect part:
       "For a scalar signal S, S'LAST_EVENT returns the largest value T of
       type Time for which S'DELAYED(T)'STABLE would return TRUE, if such a
       value for T exists; otherwise it returns 0 ns."
     */

    TIME_COPY(latest_time, EarlyTime);
    index = 0;
    while (index < prefix_signal_length) {
					/* for each scalar element of signal */
	real_el = GetRealElement(element);
	if (STIME_LESSTHAN(latest_time, qqLastEventTime(real_el))) {
	    TIME_COPY(latest_time, qqLastEventTime(real_el));
	}

        data_length = qqDataLength(element);
        index    += data_length;
        element  += data_length;
    } /* while */

    if (TIME_EQUAL(latest_time, EarlyTime)) {
					/* No events yet */
	STIME_HIGH(*time_buffer);
    }
    else {
	TIME_SUB(*time_buffer, SimTime, latest_time);
    }
} /* VK_ReadLastEvent */



/* *******************************************************************
*
* VK_ReadLastValue - read value of signal'LAST_VALUE
*
* HISTORY:
*	27-SEP-90 DSB	Created     
*	28-SEP-90 AWH	Vector signals
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*
******************************************************************* */
void VK_ReadLastValue(ptr_addr, prefix_signal_handle, prefix_signal_offset, 
	prefix_signal_length)
VK_tAddrAddr ptr_addr;			/* &out: buffer pointer address */
VK_tSignalHandle prefix_signal_handle;	/* in: prefix signal S handle */
VK_tSignalOffset prefix_signal_offset;	/* in: prefix signal S offset */
VK_tSignalLength prefix_signal_length;	/* in: prefix signal S length */
{
    tSignalElementHandle element,
                         real_el;
/**/
    GetElementHandle(element,prefix_signal_handle,prefix_signal_offset,
                                                  prefix_signal_length);

    real_el = GetRealData(element,prefix_signal_length);

    if (NULL == real_el) {
        *ptr_addr = ReadRealElement(element,prefix_signal_length,TRUE);
    }
    else {
        *ptr_addr = (VK_tDataAddr)qqPreviousValuePtr(real_el);
    }

} /* VK_ReadLastValue */


/* *******************************************************************
*
* AllocateAddressEntry - Allocate entry in address table.
*
* HISTORY:
*	21-SEP-90 AWH	Created
*	01_APR-91 WAG	W0015.  Changed VerifyMalloc to AllocateMemory.
*			Changed VerifyRealloc to ReallocateMemory.
*	03-APR-91 WAG	W0023. Modified to conform to coding style standards.
*       29-OCT-91 DKC   Removed NULL check on addr.
*
******************************************************************* */
static tAddressIndex AllocateAddressEntry (tableP, countP, addr)
tAddressTable * tableP;		/* inout: address table */
tAddressIndex * countP;		/* inout: count of entries in table */	
tAddress addr;			/* in:    new address  */
{
#define AddressTablePageSize 4	/* Amount of each allocation */
   tAddressIndex count;
   tAddressTable table;
/**/
    count = *countP;
    table = *tableP;

    if (0 == count) {
					/* Increase Allocation By One Page */
        AllocateMemoryZero(table, tAddress, AddressTablePageSize);
					/* first time */
        count = 1;                      /* entry [0] is unused */
    }
    else {

        count++;

        if ( ! ( count % AddressTablePageSize ) ) {
					/* Check if current allocation is */
					/* full. */
					/* ?? why isn't there an inequality */
					/* operator here (e.g., some count > */
					/* some size)? */

	    ReallocateMemory(table, tAddress, count + AddressTablePageSize);
					/* current + additional page */ 
	} 
    } 

    table[count] = addr;		/* store address */
					/* Signal 1 stored in index 1 */
    *tableP = table;
    *countP = count;

    return *countP;			/* return results */

} /* AllocateAddressEntry */



/* *******************************************************************
*
* GetElementTypeInfo - get type information for a signal element
*
* HISTORY:
*	22-MAR-91 HK	Created
*	05-APR-91 DSB	Got element type for array (vector) signal
*	16-MAY-91 DSB	Cleaning up tSymbInfoKind enumerators
*	30-MAY-91 DSB	Applied gc_CompositeSignal
*	11-JUN-91 DSB	Added LargePhysicalTypeDeclInfo case
*	 6-NOV-91 DKC	Added Record Handling
*       14-NOV-91 DKC   Symbolic Info attached to element now
*
******************************************************************* */
tSymbInfoPtr GetElementTypeInfo(element)
					/* returns symb. info. handle */
tSignalElementHandle element;		/* signal element */
{
    tSymbInfoPtr element_type_info;
    VK_tInteger  dimensions;
/**/
    element_type_info = GetSignalSubtypeInfo(qqNumber(element),
                                             qqDataLength(element),
                                             &dimensions);

    return element_type_info;
} /* GetElementTypeInfo */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-JUN-90 DSB	Created
*
******************************************************************* */
<static> Fntype RtnName(parm) 	/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif


