#ifndef TKSim_H
#define TKSim_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKSim.h
*
* DESCRIPTION:  
*	Interface file for TKSim.c.  
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Created
*       03-OCT-90 AWH	Added vector signals.
*	03-APR-91 WAG	W0023.  Renamed all _Eval_* routines to Evaluate*;
*			renamed _Eval_Explicit to EvaluateExplicit.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	06-MAY-91 DSB	Cleaning up signal changes
*	14-MAY-91 DSB	W0053+: Renamed tLAR_*_{Rec,Ptr} to tProcData...; made
*			Evaluate* static (private to module); renamed 
*			SpecialCycle to NeverCycle
*	16-MAY-91 DSB	Converting to unsigned absolute time; reworked 
*			SpecialTime and AncientTime to NeverTime and
*			EarlyTime
*	21-MAY-91 DSB	Cleaned up old types; renamed to remove some VK_
*	11-OCT-91 MNP	Header fixup.
*	08-JAN-92 DKC	Moved context structures to TKstructs.h
*
* SCCS ID: @(#)TKSim.h	42.1 1/10/92
*
******************************************************************* */

#include "TKCommon.h"
#include "TKStructs.h"		/* for tSignalElementHandle */
#include "TKListFns.h"
#include "TKSignals.h"

typedef enum {
    	ss_NoStatus,
    	ss_Elaborating,
    	ss_Initializing,
    	ss_Simulating,
    	ss_Terminated
    } tSimulationStatus;		/* ?? document this ?? */


/* *******************************************************************
*
* External global variables, etc.
*
******************************************************************* */


#define NeverCycle -1			/* value that never matches current
					   simulation cycle number; used to
					   indicate that something has not
					   ever happened yet */

extern tAbsTimeBuf NeverTime;		/* value that never matches current
					   simulation time; used to disable
					   sensitivity to time, and to indi-
					   cate that there is no time as which
					   something happened */
					/* (logically a constant) */
extern tRelTimeBuf EarlyTime;		/* value that is less than any valid
					   simulation time (note: signed) */
					/* (logically a constant) */


extern VK_tProcessHandle CurrentProcess;	
					/* handle of currently-executing
					   process; valid only while 
					   executing */

extern TempList EventTimeQueue;		/* queue of times at which events were
					   scheduled (not queue of events yet)
					*/

extern VK_tInteger MaxSourceCount,      /* For resolved signals */
                   MaxResolvedData;

void InitSim();   /* Module initialization function */


/* *******************************************************************
*
*
*
******************************************************************* */

TempItem QueueEvent(
#ifdef ANSI_FN_PROTOS
	tAbsTimePtr abs_time_addr,
        tEventClass event_class,
        void        *event_data
#endif
	);

int ComputeNetDrivingValue(
#ifdef ANSI_FN_PROTOS
	tSignalElementHandle element
#endif
	);

void SetNetEffectiveValue(
#ifdef ANSI_FN_PROTOS
	tSignalElementHandle element
#endif
	);

void SetPaceTime(
#ifdef ANSI_FN_PROTOS
	VK_tInteger time
#endif
	);

void SimulateModel();

#endif /* TKSim_H */
