#ifndef TKStructs_H
#define TKStructs_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKStructs.h
*
* DESCRIPTION:  
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	04-OCT-89 DSB	Created
*	...
*	13-JUL-90 DSB	Added WaitFn, WaitContext
* 	11-Sep-90 RM	Created from TKData.h.
*	25-SEP-90 AWH	Divided VK_sSignal into element and group.
*       28-Sep-90 GJB   Renamed triggercycle to TimeoutTriggercycle and
*			updated macros
*	28-Sep-90 GJB   Added signaltriggercycle and added corresponding macros
*	18-MAR-91 HK	Added references to TIME macros.
*	20-MAR-91 WAG	Removed the priority field from the tProcessRec type;
*			replaced it with a boolean field called fKernelProcess
*			that differentiates between a kernel and a user process.
*			Also replaced the AddqqPriority, SetqqPriority, and
*			qqPriority macros with AddqqKernelProcess,
*			SetqqKernelProcess, and qqKernelProcess.
*	26-MAR-91 JBR	W0003.  Modified VK_sSymbInfo for numeric types.
*	28-MAR-91 HK	Removed fTypeSymbInfo field from tSignalGroupRec.
*	02-APR-91 JBR	W0013.  Added PhysicalTypeDeclInfo to tSymbInfoKind.
*			Modified VK_sSymbInfo for physical and array types.
*	04-APR-91 WAG	Added assignment and reference macros for fields in
*			TK_sRegion structure.
*	16-APR-91 WAG	W0033.  Added a new version of the SymbInfoRec structure
*			and macros to access the fields in it.
*	18-APR-91 WAG	W0027.  Added InterfaceSignalDeclInfo and
*			LocalSignalDeclInfo items to type tSymbInfoKind.
*	18-APR-91 WAG	W0025.  Added RecordTypeDeclInfo, RecordElementDeclInfo,
*			FileTypeDeclInfo, AccessTypeDeclInfo, and SubtypeDeclInfo
*			items to type tSymbInfoKind.
*	18-APR-91 WAG	W0026.  Added InstantiationStmtInfo,
*			InterfaceConstantDeclInfo, LocalConstantDeclInfo, 
*			InterfaceVariableDeclInfo, LocalVariableDeclInfo,
*			and LocalFileDeclInfo items to type tSymbInfoKind.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	30-APR-91 DSB	Removed duplicate XInfo and XDeclInfo enumerators
*	02-MAY-91 DSB	Cleaning up generally
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*	05-MAY-91 DSB	Cleaning signal/driver changes: GroupValue->SigGrpValue
*	14-MAY-91 DSB	W0053+: Added tProcessKind; removed qqExecFn; renamed
*			qqActRec to qqProcessData
*	16-MAY-91 DSB	Cleaning up tSymbInfoKind enumerators; finished re-
*			naming tSymbInfoType to -Kind (let's be consistent!)
*	16-MAY-91 DSB	Converting to unsigned absolute time
*	21-MAY-91 DSB	Cleaned up old types; renamed to remove some VK_
*	30-MAY-91 DSB	Added gc_CompositeSignal
*	10-JUN-91 DSB	Removed old NewWaveformXx macros (see FreeTimeBuffer, 
*			etc.)
*	11-JUN-91 DSB	Added tc_LargePhysical
*	14-AUG-91 DSB	Removed unneeded CopyPrivStr from AddqqName, etc.
*       29-OCT-91 DKC   Changed element and driver values from tTempDataType
*                       to tScalarUnion. Added qqElementClass etc.
*	02-DEC-91 DSB	Renamed VK_tDataSize to VK_tDataLength, renamed 
*			parallel identifiers
*	09-JAN-92 DKC	Packed data structures
*                       - packed fields are placed at head of struct decl.
*
* SCCS ID: @(#)TKStructs.h	42.1 1/10/92
*
******************************************************************* */

#include "VKI.h"		/* get all public definitions */
#include "TKCommon.h"		/* for  */
#include "TKData.h"		/* for tSimCycleCount, etc. */
#include "VKRegister.h"	/* for VK_tSymbInfoHandle */
#include "TKRegions.h"		/* for TK_tRegionHandle */

#include "TKData.h"		/* for TIME macros */
#include "TKTemp.h"             /* for AllocateTimeBuffer, etc. */
#include "TKListFns.h"


typedef enum {
	tc_BadTypeClass,
	tc_Enumeration,	        /* - an enumeration type */
	tc_Integer,		/* - an integer type */
	tc_Physical,		/* - a (32-bit) physical type */
	tc_LargePhysical,	/* - a large (64-bit) physical type */
	tc_Floating,		/* - a floating type */
	tc_Array,		/* - an array type */
	tc_Record,		/* - a record type */
	tc_Access,		/* - an access type */
        tc_Alias,               /* - alias of another element (indirection) */
	tc_File			/* - a file type */
    } tTypeClass;		/* class of base type */


typedef enum { 
	sc_BadSignalClass,
	sc_Explicit, 		/* - explicit (declared) signal */
	sc_Transaction, 	/* - 'Transaction implicit signal */
	sc_Quiet, 		/* - 'Quiet implicit signal */
	sc_Stable,  		/* - 'Stable implicit signal */
	sc_Delayed,  		/* - 'Delayed implicit signal */
	sc_Guard		/* - implicit GUARD signal */
    } tSignalClass;		/* which implicit signal */
/* ?? clean up: should this type remain here, or should it be elsewhere? ?? */

typedef enum {
	pc_BadProcessKind,
	pc_Explicit,		/* - explicit signal evaluation */
	pc_Transaction,         /* - implicit signal ...        */
	pc_Quiet,
	pc_Stable,
	pc_Guard,               /* - guard signal evaluation    */
	pc_Delayed,		/* - 'delayed evaluation        */
	pc_ProcessStatement     /* - user process               */
    } tProcessKind;

typedef enum {
        ev_BadEvent,
        ev_ProcessWakeUp,
        ev_DriverEvent,
        ev_EvalImplicit
    } tEventClass;


typedef struct VK_sAssoc * VK_tAssocHandle;

typedef enum {
	BadRecord, 
	SymbInfoRecord, 
	RegionRecord, TypeRecord,
	SignalGroupRecord, SignalElementRecord, 
	DriverGroupRecord, DriverElementRecord, 
	PortAssocRecord, 
	ProcessRecord, SensitivitySetRecord
    } tRecordKind;

typedef tRecordKind *tRecordKindPtr;

typedef enum {
	gc_BadGroupClass,
	gc_ScalarSignal,		/* -whole signal is scalar */
	gc_VectorSignal,		/* -whole signal is vector of scalar */
	gc_CompositeSignal,		/* -whole signal is other composite */
        gc_Alias                        /* group of aliassed elements */
    } tGroupClass;			/* special class of (whole) signal */
/* Note:  Scalar and vector signals are fully implemented (specifically, their
	display is supported.  Other composite signals are supported for 
	simulation, but their display is only partially supported.  */

/* clean up: add common substructure for common fields */



/* *******************************************************************
*
* type tSymbInfoKind -
*
* HISTORY:
*	15-JUL-91 DSB	Configuration: added ComponentConfigurationInfo
*	14-AUG-91 DSB	Added LibraryRegionInfo, LibrarySymbolInfo
*	25-NOV-91 MNP	Added LoopStmtInfo
*
******************************************************************* */
typedef enum {
	BadInfo,
	LibraryRegionInfo,
	PackageDeclUnitInfo, PackageBodyUnitInfo, 
	EntityDeclUnitInfo, ArchitectureBodyUnitInfo, 
	ConfigurationDeclUnitInfo,
	PackageDeclInfo, PackageBodyInfo, 
	EntityDeclInfo, ArchitectureBodyInfo, ConfigurationDeclInfo,
	BlockStmtInfo, IfGenerateStmtInfo, ForGenerateStmtInfo,
	ProcessStmtInfo, LoopStmtInfo,
	ProcedureDeclInfo, ProcedureBodyInfo, FunctionDeclInfo,
	FunctionBodyInfo,
	EnumerationTypeDeclInfo, IntegerTypeDeclInfo, FloatingTypeDeclInfo, 
	PhysicalTypeDeclInfo, LargePhysicalTypeDeclInfo,
	ArrayTypeDeclInfo, IndexSubtypeInfo, 
	RecordTypeDeclInfo, RecordElementDeclInfo,
	FileTypeDeclInfo, AccessTypeDeclInfo, 
	SubtypeDeclInfo,
	InterfaceConstantDeclInfo, LocalConstantDeclInfo, LoopParameterInfo,
	InterfaceVariableDeclInfo, LocalVariableDeclInfo, LocalFileDeclInfo,
	InterfaceSignalDeclInfo, LocalSignalDeclInfo,
	ComponentDeclInfo, InstantiationStmtInfo, 
	ConfigurationSpecInfo, ComponentConfigurationInfo,
	LibrarySymbolInfo,
	LastInfo
    } tSymbInfoKind;

/* ?? CLEAN UP: Change field names from "one_two" or "onetwo" style to
	"fOneTwo" style as long as we still use qqOneTwo macros.
?? */

/* typedef struct VK_sSymbInfo * VK_tSymbInfoHandle; */

/* *******************************************************************
*
* type tSymbInfoRec - 
*
* HISTORY:
*	04-MAR-91 DSB	Added fChildren field
*	26-MAR-91 JBR	W0003.  Added fIntegerLeftBound, fFloatLeftBound,
*			fDirection, fIntegerRightBound, and fFloatRightBound,
*			for numeric types.
*	02-APR-91 JBR	W0013.  Added fPositionValues for physical types.
*			Added fBaseTypeInfo, fConstraintRegionInfo,
*			fConstraintOffset,	fResFuncInfo.
*	11-APR-91 WAG	Added the following fields: fPackageName,
*			fSourceFileName, fStmtPos, fBeginPos, fEndPos,
*			fDeclPos, fCompDeclInfo, fExecTableSize,
*			fTraceTable, fBreakTable, fEnclItemCount,
*			fHasSensitivityList, fContainsWaitStmt,
*			fElaborationFunction, fHandleOffset, fOffsetOffset,
*			fIntfMode, signal_kind, fFileMode.  Renamed other
*			fields to conform to new naming standards.
*	12-APR-91 WAG	Re-ordered the fields; changed names of existing
*			fields back to their original values.
*	16-APR-91 WAG	Completely re-vamped this structure.
*	15-MAY-91 DSB	Added qqRegionData
*	04-JUN-91 JBR	W0019. Added qqTraceNum.
*	11-JUN-91 DSB	Added fLargeLeftBound, fLargeRightBound
*	23-JUL-91 DSB	Added fIntfDataSize, implmenented fAnalysisTime
*	24-JUL-91 DSB	Added fBindingFunction
*	14-AUG-91 DSB	Remove qqLibraryName (now use qqEnclReg, etc.)
*	15-AUG-91 DSB	Added qqLibraryInfo
*	09-OCT-91 DSB	Cleaned up some more field names (e.g., fSignalKind)
*
******************************************************************* */
typedef struct VK_sSymbInfo *tSymbInfoPtr;
                                 /* - replaces VK_tSymbolInfoHandle locally */
typedef struct VK_sSymbInfo {
	tRecordKind    fRecordKind:8;	/* - kind of record */
	tSymbInfoKind  fSymbInfoKind:8;	/* - kind of symbolic information 
					     record */
	VK_tSignalKind fSignalKind:8;	/* - signal kind */
	char           fDirection;	/* - direction of numeric range */

	VK_tInteger    fDimensions:8;	/* - number of dimensions */
	VK_tMode       fIntfMode:4;	/* - interface object mode */
	VK_tMode       fFileMode:4;	/* - file mode */
        VK_tBoolean    fIsUnconstrained:1;
                                        /* - for array type */
	VK_tBoolean    fContainsWaitStmt:1;
                                        /* - procedure contains wait stmt. */
	VK_tBoolean    fHasSensitivityList:1;
					/* - process has sensitivity list */

				/* Identification information: */
	tSymbInfoPtr   fExtReg;	        /* - extended region */
	TempList       fExtensions;	/* - records whose qqExtReg is this
					     record */

	tSymbInfoPtr   fEnclReg;   /* - enclosing region or other  parent */
	TempList       fChildren;  /* - records whose qqEnclReg is this
					record */

	VK_tEnclItemNumber fItemNum;	/* - item number in encl. region */
	VK_tName           fName;	/* - name of most items */

	VK_tCString fSourceFileName;	/* - source file */
	VK_tInteger fTraceNum;		/* - type/region/... trace file ID */

				/* elaboration/execution: */
	VK_tExecPosIndex    fExecTableSize; /* - size of tables */
	VK_tTraceTableEntry *fTraceTable;   /* ?? COMMENT? ?? */
	VK_tBreakTableEntry *fBreakTable;   /* ?? COMMENT? ?? */

				/* Execution position information: */
	VK_tExecPosIndex    fElabPos;   /* - decl./stmt. elaboration pos. */
	VK_tExecPosIndex    fBeginPos;  /* - region begin pos. */
	VK_tExecPosIndex    fEndPos;	/* - region end pos. */

				/* Region information: */
	VK_tEnclItemNumber fEnclItemCount; /* - enclosed items count */
                         	/* - type and subtype information */
	tSymbInfoPtr fBaseTypeInfo;     /* - base type of object or subtype */
	tSymbInfoPtr fConstraintRegionInfo;
					/* - constraint path indic. */
	VK_tFieldOffset fConstraintOffset;
					/* - constr. LAR field offset */
	tSymbInfoPtr fResFuncInfo;      /* - resolution function info */

				/* Item-specificic information: */
				    /* Object information: */
	VK_tFieldOffset fValueOffset;	/* - offset of value field */

	VK_tFieldOffset fHandleOffset;	/* - offset of signal handle field */
	VK_tFieldOffset fOffsetOffset;	/* - offset of signal offset field */

				    /* For numeric types: */
	VK_tLargeData fLargeLeftBound;	/* - left bound for large physical 
						type */
	VK_tLargeData fLargeRightBound;	/* - right bound for large physical 
						type */
                                        /* - high field is used for integer
                                             and low float types */

				    /* For enumeration and physical types: */
	VK_tArraySize        fCardinality;	
                                        /* - number of literals */
	VK_tStringArrayAddr  fLiterals;	/* - literals string table */
	VK_tIntegerArrayAddr fPositionValues;
					/* - unit position values table */


				    /* For array and record types: */
	VK_tEnclItemNumber fRecordElemCount;
					/* - number of record elements */

				    /* Miscellaneous: */
	VK_tFieldOffset fIntfDataOffset;
					/* - IAR field offset in encl. LAR */
	VK_tByteSize    fIntfDataSize;	/* - function or entity IAR size */

	VK_tGenericElabFnPtr fElaborationFunction;
					/* - architecture elab. fn. */
	VK_tBindFnPtr        fBindingFunction;	
                                        /* - entity default elab. fn. */
	VK_tGenericEvalFnPtr fEvaluationFunction;
					/* - function body evaluation fn. */

	VK_tTimeStampPtr fAnalysisTime; /* - architecture analysis time */
	tSymbInfoPtr     fCompDeclInfo; /* - comp. inst's. comp. decl. */
	tSymbInfoPtr     fLibraryInfo;  /* - library symbol's library */

				/* Data associated after registration: */
	VK_tLARHeaderPtr   fRegionData;	/* - local activation record of
				       	   package declaration or body */

    } tSymbInfoRec;


/* *******************************************************************
*
* type tRegionRec - 
*
* HISTORY:
*	...
*	21-MAY-91 DSB	Added this header
*	21-MAY-91 DSB	Renamed VK_sRegion to TK_sRegion, TK_tRegionHandle
*			to TK_tRegionHandle
*
******************************************************************* */
/* typedef struct TK_sRegion * TK_tRegionHandle; */
typedef struct TK_sRegion {
	tRecordKind fRecordKind:8;
	tRegionKind fRegionKind:8;	/* - type of this region */

	VK_tBoolean if_value:8;		/* - condition value (if generate) */
        char        for_range_dir;	/* - direction of generate range */

	VK_tName fName;			/* - region simple name */
	VK_tName fSecName;		/* - region secondary name */

	VK_tInteger      fNumber;			/* - */
	TK_tRegionHandle fParentRegion;	/* - containing region (if any) */
	TempList         fChildren;	/* - contained regions */

				/* data for if generate only: */
				/* data for for generate group only: */
	VK_tInteger    for_count;
	VK_tScalarData for_range_left;	/* - left value of generate range */
	VK_tScalarData for_range_right;	/* - right value of generate range */
	tTypeClass     for_type_class;	/* - range type class */
	tSymbInfoPtr   for_type_symb_info;	/* - special for enumeration type */
				/* data for for generate block only: */
	VK_tInteger      for_index_value;/* - index value for this block */
	TK_tRegionHandle for_group_data; /* - type information from group */
				/* : */
	
    } tRegionRec;


typedef struct VK_sType * tTypeHandle; 
typedef struct VK_sType {
	tRecordKind fRecordKind:8;

	tSymbInfoPtr     fSymbInfo;     /* - symbolic info. intf. data */

	VK_tName         fName;		/* - simple name of type */
	VK_tInteger      fNumber;	/* - */
	TK_tRegionHandle fParentRegion;	/* containing region (if any) */

	VK_tInteger      fCardinality;	/* - number of enumeration literals */
	VK_tName         *fLiterals;	/* - literal array address */
    } tTypeRec;



/*****************************************************************************
*
* type tArrayDescriptor
*
* HISTORY:
*
*****************************************************************************/

typedef struct sArrayDescriptor {
        short                   index,
                                indices;  
        VK_tIndexConstraintAddr constraint;
    } tArrayDescriptor;


/*****************************************************************************
*
* type tNameSegment - for creating a name tree for elements in signals
*
* HISTORY:
*	...
*	02-DEC-91 DSB	Renamed ...DataSize to ...DataLength to follow
*			VK_tDataLength
*
*****************************************************************************/
typedef struct sNameSegment {
        struct sNameSegment     *fSibling,
                                *fChildren;
        VK_tDataLength          fDataLength;   /* words for sub elements */
        VK_tCString             fName;
	tArrayDescriptor        *array_desc; 

    }  tNameSegment;

typedef tNameSegment * tNameSegmentHandle;

typedef VK_tInteger tSignalNumber;	/* (global) scalar signal number */
typedef VK_tInteger tDriverNumber;	/* local (to process) driver number */

/*****************************************************************************
*
* Type Conversion Fn. calling data - see VC_ConvertSignal
*
*************************************************************************** */

typedef struct sConvFnData {

        VK_tAnyTypeConvFnPtr    fConvFn;   /* must be first */
        VK_tContextPtr          fContext;
        VK_tLARHeaderPtr        fCaller;
        VK_tInteger             fDataSize; /* of IARP */
	VK_tTypeGroup           fInType;
	VK_tArraySize           fInDimensions;
	VK_tIndexConstraintAddr fInConstraint;
	VK_tTypeGroup           fOutType;
	VK_tArraySize           fOutDimensions;
	VK_tIndexConstraintAddr fOutConstraint;

    } tConvFnData, *tConvFnDataPtr;

/* *******************************************************************
*
* type tPortAssocReadRec - inserted in a parent element driver list
*                        - also used for parameter aliasing
*
* HISTORY:
*	22-NOV-91 DKC Created
*
******************************************************************* */

typedef struct sPortAssoc {
	tRecordKind          fRecordKind:8;	/* - */
        char                 fSimplePort,
                             fPortDriven;       /* from below*/
        VK_tBoolean          fComputed     :1,
                             fTidied       :1,
                             fSkipRegister :1;
        tSimCycleCount       fLastActiveCycle;  /* cycle driving up */
        VK_tInteger          fActiveCount;      /* count of active elements */
        tSignalNumber        fSignalUpStart;
        VK_tSignalLength     fSignalUpLength;
        tSignalNumber        fSignalDownStart;
        VK_tSignalLength     fSignalDownLength;
        /* type conversion info */
        tConvFnDataPtr       fConvInData;
        tConvFnDataPtr       fConvOutData;
        /* resolved signal buffering */
        VK_tDataAddr         fCurrentValue,    /* as for driver     */
	                     fCurrentValuePtr; /*    group/element  */
        /* alias driver info */
	tDriverNumber        fDriverStart;	
        VK_tInteger          fUsed;

    } tPortAssocRec, *tPortAssocHandle, **tPortMap;

/*****************************************************************************
*
* Prefix Signal Data - used for implicit signals
*
*****************************************************************************/

typedef struct sPrefixData {
        tSignalNumber    fPrefixSignalStart;
        VK_tSignalLength fPrefixSignalLength;
	tRelTimeBuf      fTime;               /* - time for 'Delayed etc. */

    } tPrefixData, *tPrefixDataHandle;

typedef struct sSignalElement * tSignalElementHandle; 

typedef struct sResolutionData {
	VK_tAnyResFnPtr  fResEvalFn;	/* - resolution evaluation function */
	VK_tLARHeaderPtr fResolutionContext;
        VK_tTypeGroup    fGroupType:8;
        char             fDirection;
        VK_tInteger      fLeftBound;
        

    } tResolutionDataRec, *tResolutionDataPtr;


/* *******************************************************************
*
* type tSignalGroupRec - 
*
* HISTORY:
*	...
*	16-MAY-91 DSB	Added this header
*	16-MAY-91 DSB	Renamed qqTypeClass to qqElemTypeClass
*	 6-NOV-91 DKC	Added fRootSignal for two level grouping
*                       and removed fElemTypeClass
*	 6-DEC-91 DKC	Added Elements field (see CreateSignalGroup)
*
******************************************************************* */
typedef struct VK_sSignal {
	tRecordKind    fRecordKind :8;	/* - */
				/* characteristics data */
	VK_tSignalKind fSignalKind :8;	/* - unguarded/bus/reg. */
        tGroupClass    fGroupClass :4;  /* - whole signal type class (scalar,
    					     vector, other composite) */
	tSignalClass   fSignalClass:4;	/* - explicit/implicit/guard */
        VK_tBoolean    fNoted      :1,    /* - attached to region            */
                       fSimpleType :1,    /* - bit/bool type                 */
                       fIsResolved :1,    /* - all or partly resolved type   */
                       fIsMixed    :1,    /* - contains mixed physical types */
                       fIsPort     :1,    /* - is formal                     */
	               fTraceActive:1,
	               fTraceEvent :1;

        VK_tInteger fFormatId;  /* - formatter global signal id. */
            
                                /* symbolic data */
	tSymbInfoPtr     fSymbInfo;   /* - symbolic info. intf. data */
	VK_tLARHeaderPtr fRegionData;

				/* name data */
	tNameSegmentHandle fChildren;

                                /* parent groups */
        tPortMap           fPortMap;      /* - port association */
        VK_tSignalHandle   fRootSignal;   /* - root in nested structure */

				/* element(s) of group*/
	tSignalElementHandle fElements;  /* - array of elements */
        tSignalNumber      fSignalStart; /* - first signal element in group */
	VK_tSignalLength   fSignalLength;/* - signal group length */

				/* implicit signal identity */
        tPrefixDataHandle  fImplicitData;
					/* - parent of implicit 'Stable, etc.*/

				/* related implicit signals */
	TempList fImplicitSignals;	/* - derived 'Stable, etc. */

                                /* resolution */
        VK_tInteger        fActiveCount;  /* count of active elements */
        tResolutionDataPtr fResolutionData;

                   	/* miscellaneous */
	VK_tDataAddr fSignalBuffer;	/* - address of reused signal buffer 
                                             data for effective,driving
                                             default and 'LAST_VALUE values  */

        /* this record will be allocated space with an extension to
           hold the elements array and the signal-buffer */

    } tSignalGroupRec;			/* data for whole signal */


/* *******************************************************************
*
* type tSignalElementRec - 
*
* HISTORY:
*	...
*	15-MAY-91 DSB	Removed qqResCallerFn, added qqResEvalFn
*	16-MAY-91 DSB	Added this header
*	08-OCT-91 DSB	Added qqDiscTime
*	 6-NOV-91 DKC	Added fFormatId
*	 6-DEC-91 DKC	Moved implicit data to seperate structure held by,
*                       group. Values are now pointed to.
*
******************************************************************* */
typedef struct sSignalElement {
	tRecordKind fRecordKind   :8;	/* - */
        char        fDataLength;
        VK_tBoolean fResolved     :1,   /* - element is resolved type */
                    fGroupResolved:1,   /* - complex resolution       */
                    fMapped       :1,   /* - is port                  */
                    fEvaluating   :1,   /* - marked for evaluation    */
                    fDriven       :1,   /* - any drivers at own level */
                    fDrivingPorts :1,   /* - driven from below        */
                    fTidied       :1,   /* - net has been tidied      */
                    fIsGuard      :1,   /* - element is guard signal  */
                    fIsImplicit   :1,   /* - element is implicit sig. */
                    fComplexNet   :1;   /* - resolution or type conv. 
                                             at outer level           */

        tTypeClass  fElementClass:8; /* - class of element values */

				/* name data */
	tSignalNumber    fNumber;    /* - kernel global signal identifier */
        VK_tSignalHandle fGroup;     /* - refrence to signal group */

				/* hierarchy data */

				/* evaluation data */
	TempList          fSourceList;	  /* - drivers and associattions */
	VK_tProcessHandle fEvalProcess;	  /* - evaluation process data  */

	tRelTimeBuf fDiscTime;		/* - disconnection time for guarded
						signal element */

				/* sensitivity data */
	TempList triggeractive;		/* - activity-sensitive processes */
	TempList triggerevent;		/* - event-sensitive processes */
	TempList triggereventsets;	/* - event-sensitive sens. sets */

                                /* values */
	tScalarUnionPtr  fDefaultValuePtr;   /* - default value   */
	tScalarUnionPtr  fDrivingValuePtr;   /* - driving value   */
	tScalarUnionPtr  fEffectiveValuePtr; /* - effective value */
	tScalarUnionPtr  fPreviousValuePtr;  /* - for 'LastValue  */

				/* history data */
	tAbsTimeBuf      fLastActiveTime;    /* - time of last activity */
	tSimCycleCount   fLastActiveCycle;   /* - cycle of last activity */
	tAbsTimeBuf      fLastEventTime;     /* - time of last event */
	tSimCycleCount   fLastEventCycle;    /* - cycle of last event */

    } tSignalElementRec;

typedef tSignalElementHandle * tSignalNumberTable; 
					/* associates scalar signal numbers 
					   with addresses of signal records */


/* *******************************************************************
*
* type tDriverGroupRec - 
*
* HISTORY:
*	28-JUN-91 DSB	Added this header
*	 6-DEC-91 DKC	Group holds signal buffer - to aid complex
*                       resolution.
*
******************************************************************* */

typedef struct VK_sDriver {
	tRecordKind           fRecordKind:8;   /* - */
	tDriverNumber         fDriverStart;  /* (test kernel implementation) */
        VK_tSignalLength      fDriverLength; /* of driver group              */
        /* for alias */
        tSignalNumber         fSignalStart;  /* start of driven slice        */

    } tDriverGroupRec, *tDriverHandle;	/* ?? FOR driver group record */


/* *******************************************************************
*
* type tDriverElementRec - 
*
* HISTORY:
*	28-JUN-91 DSB	Added this header
*	28-JUN-91 DSB	Renamed qqWaveformValues to qqWaveformValuess, 
*			waveform_value to fWaveformValues; also ...Times;
*			renamed other fields to fXxYy style
*	 6-DEC-91 DKC	Changed ParentSignal to ParentElement and 
*                       Value to pointer
*
******************************************************************* */

typedef struct sDriverElement {
	tRecordKind       fRecordKind:8;
	tDriverNumber     fNumber;	    /* -  */
        TempItem          fScheduled;
        tSignalNumber     fParentElement;
	VK_tProcessHandle fOwnerProcess;    /* - driving process handle    */
	VK_tBoolean       fDriverNull;	    /* - current value is "null"   */
        VK_tSignalLength  fDataLength;
	VK_tDataAddr      fCurrentValue;    /* buffer if driving resolved  */
	VK_tDataAddr      fCurrentValuePtr; /* - current value of driver   */
	tSimCycleCount    fLastActiveCycle; /* - time of last activity     */

	/* - future values waveform */
	TempList          fWaveformValues;
	TempList          fWaveformTimes;

    } tDriverElementRec, *tDriverElementHandle, **tDriverTable;


/* *******************************************************************
*
* type tSensitivitySetRec - 
*
* HISTORY:
*	28-JUN-91 DSB	Added this header
*
******************************************************************* */

/* typedef struct VK_sSensitivitySet * VK_tSensitivitySetHandle; */
typedef struct VK_sSensitivitySet {
	tRecordKind fRecordKind:8;
	VK_tBoolean fEnabled:8;		/* - */
	VK_tInteger fNumber;		/* - required ?? */

	VK_tProcessHandle sensitive_process;	
					/* - process sensitive to signals in
					   this sensitivity set */
	TempList fSensitivityList;	/* - signals in this sensitivity set */
    } tSensitivitySetRec;


	
/*****************************************************************************
*
* Event Record
*
*****************************************************************************/

typedef struct sEventRec {
        tAbsTimeBuf fTime;             /* - time of event  */
        tEventClass fEventClass;       /* - type of event  */
        void        *fData;            /* - data for event */
        
    } tEventRec, *tEventHandle;


/* *******************************************************************
*
* Context records for internal kernel processes maintaining signal values
*
******************************************************************* */

typedef struct {
	VK_tSignalHandle fImplicitSignalHandle;

    } tProcData_Implicit_Rec,  *tProcData_Implicit_Ptr;


typedef struct {
	tProcData_Implicit_Rec fCommonData;
	
	VK_tSignalOffset fImplicitSignalOffset;
	VK_tDriverHandle delayed_driver;
	VK_tDriverOffset delayed_driver_offset;

    } tProcData_Delayed_Rec,  *tProcData_Delayed_Ptr;


typedef struct {
	tProcData_Implicit_Rec fCommonData;

	VK_tGuardEvalFnPtr evaluation_function; /* gen. code's  fn. */
	VK_tContextPtr     evaluation_context;  /* eval. context    */

    } tProcData_Guard_Rec,  *tProcData_Guard_Ptr;


/* *******************************************************************
*
* type tProcessRec - 
*
* HISTORY:
*	28-JUN-91 DSB	Added this header
*
******************************************************************* */

/* typedef struct VK_sProcess * VK_tProcessHandle; */
typedef struct VK_sProcess {
	tRecordKind  fRecordKind   :8;
	VK_tBoolean  fTimedOut     :8,  /* - wait expired                    */
	             fActivated    :8;  /* - activated (in current cycle)    */
	tProcessKind fProcessKind  :4;	/* - */
	VK_tBoolean  fKernelProcess:1,	/* - type of process: kernel or user */
                     fDebugBit     :1;	/* - single-step anywhere in process */

	VK_tInteger      fNumber;	/* - */
	TK_tRegionHandle fParentRegion;	/* - containing region */

        tDriverTable fDriverTable;      /* - associated drivers */
        VK_tInteger  fDriverCount;

        TempItem       fScheduled;      /* - wake-up event                */ 
        tSimCycleCount fTriggerCycle;   /* - resumption flag for timeouts */
	VK_tPCBP       fProcessData;	/* - process data (two types)     */

	TempList fSensitivityList;	/* - other trigger signals   */
	VK_tSensitivitySetHandle fCurrentSensitivitySet;
					/* - current sensitivity set */
        /* user process data */
	VK_tName          fName;	/* - simple name of process */
	VK_tWaitEvalFnPtr fWaitFn;
	VK_tContextPtr    fWaitContext;

    } tProcessRec;

/* *************** MACROS *************** */

/* list node access */

#define RegValue(item)       ((TK_tRegionHandle)    (TempValue(item)))
#define SymbInfoValue(item)  ((VK_tSymbInfoHandle)  (TempValue(item)))
#define TypeValue(item)      ((tTypeHandle)         (TempValue(item)))

#define SigGrpValue(item)    ((VK_tSignalHandle)    (TempValue(item)))
#define SigElemValue(item)   ((tSignalElementHandle)(TempValue(item)))
#define DrvGrpValue(item)    ((VK_tDriverHandle)    (TempValue(item)))
#define DrvElemValue(item)   ((tDriverElementHandle)(TempValue(item)))
#define PortAssocValue(item) ((tPortAssocHandle)    (TempValue(item)))
#define EventValue(item)     ((tEventHandle)        (TempValue(item)))
#define TimeValuePtr(item)   ((tAbsTimePtr)         (TempValue(item)))
#define ResDataValue(item)   ((tResolutionDataPtr)  (TempValue(item)))

#define ProcValue(item)      ((VK_tProcessHandle)   (TempValue(item)))
#define SensValue(item)      ((VK_tSensitivitySetHandle)(TempValue(item)))

/* field access macros - alaphabetically (please) */

#define AddqqActivated SetqqActivated
#define SetqqActivated(N, V) ((N)->fActivated = V)
#define qqActivated(N)       ((N)->fActivated)

#define AddqqActiveCount SetqqActiveCount
#define SetqqActiveCount(N, V) ((N)->fActiveCount = V)
#define qqActiveCount(N)       ((N)->fActiveCount)

#define AddqqActualPort SetqqActualPort
#define SetqqActualPort(N, V) ((N)->fActualPort = V)
#define qqActualPort(N)	      ((N)->fActualPort)

#define AddqqAnalysisTime SetqqAnalysisTime
#define SetqqAnalysisTime(N, V) ((N)->fAnalysisTime = V)
#define qqAnalysisTime(N)	((N)->fAnalysisTime)

#define AddqqBaseTypeInfo SetqqBaseTypeInfo
#define SetqqBaseTypeInfo(N, V) ((N)->fBaseTypeInfo = (V))
#define qqBaseTypeInfo(N)       ((N)->fBaseTypeInfo)

#define AddqqBeginPos SetqqBeginPos
#define SetqqBeginPos(N, V) ((N)->fBeginPos = (V))
#define qqBeginPos(N)       ((N)->fBeginPos)

#define AddqqBreakTable SetqqBreakTable
#define SetqqBreakTable(N, V) ((N)->fBreakTable = (V))
#define qqBreakTable(N)       ((N)->fBreakTable)

#define AddqqBindingFunction SetqqBindingFunction
#define SetqqBindingFunction(N, V) ((N)->fBindingFunction = V)
#define qqBindingFunction(N)	   ((N)->fBindingFunction)

#define AddqqBufferCount SetqqBufferCount
#define SetqqBufferCount(N, V) ((N)->fBufferCount = (V))
#define qqBufferCount(N)       ((N)->fBufferCount)

#define AddqqCaller SetqqCaller
#define SetqqCaller(N, V) ((N)->fCaller = V)
#define qqCaller(N)	  ((N)->fCaller)

#define AddqqCardinality SetqqCardinality
#define SetqqCardinality(N, V) ((N)->fCardinality = V)
#define qqCardinality(N)       ((N)->fCardinality)

#define AddqqChildren SetqqChildren
#define SetqqChildren(N, V) ((N)->fChildren = V)
#define qqChildren(N)	    ((N)->fChildren)

#define AddqqCompDeclInfo SetqqCompDeclInfo
#define SetqqCompDeclInfo(N, V) ((N)->fCompDeclInfo = V)
#define qqCompDeclInfo(N)	((N)->fCompDeclInfo)

#define AddqqCommonData SetqqCommonData
#define SetqqCommonData(N, V) ((N)->fCommonData = V)
#define qqCommonData(N)	      ((N)->fCommonData)

#define AddqqComputed SetqqComputed
#define SetqqComputed(N, V) ((N)->fComputed = V)
#define qqComputed(N)	    ((N)->fComputed)

#define AddqqConstraintOffset SetqqConstraintOffset
#define SetqqConstraintOffset(N, V) ((N)->fConstraintOffset = V)
#define qqConstraintOffset(N)	    ((N)->fConstraintOffset)

#define AddqqConstraintRegionInfo SetqqConstraintRegionInfo
#define SetqqConstraintRegionInfo(N, V) ((N)->fConstraintRegionInfo = V)
#define qqConstraintRegionInfo(N)	((N)->fConstraintRegionInfo)

#define AddqqContainsWaitStmt SetqqContainsWaitStmt
#define SetqqContainsWaitStmt(N, V) ((N)->fContainsWaitStmt = V)
#define qqContainsWaitStmt(N)	    ((N)->fContainsWaitStmt)

#define AddqqCurrentSensSet SetqqCurrentSensSet
#define SetqqCurrentSensSet(N, V)  ((N)->fCurrentSensitivitySet = V)
#define qqCurrentSensSet(N)        ((N)->fCurrentSensitivitySet)

#define AddqqCurrentValue SetqqCurrentValue
#define SetqqCurrentValue(N, V) ((N)->fCurrentValue = V)
#define qqCurrentValue(N)	((N)->fCurrentValue)

#define AddqqCurrentValuePtr SetqqCurrentValuePtr
#define SetqqCurrentValuePtr(N, V) ((N)->fCurrentValuePtr = V)
#define qqCurrentValuePtr(N)	   ((N)->fCurrentValuePtr)

#define AddqqChildren SetqqChildren
#define SetqqChildren(N, V) ((N)->fChildren = V)
#define qqChildren(N)	    ((N)->fChildren)

#define AddqqConvFn SetqqConvFn
#define SetqqConvFn(N, V) ((N)->fConvFn = V)
#define qqConvFn(N)       ((N)->fConvFn)

#define AddqqComplexNet SetqqComplexNet
#define SetqqComplexNet(N, V) ((N)->fComplexNet = V)
#define qqComplexNet(N)       ((N)->fComplexNet)

#define AddqqConvInData SetqqConvInData
#define SetqqConvInData(N, V) ((N)->fConvInData = V)
#define qqConvInData(N)       ((N)->fConvInData)

#define AddqqConvOutData SetqqConvOutData
#define SetqqConvOutData(N, V) ((N)->fConvOutData = V)
#define qqConvOutData(N)       ((N)->fConvOutData)

#define AddqqContext SetqqContext
#define SetqqContext(N, V) ((N)->fContext = V)
#define qqContext(N)	   ((N)->fContext)

#define AddqqDataSize SetqqDataSize
#define SetqqDataSize(N, V) ((N)->fDataSize = V)
#define qqDataSize(N) ((N)->fDataSize)

#define AddqqData SetqqData
#define SetqqData(N, V) ((N)->fData = V)
#define qqData(N)       ((N)->fData)

#define AddqqDataLength SetqqDataLength
#define SetqqDataLength(N, V) ((N)->fDataLength = V)
#define qqDataLength(N)       ((N)->fDataLength)

#define AddqqDriverLength SetqqDriverLength
#define SetqqDriverLength(N, V) ((N)->fDriverLength = V)
#define qqDriverLength(N)	((N)->fDriverLength)

#define AddqqDefaultValuePtr SetqqDefaultValuePtr
#define SetqqDefaultValuePtr(N, V) ((N)->fDefaultValuePtr = V)
#define qqDefaultValuePtr(N)	   ((N)->fDefaultValuePtr)

#define AddqqDataSize SetqqDataSize
#define SetqqDataSize(N, V) ((N)->fDataSize = V)
#define qqDataSize(N)	    ((N)->fDataSize)

#define AddqqDimensions SetqqDimensions
#define SetqqDimensions(N, V) ((N)->fDimensions = V)
#define qqDimensions(N)       ((N)->fDimensions)

#define AddqqDirection SetqqDirection
#define SetqqDirection(N, V) ((N)->fDirection = V)
#define qqDirection(N)       ((N)->fDirection)

#define AddqqDiscTime SetqqDiscTime
#define SetqqDiscTime(N, V) TIME_COPY((N)->fDiscTime, V)
#define qqDiscTime(N)       ((N)->fDiscTime)

#define AddqqDisplay SetqqDisplay
#define SetqqDisplay(N, V) ((N)->display = V)
#define qqDisplay(N)       ((N)->display)

#define AddqqDriven SetqqDriven
#define SetqqDriven(N, V) ((N)->fDriven = V)
#define qqDriven(N)	  ((N)->fDriven)

#define AddqqDriverCount SetqqDriverCount
#define SetqqDriverCount(N, V) ((N)->fDriverCount = V)
#define qqDriverCount(N)       ((N)->fDriverCount)

#define AddqqDriverStart SetqqDriverStart
#define SetqqDriverStart(N, V) ((N)->fDriverStart = V)
#define qqDriverStart(N)       ((N)->fDriverStart)

#define AddqqDriverNull SetqqDriverNull
#define SetqqDriverNull(N, V) ((N)->fDriverNull = V)
#define qqDriverNull(N)	((N)->fDriverNull)

#define AddqqDrivingPorts SetqqDrivingPorts
#define SetqqDrivingPorts(N, V) ((N)->fDrivingPorts = V)
#define qqDrivingPorts(N)	((N)->fDrivingPorts)

#define AddqqDriverTable SetqqDriverTable
#define SetqqDriverTable(N, V) ((N)->fDriverTable = V)
#define qqDriverTable(N)       ((N)->fDriverTable)

#define AddqqDrivingValuePtr SetqqDrivingValuePtr
#define SetqqDrivingValuePtr(N, V) ((N)->fDrivingValuePtr = V)
#define qqDrivingValuePtr(N)	((N)->fDrivingValuePtr)

#define AddqqEffectiveValuePtr SetqqEffectiveValuePtr
#define SetqqEffectiveValuePtr(N, V) ((N)->fEffectiveValuePtr = V)
#define qqEffectiveValuePtr(N)	((N)->fEffectiveValuePtr)

#define AddqqElaborationFunction SetqqElaborationFunction
#define SetqqElaborationFunction(N, V) ((N)->fElaborationFunction = V)
#define qqElaborationFunction(N)       ((N)->fElaborationFunction)

#define AddqqElabPos SetqqElabPos
#define SetqqElabPos(N, V) ((N)->fElabPos = V)
#define qqElabPos(N)	   ((N)->fElabPos)

#define AddqqElementClass SetqqElementClass
#define SetqqElementClass(N, V) ((N)->fElementClass = V)
#define qqElementClass(N)       ((N)->fElementClass)

#define AddqqElements SetqqElements
#define SetqqElements(N, V) ((N)->fElements = V)
#define qqElements(N)       ((N)->fElements)

#define AddqqEnabled SetqqEnabled
#define SetqqEnabled(N, V) ((N)->fEnabled = V)
#define qqEnabled(N)       ((N)->fEnabled)

#define AddqqEnclItemCount SetqqEnclItemCount
#define SetqqEnclItemCount(N, V) ((N)->fEnclItemCount = V)
#define qqEnclItemCount(N)	 ((N)->fEnclItemCount)

#define AddqqEnclReg SetqqEnclReg
#define SetqqEnclReg(N, V) ((N)->fEnclReg = V)
#define qqEnclReg(N)	((N)->fEnclReg)

#define AddqqEndPos SetqqEndPos
#define SetqqEndPos(N, V) ((N)->fEndPos = V)
#define qqEndPos(N)	((N)->fEndPos)

#define AddqqEvalProcess SetqqEvalProcess
#define SetqqEvalProcess(N, V) ((N)->fEvalProcess = V)
#define qqEvalProcess(N) ((N)->fEvalProcess)

#define AddqqEvaluating SetqqEvaluating
#define SetqqEvaluating(N, V) ((N)->fEvaluating = V)
#define qqEvaluating(N) ((N)->fEvaluating)

#define AddqqEvaluationFunction SetqqEvaluationFunction
#define SetqqEvaluationFunction(N, V) ((N)->fEvaluationFunction = V)
#define qqEvaluationFunction(N) ((N)->fEvaluationFunction)

#define AddqqEventClass SetqqEventClass
#define SetqqEventClass(N, V) ((N)->fEventClass = V)
#define qqEventClass(N)       ((N)->fEventClass)

#define AddqqExecTableSize SetqqExecTableSize
#define SetqqExecTableSize(N, V) ((N)->fExecTableSize = V)
#define qqExecTableSize(N) ((N)->fExecTableSize)

#define AddqqExtensions SetqqExtensions
#define SetqqExtensions(N, V) ((N)->fExtensions = V)
#define qqExtensions(N)	((N)->fExtensions)

#define AddqqExtReg SetqqExtReg
#define SetqqExtReg(N, V) ((N)->fExtReg = V)
#define qqExtReg(N)	((N)->fExtReg)

#define AddqqFileMode SetqqFileMode
#define SetqqFileMode(N, V) ((N)->fFileMode = V)
#define qqFileMode(N) ((N)->fFileMode)

#define AddqqFloatLeftBound SetqqFloatLeftBound
#define SetqqFloatLeftBound(N, V)\
                            ((N)->fLargeLeftBound.low = *(VK_tInteger *)&V)
#define qqFloatLeftBound(N) (*(VK_tFloatingData *)&(N)->fLargeLeftBound.low)

#define AddqqFloatRightBound SetqqFloatRightBound
#define SetqqFloatRightBound(N, V)\
                             ((N)->fLargeRightBound.low = *(VK_tInteger *)&V)
#define qqFloatRightBound(N) (*(VK_tFloatingData *)&(N)->fLargeRightBound.low)

#define AddqqFormalPort SetqqFormalPort
#define SetqqFormalPort(N, V) ((N)->fFormalPort = V)
#define qqFormalPort(N)	((N)->fFormalPort)

#define AddqqForCount SetqqForCount
#define SetqqForCount(N, V) ((N)->for_count = V)
#define qqForCount(N) ((N)->for_count)

#define AddqqForRangeLeft SetqqForRangeLeft
#define SetqqForRangeLeft(N, V) ((N)->for_range_left = V)
#define qqForRangeLeft(N) ((N)->for_range_left)

#define AddqqForRangeDir SetqqForRangeDir
#define SetqqForRangeDir(N, V) ((N)->for_range_dir = V)
#define qqForRangeDir(N) ((N)->for_range_dir)

#define AddqqForRangeRight SetqqForRangeRight
#define SetqqForRangeRight(N, V) ((N)->for_range_right = V)
#define qqForRangeRight(N) ((N)->for_range_right)

#define AddqqForTypeClass SetqqForTypeClass
#define SetqqForTypeClass(N, V) ((N)->for_type_class = V)
#define qqForTypeClass(N) ((N)->for_type_class)

#define AddqqForTypeSymbInfo SetqqForTypeSymbInfo
#define SetqqForTypeSymbInfo(N, V) ((N)->for_type_symb_info = V)
#define qqForTypeSymbInfo(N) ((N)->for_type_symb_info)

#define AddqqForIndexValue SetqqForIndexValue
#define SetqqForIndexValue(N, V) ((N)->for_index_value = V)
#define qqForIndexValue(N) ((N)->for_index_value)

#define AddqqForGroupData SetqqForGroupData
#define SetqqForGroupData(N, V) ((N)->for_group_data = V)
#define qqForGroupData(N) ((N)->for_group_data)

#define AddqqFormatId SetqqFormatId
#define SetqqFormatId(N, V) ((N)->fFormatId = V)
#define qqFormatId(N) ((N)->fFormatId)

#define AddqqGroup SetqqGroup
#define SetqqGroup(N, V) ((N)->fGroup = V)
#define qqGroup(N) ((N)->fGroup)

#define AddqqGroupResolved SetqqGroupResolved
#define SetqqGroupResolved(N, V) ((N)->fGroupResolved = V)
#define qqGroupResolved(N) ((N)->fGroupResolved)

#define AddqqGroupClass SetqqGroupClass
#define SetqqGroupClass(N, V) ((N)->fGroupClass = V)
#define qqGroupClass(N) ((N)->fGroupClass)

#define AddqqGroupType SetqqGroupType
#define SetqqGroupType(N, V) ((N)->fGroupType = V)
#define qqGroupType(N) ((N)->fGroupType)

#define AddqqHandleOffset SetqqHandleOffset
#define SetqqHandleOffset(N, V) ((N)->fHandleOffset = (V))
#define qqHandleOffset(N) ((N)->fHandleOffset)

#define AddqqHasSensitivityList SetqqHasSensitivityList
#define SetqqHasSensitivityList(N, V) ((N)->fHasSensitivityList = V)
#define qqHasSensitivityList(N)       ((N)->fHasSensitivityList)

#define AddqqImplicitData SetqqImplicitData
#define SetqqImplicitData(N, V) ((N)->fImplicitData = V)
#define qqImplicitData(N)	((N)->fImplicitData)

#define AddqqImplicitSignalHandle SetqqImplicitSignalHandle
#define SetqqImplicitSignalHandle(N, V)\
            (((tProcData_Implicit_Ptr)(N))->fImplicitSignalHandle = V)
#define qqImplicitSignalHandle(N)\
            (((tProcData_Implicit_Ptr)(N))->fImplicitSignalHandle)

#define AddqqImplicitSignalOffset SetqqImplicitSignalOffset
#define SetqqImplicitSignalOffset(N, V) ((N)->fImplicitSignalOffset = V)
#define qqImplicitSignalOffset(N)       ((N)->fImplicitSignalOffset)
	
#define AddqqImplicitSignals SetqqImplicitSignals
#define SetqqImplicitSignals(N, V) ((N)->fImplicitSignals = V)
#define qqImplicitSignals(N)	   ((N)->fImplicitSignals)
	
#define AddqqInType SetqqInType
#define SetqqInType(N, V) ((N)->fInType = V)
#define qqInType(N)	((N)->fInType)

#define AddqqInConstraint SetqqInConstraint
#define SetqqInConstraint(N, V) ((N)->fInConstraint = V)
#define qqInConstraint(N)	((N)->fInConstraint)

#define AddqqInDimensions SetqqInDimensions
#define SetqqInDimensions(N, V) ((N)->fInDimensions = V)
#define qqInDimensions(N)	((N)->fInDimensions)

#define AddqqIndexValue SetqqIndexValue
#define SetqqIndexValue(N, V) ((N)->for_index_value = V)
#define qqIndexValue(N)	((N)->for_index_value)

#define AddqqIndexDirection SetqqIndexDirection
#define SetqqIndexDirection(N, V) ((N)->index_direction = V)
#define qqIndexDirection(N) ((N)->index_direction)
	
#define AddqqIndexLeft SetqqIndexLeft
#define SetqqIndexLeft(N, V) ((N)->index_left = V)
#define qqIndexLeft(N) ((N)->index_left)
	
#define AddqqIndexRight SetqqIndexRight
#define SetqqIndexRight(N, V) ((N)->index_right = V)
#define qqIndexRight(N)	((N)->index_right)
	
#define AddqqIndexTypeClass SetqqIndexTypeClass
#define SetqqIndexTypeClass(N, V) ((N)->index_type_class = V)
#define qqIndexTypeClass(N) ((N)->index_type_class)
	
#define AddqqIndexTypeSymbInfo SetqqIndexTypeSymbInfo
#define SetqqIndexTypeSymbInfo(N, V) ((N)->fIndexTypeSymbInfo = V)
#define qqIndexTypeSymbInfo(N) ((N)->fIndexTypeSymbInfo)

#define AddqqIntegerLeftBound SetqqIntegerLeftBound
#define SetqqIntegerLeftBound(N, V) ((N)->fLargeLeftBound.high = V)
#define qqIntegerLeftBound(N)       ((N)->fLargeLeftBound.high)

#define AddqqIntegerRightBound SetqqIntegerRightBound
#define SetqqIntegerRightBound(N, V) ((N)->fLargeRightBound.high = V)
#define qqIntegerRightBound(N)       ((N)->fLargeRightBound.high)

#define AddqqIntfDataOffset SetqqIntfDataOffset
#define SetqqIntfDataOffset(N, V) ((N)->fIntfDataOffset = V)
#define qqIntfDataOffset(N)	  ((N)->fIntfDataOffset)

#define AddqqIntfDataSize SetqqIntfDataSize
#define SetqqIntfDataSize(N, V) ((N)->fIntfDataSize = V)
#define qqIntfDataSize(N)	((N)->fIntfDataSize)

#define AddqqIntfMode SetqqIntfMode
#define SetqqIntfMode(N, V) ((N)->fIntfMode = V)
#define qqIntfMode(N)       ((N)->fIntfMode)

#define AddqqItemNum SetqqItemNum
#define SetqqItemNum(N, V) ((N)->fItemNum = V)
#define qqItemNum(N)	((N)->fItemNum)

#define AddqqIsUnconstrained SetqqIsUnconstrained
#define SetqqIsUnconstrained(N, V) ((N)->fIsUnconstrained = V)
#define qqIsUnconstrained(N)	 ((N)->fIsUnconstrained)

#define AddqqIsGuard SetqqIsGuard
#define SetqqIsGuard(N, V) ((N)->fIsGuard = V)
#define qqIsGuard(N)	   ((N)->fIsGuard)

#define AddqqIsImplicit SetqqIsImplicit
#define SetqqIsImplicit(N, V) ((N)->fIsImplicit = V)
#define qqIsImplicit(N)	      ((N)->fIsImplicit)

#define AddqqIsMixed SetqqIsMixed
#define SetqqIsMixed(N, V) ((N)->fIsMixed = V)
#define qqIsMixed(N)	  ((N)->fIsMixed)

#define AddqqIsPort SetqqIsPort
#define SetqqIsPort(N, V) ((N)->fIsPort = V)
#define qqIsPort(N)	  ((N)->fIsPort)

#define AddqqIsResolved SetqqIsResolved
#define SetqqIsResolved(N, V) ((N)->fIsResolved = V)
#define qqIsResolved(N)	      ((N)->fIsResolved)

#define AddqqItemBaseType SetqqItemBaseType
#define SetqqItemBaseType(N, V) ((N)->fItemBaseType = V)
#define qqItemBaseType(N) ((N)->fItemBaseType)

#define AddqqKernelProcess SetqqKernelProcess
#define SetqqKernelProcess(N, V) ((N)->fKernelProcess = V)
#define qqKernelProcess(N)	 ((N)->fKernelProcess)

#define AddqqLargeLeftBound SetqqLargeLeftBound
#define SetqqLargeLeftBound(N, V) TIME_COPY((N)->fLargeLeftBound, V)
#define qqLargeLeftBound(N)       ((N)->fLargeLeftBound)

#define AddqqLargeRightBound SetqqLargeRightBound
#define SetqqLargeRightBound(N, V) TIME_COPY((N)->fLargeRightBound, V)
#define qqLargeRightBound(N) ((N)->fLargeRightBound)

#define AddqqLastActiveCycle SetqqLastActiveCycle
#define SetqqLastActiveCycle(N, V) ((N)->fLastActiveCycle = V)
#define qqLastActiveCycle(N)	((N)->fLastActiveCycle)

#define AddqqLastActiveTime SetqqLastActiveTime
#define SetqqLastActiveTime(N, V) (TIME_COPY((N)->fLastActiveTime, V))
#define qqLastActiveTime(N)	((N)->fLastActiveTime)

#define AddqqLastEventCycle SetqqLastEventCycle
#define SetqqLastEventCycle(N, V) ((N)->fLastEventCycle = V)
#define qqLastEventCycle(N)	  ((N)->fLastEventCycle)

#define AddqqLastEventTime SetqqLastEventTime
#define SetqqLastEventTime(N, V) (TIME_COPY((N)->fLastEventTime, V))
#define qqLastEventTime(N)	 ((N)->fLastEventTime)

#define AddqqLeftBound SetqqLeftBound
#define SetqqLeftBound(N, V) ((N)->fLeftBound = V)
#define qqLeftBound(N)	     ((N)->fLeftBound)

#define AddqqLibraryInfo SetqqLibraryInfo
#define SetqqLibraryInfo(N, V) ((N)->fLibraryInfo = V)
#define qqLibraryInfo(N)       ((N)->fLibraryInfo)

#define AddqqLiterals SetqqLiterals
#define SetqqLiterals(N, V) ((N)->fLiterals = V)
#define qqLiterals(N)	((N)->fLiterals)

#define AddqqMapped(N, V) ((N)->fMapped = V)
#define SetqqMapped(N, V) ((N)->fMapped = V)
#define qqMapped(N)	((N)->fMapped)

#define AddqqName(N, V) ((N)->fName = V)
#define SetqqName(N, V) ((N)->fName = V)
#define qqName(N)	((N)->fName)

#define AddqqNoted(N, V) ((N)->fNoted = V)
#define SetqqNoted(N, V) ((N)->fNoted = V)
#define qqNoted(N)	 ((N)->fNoted)

#define AddqqNumber SetqqNumber
#define SetqqNumber(N, V) ((N)->fNumber = V)
#define qqNumber(N)	((N)->fNumber)

#define AddqqOffsetOffset SetqqOffsetOffset
#define SetqqOffsetOffset(N, V) ((N)->fOffsetOffset = V)
#define qqOffsetOffset(N)	((N)->fOffsetOffset)

#define AddqqOutType SetqqOutType
#define SetqqOutType(N, V) ((N)->fOutType = V)
#define qqOutType(N)	   ((N)->fOutType)

#define AddqqOutConstraint SetqqOutConstraint
#define SetqqOutConstraint(N, V) ((N)->fOutConstraint = V)
#define qqOutConstraint(N)	 ((N)->fOutConstraint)

#define AddqqOutDimensions SetqqOutDimensions
#define SetqqOutDimensions(N, V) ((N)->fOutDimensions = V)
#define qqOutDimensions(N)	 ((N)->fOutDimensions)

#define AddqqOwnerProcess SetqqOwnerProcess
#define SetqqOwnerProcess(N, V) ((N)->fOwnerProcess = V)
#define qqOwnerProcess(N)	((N)->fOwnerProcess)

#define AddqqProcessData SetqqProcessData
#define SetqqProcessData(N, V)  ((N)->fProcessData = V)
#define qqProcessData(N)        ((N)->fProcessData)

#define AddqqParentAssoc SetqqParentAssoc
#define SetqqParentAssoc(N, V)	 ((N)->parentassoc = (V))
#define qqParentAssoc(N)	 ((N)->parentassoc)

#define AddqqParentElement SetqqParentElement
#define SetqqParentElement(N, V) ((N)->fParentElement = V)
#define qqParentElement(N)       ((N)->fParentElement)

#define AddqqParentRegion SetqqParentRegion
#define SetqqParentRegion(N, V) ((N)->fParentRegion = V)
#define qqParentRegion(N)	((N)->fParentRegion)

#define AddqqParentSignal SetqqParentSignal
#define SetqqParentSignal(N, V) ((N)->fParentSignal = V)
#define qqParentSignal(N)       ((N)->fParentSignal)

#define AddqqPortDriven SetqqPortDriven
#define SetqqPortDriven(N, V) ((N)->fPortDriven = V)
#define qqPortDriven(N)	((N)->fPortDriven)

#define AddqqPortMap SetqqPortMap
#define SetqqPortMap(N, V) ((N)->fPortMap = V)
#define qqPortMap(N)	((N)->fPortMap)

#define AddqqPositionValues SetqqPositionValues
#define SetqqPositionValues(N, V) ((N)->fPositionValues = V)
#define qqPositionValues(N)	((N)->fPositionValues)

#define AddqqPrefixSignalLength SetqqPrefixSignalLength
#define SetqqPrefixSignalLength(N, V) ((N)->fPrefixSignalLength = V)
#define qqPrefixSignalLength(N) ((N)->fPrefixSignalLength)

#define AddqqPrefixSignalHandle SetqqPrefixSignalHandle
#define SetqqPrefixSignalHandle(N, V) ((N)->fPrefixSignalHandle = V)
#define qqPrefixSignalHandle(N) ((N)->fPrefixSignalHandle)

#define AddqqPrefixSignalOffset SetqqPrefixSignalOffset
#define SetqqPrefixSignalOffset(N, V) ((N)->fPrefixSignalOffset = V)
#define qqPrefixSignalOffset(N) ((N)->fPrefixSignalOffset)

#define AddqqPrefixSignalStart SetqqPrefixSignalStart
#define SetqqPrefixSignalStart(N, V) ((N)->fPrefixSignalStart = V)
#define qqPrefixSignalStart(N) ((N)->fPrefixSignalStart)

#define AddqqProcessKind SetqqProcessKind
#define SetqqProcessKind(N, V) ((N)->fProcessKind = V)
#define qqProcessKind(N) ((N)->fProcessKind)

#define AddqqPreviousValuePtr SetqqPreviousValuePtr
#define SetqqPreviousValuePtr(N, V) ((N)->fPreviousValuePtr = V)
#define qqPreviousValuePtr(N)	((N)->fPreviousValuePtr)

#define AddqqRecordElemCount SetqqRecordElemCount
#define SetqqRecordElemCount(N, V) ((N)->fRecordElemCount = (V))
#define qqRecordElemCount(N) ((N)->fRecordElemCount)

#define AddqqRecordKind SetqqRecordKind
#define SetqqRecordKind(N, V) ((N)->fRecordKind = V)
#define qqRecordKind(N)	      ((N)->fRecordKind)

#define AddqqRegionData SetqqRegionData
#define SetqqRegionData(N, V) ((N)->fRegionData = V)
#define qqRegionData(N)	((N)->fRegionData)

#define AddqqRegionKind SetqqRegionKind
#define SetqqRegionKind(N, V) ((N)->fRegionKind = (V))
#define qqRegionKind(N) ((N)->fRegionKind)

#define AddqqResEvalFn SetqqResEvalFn
#define SetqqResEvalFn(N, V) ((N)->fResEvalFn = V)
#define qqResEvalFn(N) ((N)->fResEvalFn)

#define AddqqResFuncInfo SetqqResFuncInfo
#define SetqqResFuncInfo(N, V) ((N)->fResFuncInfo = (V))
#define qqResFuncInfo(N) ((N)->fResFuncInfo)

#define AddqqResolved SetqqResolved
#define SetqqResolved(N, V) ((N)->fResolved = V)
#define qqResolved(N) ((N)->fResolved)

#define AddqqSegment SetqqSegment
#define SetqqSegment(N, V) ((N)->fSegment = V)
#define qqSegment(N) ((N)->fSegment)

#define AddqqSibling SetqqSibling
#define SetqqSibling(N, V) ((N)->fSibling = V)
#define qqSibling(N)       ((N)->fSibling)

#define AddqqSimplePort SetqqSimplePort
#define SetqqSimplePort(N, V) ((N)->fSimplePort = V)
#define qqSimplePort(N)       ((N)->fSimplePort)

#define AddqqSkipRegister SetqqSkipRegister
#define SetqqSkipRegister(N, V) ((N)->fSkipRegister = V)
#define qqSkipRegister(N)       ((N)->fSkipRegister)
 
#define AddqqSymbInfoKind SetqqSymbInfoKind
#define SetqqSymbInfoKind(N, V) ((N)->fSymbInfoKind = (V))
#define qqSymbInfoKind(N) ((N)->fSymbInfoKind)

#define AddqqResolutionContext SetqqResolutionContext
#define SetqqResolutionContext(N, V) ((N)->fResolutionContext = V)
#define qqResolutionContext(N)       ((N)->fResolutionContext)

#define AddqqResolutionData SetqqResolutionData
#define SetqqResolutionData(N, V) ((N)->fResolutionData = V)
#define qqResolutionData(N)       ((N)->fResolutionData)

#define AddqqRootNet SetqqRootNet
#define SetqqRootNet(N, V) ((N)->rootnet = V)
#define qqRootNet(N)	((N)->rootnet)

#define AddqqRootSignal SetqqRootSignal
#define SetqqRootSignal(N, V) ((N)->fRootSignal = V)
#define qqRootSignal(N)       ((N)->fRootSignal)

#define AddqqSensitiveProcess SetqqSensitiveProcess
#define SetqqSensitiveProcess(N, V) ((N)->sensitive_process = V)
#define qqSensitiveProcess(N) ((N)->sensitive_process)

#define AddqqScheduled SetqqScheduled
#define SetqqScheduled(N, V) ((N)->fScheduled = V)
#define qqScheduled(N)       ((N)->fScheduled)

#define AddqqSensitivityList SetqqSensitivityList
#define SetqqSensitivityList(N, V) ((N)->fSensitivityList = V)
#define qqSensitivityList(N)	   ((N)->fSensitivityList)
	
#define AddqqSecName(N, V) ((N)->fSecName = V)
#define SetqqSecName(N, V) ((N)->fSecName = V)
#define qqSecName(N)	   ((N)->fSecName)

#define AddqqSimpleType(N, V) ((N)->fSimpleType = V)
#define SetqqSimpleType(N, V) ((N)->fSimpleType = V)
#define qqSimpleType(N)	((N)->fSimpleType)

#define AddqqSourceFileName(N, V) ((N)->fSourceFileName = V)
#define SetqqSourceFileName(N, V) ((N)->fSourceFileName = V)
#define qqSourceFileName(N)	((N)->fSourceFileName)

#define AddqqSignalClass SetqqSignalClass
#define SetqqSignalClass(N, V) ((N)->fSignalClass = V)
#define qqSignalClass(N)	((N)->fSignalClass)

#define AddqqSignalBuffer SetqqSignalBuffer
#define SetqqSignalBuffer(N, V) ((N)->fSignalBuffer = V)
#define qqSignalBuffer(N)	((N)->fSignalBuffer)

#define AddqqSignalKind SetqqSignalKind
#define SetqqSignalKind(N, V) ((N)->fSignalKind = V)
#define qqSignalKind(N)	((N)->fSignalKind)

#define AddqqSignalDownStart SetqqSignalDownStart
#define SetqqSignalDownStart(N, V) ((N)->fSignalDownStart = V)
#define qqSignalDownStart(N)	((N)->fSignalDownStart)

#define AddqqSignalStart SetqqSignalStart
#define SetqqSignalStart(N, V) ((N)->fSignalStart = V)
#define qqSignalStart(N)	((N)->fSignalStart)

#define AddqqSignalDownLength SetqqSignalDownLength
#define SetqqSignalDownLength(N, V) ((N)->fSignalDownLength = V)
#define qqSignalDownLength(N)	((N)->fSignalDownLength)

#define AddqqSignalLength SetqqSignalLength
#define SetqqSignalLength(N, V) ((N)->fSignalLength = V)
#define qqSignalLength(N)	((N)->fSignalLength)

#define AddqqSignalUpLength SetqqSignalUpLength
#define SetqqSignalUpLength(N, V) ((N)->fSignalUpLength = V)
#define qqSignalUpLength(N)	((N)->fSignalUpLength)

#define AddqqSignalUpStart SetqqSignalUpStart
#define SetqqSignalUpStart(N, V) ((N)->fSignalUpStart = V)
#define qqSignalUpStart(N)	((N)->fSignalUpStart)

#define AddqqSourceList SetqqSourceList
#define SetqqSourceList(N, V) ((N)->fSourceList = V)
#define qqSourceList(N)	      ((N)->fSourceList)

#define AddqqSymbInfo SetqqSymbInfo
#define SetqqSymbInfo(N, V) ((N)->fSymbInfo = V)
#define qqSymbInfo(N)	((N)->fSymbInfo)

#define AddqqTraceNum SetqqTraceNum
#define SetqqTraceNum(N, V) ((N)->fTraceNum = V)
#define qqTraceNum(N)	((N)->fTraceNum)

#define AddqqTypeConvDown SetqqTypeConvDown
#define SetqqTypeConvDown(N, V) ((N)->type_conv_down = V)
#define qqTypeConvDown(N)	((N)->type_conc_down)

#define AddqqTypeConvUp SetqqTypeConvUp
#define SetqqTypeConvUp(N, V) ((N)->type_conv_up = V)
#define qqTypeConvUp(N)	((N)->type_conv_up)

#define AddqqTidied SetqqTidied
#define SetqqTidied(N, V) ((N)->fTidied = V)
#define qqTidied(N)	  ((N)->fTidied)

#define AddqqTime SetqqTime
#define SetqqTime(N, V) (TIME_COPY((N)->fTime, V))
#define qqTime(N)	((N)->fTime)

#define AddqqTimedOut SetqqTimedOut
#define SetqqTimedOut(N, V) ((N)->fTimedOut = V)
#define qqTimedOut(N)	    ((N)->fTimedOut)

#define AddqqTraceTable SetqqTraceTable
#define SetqqTraceTable(N, V) ((N)->fTraceTable = V)
#define qqTraceTable(N)	((N)->fTraceTable)

#define AddqqTriggerActive SetqqTriggerActive
#define SetqqTriggerActive(N, V) ((N)->triggeractive = V)
#define qqTriggerActive(N)	((N)->triggeractive)

#define AddqqTriggerEvent SetqqTriggerEvent
#define SetqqTriggerEvent(N, V) ((N)->triggerevent = V)
#define qqTriggerEvent(N)	((N)->triggerevent)

#define AddqqTriggerEventSets SetqqTriggerEventSets
#define SetqqTriggerEventSets(N, V) ((N)->triggereventsets = V)
#define qqTriggerEventSets(N)	((N)->triggereventsets)

#define AddqqTriggerCycle SetqqTriggerCycle
#define SetqqTriggerCycle(N, V) ((N)->fTriggerCycle = V)
#define qqTriggerCycle(N)       ((N)->fTriggerCycle)
  
#define AddqqUsed SetqqUsed
#define SetqqUsed(N, V) ((N)->fUsed = V)
#define qqUsed(N)       ((N)->fUsed)
  
#define AddqqValueOffset SetqqValueOffset
#define SetqqValueOffset(N, V) ((N)->fValueOffset = V)
#define qqValueOffset(N)       ((N)->fValueOffset)

#define AddqqWakeUpTime SetqqWakeUpTime
#define SetqqWakeUpTime(N, V) (TIME_COPY((N)->fWakeUpTime, V))
#define qqWakeUpTime(N)	((N)->fWakeUpTime)

#define AddqqWaveformValues SetqqWaveformValues
#define SetqqWaveformValues(N, V) ((N)->fWaveformValues = V)
#define qqWaveformValues(N)       ((N)->fWaveformValues)

#define AddqqWaveformTimes SetqqWaveformTimes
#define SetqqWaveformTimes(N, V) ((N)->fWaveformTimes = V)
#define qqWaveformTimes(N)       ((N)->fWaveformTimes)

#define AddqqDebugBit SetqqDebugBit
#define SetqqDebugBit(N, V) ((N)->fDebugBit = V)
#define qqDebugBit(N)	    ((N)->fDebugBit)


#define AddqqWaitContext SetqqWaitContext
#define SetqqWaitContext(N, V) ((N)->fWaitContext = V)
#define qqWaitContext(N)       ((N)->fWaitContext)


#define AddqqWaitFn SetqqWaitFn
#define SetqqWaitFn(N, V) ((N)->fWaitFn = V)
#define qqWaitFn(N)	  ((N)->fWaitFn)

/* Element Data Access */

#define SIGNAL_DATA_BUFFERS 4

#define GetElementHandle(e,g,o,l) {e = qqElements(g);\
                                   e += o;\
                                   VERIFY_ERROR(l <= (qqSignalLength(g) - o));}

#define GetRealData(e,l) ((tc_Alias == qqElementClass(e))\
                                          ? RealDataSegment(e,l)\
                                          : e)

#define GetRealElement(e) ((tc_Alias == qqElementClass(e)) ? RealElement(e)\
                                                           : e)

#define COPY_ELEM_DATA(size,from,to) {if (1 == size) {\
                                       *(VK_tIntegerData *)(to) =\
                                       *(VK_tIntegerData *)(from);\
			              } else {\
                                       *(tScalarUnionPtr)(to) =\
                                       *(tScalarUnionPtr)(from);}}

#define SET_DATA(e,fld,ptr) COPY_ELEM_DATA(qqDataLength(e),ptr,fld(e))
#define GET_DATA(e,fld,ptr) COPY_ELEM_DATA(qqDataLength(e),fld(e),ptr)

#define SetDefaultValue(e,ptr)   SET_DATA(e,qqDefaultValuePtr,ptr)
#define GetDefaultValue(e,ptr)   GET_DATA(e,qqDefaultValuePtr,ptr)

#define SetEffectiveValue(e,ptr) SET_DATA(e,qqEffectiveValuePtr,ptr)
#define GetEffectiveValue(e,ptr) GET_DATA(e,qqEffectiveValuePtr,ptr)

#define SetDrivingValue(e,ptr)   SET_DATA(e,qqDrivingValuePtr,ptr)
#define GetDrivingValue(e,ptr)   GET_DATA(e,qqDrivingValuePtr,ptr)

#define SetPreviousValue(e,ptr)  SET_DATA(e,qqPreviousValuePtr,ptr)
#define GetPreviousValue(e,ptr)  GET_DATA(e,qqPreviousValuePtr,ptr)

/* Driver Element */

#define SetCurrentValue(d,ptr)  SET_DATA(d,qqCurrentValuePtr,ptr)
#define GetCurrentValue(d,ptr)  GET_DATA(d,qqCurrentValuePtr,ptr)

/* Port Mapping */

#define SignalIn(e,p)  (qqSignalDownStart(p) + \
                               qqNumber(e) - qqSignalUpStart(p))

#define SignalOut(e,p) (qqSignalUpStart(p) + \
                               qqNumber(e) - qqSignalDownStart(p))

#define DriverOut(e,p) (qqDriverStart(p) + \
                               qqNumber(e) - qqSignalDownStart(p))

/* Implicit Data */

#define ImplicitData(e)       (qqImplicitData(qqGroup(e)))
#define PrefixSignalStart(e)  (qqPrefixSignalStart(ImplicitData(e)))
#define PrefixSignalLength(e) (qqPrefixSignalLength(ImplicitData(e)))
#define PrefixSignalHandle(i) (qqRootSignal(qqGroup(SignalElementHandle(\
                                                    qqPrefixSignalStart(i)))))

#define ImplicitTime(e)       (qqTime(ImplicitData(e)))

/* Offset Calculation */

#define SignalOffset(e) (qqNumber(e) - qqSignalStart(qqRootSignal(qqGroup(e))))


#endif /* TKStructs_H */
