static char SCCS_ID[] = "@(#)TKSymbUtils.c	42.3 1/13/92";
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* CODE FILE: TKSymbUtils.c
*
* DESCRIPTION:  
*	??
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
*   Internal:
*	...
*
* HISTORY:
*	21-FEB-91 DSB	Created
*	26-MAR-91 JBR	W0003.  Added VK_RegisterIntegerType and
*			VK_RegisterFloatingType.
*	02-APR-91 JBR	W0013.  Added VK_RegisterPhysicalType,
*			VK_RegisterArrayTypeDecl, and VK_RegisterIndexSubtype.
*	16-APR-91 WAG	W0033.  Added NewCreateSymbInfoRecord, modified
*			CreateSymbInfoRecord to call it.  NewCreate... routine
*			will become the Create... routine eventually.
*	18-APR-91 WAG	W0027.  Added routines VK_RegisterInterfaceSignalDecl
*			and VK_RegisterLocalSignalDecl.
*	18-APR-91 WAG	W0025.  Added routines VK_RegisterRecordTypeDecl,
*			VK_RegisterRecordElementDecl, VK_RegisterFileTypeDecl,
*			VK_RegisterAccessTypeDecl, and VK_RegisterSubtypeDecl.
*	18-APR-91 WAG	W0026.  Added for routines
*			VK_RegisterInstantiationStmt,
*			VK_RegisterInterfaceConstantDecl,
*			VK_RegisterLocalConstantDecl,
*			VK_RegisterInterfaceVariableDecl,
*			VK_RegisterLocalVariableDecl, and
*			VK_RegisterLocalFileDecl.
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	04-MAY-91 DSB	Changed to signal handle / signal offset protocol
*	10-MAY-91 JBR	W0041. Added su_GetUnitNames.
*	16-MAY-91 DSB	Cleaning up tSymbInfoKind enumerators; added new ones
*			to remove BadInfo uses; fixed bug: IntegerTypeDecl
*			had EnumerationTypeDeclInfo value
*	16-MAY-91 DSB	Fixing bug: fixed "0.0" that were "0"
*	30-MAY-91 DSB	Moved su_GetTypeInfoTypeClass to here from TKSignals.c
*	05-JUN-91 DSB	Moved su_TraverseToData from TKSignals to TKSymbInfo
*	06-JUN-91 DSB	Added SIH_Std__TextIO__Text
*	12-JUN-91 DSB	Added SIH_Std__Standard, SIH_Std__Standard__String, 
*			SIH_Std__TextIO
*	17-JUN-91 JBR	Corrected types of some dummy parameters to
*			CreateSymbInfoRecord_Decls.
*	23-AUG-91 DSB	Split TKSymbInfo into TKSymbUtils and TKRegister
*
* SCCS ID: @(#)TKSymbUtils.c	42.3 1/13/92
*
******************************************************************* */

#include "TKSymbUtils.h"
#include "VKI.h"
#include "TKStructs.h"
#include "TKGlobals.h"		/* for SymbInfoList */
#include "TKIO.h"		/* for GetSymbInfoKindStr */

#define MAX_DIMENSIONS 10       /* for temporary type data */

/* *******************************************************************
*
* su_GetTypeInfoTypeClass - get type class from type's symb. info. record
*
* HISTORY:
*	15-MAY-91 DSB	Created
*	16-MAY-91 DSB	Cleaning up tSymbInfoKind enumerators
*	30-MAY-91 DSB	Added null check
*	11-JUN-91 DSB	Added LargePhysicalTypeDeclInfo case
*	
******************************************************************* */

tTypeClass su_GetTypeInfoTypeClass(type_info)
			 	/* returns type class enumeration value */
VK_tSymbInfoHandle type_info;		/* in: symb. info. for type */
{
    tTypeClass type_class;
/**/
    if (NULL == type_info) {
	type_class = tc_BadTypeClass;
    }
    else {
	switch (qqSymbInfoKind(type_info)) {
	    case EnumerationTypeDeclInfo:
					type_class = tc_Enumeration;	break;
	    case IntegerTypeDeclInfo:	type_class = tc_Integer;	break;
	    case FloatingTypeDeclInfo:	type_class = tc_Floating;	break;
	    case PhysicalTypeDeclInfo:	type_class = tc_Physical;	break;
	    case LargePhysicalTypeDeclInfo:
					type_class = tc_LargePhysical;	break;
	    case ArrayTypeDeclInfo:	type_class = tc_Array;		break;
	    case RecordTypeDeclInfo:	type_class = tc_Record;		break;
	    case FileTypeDeclInfo:	type_class = tc_File;		break;
	    case AccessTypeDeclInfo:	type_class = tc_Access;		break;
	    default:
	    	ReportInternalError("Unexpected symb. info. kind", 
			GetSymbInfoKindStr(qqSymbInfoKind(type_info)), 
			__FILE__, __LINE__);
	    break;
	} /* switch */
    }
    return type_class;
} /* su_GetTypeInfoTypeClass */



/* ?? need to document parameters: */

/* *******************************************************************
*
* su_GetUnitNames - Extract the library, unit, and file names from design
*		unit's SymbInfo
*
* HISTORY:
*	10-MAY-91 JBR	Created
*
******************************************************************* */
void su_GetUnitNames(unit_info, library, primary, secondary, file)
VK_tSymbInfoHandle unit_info;		/* in */
VK_tName * library;			/* out */
VK_tName * primary;			/* out */
VK_tName * secondary;			/* out */
VK_tName * file;			/* out */
{
    VK_tSymbInfoHandle ext_info;	/* info for related primary unit */
/**/
    switch (qqSymbInfoKind(unit_info)) {
	case PackageDeclInfo:
	case EntityDeclInfo:
	    *library = qqName(qqEnclReg(unit_info));
	    *primary = qqName(unit_info);
	    *secondary = "";
	    *file = qqSourceFileName(unit_info);
	    break;
	case ConfigurationDeclInfo:
	    *library = qqName(qqEnclReg(qqExtReg(qqExtReg(unit_info)))); 
					/* config. -> arch. -> ent. */
	    *primary = qqName(unit_info);
	    *secondary = "";
	    *file = qqSourceFileName(unit_info);
	    break;

	case PackageBodyInfo:
	case ArchitectureBodyInfo:
	    ext_info = qqExtReg(unit_info);
	    *library = qqName(qqEnclReg(ext_info));
	    *primary = qqName(ext_info);


	    *secondary = qqName(unit_info);
	    *file = qqSourceFileName(unit_info);
	    break;

	default:
	    /* In the future we could traverse the hierarchy to find the
		containing unit's SymbInfo, but for now, report error */
	    ReportInternalError("Invalid SymbInfo for su_GetUnitNames", "",
		__FILE__, __LINE__);
	    break;
    } /* switch */
} /* su_GetUnitNames */


/* *******************************************************************
*
* su_IdentifyPosition - get execution position id. information from LAR
*
* DESCRIPTION:
*	Gets source position information from special data stuffed in
*	given local activation record and from trace table for unit
*
* HISTORY:
*	17-MAY-91 DSB	Created from su_GetUnitNames
*	30-MAY-91 DSB	Removed negated line numbers code
*	13-AUG-91 DSB	Check for null TK_GetRegionInfo result
*	18-DEC-91 DKC	Corrected entry test
*
******************************************************************* */
void su_IdentifyPosition(local_act_rec, library, primary, secondary, file, 
	line_number, region_info)
VK_tLARHeaderPtr local_act_rec;		/* in: LAR of region to identify */
VK_tName * library;			/* &out: library name of unit */
VK_tName * primary;			/* &out: primary unit name */
VK_tName * secondary;			/* &out: secondary unit name */
VK_tName * file;			/* &out: source file name */
VK_tLineNumber * line_number;		/* &out: source file line number */
VK_tSymbInfoHandle * region_info;	/* &out: current region symb. info. */
{
    VK_tSymbInfoHandle ext_info;	/* info for related primary unit */
    VK_tExecPosIndex exec_pos_index;
    VK_tSymbInfoHandle unit_info;
/**/
    *library = "";
    *primary = "";
    *secondary = "";
    *file = "";
    *line_number = 0;
    *region_info = NULL;

    if (NULL == local_act_rec) {
       return;
    }

    *region_info   = TK_GetRegionInfo(local_act_rec);
    unit_info      = TK_GetUnitInfo(local_act_rec);
    exec_pos_index = TK_GetExecPos(local_act_rec);

    VERIFY_ERROR(0 <= exec_pos_index);

    if (NULL != unit_info) {

        if (NULL != qqTraceTable(unit_info)) {
            *line_number = qqTraceTable(unit_info)[exec_pos_index].line_number;
        }

        switch (qqSymbInfoKind(unit_info)) {

	    case PackageDeclInfo:
	    case EntityDeclInfo:
	        *library = qqName(qqEnclReg(unit_info));
	        *primary = qqName(unit_info);
	        *secondary = "";
	        *file = qqSourceFileName(unit_info);
	        break;

	    case ConfigurationDeclInfo:
	        *library = qqName(qqEnclReg(qqExtReg(qqExtReg(unit_info)))); 
	        *primary = qqName(unit_info);
	        *secondary = "";
	        *file = qqSourceFileName(unit_info);
	        break;

	    case PackageBodyInfo:
	    case ArchitectureBodyInfo:
	        ext_info = qqExtReg(unit_info);
	        *library = qqName(qqEnclReg(ext_info));
	        *primary = qqName(ext_info);

	        *secondary = qqName(unit_info);
	        *file = qqSourceFileName(unit_info);
	        break;

	    default:
	        /* In the future we could traverse the hierarchy to find the
		    containing unit's SymbInfo, but for now, report error */
	        ReportInternalError("Invalid SymbInfo for su_GetUnitNames", "",
		                    __FILE__, __LINE__);
	        break;
        } /* switch */
    }

} /* su_IdentifyPosition */



/* *******************************************************************
*
* su_FindLibrary - find symbolic information for library by library name
*
* DESCRIPTION:
*	Searches symbolic information record list for library matching given
*	name.  Returns symbolic information handle of matching library if any,
*	or NULL if none.
*
* HISTORY:
*	14-AUG-91 DSB	Created
*
******************************************************************* */
VK_tSymbInfoHandle su_FindLibrary(library_name)
				/* returns LibraryRegion symb. info. */
VK_tName library_name;		/* in: DLS library name string */
{
    VK_tSymbInfoHandle result;
    TempItem library_scan;
    VK_tSymbInfoHandle info_handle;
/**/
    library_scan = TempFirstItem(SymbInfoList);
    result = NULL;
    while (NULL == result && ! TempNullItem(library_scan)) {
	info_handle = SymbInfoValue(library_scan);
	if (LibraryRegionInfo == qqSymbInfoKind(info_handle) &&
		StringMatch(qqName(info_handle), library_name)) {
	    result = info_handle;
	}
	else {
	    library_scan = TempNextItem(library_scan);
	}
    }
    return result;		/* return NULL if not found */
} /* su_FindLibrary */



/* ?? needs a better name, too ?? */
/* *******************************************************************
*
* su_TraverseToData - get LAR for region relative to given region and LAR
*
* HISTORY:
*	15-MAY-91 DSB	Created
*	17-MAY-91 DSB	Fixed bugs; allow for ent.-to-arch. extension
*	05-JUN-91 DSB	Moved su_TraverseToData from TKSignals to TKSymbInfo
*	19-NOV-91 DKC	Changed Verify to handle signals declared at entity 
*                       level
*
******************************************************************* */
VK_tLARHeaderPtr su_TraverseToData(target_region_info, reference_region_info,
	reference_data)			/* returns LAR of target region */
VK_tSymbInfoHandle target_region_info;	/* in: requested region's info. */
VK_tSymbInfoHandle reference_region_info;
					/* in: referring region's info. */
VK_tLARHeaderPtr reference_data;	/* in: referring region's LAR */
{
    VK_tLARHeaderPtr target_data;
    VK_tLARHeaderPtr current_data;
    VK_tSymbInfoHandle current_info;
    VK_tBoolean looping;
/**/
    if (NULL != reference_region_info && NULL != reference_data) {
	VERIFY_ERROR(TK_GetRegionInfo(reference_data) == 
                                                      reference_region_info ||
                     qqExtReg(TK_GetRegionInfo(reference_data)) ==
		                                      reference_region_info);
    }

    if (PackageDeclInfo == qqSymbInfoKind(target_region_info)) {
					/* if package declaration: */
	target_data = qqRegionData(target_region_info);
					/* get package declaration LAR saved
					   with symb. info. by VK_TraceBegin */
    }
    else if (PackageBodyInfo == qqSymbInfoKind(target_region_info)) {
					/* if package body: */
	target_data = qqRegionData(target_region_info);
					/* get package body LAR saved
					   with symb. info. by VK_TraceBegin */
    }
    else if (target_region_info == reference_region_info) {
					/* if target region is starting 
					   region: */
	target_data = reference_data;	/* given LAR is requested LAR */
    }
    else {				/* otherwise, target LAR should be
					   ancestor of given LAR */

	/* Algorithm:  Synchronously traverse up context links of LARs and up 
		symbolic information records.  When current symb. info. matches
		requested region's symb. info., then current LAR is requested
		LAR. (Like traversing static back links to satisfy non-local
		reference.) */

	looping = True;
	current_data = reference_data;
	current_info = reference_region_info;
	while (looping) {
	    if (target_region_info == current_info ||
		target_region_info == qqExtReg(current_info)) {
					/* traversed up to requested region */
		target_data = current_data;
		looping = False;	/* exit loop */
	    }
	    else {			/* not at requested region yet */
		current_data = VK_GetContextLARP(VK_GetIARP(current_data));
					/* traverse up: LAR to IAR, then to
					   context LAR */
		current_info = qqEnclReg(current_info);
					/* synchronously traverse up symb.
					   info. records */
		if (NULL == current_info || NULL == current_data) {
					/* this should never happen, for
					   valid calls and intact data 
					   structures */
		    ReportInternalError(
			"su_TraverseToData: Out of sync. or impl. limitation", 
			"", __FILE__, __LINE__);
		    looping = False;
		}
	    }
	} /* while */
    }
    return target_data;
} /* su_TraverseToData */


/*****************************************************************************
*
* IsInterfaceObject - tell whether symb. info. record is for interface object
*
* DESCRIPTION:
*
* HISTORY:
*	18-OCT-91 DSB	Created
*
*****************************************************************************/

VK_tBoolean IsInterfaceObject(symb_info)
					/* returns True if is intf. object */
VK_tSymbInfoHandle symb_info;		/* in: */
{
    tSymbInfoKind kind;
/**/
    kind = qqSymbInfoKind(symb_info);
    return InterfaceSignalDeclInfo == kind ||
	InterfaceVariableDeclInfo == kind ||
	InterfaceConstantDeclInfo == kind;
} /* IsInterfaceObject */


/*****************************************************************************
*
* GetConstraintAddr - get address of constraint for object, element, or ...
*
* DESCRIPTION:
*
* HISTORY:
*	18-OCT-91 DSB	Created
*
*****************************************************************************/

VK_tIndexConstraintAddr GetConstraintAddr(item_info, current_region_info, 
	current_region_data)
VK_tSymbInfoHandle item_info;		/* in: object, rec. element, or array
					   type */
VK_tSymbInfoHandle current_region_info;	/* in: starting region symb. info. */
VK_tLARHeaderPtr current_region_data;	/* in: starting region context data */
{
    VK_tIndexConstraintAddr result;
    VK_tBoolean is_interface_object;
    VK_tLARHeaderPtr constraint_act_rec;
/**/
    if (NULL == qqConstraintRegionInfo(item_info)) {
	result = NULL;
    }
    else {
	constraint_act_rec = su_TraverseToData(qqConstraintRegionInfo(
		item_info), current_region_info, current_region_data);

	/* special handling: if an interface object's constraint is in
		the same region as the object, then the constraint is
		in the interface activation record of the region */

	is_interface_object = IsInterfaceObject(item_info);

	if (is_interface_object && (qqConstraintRegionInfo(item_info)
			== current_region_info)) {
	    constraint_act_rec = (VK_tLARHeaderPtr) 
		VK_GetIARP(constraint_act_rec);
	}

	result = (VK_tIndexConstraintAddr) 
		(qqConstraintOffset(item_info) + (char *) constraint_act_rec);
    }
    return result;
} /* GetConstraintAddr */


/* *******************************************************************
*
* GetScalarTypeLength - get size in words of value of given ~scalar type
*
* HISTORY:
*	25-OCT-91 DSB	Created
*
******************************************************************* */
VK_tDataLength GetScalarTypeLength(type_info)
				/* returns word length of scalar type */
VK_tSymbInfoHandle type_info;
{
    VK_tInteger size;
/**/
    switch (qqSymbInfoKind(type_info)) {
        case LargePhysicalTypeDeclInfo:
	    size = 2;
	    break;
	case EnumerationTypeDeclInfo:
	case IntegerTypeDeclInfo:
	case FloatingTypeDeclInfo:
	case PhysicalTypeDeclInfo:
	case FileTypeDeclInfo:		/* count file type as scalar here */
	case AccessTypeDeclInfo:	/* count access type as scalar here */
	    size = 1;
	    break;
	default:
	    ReportInternalError(
		"Unexpected SymbInfoKind for GetScalarTypeLength", "",
		__FILE__, __LINE__);
	    break;
    } /* switch */
    return size;
} /* GetScalarTypeLength */



/*****************************************************************************/


typedef enum {
	ts_None, 
	ts_StartType,		/* at start of type (or pseudo-type) */
	ts_InType,		/* within type */
	ts_EndType,		/* at end of type (or pseudo-type) */
	ts_RecordElement,	/* at record element */
	ts_ArrayElement,	/* at array element */
	ts_AfterElement		/* special */
    } tTraversalState;


typedef struct sLevelData * tLevelDataPtr;

typedef struct sLevelData {
	tLevelDataPtr      next;	/* previous levels */
	tTraversalState    trav_state;

	VK_tSymbInfoHandle type_info;
	VK_tInteger        dimensions,  /* total dimensions  of array type */
                           dimension;	/* current dimension of array type */
	VK_tLARHeaderPtr   type_data;
	VK_tIndexConstraintAddr 
                           constraint_addr;
        VK_tDataLength     data_length;
	tIndexArrayPtr     index_array;	

	VK_tSymbInfoHandle elem_info;	/* record element id. */
	VK_tLARHeaderPtr   elem_data;
    } tLevelDataRec;

#define InitLevelData(p) bzero(p,sizeof(tLevelDataRec))

typedef struct {	
	tTraversalFlags flags;		/* traversal callback control*/
	int             depth;		/* for debugging only */
	VK_tInteger     current_offset;	/* current offset relative to zero at
					   start of traversal */
	tLevelDataPtr   level_data;	/* state data for currrent composite
					   type level */
	tUserStatus     (*user_fn)();	/* user function to call */
	tUserDataPtr    user_data;	/* data to pass to user function */
        tUserStatus     user_status;    /* user function return */
	
    } tRootDataRec, *tRootDataPtr;




/*****************************************************************************
*
* TravGet_Dimension - User acces to dimension data
*
* DESCRIPTION:
*
* HISTORY:
*	12-JAN-92 DKC	Created
*
*****************************************************************************/

VK_tInteger TravGet_Dimension(trav_data_ptr)
tTravDataPtr trav_data_ptr;
{
    tRootDataPtr root_data = (tRootDataPtr)(trav_data_ptr->private);
/**/
    if (IsMember(TravGet_Flags(trav_data_ptr),tf_ArrayType)) {
        return (root_data->level_data->dimension +1);
    }
    else {
        return (root_data->level_data->dimension);
    }
} /* TravGet_Dimension */


/*****************************************************************************
*
* TravGet_IndexArray - 
*
* DESCRIPTION:
*
* HISTORY:
*	12-JAN-92 DKC	Created
*
*****************************************************************************/

tIndexArrayPtr TravGet_IndexArray(trav_data_ptr)
tTravDataPtr trav_data_ptr;
{
    tRootDataPtr root_data = (tRootDataPtr)(trav_data_ptr->private);
/**/
     return (root_data->level_data->index_array);   
} /* TravGet_IndexArray */
/*****************************************************************************
*
* TravGet_Offset - User access to current data offset
*
* DESCRIPTION:
*
* HISTORY:
*	12-JAN-92 DKC	Created
*
*****************************************************************************/

VK_tInteger TravGet_Offset(trav_data_ptr)
tTravDataPtr trav_data_ptr;
{
    tRootDataPtr root_data = (tRootDataPtr)(trav_data_ptr->private);
/**/
    return (root_data->current_offset);
} /* TravGet_Offset */


/*****************************************************************************
*
* TravGet_ConstraintAddr - User acces to constraint data
*
* DESCRIPTION:
*
* HISTORY:
*	12-JAN-92 DKC	Created
*
*****************************************************************************/

VK_tIndexConstraintAddr TravGet_ConstraintAddr(trav_data_ptr)
tTravDataPtr trav_data_ptr;
{
    tRootDataPtr root_data = (tRootDataPtr)(trav_data_ptr->private);
/**/
    if (IsMember(TravGet_Flags(trav_data_ptr),tf_ArrayElem|tf_PseudoElem)) {
        return (&(root_data->level_data->constraint_addr)
                                        [root_data->level_data->dimension -1]);
    }
    else {
        return (&(root_data->level_data->constraint_addr)
                                           [root_data->level_data->dimension]);
    }
} /* TravGet_ConstraintAddr */


/* *******************************************************************
*
* CallUserFunction - 
*
* HISTORY:
*	22-OCT-91 DSB	Created
*
******************************************************************* */
static int CallUserFunction(root_data, flag)
tRootDataPtr   root_data;
tTraversalFlag flag;
{
    VK_tSymbInfoHandle symb_info;
    tTravDataRec       trav_data_rec;
    tTravDataPtr       trav_data_ptr;
/**/
    if (!(flag & root_data->flags)) {
        return us_Searching;
    }

    if (IsMember(flag,tf_ScalarType|tf_RecordType|tf_ArrayType|
                      tf_PseudoType|tf_ArrayElem|tf_PseudoElem)) {

	symb_info = root_data->level_data->type_info;
    }
    else if (IsMember(flag,tf_RecordElem)) {
	symb_info = root_data->level_data->elem_info;
    }
    else {
	symb_info = NULL;
	printf("\bUnhandled flags case in CallUserFunction\n");
    }

    trav_data_ptr                     = &trav_data_rec;
    trav_data_ptr->private            = (VK_tVoidPtr) root_data;
    TravGet_Flags(trav_data_ptr)      = flag;
    TravGet_SymbInfo(trav_data_ptr)   = symb_info;

    root_data->user_status = 
        (*root_data->user_fn)(root_data->user_data, trav_data_ptr);
					/* call user function with user data
					   and traversal data */

    return (us_Searching != root_data->user_status);

} /* CallUserFunction */


/*****************************************************************************
*
* UnexpectedState - reports unexpected state
*
* DESCRIPTION:
*
* HISTORY:
*	19-NOV-91 DKC	Created
*
*****************************************************************************/

static void UnexpectedState()
{
/**/
    ReportKernelStatus(es_Warning,"Unexpected state in type traversal","",0);
} /* UnexpectedState */

/*****************************************************************************
*
* TraverseSubtypeCore - recursive core for general subtype traversal
*
* DESCRIPTION:
*
* HISTORY:
*	18-OCT-91 DSB	Created
*	19-NOV-91 DKC	Changed trave_state return to ts_EndType (on return 
*                       from processing array elements).
*
*****************************************************************************/

void TraverseSubtypeCore(root_data)
tRootDataPtr root_data;
{
    tLevelDataPtr           level_data;		/* in: state information */
    TempItem                scan;
    VK_tSymbInfoHandle      elem_info;
    VK_tLARHeaderPtr        elem_data;
    VK_tSymbInfoHandle      elem_type_info;
    VK_tLARHeaderPtr        elem_type_data;
    VK_tIndexConstraintAddr elem_constraint_addr;
    tLevelDataPtr           new_level_data;
    VK_tInteger             base_offset;
/**/
    level_data  = root_data->level_data;
    base_offset = root_data->current_offset;

    switch (level_data->trav_state) {
	case ts_StartType:
	    switch (qqSymbInfoKind(level_data->type_info)) {
		case EnumerationTypeDeclInfo:
		case IntegerTypeDeclInfo:
		case FloatingTypeDeclInfo:
		case PhysicalTypeDeclInfo:
		case LargePhysicalTypeDeclInfo:
		    level_data->trav_state = ts_EndType;
		    if (CallUserFunction(root_data, tf_ScalarType)
                        && us_Finished == root_data->user_status) {
                           goto user_finished;
                    }
		    root_data->current_offset += 
                                    GetScalarTypeLength(level_data->type_info);
		    level_data->trav_state = ts_EndType;
		    break;
		
		case FileTypeDeclInfo:
		case AccessTypeDeclInfo:
		    level_data->trav_state = ts_EndType;
					/* after start of scalar type, we will
					   be at end of type */
		    if (CallUserFunction(root_data, tf_ScalarType)
                        && us_Finished == root_data->user_status) {
                           goto user_finished;
                    }
		    root_data->current_offset +=
   				    GetScalarTypeLength(level_data->type_info);
		    level_data->trav_state = ts_EndType;
		    break;

		case RecordTypeDeclInfo:
		    level_data->trav_state = ts_InType;
					/* after start of record type, we are 
					   in type */
		    if (CallUserFunction(root_data, tf_RecordType)
                        && us_Finished == root_data->user_status) {
                           goto user_finished;
                    }			/* user function may return or call
					   TraverseSubtype_Recurse several
                                           times before returning */

		    if (ts_InType == level_data->trav_state) {
					/* if user function didn't call Next-
					   Sibling to process type's elements
					   recursively, then we'll do it */
		    	TraverseSubtypeCore(root_data);
		    }
		    else if (ts_EndType != level_data->trav_state) {
			UnexpectedState();
 		        level_data->trav_state = ts_EndType;
		    }
		    break;
		
		case ArrayTypeDeclInfo:
		    VERIFY_ERROR(level_data->dimension == 0 ||
			       level_data->dimension < level_data->dimensions);

		    if (0 == level_data->dimension) {
					/* 0 means starting whole real type */
                        tIndexArrayRec index_array[MAX_DIMENSIONS];

			level_data->dimensions =
                                    qqDimensions(level_data->type_info);

                        VERIFY_ERROR(MAX_DIMENSIONS >= level_data->dimensions);

			level_data->index_array = index_array;
			level_data->trav_state  = ts_InType;
					/* after start of array type, we are 
					   in type */
		    	if (CallUserFunction(root_data, tf_ArrayType)
                            && us_Finished == root_data->user_status) {
                            goto user_finished;
                        }		/* user function may return or call
					   TraverseSubtype_Recurse several
                                           times before returning */

		    	if (ts_InType == level_data->trav_state) {
					/* if user function didn't call Next-
					   Sibling to process type's (pseudo-)
					   elements recursively, then we'll 
					   do it */
		    	    TraverseSubtypeCore(root_data);
		    	}
		    	else if (ts_EndType != level_data->trav_state) {
			    UnexpectedState();
   		    	    level_data->trav_state  = ts_EndType;
		    	}

   		        /* end of array type */
		    	level_data->dimensions = 0;
		    	level_data->dimension  = 0;
		    }
		    else {  /* start pseudo-type for sub- dimension of array */

			level_data->trav_state = ts_InType;
					/* after start of array type, we are 
					   in type */

		    	if (CallUserFunction(root_data, tf_ArrayType)
                            && us_Finished == root_data->user_status) {
                               goto user_finished;
                        }		/* user function may return or call
					   TraverseSubtype_Recurse once
					   before returning */

		    	if (ts_InType == level_data->trav_state) {
					/* if user function didn't call Next-
					   Sibling to process type's (pseudo-)
					   elements recursively, then we'll 
					   do it */
		    	    TraverseSubtypeCore(root_data);
		    	}
		    	else if (ts_EndType != level_data->trav_state) {
			    UnexpectedState();
    			    level_data->trav_state = ts_EndType;
		    	}
			/* end of pseudo-type */
		    }
		    break;

		default:
		    ReportInternalError(
			"Unexpected SymbInfoKind for TraverseSubtypeCore", "",
			__FILE__, __LINE__);
		    break;
	    } /* switch */
	    break;	

	case ts_InType:			/* in record or array type */
	    switch (qqSymbInfoKind(level_data->type_info)) {
		case ArrayTypeDeclInfo:

		{
		    VK_tInteger             items;
                    tTraversalState         ts_state,
                                            ts_end;
                    tIndexArrayPtr          index;     
                    VK_tIndexConstraintAddr constraint;
		/**/
                    constraint = &level_data->constraint_addr[
                                              level_data->dimension];
                    index      = &level_data->index_array[
                                              level_data->dimension];
		    level_data->dimension++;
		    if (level_data->dimension == level_data->dimensions) {
                                                  /* if at bottom dimension, 
                                                     then are at
					             real array element */
                        ts_state = ts_ArrayElement;
                        ts_end   = ts_AfterElement;
                    }
                    else {
                        ts_state = ts_StartType;
                        ts_end   = ts_EndType;
                    }

                    items  = (constraint->right - constraint->left);
                    if (items < 0) {
                        items = -items;
                    }
                    items++;
                    *index = constraint->left - constraint->dir;
		    while (*index != constraint->right) {

                        *index += constraint->dir;

			level_data->trav_state = ts_state;
					/* set for each element, to
					   detect user fn.'s actions */
			if (CallUserFunction(root_data, tf_ArrayElem)) {
                            if (us_Finished == root_data->user_status) {
                                goto user_finished;
                            }
                            else {
                                root_data->current_offset = 
                                           base_offset 
                                           + items * level_data->data_length;
                                break;
			    }
                        }
			else if (ts_state == level_data->trav_state) {
			    TraverseSubtypeCore(root_data);
                            if (us_Finished == root_data->user_status) {
                                goto user_finished;
                            }
		        }
			else if (ts_end != level_data->trav_state) {
			    UnexpectedState();
			}
		    }
 		    level_data->dimension--;
                    level_data->trav_state = ts_EndType;
		    break;
		}
		case RecordTypeDeclInfo:
		    for (scan = TempFirstItem(
					qqChildren(level_data->type_info)); 
				! TempNullItem(scan); 
				scan = TempNextItem(scan)) {
			elem_info = SymbInfoValue(scan);
			elem_data = level_data->type_data;

			level_data->trav_state = ts_RecordElement;
			level_data->elem_info = elem_info;
			level_data->elem_data = elem_data;

			level_data->trav_state = ts_RecordElement;
					/* set for each element, to
					   detect user fn.'s actions */

			/* call user function for record element as element */
			if (CallUserFunction(root_data, tf_RecordElem)
                            && us_Finished == root_data->user_status) {
                            goto user_finished;
                        }
			else if (ts_RecordElement == level_data->trav_state) {
					/* if user function didn't call 
					   TraverseSubtype_Recurse to traverse
					   record element value ... */
			    
			    TraverseSubtypeCore(root_data);
					/* recurse on record element value */

                            if (us_Finished == root_data->user_status) {
                                goto user_finished;
                            }
			}
			else if (ts_AfterElement != level_data->trav_state) {
			    UnexpectedState();
			}
		    }
		    level_data->trav_state = ts_EndType;
		    break;	/* in record type */
		
		default:
		    ReportInternalError(
			"Unexpected SymbInfoKind for TraverseSubtypeCore", "",
			__FILE__, __LINE__);
		    break;
	    } /* switch */
	    break;

	case ts_RecordElement:		/* at start of record element */
        {
	    tLevelDataRec new_level_data;
					/* create traversal state record for
					   next-level type */
	    InitLevelData(&new_level_data);

	    new_level_data.next   = level_data;
	    root_data->level_data = &new_level_data;
	    root_data->depth++;

	    elem_info = level_data->elem_info;
	    elem_data = level_data->elem_data;

	    elem_type_info = qqBaseTypeInfo(elem_info);
	    elem_type_data = su_TraverseToData(qqEnclReg(elem_type_info),
		qqEnclReg(level_data->type_info), elem_data);

	    elem_constraint_addr = GetConstraintAddr(elem_info,
		qqEnclReg(qqEnclReg(level_data->elem_info)), 
		level_data->elem_data);

	    new_level_data.trav_state      = ts_StartType;
	    new_level_data.type_info       = elem_type_info;
	    new_level_data.type_data       = elem_type_data;
	    new_level_data.constraint_addr = elem_constraint_addr;

	    TraverseSubtypeCore(root_data);

	    root_data->depth--;
	    root_data->level_data = level_data;

	    level_data->trav_state = ts_AfterElement;

	}   break;

	case ts_ArrayElement:		/* at start of array element */
        {
	    tLevelDataRec new_level_data;
					/* create traversal state record for
					   next-level type */
	    InitLevelData(&new_level_data);

	    new_level_data.next   = level_data;
	    root_data->level_data = &new_level_data;
	    root_data->depth++;

	    elem_type_info = qqBaseTypeInfo(level_data->type_info);
					/* get element type from array type */
	    elem_type_data = su_TraverseToData(qqEnclReg(elem_type_info),
		qqEnclReg(level_data->type_info), level_data->type_data);

	    elem_constraint_addr = GetConstraintAddr(level_data->type_info,
		qqEnclReg(level_data->type_info), 
		level_data->type_data);

	    new_level_data.trav_state      = ts_StartType;
	    new_level_data.type_info       = elem_type_info;
	    new_level_data.type_data       = elem_type_data;
	    new_level_data.constraint_addr = elem_constraint_addr;

	    TraverseSubtypeCore(root_data);

	    root_data->depth--;
	    root_data->level_data = level_data;

	    level_data->trav_state = ts_AfterElement;

	}   break;

	case ts_EndType:
	    printf("\bTraverseSubtypeCore called with state = ts_EndType\n");
	    break;

	default:
	    ReportInternalError("Unexpected tTraversalState", "",
		__FILE__, __LINE__);
	    break;
    } /* switch */

    level_data->data_length = root_data->current_offset - base_offset;
    return;

user_finished: /* fast return */
    level_data->trav_state = ts_None;
    return;

} /* TraverseSubtypeCore */




/*****************************************************************************
*
* TraverseSubtype_Recurse - 
*
* DESCRIPTION:
*	When user function is called for type, it may call TraverseSubtype_-
*	Recurse to have all elements traversed recursively (user function will
*	be called recursively).  User function must not call TraverseSubtype_-
*	again before returning.
*	When user function is called for record or array element, may call
*	TraverseSubtype_Recurse to have element value traversed recursively
*	(user function will be called recursively).  User function must not 
*	call TraverseSubtype_Recurse again before returning.
*	(Want a _NextSibling version such that user function can call it
*	until elements run out.)
*
* HISTORY:
*	24-OCT-91 DSB	Created
*
*****************************************************************************/

void TraverseSubtype_Recurse(trav_data)
tTravDataPtr trav_data;
{
    tRootDataPtr root_data = (tRootDataPtr) trav_data->private;
/**/
    TraverseSubtypeCore(root_data);

} /* TraverseSubtype_Recurse */


static VK_tInteger TraversedLength;

/*****************************************************************************
*
* TraverseSubtype - general subtype traversal interface function
*
* DESCRIPTION:
*
* HISTORY:
*	18-OCT-91 DSB	Created
*	30-DEC-91 AWH	Added appersand to root_data in call to TraversSubtypeCore 
*
*****************************************************************************/

int TraverseSubtype(type_info, type_region_data, constraint_addr, flags, 
	user_fn, user_data)
VK_tSymbInfoHandle type_info;		/* in: lexical type to traverse */
VK_tLARHeaderPtr type_region_data;	/* in: instance information */
VK_tIndexConstraintAddr constraint_addr;
					/* in: constraint address if array */
tTraversalFlags flags;			/* in: when to call user function */
tUserStatus     (*user_fn)();		/* in: user function to call */
tUserDataPtr     user_data;		/* in: data to pass to user function */
{
    tRootDataRec  root_data;
    tLevelDataRec level_data;
/**/

    InitLevelData(&level_data);

    level_data.trav_state      = ts_StartType;
    level_data.type_info       = type_info;
    level_data.type_data       = type_region_data;
    level_data.constraint_addr = constraint_addr;

    root_data.depth            = 0;
    root_data.flags            = flags;
    root_data.current_offset   = 0;
    root_data.level_data       = &level_data;
    root_data.user_fn          = user_fn;
    root_data.user_data        = user_data;
    root_data.user_status      = us_Searching;

    TraverseSubtypeCore(&root_data);

    TraversedLength = root_data.current_offset;

    return (root_data.user_status);

} /* TraverseSubtype */


/*****************************************************************************
*
* Code to try out type traversal interface
*
*****************************************************************************/


typedef struct {
	VK_tDataAddr start_addr;
	StringN buf_ptr;
	Integer4 buf_size;
	VK_tBoolean comma_flag,
	            qualify_flag;
    } tTryUserData;


/* *******************************************************************
*
* TryUserFn2 - prototype user of TraverseSubtype for scalar subelement access
*
* HISTORY:
*	24-OCT-91 DSB	Created
*
******************************************************************* */
tUserStatus TryUserFn2(user_data, trav_data)
tTryUserData *user_data;	
tTravDataPtr trav_data;
{
    VK_tSymbInfoHandle      type_info;
    VK_tInteger             relative_offset;
    VK_tIndexConstraintAddr constraint_addr;

    tTypeClass type_class;
    tTempBuf80 temp_buf;
/**/
    if (IsMember(tf_ScalarType, TravGet_Flags(trav_data))) {
	type_info = TravGet_SymbInfo(trav_data);
	type_class = su_GetTypeInfoTypeClass(type_info);

	relative_offset = TravGet_Offset(trav_data);
	constraint_addr = TravGet_ConstraintAddr(trav_data);


	TempStrCat(user_data->buf_ptr, user_data->buf_size, "\n\t");

	TempStrCat(user_data->buf_ptr, user_data->buf_size, "at offset ");
	sprintf(temp_buf, "%3d", relative_offset);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, temp_buf);
	
	TempStrCat(user_data->buf_ptr, user_data->buf_size, " : ");
	TempStrCat(user_data->buf_ptr, user_data->buf_size, qqName(type_info));
	TempStrCat(user_data->buf_ptr, user_data->buf_size, " range ");
	FormatScalarValue(temp_buf, sizeof(temp_buf), type_class, type_info,
		&constraint_addr->left);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, temp_buf);
	if (VK_Ascending == constraint_addr->dir) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, " to ");
	}
	else if (VK_Ascending == constraint_addr->dir) {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, " downto ");
	}
	else {
	    TempStrCat(user_data->buf_ptr, user_data->buf_size, "BAD DIRECTION");
	}
	FormatScalarValue(temp_buf, sizeof(temp_buf), type_class, type_info,
		&constraint_addr->right);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, temp_buf);
	
	TempStrCat(user_data->buf_ptr, user_data->buf_size, " := ");
	FormatScalarValue(temp_buf, sizeof(temp_buf), type_class, type_info,
		user_data->start_addr + relative_offset);
	TempStrCat(user_data->buf_ptr, user_data->buf_size, temp_buf);

	TempStrCat(user_data->buf_ptr, user_data->buf_size, "; ");
		
    }
/*    printf("<<< %s >>>\n", user_data->buf_ptr);*/
    return us_Searching; /* not finished */
} /* TryUserFn2 */


/* *******************************************************************
*
* TryTraverseType - simple driver function to test TraverseSubtype
*
* HISTORY:
*	18-OCT-91 DSB	Created
*
******************************************************************* */
void TryTraverseType(buffer, buf_size, type_info, type_region_data, 
	constraint_addr, object_info, value_address, current_region_info, 
	current_region_data)
StringN buffer;
Integer4 buf_size;
VK_tSymbInfoHandle type_info;
VK_tLARHeaderPtr type_region_data;
VK_tIndexConstraintAddr constraint_addr;
VK_tSymbInfoHandle object_info;
VK_tDataAddr value_address;
VK_tSymbInfoHandle current_region_info;
VK_tLARHeaderPtr current_region_data;
{
    tTryUserData user_data;
    char big_buffer[10000];
/**/
    user_data.start_addr = value_address;
    user_data.comma_flag = False;
    user_data.qualify_flag = False;

    strcpy(big_buffer, "");
    user_data.buf_ptr = big_buffer;
    user_data.buf_size = buf_size;

    TraverseSubtype(type_info, type_region_data, constraint_addr,
	tf_ALL, TryUserFn2, &user_data);
    printf("TryUserFn2: <<< %s >>>\n", user_data.buf_ptr);

} /* TryTraverseType */


/* *******************************************************************
*
* GSLUserFn1 - prototype user of TraverseSubtype for subtype length
*
* HISTORY:
*	25-OCT-91 DSB	Created
*
******************************************************************* */
tUserStatus GSLUserFn1(user_data, trav_data)
void *user_data;	
tTravDataPtr trav_data;
{
/**/
    TraverseSubtype_Recurse(trav_data);
				   /* recursively process array elements */
    return (us_SkipLevel); 

} /* GSLUserFn1 */


/* *******************************************************************
*
* su_GetSubtypeLength - get word length given base type and constraint
*
* HISTORY:
*	18-OCT-91 DSB	Created
*
******************************************************************* */
VK_tDataLength su_GetSubtypeLength(base_type_info, type_region_data, 
	constraint_addr)
VK_tSymbInfoHandle base_type_info;	/* in: base type information */
VK_tLARHeaderPtr type_region_data;	/* in: base type enclosing region 
					   instance data */
VK_tIndexConstraintAddr constraint_addr;
					/* in: address of constraint */
{
/**/
    TraverseSubtype(base_type_info, type_region_data, constraint_addr,
		    tf_ArrayElem|tf_PseudoElem, GSLUserFn1, NULL);

    return (TraversedLength);

} /* su_GetSubtypeLength */


/*****************************************************************************
*
* Type traversal interface for GetSubTypeInfo
*
*****************************************************************************/

typedef struct {
	VK_tInteger        current_offset,
                           start_offset,
                           span,
                           end_offset,
                           dimension,
                           data_length;
        VK_tBoolean        finished;
        VK_tSymbInfoHandle subtype_info;
    } tGSIUserData;

/* *******************************************************************
*
* GSIUserFn1 - user of TraverseSubtype for subtype info
*
* HISTORY:
*	25-OCT-91 DSB	Created
*
******************************************************************* */
tUserStatus GSIUserFn1(user_data, trav_data)
tGSIUserData *user_data;	
tTravDataPtr trav_data;
{
    VK_tIndexConstraintAddr constraint;
    VK_tSymbInfoHandle      saved_info;
    VK_tInteger             start_offset,
                            item_length,
                            array_end,
                            saved_dimn,
                            items;
    VK_tBoolean             possible;
    tUserStatus             user_status = us_Searching;
    tTraversalFlag          flag;
/**/
    flag = TravGet_Flags(trav_data);

    if (user_data->current_offset <= user_data->start_offset) {
        possible   = TRUE;
        saved_info = user_data->subtype_info;
        saved_dimn = user_data->dimension;

        user_data->subtype_info = TravGet_SymbInfo(trav_data);
        user_data->dimension    = TravGet_Dimension(trav_data);
    }
    else {
        possible   = FALSE;
    }

    if (IsMember(flag,tf_RecordType)) {
        start_offset = user_data->current_offset;
	TraverseSubtype_Recurse(trav_data);
					/* recursively process record type */
        if (user_data->finished) {
            return us_Finished;
        }

        user_data->data_length = user_data->current_offset - start_offset;
    }
    else if (IsMember(flag,tf_ArrayElem|tf_PseudoElem)) {
        user_data->data_length = 0;
        start_offset           = user_data->current_offset;
	TraverseSubtype_Recurse(trav_data);
					/* recursively process array type */
        if (user_data->finished) {
            return us_Finished;
        }

        constraint = TravGet_ConstraintAddr(trav_data);

        items = constraint->right - constraint->left;
        if (0 > items) {
            items = -items;
        }
        item_length            = user_data->data_length;
        user_data->data_length = item_length * (items +1);
        array_end = start_offset + user_data->data_length;
        if (array_end <= user_data->start_offset) {
            /* start is beyond array */
            user_status               = us_SkipLevel;
            user_data->current_offset = array_end;
        }
        else if (array_end < user_data->end_offset) {
            /* end is beyond array */
            user_status               = us_SkipLevel;
            user_data->current_offset = array_end;
        }
        else if (user_data->span <= item_length) {
            /* is (in) a sub-dimension of this array 
               - normalise search for next index */
            user_data->current_offset = 0;
            user_data->start_offset   = 
                        (user_data->start_offset - start_offset) % item_length;
            user_data->end_offset     = user_data->start_offset 
                                                             + user_data->span;
        }
    }
    else if (IsMember(flag,tf_ScalarType)) {
        user_data->data_length = ScalarUnionWordLength(
                                     su_GetTypeInfoTypeClass(
                                          TravGet_SymbInfo(trav_data)));
        user_data->current_offset += user_data->data_length;
    }

    if (user_data->current_offset >= user_data->end_offset) {
        user_data->finished = TRUE;
        user_status         = us_Finished;
    }
    else if (possible) {
        user_data->subtype_info = saved_info;
        user_data->dimension    = saved_dimn;
    }

    return (user_status);

} /* GSIUserFn1 */


/* *******************************************************************
*
* su_GetSubtypeInfo - get type info for a slice
*
* HISTORY:
*	17-DEC-91 DKC	Created
* 
******************************************************************* */
VK_tSymbInfoHandle su_GetSubtypeInfo(base_type_info, type_region_data,
                                     constraint_addr,
                                     start_offset, data_length, dimensions)
VK_tSymbInfoHandle      base_type_info;	  /* in: base type information       */
VK_tLARHeaderPtr        type_region_data; /* in: base type enclosing region 
					         instance data               */
VK_tIndexConstraintAddr constraint_addr;  /* in: address of constraint       */
VK_tIntegerData    start_offset,
                   data_length,
                   *dimensions;           /* out: address for return data    */
{
    tGSIUserData user_data;
/**/
    user_data.current_offset = 0;
    user_data.dimension      = 0;
    user_data.finished       = FALSE;
    user_data.start_offset   = start_offset;
    user_data.span           = data_length;
    user_data.end_offset     = start_offset + data_length;
    user_data.subtype_info   = NULL;

    TraverseSubtype(base_type_info, type_region_data, constraint_addr,
		    tf_RecordType|tf_ArrayElem|tf_PseudoElem|tf_ScalarType,
                    GSIUserFn1, &user_data);

    if (0 == user_data.dimension) {
        *dimensions = 0;
    }
    else {
        *dimensions = qqDimensions(user_data.subtype_info) 
                             + 1 - user_data.dimension;
    }

    return user_data.subtype_info;

} /* su_GetSubtypeInfo */


#if 0
/* *******************************************************************
*
* RtnName - <description of function or procedure>
*
* HISTORY:
*	??-FEB-91 DSB	Created
*
******************************************************************* */
<static> Fntype RtnName(parm)	/* return value desc. if not void */
tParmType parm;			/* mode: description */
{
    type local;				/* description */
    type local;				/* description */
/**/

} /* RtnName */


#endif

