#ifndef TKSymbUtils_H
#define TKSymbUtils_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKSymbUtils.h
*
* DESCRIPTION:  
*	Private interface file for TKSymbUtils.c.  <then describe contents>
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*   External:
*	...
*
* HISTORY:
*	21-FEB-91 DSB	Created
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	10-MAY-91 JBR	W0041.  Added su_GetUnitNames.
*	17-MAY-91 DSB	Moved TK_{Get,Set]RegionHandle from VKI file to here
*	30-MAY-91 DSB	Moved su_GetTypeInfoTypeClass to here from TKSignals.c
*	12-JUN-91 DSB	Added SIH_Std__Standard, SIH_Std__Standard__String, 
*			SIH_Std__TextIO
*	14-AUG-91 DSB	Added su_FindLibrary
*	23-AUG-91 DSB	Split TKSymbInfo into TKSymbUtils and TKRegister
*	28-OCT-91 DSB	Added su_ prefix to several names
*
* SCCS ID: @(#)TKSymbUtils.h	42.4 1/13/92
*
******************************************************************* */

#include "VKSymbTypes.h"	/* for k_region_handle, etc. */
#include "TKStructs.h"		/* for tTypeClass, etc. */


#define TK_GetRegionHandle(cur_reg_LARP) \
	((cur_reg_LARP)->k_region_handle) /* get region's RegionHandle from
					   region's LAR */

#define TK_SetRegionHandle(cur_reg_LARP, reg_handle) \
	((cur_reg_LARP)->k_region_handle = reg_handle)
					/* set region's RegionHandle */

#define TK_GetRegionInfo(cur_reg_LARP) \
	((cur_reg_LARP)->k_region_info) /* get region's symb. info. handle from
					   region's LAR */
#define TK_SetRegionInfo(cur_reg_LARP, symb_info_handle) \
	((cur_reg_LARP)->k_region_info = symb_info_handle)
					/* set region's symb. info. handle */

#define TK_GetUnitInfo(cur_reg_LARP) \
	((cur_reg_LARP)->k_unit_info) /* get unit's symb. info. handle from
					   region's LAR */
#define TK_SetUnitInfo(cur_reg_LARP, symb_info_handle) \
	((cur_reg_LARP)->k_unit_info = symb_info_handle)
					/* set unit's symb. info. handle */

#define TK_GetExecPos(cur_reg_LARP) \
	((cur_reg_LARP)->k_exec_pos_index)
					/* get execution position index from
					   region's LAR */
#define TK_SetExecPos(cur_reg_LARP, exec_pos_index) \
	((cur_reg_LARP)->k_exec_pos_index = exec_pos_index)
					/* set unit's symb. info. handle */



tTypeClass su_GetTypeInfoTypeClass(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle type_info
#endif
	);


VK_tSymbInfoHandle su_GetSubtypeInfo(
#ifdef VK_ANSI_FN_PROTOS
        VK_tSymbInfoHandle      base_type_info,
        VK_tLARHeaderPtr        type_region_data,
        VK_tIndexConstraintAddr constraint_addr,
        VK_tIntegerData         start_offset,
        VK_tIntegerData         data_length,
        VK_tIntegerData         *dimensions
#endif
	);


void su_GetUnitNames();	/* ?? use ANSI C function prototype ?? */


void su_IdentifyPosition(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr local_act_rec,	/* in: LAR of region to identify */
	VK_tName * library,		
	VK_tName * primary,		
	VK_tName * secondary,		
	VK_tName * file,		
	VK_tLineNumber * line_number,	/* &out: last line number traced */
	VK_tSymbInfoHandle * region_info
					/* &out: current region symb. info. */
#endif
	);

VK_tLARHeaderPtr su_TraverseToData(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle target_region_info,
					/* in: requested region's info. */
	VK_tSymbInfoHandle reference_region_info,
					/* in: referring region's info. */
	VK_tLARHeaderPtr reference_data	/* in: referring region's LAR */

#endif
	);				/* returns LAR of target region */

VK_tSymbInfoHandle su_FindLibrary(
#ifdef VK_ANSI_FN_PROTOS
	VK_tName library_name		/* in: DLS library name string */
#endif
	);				/* returns LibraryRegion symb. info. */

/*****************************************************************************
*
* Declarations for general type traversal
*
*****************************************************************************/

/*****************************************************************************/
/* 
New traversal interface design notes:

* General flow:
	- User calls TraverseSubtype.  User specifies subtype to traverse by
		specifying type's symbolic information, instance data, and
		index constraint (for array type).  User provides user function
		to call and user data to pass to function.  User gives set
		of flag to specify classes of traversal points at which to 
		call user function. (Note:  This selection is not implemented 
		yet.)
	- TraverseSubtype traverses given subtype.  It calls user function at
		selected points (currently, at every point).  It passes user
		data and traversal data to user function.
	- User function uses user data (directly) and traversal data (through
		given macros ONLY) as needed.
	- User function may return to continue traversal, or call Traverse-
		Subtype_Recurse to process child items and then return.  See
		"Traversal options and state transitions" for use of 
		TraverseSubtype_Recurse.

* Notes on intended use:
	- TraverseSubtype is intended to be used to traverse a subtype for
		creating an object (e.g., a composite signal), for displaying
		a value, or for any other processing requiring similar
		traversal.
	- TraverseSubtype maintains the relative offset to the current element
		so the user can retrieve or create data at an appropriate 
		offset from the beginning of an object.
	- TraverseSubtype maintains an array of index values for all dimensions
		of an array.  This is so the user does not have to accumulate
		index values.
	- TraverseSubtype_Recurse is provided to emulate some co-routine
		processing.  See below.
		
* Traversal points (state of traversal when traversal routines call or return
	to user function):
	- at scalar type (or access type or file type)
	- at record type
	- at record element
	- at array type
	- at array element
	- at pseudo-array type (for last N-1 dimensions of array)
	- at array pseudo-element (for first N-1 dimensions of array)

* Traversal options and state transitions
    1.  when user function returns:
	- traverse to next call point and call user function
    2.  When user function calls TraverseSubtype_Recurse:
	- (The intent of TraverseSubtype_Recurse is to allow an activation of
		the user function to remain active while subelements are pro-
		cessed.  For example, the user function might print "(" to 
		begin an aggregate for a type, then call TraverseSubtype_-
		Recurse to process element types, print ")" to terminate the
		aggregate, and then return.)
	- if at composite type (start): processes elements of type and their
		types recursively (calling user function); at return, position
		is after type (for multi-dimensional array type, processes 
		pseudo-elements for array dimensions; for pseudo-type for 
		dimension, processes pseudo-elements or actual elements)
	- (not to be called for scalar type)
	- if at record or array element, or array dimension pseudo-element,
		processes element's type recursively (calling user function);
		at return, position is after element
	- (not to be called at end of type or end of element) 

  The following are not implemented.  These are kept as design notes in case
	we extend the interface:
    3.  when user function calls NextSibling:
	- if at composite type (start): processes elements and their types 
		recursively (calling user function); at return, position is 
		special "end of type" or "end or elements"
	- (if at scalar type, no operation)
	- if at record or array element: processes element's type recursively 
		(calling user function)); at return, position is next element;
		if no next element, then position is special "end of type" or 
		"end of elements"
	- if at end of type or end or elements: error or no operation
    4.  when user function calls NextPosition??:
	- if at composite type (start): at return, position is first element;
		(if not element, position is special "end of type" or "end of 
		elements") 
	- if at record or array element: at return, position is at (start of)
		element's type
	- (definition: "blocked" means to not traverse up enough to change
		the position expected after a stacked NextSibling call)
	- if at scalar type: if blocked, do nothing; otherwise, traverse up
		to enclosing type's next element or special "end of type" or
		"end of elements" position; repest NextPosition until blocked
	- if at special "end of type" or "end of elements" position: same as
		for scalar type
	- if called when blocked: might be error (user should return when 
		first blocked)
	- (Note:  probably can implement without NextSibling for now)


(One use to use switch on type class, use NextPosition to get from array or 
	record type to first element; use NextSibling to process element
	recursively and get to next element; at end, return to let 
	ancestor NextPosition or NextSibling call complete.)
*/
/*****************************************************************************/

typedef enum {
        us_Searching,  /* not finished      */
        us_SkipLevel,  /* skip to next item */
        us_Finished    /* all done          */
    } tUserStatus;


/*****************************************************************************
*
* IsMember - 
*
* (should this have prefix su_? since this is general purpose, should it be
*	in a more general utilities module?)
*
*****************************************************************************/

#define IsMember(flag, flags) (((flags) & (flag)) != 0)


/*****************************************************************************
*
* type tTraversalFlags, etc. - 
*
*****************************************************************************/

typedef long int tTraversalFlags;	/* control and status flags */

typedef enum {
        tf_ScalarType = (1<<0),	/* scalar, access, or file type */
        tf_RecordType = (1<<1),	/* record or array type */
        tf_ArrayType  = (1<<2),	/* record or array type */
        tf_RecordElem = (1<<3),	/* record element */
        tf_ArrayElem  = (1<<4),	/* array element */
        tf_PseudoType = (1<<5),	/* array dimension group */
        tf_PseudoElem = (1<<6)	/* array dimension group */
    } tTraversalFlag;

#define tf_NONE 0
#define tf_ALL \
	(tf_ScalarType | tf_RecordType | tf_ArrayType | tf_PseudoType | \
	 tf_RecordElem | tf_ArrayElem | tf_PseudoElem )


/*****************************************************************************
*
* type tIndexArrayPtr - array of one index value per array dimension
*
*****************************************************************************/
typedef VK_tInteger tIndexArrayRec;
typedef tIndexArrayRec * tIndexArrayPtr;

/*****************************************************************************
*
* types tTravDataRec, tTravDataPtr
*
* NOTE: TraverseSubtype users: DO NOT use these fields directly; read them 
*	using TravGet_ macros below
*
*****************************************************************************/
typedef struct {
	VK_tVoidPtr private;
	tTraversalFlags flags;
	VK_tSymbInfoHandle symb_info;

    } tTravDataRec, * tTravDataPtr;


/*****************************************************************************
*
* types tUserFnPtr, tUserDataPtr - 
*
*****************************************************************************/
typedef VK_tVoidPtr tUserDataPtr;

typedef tUserStatus (tUserFn)(		/* user function called by traversal */
#ifdef VK_ANSI_FN_PROTOS
	tUserDataPtr user_data,		/* in: user data */
	tTravDataPtr trav_data		/* in: current traversal point data */
#endif
	);	

typedef tUserFn * tUserFnPtr;		/* pointer to user function */



/*****************************************************************************
*
* TraverseSubtype - general subtype traversal function
*
*****************************************************************************/
int TraverseSubtype(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle type_info,	/* in: lexical type to traverse */
	VK_tLARHeaderPtr type_region_data,
					/* in: instance information */
	VK_tIndexConstraintAddr constraint_addr,
					/* in: constraint address if array */
	tTraversalFlags flags,		/* in: when to call user function */
	tUserFnPtr user_fn,		/* in: user function to call */
	tUserDataPtr user_data		/* in: data to pass to user function */
#endif
	);
/* 
The first three parameters identify (the instance of) the subtype to traverse:
Parameter type_info is the symbolic information handle of a type.  
Parameter type_region_data is the local activation record handle of an 
	instance of the region containing (an instance of) the type designated
	by type_info.
Parameter constraint_addr is the address of an index constraint defining the
	subtype, if type_info specifies an array type.
Parameter flags specifies the set of traversal points at which to call the user
	function.  NOTE: Parameter flags is not implemented yet.  The user 
	function is called at every traversal point.
Parameter user_fn is a function to be called at selected traversal points.
Parameter user_data is user data to be passed to the user function whenever it
	is called.
*/
/* Note:  Also see notes above.  */


/*****************************************************************************
*
* TraverseSubtype_Recurse - co-routine-style reentry point
*
*****************************************************************************/
void TraverseSubtype_Recurse(
#ifdef VK_ANSI_FN_PROTOS
	tTravDataPtr trav_data
#endif
	);
/* Note:  See notes above.  */



/*****************************************************************************
*
* Traversal position data retrieval macros
*
*****************************************************************************/

/* TravGet_Flags - get flags to identify current callback position */

#define TravGet_Flags(trav_data_ptr) ((trav_data_ptr)->flags)


/* TravGet_SymbInfo - get identification of type or enclosing type (for cases
	other than record element) or of record element (for record element) */

#define TravGet_SymbInfo(trav_data_ptr) ((trav_data_ptr)->symb_info)


/* TravGet_Dimension - get current dimension of array type (for pseudo-element,
	pseudo-type, or array element); note: first dimension is 1 (not 0) */

VK_tInteger VK_tTravGet_Dimension(
#ifdef VK_ANSI_FN_PROTOS
	tTravDataPtr trav_data
#endif
    );

/* TravGet_IndexArray - get index value array (for array element, or pseudo-
	element, (or dimension pseudotype)); note: only valid up to current
	dimension */

tIndexArrayPtr TravGet_IndexArray(
#ifdef VK_ANSI_FN_PROTOS
	tTravDataPtr trav_data
#endif
    );

/* TravGet_Offset - get word offset to beginning of current subelement (or
	pseudoelement); offset is relative to starting TraverseSubtype call */

VK_tInteger TravGet_Offset(
#ifdef VK_ANSI_FN_PROTOS
	tTravDataPtr trav_data
#endif
    );

/* TravGet_ConstraintAddr - get address of constraint for type */

VK_tIndexConstraintAddr TravGet_ConstraintAddr(
#ifdef VK_ANSI_FN_PROTOS
	tTravDataPtr trav_data
#endif
    );

#endif /* TKSymbUtils_H */
