#ifndef TKTemp_H
#define TKTemp_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: TKTemp.h
*
* DESCRIPTION:  
*	Interface file for TKTemp.c.
*
* HISTORY:
* 	11-Sep-90 RM	Created
*	25-APR-91 AWH	WOO21: Renamed to TK...
*	02-MAY-91 DSB	Cleaning up generally
*	16-MAY-91 DSB	Moved TKSymTab.h, much of TKMisc.h to here
*	16-MAY-91 DSB	Renamed string_match to StringMatch, 
*			partial_string_match to StringMatchPrefix
*	10-JUN-91 DSB	Added {Allocate,Free}{Time,Value}Buffer
*	29-JUL-91 DSB	Renamed VERIFY to VERIFY_ERROR (a la code generator)
*	19-NOV-91 DKC	Added AllocateBlock(Zero) macro interface to
*                       iAllocateMemory
*
* SCCS ID: @(#)TKTemp.h	42.1 1/10/92
*
******************************************************************* */

#include "TKCommon.h"		/* for  */
#include "TKData.h"		/* for tAbsTimePtr */

/*****************************************************************************
*
* tStaticStrBuf - a dynamically resizable string buffer record
*
*      If the resizable field is false then no resiszing is done.
*
* NOTES: The string is allocated memory with (Re)AllocateMemory
*
*****************************************************************************/

typedef struct sStaticStrBuf {
        char            resizable;
        unsigned short  buffer_size; /* max 64k */
        VK_tCString     string;
    } tStaticStrBuf;

typedef tStaticStrBuf *StaticStrBufPtr;

#define STATIC_STR_INITIAL {TRUE,0,""}

#define INITIALIZE_STATIC_STR(str_buff) {(str_buff)->resizable   = TRUE;\
                                         (str_buff)->buffer_size = 0;\
                                         (str_buff)->string      = NULL;}

#define DESC_STR_APPEND(buffer,buf_size,str_buff) \
    {int len = strlen(buffer);\
     (str_buff)->resizable   = FALSE;\
     (str_buff)->string      = &(buffer)[len];\
     (str_buff)->buffer_size = (buf_size) - len;}

#define DESC_STR_BUFFER(buffer,buf_size,str_buff) \
    {(str_buff)->resizable   = FALSE;\
     (str_buff)->string      = (buffer);\
     (str_buff)->buffer_size = (buf_size);}

VK_tBoolean StringMatch(
#ifdef ANSI_FN_PROTOS
	VK_tCString s1, VK_tCString s2
#endif
	);

VK_tBoolean StringMatchPrefix(
#ifdef ANSI_FN_PROTOS
	VK_tCString s1, VK_tCString s2
#endif
	);

Boolean1 TempStrCat(
#ifdef ANSI_FN_PROTOS
	char buffer[],			/* in: buffer to which to append */
	int buf_size,			/* in: total buffer size */
	StringN string			/* in: string to append */
#endif
	);			/* returns True if successful */

Boolean1 TempStrCpy(
#ifdef ANSI_FN_PROTOS
	char buffer[],			/* in: buffer to set */
	int buf_size,			/* in: total buffer size */
	StringN string			/* in: string to stuff */
#endif
	);			/* returns True if successful */

VK_tCString CopyPrivStr();
char        *AllocTempSpace();
void        DeAllocTempSpace();

VK_tCString DuplicateString(
#ifdef VK_ANSI_FN_PROTOS
	VK_tCString original
#endif
	);

/* *******************************************************************
*
* AllocateMemory - Allocate Memory, check result
*
******************************************************************* */
#define AllocateMemory(POINTER, TYPE, COUNT)\
	POINTER = (TYPE *) iAllocateMemory(sizeof(TYPE) * (COUNT), FALSE)
/* ?? STYLE: this macro should be ALLOCATE_MEMORY because it is a syntactic
	macro (the cast can not be done by a regular function) ?? */

#define AllocateMemoryZero(POINTER, TYPE, COUNT)\
	POINTER = (TYPE *) iAllocateMemory(sizeof(TYPE) * (COUNT), TRUE)

#define AllocateBlock(POINTER, TYPE, SIZE)\
	POINTER = (TYPE *) iAllocateMemory(SIZE, FALSE)
/* ?? STYLE: this macro should be ALLOCATE_MEMORY because it is a syntactic
	macro (the cast can not be done by a regular function) ?? */

#define AllocateBlockZero(POINTER, TYPE, SIZE)\
	POINTER = (TYPE *) iAllocateMemory(SIZE, TRUE)

void * iAllocateMemory(
#ifdef ANSI_FN_PROTOS
	Integer4  size		/* in: size of request in bytes */
#endif
);
/* ?? style: ");" should be indented one tab -DSB ?? */

/* *******************************************************************
*
* ReallocateMemory - Re-allocate Memory, check result
*
******************************************************************* */
#define ReallocateMemory(POINTER, TYPE, COUNT)\
	POINTER = (TYPE *) iReallocateMemory(POINTER, sizeof(TYPE) * (COUNT))


void * iReallocateMemory(
#ifdef ANSI_FN_PROTOS
	void * pointer,			/* in: pointer to existing block */
	Integer4  size		/* in: size of request in bytes */
#endif
);
/* ?? style: ");" should be indented one tab -DSB ?? */

/* *******************************************************************
*
* FreeMemory -- Free Memory allocated with AllocateMemory or ReallocateMemory
*
* Note: implemented here for completeness
*
******************************************************************* */
#define FreeMemory(POINTER) iFreeMemory(POINTER)

void iFreeMemory(
#ifdef ANSI_FN_PROTOS
	void * pointer			/* in: pointer to existing block */
#endif
);
/* ?? style: ");" should be indented one tab -DSB ?? */



void TryTraceback(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr current_region_data
#endif
	);

void ReportInternalError(
#ifdef VK_ANSI_FN_PROTOS
	VK_tCString message,		/* in: error message */
	VK_tCString extra,		/* in: extra message */
	VK_tCString file,		/* in: C source file */
	Integer4 line			/* in: line number */
#endif
	);

/* *******************************************************************
*
* ExitProgram - determine exit return code based on globals and exit
*
******************************************************************* */

void ExitProgram();

/* *******************************************************************
*
* ReportKernelStatus - report simulation error from kernel
*
******************************************************************* */

typedef enum {
        es_Info,
        es_Warning,
        es_Error,
        es_Fatal
   } tErrorSeverity;

void ReportKernelStatus(
#ifdef ANSI_FN_PROTOS
        tErrorSeverity severity,       
	VK_tCString    message,		/* main error message */
	VK_tCString    extra,		/* extra information */
	VK_tCString    source_file,	/* source file name */
	int            line_number	/* VHDL source file line number */
#endif
	);

/* *******************************************************************
*
* VERIFY_ERROR - Report failed assertion
* VERIFY_SET   - Set something for later verification
* UNVERIFIED   - Thing to do if no verification
*
******************************************************************* */
#ifdef VK_NO_VERIFY
#  define VERIFY_ERROR(COND)
#  define VERIFY_SET(STMT)
#  define UNVERIFIED(STMT) STMT
#else
#  ifdef TK_CPP_NO_LARGE_STRINGS
#    define VERIFY_ERROR(COND)\
       if (! (COND) ) {\
	  ReportVerifyFailed("<NIY>", __FILE__, __LINE__);\
       }
#  else
#    define VERIFY_ERROR(COND)\
       if (! (COND) ) {\
	  ReportVerifyFailed("COND", __FILE__, __LINE__);\
       }
#  endif
#  define VERIFY_SET(STMT) STMT
#  define UNVERIFIED(STMT) 
#endif

/* *******************************************************************
*
* VERIFY_WARN - Report failed assertion (less severe)
*
******************************************************************* */

void TempVerifyWarn(
#ifdef ANSI_FN_PROTOS
	char *cond_str,
	int line,
	char *file
#endif
	);

#ifdef VK_NO_VERIFY
#  define VERIFY_WARN(COND)
#else
#  ifdef TK_CPP_NO_LARGE_STRINGS
#    define VERIFY_WARN(COND)\
       if (! (COND) ) {\
	  TempVerifyWarn("<NIY>", __FILE__, __LINE__);\
       }
#  else
#    define VERIFY_WARN(COND)\
       if (! (COND) ) {\
	  TempVerifyWarn("COND", __FILE__, __LINE__);\
       }
#  endif
#endif

/* *******************************************************************
*
* TerminateKernel - terminate simulation
*
******************************************************************* */

void TerminateKernel(
#ifdef ANSI_FN_PROTOS
	void
#endif
);
/* ?? style: ");" should be indented one tab -DSB ?? */

/* *******************************************************************
*
* EarlyTerminate - base on error flags terminate
*
******************************************************************* */

void EarlyTerminate(
#ifdef ANSI_FN_PROTOS
	int summarize
#endif
    );


#define ReportKernelError(message, extra, source_file, line_number)\
        ReportKernelStatus(es_Fatal,message, extra, source_file, line_number)

#define ReportKernelNonFatal(message, extra, source_file, line_number)\
        ReportKernelStatus(es_Error,message, extra, source_file, line_number)

/* *******************************************************************
*
* ReportVerifyFailed - report verify failed
*
******************************************************************* */
void ReportVerifyFailed (
#ifdef ANSI_FN_PROTOS
	VK_tCString condition,		/* in: boolean condition */
	VK_tCString file,		/* in: C source file */
	Integer4    line  		/* in: line number */
#endif
    );


/* *******************************************************************
*
* ReportVerifyMemoryFailed - report verify memory operation failed
*
******************************************************************* */
void ReportVerifyMemoryFailed (
#ifdef ANSI_FN_PROTOS
	VK_tCString operation,		/* in: memory function */
	VK_tCString allocation,		/* in: type allocated */
	VK_tCString file,		/* in: C source file */
	Integer4 line 		/* in: line number */
#endif
);
/* ?? style: ");" should be indented one tab -DSB ?? */


#endif /* TKTemp_H */

