#ifndef VKConfig_H
#define VKConfig_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc. All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKConfig.h
*
* DESCRIPTION:  
*	Host-dependent configuration file for VHDL Kernel Interface 
*	(uncustomized VKI).
*
* PREPROCESSOR CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
*	28-FEB-91 DSB	Created
*	19-MAR-91 DSB	Cleaning up.
*
* SCCS ID: @(#)VKConfig.h	42.2 1/13/92
*
******************************************************************* */


/* *******************************************************************
*
* control macro TK_SPECIFIC - controls code for test kernel's implementation
*
* USAGE:  This is not part of the VKI; it controls code that is not
*	part of the VKI.
*
******************************************************************* */

#ifndef TK_SPECIFIC			/* (overridable without editting) */
#  define TK_SPECIFIC 1			/* default VKI is for test kernel */
#endif


/* *******************************************************************
*
* Preprocessor code to determine machine type, etc.
*
* USAGE:  Default VKI only.
*
******************************************************************* */

#ifdef VAXC
#  define VK_OS_VMS			/* operating system is VMS */
#  define VK_HW_VAX			/* hardware is VAX */
#else

#ifdef sun
#  define VK_OS_SUNOS			/* operating system is SunOS */
#  ifdef sparc
#    define VK_HW_SUN4			/* hardware is Sun 4 */
#  else
#    ifdef mc68020
#      define VK_HW_SUN3		/* hardware is Sun 3 */
#    else
      #error Unknown Sun hardware version
#    endif
#  endif
#else

#ifdef aegis
#  define VK_OS_AEGIS			/* operating system is AEGIS */
#  define VK_HW_APOLLO			/* hardware is Apollo */
#else

#ifdef _IBMR2
#  define VK_OS_AIX3			/* operating system is AIX v. ? */
#  define VK_HW_RS			/* hardware is IBM RS (6000) */
#else

  #error Host hardware type and operation system are not determined

#endif	/* _IBMR2 */
#endif	/* aegis */
#endif	/* sun */
#endif	/* VAXC */


/* *******************************************************************
*
* Preprocessor code to select various ifdef controls
*
******************************************************************* */


/* *******************************************************************
*
* Control macros VK_LITTLE_ENDIAN and VK_BIG_ENDIAN
*
* DESCRIPTION:
*	Macros VK_LITTLE_ENDIAN and VK_BIG_ENDIAN specify the packing of bytes
*	within words.  If "*((char *) & int_var)" returns the least significant
*	byte of int_var, then a machine is little-endian.  If the expression 
*	returns	the most significant byte, then it is big-endian.
*
*	These macros are used to select the appropriate structure declaration
*	to construct two-word records that are consistent with the machine's
*	byte ordering within words.
*
* USAGE: Defined by VKI; used by run-time library; used by default VKI; not
*	used by generated code.
*
******************************************************************* */

#ifdef VK_HW_VAX
#  define VK_LITTLE_ENDIAN
#else

#ifdef VK_HW_SUN4
#  define VK_BIG_ENDIAN
#else

#ifdef VK_HW_SUN3
#  define VK_BIG_ENDIAN
#else

#ifdef VK_HW_RS
#  define VK_BIG_ENDIAN
#else

  #error Unknown host code for VK_LITTLE_ENDIAN vs. VK_BIG_ENDIAN selection

#endif /* VK_HW_RS */
#endif /* VK_HW_SUN3 */
#endif /* VK_HW_SUN4 */
#endif /* VK_HW_VAX */


/* *******************************************************************
*
* Control macros VK_ANSI_FN_PROTOS, etc.
*
* DESCRIPTION:
*   VK_ANSI_FN_PROTOS selects ANSI C function prototypes vs. old C function 
*	allusions.  It is used by the run-time library also.
*   VK_ANSI_VOID_STAR selects ANSI C "void *" for generic pointers vs. faking
*	it with "char *".
*   VK_ANSI_ENUM selects between ANSI C interpretation of enumeration types as 
*	integers vs. interpretation as distinct from integers.
*
*   (TK_ANSI_VAR_ARGS selects between old C and ANSI C versions of va_start; 
*	it is intended for ONLY the test kernel).
*
* USAGE:  Defined by VKI; used by default VKI; used by RTL; not used by
*	generated code.
*
******************************************************************* */

#ifdef __STDC__
#  if __STDC__				/* if ANSI C, then has features */
#    define VK_ANSI_FN_PROTOS
#    define VK_ANSI_VOID_STAR
#    define VK_ANSI_ENUM
#    define TK_ANSI_VAR_ARGS
#  else
#    define VK_NOT_SELECTED_YET
#  endif
#else
#  define VK_NOT_SELECTED_YET
#endif

#ifdef VK_NOT_SELECTED_YET

# ifdef VK_HW_VAX
#    define VK_ANSI_FN_PROTOS
#      define VK_ANSI_VOID_STAR
     /* no VK_ANSI_ENUM */
#      define TK_ANSI_VAR_ARGS
#  else

#  ifdef VK_HW_SUN4
     /* no VK_ANSI_FN_PROTOS */
     /* no VK_ANSI_VOID_STAR */
     /* no VK_ANSI_ENUM */
     /* no TK_ANSI_VAR_ARGS */
#  else

#  ifdef VK_HW_SUN3
     /* no VK_ANSI_FN_PROTOS */
     /* no VK_ANSI_VOID_STAR */
     /* no VK_ANSI_ENUM */
     /* no TK_ANSI_VAR_ARGS */
#  else

#  ifdef VK_HW_RS
#    define VK_ANSI_FN_PROTOS
#    define VK_ANSI_VOID_STAR
#    define VK_ANSI_ENUM
#    define TK_ANSI_VAR_ARGS
#  else

    #error Unknown host code for ANSI features

#  endif /* VK_HW_RS */
#  endif /* VK_HW_SUN3 */
#  endif /* VK_HW_SUN4 */
#  endif /* VK_HW_VAX */

#endif /* VK_NOT_SELECTED_YET */

#ifdef TK_ANSI_VAR_ARGS
#  define VA_ALIST 
#  define VA_DCL 
#  define VA_START(v,last_arg) va_start(v,last_arg)
#else
#  define VA_ALIST             ,va_alist
#  define VA_DCL               va_dcl
#  define VA_START(v,last_arg) va_start(v)
#endif

#endif /* VKConfig_H */




