#ifndef VKElaborate_H
#define VKElaborate_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKElaborate.h
*
* DESCRIPTION:  
*	VHDL kernel interface routines for instantiation and elaboration.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
*	23-JUL-91 DSB	Added VK_ElaborateDefaultArchitecture, basic
*			VK_ElaborateDefaultEntity
*	23-AUG-91 DSB	Created from VKMisc.h
*
* SCCS ID: @(#)VKElaborate.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"	/* for -VHDLString, -CString, etc. */
#include "VKCodeIntf.h"		/* for VK_tLARHeaderPtr */



/* *******************************************************************
*
* 0:  Test kernel registration, instantiation, and elaboration protocols
*
******************************************************************* */


/*****************************************************************************
*
* TK_RegisterUnits -  call registration routine in generated config. module
*
* NOTE: TEST KERNEL IMPLEMENTATION ONLY
*
*****************************************************************************/
void TK_RegisterUnits(
#ifdef VK_ANSI_FN_PROTOS
	void
#endif
	);


/* *******************************************************************
*
* TK_SpecifyTopUnit - specify top unit to instantiate
*
* NOTE: TEST KERNEL IMPLEMENTATION ONLY
*
******************************************************************* */
void TK_SpecifyTopUnit(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle unit_info	/* architecture body, configuration
					   declaration, or entity decl. */
#endif
	);


/* *******************************************************************
*
* 1:  Elaboration protocols
*
******************************************************************* */

/* Elaboration protocol:  General */

/* Specification:  
    The kernel must instantiate design entities for three cases:
	1)  a top-level entity (that is, one not instantiated under a VHDL
		component instantiation statement)
	2)  when the VHDL configuration information for a component 
		instantiation does not specify an entity declaration
	3)  when the VHDL configuration information for a component
		instantiation specifies an entity declaration but not a 
		specify architecture body.
    For the first case, the kernel may instantiate whichever design 
	entity(-ies) or configuration declaration(s) it wants, for example
	to instantiate multiple VHDL entities under a non-VHDL netlist.
    For the second or third case, the kernel may select and instantiate a VHDL
	design entity according to VHDL rules.  (It may also instantiate 
	non-VHDL under the VHDL component instantiation.)

    The kernel is not directly involved in a fourth case of instantiation.
	When the VHDL configuration information for a component instantiation
	specifies an entity declaration and an architecture (or a configuration
	declaration), the generated code for the component instantiation and
	configuration specification directly instantiates the design entity
	(or configuration declaration).
*/

/* Note:  The kernel_data parameter of VK_tBindFn and of VK_GetActualGeneric
	and VK_GetActualPort is intended to make it easy for the kernel to
	pass context information from VK_ElaborateDefaultEntity and 
	VK_ElaborateDefaultArchitecture to VK_GetActualGeneric and 
	VK_GetActualPort. */

/* Elaboration protocol:  Instantiating a design entity at the top level */

/* Specification:  The kernel must perform the following actions:
    The kernel must select an architecture body or a configuration declaration
	(according to whatever rules the kernel wants), and the associated
	entity declaration.    
    The kernel must create a new interface activation record for the entity
	declaration, allocating it with the interface activation record size
	registered with the entity.   The portion of the interface activation
	record not defined by VK_tIARHeader must be initialized to zero.  The 
	context_LARP field should be set to NULL.  The caller_LARP  field may 
	be set to whatever the caller wants (for example, a local activation 
	record created by the kernel to represent the equivalent region of the
	equivalent configuration specification, and with the IARP field set to
	the value of VK_ElaborateDefaultEntity's intf_data parameter). 
    The kernel must call the default binding function for the entity 
	declaration.  The kernel_data argument (see VK_tBindFnPtr) is a 
	pointer defined by the kernel.  The entity_intf_data argument must be
	(the pointer to) the new interface activation record.
    (The default binding function will call VK_GetActualGeneric to request the
	actual value for each generic of the entity declaration and 
	VK_GetActualPort to request the actual signal for each port of the 
	entity.  It will check that the return actuals are valid (with respect
	to subtype checks and unconnected port checks).  It will use the 
	passed-in value of kernel_data and pass it through 
	VK_GetActualGeneric or VK_GetActualPort.)

    The kernel must select an architecture body and call the elaboration 
	function for the selected architecture body, passing the new interface
	activation record.
*/



/* Elaboration protocol:  Instantiating a fully-configured component 
	instantiation. */
/* Specification:  The generated code will not make any special kernel call
	to elaborate such a component instantiation and configuration
	specification. */
/* Note: The generated code will not call the kernel to elaborate a component
	instantiation when an entity aspect of the form "entity E(A)",
	"configuration C", or "open" is specified for a component instantiation
	(in a configuration specification or a component configuration). */


/* *******************************************************************
*
* 2:  Default configuration elaboration support 
*
******************************************************************* */

/* *******************************************************************
*
* VK_ElaborateDefaultEntity - select and elaborate default entity and 
*	architecture for component instantiation
*
* DESCRIPTION:
*	The generated code will call VK_ElaborateDefaultEntity to elaborate 
*	a component instantiation when an implicit binding indication applies 
*	to a component instantiation (i.e., when no explicit configuration 
*	specification or component configuration applies).
*
******************************************************************* */
void VK_ElaborateDefaultEntity(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle instance_info,
					/* identification of component 
					   instantiation */
	VK_tIARHeaderPtr intf_data	/* interface activation record for 
					   elaboration function */
#endif
	);
/* Specification:  The kernel must perform the following actions:
    The kernel must select an entity declaration according to the default
	entity selection rules in the LRM.
    The kernel is responsible for checking that the entity declaration is 
	appropriate for the component instantiation, according to the LRM rules
	about the correspondence between the names, base types, and modes of 
	entity ports and those of component declaration ports.
    As for instantiating a design entity at the top level, the kernel must:
	create a new interface activation record,
	call the default binding function (which will request actuals), and
	call the elaboration function for a selected architecture body.
*/


/* *******************************************************************
*
* VK_ElaborateDefaultArchitecture - select and elaborate default architecture
*	for given entity declaration
*
* DESCRIPTION:
*	The generated code will call VK_ElaborateDefaultArchitecture to 
*	elaborate a component instantiation statement when an entity aspect of
*	the form "entity E" is specified for a component instantiation (in a 
*	configuration specification or a component configuration) unless 
*	additional configuration constructs specify or imply an entity.
*
******************************************************************* */
void VK_ElaborateDefaultArchitecture(
#ifdef VK_ANSI_FN_PROTOS
	VK_tSymbInfoHandle entity_info,	/* identification of entity */
	VK_tIARHeaderPtr intf_data	/* interface activation record for 
					   elaboration function */
#endif
	);
/* Specification:  The kernel must perform the following actions:
    The kernel must select an architecture body and call the registered
	elaboration function for the selected architecture body, passing the 
	given interface activation record.
*/



/* *******************************************************************
*
* VK_GetActualGeneric - get actual value for default association of generic
*
* DESCRIPTION:
*	The generated code will call VK_GetActualGeneric from a default binding
*	function called by the kernel when the kernel instantiates a design 
*	entity (for VK_ElaborateDefaultArchitecture, for VK_ElaborateDefault-
*	Entity, or otherwise.)
*
******************************************************************* */
void VK_GetActualGeneric(
#ifdef VK_ANSI_FN_PROTOS
	VK_tVoidPtr kernel_data,	/* blind pointer to kernel data */
	VK_tSymbInfoHandle formal_info,	/* generic constant identification */
	VK_tAnyConstraintAddr formal_constraint_addr,
					/* NULL or formal's constraint */
	VK_tIndexConstraintAddr * actual_constraint_addr_addr,
					/* to return actual's constraint */
	VK_tDataAddr * actual_value_addr_addr
					/* to return actual's value */
#endif
	);
/* Specification:  To represent an actual designator of "open", 
	VK_GetActualGeneric must store NULL in the location addressed by
	actual_value_addr_addr. */
/* Specification:  Parameters formal_constraint_addr and 
	actual_constraint_addr_addr are defined as for VK_GetActualPort. */


/* *******************************************************************
*
* VK_GetActualPort - get actual signal for default association of port
*
* DESCRIPTION:
*	The generated code will call VK_GetActualGeneric from a default binding
*	function called by the kernel when the kernel instantiates a design 
*	entity (for VK_ElaborateDefaultArchitecture, for VK_ElaborateDefault-
*	Entity, or otherwise.)
*
******************************************************************* */
void VK_GetActualPort(
#ifdef VK_ANSI_FN_PROTOS
	VK_tVoidPtr kernel_data,	/* blind pointer to kernel data */
	VK_tSymbInfoHandle formal_info,	/* port identification */
	VK_tAnyConstraintAddr formal_constraint_addr,
					/* NULL or formal's constraint */
	VK_tIndexConstraintAddr * actual_constraint_addr_addr,
					/* to return actual's constraint */
	VK_tSignalHandle * actual_handle_addr,
					/* to return actual's handle */
	VK_tSignalOffset * actual_offset_addr
					/* to return actual's base offset */
#endif
	);
/* Specification:  To represent an actual designator of "open", 
	VK_GetActualPort must store NULL in the location addressed by
	actual_handle_addr. */
/* Specification:  Parameter formal_constraint_addr will be the address of
	an discrete range constraint (VK_tIntegerRangeConstraint * or 
	VK_tDiscreteRangePtr), a floating point range constraint 
	(VK_tFloatingRangeConstraint *), a large scalar constraint 
	(VK_tLargeRangeConstraint *), or an array index constraint 
	(VK_tIndexConstraintAddr).  It will be a null pointer for record type.
	Parameter formal_constraint_addr is also optional; it may be null 
	until the code generator implements it. */
/* Note: Parameter formal_constraint_addr is provided to tell the kernel the
	constraint against which the generated code will check the constraint
	of the actual, if the formal is constrained, to give the kernel a 
	chance to match it. */
/* Specification:  Parameter actual_constraint_addr_addr is the address of a 
	pointer to an index constraint.  VK_GetActualPort must fill in the
	addressed pointer if the parameter is of an array type.  Parameter
	actual_constraint_addr will be null for any other type class. */


#endif /* VKElaborate_H */
