#!/bin/csh -f
#
# FILE: runtests.csh
#
# DESCRIPTION:
#	This script runs VHDL test cases through the RVCG system.
#       It uses the RVCG script vtest.
#
#	Environment:
#	  VTIP variables and aliases: vhdl $spi_obj $spi_defs
#	  RVCG variables and aliases: $CG_VKI $CG_RTLI $CG_VCG $CG_TKO 
#		$CG_CFGE
#  
#		
#
# HISTORY:
#	11-DEC-91 AWH 	Created.
#
# SCCS ID: %W% %E%
#
############################################################################
#
#   Configuration Options
#
set CG_T_DIR="$RVCGR/tests"
set CG_T_RES="~vsim/test_models/results-profiled"
set CG_T_VHDL="$CG_T_DIR/vhdl"
setenv CG_TKO ~vsim/profiling/tko
#
#   Prompt user for test suite
#
# loop for main screen
main_screen:
@ CONTINUE = 1
while ($CONTINUE)
    # Print Prompt
    cat << END_OF_INTRO
***************************************************************************
*                 runtests - run RVCG VHDL test cases                      *
***************************************************************************
Available Options:

    1)  Run tests in the step_tests directory.

    2)  Run tests in the random_tests directory.

    3)  Run tests in the self_tests directory.

    4)  Run tests in the bug_tests directory.

    5)  Run tests in user specified directory.

    6)  Exit

END_OF_INTRO
    # get choice
    echo -n "Option? "
    set OPTION=$<

    # Determine action
    switch ("$OPTION")
    case "1":
	set Suite="step_tests"
	@ CONTINUE = 0
	breaksw
    case "2":
	set Suite="random_tests"
	@ CONTINUE = 0
	breaksw
    case "3":
	set Suite="self_tests"
	@ CONTINUE = 0
	breaksw
    case "4":
	set Suite="bug_tests"
	@ CONTINUE = 0
	breaksw
    case "5":
	set echo
	echo -n "Enter directory containing VHDL files: "
	set SourceDir=$<
	if ( ! -d $SourceDir ) then
		echo "Error: $SourceDir invalid directory"
		goto main_screen
	endif
	echo -n "Enter directory for test results: "
	set TargetDir=$<
	if ( ! -d $TargetDir ) then
		mkdir $TargetDir
		if ( ! -d $TargetDir ) then
			echo "Error: $TargetDir invalid directory"
			goto main_screen
		endif
	endif
	set LogFile="test.log"
	goto start_test
	breaksw
    case "6":
	exit 0
    endsw
end
set LogFile="${Suite}.log"
set SourceDir="$CG_T_VHDL/$Suite"
set TargetDir="$CG_T_RES/$Suite"

start_test:

#
#   Get File List
#
cd $SourceDir
#set RawFileList=`echo *`
#set FileList=`echo $RawFileList | tr ' ' '\012' | grep '.vhd$'`
#set FileList_2=`echo $RawFileList | tr ' ' '\012' | grep '.vhdl$'`
#set FileList="$FileList_1 $FileList_2"

set FileList=`echo * | tr ' ' '\012' | egrep 'vhd$|.vhdl$'`

if ( 0 == $#FileList ) then
    echo "Error: no VHDL files found in directory $SourceDir"
    exit 1
endif

#
#   
#
if ( ! -d $TargetDir ) then
    mkdir $TargetDir
endif

cd $TargetDir

setenv dls_test `pwd`
setenv WORK dls_test

cat << _EOM_ | tee $LogFile
***************************************************
Test Directory: $SourceDir
Results Directory: $TargetDir
Date: `date`
_EOM_

foreach file ($FileList)
  cat << _EOM_ | tee -a $LogFile


=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
Test: $file

_EOM_

  ~vsim/profiling/vtest.csh $SourceDir/$file | tee -a $LogFile
  if (0 != $status ) then
    exit 1
  endif 
end

















