--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step06.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
-- This is part of CLSI's Retargetable VHDL Code Generator.
--
-- File: Step06.vhdl
--
-- Test case: Wait on Event with Dynamic Sensitivity
--
-- VHDL Kernel Interface features exercised:
--	wait on event, ...
---------------------------------------------------------------------------

entity step06 is
end step06; -- entity


architecture a of step06 is
    signal done : BIT;
    signal sense_t1 : BIT;
    signal signal_3_4 : BIT;
    signal signal_5_6_a, signal_5_6_b : BIT;
    signal clock1, clock2 : BIT := '0';
 
begin
    done <= '1' after 1000 ns;		

					-- first set up some waveforms
    clock1	<= '1' after 10 ns, '0' after 20 ns,
		   '1' after 30 ns, '0' after 40 ns;
    clock2	<= '1' after 15 ns, '0' after 25 ns,
		   '1' after 35 ns, '0' after 45 ns;

    process
    begin
	assert FALSE
		report "CHECK: Step06 tests: that sense_t1 rose"
		severity NOTE;
	wait;
    end process;


-- Test 1: two waits with same sensitivity to one signal
  T1:
    process
    begin
	assert NOW = 0 ns
		report "FAIL: Step06 test 1a: Initial time is not 0 ns.";

	wait on clock1;			-- wait for first clock1 event (rise)
	sense_t1 <= '1';

	if not (clock1 = '1') then
	    assert FALSE report "FAIL: Step06 test 1a: ";
	elsif not (NOW >= 10 ns) then
	    assert FALSE report "FAIL: Step06 test 1a: wait resumed early";
	elsif not (NOW <= 10 ns) then
	    assert FALSE report "FAIL: Step06 test 1a: wait resumed late";
	else
	    assert FALSE report "PASS: Step06 test 1a" severity NOTE;
	end if;

	wait on clock1;			-- wait for second clock1 event (fall)

	if not (clock1 = '0') then
	    assert FALSE report "FAIL: Step06 test 1b: ";
	elsif not (NOW >= 20 ns) then
	    assert FALSE report "FAIL: Step06 test 1b: wait resumed early";
	elsif not (NOW <= 20 ns) then
	    assert FALSE report "FAIL: Step06 test 1b: wait resumed late";
	else
	    assert FALSE report "PASS: Step06 test 1b" severity NOTE;
	end if;

	wait;
	assert FALSE
		report "FAIL: Step06 test 1c: 'wait;' resumed";
    end process;


-- Test 2: waits with different sensitivity, one signal each
  T2:
    process
    begin
	wait on clock1;			-- wait for first clock1 event (rise)
	
	if not (clock1 = '1') then
	    assert FALSE report "FAIL: Step06 test 2a: ";
	elsif not (NOW = 10 ns) then
	    assert FALSE 
		report "FAIL: Step06 test 2a: wait resumed at wrong time";
	else
	    assert FALSE report "PASS: Step06 test 2a" severity NOTE;
	end if;


	wait on clock2;			-- wait for clock2 event (rise)

	if not (clock2 = '1') then
	    assert FALSE report "FAIL: Step06 test 2b: ";
	elsif not (NOW = 15 ns) then
	    assert FALSE 
		report "FAIL: Step06 test 2b: wait resumed at wrong time";
	else
	    assert FALSE report "PASS: Step06 test 2b" severity NOTE;
	end if;


	wait on clock2;			-- wait for clock2 event (fall)
					-- (make sure we ignore clock1 event)

	if not (clock2 = '0') then
	    assert FALSE report "FAIL: Step06 test 2c: ";
	elsif not (NOW = 25 ns) then
	    assert FALSE 
		report "FAIL: Step06 test 2c: wait resumed at wrong time";
	else
	    assert FALSE report "PASS: Step06 test 2c" severity NOTE;
	end if;

	wait;
    end process;



-- Test 3: wait sensitive to multiple signals
  T3:
    process
    begin


	wait on clock1, clock2;		-- wait for first clock1 event (rise)
	
	if not (clock1 = '1') then
	    assert FALSE report "FAIL: Step06 test 3a: ";
	elsif not (clock2 = '0') then
	    assert FALSE report "FAIL: Step06 test 3a: ";
	elsif not (NOW = 10 ns) then
	    assert FALSE 
		report "FAIL: Step06 test 3a: wait resumed at wrong time";
	else
	    assert FALSE report "PASS: Step06 test 3a" severity NOTE;
	end if;


	wait on clock1, clock2;		-- wait for first clock2 event (rise)
	
	if not (clock1 = '1') then
	    assert FALSE report "FAIL: Step06 test 3b: ";
	elsif not (clock1 = '1') then
	    assert FALSE report "FAIL: Step06 test 3b: ";
	elsif not (NOW = 15 ns) then
	    assert FALSE 
		report "FAIL: Step06 test 3b: wait resumed at wrong time";
	else
	    assert FALSE report "PASS: Step06 test 3b" severity NOTE;
	end if;


	wait on clock2;		-- wait one clock2 for fall at 25 ns
					-- must ignore clock1 fall
	
	if not (clock1 = '0') then
	    assert FALSE report "FAIL: Step06 test 3c: ";
	elsif not (clock1 = '0') then
	    assert FALSE report "FAIL: Step06 test 3c: ";
	elsif not (NOW = 25 ns) then
	    assert FALSE 
		report "FAIL: Step06 test 3c: wait resumed at wrong time";
	else
	    assert FALSE report "PASS: Step06 test 3c" severity NOTE;
	end if;


	wait;
	assert FALSE
		report "FAIL: Step06 test 3: 'wait;' resumed";
    end process;



end a; -- architecture
