--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step07.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
-- This is part of CLSI's Retargetable VHDL Code Generator.
--
-- File: Step07.vhdl
--
-- Test case: Full Wait 
--
-- VHDL Kernel Interface features exercised:
--	wait on event
--	wait for time
--	resuspend (for wait until condition)
---------------------------------------------------------------------------

entity step07 is
end step07; -- entity


architecture a of step07 is
    signal done : BIT;
    signal sense_2 : BIT;
    signal signal_3_4 : BIT;
    signal signal_5_6_a, signal_5_6_b : BIT;
    signal clock1, clock2 : BIT;
 
begin
    done <= '1' after 1000 ns;

    clock1	<= '0' after 10 ns, '1' after 20 ns,
		   '0' after 30 ns, '1' after 40 ns;
    clock2	<= '0' after 15 ns, '1' after 25 ns,
		   '0' after 35 ns, '1' after 45 ns;


-- Test 1: wait on nothing should wait forever

 T1:
    process
    begin
	wait;				-- wait forever
	assert FALSE
	    report "FAIL: Step07 test 1: The wait statement did not wait forever";
    end process;


-- Test 2: wait for time should wake up on time

 T2a:
    process				-- wait for time
    begin
	wait for 10 ns;			-- wait

	if NOW = 10 ns then
	    assert FALSE report "PASS: Step07 test 2a" severity NOTE;
	else
	    assert FALSE report
		"FAIL: Step07 test 2a: wait for 10 ns woke up at wrong time";
	end if;
	sense_2 <= '1';			-- signal other process that T2a awoke
	wait;				-- forever
	assert FALSE
	    report "FAIL: Step07 test 2b: final wait didn't wait forever";
    end process;

 T2b:
    process				-- check that T2A awoke 
    begin
	wait on done;			-- wait until end
					-- (yes, we are using wait to test
					-- wait; this case should work by now;
					-- it is in a previous Test)
	
	if sense_2 = '1' then
	    assert FALSE report "PASS: Step07 test 2c" severity NOTE;
	else
	    assert FALSE
	    	report "FAIL: Step07 test 2c: wait for 10 ns never woke up";
	end if;

	wait;				-- wait forever
    end process;


-- For case 3 and 4
    signal_3_4 <= '1' after 20 ns;


-- Test 3: wait for event or time with short time should wake up on time

T3a:
    process
    begin
	wait on signal_3_4 for 10 ns;
	if NOW = 10 ns then
	    assert FALSE report "PASS: Step07 test 3a" severity NOTE;
	elsif NOW = 20 ns then
	    assert FALSE
		report 
		"FAIL: Step07 test 3a: wait missed wakeup time and woke on event";
	else
	    assert FALSE 
		report "FAIL: Step07 test 3b: no diagnosis";
	end if;

	wait;				-- wait forever
	assert FALSE
	    report "FAIL: Step07 test 3b: final wait didn't wait forever";
    end process;


-- Test 4: wait for event or time with long time should wake up on event

T4a:
    process
    begin
	wait on signal_3_4 for 30 ns;
	if NOW = 20 ns then
	    assert FALSE report "PASS: Step07 test 4a" severity NOTE;
	elsif NOW = 30 ns then
	    assert FALSE
		report 
		"FAIL: Step07 test 4a: wait missed wakeup event and woke on time";
	else
	    assert FALSE 
		report "FAIL: Step07 test 4a: no diagnosis";
	end if;

	wait;				-- wait forever
	assert FALSE
	    report "FAIL: Step07 test 4b: final wait didn't wait forever";
    end process;


-- For tests 5 and 6:

    signal_5_6_a <= '1' after 10 ns, '0' after 20 ns;	-- for events
    signal_5_6_b <= '1' after 15 ns;			-- for condition


-- Test 5: wait for (event and condition) or time; give event with false
--	condition, then event with true condition; should (fully) wake up on
--	second event

T5a:
    process
    begin
	wait on signal_5_6_a until signal_5_6_b = '1' for 30 ns;
	if NOW = 20 ns then
	    assert FALSE report "PASS: Step07 test 5a" severity NOTE;
	elsif NOW = 10 ns then
	    assert FALSE
		report "FAIL: Step07 test 5a: wait did not resuspend";
	elsif NOW = 30 ns then
	    assert FALSE
		report 
		"FAIL: Step07 test 5a: wait missed wakeup event and woke on time";
	else
	    assert FALSE 
		report "FAIL: Step07 test 5a: no diagnosis";
	end if;

	wait;				-- wait forever
	assert FALSE
	    report "FAIL: Step07 test 5b: final wait didn't wait forever";
    end process;


-- Test 6: wait for (event and condition) or time; give event with false
--	condition, then event with true condition; should (fully) wake up on
--	second event

T6a:
    process
    begin
	wait on signal_5_6_a until signal_5_6_b = '1' for 14 ns;
	if NOW = 14 ns then
	    assert FALSE report "PASS: Step07 test 6a" severity NOTE;
	elsif NOW = 10 ns then
	    assert FALSE
		report "FAIL: Step07 test 6a: wait did not resuspend";
	elsif NOW = 20 ns then
	    assert FALSE
		report 
		"FAIL: Step07 test 6a: wait missed time and woke up on event";
	else
	    assert FALSE 
		report "FAIL: Step07 test 6a: no diagnosis";
	end if;

	wait;				-- wait forever
	assert FALSE
	    report "FAIL: Step07 test 6b: final wait didn't wait forever";
    end process;

end a; -- architecture
