--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step08.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
-- This is part of CLSI's Retargetable VHDL Code Generator.
--
-- File: Step08.vhdl
--
-- Test case: Function-valued signal attributes 
--	('Active, 'Event, 'Last_Active, 'Last_Event, 'Last_Value)
--
-- VHDL Kernel Interface features exercised:
--	reading 'Event, 'Active, etc.
---------------------------------------------------------------------------

entity step08 is
end step08; -- entity


architecture a of step08 is
    signal clock1 : BIT := '0';		-- signal to sample

    signal clock1_delayed : BIT;	-- copy of clock1, delayed by same 
					--   amount that sense_* will be
					--   delayed (helps read waveform)

    signal sense_flag : BOOLEAN;	-- marker indicating sample times on
					--   waveform (delayed)
    signal sense_active : BOOLEAN;	-- sense 'ACTIVE (delayed)
    signal sense_event : BOOLEAN;	-- sense 'EVENT (delayd)
    signal sense_last_active : TIME := 0 ns;	-- sense 'LAST_ACTIVE (delayed)
    signal sense_last_event : TIME := 0 ns;	-- sense 'LAST_EVENT (delayed)
    signal sense_last_value : BIT;	-- sense 'LAST_VALUE (delayed)
 
begin
					-- drive waveforms on a signal
    clock1 <= 
	'0' after 10 ns, 		-- non-event transaction
	'0' after 20 ns, 		-- non-event transaction
	'1' after 30 ns, 		-- event
	'1' after 40 ns;		-- non-event transaction

    clock1_delayed <= clock1;		-- maintain delayed clock1

    process
	variable okay_now : BOOLEAN;
	variable active_okay : BOOLEAN := TRUE;
	variable event_okay : BOOLEAN := TRUE;
	variable last_event_okay : BOOLEAN := TRUE;
	variable last_active_okay : BOOLEAN := TRUE;
	variable last_value_okay : BOOLEAN := TRUE;

	procedure Check(
		active_value : BOOLEAN;		active_message : STRING;
		event_value : BOOLEAN;		event_message : STRING;
		last_active_value : TIME;	last_active_message : STRING;
		last_event_value : TIME;	last_event_message : STRING;
		last_value_value : BIT;		last_value_message : STRING;
		okay_message : STRING
		) is
	begin
		-- non-local references: okay_not, xx_okay, clock1
	    
	    okay_now := TRUE;
	    if clock1'ACTIVE /= active_value then
	    	okay_now := FALSE;
	    	active_okay := FALSE;
	    	assert FALSE report active_message;
	    end if;
	    if clock1'EVENT /= event_value then
	    	okay_now := FALSE;
	    	event_okay := FALSE;
	    	assert FALSE report event_message;
	    end if;
	    if clock1'LAST_ACTIVE /= last_active_value then
	    	okay_now := FALSE;
	    	last_active_okay := FALSE;
	    	assert FALSE report last_active_message;
	    end if;
	    if clock1'LAST_EVENT /= last_event_value then
	    	okay_now := FALSE;
	    	last_event_okay := FALSE;
	    	assert FALSE report last_event_message;
	    end if;
	    if clock1'LAST_VALUE /= last_value_value then
	    	okay_now := FALSE;
	    	last_value_okay := FALSE;
	    	assert FALSE report last_value_message;
	    end if;

	    if okay_now then
	    	assert FALSE report okay_message severity NOTE;
	    end if;
	end;

    begin

-- Test 1: check values at simulation initialization
--	no activity; assumed previous activity at TIME'LOW;
--	no event; assumed previous event at TIME'LOW;
--	last value equals initial (and current) value

	Check(
		FALSE,	"FAIL: Step08 test 1a: 'ACTIVE is wrong",
		FALSE,	"FAIL: Step08 test 1b: 'EVENT is wrong",
		TIME'HIGH, "FAIL: Step08 test 1c: 'LAST_ACTIVE is probably wrong",
		TIME'HIGH, "FAIL: Step08 test 1d: 'LAST_EVENT is probably wrong",
		'0',	"FAIL: Step08 test 1e: 'LAST_VALUE is wrong",
		"PASS: Step08 test 1");


-- Test 2: check values after non-event transaction after finite time

	wait for 20 ns - NOW;		-- sample when zero is reassigned 

	sense_flag <= NOT sense_flag;	-- include results in waveform output
	sense_active <= clock1'ACTIVE;
	sense_event <= clock1'EVENT;
	sense_last_active <= clock1'LAST_ACTIVE;
	sense_last_event <= clock1'LAST_EVENT;
	sense_last_value <= clock1'LAST_VALUE;

	assert NOW = 20 ns;

	Check(
		TRUE,	"FAIL: Step08 test 2a: 'ACTIVE is wrong", 
		FALSE,	"FAIL: Step08 test 2b: 'EVENT is wrong", 
		0 ns,	"FAIL: Step08 test 2c: 'LAST_ACTIVE is probably wrong",
		TIME'HIGH,	
			"FAIL: Step08 test 2d: 'LAST_EVENT is probably wrong",
		'0',	"FAIL: Step08 test 2e: 'LAST_VALUE is wrong",
			"PASS: Step08 test 2");


-- Test 3: check values after finite time when there is no activity

	wait for 25 ns - NOW;

	sense_flag <= NOT sense_flag;
	sense_active <= clock1'ACTIVE;
	sense_event <= clock1'EVENT;
	sense_last_active <= clock1'LAST_ACTIVE;
	sense_last_event <= clock1'LAST_EVENT;
	sense_last_value <= clock1'LAST_VALUE;

	assert NOW = 25 ns;
 
	Check(
		FALSE,	"FAIL: Step08 test 3a: 'ACTIVE is wrong",
		FALSE,	"FAIL: Step08 test 3b: 'EVENT is wrong",
		5 ns,	"FAIL: Step08 test 3c: 'LAST_ACTIVE is probably wrong",
		TIME'HIGH,	"FAIL: Step08 test 3d: 'LAST_EVENT is probably wrong",
		'0',	"FAIL: Step08 test 3e: 'LAST_VALUE is wrong",
			"PASS: Step08 test 3" );

-- Test 4: check values after event after finite time

	wait for 30 ns - NOW;		-- sample when zero changes to one
	sense_flag <= NOT sense_flag;
	sense_active <= clock1'ACTIVE;
	sense_event <= clock1'EVENT;
	sense_last_active <= clock1'LAST_ACTIVE;
	sense_last_event <= clock1'LAST_EVENT;
	sense_last_value <= clock1'LAST_VALUE;

	assert NOW = 30 ns;

	Check(
		TRUE,	"FAIL: Step08 test 4a: 'ACTIVE is wrong",
		TRUE,	"FAIL: Step08 test 4b: 'EVENT is wrong",
		0 ns,	"FAIL: Step08 test 4c: 'LAST_ACTIVE is probably wrong",
		0 ns,	"FAIL: Step08 test 4d: 'LAST_EVENT is probably wrong",
		'0',	"FAIL: Step08 test 4e: 'LAST_VALUE is wrong",
			"PASS: Step08 test 4" );


-- Test 5: check values after delta time when there is no activity

	wait for 0 ns;		-- sample on simulation cycle (one 
					-- delta) after previous sample, to
					-- verify that 'EVENT and 'ACTIVE apply
					-- for the simulation cycle, not all
					-- simulation cycles at the current 
					-- time
	sense_flag <= NOT sense_flag;
	sense_active <= clock1'ACTIVE;
	sense_event <= clock1'EVENT;
	sense_last_active <= clock1'LAST_ACTIVE;
	sense_last_event <= clock1'LAST_EVENT;
	sense_last_value <= clock1'LAST_VALUE;
	
	assert NOW = 30 ns;	-- plus one cycle

	Check(
		FALSE,	"FAIL: Step08 test 5a: 'ACTIVE is wrong",
		FALSE,	"FAIL: Step08 test 5b: 'EVENT is wrong",
		0 ns,	"FAIL: Step08 test 5c: 'LAST_ACTIVE is probably wrong",
		0 ns,	"FAIL: Step08 test 5d: 'LAST_EVENT is probably wrong",
		'0',	"FAIL: Step08 test 5e: 'LAST_VALUE is wrong",
			"PASS: Step08 test 5" );

-- Test 6: check values after non-event transaction after finite time

	wait for 40 ns - NOW;

	sense_flag <= NOT sense_flag;
	sense_active <= clock1'ACTIVE;
	sense_event <= clock1'EVENT;
	sense_last_active <= clock1'LAST_ACTIVE;
	sense_last_event <= clock1'LAST_EVENT;
	sense_last_value <= clock1'LAST_VALUE;
	
	Check(
		TRUE,	"FAIL: Step08 test 6a: 'ACTIVE is wrong",
		FALSE,	"FAIL: Step08 test 6b: 'EVENT is wrong",
		0 ns,	"FAIL: Step08 test 6c: 'LAST_ACTIVE is probably wrong",
		10 ns,	"FAIL: Step08 test 6d: 'LAST_EVENT is probably wrong",
		'0',	"FAIL: Step08 test 6e: 'LAST_VALUE is wrong",
			"PASS: Step08 test 6" );

	if active_okay then 
	    assert FALSE report "PASS: 'Active seems okay" severity NOTE;
	end if;
	if event_okay then 
	    assert FALSE report "PASS: 'Event seems okay" severity NOTE;
	end if;
	if last_active_okay then 
	    assert FALSE report "PASS: 'Last_Active seems okay" severity NOTE;
	end if;
	if last_event_okay then 
	    assert FALSE report "PASS: 'Last_Event seems okay" severity NOTE;
	end if;
	if last_value_okay then 
	    assert FALSE report "PASS: 'Last_Value seems okay" severity NOTE;
	end if;


	wait;				-- forever
    end process;

end a; -- architecture
