--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step09.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
-- This is part of CLSI's Retargetable VHDL Code Generator.
--
-- File: Step09.vhdl
--
-- Test case: Complex Signal Assignments
--
-- VHDL Kernel Interface features exercised:
--	waveform editting (truncation, etc.)
---------------------------------------------------------------------------

entity step09 is
end step09; -- entity


architecture a of step09 is
    signal s1, s2, s3, s4, s5, s6 : BIT := '0';
 
begin
    process
    begin
	-- at 0 ns, schedule following waveform on all signals:

	--	         111111111111111111
	--               ^                 V
	--  ..00000000000                  0000000...
	--       |       |        |        |        |
	-- Time: 5 ns    10 ns    15 ns    20 ns
	
	-- then at 5 ns:
	-- Tests Xx.1:
	--   on s1: apply nothing
	--				result is 1 at 10 ns, 0 at 20 ns
	-- Tests Xx.2:
	--   on s2: apply 1 after 0 ns
	--				result is 1 at 5 ns + 1 cycle
	-- Tests Xx.3:
	--   on s3: apply 0 at 15 ns
	--				result is 0 at 15 ns
	-- Tests Xx.4:
	--   on s4: apply 1 at 15 ns
	--				result is 1 at 10 ns, 1 at 15 ns
	-- Tests Xx.5:
	--   on s5: apply 0 at 15 ns with transport delay
	--				result is 1 at 10 ns, 0 at 15 ns
	-- Tests Xx.6:
	--   on s6: apply 1 at 15 ns with transport delay
	--				result is 1 at 10 ns, 1 at 15 ns

	s1 <= '1' after 10 ns, '0' after 20 ns;
	s2 <= '1' after 10 ns, '0' after 20 ns;
	s3 <= '1' after 10 ns, '0' after 20 ns;
	s4 <= '1' after 10 ns, '0' after 20 ns;
	s5 <= '1' after 10 ns, '0' after 20 ns;
	s6 <= '1' after 10 ns, '0' after 20 ns;

	wait for 5 ns;
					-- leave waveform
	s2 <= '1' after 0 ns;		-- force s2 high in next cycle
	s3 <= '0' after 10 ns;		-- make short pulse that disappears
	s4 <= '1' after 10 ns;		-- retain original rise
	s5 <= transport '0' after 10 ns;	-- make short pulse that stays
	s6 <= transport '1' after 10 ns;	-- retain original rise

	wait;				-- forever
    end process;

    process
    begin
					-- time is 0 ns + 0 cycles
	assert s1 = '0'
		report "FAIL: Step09 test 1: s1 is wrong initially";

	wait for 5 ns - NOW;		-- 5 ns

	assert s1 = '0' report "FAIL: Step09 test 2a.1: s1 is wrong";
	assert s2 = '0' report "FAIL: Step09 test 2a.2: s2 is wrong";
	assert s3 = '0' report "FAIL: Step09 test 2a.3: s3 is wrong";
	assert s4 = '0' report "FAIL: Step09 test 2a.4: s4 is wrong";
	assert s5 = '0' report "FAIL: Step09 test 2a.5: s5 is wrong";
	assert s6 = '0' report "FAIL: Step09 test 2a.6: s6 is wrong";

	assert not s1'ACTIVE report "FAIL: Step09 test 2b.1: s1 is wrong";
	assert not s2'ACTIVE report "FAIL: Step09 test 2b.2: s2 is wrong";
	assert not s3'ACTIVE report "FAIL: Step09 test 2b.3: s3 is wrong";
	assert not s4'ACTIVE report "FAIL: Step09 test 2b.4: s4 is wrong";
	assert not s5'ACTIVE report "FAIL: Step09 test 2b.5: s5 is wrong";
	assert not s6'ACTIVE report "FAIL: Step09 test 2b.6: s6 is wrong";

	wait for 0 ns;			-- 5 ns + 1 cycle
	assert s1 = '0' report "FAIL: Step09 test 3a.1: s1 is wrong";
	assert s2 = '1' report "FAIL: Step09 test 3a.2: s2 is wrong";
	assert s3 = '0' report "FAIL: Step09 test 3a.3: s3 is wrong";
	assert s4 = '0' report "FAIL: Step09 test 3a.4: s4 is wrong";
	assert s5 = '0' report "FAIL: Step09 test 3a.5: s5 is wrong";
	assert s6 = '0' report "FAIL: Step09 test 3a.6: s6 is wrong";

	assert not s1'ACTIVE report "FAIL: Step09 test 3b.1: s1 is wrong";
	assert     s2'ACTIVE report "FAIL: Step09 test 3b.2: s2 is wrong";
	assert not s3'ACTIVE report "FAIL: Step09 test 3b.3: s3 is wrong";
	assert not s4'ACTIVE report "FAIL: Step09 test 3b.4: s4 is wrong";
	assert not s5'ACTIVE report "FAIL: Step09 test 3b.5: s5 is wrong";
	assert not s6'ACTIVE report "FAIL: Step09 test 3b.6: s6 is wrong";

	wait for 10 ns - NOW;
	assert s1 = '1' report "FAIL: Step09 test 4a.1: s1 is wrong";
	assert s2 = '1' report "FAIL: Step09 test 4a.2: s2 is wrong";
	assert s3 = '0' report "FAIL: Step09 test 4a.3: s3 is wrong";
	assert s4 = '1' report "FAIL: Step09 test 4a.4: s4 is wrong";
	assert s5 = '1' report "FAIL: Step09 test 4a.5: s5 is wrong";
	assert s6 = '1' report "FAIL: Step09 test 4a.6: s6 is wrong";

	assert     s1'ACTIVE report "FAIL: Step09 test 4b.1: s1 is wrong";
	assert not s2'ACTIVE report "FAIL: Step09 test 4b.2: s2 is wrong";
	assert not s3'ACTIVE report "FAIL: Step09 test 4b.3: s3 is wrong";
	assert     s4'ACTIVE report "FAIL: Step09 test 4b.4: s4 is wrong";
	assert     s5'ACTIVE report "FAIL: Step09 test 4b.5: s5 is wrong";
	assert     s6'ACTIVE report "FAIL: Step09 test 4b.6: s6 is wrong";

	wait for 15 ns - NOW;
	assert s1 = '1' report "FAIL: Step09 test 5a.1: s1 is wrong";
	assert s2 = '1' report "FAIL: Step09 test 5a.2: s2 is wrong";
	assert s3 = '0' report "FAIL: Step09 test 5a.3: s3 is wrong";
	assert s4 = '1' report "FAIL: Step09 test 5a.4: s4 is wrong";
	assert s5 = '0' report "FAIL: Step09 test 5a.5: s5 is wrong";
	assert s6 = '1' report "FAIL: Step09 test 5a.6: s6 is wrong";

	assert not s1'ACTIVE report "FAIL: Step09 test 5b.1: s1 is wrong";
	assert not s2'ACTIVE report "FAIL: Step09 test 5b.2: s2 is wrong";
	assert     s3'ACTIVE report "FAIL: Step09 test 5b.3: s3 is wrong";
	assert     s4'ACTIVE report "FAIL: Step09 test 5b.4: s4 is wrong";
	assert     s5'ACTIVE report "FAIL: Step09 test 5b.5: s5 is wrong";
	assert     s6'ACTIVE report "FAIL: Step09 test 5b.6: s6 is wrong";


	wait for 20 ns - NOW;
	assert s1 = '0' report "FAIL: Step09 test 6.1: s1 is wrong";
	assert s2 = '1' report "FAIL: Step09 test 6.2: s2 is wrong";
	assert s3 = '0' report "FAIL: Step09 test 6.3: s3 is wrong";
	assert s4 = '1' report "FAIL: Step09 test 6.4: s4 is wrong";
	assert s5 = '0' report "FAIL: Step09 test 6.5: s5 is wrong";
	assert s6 = '1' report "FAIL: Step09 test 6.6: s6 is wrong";

	assert     s1'ACTIVE report "FAIL: Step09 test 6.1: s1 is wrong";
	assert not s2'ACTIVE report "FAIL: Step09 test 6.2: s2 is wrong";
	assert not s3'ACTIVE report "FAIL: Step09 test 6.3: s3 is wrong";
	assert not s4'ACTIVE report "FAIL: Step09 test 6.4: s4 is wrong";
	assert not s5'ACTIVE report "FAIL: Step09 test 6.5: s5 is wrong";
	assert not s6'ACTIVE report "FAIL: Step09 test 6.6: s6 is wrong";

	wait;
    end process;


end a; -- architecture
