--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step12.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
-- This is part of CLSI's Retargetable VHDL Code Generator.
--
-- File: Step12.vhdl
--
-- Test case: Unguarded Signal Resolution
--
-- VHDL Kernel Interface features exercised:
--	Resolution function of CreateExplicitSignal
--	Kernel handling of unguarded resolution
---------------------------------------------------------------------------

entity step12 is
end;

architecture a of step12 is

    constant special : INTEGER := 10;
    constant initial : INTEGER := 0;
    constant initial_average : INTEGER := 0;		-- (0 + 0) / 2
    constant low : INTEGER := 3;
    constant high : INTEGER := 5;
    constant average_high_low : INTEGER := 4;		-- (1 + 5) / 2
    constant average_low_initial : INTEGER := 1;	-- (3 + 0) / 2;
    constant average_high_initial : INTEGER := 2;	-- (5 + 0) / 2;

    type INTEGER_VECTOR is ARRAY(NATURAL range <>) of INTEGER;

	-- resolution function: computes average of input vector element values
    function resolve(inp : INTEGER_VECTOR) return INTEGER is
	variable sum : INTEGER;
    begin
	sum := 0;
	for i in inp'RANGE loop
	    sum := sum + inp(i);	-- sum element values
	end loop;
	if inp'LENGTH = 0 then
	    return special;
	else
	    return sum / inp'LENGTH;	-- compute average
					-- note integer division w/ truncation
	end if;
    end;

    signal resolved_2 : resolve INTEGER := initial;
					-- will have two drivers to resolve
    signal resolved_0 : resolve INTEGER := initial;
					-- will have NO drivers to resolve
    signal ref_resolved : INTEGER := initial;

begin
	-- one implicit process driving resolved_2
    resolved_2 <= low after 5 ns, high after 15 ns;

	-- another implicit process driving resolved_2
    resolved_2 <= high after 10 ns, low after 20 ns;

    ref_resolved <= 			-- set up reference waveform to compare
	average_low_initial after 5 ns,
	average_high_low after 10 ns,
	high after 15 ns,
	average_high_low after 20 ns;

-- Test 1: compare resolved value with expected values

    process(resolved_2, ref_resolved,
	    resolved_2'Transaction, ref_resolved'Transaction)
    begin
	if resolved_2 /= ref_resolved then
	    assert FALSE report "FAIL: Step12 test 1a: ";
        elsif resolved_2'ACTIVE /= ref_resolved'ACTIVE then
	    assert FALSE report "FAIL: Step12 test 1b: ";
	elsif resolved_2'EVENT /= ref_resolved'EVENT then
	    assert FALSE report "FAIL: Step12 test 1c: ";
	else
	    assert FALSE report "PASS: Step12 test 1" severity NOTE;
	end if;

    end process;

end;
