--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step13.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
-- This is part of CLSI's Retargetable VHDL Code Generator.
--
-- File: Step13.vhdl
--
-- Test case: Bus and Register Resolution
--
-- VHDL Kernel Interface features exercised:
--	Resolution function of CreateExplicitSignal
--	Kernel handling of buses and registers
---------------------------------------------------------------------------

entity step13 is
end;

architecture a of step13 is

    constant special : INTEGER := 10;
    constant initial : INTEGER := 0;
    constant initial_average : INTEGER := 0;	-- (0 + 0) / 2
    constant low : INTEGER := 3;
    constant high : INTEGER := 5;
    constant average_high_low : INTEGER := 4;	-- (1 + 5) / 2
    constant average_low_initial : INTEGER := 1;	-- (3 + 0) / 2;

    type INTEGER_VECTOR is ARRAY(NATURAL range <>) of INTEGER;

    function resolve(inp : INTEGER_VECTOR) return INTEGER is
	variable sum : INTEGER;
    begin
	sum := 0;
	for i in inp'RANGE loop
	    sum := sum + inp(i);
	end loop;
	if inp'LENGTH = 0 then
	    return special;
	else
	    return sum / inp'LENGTH;	-- compute average 
	end if;
    end;


    signal gate_1 : BOOLEAN;


    signal unguarded_2 : resolve INTEGER := initial;
					-- will have two drivers to resolve
    signal unguarded_0 : resolve INTEGER := initial;
					-- will have NO drivers to resolve
    signal ref_unguarded : INTEGER := initial;

    signal bus_2 : resolve INTEGER bus := initial;
					-- will have two drivers to resolve
    signal bus_0 : resolve INTEGER bus := initial;
					-- will have NO drivers to resolve
    signal ref_bus : INTEGER := initial;

    signal register_2 : resolve INTEGER register := initial;
					-- will have two drivers to resolve
    signal register_0 : resolve INTEGER register := initial;
					-- will have NO drivers to resolve
    signal ref_register : INTEGER := initial;


    disconnect bus_2 : INTEGER after 2 ns;
    disconnect register_2 : INTEGER after 2 ns;
begin
    gate_1 <= FALSE, TRUE after 100 ns, FALSE after 200 ns;

  B1:
    block (gate_1)
    begin
	unguarded_2 <= guarded low after 3 ns;
	bus_2 <= guarded low after 3 ns;
	register_2 <= guarded low after 3 ns;
	-- Note: Concurrent signals assignments have equivalent processes.
	-- 	The equivalent process for an assignment to a bus or a
	--	register includes an assignment with a null waveform element.
	--	This null waveform element is represented in the generated
	--	code.
    end block;

  B2:
    block (gate_1)
    begin
	unguarded_2 <= guarded high after 6 ns;
	bus_2 <= guarded high after 6 ns;
	register_2 <= guarded high after 6 ns;
    end block;

    ref_unguarded <= 
--	special after 2 ns, 		-- unguarded does not disconnect
	average_low_initial after 103 ns, 		-- B1 turns on
	average_high_low after 106 ns;--,		-- B2 kicks in
--	special after 202 ns;		-- unguarded does not disconnect

    ref_bus <= 
	special after 2 ns, 		-- initial disconnection
	low after 103 ns, 		-- B1 turns on
	average_high_low after 106 ns,		-- B2 kicks in
	special after 202 ns;		-- disconnection

    ref_register <= 
	initial after 2 ns, 		-- initial disc. keeps old value
	low after 103 ns, 		-- B1 turns on
	average_high_low after 106 ns,		-- B2 kicks in
	average_high_low after 202 ns;		-- disconnection keeps previous value

    process(unguarded_2, bus_2, register_2, 
	     ref_unguarded, ref_bus, ref_register,
	    unguarded_2'Transaction, ref_unguarded'Transaction,
	     bus_2'Transaction, ref_bus'Transaction, 
	     register_2'Transaction, ref_register'Transaction)
	variable error : BOOLEAN;
    begin
	error := FALSE;

-- Test 1: that unguarded signal is right
	if unguarded_2 /= ref_unguarded then
	    assert FALSE report "FAIL: Step13 test 1a: ";
        elsif unguarded_2'ACTIVE /= ref_unguarded'ACTIVE then
	    assert FALSE report "FAIL: Step13 test 1b: ";
	elsif unguarded_2'EVENT /= ref_unguarded'EVENT then
	    assert FALSE report "FAIL: Step13 test 1c: ";
	else
	    assert FALSE report "PASS: Step13 test 1" severity NOTE;
	    error := TRUE;
	end if;

-- Test 2: that bus signal is right
	if bus_2 /= ref_bus then
	    assert FALSE report "FAIL: Step13 test 2a: ";
        elsif bus_2'ACTIVE /= ref_bus'ACTIVE then
	    assert FALSE report "FAIL: Step13 test 2b: ";
	elsif bus_2'EVENT /= ref_bus'EVENT then
	    assert FALSE report "FAIL: Step13 test 2c: ";
	else
	    assert FALSE report "PASS: Step13 test 2" severity NOTE;
	    error := TRUE;
	end if;

-- Test 3: that register signal is right
	if register_2 /= ref_register then
	    assert FALSE report "FAIL: Step13 test 3a: ";
        elsif register_2'ACTIVE /= ref_register'ACTIVE then
	    assert FALSE report "FAIL: Step13 test 3b: ";
	elsif register_2'EVENT /= ref_register'EVENT then
	    assert FALSE report "FAIL: Step13 test 3c: ";
	else
	    assert FALSE report "PASS: Step13 test 3" severity NOTE;
	    error := TRUE;
	end if;

	if not error then
	    assert FALSE report "PASS: Step13 tests: this time" severity NOTE;
	end if;
    end process;

end;
