--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step16.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
-- This is part of CLSI's Retargetable VHDL Code Generator.
--
-- File: Step16.vhdl
--
-- Test case: Function-valued signal attributes for vectors
--
-- VHDL Kernel Interface features execised:
--  reading 'Event, 'Active, 'Last_Event, 'Last_Active and 'Last_Value
---------------------------------------------------------------------------

entity step16 is
end step16;

architecture a of step16 is
    signal S: BIT_VECTOR (0 to 3);

begin
    
                                -- assign future values vector elements
    S(0) <= '1' after 30ns, '0' after 60ns, '0' after 140ns;
    S(1) <= '1' after 20ns, '0' after 50ns;
    S(2) <= '0' after 10ns, '1' after 70ns, '1' after 130ns;
    S(3) <= '1' after 40ns, '0' after 50ns, '1' after 100ns, '0' after 110ns;

    process

        type TimeVector is array (INTEGER range <>) of TIME;

        function MinTime(TimeV: TimeVector) return TIME is
            variable min_time : TIME := TIME'HIGH;
        begin
            for i in TimeV'RANGE loop
                if TimeV(i) < min_time then
                    min_time := TimeV(i);
                end if;
            end loop;
            return min_time;
        end;

        procedure Verify(test: STRING) is
            constant BadActive:     STRING :=
                "S'ACTIVE \= 'ACTIVE of its subelements." ;
            constant BadEvent:      STRING :=
                "S'EVENT \= 'EVENT of its subelements." ;
            constant BadLastActive: STRING :=
                "S'LAST_ACTIVE \= 'LAST_ACTIVE of its subelements.";
            constant BadLastEvent:  STRING :=
                "S'LAST_EVENT \= 'LAST_EVENT of its subelements.";
            constant BadLastValue:  STRING :=
                "S'LAST_VALUE \= 'LAST_VALUE of its subelements.";
        begin
            assert S'ACTIVE =   S(0)'ACTIVE or S(1)'ACTIVE or
                                S(2)'ACTIVE or S(3)'ACTIVE
                report "FAIL: Step16 test " & test & ": " & BadActive;
            assert S'EVENT =    S(0)'EVENT  or S(1)'EVENT or
                                S(2)'EVENT  or S(3)'EVENT
                report "FAIL: Step16 test " & test & ": " & BadEvent;
            assert S'LAST_ACTIVE = MinTime( S(0)'LAST_ACTIVE &
                                            S(1)'LAST_ACTIVE &
                                            S(2)'LAST_ACTIVE &
                                            S(3)'LAST_ACTIVE)
                report "FAIL: Step16 test " & test & ": " & BadLastActive;
            assert S'LAST_EVENT = MinTime(  S(0)'LAST_EVENT &
                                            S(1)'LAST_EVENT &
                                            S(2)'LAST_EVENT &
                                            S(3)'LAST_EVENT)
                report "FAIL: Step16 test " & test & ": " & BadLastEvent;
            assert S'LAST_VALUE = ( S(0)'LAST_VALUE, S(1)'LAST_VALUE,
                                    S(2)'LAST_VALUE, S(3)'LAST_VALUE)
                report "FAIL: Step16 test " & test & ": " & BadLastValue;
        end;

    begin

-- Test 1: check that 'Active works for vectors and slices.

        wait for 10ns - NOW;    -- wait first transaction (non-event)

        assert FALSE report "TEST: Step16 test 1: check 'Active." severity NOTE;
        Verify("1");

                                -- Check 'Active on vector.
        if not S'ACTIVE then
            assert FALSE
                report "FAIL: Step16 test 1a: S'Active failed.";
        else
            assert FALSE
                report "PASS: Step16 test 1a" severity NOTE;
        end if;

                                -- Check 'Active on slice.
        if S(0 to 1)'ACTIVE then
            assert FALSE
                report "FAIL: Step16 test 1b: 'Active failed for slice.";
        else 
            assert FALSE
                report "PASS: Step16 test 1b" severity NOTE;
        end if;

                                -- Check 'Active one delta cycle later
        wait for 0ns;
        Verify("1c");

        if S'ACTIVE then
            assert FALSE
                report "FAIL: Step16 test 1c: 'Active failed for slice.";
        else
            assert FALSE
                report "PASS: Step16 test 1c" severity NOTE;
        end if;

-- Test 2: check that 'Event works on vectors and slices.

        wait for 20ns - NOW;

        assert FALSE report "TEST: Step16 test 2: check 'Event." severity NOTE;
        Verify("2");

        if not S'EVENT then
            assert FALSE
                report "FAIL: Step16 test 2a: S'Event failed.";
        else
            assert FALSE
                report "PASS: Step16 test 2a" severity NOTE;
        end if;

        if S(2 to 3)'EVENT then
            assert FALSE
                report "FAIL: Step16 test 2b: 'Event failed for slice.";
        else
            assert FALSE
                report "PASS: Step16 test 2b" severity NOTE;
        end if;

        wait for 0ns;           -- Check 'Event one delta cycle later
        Verify("2c");

        if S'EVENT then
            assert FALSE
                report "FAIL: Step16 test 2c: S'Event failed.";
        else
            assert FALSE
                report "PASS: Step16 test 2c" severity NOTE;
        end if;

-- Test 3: check that 'Last_Active worsk on vectors and slices.

        wait for 30ns - NOW;

        assert FALSE report "TEST: Step16 test 3: check 'Last_Active."
            severity NOTE;
        Verify("3");

        if not (S'LAST_ACTIVE = 0ns) then
            assert FALSE
                report "FAIL: Step16 test 3a: S'Last_Active failed.";
        else
            assert FALSE
                report "PASS: Step16 test 3a" severity NOTE;
        end if;

        if not (S(2 to 3)'LAST_ACTIVE = 20ns) then
            assert FALSE
                report "FAIL: Step16 test 3b: 'Last_Active failed for slice.";
        else
            assert FALSE
                report "PASS: Step16 test 3b" severity NOTE;
        end if;

-- Test 4: check that 'Last_Event works on vectors and slices.

        wait for 40ns - NOW;

        assert FALSE report "TEST: Step16 test 4: check 'Last_Event."
            severity NOTE;
        Verify("4");

        if not (S'LAST_EVENT = 0ns) then
            assert FALSE
                report "FAIL: Step16 test 4a: S'Last_Event failed.";
        else
            assert FALSE
                report "PASS: Step16 test 4a" severity NOTE;
        end if;


        if not (S(1 to 2)'LAST_EVENT = 20ns) then
            assert FALSE
                report "FAIL: Step16 test 4b: 'Last_Event failed for slice.";
        else
            assert FALSE
                report "PASS: Step16 test 4b" severity NOTE;
        end if;

-- Test 5: check that 'Last_Value works on vectors and slices.

        wait for 50ns - NOW;

        assert FALSE report "TEST: Step16 test 5: check 'Last_Value"
            severity NOTE;
        Verify("5");

        if not (S'LAST_VALUE = b"0101") then
            assert FALSE
                report "FAIL: Step16 test 5a: S'Last_Value failed.";
        else
            assert FALSE
                report "PASS: Step16 test 5a" severity NOTE;
        end if;

        if not (S(1 to 3)'LAST_VALUE = b"101") then
            assert FALSE
                report "FAIL: Step16 test 5b: 'Last_Value failed for slice.";
        else
            assert FALSE
                report "PASS: Step16 test 5b" severity NOTE;
        end if;

-- Test 6: Random test of all attributes...

        wait for 150ns - NOW;
        Verify("6");
        
        wait;                   -- wait forever;
    end process;

end a;
