--##*************************************************************************
--
--+ File: RVCG:[work/tests/project/step_tests]step17.vhdl
--
--+ Copyright: (c) 1991 by CAD Language Systems, Inc. All rights reserved.
--
--+ LRM_Version: IEEE Std. 1076-1987 (incl. "Sense of the VASG" I)
--
--+ Subsets: None
--
--+ Description: 
--
--+ Test_point -  (None)
--
--+ Test_objective - 
--
--+ References: 
--
--+ Instructions: 
--
--+ Keywords: 
--
--+ Test_Results: 
--
--+ Test_Type: 
--
--+ History: AWH/CLSI 12-Apr-91 - Test header automatically added
--
--+ Comments: 
--***********************************************************     
-- Begin test: >>>
---------------------------------------------------------------------------
--
-- File: Step17.vhdl
--
-- Test case: Implicit Signals of Vector signals
--
-- VHDL Kernel Interface features exercised:
--  create implicit signals 'Transaction, 'Quiet, 'Stable, and 'Delayed.
--  read implicit signals
---------------------------------------------------------------------------

entity step17 is
end step17;

architecture a of step17 is

    signal S: BIT_VECTOR(0 to 3);

    procedure Verify(test: STRING) is
        constant BadStable: STRING :=
            "S'STABLE \= 'STABLE of its subelements";
        constant BadQuiet: STRING :=
            "S'QUIET \= 'QUIET of its subelements";
        constant BadTrans: STRING :=
            "S'TRANSACTION \= 'TRANSACTION of its subelements";
    begin
        assert not (S'STABLE =  S(0)'STABLE and S(1)'STABLE and
                                S(2)'STABLE and S(3)'STABLE)
            report "FAIL: Step 17 test " & test & ": " & BadStable;
        assert not (S'QUIET =   S(0)'QUIET and S(1)'QUIET and
                                S(2)'QUIET and S(3)'QUIET)
            report "FAIL: Step 17 " & test & ": " & BadQuiet;
        assert S'TRANSACTION'ACTIVE =  (S(0)'TRANSACTION'ACTIVE or
                                            S(1)'TRANSACTION'ACTIVE or
                                            S(2)'TRANSACTION'ACTIVE or
                                            S(3)'TRANSACTION'ACTIVE)
            report "FAIL: Step 17 " & test & ": " & BadTrans;
    end;

begin

    S(0) <= '1' after 40ns, '0' after 80ns;
    S(1) <= '1' after 60ns, '0' after 110ns;
    S(2) <= '1' after 60ns, '0' after 70ns, '0' after 110ns;
    S(3) <= '1' after 90ns, '0' after 100ns, '0' after 110ns;

    process
                                -- Declare a variable to store S'Trans at 0ns.
        variable S_Transaction : BIT;
                                -- Declare a variable to store S(0 to 1)'Trans
                                -- at 0ns.
        variable S_slice_Trans : BIT;
    begin

        S_Transaction := S'TRANSACTION;
        S_slice_Trans := S(0 to 1)'TRANSACTION;

-- Test 1: that 'Delayed works for vectors and slices.

        wait for 60ns - NOW;

        assert FALSE report "TEST: Step17 test 1: check 'Delayed."
            severity NOTE;
        Verify("1");

        if not (S(0)'DELAYED(20ns) = '1' and S(0)'DELAYED(20ns)'EVENT) then
            assert FALSE
                report "FAIL: Step17 test 1a: S'DELAYED(20ns) failed.";
        else
            assert FALSE
                report "PASS: Step17 test 1a" severity NOTE;
        end if;

        if not (S(2)'DELAYED(0ns) = '0') then
            assert FALSE
                report "FAIL: Step17 test 1b: S(1 to 2)'DELAYED(0ns) failed.";
        else
            assert FALSE
                report "PASS: Step17 test 1b" severity NOTE;
        end if;

-- Test 2: that 'Transaction works for vectors and slices.

        wait for 70ns - NOW;

        assert FALSE report "TEST: Step17 test 2: check 'Transaction."
            severity NOTE;
        Verify("2");

        if not (S'TRANSACTION = not S_Transaction) then
            assert FALSE
                report "FAIL: Step17 test 2a: S'TRANSACTION failed.";
        else
            assert FALSE 
                report "PASS: Step17 test 2a" severity NOTE;
        end if;

        if not (S(0 to 1)'TRANSACTION = S_slice_Trans) then
            assert FALSE
                report "FAIL: Step17 test 2b: 'TRANSACTION failed on a slice.";
        else
            assert FALSE
                report "PASS: Step17 test 2b:" severity NOTE;
        end if;

-- Test 3: that 'Stable works for vectors and slices.

        assert FALSE report "TEST: Step17 test 3: check 'Stable."
            severity NOTE;

        if not (S'STABLE = FALSE) then
            assert FALSE
                report "FAIL: Step17 test 3a: S'STABLE failed.";
        else
            assert FALSE
                report "PASS: Step17 test 3a" severity NOTE;
        end if;
        
        if not (S(0 to 1)'STABLE = TRUE) then
            assert FALSE
                report "FAIL: Step17 test 3b: 'STABLE on a slice failed.";
        else
            assert FALSE
                report "PASS: Step17 test 3b:" severity NOTE;
        end if;

-- Test 4: that 'Quiet works for vectors and slices.

        wait for 90ns - NOW;

        assert FALSE report "TEST: Step17 step 4: check 'Quiet."
            severity NOTE;
        Verify("4");

        if not (S'QUIET = FALSE) then
            assert FALSE
                report "FAIL: Step17 test 4a: S'QUIET failed.";
        else
            assert FALSE
                report "PASS: Step17 test 4a" severity NOTE;
        end if;

        if not (S(0 to 2)'QUIET = TRUE) then
            assert FALSE
                report "FAIL: Step17 test 4b: 'QUIET failed on a slice.";
        else
            assert FALSE
                report "PASS: Step17 test 4b:" severity NOTE;
        end if;

        wait;                   -- wait forever
    end process;

end a;
